function _P(t){return t&&t.__esModule&&Object.prototype.hasOwnProperty.call(t,"default")?t.default:t}var w={exports:{}},be={};/*
object-assign
(c) Sindre Sorhus
@license MIT
*/var A0=Object.getOwnPropertySymbols,BP=Object.prototype.hasOwnProperty,qP=Object.prototype.propertyIsEnumerable;function zP(t){if(t==null)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(t)}function FP(){try{if(!Object.assign)return!1;var t=new String("abc");if(t[5]="de",Object.getOwnPropertyNames(t)[0]==="5")return!1;for(var e={},n=0;n<10;n++)e["_"+String.fromCharCode(n)]=n;var r=Object.getOwnPropertyNames(e).map(function(o){return e[o]});if(r.join("")!=="0123456789")return!1;var i={};return"abcdefghijklmnopqrst".split("").forEach(function(o){i[o]=o}),Object.keys(Object.assign({},i)).join("")==="abcdefghijklmnopqrst"}catch{return!1}}var D0=FP()?Object.assign:function(t,e){for(var n,r=zP(t),i,o=1;o<arguments.length;o++){n=Object(arguments[o]);for(var s in n)BP.call(n,s)&&(r[s]=n[s]);if(A0){i=A0(n);for(var l=0;l<i.length;l++)qP.call(n,i[l])&&(r[i[l]]=n[i[l]])}}return r};/** @license React v17.0.2
 * react.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var ph=D0,uo=60103,L0=60106;be.Fragment=60107;be.StrictMode=60108;be.Profiler=60114;var W0=60109,Q0=60110,N0=60112;be.Suspense=60113;var _0=60115,B0=60116;if(typeof Symbol=="function"&&Symbol.for){var Tn=Symbol.for;uo=Tn("react.element"),L0=Tn("react.portal"),be.Fragment=Tn("react.fragment"),be.StrictMode=Tn("react.strict_mode"),be.Profiler=Tn("react.profiler"),W0=Tn("react.provider"),Q0=Tn("react.context"),N0=Tn("react.forward_ref"),be.Suspense=Tn("react.suspense"),_0=Tn("react.memo"),B0=Tn("react.lazy")}var q0=typeof Symbol=="function"&&Symbol.iterator;function VP(t){return t===null||typeof t!="object"?null:(t=q0&&t[q0]||t["@@iterator"],typeof t=="function"?t:null)}function Es(t){for(var e="https://reactjs.org/docs/error-decoder.html?invariant="+t,n=1;n<arguments.length;n++)e+="&args[]="+encodeURIComponent(arguments[n]);return"Minified React error #"+t+"; visit "+e+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}var z0={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},F0={};function fo(t,e,n){this.props=t,this.context=e,this.refs=F0,this.updater=n||z0}fo.prototype.isReactComponent={};fo.prototype.setState=function(t,e){if(typeof t!="object"&&typeof t!="function"&&t!=null)throw Error(Es(85));this.updater.enqueueSetState(this,t,e,"setState")};fo.prototype.forceUpdate=function(t){this.updater.enqueueForceUpdate(this,t,"forceUpdate")};function V0(){}V0.prototype=fo.prototype;function mh(t,e,n){this.props=t,this.context=e,this.refs=F0,this.updater=n||z0}var gh=mh.prototype=new V0;gh.constructor=mh;ph(gh,fo.prototype);gh.isPureReactComponent=!0;var yh={current:null},U0=Object.prototype.hasOwnProperty,j0={key:!0,ref:!0,__self:!0,__source:!0};function X0(t,e,n){var r,i={},o=null,s=null;if(e!=null)for(r in e.ref!==void 0&&(s=e.ref),e.key!==void 0&&(o=""+e.key),e)U0.call(e,r)&&!j0.hasOwnProperty(r)&&(i[r]=e[r]);var l=arguments.length-2;if(l===1)i.children=n;else if(1<l){for(var a=Array(l),c=0;c<l;c++)a[c]=arguments[c+2];i.children=a}if(t&&t.defaultProps)for(r in l=t.defaultProps,l)i[r]===void 0&&(i[r]=l[r]);return{$$typeof:uo,type:t,key:o,ref:s,props:i,_owner:yh.current}}function UP(t,e){return{$$typeof:uo,type:t.type,key:e,ref:t.ref,props:t.props,_owner:t._owner}}function Oh(t){return typeof t=="object"&&t!==null&&t.$$typeof===uo}function jP(t){var e={"=":"=0",":":"=2"};return"$"+t.replace(/[=:]/g,function(n){return e[n]})}var H0=/\/+/g;function vh(t,e){return typeof t=="object"&&t!==null&&t.key!=null?jP(""+t.key):e.toString(36)}function za(t,e,n,r,i){var o=typeof t;(o==="undefined"||o==="boolean")&&(t=null);var s=!1;if(t===null)s=!0;else switch(o){case"string":case"number":s=!0;break;case"object":switch(t.$$typeof){case uo:case L0:s=!0}}if(s)return s=t,i=i(s),t=r===""?"."+vh(s,0):r,Array.isArray(i)?(n="",t!=null&&(n=t.replace(H0,"$&/")+"/"),za(i,e,n,"",function(c){return c})):i!=null&&(Oh(i)&&(i=UP(i,n+(!i.key||s&&s.key===i.key?"":(""+i.key).replace(H0,"$&/")+"/")+t)),e.push(i)),1;if(s=0,r=r===""?".":r+":",Array.isArray(t))for(var l=0;l<t.length;l++){o=t[l];var a=r+vh(o,l);s+=za(o,e,n,a,i)}else if(a=VP(t),typeof a=="function")for(t=a.call(t),l=0;!(o=t.next()).done;)o=o.value,a=r+vh(o,l++),s+=za(o,e,n,a,i);else if(o==="object")throw e=""+t,Error(Es(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e));return s}function Fa(t,e,n){if(t==null)return t;var r=[],i=0;return za(t,r,"","",function(o){return e.call(n,o,i++)}),r}function XP(t){if(t._status===-1){var e=t._result;e=e(),t._status=0,t._result=e,e.then(function(n){t._status===0&&(n=n.default,t._status=1,t._result=n)},function(n){t._status===0&&(t._status=2,t._result=n)})}if(t._status===1)return t._result;throw t._result}var G0={current:null};function hr(){var t=G0.current;if(t===null)throw Error(Es(321));return t}var HP={ReactCurrentDispatcher:G0,ReactCurrentBatchConfig:{transition:0},ReactCurrentOwner:yh,IsSomeRendererActing:{current:!1},assign:ph};be.Children={map:Fa,forEach:function(t,e,n){Fa(t,function(){e.apply(this,arguments)},n)},count:function(t){var e=0;return Fa(t,function(){e++}),e},toArray:function(t){return Fa(t,function(e){return e})||[]},only:function(t){if(!Oh(t))throw Error(Es(143));return t}};be.Component=fo;be.PureComponent=mh;be.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=HP;be.cloneElement=function(t,e,n){if(t==null)throw Error(Es(267,t));var r=ph({},t.props),i=t.key,o=t.ref,s=t._owner;if(e!=null){if(e.ref!==void 0&&(o=e.ref,s=yh.current),e.key!==void 0&&(i=""+e.key),t.type&&t.type.defaultProps)var l=t.type.defaultProps;for(a in e)U0.call(e,a)&&!j0.hasOwnProperty(a)&&(r[a]=e[a]===void 0&&l!==void 0?l[a]:e[a])}var a=arguments.length-2;if(a===1)r.children=n;else if(1<a){l=Array(a);for(var c=0;c<a;c++)l[c]=arguments[c+2];r.children=l}return{$$typeof:uo,type:t.type,key:i,ref:o,props:r,_owner:s}};be.createContext=function(t,e){return e===void 0&&(e=null),t={$$typeof:Q0,_calculateChangedBits:e,_currentValue:t,_currentValue2:t,_threadCount:0,Provider:null,Consumer:null},t.Provider={$$typeof:W0,_context:t},t.Consumer=t};be.createElement=X0;be.createFactory=function(t){var e=X0.bind(null,t);return e.type=t,e};be.createRef=function(){return{current:null}};be.forwardRef=function(t){return{$$typeof:N0,render:t}};be.isValidElement=Oh;be.lazy=function(t){return{$$typeof:B0,_payload:{_status:-1,_result:t},_init:XP}};be.memo=function(t,e){return{$$typeof:_0,type:t,compare:e===void 0?null:e}};be.useCallback=function(t,e){return hr().useCallback(t,e)};be.useContext=function(t,e){return hr().useContext(t,e)};be.useDebugValue=function(){};be.useEffect=function(t,e){return hr().useEffect(t,e)};be.useImperativeHandle=function(t,e,n){return hr().useImperativeHandle(t,e,n)};be.useLayoutEffect=function(t,e){return hr().useLayoutEffect(t,e)};be.useMemo=function(t,e){return hr().useMemo(t,e)};be.useReducer=function(t,e,n){return hr().useReducer(t,e,n)};be.useRef=function(t){return hr().useRef(t)};be.useState=function(t){return hr().useState(t)};be.version="17.0.2";w.exports=be;var In=w.exports,Z0={exports:{}},GP="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED",ZP=GP,YP=ZP;function Y0(){}function K0(){}K0.resetWarningCache=Y0;var KP=function(){function t(r,i,o,s,l,a){if(a!==YP){var c=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw c.name="Invariant Violation",c}}t.isRequired=t;function e(){return t}var n={array:t,bigint:t,bool:t,func:t,number:t,object:t,string:t,symbol:t,any:t,arrayOf:e,element:t,elementType:t,instanceOf:e,node:t,objectOf:e,oneOf:e,oneOfType:e,shape:e,exact:e,checkPropTypes:K0,resetWarningCache:Y0};return n.PropTypes=n,n};Z0.exports=KP();var _r=In.createContext(null);function JP(t){t()}var J0=JP,eC=function(e){return J0=e},tC=function(){return J0};function nC(){var t=tC(),e=null,n=null;return{clear:function(){e=null,n=null},notify:function(){t(function(){for(var i=e;i;)i.callback(),i=i.next})},get:function(){for(var i=[],o=e;o;)i.push(o),o=o.next;return i},subscribe:function(i){var o=!0,s=n={callback:i,next:null,prev:n};return s.prev?s.prev.next=s:e=s,function(){!o||e===null||(o=!1,s.next?s.next.prev=s.prev:n=s.prev,s.prev?s.prev.next=s.next:e=s.next)}}}}var ey={notify:function(){},get:function(){return[]}};function ty(t,e){var n,r=ey;function i(f){return a(),r.subscribe(f)}function o(){r.notify()}function s(){u.onStateChange&&u.onStateChange()}function l(){return Boolean(n)}function a(){n||(n=e?e.addNestedSub(s):t.subscribe(s),r=nC())}function c(){n&&(n(),n=void 0,r.clear(),r=ey)}var u={addNestedSub:i,notifyNestedSubs:o,handleChangeWrapper:s,isSubscribed:l,trySubscribe:a,tryUnsubscribe:c,getListeners:function(){return r}};return u}var xh=typeof window!="undefined"&&typeof window.document!="undefined"&&typeof window.document.createElement!="undefined"?w.exports.useLayoutEffect:w.exports.useEffect,Va={exports:{}},Ms={};/** @license React v17.0.2
 * react-jsx-runtime.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var rC=w.exports,ny=60103;Ms.Fragment=60107;if(typeof Symbol=="function"&&Symbol.for){var ry=Symbol.for;ny=ry("react.element"),Ms.Fragment=ry("react.fragment")}var iC=rC.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,oC=Object.prototype.hasOwnProperty,sC={key:!0,ref:!0,__self:!0,__source:!0};function iy(t,e,n){var r,i={},o=null,s=null;n!==void 0&&(o=""+n),e.key!==void 0&&(o=""+e.key),e.ref!==void 0&&(s=e.ref);for(r in e)oC.call(e,r)&&!sC.hasOwnProperty(r)&&(i[r]=e[r]);if(t&&t.defaultProps)for(r in e=t.defaultProps,e)i[r]===void 0&&(i[r]=e[r]);return{$$typeof:ny,type:t,key:o,ref:s,props:i,_owner:iC.current}}Ms.jsx=iy;Ms.jsxs=iy;Va.exports=Ms;const I=Va.exports.jsx,Ue=Va.exports.jsxs,d7=Va.exports.Fragment;function p7(t){var e=t.store,n=t.context,r=t.children,i=w.exports.useMemo(function(){var l=ty(e);return l.onStateChange=l.notifyNestedSubs,{store:e,subscription:l}},[e]),o=w.exports.useMemo(function(){return e.getState()},[e]);xh(function(){var l=i.subscription;return l.trySubscribe(),o!==e.getState()&&l.notifyNestedSubs(),function(){l.tryUnsubscribe(),l.onStateChange=null}},[i,o]);var s=n||_r;return I(s.Provider,{value:i,children:r})}function $(){return $=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t},$.apply(this,arguments)}function H(t,e){if(t==null)return{};var n={},r=Object.keys(t),i,o;for(o=0;o<r.length;o++)i=r[o],!(e.indexOf(i)>=0)&&(n[i]=t[i]);return n}var oy={exports:{}},Re={};/** @license React v16.13.1
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var st=typeof Symbol=="function"&&Symbol.for,wh=st?Symbol.for("react.element"):60103,bh=st?Symbol.for("react.portal"):60106,Ua=st?Symbol.for("react.fragment"):60107,ja=st?Symbol.for("react.strict_mode"):60108,Xa=st?Symbol.for("react.profiler"):60114,Ha=st?Symbol.for("react.provider"):60109,Ga=st?Symbol.for("react.context"):60110,$h=st?Symbol.for("react.async_mode"):60111,Za=st?Symbol.for("react.concurrent_mode"):60111,Ya=st?Symbol.for("react.forward_ref"):60112,Ka=st?Symbol.for("react.suspense"):60113,lC=st?Symbol.for("react.suspense_list"):60120,Ja=st?Symbol.for("react.memo"):60115,ec=st?Symbol.for("react.lazy"):60116,aC=st?Symbol.for("react.block"):60121,cC=st?Symbol.for("react.fundamental"):60117,uC=st?Symbol.for("react.responder"):60118,fC=st?Symbol.for("react.scope"):60119;function Kt(t){if(typeof t=="object"&&t!==null){var e=t.$$typeof;switch(e){case wh:switch(t=t.type,t){case $h:case Za:case Ua:case Xa:case ja:case Ka:return t;default:switch(t=t&&t.$$typeof,t){case Ga:case Ya:case ec:case Ja:case Ha:return t;default:return e}}case bh:return e}}}function sy(t){return Kt(t)===Za}Re.AsyncMode=$h;Re.ConcurrentMode=Za;Re.ContextConsumer=Ga;Re.ContextProvider=Ha;Re.Element=wh;Re.ForwardRef=Ya;Re.Fragment=Ua;Re.Lazy=ec;Re.Memo=Ja;Re.Portal=bh;Re.Profiler=Xa;Re.StrictMode=ja;Re.Suspense=Ka;Re.isAsyncMode=function(t){return sy(t)||Kt(t)===$h};Re.isConcurrentMode=sy;Re.isContextConsumer=function(t){return Kt(t)===Ga};Re.isContextProvider=function(t){return Kt(t)===Ha};Re.isElement=function(t){return typeof t=="object"&&t!==null&&t.$$typeof===wh};Re.isForwardRef=function(t){return Kt(t)===Ya};Re.isFragment=function(t){return Kt(t)===Ua};Re.isLazy=function(t){return Kt(t)===ec};Re.isMemo=function(t){return Kt(t)===Ja};Re.isPortal=function(t){return Kt(t)===bh};Re.isProfiler=function(t){return Kt(t)===Xa};Re.isStrictMode=function(t){return Kt(t)===ja};Re.isSuspense=function(t){return Kt(t)===Ka};Re.isValidElementType=function(t){return typeof t=="string"||typeof t=="function"||t===Ua||t===Za||t===Xa||t===ja||t===Ka||t===lC||typeof t=="object"&&t!==null&&(t.$$typeof===ec||t.$$typeof===Ja||t.$$typeof===Ha||t.$$typeof===Ga||t.$$typeof===Ya||t.$$typeof===cC||t.$$typeof===uC||t.$$typeof===fC||t.$$typeof===aC)};Re.typeOf=Kt;oy.exports=Re;var ly=oy.exports,hC={$$typeof:!0,render:!0,defaultProps:!0,displayName:!0,propTypes:!0},dC={$$typeof:!0,compare:!0,defaultProps:!0,displayName:!0,propTypes:!0,type:!0},ay={};ay[ly.ForwardRef]=hC;ay[ly.Memo]=dC;var Me={};/** @license React v17.0.2
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var tc=60103,nc=60106,As=60107,Ds=60108,Ls=60114,Ws=60109,Qs=60110,Ns=60112,_s=60113,Sh=60120,Bs=60115,qs=60116,cy=60121,uy=60122,fy=60117,hy=60129,dy=60131;if(typeof Symbol=="function"&&Symbol.for){var ht=Symbol.for;tc=ht("react.element"),nc=ht("react.portal"),As=ht("react.fragment"),Ds=ht("react.strict_mode"),Ls=ht("react.profiler"),Ws=ht("react.provider"),Qs=ht("react.context"),Ns=ht("react.forward_ref"),_s=ht("react.suspense"),Sh=ht("react.suspense_list"),Bs=ht("react.memo"),qs=ht("react.lazy"),cy=ht("react.block"),uy=ht("react.server.block"),fy=ht("react.fundamental"),hy=ht("react.debug_trace_mode"),dy=ht("react.legacy_hidden")}function En(t){if(typeof t=="object"&&t!==null){var e=t.$$typeof;switch(e){case tc:switch(t=t.type,t){case As:case Ls:case Ds:case _s:case Sh:return t;default:switch(t=t&&t.$$typeof,t){case Qs:case Ns:case qs:case Bs:case Ws:return t;default:return e}}case nc:return e}}}var pC=Ws,mC=tc,gC=Ns,yC=As,OC=qs,vC=Bs,xC=nc,wC=Ls,bC=Ds,$C=_s;Me.ContextConsumer=Qs;Me.ContextProvider=pC;Me.Element=mC;Me.ForwardRef=gC;Me.Fragment=yC;Me.Lazy=OC;Me.Memo=vC;Me.Portal=xC;Me.Profiler=wC;Me.StrictMode=bC;Me.Suspense=$C;Me.isAsyncMode=function(){return!1};Me.isConcurrentMode=function(){return!1};Me.isContextConsumer=function(t){return En(t)===Qs};Me.isContextProvider=function(t){return En(t)===Ws};Me.isElement=function(t){return typeof t=="object"&&t!==null&&t.$$typeof===tc};Me.isForwardRef=function(t){return En(t)===Ns};Me.isFragment=function(t){return En(t)===As};Me.isLazy=function(t){return En(t)===qs};Me.isMemo=function(t){return En(t)===Bs};Me.isPortal=function(t){return En(t)===nc};Me.isProfiler=function(t){return En(t)===Ls};Me.isStrictMode=function(t){return En(t)===Ds};Me.isSuspense=function(t){return En(t)===_s};Me.isValidElementType=function(t){return typeof t=="string"||typeof t=="function"||t===As||t===Ls||t===hy||t===Ds||t===_s||t===Sh||t===dy||typeof t=="object"&&t!==null&&(t.$$typeof===qs||t.$$typeof===Bs||t.$$typeof===Ws||t.$$typeof===Qs||t.$$typeof===Ns||t.$$typeof===fy||t.$$typeof===cy||t[0]===uy)};Me.typeOf=En;function py(){var t=w.exports.useContext(_r);return t}function my(t){t===void 0&&(t=_r);var e=t===_r?py:function(){return w.exports.useContext(t)};return function(){var r=e(),i=r.store;return i}}var SC=my();function kC(t){t===void 0&&(t=_r);var e=t===_r?SC:my(t);return function(){var r=e();return r.dispatch}}var m7=kC(),PC=function(e,n){return e===n};function CC(t,e,n,r){var i=w.exports.useReducer(function(p){return p+1},0),o=i[1],s=w.exports.useMemo(function(){return ty(n,r)},[n,r]),l=w.exports.useRef(),a=w.exports.useRef(),c=w.exports.useRef(),u=w.exports.useRef(),f=n.getState(),h;try{if(t!==a.current||f!==c.current||l.current){var d=t(f);u.current===void 0||!e(d,u.current)?h=d:h=u.current}else h=u.current}catch(p){throw l.current&&(p.message+=`
The error may be correlated with this previous error:
`+l.current.stack+`

`),p}return xh(function(){a.current=t,c.current=f,u.current=h,l.current=void 0}),xh(function(){function p(){try{var m=n.getState();if(m===c.current)return;var g=a.current(m);if(e(g,u.current))return;u.current=g,c.current=m}catch(y){l.current=y}o()}return s.onStateChange=p,s.trySubscribe(),p(),function(){return s.tryUnsubscribe()}},[n,s]),h}function RC(t){t===void 0&&(t=_r);var e=t===_r?py:function(){return w.exports.useContext(t)};return function(r,i){i===void 0&&(i=PC);var o=e(),s=o.store,l=o.subscription,a=CC(r,i,s,l);return w.exports.useDebugValue(a),a}}var g7=RC(),rc={exports:{}},fn={},gy={exports:{}},yy={};/** @license React v0.20.2
 * scheduler.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */(function(t){var e,n,r,i;if(typeof performance=="object"&&typeof performance.now=="function"){var o=performance;t.unstable_now=function(){return o.now()}}else{var s=Date,l=s.now();t.unstable_now=function(){return s.now()-l}}if(typeof window=="undefined"||typeof MessageChannel!="function"){var a=null,c=null,u=function(){if(a!==null)try{var W=t.unstable_now();a(!0,W),a=null}catch(B){throw setTimeout(u,0),B}};e=function(W){a!==null?setTimeout(e,0,W):(a=W,setTimeout(u,0))},n=function(W,B){c=setTimeout(W,B)},r=function(){clearTimeout(c)},t.unstable_shouldYield=function(){return!1},i=t.unstable_forceFrameRate=function(){}}else{var f=window.setTimeout,h=window.clearTimeout;if(typeof console!="undefined"){var d=window.cancelAnimationFrame;typeof window.requestAnimationFrame!="function"&&console.error("This browser doesn't support requestAnimationFrame. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills"),typeof d!="function"&&console.error("This browser doesn't support cancelAnimationFrame. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills")}var p=!1,m=null,g=-1,y=5,O=0;t.unstable_shouldYield=function(){return t.unstable_now()>=O},i=function(){},t.unstable_forceFrameRate=function(W){0>W||125<W?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):y=0<W?Math.floor(1e3/W):5};var v=new MessageChannel,x=v.port2;v.port1.onmessage=function(){if(m!==null){var W=t.unstable_now();O=W+y;try{m(!0,W)?x.postMessage(null):(p=!1,m=null)}catch(B){throw x.postMessage(null),B}}else p=!1},e=function(W){m=W,p||(p=!0,x.postMessage(null))},n=function(W,B){g=f(function(){W(t.unstable_now())},B)},r=function(){h(g),g=-1}}function b(W,B){var Z=W.length;W.push(B);e:for(;;){var ee=Z-1>>>1,pe=W[ee];if(pe!==void 0&&0<P(pe,B))W[ee]=B,W[Z]=pe,Z=ee;else break e}}function S(W){return W=W[0],W===void 0?null:W}function k(W){var B=W[0];if(B!==void 0){var Z=W.pop();if(Z!==B){W[0]=Z;e:for(var ee=0,pe=W.length;ee<pe;){var J=2*(ee+1)-1,ue=W[J],se=J+1,we=W[se];if(ue!==void 0&&0>P(ue,Z))we!==void 0&&0>P(we,ue)?(W[ee]=we,W[se]=Z,ee=se):(W[ee]=ue,W[J]=Z,ee=J);else if(we!==void 0&&0>P(we,Z))W[ee]=we,W[se]=Z,ee=se;else break e}}return B}return null}function P(W,B){var Z=W.sortIndex-B.sortIndex;return Z!==0?Z:W.id-B.id}var C=[],R=[],T=1,L=null,M=3,q=!1,F=!1,Q=!1;function E(W){for(var B=S(R);B!==null;){if(B.callback===null)k(R);else if(B.startTime<=W)k(R),B.sortIndex=B.expirationTime,b(C,B);else break;B=S(R)}}function D(W){if(Q=!1,E(W),!F)if(S(C)!==null)F=!0,e(X);else{var B=S(R);B!==null&&n(D,B.startTime-W)}}function X(W,B){F=!1,Q&&(Q=!1,r()),q=!0;var Z=M;try{for(E(B),L=S(C);L!==null&&(!(L.expirationTime>B)||W&&!t.unstable_shouldYield());){var ee=L.callback;if(typeof ee=="function"){L.callback=null,M=L.priorityLevel;var pe=ee(L.expirationTime<=B);B=t.unstable_now(),typeof pe=="function"?L.callback=pe:L===S(C)&&k(C),E(B)}else k(C);L=S(C)}if(L!==null)var J=!0;else{var ue=S(R);ue!==null&&n(D,ue.startTime-B),J=!1}return J}finally{L=null,M=Z,q=!1}}var ae=i;t.unstable_IdlePriority=5,t.unstable_ImmediatePriority=1,t.unstable_LowPriority=4,t.unstable_NormalPriority=3,t.unstable_Profiling=null,t.unstable_UserBlockingPriority=2,t.unstable_cancelCallback=function(W){W.callback=null},t.unstable_continueExecution=function(){F||q||(F=!0,e(X))},t.unstable_getCurrentPriorityLevel=function(){return M},t.unstable_getFirstCallbackNode=function(){return S(C)},t.unstable_next=function(W){switch(M){case 1:case 2:case 3:var B=3;break;default:B=M}var Z=M;M=B;try{return W()}finally{M=Z}},t.unstable_pauseExecution=function(){},t.unstable_requestPaint=ae,t.unstable_runWithPriority=function(W,B){switch(W){case 1:case 2:case 3:case 4:case 5:break;default:W=3}var Z=M;M=W;try{return B()}finally{M=Z}},t.unstable_scheduleCallback=function(W,B,Z){var ee=t.unstable_now();switch(typeof Z=="object"&&Z!==null?(Z=Z.delay,Z=typeof Z=="number"&&0<Z?ee+Z:ee):Z=ee,W){case 1:var pe=-1;break;case 2:pe=250;break;case 5:pe=1073741823;break;case 4:pe=1e4;break;default:pe=5e3}return pe=Z+pe,W={id:T++,callback:B,priorityLevel:W,startTime:Z,expirationTime:pe,sortIndex:-1},Z>ee?(W.sortIndex=Z,b(R,W),S(C)===null&&W===S(R)&&(Q?r():Q=!0,n(D,Z-ee))):(W.sortIndex=pe,b(C,W),F||q||(F=!0,e(X))),W},t.unstable_wrapCallback=function(W){var B=M;return function(){var Z=M;M=B;try{return W.apply(this,arguments)}finally{M=Z}}}})(yy);gy.exports=yy;/** @license React v17.0.2
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var ic=w.exports,qe=D0,lt=gy.exports;function z(t){for(var e="https://reactjs.org/docs/error-decoder.html?invariant="+t,n=1;n<arguments.length;n++)e+="&args[]="+encodeURIComponent(arguments[n]);return"Minified React error #"+t+"; visit "+e+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}if(!ic)throw Error(z(227));var Oy=new Set,zs={};function Ci(t,e){ho(t,e),ho(t+"Capture",e)}function ho(t,e){for(zs[t]=e,t=0;t<e.length;t++)Oy.add(e[t])}var dr=!(typeof window=="undefined"||typeof window.document=="undefined"||typeof window.document.createElement=="undefined"),TC=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,vy=Object.prototype.hasOwnProperty,xy={},wy={};function IC(t){return vy.call(wy,t)?!0:vy.call(xy,t)?!1:TC.test(t)?wy[t]=!0:(xy[t]=!0,!1)}function EC(t,e,n,r){if(n!==null&&n.type===0)return!1;switch(typeof e){case"function":case"symbol":return!0;case"boolean":return r?!1:n!==null?!n.acceptsBooleans:(t=t.toLowerCase().slice(0,5),t!=="data-"&&t!=="aria-");default:return!1}}function MC(t,e,n,r){if(e===null||typeof e=="undefined"||EC(t,e,n,r))return!0;if(r)return!1;if(n!==null)switch(n.type){case 3:return!e;case 4:return e===!1;case 5:return isNaN(e);case 6:return isNaN(e)||1>e}return!1}function Mt(t,e,n,r,i,o,s){this.acceptsBooleans=e===2||e===3||e===4,this.attributeName=r,this.attributeNamespace=i,this.mustUseProperty=n,this.propertyName=t,this.type=e,this.sanitizeURL=o,this.removeEmptyString=s}var dt={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(t){dt[t]=new Mt(t,0,!1,t,null,!1,!1)});[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach(function(t){var e=t[0];dt[e]=new Mt(e,1,!1,t[1],null,!1,!1)});["contentEditable","draggable","spellCheck","value"].forEach(function(t){dt[t]=new Mt(t,2,!1,t.toLowerCase(),null,!1,!1)});["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach(function(t){dt[t]=new Mt(t,2,!1,t,null,!1,!1)});"allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture disableRemotePlayback formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(t){dt[t]=new Mt(t,3,!1,t.toLowerCase(),null,!1,!1)});["checked","multiple","muted","selected"].forEach(function(t){dt[t]=new Mt(t,3,!0,t,null,!1,!1)});["capture","download"].forEach(function(t){dt[t]=new Mt(t,4,!1,t,null,!1,!1)});["cols","rows","size","span"].forEach(function(t){dt[t]=new Mt(t,6,!1,t,null,!1,!1)});["rowSpan","start"].forEach(function(t){dt[t]=new Mt(t,5,!1,t.toLowerCase(),null,!1,!1)});var kh=/[\-:]([a-z])/g;function Ph(t){return t[1].toUpperCase()}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(t){var e=t.replace(kh,Ph);dt[e]=new Mt(e,1,!1,t,null,!1,!1)});"xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(t){var e=t.replace(kh,Ph);dt[e]=new Mt(e,1,!1,t,"http://www.w3.org/1999/xlink",!1,!1)});["xml:base","xml:lang","xml:space"].forEach(function(t){var e=t.replace(kh,Ph);dt[e]=new Mt(e,1,!1,t,"http://www.w3.org/XML/1998/namespace",!1,!1)});["tabIndex","crossOrigin"].forEach(function(t){dt[t]=new Mt(t,1,!1,t.toLowerCase(),null,!1,!1)});dt.xlinkHref=new Mt("xlinkHref",1,!1,"xlink:href","http://www.w3.org/1999/xlink",!0,!1);["src","href","action","formAction"].forEach(function(t){dt[t]=new Mt(t,1,!1,t.toLowerCase(),null,!0,!0)});function Ch(t,e,n,r){var i=dt.hasOwnProperty(e)?dt[e]:null,o=i!==null?i.type===0:r?!1:!(!(2<e.length)||e[0]!=="o"&&e[0]!=="O"||e[1]!=="n"&&e[1]!=="N");o||(MC(e,n,i,r)&&(n=null),r||i===null?IC(e)&&(n===null?t.removeAttribute(e):t.setAttribute(e,""+n)):i.mustUseProperty?t[i.propertyName]=n===null?i.type===3?!1:"":n:(e=i.attributeName,r=i.attributeNamespace,n===null?t.removeAttribute(e):(i=i.type,n=i===3||i===4&&n===!0?"":""+n,r?t.setAttributeNS(r,e,n):t.setAttribute(e,n))))}var Ri=ic.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,Fs=60103,Ti=60106,Br=60107,Rh=60108,Vs=60114,Th=60109,Ih=60110,oc=60112,Us=60113,sc=60120,lc=60115,Eh=60116,Mh=60121,Ah=60128,by=60129,Dh=60130,Lh=60131;if(typeof Symbol=="function"&&Symbol.for){var at=Symbol.for;Fs=at("react.element"),Ti=at("react.portal"),Br=at("react.fragment"),Rh=at("react.strict_mode"),Vs=at("react.profiler"),Th=at("react.provider"),Ih=at("react.context"),oc=at("react.forward_ref"),Us=at("react.suspense"),sc=at("react.suspense_list"),lc=at("react.memo"),Eh=at("react.lazy"),Mh=at("react.block"),at("react.scope"),Ah=at("react.opaque.id"),by=at("react.debug_trace_mode"),Dh=at("react.offscreen"),Lh=at("react.legacy_hidden")}var $y=typeof Symbol=="function"&&Symbol.iterator;function js(t){return t===null||typeof t!="object"?null:(t=$y&&t[$y]||t["@@iterator"],typeof t=="function"?t:null)}var Wh;function Xs(t){if(Wh===void 0)try{throw Error()}catch(n){var e=n.stack.trim().match(/\n( *(at )?)/);Wh=e&&e[1]||""}return`
`+Wh+t}var Qh=!1;function ac(t,e){if(!t||Qh)return"";Qh=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{if(e)if(e=function(){throw Error()},Object.defineProperty(e.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(e,[])}catch(a){var r=a}Reflect.construct(t,[],e)}else{try{e.call()}catch(a){r=a}t.call(e.prototype)}else{try{throw Error()}catch(a){r=a}t()}}catch(a){if(a&&r&&typeof a.stack=="string"){for(var i=a.stack.split(`
`),o=r.stack.split(`
`),s=i.length-1,l=o.length-1;1<=s&&0<=l&&i[s]!==o[l];)l--;for(;1<=s&&0<=l;s--,l--)if(i[s]!==o[l]){if(s!==1||l!==1)do if(s--,l--,0>l||i[s]!==o[l])return`
`+i[s].replace(" at new "," at ");while(1<=s&&0<=l);break}}}finally{Qh=!1,Error.prepareStackTrace=n}return(t=t?t.displayName||t.name:"")?Xs(t):""}function AC(t){switch(t.tag){case 5:return Xs(t.type);case 16:return Xs("Lazy");case 13:return Xs("Suspense");case 19:return Xs("SuspenseList");case 0:case 2:case 15:return t=ac(t.type,!1),t;case 11:return t=ac(t.type.render,!1),t;case 22:return t=ac(t.type._render,!1),t;case 1:return t=ac(t.type,!0),t;default:return""}}function po(t){if(t==null)return null;if(typeof t=="function")return t.displayName||t.name||null;if(typeof t=="string")return t;switch(t){case Br:return"Fragment";case Ti:return"Portal";case Vs:return"Profiler";case Rh:return"StrictMode";case Us:return"Suspense";case sc:return"SuspenseList"}if(typeof t=="object")switch(t.$$typeof){case Ih:return(t.displayName||"Context")+".Consumer";case Th:return(t._context.displayName||"Context")+".Provider";case oc:var e=t.render;return e=e.displayName||e.name||"",t.displayName||(e!==""?"ForwardRef("+e+")":"ForwardRef");case lc:return po(t.type);case Mh:return po(t._render);case Eh:e=t._payload,t=t._init;try{return po(t(e))}catch{}}return null}function qr(t){switch(typeof t){case"boolean":case"number":case"object":case"string":case"undefined":return t;default:return""}}function Sy(t){var e=t.type;return(t=t.nodeName)&&t.toLowerCase()==="input"&&(e==="checkbox"||e==="radio")}function DC(t){var e=Sy(t)?"checked":"value",n=Object.getOwnPropertyDescriptor(t.constructor.prototype,e),r=""+t[e];if(!t.hasOwnProperty(e)&&typeof n!="undefined"&&typeof n.get=="function"&&typeof n.set=="function"){var i=n.get,o=n.set;return Object.defineProperty(t,e,{configurable:!0,get:function(){return i.call(this)},set:function(s){r=""+s,o.call(this,s)}}),Object.defineProperty(t,e,{enumerable:n.enumerable}),{getValue:function(){return r},setValue:function(s){r=""+s},stopTracking:function(){t._valueTracker=null,delete t[e]}}}}function cc(t){t._valueTracker||(t._valueTracker=DC(t))}function ky(t){if(!t)return!1;var e=t._valueTracker;if(!e)return!0;var n=e.getValue(),r="";return t&&(r=Sy(t)?t.checked?"true":"false":t.value),t=r,t!==n?(e.setValue(t),!0):!1}function uc(t){if(t=t||(typeof document!="undefined"?document:void 0),typeof t=="undefined")return null;try{return t.activeElement||t.body}catch{return t.body}}function Nh(t,e){var n=e.checked;return qe({},e,{defaultChecked:void 0,defaultValue:void 0,value:void 0,checked:n!=null?n:t._wrapperState.initialChecked})}function Py(t,e){var n=e.defaultValue==null?"":e.defaultValue,r=e.checked!=null?e.checked:e.defaultChecked;n=qr(e.value!=null?e.value:n),t._wrapperState={initialChecked:r,initialValue:n,controlled:e.type==="checkbox"||e.type==="radio"?e.checked!=null:e.value!=null}}function Cy(t,e){e=e.checked,e!=null&&Ch(t,"checked",e,!1)}function _h(t,e){Cy(t,e);var n=qr(e.value),r=e.type;if(n!=null)r==="number"?(n===0&&t.value===""||t.value!=n)&&(t.value=""+n):t.value!==""+n&&(t.value=""+n);else if(r==="submit"||r==="reset"){t.removeAttribute("value");return}e.hasOwnProperty("value")?Bh(t,e.type,n):e.hasOwnProperty("defaultValue")&&Bh(t,e.type,qr(e.defaultValue)),e.checked==null&&e.defaultChecked!=null&&(t.defaultChecked=!!e.defaultChecked)}function Ry(t,e,n){if(e.hasOwnProperty("value")||e.hasOwnProperty("defaultValue")){var r=e.type;if(!(r!=="submit"&&r!=="reset"||e.value!==void 0&&e.value!==null))return;e=""+t._wrapperState.initialValue,n||e===t.value||(t.value=e),t.defaultValue=e}n=t.name,n!==""&&(t.name=""),t.defaultChecked=!!t._wrapperState.initialChecked,n!==""&&(t.name=n)}function Bh(t,e,n){(e!=="number"||uc(t.ownerDocument)!==t)&&(n==null?t.defaultValue=""+t._wrapperState.initialValue:t.defaultValue!==""+n&&(t.defaultValue=""+n))}function LC(t){var e="";return ic.Children.forEach(t,function(n){n!=null&&(e+=n)}),e}function qh(t,e){return t=qe({children:void 0},e),(e=LC(e.children))&&(t.children=e),t}function mo(t,e,n,r){if(t=t.options,e){e={};for(var i=0;i<n.length;i++)e["$"+n[i]]=!0;for(n=0;n<t.length;n++)i=e.hasOwnProperty("$"+t[n].value),t[n].selected!==i&&(t[n].selected=i),i&&r&&(t[n].defaultSelected=!0)}else{for(n=""+qr(n),e=null,i=0;i<t.length;i++){if(t[i].value===n){t[i].selected=!0,r&&(t[i].defaultSelected=!0);return}e!==null||t[i].disabled||(e=t[i])}e!==null&&(e.selected=!0)}}function zh(t,e){if(e.dangerouslySetInnerHTML!=null)throw Error(z(91));return qe({},e,{value:void 0,defaultValue:void 0,children:""+t._wrapperState.initialValue})}function Ty(t,e){var n=e.value;if(n==null){if(n=e.children,e=e.defaultValue,n!=null){if(e!=null)throw Error(z(92));if(Array.isArray(n)){if(!(1>=n.length))throw Error(z(93));n=n[0]}e=n}e==null&&(e=""),n=e}t._wrapperState={initialValue:qr(n)}}function Iy(t,e){var n=qr(e.value),r=qr(e.defaultValue);n!=null&&(n=""+n,n!==t.value&&(t.value=n),e.defaultValue==null&&t.defaultValue!==n&&(t.defaultValue=n)),r!=null&&(t.defaultValue=""+r)}function Ey(t){var e=t.textContent;e===t._wrapperState.initialValue&&e!==""&&e!==null&&(t.value=e)}var Fh={html:"http://www.w3.org/1999/xhtml",mathml:"http://www.w3.org/1998/Math/MathML",svg:"http://www.w3.org/2000/svg"};function My(t){switch(t){case"svg":return"http://www.w3.org/2000/svg";case"math":return"http://www.w3.org/1998/Math/MathML";default:return"http://www.w3.org/1999/xhtml"}}function Vh(t,e){return t==null||t==="http://www.w3.org/1999/xhtml"?My(e):t==="http://www.w3.org/2000/svg"&&e==="foreignObject"?"http://www.w3.org/1999/xhtml":t}var fc,Ay=function(t){return typeof MSApp!="undefined"&&MSApp.execUnsafeLocalFunction?function(e,n,r,i){MSApp.execUnsafeLocalFunction(function(){return t(e,n,r,i)})}:t}(function(t,e){if(t.namespaceURI!==Fh.svg||"innerHTML"in t)t.innerHTML=e;else{for(fc=fc||document.createElement("div"),fc.innerHTML="<svg>"+e.valueOf().toString()+"</svg>",e=fc.firstChild;t.firstChild;)t.removeChild(t.firstChild);for(;e.firstChild;)t.appendChild(e.firstChild)}});function Hs(t,e){if(e){var n=t.firstChild;if(n&&n===t.lastChild&&n.nodeType===3){n.nodeValue=e;return}}t.textContent=e}var Gs={animationIterationCount:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},WC=["Webkit","ms","Moz","O"];Object.keys(Gs).forEach(function(t){WC.forEach(function(e){e=e+t.charAt(0).toUpperCase()+t.substring(1),Gs[e]=Gs[t]})});function Dy(t,e,n){return e==null||typeof e=="boolean"||e===""?"":n||typeof e!="number"||e===0||Gs.hasOwnProperty(t)&&Gs[t]?(""+e).trim():e+"px"}function Ly(t,e){t=t.style;for(var n in e)if(e.hasOwnProperty(n)){var r=n.indexOf("--")===0,i=Dy(n,e[n],r);n==="float"&&(n="cssFloat"),r?t.setProperty(n,i):t[n]=i}}var QC=qe({menuitem:!0},{area:!0,base:!0,br:!0,col:!0,embed:!0,hr:!0,img:!0,input:!0,keygen:!0,link:!0,meta:!0,param:!0,source:!0,track:!0,wbr:!0});function Uh(t,e){if(e){if(QC[t]&&(e.children!=null||e.dangerouslySetInnerHTML!=null))throw Error(z(137,t));if(e.dangerouslySetInnerHTML!=null){if(e.children!=null)throw Error(z(60));if(!(typeof e.dangerouslySetInnerHTML=="object"&&"__html"in e.dangerouslySetInnerHTML))throw Error(z(61))}if(e.style!=null&&typeof e.style!="object")throw Error(z(62))}}function jh(t,e){if(t.indexOf("-")===-1)return typeof e.is=="string";switch(t){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}function Xh(t){return t=t.target||t.srcElement||window,t.correspondingUseElement&&(t=t.correspondingUseElement),t.nodeType===3?t.parentNode:t}var Hh=null,go=null,yo=null;function Wy(t){if(t=pl(t)){if(typeof Hh!="function")throw Error(z(280));var e=t.stateNode;e&&(e=Tc(e),Hh(t.stateNode,t.type,e))}}function Qy(t){go?yo?yo.push(t):yo=[t]:go=t}function Ny(){if(go){var t=go,e=yo;if(yo=go=null,Wy(t),e)for(t=0;t<e.length;t++)Wy(e[t])}}function Gh(t,e){return t(e)}function _y(t,e,n,r,i){return t(e,n,r,i)}function Zh(){}var By=Gh,Ii=!1,Yh=!1;function Kh(){(go!==null||yo!==null)&&(Zh(),Ny())}function NC(t,e,n){if(Yh)return t(e,n);Yh=!0;try{return By(t,e,n)}finally{Yh=!1,Kh()}}function Zs(t,e){var n=t.stateNode;if(n===null)return null;var r=Tc(n);if(r===null)return null;n=r[e];e:switch(e){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(r=!r.disabled)||(t=t.type,r=!(t==="button"||t==="input"||t==="select"||t==="textarea")),t=!r;break e;default:t=!1}if(t)return null;if(n&&typeof n!="function")throw Error(z(231,e,typeof n));return n}var Jh=!1;if(dr)try{var Ys={};Object.defineProperty(Ys,"passive",{get:function(){Jh=!0}}),window.addEventListener("test",Ys,Ys),window.removeEventListener("test",Ys,Ys)}catch{Jh=!1}function _C(t,e,n,r,i,o,s,l,a){var c=Array.prototype.slice.call(arguments,3);try{e.apply(n,c)}catch(u){this.onError(u)}}var Ks=!1,hc=null,dc=!1,ed=null,BC={onError:function(t){Ks=!0,hc=t}};function qC(t,e,n,r,i,o,s,l,a){Ks=!1,hc=null,_C.apply(BC,arguments)}function zC(t,e,n,r,i,o,s,l,a){if(qC.apply(this,arguments),Ks){if(Ks){var c=hc;Ks=!1,hc=null}else throw Error(z(198));dc||(dc=!0,ed=c)}}function Ei(t){var e=t,n=t;if(t.alternate)for(;e.return;)e=e.return;else{t=e;do e=t,(e.flags&1026)!=0&&(n=e.return),t=e.return;while(t)}return e.tag===3?n:null}function qy(t){if(t.tag===13){var e=t.memoizedState;if(e===null&&(t=t.alternate,t!==null&&(e=t.memoizedState)),e!==null)return e.dehydrated}return null}function zy(t){if(Ei(t)!==t)throw Error(z(188))}function FC(t){var e=t.alternate;if(!e){if(e=Ei(t),e===null)throw Error(z(188));return e!==t?null:t}for(var n=t,r=e;;){var i=n.return;if(i===null)break;var o=i.alternate;if(o===null){if(r=i.return,r!==null){n=r;continue}break}if(i.child===o.child){for(o=i.child;o;){if(o===n)return zy(i),t;if(o===r)return zy(i),e;o=o.sibling}throw Error(z(188))}if(n.return!==r.return)n=i,r=o;else{for(var s=!1,l=i.child;l;){if(l===n){s=!0,n=i,r=o;break}if(l===r){s=!0,r=i,n=o;break}l=l.sibling}if(!s){for(l=o.child;l;){if(l===n){s=!0,n=o,r=i;break}if(l===r){s=!0,r=o,n=i;break}l=l.sibling}if(!s)throw Error(z(189))}}if(n.alternate!==r)throw Error(z(190))}if(n.tag!==3)throw Error(z(188));return n.stateNode.current===n?t:e}function Fy(t){if(t=FC(t),!t)return null;for(var e=t;;){if(e.tag===5||e.tag===6)return e;if(e.child)e.child.return=e,e=e.child;else{if(e===t)break;for(;!e.sibling;){if(!e.return||e.return===t)return null;e=e.return}e.sibling.return=e.return,e=e.sibling}}return null}function Vy(t,e){for(var n=t.alternate;e!==null;){if(e===t||e===n)return!0;e=e.return}return!1}var Uy,td,jy,Xy,nd=!1,Bn=[],zr=null,Fr=null,Vr=null,Js=new Map,el=new Map,tl=[],Hy="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset submit".split(" ");function rd(t,e,n,r,i){return{blockedOn:t,domEventName:e,eventSystemFlags:n|16,nativeEvent:i,targetContainers:[r]}}function Gy(t,e){switch(t){case"focusin":case"focusout":zr=null;break;case"dragenter":case"dragleave":Fr=null;break;case"mouseover":case"mouseout":Vr=null;break;case"pointerover":case"pointerout":Js.delete(e.pointerId);break;case"gotpointercapture":case"lostpointercapture":el.delete(e.pointerId)}}function nl(t,e,n,r,i,o){return t===null||t.nativeEvent!==o?(t=rd(e,n,r,i,o),e!==null&&(e=pl(e),e!==null&&td(e)),t):(t.eventSystemFlags|=r,e=t.targetContainers,i!==null&&e.indexOf(i)===-1&&e.push(i),t)}function VC(t,e,n,r,i){switch(e){case"focusin":return zr=nl(zr,t,e,n,r,i),!0;case"dragenter":return Fr=nl(Fr,t,e,n,r,i),!0;case"mouseover":return Vr=nl(Vr,t,e,n,r,i),!0;case"pointerover":var o=i.pointerId;return Js.set(o,nl(Js.get(o)||null,t,e,n,r,i)),!0;case"gotpointercapture":return o=i.pointerId,el.set(o,nl(el.get(o)||null,t,e,n,r,i)),!0}return!1}function UC(t){var e=Mi(t.target);if(e!==null){var n=Ei(e);if(n!==null){if(e=n.tag,e===13){if(e=qy(n),e!==null){t.blockedOn=e,Xy(t.lanePriority,function(){lt.unstable_runWithPriority(t.priority,function(){jy(n)})});return}}else if(e===3&&n.stateNode.hydrate){t.blockedOn=n.tag===3?n.stateNode.containerInfo:null;return}}}t.blockedOn=null}function pc(t){if(t.blockedOn!==null)return!1;for(var e=t.targetContainers;0<e.length;){var n=cd(t.domEventName,t.eventSystemFlags,e[0],t.nativeEvent);if(n!==null)return e=pl(n),e!==null&&td(e),t.blockedOn=n,!1;e.shift()}return!0}function Zy(t,e,n){pc(t)&&n.delete(e)}function jC(){for(nd=!1;0<Bn.length;){var t=Bn[0];if(t.blockedOn!==null){t=pl(t.blockedOn),t!==null&&Uy(t);break}for(var e=t.targetContainers;0<e.length;){var n=cd(t.domEventName,t.eventSystemFlags,e[0],t.nativeEvent);if(n!==null){t.blockedOn=n;break}e.shift()}t.blockedOn===null&&Bn.shift()}zr!==null&&pc(zr)&&(zr=null),Fr!==null&&pc(Fr)&&(Fr=null),Vr!==null&&pc(Vr)&&(Vr=null),Js.forEach(Zy),el.forEach(Zy)}function rl(t,e){t.blockedOn===e&&(t.blockedOn=null,nd||(nd=!0,lt.unstable_scheduleCallback(lt.unstable_NormalPriority,jC)))}function Yy(t){function e(i){return rl(i,t)}if(0<Bn.length){rl(Bn[0],t);for(var n=1;n<Bn.length;n++){var r=Bn[n];r.blockedOn===t&&(r.blockedOn=null)}}for(zr!==null&&rl(zr,t),Fr!==null&&rl(Fr,t),Vr!==null&&rl(Vr,t),Js.forEach(e),el.forEach(e),n=0;n<tl.length;n++)r=tl[n],r.blockedOn===t&&(r.blockedOn=null);for(;0<tl.length&&(n=tl[0],n.blockedOn===null);)UC(n),n.blockedOn===null&&tl.shift()}function mc(t,e){var n={};return n[t.toLowerCase()]=e.toLowerCase(),n["Webkit"+t]="webkit"+e,n["Moz"+t]="moz"+e,n}var Oo={animationend:mc("Animation","AnimationEnd"),animationiteration:mc("Animation","AnimationIteration"),animationstart:mc("Animation","AnimationStart"),transitionend:mc("Transition","TransitionEnd")},id={},Ky={};dr&&(Ky=document.createElement("div").style,"AnimationEvent"in window||(delete Oo.animationend.animation,delete Oo.animationiteration.animation,delete Oo.animationstart.animation),"TransitionEvent"in window||delete Oo.transitionend.transition);function gc(t){if(id[t])return id[t];if(!Oo[t])return t;var e=Oo[t],n;for(n in e)if(e.hasOwnProperty(n)&&n in Ky)return id[t]=e[n];return t}var Jy=gc("animationend"),eO=gc("animationiteration"),tO=gc("animationstart"),nO=gc("transitionend"),rO=new Map,od=new Map,XC=["abort","abort",Jy,"animationEnd",eO,"animationIteration",tO,"animationStart","canplay","canPlay","canplaythrough","canPlayThrough","durationchange","durationChange","emptied","emptied","encrypted","encrypted","ended","ended","error","error","gotpointercapture","gotPointerCapture","load","load","loadeddata","loadedData","loadedmetadata","loadedMetadata","loadstart","loadStart","lostpointercapture","lostPointerCapture","playing","playing","progress","progress","seeking","seeking","stalled","stalled","suspend","suspend","timeupdate","timeUpdate",nO,"transitionEnd","waiting","waiting"];function sd(t,e){for(var n=0;n<t.length;n+=2){var r=t[n],i=t[n+1];i="on"+(i[0].toUpperCase()+i.slice(1)),od.set(r,e),rO.set(r,i),Ci(i,[r])}}var HC=lt.unstable_now;HC();var Ae=8;function vo(t){if((1&t)!=0)return Ae=15,1;if((2&t)!=0)return Ae=14,2;if((4&t)!=0)return Ae=13,4;var e=24&t;return e!==0?(Ae=12,e):(t&32)!=0?(Ae=11,32):(e=192&t,e!==0?(Ae=10,e):(t&256)!=0?(Ae=9,256):(e=3584&t,e!==0?(Ae=8,e):(t&4096)!=0?(Ae=7,4096):(e=4186112&t,e!==0?(Ae=6,e):(e=62914560&t,e!==0?(Ae=5,e):t&67108864?(Ae=4,67108864):(t&134217728)!=0?(Ae=3,134217728):(e=805306368&t,e!==0?(Ae=2,e):(1073741824&t)!=0?(Ae=1,1073741824):(Ae=8,t))))))}function GC(t){switch(t){case 99:return 15;case 98:return 10;case 97:case 96:return 8;case 95:return 2;default:return 0}}function ZC(t){switch(t){case 15:case 14:return 99;case 13:case 12:case 11:case 10:return 98;case 9:case 8:case 7:case 6:case 4:case 5:return 97;case 3:case 2:case 1:return 95;case 0:return 90;default:throw Error(z(358,t))}}function il(t,e){var n=t.pendingLanes;if(n===0)return Ae=0;var r=0,i=0,o=t.expiredLanes,s=t.suspendedLanes,l=t.pingedLanes;if(o!==0)r=o,i=Ae=15;else if(o=n&134217727,o!==0){var a=o&~s;a!==0?(r=vo(a),i=Ae):(l&=o,l!==0&&(r=vo(l),i=Ae))}else o=n&~s,o!==0?(r=vo(o),i=Ae):l!==0&&(r=vo(l),i=Ae);if(r===0)return 0;if(r=31-Ur(r),r=n&((0>r?0:1<<r)<<1)-1,e!==0&&e!==r&&(e&s)==0){if(vo(e),i<=Ae)return e;Ae=i}if(e=t.entangledLanes,e!==0)for(t=t.entanglements,e&=r;0<e;)n=31-Ur(e),i=1<<n,r|=t[n],e&=~i;return r}function iO(t){return t=t.pendingLanes&-1073741825,t!==0?t:t&1073741824?1073741824:0}function yc(t,e){switch(t){case 15:return 1;case 14:return 2;case 12:return t=xo(24&~e),t===0?yc(10,e):t;case 10:return t=xo(192&~e),t===0?yc(8,e):t;case 8:return t=xo(3584&~e),t===0&&(t=xo(4186112&~e),t===0&&(t=512)),t;case 2:return e=xo(805306368&~e),e===0&&(e=268435456),e}throw Error(z(358,t))}function xo(t){return t&-t}function ld(t){for(var e=[],n=0;31>n;n++)e.push(t);return e}function Oc(t,e,n){t.pendingLanes|=e;var r=e-1;t.suspendedLanes&=r,t.pingedLanes&=r,t=t.eventTimes,e=31-Ur(e),t[e]=n}var Ur=Math.clz32?Math.clz32:JC,YC=Math.log,KC=Math.LN2;function JC(t){return t===0?32:31-(YC(t)/KC|0)|0}var eR=lt.unstable_UserBlockingPriority,tR=lt.unstable_runWithPriority,vc=!0;function nR(t,e,n,r){Ii||Zh();var i=ad,o=Ii;Ii=!0;try{_y(i,t,e,n,r)}finally{(Ii=o)||Kh()}}function rR(t,e,n,r){tR(eR,ad.bind(null,t,e,n,r))}function ad(t,e,n,r){if(vc){var i;if((i=(e&4)==0)&&0<Bn.length&&-1<Hy.indexOf(t))t=rd(null,t,e,n,r),Bn.push(t);else{var o=cd(t,e,n,r);if(o===null)i&&Gy(t,r);else{if(i){if(-1<Hy.indexOf(t)){t=rd(o,t,e,n,r),Bn.push(t);return}if(VC(o,t,e,n,r))return;Gy(t,r)}DO(t,e,r,null,n)}}}}function cd(t,e,n,r){var i=Xh(r);if(i=Mi(i),i!==null){var o=Ei(i);if(o===null)i=null;else{var s=o.tag;if(s===13){if(i=qy(o),i!==null)return i;i=null}else if(s===3){if(o.stateNode.hydrate)return o.tag===3?o.stateNode.containerInfo:null;i=null}else o!==i&&(i=null)}}return DO(t,e,r,i,n),null}var jr=null,ud=null,xc=null;function oO(){if(xc)return xc;var t,e=ud,n=e.length,r,i="value"in jr?jr.value:jr.textContent,o=i.length;for(t=0;t<n&&e[t]===i[t];t++);var s=n-t;for(r=1;r<=s&&e[n-r]===i[o-r];r++);return xc=i.slice(t,1<r?1-r:void 0)}function wc(t){var e=t.keyCode;return"charCode"in t?(t=t.charCode,t===0&&e===13&&(t=13)):t=e,t===10&&(t=13),32<=t||t===13?t:0}function bc(){return!0}function sO(){return!1}function Jt(t){function e(n,r,i,o,s){this._reactName=n,this._targetInst=i,this.type=r,this.nativeEvent=o,this.target=s,this.currentTarget=null;for(var l in t)t.hasOwnProperty(l)&&(n=t[l],this[l]=n?n(o):o[l]);return this.isDefaultPrevented=(o.defaultPrevented!=null?o.defaultPrevented:o.returnValue===!1)?bc:sO,this.isPropagationStopped=sO,this}return qe(e.prototype,{preventDefault:function(){this.defaultPrevented=!0;var n=this.nativeEvent;n&&(n.preventDefault?n.preventDefault():typeof n.returnValue!="unknown"&&(n.returnValue=!1),this.isDefaultPrevented=bc)},stopPropagation:function(){var n=this.nativeEvent;n&&(n.stopPropagation?n.stopPropagation():typeof n.cancelBubble!="unknown"&&(n.cancelBubble=!0),this.isPropagationStopped=bc)},persist:function(){},isPersistent:bc}),e}var wo={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(t){return t.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},fd=Jt(wo),ol=qe({},wo,{view:0,detail:0}),iR=Jt(ol),hd,dd,sl,$c=qe({},ol,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:md,button:0,buttons:0,relatedTarget:function(t){return t.relatedTarget===void 0?t.fromElement===t.srcElement?t.toElement:t.fromElement:t.relatedTarget},movementX:function(t){return"movementX"in t?t.movementX:(t!==sl&&(sl&&t.type==="mousemove"?(hd=t.screenX-sl.screenX,dd=t.screenY-sl.screenY):dd=hd=0,sl=t),hd)},movementY:function(t){return"movementY"in t?t.movementY:dd}}),lO=Jt($c),oR=qe({},$c,{dataTransfer:0}),sR=Jt(oR),lR=qe({},ol,{relatedTarget:0}),pd=Jt(lR),aR=qe({},wo,{animationName:0,elapsedTime:0,pseudoElement:0}),cR=Jt(aR),uR=qe({},wo,{clipboardData:function(t){return"clipboardData"in t?t.clipboardData:window.clipboardData}}),fR=Jt(uR),hR=qe({},wo,{data:0}),aO=Jt(hR),dR={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},pR={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},mR={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function gR(t){var e=this.nativeEvent;return e.getModifierState?e.getModifierState(t):(t=mR[t])?!!e[t]:!1}function md(){return gR}var yR=qe({},ol,{key:function(t){if(t.key){var e=dR[t.key]||t.key;if(e!=="Unidentified")return e}return t.type==="keypress"?(t=wc(t),t===13?"Enter":String.fromCharCode(t)):t.type==="keydown"||t.type==="keyup"?pR[t.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:md,charCode:function(t){return t.type==="keypress"?wc(t):0},keyCode:function(t){return t.type==="keydown"||t.type==="keyup"?t.keyCode:0},which:function(t){return t.type==="keypress"?wc(t):t.type==="keydown"||t.type==="keyup"?t.keyCode:0}}),OR=Jt(yR),vR=qe({},$c,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),cO=Jt(vR),xR=qe({},ol,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:md}),wR=Jt(xR),bR=qe({},wo,{propertyName:0,elapsedTime:0,pseudoElement:0}),$R=Jt(bR),SR=qe({},$c,{deltaX:function(t){return"deltaX"in t?t.deltaX:"wheelDeltaX"in t?-t.wheelDeltaX:0},deltaY:function(t){return"deltaY"in t?t.deltaY:"wheelDeltaY"in t?-t.wheelDeltaY:"wheelDelta"in t?-t.wheelDelta:0},deltaZ:0,deltaMode:0}),kR=Jt(SR),PR=[9,13,27,32],gd=dr&&"CompositionEvent"in window,ll=null;dr&&"documentMode"in document&&(ll=document.documentMode);var CR=dr&&"TextEvent"in window&&!ll,uO=dr&&(!gd||ll&&8<ll&&11>=ll),fO=String.fromCharCode(32),hO=!1;function dO(t,e){switch(t){case"keyup":return PR.indexOf(e.keyCode)!==-1;case"keydown":return e.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function pO(t){return t=t.detail,typeof t=="object"&&"data"in t?t.data:null}var bo=!1;function RR(t,e){switch(t){case"compositionend":return pO(e);case"keypress":return e.which!==32?null:(hO=!0,fO);case"textInput":return t=e.data,t===fO&&hO?null:t;default:return null}}function TR(t,e){if(bo)return t==="compositionend"||!gd&&dO(t,e)?(t=oO(),xc=ud=jr=null,bo=!1,t):null;switch(t){case"paste":return null;case"keypress":if(!(e.ctrlKey||e.altKey||e.metaKey)||e.ctrlKey&&e.altKey){if(e.char&&1<e.char.length)return e.char;if(e.which)return String.fromCharCode(e.which)}return null;case"compositionend":return uO&&e.locale!=="ko"?null:e.data;default:return null}}var IR={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function mO(t){var e=t&&t.nodeName&&t.nodeName.toLowerCase();return e==="input"?!!IR[t.type]:e==="textarea"}function gO(t,e,n,r){Qy(r),e=kc(e,"onChange"),0<e.length&&(n=new fd("onChange","change",null,n,r),t.push({event:n,listeners:e}))}var al=null,cl=null;function ER(t){TO(t,0)}function Sc(t){var e=Co(t);if(ky(e))return t}function MR(t,e){if(t==="change")return e}var yO=!1;if(dr){var yd;if(dr){var Od="oninput"in document;if(!Od){var OO=document.createElement("div");OO.setAttribute("oninput","return;"),Od=typeof OO.oninput=="function"}yd=Od}else yd=!1;yO=yd&&(!document.documentMode||9<document.documentMode)}function vO(){al&&(al.detachEvent("onpropertychange",xO),cl=al=null)}function xO(t){if(t.propertyName==="value"&&Sc(cl)){var e=[];if(gO(e,cl,t,Xh(t)),t=ER,Ii)t(e);else{Ii=!0;try{Gh(t,e)}finally{Ii=!1,Kh()}}}}function AR(t,e,n){t==="focusin"?(vO(),al=e,cl=n,al.attachEvent("onpropertychange",xO)):t==="focusout"&&vO()}function DR(t){if(t==="selectionchange"||t==="keyup"||t==="keydown")return Sc(cl)}function LR(t,e){if(t==="click")return Sc(e)}function WR(t,e){if(t==="input"||t==="change")return Sc(e)}function QR(t,e){return t===e&&(t!==0||1/t==1/e)||t!==t&&e!==e}var hn=typeof Object.is=="function"?Object.is:QR,NR=Object.prototype.hasOwnProperty;function ul(t,e){if(hn(t,e))return!0;if(typeof t!="object"||t===null||typeof e!="object"||e===null)return!1;var n=Object.keys(t),r=Object.keys(e);if(n.length!==r.length)return!1;for(r=0;r<n.length;r++)if(!NR.call(e,n[r])||!hn(t[n[r]],e[n[r]]))return!1;return!0}function wO(t){for(;t&&t.firstChild;)t=t.firstChild;return t}function bO(t,e){var n=wO(t);t=0;for(var r;n;){if(n.nodeType===3){if(r=t+n.textContent.length,t<=e&&r>=e)return{node:n,offset:e-t};t=r}e:{for(;n;){if(n.nextSibling){n=n.nextSibling;break e}n=n.parentNode}n=void 0}n=wO(n)}}function $O(t,e){return t&&e?t===e?!0:t&&t.nodeType===3?!1:e&&e.nodeType===3?$O(t,e.parentNode):"contains"in t?t.contains(e):t.compareDocumentPosition?!!(t.compareDocumentPosition(e)&16):!1:!1}function SO(){for(var t=window,e=uc();e instanceof t.HTMLIFrameElement;){try{var n=typeof e.contentWindow.location.href=="string"}catch{n=!1}if(n)t=e.contentWindow;else break;e=uc(t.document)}return e}function vd(t){var e=t&&t.nodeName&&t.nodeName.toLowerCase();return e&&(e==="input"&&(t.type==="text"||t.type==="search"||t.type==="tel"||t.type==="url"||t.type==="password")||e==="textarea"||t.contentEditable==="true")}var _R=dr&&"documentMode"in document&&11>=document.documentMode,$o=null,xd=null,fl=null,wd=!1;function kO(t,e,n){var r=n.window===n?n.document:n.nodeType===9?n:n.ownerDocument;wd||$o==null||$o!==uc(r)||(r=$o,"selectionStart"in r&&vd(r)?r={start:r.selectionStart,end:r.selectionEnd}:(r=(r.ownerDocument&&r.ownerDocument.defaultView||window).getSelection(),r={anchorNode:r.anchorNode,anchorOffset:r.anchorOffset,focusNode:r.focusNode,focusOffset:r.focusOffset}),fl&&ul(fl,r)||(fl=r,r=kc(xd,"onSelect"),0<r.length&&(e=new fd("onSelect","select",null,e,n),t.push({event:e,listeners:r}),e.target=$o)))}sd("cancel cancel click click close close contextmenu contextMenu copy copy cut cut auxclick auxClick dblclick doubleClick dragend dragEnd dragstart dragStart drop drop focusin focus focusout blur input input invalid invalid keydown keyDown keypress keyPress keyup keyUp mousedown mouseDown mouseup mouseUp paste paste pause pause play play pointercancel pointerCancel pointerdown pointerDown pointerup pointerUp ratechange rateChange reset reset seeked seeked submit submit touchcancel touchCancel touchend touchEnd touchstart touchStart volumechange volumeChange".split(" "),0);sd("drag drag dragenter dragEnter dragexit dragExit dragleave dragLeave dragover dragOver mousemove mouseMove mouseout mouseOut mouseover mouseOver pointermove pointerMove pointerout pointerOut pointerover pointerOver scroll scroll toggle toggle touchmove touchMove wheel wheel".split(" "),1);sd(XC,2);for(var PO="change selectionchange textInput compositionstart compositionend compositionupdate".split(" "),bd=0;bd<PO.length;bd++)od.set(PO[bd],0);ho("onMouseEnter",["mouseout","mouseover"]);ho("onMouseLeave",["mouseout","mouseover"]);ho("onPointerEnter",["pointerout","pointerover"]);ho("onPointerLeave",["pointerout","pointerover"]);Ci("onChange","change click focusin focusout input keydown keyup selectionchange".split(" "));Ci("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" "));Ci("onBeforeInput",["compositionend","keypress","textInput","paste"]);Ci("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" "));Ci("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" "));Ci("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var hl="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),CO=new Set("cancel close invalid load scroll toggle".split(" ").concat(hl));function RO(t,e,n){var r=t.type||"unknown-event";t.currentTarget=n,zC(r,e,void 0,t),t.currentTarget=null}function TO(t,e){e=(e&4)!=0;for(var n=0;n<t.length;n++){var r=t[n],i=r.event;r=r.listeners;e:{var o=void 0;if(e)for(var s=r.length-1;0<=s;s--){var l=r[s],a=l.instance,c=l.currentTarget;if(l=l.listener,a!==o&&i.isPropagationStopped())break e;RO(i,l,c),o=a}else for(s=0;s<r.length;s++){if(l=r[s],a=l.instance,c=l.currentTarget,l=l.listener,a!==o&&i.isPropagationStopped())break e;RO(i,l,c),o=a}}}if(dc)throw t=ed,dc=!1,ed=null,t}function Le(t,e){var n=BO(e),r=t+"__bubble";n.has(r)||(AO(e,t,2,!1),n.add(r))}var IO="_reactListening"+Math.random().toString(36).slice(2);function EO(t){t[IO]||(t[IO]=!0,Oy.forEach(function(e){CO.has(e)||MO(e,!1,t,null),MO(e,!0,t,null)}))}function MO(t,e,n,r){var i=4<arguments.length&&arguments[4]!==void 0?arguments[4]:0,o=n;if(t==="selectionchange"&&n.nodeType!==9&&(o=n.ownerDocument),r!==null&&!e&&CO.has(t)){if(t!=="scroll")return;i|=2,o=r}var s=BO(o),l=t+"__"+(e?"capture":"bubble");s.has(l)||(e&&(i|=4),AO(o,t,i,e),s.add(l))}function AO(t,e,n,r){var i=od.get(e);switch(i===void 0?2:i){case 0:i=nR;break;case 1:i=rR;break;default:i=ad}n=i.bind(null,e,n,t),i=void 0,!Jh||e!=="touchstart"&&e!=="touchmove"&&e!=="wheel"||(i=!0),r?i!==void 0?t.addEventListener(e,n,{capture:!0,passive:i}):t.addEventListener(e,n,!0):i!==void 0?t.addEventListener(e,n,{passive:i}):t.addEventListener(e,n,!1)}function DO(t,e,n,r,i){var o=r;if((e&1)==0&&(e&2)==0&&r!==null)e:for(;;){if(r===null)return;var s=r.tag;if(s===3||s===4){var l=r.stateNode.containerInfo;if(l===i||l.nodeType===8&&l.parentNode===i)break;if(s===4)for(s=r.return;s!==null;){var a=s.tag;if((a===3||a===4)&&(a=s.stateNode.containerInfo,a===i||a.nodeType===8&&a.parentNode===i))return;s=s.return}for(;l!==null;){if(s=Mi(l),s===null)return;if(a=s.tag,a===5||a===6){r=o=s;continue e}l=l.parentNode}}r=r.return}NC(function(){var c=o,u=Xh(n),f=[];e:{var h=rO.get(t);if(h!==void 0){var d=fd,p=t;switch(t){case"keypress":if(wc(n)===0)break e;case"keydown":case"keyup":d=OR;break;case"focusin":p="focus",d=pd;break;case"focusout":p="blur",d=pd;break;case"beforeblur":case"afterblur":d=pd;break;case"click":if(n.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":d=lO;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":d=sR;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":d=wR;break;case Jy:case eO:case tO:d=cR;break;case nO:d=$R;break;case"scroll":d=iR;break;case"wheel":d=kR;break;case"copy":case"cut":case"paste":d=fR;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":d=cO}var m=(e&4)!=0,g=!m&&t==="scroll",y=m?h!==null?h+"Capture":null:h;m=[];for(var O=c,v;O!==null;){v=O;var x=v.stateNode;if(v.tag===5&&x!==null&&(v=x,y!==null&&(x=Zs(O,y),x!=null&&m.push(dl(O,x,v)))),g)break;O=O.return}0<m.length&&(h=new d(h,p,null,n,u),f.push({event:h,listeners:m}))}}if((e&7)==0){e:{if(h=t==="mouseover"||t==="pointerover",d=t==="mouseout"||t==="pointerout",h&&(e&16)==0&&(p=n.relatedTarget||n.fromElement)&&(Mi(p)||p[Po]))break e;if((d||h)&&(h=u.window===u?u:(h=u.ownerDocument)?h.defaultView||h.parentWindow:window,d?(p=n.relatedTarget||n.toElement,d=c,p=p?Mi(p):null,p!==null&&(g=Ei(p),p!==g||p.tag!==5&&p.tag!==6)&&(p=null)):(d=null,p=c),d!==p)){if(m=lO,x="onMouseLeave",y="onMouseEnter",O="mouse",(t==="pointerout"||t==="pointerover")&&(m=cO,x="onPointerLeave",y="onPointerEnter",O="pointer"),g=d==null?h:Co(d),v=p==null?h:Co(p),h=new m(x,O+"leave",d,n,u),h.target=g,h.relatedTarget=v,x=null,Mi(u)===c&&(m=new m(y,O+"enter",p,n,u),m.target=v,m.relatedTarget=g,x=m),g=x,d&&p)t:{for(m=d,y=p,O=0,v=m;v;v=So(v))O++;for(v=0,x=y;x;x=So(x))v++;for(;0<O-v;)m=So(m),O--;for(;0<v-O;)y=So(y),v--;for(;O--;){if(m===y||y!==null&&m===y.alternate)break t;m=So(m),y=So(y)}m=null}else m=null;d!==null&&LO(f,h,d,m,!1),p!==null&&g!==null&&LO(f,g,p,m,!0)}}e:{if(h=c?Co(c):window,d=h.nodeName&&h.nodeName.toLowerCase(),d==="select"||d==="input"&&h.type==="file")var b=MR;else if(mO(h))if(yO)b=WR;else{b=DR;var S=AR}else(d=h.nodeName)&&d.toLowerCase()==="input"&&(h.type==="checkbox"||h.type==="radio")&&(b=LR);if(b&&(b=b(t,c))){gO(f,b,n,u);break e}S&&S(t,h,c),t==="focusout"&&(S=h._wrapperState)&&S.controlled&&h.type==="number"&&Bh(h,"number",h.value)}switch(S=c?Co(c):window,t){case"focusin":(mO(S)||S.contentEditable==="true")&&($o=S,xd=c,fl=null);break;case"focusout":fl=xd=$o=null;break;case"mousedown":wd=!0;break;case"contextmenu":case"mouseup":case"dragend":wd=!1,kO(f,n,u);break;case"selectionchange":if(_R)break;case"keydown":case"keyup":kO(f,n,u)}var k;if(gd)e:{switch(t){case"compositionstart":var P="onCompositionStart";break e;case"compositionend":P="onCompositionEnd";break e;case"compositionupdate":P="onCompositionUpdate";break e}P=void 0}else bo?dO(t,n)&&(P="onCompositionEnd"):t==="keydown"&&n.keyCode===229&&(P="onCompositionStart");P&&(uO&&n.locale!=="ko"&&(bo||P!=="onCompositionStart"?P==="onCompositionEnd"&&bo&&(k=oO()):(jr=u,ud="value"in jr?jr.value:jr.textContent,bo=!0)),S=kc(c,P),0<S.length&&(P=new aO(P,t,null,n,u),f.push({event:P,listeners:S}),k?P.data=k:(k=pO(n),k!==null&&(P.data=k)))),(k=CR?RR(t,n):TR(t,n))&&(c=kc(c,"onBeforeInput"),0<c.length&&(u=new aO("onBeforeInput","beforeinput",null,n,u),f.push({event:u,listeners:c}),u.data=k))}TO(f,e)})}function dl(t,e,n){return{instance:t,listener:e,currentTarget:n}}function kc(t,e){for(var n=e+"Capture",r=[];t!==null;){var i=t,o=i.stateNode;i.tag===5&&o!==null&&(i=o,o=Zs(t,n),o!=null&&r.unshift(dl(t,o,i)),o=Zs(t,e),o!=null&&r.push(dl(t,o,i))),t=t.return}return r}function So(t){if(t===null)return null;do t=t.return;while(t&&t.tag!==5);return t||null}function LO(t,e,n,r,i){for(var o=e._reactName,s=[];n!==null&&n!==r;){var l=n,a=l.alternate,c=l.stateNode;if(a!==null&&a===r)break;l.tag===5&&c!==null&&(l=c,i?(a=Zs(n,o),a!=null&&s.unshift(dl(n,a,l))):i||(a=Zs(n,o),a!=null&&s.push(dl(n,a,l)))),n=n.return}s.length!==0&&t.push({event:e,listeners:s})}function Pc(){}var $d=null,Sd=null;function WO(t,e){switch(t){case"button":case"input":case"select":case"textarea":return!!e.autoFocus}return!1}function kd(t,e){return t==="textarea"||t==="option"||t==="noscript"||typeof e.children=="string"||typeof e.children=="number"||typeof e.dangerouslySetInnerHTML=="object"&&e.dangerouslySetInnerHTML!==null&&e.dangerouslySetInnerHTML.__html!=null}var QO=typeof setTimeout=="function"?setTimeout:void 0,BR=typeof clearTimeout=="function"?clearTimeout:void 0;function Pd(t){t.nodeType===1?t.textContent="":t.nodeType===9&&(t=t.body,t!=null&&(t.textContent=""))}function ko(t){for(;t!=null;t=t.nextSibling){var e=t.nodeType;if(e===1||e===3)break}return t}function NO(t){t=t.previousSibling;for(var e=0;t;){if(t.nodeType===8){var n=t.data;if(n==="$"||n==="$!"||n==="$?"){if(e===0)return t;e--}else n==="/$"&&e++}t=t.previousSibling}return null}var Cd=0;function qR(t){return{$$typeof:Ah,toString:t,valueOf:t}}var Cc=Math.random().toString(36).slice(2),Xr="__reactFiber$"+Cc,Rc="__reactProps$"+Cc,Po="__reactContainer$"+Cc,_O="__reactEvents$"+Cc;function Mi(t){var e=t[Xr];if(e)return e;for(var n=t.parentNode;n;){if(e=n[Po]||n[Xr]){if(n=e.alternate,e.child!==null||n!==null&&n.child!==null)for(t=NO(t);t!==null;){if(n=t[Xr])return n;t=NO(t)}return e}t=n,n=t.parentNode}return null}function pl(t){return t=t[Xr]||t[Po],!t||t.tag!==5&&t.tag!==6&&t.tag!==13&&t.tag!==3?null:t}function Co(t){if(t.tag===5||t.tag===6)return t.stateNode;throw Error(z(33))}function Tc(t){return t[Rc]||null}function BO(t){var e=t[_O];return e===void 0&&(e=t[_O]=new Set),e}var Rd=[],Ro=-1;function Hr(t){return{current:t}}function We(t){0>Ro||(t.current=Rd[Ro],Rd[Ro]=null,Ro--)}function je(t,e){Ro++,Rd[Ro]=t.current,t.current=e}var Gr={},xt=Hr(Gr),Qt=Hr(!1),Ai=Gr;function To(t,e){var n=t.type.contextTypes;if(!n)return Gr;var r=t.stateNode;if(r&&r.__reactInternalMemoizedUnmaskedChildContext===e)return r.__reactInternalMemoizedMaskedChildContext;var i={},o;for(o in n)i[o]=e[o];return r&&(t=t.stateNode,t.__reactInternalMemoizedUnmaskedChildContext=e,t.__reactInternalMemoizedMaskedChildContext=i),i}function Nt(t){return t=t.childContextTypes,t!=null}function Ic(){We(Qt),We(xt)}function qO(t,e,n){if(xt.current!==Gr)throw Error(z(168));je(xt,e),je(Qt,n)}function zO(t,e,n){var r=t.stateNode;if(t=e.childContextTypes,typeof r.getChildContext!="function")return n;r=r.getChildContext();for(var i in r)if(!(i in t))throw Error(z(108,po(e)||"Unknown",i));return qe({},n,r)}function Ec(t){return t=(t=t.stateNode)&&t.__reactInternalMemoizedMergedChildContext||Gr,Ai=xt.current,je(xt,t),je(Qt,Qt.current),!0}function FO(t,e,n){var r=t.stateNode;if(!r)throw Error(z(169));n?(t=zO(t,e,Ai),r.__reactInternalMemoizedMergedChildContext=t,We(Qt),We(xt),je(xt,t)):We(Qt),je(Qt,n)}var Td=null,Di=null,zR=lt.unstable_runWithPriority,Id=lt.unstable_scheduleCallback,Ed=lt.unstable_cancelCallback,FR=lt.unstable_shouldYield,VO=lt.unstable_requestPaint,Md=lt.unstable_now,VR=lt.unstable_getCurrentPriorityLevel,Mc=lt.unstable_ImmediatePriority,UO=lt.unstable_UserBlockingPriority,jO=lt.unstable_NormalPriority,XO=lt.unstable_LowPriority,HO=lt.unstable_IdlePriority,Ad={},UR=VO!==void 0?VO:function(){},pr=null,Ac=null,Dd=!1,GO=Md(),wt=1e4>GO?Md:function(){return Md()-GO};function Io(){switch(VR()){case Mc:return 99;case UO:return 98;case jO:return 97;case XO:return 96;case HO:return 95;default:throw Error(z(332))}}function ZO(t){switch(t){case 99:return Mc;case 98:return UO;case 97:return jO;case 96:return XO;case 95:return HO;default:throw Error(z(332))}}function Li(t,e){return t=ZO(t),zR(t,e)}function ml(t,e,n){return t=ZO(t),Id(t,e,n)}function qn(){if(Ac!==null){var t=Ac;Ac=null,Ed(t)}YO()}function YO(){if(!Dd&&pr!==null){Dd=!0;var t=0;try{var e=pr;Li(99,function(){for(;t<e.length;t++){var n=e[t];do n=n(!0);while(n!==null)}}),pr=null}catch(n){throw pr!==null&&(pr=pr.slice(t+1)),Id(Mc,qn),n}finally{Dd=!1}}}var jR=Ri.ReactCurrentBatchConfig;function Mn(t,e){if(t&&t.defaultProps){e=qe({},e),t=t.defaultProps;for(var n in t)e[n]===void 0&&(e[n]=t[n]);return e}return e}var Dc=Hr(null),Lc=null,Eo=null,Wc=null;function Ld(){Wc=Eo=Lc=null}function Wd(t){var e=Dc.current;We(Dc),t.type._context._currentValue=e}function KO(t,e){for(;t!==null;){var n=t.alternate;if((t.childLanes&e)===e){if(n===null||(n.childLanes&e)===e)break;n.childLanes|=e}else t.childLanes|=e,n!==null&&(n.childLanes|=e);t=t.return}}function Mo(t,e){Lc=t,Wc=Eo=null,t=t.dependencies,t!==null&&t.firstContext!==null&&((t.lanes&e)!=0&&(An=!0),t.firstContext=null)}function dn(t,e){if(Wc!==t&&e!==!1&&e!==0)if((typeof e!="number"||e===1073741823)&&(Wc=t,e=1073741823),e={context:t,observedBits:e,next:null},Eo===null){if(Lc===null)throw Error(z(308));Eo=e,Lc.dependencies={lanes:0,firstContext:e,responders:null}}else Eo=Eo.next=e;return t._currentValue}var Zr=!1;function Qd(t){t.updateQueue={baseState:t.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null},effects:null}}function JO(t,e){t=t.updateQueue,e.updateQueue===t&&(e.updateQueue={baseState:t.baseState,firstBaseUpdate:t.firstBaseUpdate,lastBaseUpdate:t.lastBaseUpdate,shared:t.shared,effects:t.effects})}function Yr(t,e){return{eventTime:t,lane:e,tag:0,payload:null,callback:null,next:null}}function Kr(t,e){if(t=t.updateQueue,t!==null){t=t.shared;var n=t.pending;n===null?e.next=e:(e.next=n.next,n.next=e),t.pending=e}}function ev(t,e){var n=t.updateQueue,r=t.alternate;if(r!==null&&(r=r.updateQueue,n===r)){var i=null,o=null;if(n=n.firstBaseUpdate,n!==null){do{var s={eventTime:n.eventTime,lane:n.lane,tag:n.tag,payload:n.payload,callback:n.callback,next:null};o===null?i=o=s:o=o.next=s,n=n.next}while(n!==null);o===null?i=o=e:o=o.next=e}else i=o=e;n={baseState:r.baseState,firstBaseUpdate:i,lastBaseUpdate:o,shared:r.shared,effects:r.effects},t.updateQueue=n;return}t=n.lastBaseUpdate,t===null?n.firstBaseUpdate=e:t.next=e,n.lastBaseUpdate=e}function gl(t,e,n,r){var i=t.updateQueue;Zr=!1;var o=i.firstBaseUpdate,s=i.lastBaseUpdate,l=i.shared.pending;if(l!==null){i.shared.pending=null;var a=l,c=a.next;a.next=null,s===null?o=c:s.next=c,s=a;var u=t.alternate;if(u!==null){u=u.updateQueue;var f=u.lastBaseUpdate;f!==s&&(f===null?u.firstBaseUpdate=c:f.next=c,u.lastBaseUpdate=a)}}if(o!==null){f=i.baseState,s=0,u=c=a=null;do{l=o.lane;var h=o.eventTime;if((r&l)===l){u!==null&&(u=u.next={eventTime:h,lane:0,tag:o.tag,payload:o.payload,callback:o.callback,next:null});e:{var d=t,p=o;switch(l=e,h=n,p.tag){case 1:if(d=p.payload,typeof d=="function"){f=d.call(h,f,l);break e}f=d;break e;case 3:d.flags=d.flags&-4097|64;case 0:if(d=p.payload,l=typeof d=="function"?d.call(h,f,l):d,l==null)break e;f=qe({},f,l);break e;case 2:Zr=!0}}o.callback!==null&&(t.flags|=32,l=i.effects,l===null?i.effects=[o]:l.push(o))}else h={eventTime:h,lane:l,tag:o.tag,payload:o.payload,callback:o.callback,next:null},u===null?(c=u=h,a=f):u=u.next=h,s|=l;if(o=o.next,o===null){if(l=i.shared.pending,l===null)break;o=l.next,l.next=null,i.lastBaseUpdate=l,i.shared.pending=null}}while(1);u===null&&(a=f),i.baseState=a,i.firstBaseUpdate=c,i.lastBaseUpdate=u,Rl|=s,t.lanes=s,t.memoizedState=f}}function tv(t,e,n){if(t=e.effects,e.effects=null,t!==null)for(e=0;e<t.length;e++){var r=t[e],i=r.callback;if(i!==null){if(r.callback=null,r=n,typeof i!="function")throw Error(z(191,i));i.call(r)}}}var nv=new ic.Component().refs;function Qc(t,e,n,r){e=t.memoizedState,n=n(r,e),n=n==null?e:qe({},e,n),t.memoizedState=n,t.lanes===0&&(t.updateQueue.baseState=n)}var Nc={isMounted:function(t){return(t=t._reactInternals)?Ei(t)===t:!1},enqueueSetState:function(t,e,n){t=t._reactInternals;var r=en(),i=ti(t),o=Yr(r,i);o.payload=e,n!=null&&(o.callback=n),Kr(t,o),ni(t,i,r)},enqueueReplaceState:function(t,e,n){t=t._reactInternals;var r=en(),i=ti(t),o=Yr(r,i);o.tag=1,o.payload=e,n!=null&&(o.callback=n),Kr(t,o),ni(t,i,r)},enqueueForceUpdate:function(t,e){t=t._reactInternals;var n=en(),r=ti(t),i=Yr(n,r);i.tag=2,e!=null&&(i.callback=e),Kr(t,i),ni(t,r,n)}};function rv(t,e,n,r,i,o,s){return t=t.stateNode,typeof t.shouldComponentUpdate=="function"?t.shouldComponentUpdate(r,o,s):e.prototype&&e.prototype.isPureReactComponent?!ul(n,r)||!ul(i,o):!0}function iv(t,e,n){var r=!1,i=Gr,o=e.contextType;return typeof o=="object"&&o!==null?o=dn(o):(i=Nt(e)?Ai:xt.current,r=e.contextTypes,o=(r=r!=null)?To(t,i):Gr),e=new e(n,o),t.memoizedState=e.state!==null&&e.state!==void 0?e.state:null,e.updater=Nc,t.stateNode=e,e._reactInternals=t,r&&(t=t.stateNode,t.__reactInternalMemoizedUnmaskedChildContext=i,t.__reactInternalMemoizedMaskedChildContext=o),e}function ov(t,e,n,r){t=e.state,typeof e.componentWillReceiveProps=="function"&&e.componentWillReceiveProps(n,r),typeof e.UNSAFE_componentWillReceiveProps=="function"&&e.UNSAFE_componentWillReceiveProps(n,r),e.state!==t&&Nc.enqueueReplaceState(e,e.state,null)}function Nd(t,e,n,r){var i=t.stateNode;i.props=n,i.state=t.memoizedState,i.refs=nv,Qd(t);var o=e.contextType;typeof o=="object"&&o!==null?i.context=dn(o):(o=Nt(e)?Ai:xt.current,i.context=To(t,o)),gl(t,n,i,r),i.state=t.memoizedState,o=e.getDerivedStateFromProps,typeof o=="function"&&(Qc(t,e,o,n),i.state=t.memoizedState),typeof e.getDerivedStateFromProps=="function"||typeof i.getSnapshotBeforeUpdate=="function"||typeof i.UNSAFE_componentWillMount!="function"&&typeof i.componentWillMount!="function"||(e=i.state,typeof i.componentWillMount=="function"&&i.componentWillMount(),typeof i.UNSAFE_componentWillMount=="function"&&i.UNSAFE_componentWillMount(),e!==i.state&&Nc.enqueueReplaceState(i,i.state,null),gl(t,n,i,r),i.state=t.memoizedState),typeof i.componentDidMount=="function"&&(t.flags|=4)}var _c=Array.isArray;function yl(t,e,n){if(t=n.ref,t!==null&&typeof t!="function"&&typeof t!="object"){if(n._owner){if(n=n._owner,n){if(n.tag!==1)throw Error(z(309));var r=n.stateNode}if(!r)throw Error(z(147,t));var i=""+t;return e!==null&&e.ref!==null&&typeof e.ref=="function"&&e.ref._stringRef===i?e.ref:(e=function(o){var s=r.refs;s===nv&&(s=r.refs={}),o===null?delete s[i]:s[i]=o},e._stringRef=i,e)}if(typeof t!="string")throw Error(z(284));if(!n._owner)throw Error(z(290,t))}return t}function Bc(t,e){if(t.type!=="textarea")throw Error(z(31,Object.prototype.toString.call(e)==="[object Object]"?"object with keys {"+Object.keys(e).join(", ")+"}":e))}function sv(t){function e(g,y){if(t){var O=g.lastEffect;O!==null?(O.nextEffect=y,g.lastEffect=y):g.firstEffect=g.lastEffect=y,y.nextEffect=null,y.flags=8}}function n(g,y){if(!t)return null;for(;y!==null;)e(g,y),y=y.sibling;return null}function r(g,y){for(g=new Map;y!==null;)y.key!==null?g.set(y.key,y):g.set(y.index,y),y=y.sibling;return g}function i(g,y){return g=oi(g,y),g.index=0,g.sibling=null,g}function o(g,y,O){return g.index=O,t?(O=g.alternate,O!==null?(O=O.index,O<y?(g.flags=2,y):O):(g.flags=2,y)):y}function s(g){return t&&g.alternate===null&&(g.flags=2),g}function l(g,y,O,v){return y===null||y.tag!==6?(y=xp(O,g.mode,v),y.return=g,y):(y=i(y,O),y.return=g,y)}function a(g,y,O,v){return y!==null&&y.elementType===O.type?(v=i(y,O.props),v.ref=yl(g,y,O),v.return=g,v):(v=iu(O.type,O.key,O.props,null,g.mode,v),v.ref=yl(g,y,O),v.return=g,v)}function c(g,y,O,v){return y===null||y.tag!==4||y.stateNode.containerInfo!==O.containerInfo||y.stateNode.implementation!==O.implementation?(y=wp(O,g.mode,v),y.return=g,y):(y=i(y,O.children||[]),y.return=g,y)}function u(g,y,O,v,x){return y===null||y.tag!==7?(y=Bo(O,g.mode,v,x),y.return=g,y):(y=i(y,O),y.return=g,y)}function f(g,y,O){if(typeof y=="string"||typeof y=="number")return y=xp(""+y,g.mode,O),y.return=g,y;if(typeof y=="object"&&y!==null){switch(y.$$typeof){case Fs:return O=iu(y.type,y.key,y.props,null,g.mode,O),O.ref=yl(g,null,y),O.return=g,O;case Ti:return y=wp(y,g.mode,O),y.return=g,y}if(_c(y)||js(y))return y=Bo(y,g.mode,O,null),y.return=g,y;Bc(g,y)}return null}function h(g,y,O,v){var x=y!==null?y.key:null;if(typeof O=="string"||typeof O=="number")return x!==null?null:l(g,y,""+O,v);if(typeof O=="object"&&O!==null){switch(O.$$typeof){case Fs:return O.key===x?O.type===Br?u(g,y,O.props.children,v,x):a(g,y,O,v):null;case Ti:return O.key===x?c(g,y,O,v):null}if(_c(O)||js(O))return x!==null?null:u(g,y,O,v,null);Bc(g,O)}return null}function d(g,y,O,v,x){if(typeof v=="string"||typeof v=="number")return g=g.get(O)||null,l(y,g,""+v,x);if(typeof v=="object"&&v!==null){switch(v.$$typeof){case Fs:return g=g.get(v.key===null?O:v.key)||null,v.type===Br?u(y,g,v.props.children,x,v.key):a(y,g,v,x);case Ti:return g=g.get(v.key===null?O:v.key)||null,c(y,g,v,x)}if(_c(v)||js(v))return g=g.get(O)||null,u(y,g,v,x,null);Bc(y,v)}return null}function p(g,y,O,v){for(var x=null,b=null,S=y,k=y=0,P=null;S!==null&&k<O.length;k++){S.index>k?(P=S,S=null):P=S.sibling;var C=h(g,S,O[k],v);if(C===null){S===null&&(S=P);break}t&&S&&C.alternate===null&&e(g,S),y=o(C,y,k),b===null?x=C:b.sibling=C,b=C,S=P}if(k===O.length)return n(g,S),x;if(S===null){for(;k<O.length;k++)S=f(g,O[k],v),S!==null&&(y=o(S,y,k),b===null?x=S:b.sibling=S,b=S);return x}for(S=r(g,S);k<O.length;k++)P=d(S,g,k,O[k],v),P!==null&&(t&&P.alternate!==null&&S.delete(P.key===null?k:P.key),y=o(P,y,k),b===null?x=P:b.sibling=P,b=P);return t&&S.forEach(function(R){return e(g,R)}),x}function m(g,y,O,v){var x=js(O);if(typeof x!="function")throw Error(z(150));if(O=x.call(O),O==null)throw Error(z(151));for(var b=x=null,S=y,k=y=0,P=null,C=O.next();S!==null&&!C.done;k++,C=O.next()){S.index>k?(P=S,S=null):P=S.sibling;var R=h(g,S,C.value,v);if(R===null){S===null&&(S=P);break}t&&S&&R.alternate===null&&e(g,S),y=o(R,y,k),b===null?x=R:b.sibling=R,b=R,S=P}if(C.done)return n(g,S),x;if(S===null){for(;!C.done;k++,C=O.next())C=f(g,C.value,v),C!==null&&(y=o(C,y,k),b===null?x=C:b.sibling=C,b=C);return x}for(S=r(g,S);!C.done;k++,C=O.next())C=d(S,g,k,C.value,v),C!==null&&(t&&C.alternate!==null&&S.delete(C.key===null?k:C.key),y=o(C,y,k),b===null?x=C:b.sibling=C,b=C);return t&&S.forEach(function(T){return e(g,T)}),x}return function(g,y,O,v){var x=typeof O=="object"&&O!==null&&O.type===Br&&O.key===null;x&&(O=O.props.children);var b=typeof O=="object"&&O!==null;if(b)switch(O.$$typeof){case Fs:e:{for(b=O.key,x=y;x!==null;){if(x.key===b){switch(x.tag){case 7:if(O.type===Br){n(g,x.sibling),y=i(x,O.props.children),y.return=g,g=y;break e}break;default:if(x.elementType===O.type){n(g,x.sibling),y=i(x,O.props),y.ref=yl(g,x,O),y.return=g,g=y;break e}}n(g,x);break}else e(g,x);x=x.sibling}O.type===Br?(y=Bo(O.props.children,g.mode,v,O.key),y.return=g,g=y):(v=iu(O.type,O.key,O.props,null,g.mode,v),v.ref=yl(g,y,O),v.return=g,g=v)}return s(g);case Ti:e:{for(x=O.key;y!==null;){if(y.key===x)if(y.tag===4&&y.stateNode.containerInfo===O.containerInfo&&y.stateNode.implementation===O.implementation){n(g,y.sibling),y=i(y,O.children||[]),y.return=g,g=y;break e}else{n(g,y);break}else e(g,y);y=y.sibling}y=wp(O,g.mode,v),y.return=g,g=y}return s(g)}if(typeof O=="string"||typeof O=="number")return O=""+O,y!==null&&y.tag===6?(n(g,y.sibling),y=i(y,O),y.return=g,g=y):(n(g,y),y=xp(O,g.mode,v),y.return=g,g=y),s(g);if(_c(O))return p(g,y,O,v);if(js(O))return m(g,y,O,v);if(b&&Bc(g,O),typeof O=="undefined"&&!x)switch(g.tag){case 1:case 22:case 0:case 11:case 15:throw Error(z(152,po(g.type)||"Component"))}return n(g,y)}}var qc=sv(!0),lv=sv(!1),Ol={},zn=Hr(Ol),vl=Hr(Ol),xl=Hr(Ol);function Wi(t){if(t===Ol)throw Error(z(174));return t}function _d(t,e){switch(je(xl,e),je(vl,t),je(zn,Ol),t=e.nodeType,t){case 9:case 11:e=(e=e.documentElement)?e.namespaceURI:Vh(null,"");break;default:t=t===8?e.parentNode:e,e=t.namespaceURI||null,t=t.tagName,e=Vh(e,t)}We(zn),je(zn,e)}function Ao(){We(zn),We(vl),We(xl)}function av(t){Wi(xl.current);var e=Wi(zn.current),n=Vh(e,t.type);e!==n&&(je(vl,t),je(zn,n))}function Bd(t){vl.current===t&&(We(zn),We(vl))}var Xe=Hr(0);function zc(t){for(var e=t;e!==null;){if(e.tag===13){var n=e.memoizedState;if(n!==null&&(n=n.dehydrated,n===null||n.data==="$?"||n.data==="$!"))return e}else if(e.tag===19&&e.memoizedProps.revealOrder!==void 0){if((e.flags&64)!=0)return e}else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break;for(;e.sibling===null;){if(e.return===null||e.return===t)return null;e=e.return}e.sibling.return=e.return,e=e.sibling}return null}var mr=null,Jr=null,Fn=!1;function cv(t,e){var n=gn(5,null,null,0);n.elementType="DELETED",n.type="DELETED",n.stateNode=e,n.return=t,n.flags=8,t.lastEffect!==null?(t.lastEffect.nextEffect=n,t.lastEffect=n):t.firstEffect=t.lastEffect=n}function uv(t,e){switch(t.tag){case 5:var n=t.type;return e=e.nodeType!==1||n.toLowerCase()!==e.nodeName.toLowerCase()?null:e,e!==null?(t.stateNode=e,!0):!1;case 6:return e=t.pendingProps===""||e.nodeType!==3?null:e,e!==null?(t.stateNode=e,!0):!1;case 13:return!1;default:return!1}}function qd(t){if(Fn){var e=Jr;if(e){var n=e;if(!uv(t,e)){if(e=ko(n.nextSibling),!e||!uv(t,e)){t.flags=t.flags&-1025|2,Fn=!1,mr=t;return}cv(mr,n)}mr=t,Jr=ko(e.firstChild)}else t.flags=t.flags&-1025|2,Fn=!1,mr=t}}function fv(t){for(t=t.return;t!==null&&t.tag!==5&&t.tag!==3&&t.tag!==13;)t=t.return;mr=t}function Fc(t){if(t!==mr)return!1;if(!Fn)return fv(t),Fn=!0,!1;var e=t.type;if(t.tag!==5||e!=="head"&&e!=="body"&&!kd(e,t.memoizedProps))for(e=Jr;e;)cv(t,e),e=ko(e.nextSibling);if(fv(t),t.tag===13){if(t=t.memoizedState,t=t!==null?t.dehydrated:null,!t)throw Error(z(317));e:{for(t=t.nextSibling,e=0;t;){if(t.nodeType===8){var n=t.data;if(n==="/$"){if(e===0){Jr=ko(t.nextSibling);break e}e--}else n!=="$"&&n!=="$!"&&n!=="$?"||e++}t=t.nextSibling}Jr=null}}else Jr=mr?ko(t.stateNode.nextSibling):null;return!0}function zd(){Jr=mr=null,Fn=!1}var Do=[];function Fd(){for(var t=0;t<Do.length;t++)Do[t]._workInProgressVersionPrimary=null;Do.length=0}var wl=Ri.ReactCurrentDispatcher,pn=Ri.ReactCurrentBatchConfig,bl=0,Ze=null,bt=null,pt=null,Vc=!1,$l=!1;function _t(){throw Error(z(321))}function Vd(t,e){if(e===null)return!1;for(var n=0;n<e.length&&n<t.length;n++)if(!hn(t[n],e[n]))return!1;return!0}function Ud(t,e,n,r,i,o){if(bl=o,Ze=e,e.memoizedState=null,e.updateQueue=null,e.lanes=0,wl.current=t===null||t.memoizedState===null?HR:GR,t=n(r,i),$l){o=0;do{if($l=!1,!(25>o))throw Error(z(301));o+=1,pt=bt=null,e.updateQueue=null,wl.current=ZR,t=n(r,i)}while($l)}if(wl.current=Hc,e=bt!==null&&bt.next!==null,bl=0,pt=bt=Ze=null,Vc=!1,e)throw Error(z(300));return t}function Qi(){var t={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return pt===null?Ze.memoizedState=pt=t:pt=pt.next=t,pt}function Ni(){if(bt===null){var t=Ze.alternate;t=t!==null?t.memoizedState:null}else t=bt.next;var e=pt===null?Ze.memoizedState:pt.next;if(e!==null)pt=e,bt=t;else{if(t===null)throw Error(z(310));bt=t,t={memoizedState:bt.memoizedState,baseState:bt.baseState,baseQueue:bt.baseQueue,queue:bt.queue,next:null},pt===null?Ze.memoizedState=pt=t:pt=pt.next=t}return pt}function Vn(t,e){return typeof e=="function"?e(t):e}function Sl(t){var e=Ni(),n=e.queue;if(n===null)throw Error(z(311));n.lastRenderedReducer=t;var r=bt,i=r.baseQueue,o=n.pending;if(o!==null){if(i!==null){var s=i.next;i.next=o.next,o.next=s}r.baseQueue=i=o,n.pending=null}if(i!==null){i=i.next,r=r.baseState;var l=s=o=null,a=i;do{var c=a.lane;if((bl&c)===c)l!==null&&(l=l.next={lane:0,action:a.action,eagerReducer:a.eagerReducer,eagerState:a.eagerState,next:null}),r=a.eagerReducer===t?a.eagerState:t(r,a.action);else{var u={lane:c,action:a.action,eagerReducer:a.eagerReducer,eagerState:a.eagerState,next:null};l===null?(s=l=u,o=r):l=l.next=u,Ze.lanes|=c,Rl|=c}a=a.next}while(a!==null&&a!==i);l===null?o=r:l.next=s,hn(r,e.memoizedState)||(An=!0),e.memoizedState=r,e.baseState=o,e.baseQueue=l,n.lastRenderedState=r}return[e.memoizedState,n.dispatch]}function kl(t){var e=Ni(),n=e.queue;if(n===null)throw Error(z(311));n.lastRenderedReducer=t;var r=n.dispatch,i=n.pending,o=e.memoizedState;if(i!==null){n.pending=null;var s=i=i.next;do o=t(o,s.action),s=s.next;while(s!==i);hn(o,e.memoizedState)||(An=!0),e.memoizedState=o,e.baseQueue===null&&(e.baseState=o),n.lastRenderedState=o}return[o,r]}function hv(t,e,n){var r=e._getVersion;r=r(e._source);var i=e._workInProgressVersionPrimary;if(i!==null?t=i===r:(t=t.mutableReadLanes,(t=(bl&t)===t)&&(e._workInProgressVersionPrimary=r,Do.push(e))),t)return n(e._source);throw Do.push(e),Error(z(350))}function dv(t,e,n,r){var i=At;if(i===null)throw Error(z(349));var o=e._getVersion,s=o(e._source),l=wl.current,a=l.useState(function(){return hv(i,e,n)}),c=a[1],u=a[0];a=pt;var f=t.memoizedState,h=f.refs,d=h.getSnapshot,p=f.source;f=f.subscribe;var m=Ze;return t.memoizedState={refs:h,source:e,subscribe:r},l.useEffect(function(){h.getSnapshot=n,h.setSnapshot=c;var g=o(e._source);if(!hn(s,g)){g=n(e._source),hn(u,g)||(c(g),g=ti(m),i.mutableReadLanes|=g&i.pendingLanes),g=i.mutableReadLanes,i.entangledLanes|=g;for(var y=i.entanglements,O=g;0<O;){var v=31-Ur(O),x=1<<v;y[v]|=g,O&=~x}}},[n,e,r]),l.useEffect(function(){return r(e._source,function(){var g=h.getSnapshot,y=h.setSnapshot;try{y(g(e._source));var O=ti(m);i.mutableReadLanes|=O&i.pendingLanes}catch(v){y(function(){throw v})}})},[e,r]),hn(d,n)&&hn(p,e)&&hn(f,r)||(t={pending:null,dispatch:null,lastRenderedReducer:Vn,lastRenderedState:u},t.dispatch=c=Gd.bind(null,Ze,t),a.queue=t,a.baseQueue=null,u=hv(i,e,n),a.memoizedState=a.baseState=u),u}function pv(t,e,n){var r=Ni();return dv(r,t,e,n)}function Pl(t){var e=Qi();return typeof t=="function"&&(t=t()),e.memoizedState=e.baseState=t,t=e.queue={pending:null,dispatch:null,lastRenderedReducer:Vn,lastRenderedState:t},t=t.dispatch=Gd.bind(null,Ze,t),[e.memoizedState,t]}function Uc(t,e,n,r){return t={tag:t,create:e,destroy:n,deps:r,next:null},e=Ze.updateQueue,e===null?(e={lastEffect:null},Ze.updateQueue=e,e.lastEffect=t.next=t):(n=e.lastEffect,n===null?e.lastEffect=t.next=t:(r=n.next,n.next=t,t.next=r,e.lastEffect=t)),t}function mv(t){var e=Qi();return t={current:t},e.memoizedState=t}function jc(){return Ni().memoizedState}function jd(t,e,n,r){var i=Qi();Ze.flags|=t,i.memoizedState=Uc(1|e,n,void 0,r===void 0?null:r)}function Xd(t,e,n,r){var i=Ni();r=r===void 0?null:r;var o=void 0;if(bt!==null){var s=bt.memoizedState;if(o=s.destroy,r!==null&&Vd(r,s.deps)){Uc(e,n,o,r);return}}Ze.flags|=t,i.memoizedState=Uc(1|e,n,o,r)}function gv(t,e){return jd(516,4,t,e)}function Xc(t,e){return Xd(516,4,t,e)}function yv(t,e){return Xd(4,2,t,e)}function Ov(t,e){if(typeof e=="function")return t=t(),e(t),function(){e(null)};if(e!=null)return t=t(),e.current=t,function(){e.current=null}}function vv(t,e,n){return n=n!=null?n.concat([t]):null,Xd(4,2,Ov.bind(null,e,t),n)}function Hd(){}function xv(t,e){var n=Ni();e=e===void 0?null:e;var r=n.memoizedState;return r!==null&&e!==null&&Vd(e,r[1])?r[0]:(n.memoizedState=[t,e],t)}function wv(t,e){var n=Ni();e=e===void 0?null:e;var r=n.memoizedState;return r!==null&&e!==null&&Vd(e,r[1])?r[0]:(t=t(),n.memoizedState=[t,e],t)}function XR(t,e){var n=Io();Li(98>n?98:n,function(){t(!0)}),Li(97<n?97:n,function(){var r=pn.transition;pn.transition=1;try{t(!1),e()}finally{pn.transition=r}})}function Gd(t,e,n){var r=en(),i=ti(t),o={lane:i,action:n,eagerReducer:null,eagerState:null,next:null},s=e.pending;if(s===null?o.next=o:(o.next=s.next,s.next=o),e.pending=o,s=t.alternate,t===Ze||s!==null&&s===Ze)$l=Vc=!0;else{if(t.lanes===0&&(s===null||s.lanes===0)&&(s=e.lastRenderedReducer,s!==null))try{var l=e.lastRenderedState,a=s(l,n);if(o.eagerReducer=s,o.eagerState=a,hn(a,l))return}catch{}finally{}ni(t,i,r)}}var Hc={readContext:dn,useCallback:_t,useContext:_t,useEffect:_t,useImperativeHandle:_t,useLayoutEffect:_t,useMemo:_t,useReducer:_t,useRef:_t,useState:_t,useDebugValue:_t,useDeferredValue:_t,useTransition:_t,useMutableSource:_t,useOpaqueIdentifier:_t,unstable_isNewReconciler:!1},HR={readContext:dn,useCallback:function(t,e){return Qi().memoizedState=[t,e===void 0?null:e],t},useContext:dn,useEffect:gv,useImperativeHandle:function(t,e,n){return n=n!=null?n.concat([t]):null,jd(4,2,Ov.bind(null,e,t),n)},useLayoutEffect:function(t,e){return jd(4,2,t,e)},useMemo:function(t,e){var n=Qi();return e=e===void 0?null:e,t=t(),n.memoizedState=[t,e],t},useReducer:function(t,e,n){var r=Qi();return e=n!==void 0?n(e):e,r.memoizedState=r.baseState=e,t=r.queue={pending:null,dispatch:null,lastRenderedReducer:t,lastRenderedState:e},t=t.dispatch=Gd.bind(null,Ze,t),[r.memoizedState,t]},useRef:mv,useState:Pl,useDebugValue:Hd,useDeferredValue:function(t){var e=Pl(t),n=e[0],r=e[1];return gv(function(){var i=pn.transition;pn.transition=1;try{r(t)}finally{pn.transition=i}},[t]),n},useTransition:function(){var t=Pl(!1),e=t[0];return t=XR.bind(null,t[1]),mv(t),[t,e]},useMutableSource:function(t,e,n){var r=Qi();return r.memoizedState={refs:{getSnapshot:e,setSnapshot:null},source:t,subscribe:n},dv(r,t,e,n)},useOpaqueIdentifier:function(){if(Fn){var t=!1,e=qR(function(){throw t||(t=!0,n("r:"+(Cd++).toString(36))),Error(z(355))}),n=Pl(e)[1];return(Ze.mode&2)==0&&(Ze.flags|=516,Uc(5,function(){n("r:"+(Cd++).toString(36))},void 0,null)),e}return e="r:"+(Cd++).toString(36),Pl(e),e},unstable_isNewReconciler:!1},GR={readContext:dn,useCallback:xv,useContext:dn,useEffect:Xc,useImperativeHandle:vv,useLayoutEffect:yv,useMemo:wv,useReducer:Sl,useRef:jc,useState:function(){return Sl(Vn)},useDebugValue:Hd,useDeferredValue:function(t){var e=Sl(Vn),n=e[0],r=e[1];return Xc(function(){var i=pn.transition;pn.transition=1;try{r(t)}finally{pn.transition=i}},[t]),n},useTransition:function(){var t=Sl(Vn)[0];return[jc().current,t]},useMutableSource:pv,useOpaqueIdentifier:function(){return Sl(Vn)[0]},unstable_isNewReconciler:!1},ZR={readContext:dn,useCallback:xv,useContext:dn,useEffect:Xc,useImperativeHandle:vv,useLayoutEffect:yv,useMemo:wv,useReducer:kl,useRef:jc,useState:function(){return kl(Vn)},useDebugValue:Hd,useDeferredValue:function(t){var e=kl(Vn),n=e[0],r=e[1];return Xc(function(){var i=pn.transition;pn.transition=1;try{r(t)}finally{pn.transition=i}},[t]),n},useTransition:function(){var t=kl(Vn)[0];return[jc().current,t]},useMutableSource:pv,useOpaqueIdentifier:function(){return kl(Vn)[0]},unstable_isNewReconciler:!1},YR=Ri.ReactCurrentOwner,An=!1;function Bt(t,e,n,r){e.child=t===null?lv(e,null,n,r):qc(e,t.child,n,r)}function bv(t,e,n,r,i){n=n.render;var o=e.ref;return Mo(e,i),r=Ud(t,e,n,r,o,i),t!==null&&!An?(e.updateQueue=t.updateQueue,e.flags&=-517,t.lanes&=~i,gr(t,e,i)):(e.flags|=1,Bt(t,e,r,i),e.child)}function $v(t,e,n,r,i,o){if(t===null){var s=n.type;return typeof s=="function"&&!Op(s)&&s.defaultProps===void 0&&n.compare===null&&n.defaultProps===void 0?(e.tag=15,e.type=s,Sv(t,e,s,r,i,o)):(t=iu(n.type,null,r,e,e.mode,o),t.ref=e.ref,t.return=e,e.child=t)}return s=t.child,(i&o)==0&&(i=s.memoizedProps,n=n.compare,n=n!==null?n:ul,n(i,r)&&t.ref===e.ref)?gr(t,e,o):(e.flags|=1,t=oi(s,r),t.ref=e.ref,t.return=e,e.child=t)}function Sv(t,e,n,r,i,o){if(t!==null&&ul(t.memoizedProps,r)&&t.ref===e.ref)if(An=!1,(o&i)!=0)(t.flags&16384)!=0&&(An=!0);else return e.lanes=t.lanes,gr(t,e,o);return Yd(t,e,n,r,o)}function Zd(t,e,n){var r=e.pendingProps,i=r.children,o=t!==null?t.memoizedState:null;if(r.mode==="hidden"||r.mode==="unstable-defer-without-hiding")if((e.mode&4)==0)e.memoizedState={baseLanes:0},ru(e,n);else if((n&1073741824)!=0)e.memoizedState={baseLanes:0},ru(e,o!==null?o.baseLanes:n);else return t=o!==null?o.baseLanes|n:n,e.lanes=e.childLanes=1073741824,e.memoizedState={baseLanes:t},ru(e,t),null;else o!==null?(r=o.baseLanes|n,e.memoizedState=null):r=n,ru(e,r);return Bt(t,e,i,n),e.child}function kv(t,e){var n=e.ref;(t===null&&n!==null||t!==null&&t.ref!==n)&&(e.flags|=128)}function Yd(t,e,n,r,i){var o=Nt(n)?Ai:xt.current;return o=To(e,o),Mo(e,i),n=Ud(t,e,n,r,o,i),t!==null&&!An?(e.updateQueue=t.updateQueue,e.flags&=-517,t.lanes&=~i,gr(t,e,i)):(e.flags|=1,Bt(t,e,n,i),e.child)}function Pv(t,e,n,r,i){if(Nt(n)){var o=!0;Ec(e)}else o=!1;if(Mo(e,i),e.stateNode===null)t!==null&&(t.alternate=null,e.alternate=null,e.flags|=2),iv(e,n,r),Nd(e,n,r,i),r=!0;else if(t===null){var s=e.stateNode,l=e.memoizedProps;s.props=l;var a=s.context,c=n.contextType;typeof c=="object"&&c!==null?c=dn(c):(c=Nt(n)?Ai:xt.current,c=To(e,c));var u=n.getDerivedStateFromProps,f=typeof u=="function"||typeof s.getSnapshotBeforeUpdate=="function";f||typeof s.UNSAFE_componentWillReceiveProps!="function"&&typeof s.componentWillReceiveProps!="function"||(l!==r||a!==c)&&ov(e,s,r,c),Zr=!1;var h=e.memoizedState;s.state=h,gl(e,r,s,i),a=e.memoizedState,l!==r||h!==a||Qt.current||Zr?(typeof u=="function"&&(Qc(e,n,u,r),a=e.memoizedState),(l=Zr||rv(e,n,l,r,h,a,c))?(f||typeof s.UNSAFE_componentWillMount!="function"&&typeof s.componentWillMount!="function"||(typeof s.componentWillMount=="function"&&s.componentWillMount(),typeof s.UNSAFE_componentWillMount=="function"&&s.UNSAFE_componentWillMount()),typeof s.componentDidMount=="function"&&(e.flags|=4)):(typeof s.componentDidMount=="function"&&(e.flags|=4),e.memoizedProps=r,e.memoizedState=a),s.props=r,s.state=a,s.context=c,r=l):(typeof s.componentDidMount=="function"&&(e.flags|=4),r=!1)}else{s=e.stateNode,JO(t,e),l=e.memoizedProps,c=e.type===e.elementType?l:Mn(e.type,l),s.props=c,f=e.pendingProps,h=s.context,a=n.contextType,typeof a=="object"&&a!==null?a=dn(a):(a=Nt(n)?Ai:xt.current,a=To(e,a));var d=n.getDerivedStateFromProps;(u=typeof d=="function"||typeof s.getSnapshotBeforeUpdate=="function")||typeof s.UNSAFE_componentWillReceiveProps!="function"&&typeof s.componentWillReceiveProps!="function"||(l!==f||h!==a)&&ov(e,s,r,a),Zr=!1,h=e.memoizedState,s.state=h,gl(e,r,s,i);var p=e.memoizedState;l!==f||h!==p||Qt.current||Zr?(typeof d=="function"&&(Qc(e,n,d,r),p=e.memoizedState),(c=Zr||rv(e,n,c,r,h,p,a))?(u||typeof s.UNSAFE_componentWillUpdate!="function"&&typeof s.componentWillUpdate!="function"||(typeof s.componentWillUpdate=="function"&&s.componentWillUpdate(r,p,a),typeof s.UNSAFE_componentWillUpdate=="function"&&s.UNSAFE_componentWillUpdate(r,p,a)),typeof s.componentDidUpdate=="function"&&(e.flags|=4),typeof s.getSnapshotBeforeUpdate=="function"&&(e.flags|=256)):(typeof s.componentDidUpdate!="function"||l===t.memoizedProps&&h===t.memoizedState||(e.flags|=4),typeof s.getSnapshotBeforeUpdate!="function"||l===t.memoizedProps&&h===t.memoizedState||(e.flags|=256),e.memoizedProps=r,e.memoizedState=p),s.props=r,s.state=p,s.context=a,r=c):(typeof s.componentDidUpdate!="function"||l===t.memoizedProps&&h===t.memoizedState||(e.flags|=4),typeof s.getSnapshotBeforeUpdate!="function"||l===t.memoizedProps&&h===t.memoizedState||(e.flags|=256),r=!1)}return Kd(t,e,n,r,o,i)}function Kd(t,e,n,r,i,o){kv(t,e);var s=(e.flags&64)!=0;if(!r&&!s)return i&&FO(e,n,!1),gr(t,e,o);r=e.stateNode,YR.current=e;var l=s&&typeof n.getDerivedStateFromError!="function"?null:r.render();return e.flags|=1,t!==null&&s?(e.child=qc(e,t.child,null,o),e.child=qc(e,null,l,o)):Bt(t,e,l,o),e.memoizedState=r.state,i&&FO(e,n,!0),e.child}function Cv(t){var e=t.stateNode;e.pendingContext?qO(t,e.pendingContext,e.pendingContext!==e.context):e.context&&qO(t,e.context,!1),_d(t,e.containerInfo)}var Gc={dehydrated:null,retryLane:0};function Rv(t,e,n){var r=e.pendingProps,i=Xe.current,o=!1,s;return(s=(e.flags&64)!=0)||(s=t!==null&&t.memoizedState===null?!1:(i&2)!=0),s?(o=!0,e.flags&=-65):t!==null&&t.memoizedState===null||r.fallback===void 0||r.unstable_avoidThisFallback===!0||(i|=1),je(Xe,i&1),t===null?(r.fallback!==void 0&&qd(e),t=r.children,i=r.fallback,o?(t=Tv(e,t,i,n),e.child.memoizedState={baseLanes:n},e.memoizedState=Gc,t):typeof r.unstable_expectedLoadTime=="number"?(t=Tv(e,t,i,n),e.child.memoizedState={baseLanes:n},e.memoizedState=Gc,e.lanes=33554432,t):(n=vp({mode:"visible",children:t},e.mode,n,null),n.return=e,e.child=n)):t.memoizedState!==null?o?(r=Ev(t,e,r.children,r.fallback,n),o=e.child,i=t.child.memoizedState,o.memoizedState=i===null?{baseLanes:n}:{baseLanes:i.baseLanes|n},o.childLanes=t.childLanes&~n,e.memoizedState=Gc,r):(n=Iv(t,e,r.children,n),e.memoizedState=null,n):o?(r=Ev(t,e,r.children,r.fallback,n),o=e.child,i=t.child.memoizedState,o.memoizedState=i===null?{baseLanes:n}:{baseLanes:i.baseLanes|n},o.childLanes=t.childLanes&~n,e.memoizedState=Gc,r):(n=Iv(t,e,r.children,n),e.memoizedState=null,n)}function Tv(t,e,n,r){var i=t.mode,o=t.child;return e={mode:"hidden",children:e},(i&2)==0&&o!==null?(o.childLanes=0,o.pendingProps=e):o=vp(e,i,0,null),n=Bo(n,i,r,null),o.return=t,n.return=t,o.sibling=n,t.child=o,n}function Iv(t,e,n,r){var i=t.child;return t=i.sibling,n=oi(i,{mode:"visible",children:n}),(e.mode&2)==0&&(n.lanes=r),n.return=e,n.sibling=null,t!==null&&(t.nextEffect=null,t.flags=8,e.firstEffect=e.lastEffect=t),e.child=n}function Ev(t,e,n,r,i){var o=e.mode,s=t.child;t=s.sibling;var l={mode:"hidden",children:n};return(o&2)==0&&e.child!==s?(n=e.child,n.childLanes=0,n.pendingProps=l,s=n.lastEffect,s!==null?(e.firstEffect=n.firstEffect,e.lastEffect=s,s.nextEffect=null):e.firstEffect=e.lastEffect=null):n=oi(s,l),t!==null?r=oi(t,r):(r=Bo(r,o,i,null),r.flags|=2),r.return=e,n.return=e,n.sibling=r,e.child=n,r}function Mv(t,e){t.lanes|=e;var n=t.alternate;n!==null&&(n.lanes|=e),KO(t.return,e)}function Jd(t,e,n,r,i,o){var s=t.memoizedState;s===null?t.memoizedState={isBackwards:e,rendering:null,renderingStartTime:0,last:r,tail:n,tailMode:i,lastEffect:o}:(s.isBackwards=e,s.rendering=null,s.renderingStartTime=0,s.last=r,s.tail=n,s.tailMode=i,s.lastEffect=o)}function Av(t,e,n){var r=e.pendingProps,i=r.revealOrder,o=r.tail;if(Bt(t,e,r.children,n),r=Xe.current,(r&2)!=0)r=r&1|2,e.flags|=64;else{if(t!==null&&(t.flags&64)!=0)e:for(t=e.child;t!==null;){if(t.tag===13)t.memoizedState!==null&&Mv(t,n);else if(t.tag===19)Mv(t,n);else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;t=t.return}t.sibling.return=t.return,t=t.sibling}r&=1}if(je(Xe,r),(e.mode&2)==0)e.memoizedState=null;else switch(i){case"forwards":for(n=e.child,i=null;n!==null;)t=n.alternate,t!==null&&zc(t)===null&&(i=n),n=n.sibling;n=i,n===null?(i=e.child,e.child=null):(i=n.sibling,n.sibling=null),Jd(e,!1,i,n,o,e.lastEffect);break;case"backwards":for(n=null,i=e.child,e.child=null;i!==null;){if(t=i.alternate,t!==null&&zc(t)===null){e.child=i;break}t=i.sibling,i.sibling=n,n=i,i=t}Jd(e,!0,n,null,o,e.lastEffect);break;case"together":Jd(e,!1,null,null,void 0,e.lastEffect);break;default:e.memoizedState=null}return e.child}function gr(t,e,n){if(t!==null&&(e.dependencies=t.dependencies),Rl|=e.lanes,(n&e.childLanes)!=0){if(t!==null&&e.child!==t.child)throw Error(z(153));if(e.child!==null){for(t=e.child,n=oi(t,t.pendingProps),e.child=n,n.return=e;t.sibling!==null;)t=t.sibling,n=n.sibling=oi(t,t.pendingProps),n.return=e;n.sibling=null}return e.child}return null}var Dv,ep,Lv,Wv;Dv=function(t,e){for(var n=e.child;n!==null;){if(n.tag===5||n.tag===6)t.appendChild(n.stateNode);else if(n.tag!==4&&n.child!==null){n.child.return=n,n=n.child;continue}if(n===e)break;for(;n.sibling===null;){if(n.return===null||n.return===e)return;n=n.return}n.sibling.return=n.return,n=n.sibling}};ep=function(){};Lv=function(t,e,n,r){var i=t.memoizedProps;if(i!==r){t=e.stateNode,Wi(zn.current);var o=null;switch(n){case"input":i=Nh(t,i),r=Nh(t,r),o=[];break;case"option":i=qh(t,i),r=qh(t,r),o=[];break;case"select":i=qe({},i,{value:void 0}),r=qe({},r,{value:void 0}),o=[];break;case"textarea":i=zh(t,i),r=zh(t,r),o=[];break;default:typeof i.onClick!="function"&&typeof r.onClick=="function"&&(t.onclick=Pc)}Uh(n,r);var s;n=null;for(c in i)if(!r.hasOwnProperty(c)&&i.hasOwnProperty(c)&&i[c]!=null)if(c==="style"){var l=i[c];for(s in l)l.hasOwnProperty(s)&&(n||(n={}),n[s]="")}else c!=="dangerouslySetInnerHTML"&&c!=="children"&&c!=="suppressContentEditableWarning"&&c!=="suppressHydrationWarning"&&c!=="autoFocus"&&(zs.hasOwnProperty(c)?o||(o=[]):(o=o||[]).push(c,null));for(c in r){var a=r[c];if(l=i!=null?i[c]:void 0,r.hasOwnProperty(c)&&a!==l&&(a!=null||l!=null))if(c==="style")if(l){for(s in l)!l.hasOwnProperty(s)||a&&a.hasOwnProperty(s)||(n||(n={}),n[s]="");for(s in a)a.hasOwnProperty(s)&&l[s]!==a[s]&&(n||(n={}),n[s]=a[s])}else n||(o||(o=[]),o.push(c,n)),n=a;else c==="dangerouslySetInnerHTML"?(a=a?a.__html:void 0,l=l?l.__html:void 0,a!=null&&l!==a&&(o=o||[]).push(c,a)):c==="children"?typeof a!="string"&&typeof a!="number"||(o=o||[]).push(c,""+a):c!=="suppressContentEditableWarning"&&c!=="suppressHydrationWarning"&&(zs.hasOwnProperty(c)?(a!=null&&c==="onScroll"&&Le("scroll",t),o||l===a||(o=[])):typeof a=="object"&&a!==null&&a.$$typeof===Ah?a.toString():(o=o||[]).push(c,a))}n&&(o=o||[]).push("style",n);var c=o;(e.updateQueue=c)&&(e.flags|=4)}};Wv=function(t,e,n,r){n!==r&&(e.flags|=4)};function Cl(t,e){if(!Fn)switch(t.tailMode){case"hidden":e=t.tail;for(var n=null;e!==null;)e.alternate!==null&&(n=e),e=e.sibling;n===null?t.tail=null:n.sibling=null;break;case"collapsed":n=t.tail;for(var r=null;n!==null;)n.alternate!==null&&(r=n),n=n.sibling;r===null?e||t.tail===null?t.tail=null:t.tail.sibling=null:r.sibling=null}}function KR(t,e,n){var r=e.pendingProps;switch(e.tag){case 2:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return null;case 1:return Nt(e.type)&&Ic(),null;case 3:return Ao(),We(Qt),We(xt),Fd(),r=e.stateNode,r.pendingContext&&(r.context=r.pendingContext,r.pendingContext=null),(t===null||t.child===null)&&(Fc(e)?e.flags|=4:r.hydrate||(e.flags|=256)),ep(e),null;case 5:Bd(e);var i=Wi(xl.current);if(n=e.type,t!==null&&e.stateNode!=null)Lv(t,e,n,r,i),t.ref!==e.ref&&(e.flags|=128);else{if(!r){if(e.stateNode===null)throw Error(z(166));return null}if(t=Wi(zn.current),Fc(e)){r=e.stateNode,n=e.type;var o=e.memoizedProps;switch(r[Xr]=e,r[Rc]=o,n){case"dialog":Le("cancel",r),Le("close",r);break;case"iframe":case"object":case"embed":Le("load",r);break;case"video":case"audio":for(t=0;t<hl.length;t++)Le(hl[t],r);break;case"source":Le("error",r);break;case"img":case"image":case"link":Le("error",r),Le("load",r);break;case"details":Le("toggle",r);break;case"input":Py(r,o),Le("invalid",r);break;case"select":r._wrapperState={wasMultiple:!!o.multiple},Le("invalid",r);break;case"textarea":Ty(r,o),Le("invalid",r)}Uh(n,o),t=null;for(var s in o)o.hasOwnProperty(s)&&(i=o[s],s==="children"?typeof i=="string"?r.textContent!==i&&(t=["children",i]):typeof i=="number"&&r.textContent!==""+i&&(t=["children",""+i]):zs.hasOwnProperty(s)&&i!=null&&s==="onScroll"&&Le("scroll",r));switch(n){case"input":cc(r),Ry(r,o,!0);break;case"textarea":cc(r),Ey(r);break;case"select":case"option":break;default:typeof o.onClick=="function"&&(r.onclick=Pc)}r=t,e.updateQueue=r,r!==null&&(e.flags|=4)}else{switch(s=i.nodeType===9?i:i.ownerDocument,t===Fh.html&&(t=My(n)),t===Fh.html?n==="script"?(t=s.createElement("div"),t.innerHTML="<script><\/script>",t=t.removeChild(t.firstChild)):typeof r.is=="string"?t=s.createElement(n,{is:r.is}):(t=s.createElement(n),n==="select"&&(s=t,r.multiple?s.multiple=!0:r.size&&(s.size=r.size))):t=s.createElementNS(t,n),t[Xr]=e,t[Rc]=r,Dv(t,e,!1,!1),e.stateNode=t,s=jh(n,r),n){case"dialog":Le("cancel",t),Le("close",t),i=r;break;case"iframe":case"object":case"embed":Le("load",t),i=r;break;case"video":case"audio":for(i=0;i<hl.length;i++)Le(hl[i],t);i=r;break;case"source":Le("error",t),i=r;break;case"img":case"image":case"link":Le("error",t),Le("load",t),i=r;break;case"details":Le("toggle",t),i=r;break;case"input":Py(t,r),i=Nh(t,r),Le("invalid",t);break;case"option":i=qh(t,r);break;case"select":t._wrapperState={wasMultiple:!!r.multiple},i=qe({},r,{value:void 0}),Le("invalid",t);break;case"textarea":Ty(t,r),i=zh(t,r),Le("invalid",t);break;default:i=r}Uh(n,i);var l=i;for(o in l)if(l.hasOwnProperty(o)){var a=l[o];o==="style"?Ly(t,a):o==="dangerouslySetInnerHTML"?(a=a?a.__html:void 0,a!=null&&Ay(t,a)):o==="children"?typeof a=="string"?(n!=="textarea"||a!=="")&&Hs(t,a):typeof a=="number"&&Hs(t,""+a):o!=="suppressContentEditableWarning"&&o!=="suppressHydrationWarning"&&o!=="autoFocus"&&(zs.hasOwnProperty(o)?a!=null&&o==="onScroll"&&Le("scroll",t):a!=null&&Ch(t,o,a,s))}switch(n){case"input":cc(t),Ry(t,r,!1);break;case"textarea":cc(t),Ey(t);break;case"option":r.value!=null&&t.setAttribute("value",""+qr(r.value));break;case"select":t.multiple=!!r.multiple,o=r.value,o!=null?mo(t,!!r.multiple,o,!1):r.defaultValue!=null&&mo(t,!!r.multiple,r.defaultValue,!0);break;default:typeof i.onClick=="function"&&(t.onclick=Pc)}WO(n,r)&&(e.flags|=4)}e.ref!==null&&(e.flags|=128)}return null;case 6:if(t&&e.stateNode!=null)Wv(t,e,t.memoizedProps,r);else{if(typeof r!="string"&&e.stateNode===null)throw Error(z(166));n=Wi(xl.current),Wi(zn.current),Fc(e)?(r=e.stateNode,n=e.memoizedProps,r[Xr]=e,r.nodeValue!==n&&(e.flags|=4)):(r=(n.nodeType===9?n:n.ownerDocument).createTextNode(r),r[Xr]=e,e.stateNode=r)}return null;case 13:return We(Xe),r=e.memoizedState,(e.flags&64)!=0?(e.lanes=n,e):(r=r!==null,n=!1,t===null?e.memoizedProps.fallback!==void 0&&Fc(e):n=t.memoizedState!==null,r&&!n&&(e.mode&2)!=0&&(t===null&&e.memoizedProps.unstable_avoidThisFallback!==!0||(Xe.current&1)!=0?mt===0&&(mt=3):((mt===0||mt===3)&&(mt=4),At===null||(Rl&134217727)==0&&(Wo&134217727)==0||No(At,$t))),(r||n)&&(e.flags|=4),null);case 4:return Ao(),ep(e),t===null&&EO(e.stateNode.containerInfo),null;case 10:return Wd(e),null;case 17:return Nt(e.type)&&Ic(),null;case 19:if(We(Xe),r=e.memoizedState,r===null)return null;if(o=(e.flags&64)!=0,s=r.rendering,s===null)if(o)Cl(r,!1);else{if(mt!==0||t!==null&&(t.flags&64)!=0)for(t=e.child;t!==null;){if(s=zc(t),s!==null){for(e.flags|=64,Cl(r,!1),o=s.updateQueue,o!==null&&(e.updateQueue=o,e.flags|=4),r.lastEffect===null&&(e.firstEffect=null),e.lastEffect=r.lastEffect,r=n,n=e.child;n!==null;)o=n,t=r,o.flags&=2,o.nextEffect=null,o.firstEffect=null,o.lastEffect=null,s=o.alternate,s===null?(o.childLanes=0,o.lanes=t,o.child=null,o.memoizedProps=null,o.memoizedState=null,o.updateQueue=null,o.dependencies=null,o.stateNode=null):(o.childLanes=s.childLanes,o.lanes=s.lanes,o.child=s.child,o.memoizedProps=s.memoizedProps,o.memoizedState=s.memoizedState,o.updateQueue=s.updateQueue,o.type=s.type,t=s.dependencies,o.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),n=n.sibling;return je(Xe,Xe.current&1|2),e.child}t=t.sibling}r.tail!==null&&wt()>fp&&(e.flags|=64,o=!0,Cl(r,!1),e.lanes=33554432)}else{if(!o)if(t=zc(s),t!==null){if(e.flags|=64,o=!0,n=t.updateQueue,n!==null&&(e.updateQueue=n,e.flags|=4),Cl(r,!0),r.tail===null&&r.tailMode==="hidden"&&!s.alternate&&!Fn)return e=e.lastEffect=r.lastEffect,e!==null&&(e.nextEffect=null),null}else 2*wt()-r.renderingStartTime>fp&&n!==1073741824&&(e.flags|=64,o=!0,Cl(r,!1),e.lanes=33554432);r.isBackwards?(s.sibling=e.child,e.child=s):(n=r.last,n!==null?n.sibling=s:e.child=s,r.last=s)}return r.tail!==null?(n=r.tail,r.rendering=n,r.tail=n.sibling,r.lastEffect=e.lastEffect,r.renderingStartTime=wt(),n.sibling=null,e=Xe.current,je(Xe,o?e&1|2:e&1),n):null;case 23:case 24:return yp(),t!==null&&t.memoizedState!==null!=(e.memoizedState!==null)&&r.mode!=="unstable-defer-without-hiding"&&(e.flags|=4),null}throw Error(z(156,e.tag))}function JR(t){switch(t.tag){case 1:Nt(t.type)&&Ic();var e=t.flags;return e&4096?(t.flags=e&-4097|64,t):null;case 3:if(Ao(),We(Qt),We(xt),Fd(),e=t.flags,(e&64)!=0)throw Error(z(285));return t.flags=e&-4097|64,t;case 5:return Bd(t),null;case 13:return We(Xe),e=t.flags,e&4096?(t.flags=e&-4097|64,t):null;case 19:return We(Xe),null;case 4:return Ao(),null;case 10:return Wd(t),null;case 23:case 24:return yp(),null;default:return null}}function tp(t,e){try{var n="",r=e;do n+=AC(r),r=r.return;while(r);var i=n}catch(o){i=`
Error generating stack: `+o.message+`
`+o.stack}return{value:t,source:e,stack:i}}function np(t,e){try{console.error(e.value)}catch(n){setTimeout(function(){throw n})}}var e2=typeof WeakMap=="function"?WeakMap:Map;function Qv(t,e,n){n=Yr(-1,n),n.tag=3,n.payload={element:null};var r=e.value;return n.callback=function(){Kc||(Kc=!0,hp=r),np(t,e)},n}function Nv(t,e,n){n=Yr(-1,n),n.tag=3;var r=t.type.getDerivedStateFromError;if(typeof r=="function"){var i=e.value;n.payload=function(){return np(t,e),r(i)}}var o=t.stateNode;return o!==null&&typeof o.componentDidCatch=="function"&&(n.callback=function(){typeof r!="function"&&(Un===null?Un=new Set([this]):Un.add(this),np(t,e));var s=e.stack;this.componentDidCatch(e.value,{componentStack:s!==null?s:""})}),n}var t2=typeof WeakSet=="function"?WeakSet:Set;function _v(t){var e=t.ref;if(e!==null)if(typeof e=="function")try{e(null)}catch(n){ii(t,n)}else e.current=null}function n2(t,e){switch(e.tag){case 0:case 11:case 15:case 22:return;case 1:if(e.flags&256&&t!==null){var n=t.memoizedProps,r=t.memoizedState;t=e.stateNode,e=t.getSnapshotBeforeUpdate(e.elementType===e.type?n:Mn(e.type,n),r),t.__reactInternalSnapshotBeforeUpdate=e}return;case 3:e.flags&256&&Pd(e.stateNode.containerInfo);return;case 5:case 6:case 4:case 17:return}throw Error(z(163))}function r2(t,e,n){switch(n.tag){case 0:case 11:case 15:case 22:if(e=n.updateQueue,e=e!==null?e.lastEffect:null,e!==null){t=e=e.next;do{if((t.tag&3)==3){var r=t.create;t.destroy=r()}t=t.next}while(t!==e)}if(e=n.updateQueue,e=e!==null?e.lastEffect:null,e!==null){t=e=e.next;do{var i=t;r=i.next,i=i.tag,(i&4)!=0&&(i&1)!=0&&(e1(n,t),f2(n,t)),t=r}while(t!==e)}return;case 1:t=n.stateNode,n.flags&4&&(e===null?t.componentDidMount():(r=n.elementType===n.type?e.memoizedProps:Mn(n.type,e.memoizedProps),t.componentDidUpdate(r,e.memoizedState,t.__reactInternalSnapshotBeforeUpdate))),e=n.updateQueue,e!==null&&tv(n,e,t);return;case 3:if(e=n.updateQueue,e!==null){if(t=null,n.child!==null)switch(n.child.tag){case 5:t=n.child.stateNode;break;case 1:t=n.child.stateNode}tv(n,e,t)}return;case 5:t=n.stateNode,e===null&&n.flags&4&&WO(n.type,n.memoizedProps)&&t.focus();return;case 6:return;case 4:return;case 12:return;case 13:n.memoizedState===null&&(n=n.alternate,n!==null&&(n=n.memoizedState,n!==null&&(n=n.dehydrated,n!==null&&Yy(n))));return;case 19:case 17:case 20:case 21:case 23:case 24:return}throw Error(z(163))}function Bv(t,e){for(var n=t;;){if(n.tag===5){var r=n.stateNode;if(e)r=r.style,typeof r.setProperty=="function"?r.setProperty("display","none","important"):r.display="none";else{r=n.stateNode;var i=n.memoizedProps.style;i=i!=null&&i.hasOwnProperty("display")?i.display:null,r.style.display=Dy("display",i)}}else if(n.tag===6)n.stateNode.nodeValue=e?"":n.memoizedProps;else if((n.tag!==23&&n.tag!==24||n.memoizedState===null||n===t)&&n.child!==null){n.child.return=n,n=n.child;continue}if(n===t)break;for(;n.sibling===null;){if(n.return===null||n.return===t)return;n=n.return}n.sibling.return=n.return,n=n.sibling}}function qv(t,e){if(Di&&typeof Di.onCommitFiberUnmount=="function")try{Di.onCommitFiberUnmount(Td,e)}catch{}switch(e.tag){case 0:case 11:case 14:case 15:case 22:if(t=e.updateQueue,t!==null&&(t=t.lastEffect,t!==null)){var n=t=t.next;do{var r=n,i=r.destroy;if(r=r.tag,i!==void 0)if((r&4)!=0)e1(e,n);else{r=e;try{i()}catch(o){ii(r,o)}}n=n.next}while(n!==t)}break;case 1:if(_v(e),t=e.stateNode,typeof t.componentWillUnmount=="function")try{t.props=e.memoizedProps,t.state=e.memoizedState,t.componentWillUnmount()}catch(o){ii(e,o)}break;case 5:_v(e);break;case 4:Uv(t,e)}}function zv(t){t.alternate=null,t.child=null,t.dependencies=null,t.firstEffect=null,t.lastEffect=null,t.memoizedProps=null,t.memoizedState=null,t.pendingProps=null,t.return=null,t.updateQueue=null}function Fv(t){return t.tag===5||t.tag===3||t.tag===4}function Vv(t){e:{for(var e=t.return;e!==null;){if(Fv(e))break e;e=e.return}throw Error(z(160))}var n=e;switch(e=n.stateNode,n.tag){case 5:var r=!1;break;case 3:e=e.containerInfo,r=!0;break;case 4:e=e.containerInfo,r=!0;break;default:throw Error(z(161))}n.flags&16&&(Hs(e,""),n.flags&=-17);e:t:for(n=t;;){for(;n.sibling===null;){if(n.return===null||Fv(n.return)){n=null;break e}n=n.return}for(n.sibling.return=n.return,n=n.sibling;n.tag!==5&&n.tag!==6&&n.tag!==18;){if(n.flags&2||n.child===null||n.tag===4)continue t;n.child.return=n,n=n.child}if(!(n.flags&2)){n=n.stateNode;break e}}r?rp(t,n,e):ip(t,n,e)}function rp(t,e,n){var r=t.tag,i=r===5||r===6;if(i)t=i?t.stateNode:t.stateNode.instance,e?n.nodeType===8?n.parentNode.insertBefore(t,e):n.insertBefore(t,e):(n.nodeType===8?(e=n.parentNode,e.insertBefore(t,n)):(e=n,e.appendChild(t)),n=n._reactRootContainer,n!=null||e.onclick!==null||(e.onclick=Pc));else if(r!==4&&(t=t.child,t!==null))for(rp(t,e,n),t=t.sibling;t!==null;)rp(t,e,n),t=t.sibling}function ip(t,e,n){var r=t.tag,i=r===5||r===6;if(i)t=i?t.stateNode:t.stateNode.instance,e?n.insertBefore(t,e):n.appendChild(t);else if(r!==4&&(t=t.child,t!==null))for(ip(t,e,n),t=t.sibling;t!==null;)ip(t,e,n),t=t.sibling}function Uv(t,e){for(var n=e,r=!1,i,o;;){if(!r){r=n.return;e:for(;;){if(r===null)throw Error(z(160));switch(i=r.stateNode,r.tag){case 5:o=!1;break e;case 3:i=i.containerInfo,o=!0;break e;case 4:i=i.containerInfo,o=!0;break e}r=r.return}r=!0}if(n.tag===5||n.tag===6){e:for(var s=t,l=n,a=l;;)if(qv(s,a),a.child!==null&&a.tag!==4)a.child.return=a,a=a.child;else{if(a===l)break e;for(;a.sibling===null;){if(a.return===null||a.return===l)break e;a=a.return}a.sibling.return=a.return,a=a.sibling}o?(s=i,l=n.stateNode,s.nodeType===8?s.parentNode.removeChild(l):s.removeChild(l)):i.removeChild(n.stateNode)}else if(n.tag===4){if(n.child!==null){i=n.stateNode.containerInfo,o=!0,n.child.return=n,n=n.child;continue}}else if(qv(t,n),n.child!==null){n.child.return=n,n=n.child;continue}if(n===e)break;for(;n.sibling===null;){if(n.return===null||n.return===e)return;n=n.return,n.tag===4&&(r=!1)}n.sibling.return=n.return,n=n.sibling}}function op(t,e){switch(e.tag){case 0:case 11:case 14:case 15:case 22:var n=e.updateQueue;if(n=n!==null?n.lastEffect:null,n!==null){var r=n=n.next;do(r.tag&3)==3&&(t=r.destroy,r.destroy=void 0,t!==void 0&&t()),r=r.next;while(r!==n)}return;case 1:return;case 5:if(n=e.stateNode,n!=null){r=e.memoizedProps;var i=t!==null?t.memoizedProps:r;t=e.type;var o=e.updateQueue;if(e.updateQueue=null,o!==null){for(n[Rc]=r,t==="input"&&r.type==="radio"&&r.name!=null&&Cy(n,r),jh(t,i),e=jh(t,r),i=0;i<o.length;i+=2){var s=o[i],l=o[i+1];s==="style"?Ly(n,l):s==="dangerouslySetInnerHTML"?Ay(n,l):s==="children"?Hs(n,l):Ch(n,s,l,e)}switch(t){case"input":_h(n,r);break;case"textarea":Iy(n,r);break;case"select":t=n._wrapperState.wasMultiple,n._wrapperState.wasMultiple=!!r.multiple,o=r.value,o!=null?mo(n,!!r.multiple,o,!1):t!==!!r.multiple&&(r.defaultValue!=null?mo(n,!!r.multiple,r.defaultValue,!0):mo(n,!!r.multiple,r.multiple?[]:"",!1))}}}return;case 6:if(e.stateNode===null)throw Error(z(162));e.stateNode.nodeValue=e.memoizedProps;return;case 3:n=e.stateNode,n.hydrate&&(n.hydrate=!1,Yy(n.containerInfo));return;case 12:return;case 13:e.memoizedState!==null&&(up=wt(),Bv(e.child,!0)),jv(e);return;case 19:jv(e);return;case 17:return;case 23:case 24:Bv(e,e.memoizedState!==null);return}throw Error(z(163))}function jv(t){var e=t.updateQueue;if(e!==null){t.updateQueue=null;var n=t.stateNode;n===null&&(n=t.stateNode=new t2),e.forEach(function(r){var i=p2.bind(null,t,r);n.has(r)||(n.add(r),r.then(i,i))})}}function i2(t,e){return t!==null&&(t=t.memoizedState,t===null||t.dehydrated!==null)?(e=e.memoizedState,e!==null&&e.dehydrated===null):!1}var o2=Math.ceil,Zc=Ri.ReactCurrentDispatcher,sp=Ri.ReactCurrentOwner,le=0,At=null,nt=null,$t=0,_i=0,lp=Hr(0),mt=0,Yc=null,Lo=0,Rl=0,Wo=0,ap=0,cp=null,up=0,fp=1/0;function Qo(){fp=wt()+500}var Y=null,Kc=!1,hp=null,Un=null,ei=!1,Tl=null,Il=90,dp=[],pp=[],yr=null,El=0,mp=null,Jc=-1,Or=0,eu=0,Ml=null,tu=!1;function en(){return(le&48)!=0?wt():Jc!==-1?Jc:Jc=wt()}function ti(t){if(t=t.mode,(t&2)==0)return 1;if((t&4)==0)return Io()===99?1:2;if(Or===0&&(Or=Lo),jR.transition!==0){eu!==0&&(eu=cp!==null?cp.pendingLanes:0),t=Or;var e=4186112&~eu;return e&=-e,e===0&&(t=4186112&~t,e=t&-t,e===0&&(e=8192)),e}return t=Io(),(le&4)!=0&&t===98?t=yc(12,Or):(t=GC(t),t=yc(t,Or)),t}function ni(t,e,n){if(50<El)throw El=0,mp=null,Error(z(185));if(t=nu(t,e),t===null)return null;Oc(t,e,n),t===At&&(Wo|=e,mt===4&&No(t,$t));var r=Io();e===1?(le&8)!=0&&(le&48)==0?gp(t):(mn(t,n),le===0&&(Qo(),qn())):((le&4)==0||r!==98&&r!==99||(yr===null?yr=new Set([t]):yr.add(t)),mn(t,n)),cp=t}function nu(t,e){t.lanes|=e;var n=t.alternate;for(n!==null&&(n.lanes|=e),n=t,t=t.return;t!==null;)t.childLanes|=e,n=t.alternate,n!==null&&(n.childLanes|=e),n=t,t=t.return;return n.tag===3?n.stateNode:null}function mn(t,e){for(var n=t.callbackNode,r=t.suspendedLanes,i=t.pingedLanes,o=t.expirationTimes,s=t.pendingLanes;0<s;){var l=31-Ur(s),a=1<<l,c=o[l];if(c===-1){if((a&r)==0||(a&i)!=0){c=e,vo(a);var u=Ae;o[l]=10<=u?c+250:6<=u?c+5e3:-1}}else c<=e&&(t.expiredLanes|=a);s&=~a}if(r=il(t,t===At?$t:0),e=Ae,r===0)n!==null&&(n!==Ad&&Ed(n),t.callbackNode=null,t.callbackPriority=0);else{if(n!==null){if(t.callbackPriority===e)return;n!==Ad&&Ed(n)}e===15?(n=gp.bind(null,t),pr===null?(pr=[n],Ac=Id(Mc,YO)):pr.push(n),n=Ad):e===14?n=ml(99,gp.bind(null,t)):(n=ZC(e),n=ml(n,Xv.bind(null,t))),t.callbackPriority=e,t.callbackNode=n}}function Xv(t){if(Jc=-1,eu=Or=0,(le&48)!=0)throw Error(z(327));var e=t.callbackNode;if(ri()&&t.callbackNode!==e)return null;var n=il(t,t===At?$t:0);if(n===0)return null;var r=n,i=le;le|=16;var o=Yv();(At!==t||$t!==r)&&(Qo(),_o(t,r));do try{a2();break}catch(l){Zv(t,l)}while(1);if(Ld(),Zc.current=o,le=i,nt!==null?r=0:(At=null,$t=0,r=mt),(Lo&Wo)!=0)_o(t,0);else if(r!==0){if(r===2&&(le|=64,t.hydrate&&(t.hydrate=!1,Pd(t.containerInfo)),n=iO(t),n!==0&&(r=Al(t,n))),r===1)throw e=Yc,_o(t,0),No(t,n),mn(t,wt()),e;switch(t.finishedWork=t.current.alternate,t.finishedLanes=n,r){case 0:case 1:throw Error(z(345));case 2:Bi(t);break;case 3:if(No(t,n),(n&62914560)===n&&(r=up+500-wt(),10<r)){if(il(t,0)!==0)break;if(i=t.suspendedLanes,(i&n)!==n){en(),t.pingedLanes|=t.suspendedLanes&i;break}t.timeoutHandle=QO(Bi.bind(null,t),r);break}Bi(t);break;case 4:if(No(t,n),(n&4186112)===n)break;for(r=t.eventTimes,i=-1;0<n;){var s=31-Ur(n);o=1<<s,s=r[s],s>i&&(i=s),n&=~o}if(n=i,n=wt()-n,n=(120>n?120:480>n?480:1080>n?1080:1920>n?1920:3e3>n?3e3:4320>n?4320:1960*o2(n/1960))-n,10<n){t.timeoutHandle=QO(Bi.bind(null,t),n);break}Bi(t);break;case 5:Bi(t);break;default:throw Error(z(329))}}return mn(t,wt()),t.callbackNode===e?Xv.bind(null,t):null}function No(t,e){for(e&=~ap,e&=~Wo,t.suspendedLanes|=e,t.pingedLanes&=~e,t=t.expirationTimes;0<e;){var n=31-Ur(e),r=1<<n;t[n]=-1,e&=~r}}function gp(t){if((le&48)!=0)throw Error(z(327));if(ri(),t===At&&(t.expiredLanes&$t)!=0){var e=$t,n=Al(t,e);(Lo&Wo)!=0&&(e=il(t,e),n=Al(t,e))}else e=il(t,0),n=Al(t,e);if(t.tag!==0&&n===2&&(le|=64,t.hydrate&&(t.hydrate=!1,Pd(t.containerInfo)),e=iO(t),e!==0&&(n=Al(t,e))),n===1)throw n=Yc,_o(t,0),No(t,e),mn(t,wt()),n;return t.finishedWork=t.current.alternate,t.finishedLanes=e,Bi(t),mn(t,wt()),null}function s2(){if(yr!==null){var t=yr;yr=null,t.forEach(function(e){e.expiredLanes|=24&e.pendingLanes,mn(e,wt())})}qn()}function Hv(t,e){var n=le;le|=1;try{return t(e)}finally{le=n,le===0&&(Qo(),qn())}}function Gv(t,e){var n=le;le&=-2,le|=8;try{return t(e)}finally{le=n,le===0&&(Qo(),qn())}}function ru(t,e){je(lp,_i),_i|=e,Lo|=e}function yp(){_i=lp.current,We(lp)}function _o(t,e){t.finishedWork=null,t.finishedLanes=0;var n=t.timeoutHandle;if(n!==-1&&(t.timeoutHandle=-1,BR(n)),nt!==null)for(n=nt.return;n!==null;){var r=n;switch(r.tag){case 1:r=r.type.childContextTypes,r!=null&&Ic();break;case 3:Ao(),We(Qt),We(xt),Fd();break;case 5:Bd(r);break;case 4:Ao();break;case 13:We(Xe);break;case 19:We(Xe);break;case 10:Wd(r);break;case 23:case 24:yp()}n=n.return}At=t,nt=oi(t.current,null),$t=_i=Lo=e,mt=0,Yc=null,ap=Wo=Rl=0}function Zv(t,e){do{var n=nt;try{if(Ld(),wl.current=Hc,Vc){for(var r=Ze.memoizedState;r!==null;){var i=r.queue;i!==null&&(i.pending=null),r=r.next}Vc=!1}if(bl=0,pt=bt=Ze=null,$l=!1,sp.current=null,n===null||n.return===null){mt=1,Yc=e,nt=null;break}e:{var o=t,s=n.return,l=n,a=e;if(e=$t,l.flags|=2048,l.firstEffect=l.lastEffect=null,a!==null&&typeof a=="object"&&typeof a.then=="function"){var c=a;if((l.mode&2)==0){var u=l.alternate;u?(l.updateQueue=u.updateQueue,l.memoizedState=u.memoizedState,l.lanes=u.lanes):(l.updateQueue=null,l.memoizedState=null)}var f=(Xe.current&1)!=0,h=s;do{var d;if(d=h.tag===13){var p=h.memoizedState;if(p!==null)d=p.dehydrated!==null;else{var m=h.memoizedProps;d=m.fallback===void 0?!1:m.unstable_avoidThisFallback!==!0?!0:!f}}if(d){var g=h.updateQueue;if(g===null){var y=new Set;y.add(c),h.updateQueue=y}else g.add(c);if((h.mode&2)==0){if(h.flags|=64,l.flags|=16384,l.flags&=-2981,l.tag===1)if(l.alternate===null)l.tag=17;else{var O=Yr(-1,1);O.tag=2,Kr(l,O)}l.lanes|=1;break e}a=void 0,l=e;var v=o.pingCache;if(v===null?(v=o.pingCache=new e2,a=new Set,v.set(c,a)):(a=v.get(c),a===void 0&&(a=new Set,v.set(c,a))),!a.has(l)){a.add(l);var x=d2.bind(null,o,c,l);c.then(x,x)}h.flags|=4096,h.lanes=e;break e}h=h.return}while(h!==null);a=Error((po(l.type)||"A React component")+` suspended while rendering, but no fallback UI was specified.

Add a <Suspense fallback=...> component higher in the tree to provide a loading indicator or placeholder to display.`)}mt!==5&&(mt=2),a=tp(a,l),h=s;do{switch(h.tag){case 3:o=a,h.flags|=4096,e&=-e,h.lanes|=e;var b=Qv(h,o,e);ev(h,b);break e;case 1:o=a;var S=h.type,k=h.stateNode;if((h.flags&64)==0&&(typeof S.getDerivedStateFromError=="function"||k!==null&&typeof k.componentDidCatch=="function"&&(Un===null||!Un.has(k)))){h.flags|=4096,e&=-e,h.lanes|=e;var P=Nv(h,o,e);ev(h,P);break e}}h=h.return}while(h!==null)}Jv(n)}catch(C){e=C,nt===n&&n!==null&&(nt=n=n.return);continue}break}while(1)}function Yv(){var t=Zc.current;return Zc.current=Hc,t===null?Hc:t}function Al(t,e){var n=le;le|=16;var r=Yv();At===t&&$t===e||_o(t,e);do try{l2();break}catch(i){Zv(t,i)}while(1);if(Ld(),le=n,Zc.current=r,nt!==null)throw Error(z(261));return At=null,$t=0,mt}function l2(){for(;nt!==null;)Kv(nt)}function a2(){for(;nt!==null&&!FR();)Kv(nt)}function Kv(t){var e=n1(t.alternate,t,_i);t.memoizedProps=t.pendingProps,e===null?Jv(t):nt=e,sp.current=null}function Jv(t){var e=t;do{var n=e.alternate;if(t=e.return,(e.flags&2048)==0){if(n=KR(n,e,_i),n!==null){nt=n;return}if(n=e,n.tag!==24&&n.tag!==23||n.memoizedState===null||(_i&1073741824)!=0||(n.mode&4)==0){for(var r=0,i=n.child;i!==null;)r|=i.lanes|i.childLanes,i=i.sibling;n.childLanes=r}t!==null&&(t.flags&2048)==0&&(t.firstEffect===null&&(t.firstEffect=e.firstEffect),e.lastEffect!==null&&(t.lastEffect!==null&&(t.lastEffect.nextEffect=e.firstEffect),t.lastEffect=e.lastEffect),1<e.flags&&(t.lastEffect!==null?t.lastEffect.nextEffect=e:t.firstEffect=e,t.lastEffect=e))}else{if(n=JR(e),n!==null){n.flags&=2047,nt=n;return}t!==null&&(t.firstEffect=t.lastEffect=null,t.flags|=2048)}if(e=e.sibling,e!==null){nt=e;return}nt=e=t}while(e!==null);mt===0&&(mt=5)}function Bi(t){var e=Io();return Li(99,c2.bind(null,t,e)),null}function c2(t,e){do ri();while(Tl!==null);if((le&48)!=0)throw Error(z(327));var n=t.finishedWork;if(n===null)return null;if(t.finishedWork=null,t.finishedLanes=0,n===t.current)throw Error(z(177));t.callbackNode=null;var r=n.lanes|n.childLanes,i=r,o=t.pendingLanes&~i;t.pendingLanes=i,t.suspendedLanes=0,t.pingedLanes=0,t.expiredLanes&=i,t.mutableReadLanes&=i,t.entangledLanes&=i,i=t.entanglements;for(var s=t.eventTimes,l=t.expirationTimes;0<o;){var a=31-Ur(o),c=1<<a;i[a]=0,s[a]=-1,l[a]=-1,o&=~c}if(yr!==null&&(r&24)==0&&yr.has(t)&&yr.delete(t),t===At&&(nt=At=null,$t=0),1<n.flags?n.lastEffect!==null?(n.lastEffect.nextEffect=n,r=n.firstEffect):r=n:r=n.firstEffect,r!==null){if(i=le,le|=32,sp.current=null,$d=vc,s=SO(),vd(s)){if("selectionStart"in s)l={start:s.selectionStart,end:s.selectionEnd};else e:if(l=(l=s.ownerDocument)&&l.defaultView||window,(c=l.getSelection&&l.getSelection())&&c.rangeCount!==0){l=c.anchorNode,o=c.anchorOffset,a=c.focusNode,c=c.focusOffset;try{l.nodeType,a.nodeType}catch{l=null;break e}var u=0,f=-1,h=-1,d=0,p=0,m=s,g=null;t:for(;;){for(var y;m!==l||o!==0&&m.nodeType!==3||(f=u+o),m!==a||c!==0&&m.nodeType!==3||(h=u+c),m.nodeType===3&&(u+=m.nodeValue.length),(y=m.firstChild)!==null;)g=m,m=y;for(;;){if(m===s)break t;if(g===l&&++d===o&&(f=u),g===a&&++p===c&&(h=u),(y=m.nextSibling)!==null)break;m=g,g=m.parentNode}m=y}l=f===-1||h===-1?null:{start:f,end:h}}else l=null;l=l||{start:0,end:0}}else l=null;Sd={focusedElem:s,selectionRange:l},vc=!1,Ml=null,tu=!1,Y=r;do try{u2()}catch(C){if(Y===null)throw Error(z(330));ii(Y,C),Y=Y.nextEffect}while(Y!==null);Ml=null,Y=r;do try{for(s=t;Y!==null;){var O=Y.flags;if(O&16&&Hs(Y.stateNode,""),O&128){var v=Y.alternate;if(v!==null){var x=v.ref;x!==null&&(typeof x=="function"?x(null):x.current=null)}}switch(O&1038){case 2:Vv(Y),Y.flags&=-3;break;case 6:Vv(Y),Y.flags&=-3,op(Y.alternate,Y);break;case 1024:Y.flags&=-1025;break;case 1028:Y.flags&=-1025,op(Y.alternate,Y);break;case 4:op(Y.alternate,Y);break;case 8:l=Y,Uv(s,l);var b=l.alternate;zv(l),b!==null&&zv(b)}Y=Y.nextEffect}}catch(C){if(Y===null)throw Error(z(330));ii(Y,C),Y=Y.nextEffect}while(Y!==null);if(x=Sd,v=SO(),O=x.focusedElem,s=x.selectionRange,v!==O&&O&&O.ownerDocument&&$O(O.ownerDocument.documentElement,O)){for(s!==null&&vd(O)&&(v=s.start,x=s.end,x===void 0&&(x=v),"selectionStart"in O?(O.selectionStart=v,O.selectionEnd=Math.min(x,O.value.length)):(x=(v=O.ownerDocument||document)&&v.defaultView||window,x.getSelection&&(x=x.getSelection(),l=O.textContent.length,b=Math.min(s.start,l),s=s.end===void 0?b:Math.min(s.end,l),!x.extend&&b>s&&(l=s,s=b,b=l),l=bO(O,b),o=bO(O,s),l&&o&&(x.rangeCount!==1||x.anchorNode!==l.node||x.anchorOffset!==l.offset||x.focusNode!==o.node||x.focusOffset!==o.offset)&&(v=v.createRange(),v.setStart(l.node,l.offset),x.removeAllRanges(),b>s?(x.addRange(v),x.extend(o.node,o.offset)):(v.setEnd(o.node,o.offset),x.addRange(v)))))),v=[],x=O;x=x.parentNode;)x.nodeType===1&&v.push({element:x,left:x.scrollLeft,top:x.scrollTop});for(typeof O.focus=="function"&&O.focus(),O=0;O<v.length;O++)x=v[O],x.element.scrollLeft=x.left,x.element.scrollTop=x.top}vc=!!$d,Sd=$d=null,t.current=n,Y=r;do try{for(O=t;Y!==null;){var S=Y.flags;if(S&36&&r2(O,Y.alternate,Y),S&128){v=void 0;var k=Y.ref;if(k!==null){var P=Y.stateNode;switch(Y.tag){case 5:v=P;break;default:v=P}typeof k=="function"?k(v):k.current=v}}Y=Y.nextEffect}}catch(C){if(Y===null)throw Error(z(330));ii(Y,C),Y=Y.nextEffect}while(Y!==null);Y=null,UR(),le=i}else t.current=n;if(ei)ei=!1,Tl=t,Il=e;else for(Y=r;Y!==null;)e=Y.nextEffect,Y.nextEffect=null,Y.flags&8&&(S=Y,S.sibling=null,S.stateNode=null),Y=e;if(r=t.pendingLanes,r===0&&(Un=null),r===1?t===mp?El++:(El=0,mp=t):El=0,n=n.stateNode,Di&&typeof Di.onCommitFiberRoot=="function")try{Di.onCommitFiberRoot(Td,n,void 0,(n.current.flags&64)==64)}catch{}if(mn(t,wt()),Kc)throw Kc=!1,t=hp,hp=null,t;return(le&8)!=0||qn(),null}function u2(){for(;Y!==null;){var t=Y.alternate;tu||Ml===null||((Y.flags&8)!=0?Vy(Y,Ml)&&(tu=!0):Y.tag===13&&i2(t,Y)&&Vy(Y,Ml)&&(tu=!0));var e=Y.flags;(e&256)!=0&&n2(t,Y),(e&512)==0||ei||(ei=!0,ml(97,function(){return ri(),null})),Y=Y.nextEffect}}function ri(){if(Il!==90){var t=97<Il?97:Il;return Il=90,Li(t,h2)}return!1}function f2(t,e){dp.push(e,t),ei||(ei=!0,ml(97,function(){return ri(),null}))}function e1(t,e){pp.push(e,t),ei||(ei=!0,ml(97,function(){return ri(),null}))}function h2(){if(Tl===null)return!1;var t=Tl;if(Tl=null,(le&48)!=0)throw Error(z(331));var e=le;le|=32;var n=pp;pp=[];for(var r=0;r<n.length;r+=2){var i=n[r],o=n[r+1],s=i.destroy;if(i.destroy=void 0,typeof s=="function")try{s()}catch(a){if(o===null)throw Error(z(330));ii(o,a)}}for(n=dp,dp=[],r=0;r<n.length;r+=2){i=n[r],o=n[r+1];try{var l=i.create;i.destroy=l()}catch(a){if(o===null)throw Error(z(330));ii(o,a)}}for(l=t.current.firstEffect;l!==null;)t=l.nextEffect,l.nextEffect=null,l.flags&8&&(l.sibling=null,l.stateNode=null),l=t;return le=e,qn(),!0}function t1(t,e,n){e=tp(n,e),e=Qv(t,e,1),Kr(t,e),e=en(),t=nu(t,1),t!==null&&(Oc(t,1,e),mn(t,e))}function ii(t,e){if(t.tag===3)t1(t,t,e);else for(var n=t.return;n!==null;){if(n.tag===3){t1(n,t,e);break}else if(n.tag===1){var r=n.stateNode;if(typeof n.type.getDerivedStateFromError=="function"||typeof r.componentDidCatch=="function"&&(Un===null||!Un.has(r))){t=tp(e,t);var i=Nv(n,t,1);if(Kr(n,i),i=en(),n=nu(n,1),n!==null)Oc(n,1,i),mn(n,i);else if(typeof r.componentDidCatch=="function"&&(Un===null||!Un.has(r)))try{r.componentDidCatch(e,t)}catch{}break}}n=n.return}}function d2(t,e,n){var r=t.pingCache;r!==null&&r.delete(e),e=en(),t.pingedLanes|=t.suspendedLanes&n,At===t&&($t&n)===n&&(mt===4||mt===3&&($t&62914560)===$t&&500>wt()-up?_o(t,0):ap|=n),mn(t,e)}function p2(t,e){var n=t.stateNode;n!==null&&n.delete(e),e=0,e===0&&(e=t.mode,(e&2)==0?e=1:(e&4)==0?e=Io()===99?1:2:(Or===0&&(Or=Lo),e=xo(62914560&~Or),e===0&&(e=4194304))),n=en(),t=nu(t,e),t!==null&&(Oc(t,e,n),mn(t,n))}var n1;n1=function(t,e,n){var r=e.lanes;if(t!==null)if(t.memoizedProps!==e.pendingProps||Qt.current)An=!0;else if((n&r)!=0)An=(t.flags&16384)!=0;else{switch(An=!1,e.tag){case 3:Cv(e),zd();break;case 5:av(e);break;case 1:Nt(e.type)&&Ec(e);break;case 4:_d(e,e.stateNode.containerInfo);break;case 10:r=e.memoizedProps.value;var i=e.type._context;je(Dc,i._currentValue),i._currentValue=r;break;case 13:if(e.memoizedState!==null)return(n&e.child.childLanes)!=0?Rv(t,e,n):(je(Xe,Xe.current&1),e=gr(t,e,n),e!==null?e.sibling:null);je(Xe,Xe.current&1);break;case 19:if(r=(n&e.childLanes)!=0,(t.flags&64)!=0){if(r)return Av(t,e,n);e.flags|=64}if(i=e.memoizedState,i!==null&&(i.rendering=null,i.tail=null,i.lastEffect=null),je(Xe,Xe.current),r)break;return null;case 23:case 24:return e.lanes=0,Zd(t,e,n)}return gr(t,e,n)}else An=!1;switch(e.lanes=0,e.tag){case 2:if(r=e.type,t!==null&&(t.alternate=null,e.alternate=null,e.flags|=2),t=e.pendingProps,i=To(e,xt.current),Mo(e,n),i=Ud(null,e,r,t,i,n),e.flags|=1,typeof i=="object"&&i!==null&&typeof i.render=="function"&&i.$$typeof===void 0){if(e.tag=1,e.memoizedState=null,e.updateQueue=null,Nt(r)){var o=!0;Ec(e)}else o=!1;e.memoizedState=i.state!==null&&i.state!==void 0?i.state:null,Qd(e);var s=r.getDerivedStateFromProps;typeof s=="function"&&Qc(e,r,s,t),i.updater=Nc,e.stateNode=i,i._reactInternals=e,Nd(e,r,t,n),e=Kd(null,e,r,!0,o,n)}else e.tag=0,Bt(null,e,i,n),e=e.child;return e;case 16:i=e.elementType;e:{switch(t!==null&&(t.alternate=null,e.alternate=null,e.flags|=2),t=e.pendingProps,o=i._init,i=o(i._payload),e.type=i,o=e.tag=g2(i),t=Mn(i,t),o){case 0:e=Yd(null,e,i,t,n);break e;case 1:e=Pv(null,e,i,t,n);break e;case 11:e=bv(null,e,i,t,n);break e;case 14:e=$v(null,e,i,Mn(i.type,t),r,n);break e}throw Error(z(306,i,""))}return e;case 0:return r=e.type,i=e.pendingProps,i=e.elementType===r?i:Mn(r,i),Yd(t,e,r,i,n);case 1:return r=e.type,i=e.pendingProps,i=e.elementType===r?i:Mn(r,i),Pv(t,e,r,i,n);case 3:if(Cv(e),r=e.updateQueue,t===null||r===null)throw Error(z(282));if(r=e.pendingProps,i=e.memoizedState,i=i!==null?i.element:null,JO(t,e),gl(e,r,null,n),r=e.memoizedState.element,r===i)zd(),e=gr(t,e,n);else{if(i=e.stateNode,(o=i.hydrate)&&(Jr=ko(e.stateNode.containerInfo.firstChild),mr=e,o=Fn=!0),o){if(t=i.mutableSourceEagerHydrationData,t!=null)for(i=0;i<t.length;i+=2)o=t[i],o._workInProgressVersionPrimary=t[i+1],Do.push(o);for(n=lv(e,null,r,n),e.child=n;n;)n.flags=n.flags&-3|1024,n=n.sibling}else Bt(t,e,r,n),zd();e=e.child}return e;case 5:return av(e),t===null&&qd(e),r=e.type,i=e.pendingProps,o=t!==null?t.memoizedProps:null,s=i.children,kd(r,i)?s=null:o!==null&&kd(r,o)&&(e.flags|=16),kv(t,e),Bt(t,e,s,n),e.child;case 6:return t===null&&qd(e),null;case 13:return Rv(t,e,n);case 4:return _d(e,e.stateNode.containerInfo),r=e.pendingProps,t===null?e.child=qc(e,null,r,n):Bt(t,e,r,n),e.child;case 11:return r=e.type,i=e.pendingProps,i=e.elementType===r?i:Mn(r,i),bv(t,e,r,i,n);case 7:return Bt(t,e,e.pendingProps,n),e.child;case 8:return Bt(t,e,e.pendingProps.children,n),e.child;case 12:return Bt(t,e,e.pendingProps.children,n),e.child;case 10:e:{r=e.type._context,i=e.pendingProps,s=e.memoizedProps,o=i.value;var l=e.type._context;if(je(Dc,l._currentValue),l._currentValue=o,s!==null)if(l=s.value,o=hn(l,o)?0:(typeof r._calculateChangedBits=="function"?r._calculateChangedBits(l,o):1073741823)|0,o===0){if(s.children===i.children&&!Qt.current){e=gr(t,e,n);break e}}else for(l=e.child,l!==null&&(l.return=e);l!==null;){var a=l.dependencies;if(a!==null){s=l.child;for(var c=a.firstContext;c!==null;){if(c.context===r&&(c.observedBits&o)!=0){l.tag===1&&(c=Yr(-1,n&-n),c.tag=2,Kr(l,c)),l.lanes|=n,c=l.alternate,c!==null&&(c.lanes|=n),KO(l.return,n),a.lanes|=n;break}c=c.next}}else s=l.tag===10&&l.type===e.type?null:l.child;if(s!==null)s.return=l;else for(s=l;s!==null;){if(s===e){s=null;break}if(l=s.sibling,l!==null){l.return=s.return,s=l;break}s=s.return}l=s}Bt(t,e,i.children,n),e=e.child}return e;case 9:return i=e.type,o=e.pendingProps,r=o.children,Mo(e,n),i=dn(i,o.unstable_observedBits),r=r(i),e.flags|=1,Bt(t,e,r,n),e.child;case 14:return i=e.type,o=Mn(i,e.pendingProps),o=Mn(i.type,o),$v(t,e,i,o,r,n);case 15:return Sv(t,e,e.type,e.pendingProps,r,n);case 17:return r=e.type,i=e.pendingProps,i=e.elementType===r?i:Mn(r,i),t!==null&&(t.alternate=null,e.alternate=null,e.flags|=2),e.tag=1,Nt(r)?(t=!0,Ec(e)):t=!1,Mo(e,n),iv(e,r,i),Nd(e,r,i,n),Kd(null,e,r,!0,t,n);case 19:return Av(t,e,n);case 23:return Zd(t,e,n);case 24:return Zd(t,e,n)}throw Error(z(156,e.tag))};function m2(t,e,n,r){this.tag=t,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.ref=null,this.pendingProps=e,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=r,this.flags=0,this.lastEffect=this.firstEffect=this.nextEffect=null,this.childLanes=this.lanes=0,this.alternate=null}function gn(t,e,n,r){return new m2(t,e,n,r)}function Op(t){return t=t.prototype,!(!t||!t.isReactComponent)}function g2(t){if(typeof t=="function")return Op(t)?1:0;if(t!=null){if(t=t.$$typeof,t===oc)return 11;if(t===lc)return 14}return 2}function oi(t,e){var n=t.alternate;return n===null?(n=gn(t.tag,e,t.key,t.mode),n.elementType=t.elementType,n.type=t.type,n.stateNode=t.stateNode,n.alternate=t,t.alternate=n):(n.pendingProps=e,n.type=t.type,n.flags=0,n.nextEffect=null,n.firstEffect=null,n.lastEffect=null),n.childLanes=t.childLanes,n.lanes=t.lanes,n.child=t.child,n.memoizedProps=t.memoizedProps,n.memoizedState=t.memoizedState,n.updateQueue=t.updateQueue,e=t.dependencies,n.dependencies=e===null?null:{lanes:e.lanes,firstContext:e.firstContext},n.sibling=t.sibling,n.index=t.index,n.ref=t.ref,n}function iu(t,e,n,r,i,o){var s=2;if(r=t,typeof t=="function")Op(t)&&(s=1);else if(typeof t=="string")s=5;else e:switch(t){case Br:return Bo(n.children,i,o,e);case by:s=8,i|=16;break;case Rh:s=8,i|=1;break;case Vs:return t=gn(12,n,e,i|8),t.elementType=Vs,t.type=Vs,t.lanes=o,t;case Us:return t=gn(13,n,e,i),t.type=Us,t.elementType=Us,t.lanes=o,t;case sc:return t=gn(19,n,e,i),t.elementType=sc,t.lanes=o,t;case Dh:return vp(n,i,o,e);case Lh:return t=gn(24,n,e,i),t.elementType=Lh,t.lanes=o,t;default:if(typeof t=="object"&&t!==null)switch(t.$$typeof){case Th:s=10;break e;case Ih:s=9;break e;case oc:s=11;break e;case lc:s=14;break e;case Eh:s=16,r=null;break e;case Mh:s=22;break e}throw Error(z(130,t==null?t:typeof t,""))}return e=gn(s,n,e,i),e.elementType=t,e.type=r,e.lanes=o,e}function Bo(t,e,n,r){return t=gn(7,t,r,e),t.lanes=n,t}function vp(t,e,n,r){return t=gn(23,t,r,e),t.elementType=Dh,t.lanes=n,t}function xp(t,e,n){return t=gn(6,t,null,e),t.lanes=n,t}function wp(t,e,n){return e=gn(4,t.children!==null?t.children:[],t.key,e),e.lanes=n,e.stateNode={containerInfo:t.containerInfo,pendingChildren:null,implementation:t.implementation},e}function y2(t,e,n){this.tag=e,this.containerInfo=t,this.finishedWork=this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.pendingContext=this.context=null,this.hydrate=n,this.callbackNode=null,this.callbackPriority=0,this.eventTimes=ld(0),this.expirationTimes=ld(-1),this.entangledLanes=this.finishedLanes=this.mutableReadLanes=this.expiredLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=ld(0),this.mutableSourceEagerHydrationData=null}function O2(t,e,n){var r=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:Ti,key:r==null?null:""+r,children:t,containerInfo:e,implementation:n}}function ou(t,e,n,r){var i=e.current,o=en(),s=ti(i);e:if(n){n=n._reactInternals;t:{if(Ei(n)!==n||n.tag!==1)throw Error(z(170));var l=n;do{switch(l.tag){case 3:l=l.stateNode.context;break t;case 1:if(Nt(l.type)){l=l.stateNode.__reactInternalMemoizedMergedChildContext;break t}}l=l.return}while(l!==null);throw Error(z(171))}if(n.tag===1){var a=n.type;if(Nt(a)){n=zO(n,a,l);break e}}n=l}else n=Gr;return e.context===null?e.context=n:e.pendingContext=n,e=Yr(o,s),e.payload={element:t},r=r===void 0?null:r,r!==null&&(e.callback=r),Kr(i,e),ni(i,s,o),s}function bp(t){if(t=t.current,!t.child)return null;switch(t.child.tag){case 5:return t.child.stateNode;default:return t.child.stateNode}}function r1(t,e){if(t=t.memoizedState,t!==null&&t.dehydrated!==null){var n=t.retryLane;t.retryLane=n!==0&&n<e?n:e}}function $p(t,e){r1(t,e),(t=t.alternate)&&r1(t,e)}function v2(){return null}function Sp(t,e,n){var r=n!=null&&n.hydrationOptions!=null&&n.hydrationOptions.mutableSources||null;if(n=new y2(t,e,n!=null&&n.hydrate===!0),e=gn(3,null,null,e===2?7:e===1?3:0),n.current=e,e.stateNode=n,Qd(e),t[Po]=n.current,EO(t.nodeType===8?t.parentNode:t),r)for(t=0;t<r.length;t++){e=r[t];var i=e._getVersion;i=i(e._source),n.mutableSourceEagerHydrationData==null?n.mutableSourceEagerHydrationData=[e,i]:n.mutableSourceEagerHydrationData.push(e,i)}this._internalRoot=n}Sp.prototype.render=function(t){ou(t,this._internalRoot,null,null)};Sp.prototype.unmount=function(){var t=this._internalRoot,e=t.containerInfo;ou(null,t,null,function(){e[Po]=null})};function Dl(t){return!(!t||t.nodeType!==1&&t.nodeType!==9&&t.nodeType!==11&&(t.nodeType!==8||t.nodeValue!==" react-mount-point-unstable "))}function x2(t,e){if(e||(e=t?t.nodeType===9?t.documentElement:t.firstChild:null,e=!(!e||e.nodeType!==1||!e.hasAttribute("data-reactroot"))),!e)for(var n;n=t.lastChild;)t.removeChild(n);return new Sp(t,0,e?{hydrate:!0}:void 0)}function su(t,e,n,r,i){var o=n._reactRootContainer;if(o){var s=o._internalRoot;if(typeof i=="function"){var l=i;i=function(){var c=bp(s);l.call(c)}}ou(e,s,t,i)}else{if(o=n._reactRootContainer=x2(n,r),s=o._internalRoot,typeof i=="function"){var a=i;i=function(){var c=bp(s);a.call(c)}}Gv(function(){ou(e,s,t,i)})}return bp(s)}Uy=function(t){if(t.tag===13){var e=en();ni(t,4,e),$p(t,4)}};td=function(t){if(t.tag===13){var e=en();ni(t,67108864,e),$p(t,67108864)}};jy=function(t){if(t.tag===13){var e=en(),n=ti(t);ni(t,n,e),$p(t,n)}};Xy=function(t,e){return e()};Hh=function(t,e,n){switch(e){case"input":if(_h(t,n),e=n.name,n.type==="radio"&&e!=null){for(n=t;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll("input[name="+JSON.stringify(""+e)+'][type="radio"]'),e=0;e<n.length;e++){var r=n[e];if(r!==t&&r.form===t.form){var i=Tc(r);if(!i)throw Error(z(90));ky(r),_h(r,i)}}}break;case"textarea":Iy(t,n);break;case"select":e=n.value,e!=null&&mo(t,!!n.multiple,e,!1)}};Gh=Hv;_y=function(t,e,n,r,i){var o=le;le|=4;try{return Li(98,t.bind(null,e,n,r,i))}finally{le=o,le===0&&(Qo(),qn())}};Zh=function(){(le&49)==0&&(s2(),ri())};By=function(t,e){var n=le;le|=2;try{return t(e)}finally{le=n,le===0&&(Qo(),qn())}};function i1(t,e){var n=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!Dl(e))throw Error(z(200));return O2(t,e,null,n)}var w2={Events:[pl,Co,Tc,Qy,Ny,ri,{current:!1}]},Ll={findFiberByHostInstance:Mi,bundleType:0,version:"17.0.2",rendererPackageName:"react-dom"},b2={bundleType:Ll.bundleType,version:Ll.version,rendererPackageName:Ll.rendererPackageName,rendererConfig:Ll.rendererConfig,overrideHookState:null,overrideHookStateDeletePath:null,overrideHookStateRenamePath:null,overrideProps:null,overridePropsDeletePath:null,overridePropsRenamePath:null,setSuspenseHandler:null,scheduleUpdate:null,currentDispatcherRef:Ri.ReactCurrentDispatcher,findHostInstanceByFiber:function(t){return t=Fy(t),t===null?null:t.stateNode},findFiberByHostInstance:Ll.findFiberByHostInstance||v2,findHostInstancesForRefresh:null,scheduleRefresh:null,scheduleRoot:null,setRefreshHandler:null,getCurrentFiber:null};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__!="undefined"){var lu=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!lu.isDisabled&&lu.supportsFiber)try{Td=lu.inject(b2),Di=lu}catch{}}fn.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED=w2;fn.createPortal=i1;fn.findDOMNode=function(t){if(t==null)return null;if(t.nodeType===1)return t;var e=t._reactInternals;if(e===void 0)throw typeof t.render=="function"?Error(z(188)):Error(z(268,Object.keys(t)));return t=Fy(e),t=t===null?null:t.stateNode,t};fn.flushSync=function(t,e){var n=le;if((n&48)!=0)return t(e);le|=1;try{if(t)return Li(99,t.bind(null,e))}finally{le=n,qn()}};fn.hydrate=function(t,e,n){if(!Dl(e))throw Error(z(200));return su(null,t,e,!0,n)};fn.render=function(t,e,n){if(!Dl(e))throw Error(z(200));return su(null,t,e,!1,n)};fn.unmountComponentAtNode=function(t){if(!Dl(t))throw Error(z(40));return t._reactRootContainer?(Gv(function(){su(null,null,t,!1,function(){t._reactRootContainer=null,t[Po]=null})}),!0):!1};fn.unstable_batchedUpdates=Hv;fn.unstable_createPortal=function(t,e){return i1(t,e,2<arguments.length&&arguments[2]!==void 0?arguments[2]:null)};fn.unstable_renderSubtreeIntoContainer=function(t,e,n,r){if(!Dl(n))throw Error(z(200));if(t==null||t._reactInternals===void 0)throw Error(z(38));return su(t,e,n,!1,r)};fn.version="17.0.2";function o1(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__=="undefined"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(o1)}catch(t){console.error(t)}}o1(),rc.exports=fn;var kp=rc.exports;eC(rc.exports.unstable_batchedUpdates);function $2(t,e,n){return e in t?Object.defineProperty(t,e,{value:n,enumerable:!0,configurable:!0,writable:!0}):t[e]=n,t}function s1(t,e){var n=Object.keys(t);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(t);e&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(t,i).enumerable})),n.push.apply(n,r)}return n}function l1(t){for(var e=1;e<arguments.length;e++){var n=arguments[e]!=null?arguments[e]:{};e%2?s1(Object(n),!0).forEach(function(r){$2(t,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):s1(Object(n)).forEach(function(r){Object.defineProperty(t,r,Object.getOwnPropertyDescriptor(n,r))})}return t}function St(t){return"Minified Redux error #"+t+"; visit https://redux.js.org/Errors?code="+t+" for the full message or use the non-minified dev environment for full errors. "}var a1=function(){return typeof Symbol=="function"&&Symbol.observable||"@@observable"}(),Pp=function(){return Math.random().toString(36).substring(7).split("").join(".")},au={INIT:"@@redux/INIT"+Pp(),REPLACE:"@@redux/REPLACE"+Pp(),PROBE_UNKNOWN_ACTION:function(){return"@@redux/PROBE_UNKNOWN_ACTION"+Pp()}};function S2(t){if(typeof t!="object"||t===null)return!1;for(var e=t;Object.getPrototypeOf(e)!==null;)e=Object.getPrototypeOf(e);return Object.getPrototypeOf(t)===e}function k2(t,e,n){var r;if(typeof e=="function"&&typeof n=="function"||typeof n=="function"&&typeof arguments[3]=="function")throw new Error(St(0));if(typeof e=="function"&&typeof n=="undefined"&&(n=e,e=void 0),typeof n!="undefined"){if(typeof n!="function")throw new Error(St(1));return n(k2)(t,e)}if(typeof t!="function")throw new Error(St(2));var i=t,o=e,s=[],l=s,a=!1;function c(){l===s&&(l=s.slice())}function u(){if(a)throw new Error(St(3));return o}function f(m){if(typeof m!="function")throw new Error(St(4));if(a)throw new Error(St(5));var g=!0;return c(),l.push(m),function(){if(!!g){if(a)throw new Error(St(6));g=!1,c();var O=l.indexOf(m);l.splice(O,1),s=null}}}function h(m){if(!S2(m))throw new Error(St(7));if(typeof m.type=="undefined")throw new Error(St(8));if(a)throw new Error(St(9));try{a=!0,o=i(o,m)}finally{a=!1}for(var g=s=l,y=0;y<g.length;y++){var O=g[y];O()}return m}function d(m){if(typeof m!="function")throw new Error(St(10));i=m,h({type:au.REPLACE})}function p(){var m,g=f;return m={subscribe:function(O){if(typeof O!="object"||O===null)throw new Error(St(11));function v(){O.next&&O.next(u())}v();var x=g(v);return{unsubscribe:x}}},m[a1]=function(){return this},m}return h({type:au.INIT}),r={dispatch:h,subscribe:f,getState:u,replaceReducer:d},r[a1]=p,r}function P2(t){Object.keys(t).forEach(function(e){var n=t[e],r=n(void 0,{type:au.INIT});if(typeof r=="undefined")throw new Error(St(12));if(typeof n(void 0,{type:au.PROBE_UNKNOWN_ACTION()})=="undefined")throw new Error(St(13))})}function y7(t){for(var e=Object.keys(t),n={},r=0;r<e.length;r++){var i=e[r];typeof t[i]=="function"&&(n[i]=t[i])}var o=Object.keys(n),s;try{P2(n)}catch(l){s=l}return function(a,c){if(a===void 0&&(a={}),s)throw s;for(var u=!1,f={},h=0;h<o.length;h++){var d=o[h],p=n[d],m=a[d],g=p(m,c);if(typeof g=="undefined")throw c&&c.type,new Error(St(14));f[d]=g,u=u||g!==m}return u=u||o.length!==Object.keys(a).length,u?f:a}}function C2(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return e.length===0?function(r){return r}:e.length===1?e[0]:e.reduce(function(r,i){return function(){return r(i.apply(void 0,arguments))}})}function O7(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return function(r){return function(){var i=r.apply(void 0,arguments),o=function(){throw new Error(St(15))},s={getState:i.getState,dispatch:function(){return o.apply(void 0,arguments)}},l=e.map(function(a){return a(s)});return o=C2.apply(void 0,l)(i.dispatch),l1(l1({},i),{},{dispatch:o})}}}const R2={black:"#000",white:"#fff"};var Wl=R2;const T2={50:"#ffebee",100:"#ffcdd2",200:"#ef9a9a",300:"#e57373",400:"#ef5350",500:"#f44336",600:"#e53935",700:"#d32f2f",800:"#c62828",900:"#b71c1c",A100:"#ff8a80",A200:"#ff5252",A400:"#ff1744",A700:"#d50000"};var qo=T2;const I2={50:"#fce4ec",100:"#f8bbd0",200:"#f48fb1",300:"#f06292",400:"#ec407a",500:"#e91e63",600:"#d81b60",700:"#c2185b",800:"#ad1457",900:"#880e4f",A100:"#ff80ab",A200:"#ff4081",A400:"#f50057",A700:"#c51162"};var v7=I2;const E2={50:"#f3e5f5",100:"#e1bee7",200:"#ce93d8",300:"#ba68c8",400:"#ab47bc",500:"#9c27b0",600:"#8e24aa",700:"#7b1fa2",800:"#6a1b9a",900:"#4a148c",A100:"#ea80fc",A200:"#e040fb",A400:"#d500f9",A700:"#aa00ff"};var zo=E2;const M2={50:"#e8eaf6",100:"#c5cae9",200:"#9fa8da",300:"#7986cb",400:"#5c6bc0",500:"#3f51b5",600:"#3949ab",700:"#303f9f",800:"#283593",900:"#1a237e",A100:"#8c9eff",A200:"#536dfe",A400:"#3d5afe",A700:"#304ffe"};var x7=M2;const A2={50:"#e3f2fd",100:"#bbdefb",200:"#90caf9",300:"#64b5f6",400:"#42a5f5",500:"#2196f3",600:"#1e88e5",700:"#1976d2",800:"#1565c0",900:"#0d47a1",A100:"#82b1ff",A200:"#448aff",A400:"#2979ff",A700:"#2962ff"};var Fo=A2;const D2={50:"#e1f5fe",100:"#b3e5fc",200:"#81d4fa",300:"#4fc3f7",400:"#29b6f6",500:"#03a9f4",600:"#039be5",700:"#0288d1",800:"#0277bd",900:"#01579b",A100:"#80d8ff",A200:"#40c4ff",A400:"#00b0ff",A700:"#0091ea"};var Vo=D2;const L2={50:"#e0f2f1",100:"#b2dfdb",200:"#80cbc4",300:"#4db6ac",400:"#26a69a",500:"#009688",600:"#00897b",700:"#00796b",800:"#00695c",900:"#004d40",A100:"#a7ffeb",A200:"#64ffda",A400:"#1de9b6",A700:"#00bfa5"};var w7=L2;const W2={50:"#e8f5e9",100:"#c8e6c9",200:"#a5d6a7",300:"#81c784",400:"#66bb6a",500:"#4caf50",600:"#43a047",700:"#388e3c",800:"#2e7d32",900:"#1b5e20",A100:"#b9f6ca",A200:"#69f0ae",A400:"#00e676",A700:"#00c853"};var Uo=W2;const Q2={50:"#f1f8e9",100:"#dcedc8",200:"#c5e1a5",300:"#aed581",400:"#9ccc65",500:"#8bc34a",600:"#7cb342",700:"#689f38",800:"#558b2f",900:"#33691e",A100:"#ccff90",A200:"#b2ff59",A400:"#76ff03",A700:"#64dd17"};var b7=Q2;const N2={50:"#fff3e0",100:"#ffe0b2",200:"#ffcc80",300:"#ffb74d",400:"#ffa726",500:"#ff9800",600:"#fb8c00",700:"#f57c00",800:"#ef6c00",900:"#e65100",A100:"#ffd180",A200:"#ffab40",A400:"#ff9100",A700:"#ff6d00"};var Ql=N2;const _2={50:"#fbe9e7",100:"#ffccbc",200:"#ffab91",300:"#ff8a65",400:"#ff7043",500:"#ff5722",600:"#f4511e",700:"#e64a19",800:"#d84315",900:"#bf360c",A100:"#ff9e80",A200:"#ff6e40",A400:"#ff3d00",A700:"#dd2c00"};var $7=_2;const B2={50:"#fafafa",100:"#f5f5f5",200:"#eeeeee",300:"#e0e0e0",400:"#bdbdbd",500:"#9e9e9e",600:"#757575",700:"#616161",800:"#424242",900:"#212121",A100:"#f5f5f5",A200:"#eeeeee",A400:"#bdbdbd",A700:"#616161"};var q2=B2;function c1(t){var e=Object.create(null);return function(n){return e[n]===void 0&&(e[n]=t(n)),e[n]}}var z2=/^((children|dangerouslySetInnerHTML|key|ref|autoFocus|defaultValue|defaultChecked|innerHTML|suppressContentEditableWarning|suppressHydrationWarning|valueLink|accept|acceptCharset|accessKey|action|allow|allowUserMedia|allowPaymentRequest|allowFullScreen|allowTransparency|alt|async|autoComplete|autoPlay|capture|cellPadding|cellSpacing|challenge|charSet|checked|cite|classID|className|cols|colSpan|content|contentEditable|contextMenu|controls|controlsList|coords|crossOrigin|data|dateTime|decoding|default|defer|dir|disabled|disablePictureInPicture|download|draggable|encType|enterKeyHint|form|formAction|formEncType|formMethod|formNoValidate|formTarget|frameBorder|headers|height|hidden|high|href|hrefLang|htmlFor|httpEquiv|id|inputMode|integrity|is|keyParams|keyType|kind|label|lang|list|loading|loop|low|marginHeight|marginWidth|max|maxLength|media|mediaGroup|method|min|minLength|multiple|muted|name|nonce|noValidate|open|optimum|pattern|placeholder|playsInline|poster|preload|profile|radioGroup|readOnly|referrerPolicy|rel|required|reversed|role|rows|rowSpan|sandbox|scope|scoped|scrolling|seamless|selected|shape|size|sizes|slot|span|spellCheck|src|srcDoc|srcLang|srcSet|start|step|style|summary|tabIndex|target|title|translate|type|useMap|value|width|wmode|wrap|about|datatype|inlist|prefix|property|resource|typeof|vocab|autoCapitalize|autoCorrect|autoSave|color|incremental|fallback|inert|itemProp|itemScope|itemType|itemID|itemRef|on|option|results|security|unselectable|accentHeight|accumulate|additive|alignmentBaseline|allowReorder|alphabetic|amplitude|arabicForm|ascent|attributeName|attributeType|autoReverse|azimuth|baseFrequency|baselineShift|baseProfile|bbox|begin|bias|by|calcMode|capHeight|clip|clipPathUnits|clipPath|clipRule|colorInterpolation|colorInterpolationFilters|colorProfile|colorRendering|contentScriptType|contentStyleType|cursor|cx|cy|d|decelerate|descent|diffuseConstant|direction|display|divisor|dominantBaseline|dur|dx|dy|edgeMode|elevation|enableBackground|end|exponent|externalResourcesRequired|fill|fillOpacity|fillRule|filter|filterRes|filterUnits|floodColor|floodOpacity|focusable|fontFamily|fontSize|fontSizeAdjust|fontStretch|fontStyle|fontVariant|fontWeight|format|from|fr|fx|fy|g1|g2|glyphName|glyphOrientationHorizontal|glyphOrientationVertical|glyphRef|gradientTransform|gradientUnits|hanging|horizAdvX|horizOriginX|ideographic|imageRendering|in|in2|intercept|k|k1|k2|k3|k4|kernelMatrix|kernelUnitLength|kerning|keyPoints|keySplines|keyTimes|lengthAdjust|letterSpacing|lightingColor|limitingConeAngle|local|markerEnd|markerMid|markerStart|markerHeight|markerUnits|markerWidth|mask|maskContentUnits|maskUnits|mathematical|mode|numOctaves|offset|opacity|operator|order|orient|orientation|origin|overflow|overlinePosition|overlineThickness|panose1|paintOrder|pathLength|patternContentUnits|patternTransform|patternUnits|pointerEvents|points|pointsAtX|pointsAtY|pointsAtZ|preserveAlpha|preserveAspectRatio|primitiveUnits|r|radius|refX|refY|renderingIntent|repeatCount|repeatDur|requiredExtensions|requiredFeatures|restart|result|rotate|rx|ry|scale|seed|shapeRendering|slope|spacing|specularConstant|specularExponent|speed|spreadMethod|startOffset|stdDeviation|stemh|stemv|stitchTiles|stopColor|stopOpacity|strikethroughPosition|strikethroughThickness|string|stroke|strokeDasharray|strokeDashoffset|strokeLinecap|strokeLinejoin|strokeMiterlimit|strokeOpacity|strokeWidth|surfaceScale|systemLanguage|tableValues|targetX|targetY|textAnchor|textDecoration|textRendering|textLength|to|transform|u1|u2|underlinePosition|underlineThickness|unicode|unicodeBidi|unicodeRange|unitsPerEm|vAlphabetic|vHanging|vIdeographic|vMathematical|values|vectorEffect|version|vertAdvY|vertOriginX|vertOriginY|viewBox|viewTarget|visibility|widths|wordSpacing|writingMode|x|xHeight|x1|x2|xChannelSelector|xlinkActuate|xlinkArcrole|xlinkHref|xlinkRole|xlinkShow|xlinkTitle|xlinkType|xmlBase|xmlns|xmlnsXlink|xmlLang|xmlSpace|y|y1|y2|yChannelSelector|z|zoomAndPan|for|class|autofocus)|(([Dd][Aa][Tt][Aa]|[Aa][Rr][Ii][Aa]|x)-.*))$/,F2=c1(function(t){return z2.test(t)||t.charCodeAt(0)===111&&t.charCodeAt(1)===110&&t.charCodeAt(2)<91});function V2(t){if(t.sheet)return t.sheet;for(var e=0;e<document.styleSheets.length;e++)if(document.styleSheets[e].ownerNode===t)return document.styleSheets[e]}function U2(t){var e=document.createElement("style");return e.setAttribute("data-emotion",t.key),t.nonce!==void 0&&e.setAttribute("nonce",t.nonce),e.appendChild(document.createTextNode("")),e.setAttribute("data-s",""),e}var u1=function(){function t(n){var r=this;this._insertTag=function(i){var o;r.tags.length===0?r.insertionPoint?o=r.insertionPoint.nextSibling:r.prepend?o=r.container.firstChild:o=r.before:o=r.tags[r.tags.length-1].nextSibling,r.container.insertBefore(i,o),r.tags.push(i)},this.isSpeedy=n.speedy===void 0?!0:n.speedy,this.tags=[],this.ctr=0,this.nonce=n.nonce,this.key=n.key,this.container=n.container,this.prepend=n.prepend,this.insertionPoint=n.insertionPoint,this.before=null}var e=t.prototype;return e.hydrate=function(r){r.forEach(this._insertTag)},e.insert=function(r){this.ctr%(this.isSpeedy?65e3:1)==0&&this._insertTag(U2(this));var i=this.tags[this.tags.length-1];if(this.isSpeedy){var o=V2(i);try{o.insertRule(r,o.cssRules.length)}catch{}}else i.appendChild(document.createTextNode(r));this.ctr++},e.flush=function(){this.tags.forEach(function(r){return r.parentNode&&r.parentNode.removeChild(r)}),this.tags=[],this.ctr=0},t}(),kt="-ms-",cu="-moz-",ke="-webkit-",f1="comm",Cp="rule",Rp="decl",j2="@import",h1="@keyframes",X2=Math.abs,uu=String.fromCharCode,H2=Object.assign;function G2(t,e){return(((e<<2^qt(t,0))<<2^qt(t,1))<<2^qt(t,2))<<2^qt(t,3)}function d1(t){return t.trim()}function Z2(t,e){return(t=e.exec(t))?t[0]:t}function Ce(t,e,n){return t.replace(e,n)}function Tp(t,e){return t.indexOf(e)}function qt(t,e){return t.charCodeAt(e)|0}function Nl(t,e,n){return t.slice(e,n)}function jn(t){return t.length}function Ip(t){return t.length}function fu(t,e){return e.push(t),t}function Y2(t,e){return t.map(e).join("")}var hu=1,jo=1,p1=0,zt=0,Je=0,Xo="";function du(t,e,n,r,i,o,s){return{value:t,root:e,parent:n,type:r,props:i,children:o,line:hu,column:jo,length:s,return:""}}function _l(t,e){return H2(du("",null,null,"",null,null,0),t,{length:-t.length},e)}function K2(){return Je}function J2(){return Je=zt>0?qt(Xo,--zt):0,jo--,Je===10&&(jo=1,hu--),Je}function tn(){return Je=zt<p1?qt(Xo,zt++):0,jo++,Je===10&&(jo=1,hu++),Je}function Xn(){return qt(Xo,zt)}function pu(){return zt}function Bl(t,e){return Nl(Xo,t,e)}function ql(t){switch(t){case 0:case 9:case 10:case 13:case 32:return 5;case 33:case 43:case 44:case 47:case 62:case 64:case 126:case 59:case 123:case 125:return 4;case 58:return 3;case 34:case 39:case 40:case 91:return 2;case 41:case 93:return 1}return 0}function m1(t){return hu=jo=1,p1=jn(Xo=t),zt=0,[]}function g1(t){return Xo="",t}function mu(t){return d1(Bl(zt-1,Ep(t===91?t+2:t===40?t+1:t)))}function eT(t){for(;(Je=Xn())&&Je<33;)tn();return ql(t)>2||ql(Je)>3?"":" "}function tT(t,e){for(;--e&&tn()&&!(Je<48||Je>102||Je>57&&Je<65||Je>70&&Je<97););return Bl(t,pu()+(e<6&&Xn()==32&&tn()==32))}function Ep(t){for(;tn();)switch(Je){case t:return zt;case 34:case 39:t!==34&&t!==39&&Ep(Je);break;case 40:t===41&&Ep(t);break;case 92:tn();break}return zt}function nT(t,e){for(;tn()&&t+Je!==47+10;)if(t+Je===42+42&&Xn()===47)break;return"/*"+Bl(e,zt-1)+"*"+uu(t===47?t:tn())}function rT(t){for(;!ql(Xn());)tn();return Bl(t,zt)}function iT(t){return g1(gu("",null,null,null,[""],t=m1(t),0,[0],t))}function gu(t,e,n,r,i,o,s,l,a){for(var c=0,u=0,f=s,h=0,d=0,p=0,m=1,g=1,y=1,O=0,v="",x=i,b=o,S=r,k=v;g;)switch(p=O,O=tn()){case 40:if(p!=108&&k.charCodeAt(f-1)==58){Tp(k+=Ce(mu(O),"&","&\f"),"&\f")!=-1&&(y=-1);break}case 34:case 39:case 91:k+=mu(O);break;case 9:case 10:case 13:case 32:k+=eT(p);break;case 92:k+=tT(pu()-1,7);continue;case 47:switch(Xn()){case 42:case 47:fu(oT(nT(tn(),pu()),e,n),a);break;default:k+="/"}break;case 123*m:l[c++]=jn(k)*y;case 125*m:case 59:case 0:switch(O){case 0:case 125:g=0;case 59+u:d>0&&jn(k)-f&&fu(d>32?O1(k+";",r,n,f-1):O1(Ce(k," ","")+";",r,n,f-2),a);break;case 59:k+=";";default:if(fu(S=y1(k,e,n,c,u,i,l,v,x=[],b=[],f),o),O===123)if(u===0)gu(k,e,S,S,x,o,f,l,b);else switch(h){case 100:case 109:case 115:gu(t,S,S,r&&fu(y1(t,S,S,0,0,i,l,v,i,x=[],f),b),i,b,f,l,r?x:b);break;default:gu(k,S,S,S,[""],b,0,l,b)}}c=u=d=0,m=y=1,v=k="",f=s;break;case 58:f=1+jn(k),d=p;default:if(m<1){if(O==123)--m;else if(O==125&&m++==0&&J2()==125)continue}switch(k+=uu(O),O*m){case 38:y=u>0?1:(k+="\f",-1);break;case 44:l[c++]=(jn(k)-1)*y,y=1;break;case 64:Xn()===45&&(k+=mu(tn())),h=Xn(),u=f=jn(v=k+=rT(pu())),O++;break;case 45:p===45&&jn(k)==2&&(m=0)}}return o}function y1(t,e,n,r,i,o,s,l,a,c,u){for(var f=i-1,h=i===0?o:[""],d=Ip(h),p=0,m=0,g=0;p<r;++p)for(var y=0,O=Nl(t,f+1,f=X2(m=s[p])),v=t;y<d;++y)(v=d1(m>0?h[y]+" "+O:Ce(O,/&\f/g,h[y])))&&(a[g++]=v);return du(t,e,n,i===0?Cp:l,a,c,u)}function oT(t,e,n){return du(t,e,n,f1,uu(K2()),Nl(t,2,-2),0)}function O1(t,e,n,r){return du(t,e,n,Rp,Nl(t,0,r),Nl(t,r+1,-1),r)}function v1(t,e){switch(G2(t,e)){case 5103:return ke+"print-"+t+t;case 5737:case 4201:case 3177:case 3433:case 1641:case 4457:case 2921:case 5572:case 6356:case 5844:case 3191:case 6645:case 3005:case 6391:case 5879:case 5623:case 6135:case 4599:case 4855:case 4215:case 6389:case 5109:case 5365:case 5621:case 3829:return ke+t+t;case 5349:case 4246:case 4810:case 6968:case 2756:return ke+t+cu+t+kt+t+t;case 6828:case 4268:return ke+t+kt+t+t;case 6165:return ke+t+kt+"flex-"+t+t;case 5187:return ke+t+Ce(t,/(\w+).+(:[^]+)/,ke+"box-$1$2"+kt+"flex-$1$2")+t;case 5443:return ke+t+kt+"flex-item-"+Ce(t,/flex-|-self/,"")+t;case 4675:return ke+t+kt+"flex-line-pack"+Ce(t,/align-content|flex-|-self/,"")+t;case 5548:return ke+t+kt+Ce(t,"shrink","negative")+t;case 5292:return ke+t+kt+Ce(t,"basis","preferred-size")+t;case 6060:return ke+"box-"+Ce(t,"-grow","")+ke+t+kt+Ce(t,"grow","positive")+t;case 4554:return ke+Ce(t,/([^-])(transform)/g,"$1"+ke+"$2")+t;case 6187:return Ce(Ce(Ce(t,/(zoom-|grab)/,ke+"$1"),/(image-set)/,ke+"$1"),t,"")+t;case 5495:case 3959:return Ce(t,/(image-set\([^]*)/,ke+"$1$`$1");case 4968:return Ce(Ce(t,/(.+:)(flex-)?(.*)/,ke+"box-pack:$3"+kt+"flex-pack:$3"),/s.+-b[^;]+/,"justify")+ke+t+t;case 4095:case 3583:case 4068:case 2532:return Ce(t,/(.+)-inline(.+)/,ke+"$1$2")+t;case 8116:case 7059:case 5753:case 5535:case 5445:case 5701:case 4933:case 4677:case 5533:case 5789:case 5021:case 4765:if(jn(t)-1-e>6)switch(qt(t,e+1)){case 109:if(qt(t,e+4)!==45)break;case 102:return Ce(t,/(.+:)(.+)-([^]+)/,"$1"+ke+"$2-$3$1"+cu+(qt(t,e+3)==108?"$3":"$2-$3"))+t;case 115:return~Tp(t,"stretch")?v1(Ce(t,"stretch","fill-available"),e)+t:t}break;case 4949:if(qt(t,e+1)!==115)break;case 6444:switch(qt(t,jn(t)-3-(~Tp(t,"!important")&&10))){case 107:return Ce(t,":",":"+ke)+t;case 101:return Ce(t,/(.+:)([^;!]+)(;|!.+)?/,"$1"+ke+(qt(t,14)===45?"inline-":"")+"box$3$1"+ke+"$2$3$1"+kt+"$2box$3")+t}break;case 5936:switch(qt(t,e+11)){case 114:return ke+t+kt+Ce(t,/[svh]\w+-[tblr]{2}/,"tb")+t;case 108:return ke+t+kt+Ce(t,/[svh]\w+-[tblr]{2}/,"tb-rl")+t;case 45:return ke+t+kt+Ce(t,/[svh]\w+-[tblr]{2}/,"lr")+t}return ke+t+kt+t+t}return t}function Ho(t,e){for(var n="",r=Ip(t),i=0;i<r;i++)n+=e(t[i],i,t,e)||"";return n}function sT(t,e,n,r){switch(t.type){case j2:case Rp:return t.return=t.return||t.value;case f1:return"";case h1:return t.return=t.value+"{"+Ho(t.children,r)+"}";case Cp:t.value=t.props.join(",")}return jn(n=Ho(t.children,r))?t.return=t.value+"{"+n+"}":""}function lT(t){var e=Ip(t);return function(n,r,i,o){for(var s="",l=0;l<e;l++)s+=t[l](n,r,i,o)||"";return s}}function aT(t){return function(e){e.root||(e=e.return)&&t(e)}}function cT(t,e,n,r){if(t.length>-1&&!t.return)switch(t.type){case Rp:t.return=v1(t.value,t.length);break;case h1:return Ho([_l(t,{value:Ce(t.value,"@","@"+ke)})],r);case Cp:if(t.length)return Y2(t.props,function(i){switch(Z2(i,/(::plac\w+|:read-\w+)/)){case":read-only":case":read-write":return Ho([_l(t,{props:[Ce(i,/:(read-\w+)/,":"+cu+"$1")]})],r);case"::placeholder":return Ho([_l(t,{props:[Ce(i,/:(plac\w+)/,":"+ke+"input-$1")]}),_l(t,{props:[Ce(i,/:(plac\w+)/,":"+cu+"$1")]}),_l(t,{props:[Ce(i,/:(plac\w+)/,kt+"input-$1")]})],r)}return""})}}var uT=function(e,n,r){for(var i=0,o=0;i=o,o=Xn(),i===38&&o===12&&(n[r]=1),!ql(o);)tn();return Bl(e,zt)},fT=function(e,n){var r=-1,i=44;do switch(ql(i)){case 0:i===38&&Xn()===12&&(n[r]=1),e[r]+=uT(zt-1,n,r);break;case 2:e[r]+=mu(i);break;case 4:if(i===44){e[++r]=Xn()===58?"&\f":"",n[r]=e[r].length;break}default:e[r]+=uu(i)}while(i=tn());return e},hT=function(e,n){return g1(fT(m1(e),n))},x1=new WeakMap,dT=function(e){if(!(e.type!=="rule"||!e.parent||e.length<1)){for(var n=e.value,r=e.parent,i=e.column===r.column&&e.line===r.line;r.type!=="rule";)if(r=r.parent,!r)return;if(!(e.props.length===1&&n.charCodeAt(0)!==58&&!x1.get(r))&&!i){x1.set(e,!0);for(var o=[],s=hT(n,o),l=r.props,a=0,c=0;a<s.length;a++)for(var u=0;u<l.length;u++,c++)e.props[c]=o[a]?s[a].replace(/&\f/g,l[u]):l[u]+" "+s[a]}}},pT=function(e){if(e.type==="decl"){var n=e.value;n.charCodeAt(0)===108&&n.charCodeAt(2)===98&&(e.return="",e.value="")}},mT=[cT],gT=function(e){var n=e.key;if(n==="css"){var r=document.querySelectorAll("style[data-emotion]:not([data-s])");Array.prototype.forEach.call(r,function(m){var g=m.getAttribute("data-emotion");g.indexOf(" ")!==-1&&(document.head.appendChild(m),m.setAttribute("data-s",""))})}var i=e.stylisPlugins||mT,o={},s,l=[];s=e.container||document.head,Array.prototype.forEach.call(document.querySelectorAll('style[data-emotion^="'+n+' "]'),function(m){for(var g=m.getAttribute("data-emotion").split(" "),y=1;y<g.length;y++)o[g[y]]=!0;l.push(m)});var a,c=[dT,pT];{var u,f=[sT,aT(function(m){u.insert(m)})],h=lT(c.concat(i,f)),d=function(g){return Ho(iT(g),h)};a=function(g,y,O,v){u=O,d(g?g+"{"+y.styles+"}":y.styles),v&&(p.inserted[y.name]=!0)}}var p={key:n,sheet:new u1({key:n,container:s,nonce:e.nonce,speedy:e.speedy,prepend:e.prepend,insertionPoint:e.insertionPoint}),nonce:e.nonce,inserted:o,registered:{},insert:a};return p.sheet.hydrate(l),p},w1=gT,yT=!0;function OT(t,e,n){var r="";return n.split(" ").forEach(function(i){t[i]!==void 0?e.push(t[i]+";"):r+=i+" "}),r}var b1=function(e,n,r){var i=e.key+"-"+n.name;if((r===!1||yT===!1)&&e.registered[i]===void 0&&(e.registered[i]=n.styles),e.inserted[n.name]===void 0){var o=n;do e.insert(n===o?"."+i:"",o,e.sheet,!0),o=o.next;while(o!==void 0)}};function vT(t){for(var e=0,n,r=0,i=t.length;i>=4;++r,i-=4)n=t.charCodeAt(r)&255|(t.charCodeAt(++r)&255)<<8|(t.charCodeAt(++r)&255)<<16|(t.charCodeAt(++r)&255)<<24,n=(n&65535)*1540483477+((n>>>16)*59797<<16),n^=n>>>24,e=(n&65535)*1540483477+((n>>>16)*59797<<16)^(e&65535)*1540483477+((e>>>16)*59797<<16);switch(i){case 3:e^=(t.charCodeAt(r+2)&255)<<16;case 2:e^=(t.charCodeAt(r+1)&255)<<8;case 1:e^=t.charCodeAt(r)&255,e=(e&65535)*1540483477+((e>>>16)*59797<<16)}return e^=e>>>13,e=(e&65535)*1540483477+((e>>>16)*59797<<16),((e^e>>>15)>>>0).toString(36)}var xT={animationIterationCount:1,borderImageOutset:1,borderImageSlice:1,borderImageWidth:1,boxFlex:1,boxFlexGroup:1,boxOrdinalGroup:1,columnCount:1,columns:1,flex:1,flexGrow:1,flexPositive:1,flexShrink:1,flexNegative:1,flexOrder:1,gridRow:1,gridRowEnd:1,gridRowSpan:1,gridRowStart:1,gridColumn:1,gridColumnEnd:1,gridColumnSpan:1,gridColumnStart:1,msGridRow:1,msGridRowSpan:1,msGridColumn:1,msGridColumnSpan:1,fontWeight:1,lineHeight:1,opacity:1,order:1,orphans:1,tabSize:1,widows:1,zIndex:1,zoom:1,WebkitLineClamp:1,fillOpacity:1,floodOpacity:1,stopOpacity:1,strokeDasharray:1,strokeDashoffset:1,strokeMiterlimit:1,strokeOpacity:1,strokeWidth:1},wT=/[A-Z]|^ms/g,bT=/_EMO_([^_]+?)_([^]*?)_EMO_/g,$1=function(e){return e.charCodeAt(1)===45},S1=function(e){return e!=null&&typeof e!="boolean"},Mp=c1(function(t){return $1(t)?t:t.replace(wT,"-$&").toLowerCase()}),k1=function(e,n){switch(e){case"animation":case"animationName":if(typeof n=="string")return n.replace(bT,function(r,i,o){return Hn={name:i,styles:o,next:Hn},i})}return xT[e]!==1&&!$1(e)&&typeof n=="number"&&n!==0?n+"px":n};function zl(t,e,n){if(n==null)return"";if(n.__emotion_styles!==void 0)return n;switch(typeof n){case"boolean":return"";case"object":{if(n.anim===1)return Hn={name:n.name,styles:n.styles,next:Hn},n.name;if(n.styles!==void 0){var r=n.next;if(r!==void 0)for(;r!==void 0;)Hn={name:r.name,styles:r.styles,next:Hn},r=r.next;var i=n.styles+";";return i}return $T(t,e,n)}case"function":{if(t!==void 0){var o=Hn,s=n(t);return Hn=o,zl(t,e,s)}break}}if(e==null)return n;var l=e[n];return l!==void 0?l:n}function $T(t,e,n){var r="";if(Array.isArray(n))for(var i=0;i<n.length;i++)r+=zl(t,e,n[i])+";";else for(var o in n){var s=n[o];if(typeof s!="object")e!=null&&e[s]!==void 0?r+=o+"{"+e[s]+"}":S1(s)&&(r+=Mp(o)+":"+k1(o,s)+";");else if(Array.isArray(s)&&typeof s[0]=="string"&&(e==null||e[s[0]]===void 0))for(var l=0;l<s.length;l++)S1(s[l])&&(r+=Mp(o)+":"+k1(o,s[l])+";");else{var a=zl(t,e,s);switch(o){case"animation":case"animationName":{r+=Mp(o)+":"+a+";";break}default:r+=o+"{"+a+"}"}}}return r}var P1=/label:\s*([^\s;\n{]+)\s*(;|$)/g,Hn,Ap=function(e,n,r){if(e.length===1&&typeof e[0]=="object"&&e[0]!==null&&e[0].styles!==void 0)return e[0];var i=!0,o="";Hn=void 0;var s=e[0];s==null||s.raw===void 0?(i=!1,o+=zl(r,n,s)):o+=s[0];for(var l=1;l<e.length;l++)o+=zl(r,n,e[l]),i&&(o+=s[l]);P1.lastIndex=0;for(var a="",c;(c=P1.exec(o))!==null;)a+="-"+c[1];var u=vT(o)+a;return{name:u,styles:o,next:Hn}},C1=w.exports.createContext(typeof HTMLElement!="undefined"?w1({key:"css"}):null),ST=C1.Provider,R1=function(e){return w.exports.forwardRef(function(n,r){var i=w.exports.useContext(C1);return e(n,i,r)})},Dp=w.exports.createContext({}),kT=R1(function(t,e){var n=t.styles,r=Ap([n],void 0,w.exports.useContext(Dp)),i=w.exports.useRef();return w.exports.useLayoutEffect(function(){var o=e.key+"-global",s=new u1({key:o,nonce:e.sheet.nonce,container:e.sheet.container,speedy:e.sheet.isSpeedy}),l=!1,a=document.querySelector('style[data-emotion="'+o+" "+r.name+'"]');return e.sheet.tags.length&&(s.before=e.sheet.tags[0]),a!==null&&(l=!0,a.setAttribute("data-emotion",o),s.hydrate([a])),i.current=[s,l],function(){s.flush()}},[e]),w.exports.useLayoutEffect(function(){var o=i.current,s=o[0],l=o[1];if(l){o[1]=!1;return}if(r.next!==void 0&&b1(e,r.next,!0),s.tags.length){var a=s.tags[s.tags.length-1].nextElementSibling;s.before=a,s.flush()}e.insert("",r,s,!1)},[e,r.name]),null});function Lp(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return Ap(e)}var Fl=function(){var e=Lp.apply(void 0,arguments),n="animation-"+e.name;return{name:n,styles:"@keyframes "+n+"{"+e.styles+"}",anim:1,toString:function(){return"_EMO_"+this.name+"_"+this.styles+"_EMO_"}}},PT=F2,CT=function(e){return e!=="theme"},T1=function(e){return typeof e=="string"&&e.charCodeAt(0)>96?PT:CT},I1=function(e,n,r){var i;if(n){var o=n.shouldForwardProp;i=e.__emotion_forwardProp&&o?function(s){return e.__emotion_forwardProp(s)&&o(s)}:o}return typeof i!="function"&&r&&(i=e.__emotion_forwardProp),i},RT=function(){return null},TT=function t(e,n){var r=e.__emotion_real===e,i=r&&e.__emotion_base||e,o,s;n!==void 0&&(o=n.label,s=n.target);var l=I1(e,n,r),a=l||T1(i),c=!a("as");return function(){var u=arguments,f=r&&e.__emotion_styles!==void 0?e.__emotion_styles.slice(0):[];if(o!==void 0&&f.push("label:"+o+";"),u[0]==null||u[0].raw===void 0)f.push.apply(f,u);else{f.push(u[0][0]);for(var h=u.length,d=1;d<h;d++)f.push(u[d],u[0][d])}var p=R1(function(m,g,y){var O=c&&m.as||i,v="",x=[],b=m;if(m.theme==null){b={};for(var S in m)b[S]=m[S];b.theme=w.exports.useContext(Dp)}typeof m.className=="string"?v=OT(g.registered,x,m.className):m.className!=null&&(v=m.className+" ");var k=Ap(f.concat(x),g.registered,b);b1(g,k,typeof O=="string"),v+=g.key+"-"+k.name,s!==void 0&&(v+=" "+s);var P=c&&l===void 0?T1(O):a,C={};for(var R in m)c&&R==="as"||P(R)&&(C[R]=m[R]);C.className=v,C.ref=y;var T=w.exports.createElement(O,C),L=w.exports.createElement(RT,null);return w.exports.createElement(w.exports.Fragment,null,L,T)});return p.displayName=o!==void 0?o:"Styled("+(typeof i=="string"?i:i.displayName||i.name||"Component")+")",p.defaultProps=e.defaultProps,p.__emotion_real=p,p.__emotion_base=i,p.__emotion_styles=f,p.__emotion_forwardProp=l,Object.defineProperty(p,"toString",{value:function(){return"."+s}}),p.withComponent=function(m,g){return t(m,$({},n,g,{shouldForwardProp:I1(p,g,!0)})).apply(void 0,f)},p}},IT=["a","abbr","address","area","article","aside","audio","b","base","bdi","bdo","big","blockquote","body","br","button","canvas","caption","cite","code","col","colgroup","data","datalist","dd","del","details","dfn","dialog","div","dl","dt","em","embed","fieldset","figcaption","figure","footer","form","h1","h2","h3","h4","h5","h6","head","header","hgroup","hr","html","i","iframe","img","input","ins","kbd","keygen","label","legend","li","link","main","map","mark","marquee","menu","menuitem","meta","meter","nav","noscript","object","ol","optgroup","option","output","p","param","picture","pre","progress","q","rp","rt","ruby","s","samp","script","section","select","small","source","span","strong","style","sub","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","title","tr","track","u","ul","var","video","wbr","circle","clipPath","defs","ellipse","foreignObject","g","image","line","linearGradient","mask","path","pattern","polygon","polyline","radialGradient","rect","stop","svg","text","tspan"],Wp=TT.bind();IT.forEach(function(t){Wp[t]=Wp(t)});var ET=Wp;const MT=w1({key:"css",prepend:!0});function S7(t){const{injectFirst:e,children:n}=t;return e?I(ST,{value:MT,children:n}):n}function AT(t){return t==null||Object.keys(t).length===0}function DT(t){const{styles:e,defaultTheme:n={}}=t;return I(kT,{styles:typeof e=="function"?i=>e(AT(i)?n:i):e})}/** @license MUI v5.2.6
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */function E1(t,e){return ET(t,e)}function Vl(t){return t!==null&&typeof t=="object"&&t.constructor===Object}function vr(t,e,n={clone:!0}){const r=n.clone?$({},t):t;return Vl(t)&&Vl(e)&&Object.keys(e).forEach(i=>{i!=="__proto__"&&(Vl(e[i])&&i in t&&Vl(t[i])?r[i]=vr(t[i],e[i],n):r[i]=e[i])}),r}function Ul(t){let e="https://mui.com/production-error/?code="+t;for(let n=1;n<arguments.length;n+=1)e+="&args[]="+encodeURIComponent(arguments[n]);return"Minified MUI error #"+t+"; visit "+e+" for the full message."}function V(t){if(typeof t!="string")throw new Error(Ul(7));return t.charAt(0).toUpperCase()+t.slice(1)}function M1(...t){return t.reduce((e,n)=>n==null?e:function(...i){e.apply(this,i),n.apply(this,i)},()=>{})}function jl(t,e=166){let n;function r(...i){const o=()=>{t.apply(this,i)};clearTimeout(n),n=setTimeout(o,e)}return r.clear=()=>{clearTimeout(n)},r}function LT(t,e){return w.exports.isValidElement(t)&&e.indexOf(t.type.muiName)!==-1}function Pt(t){return t&&t.ownerDocument||document}function xr(t){return Pt(t).defaultView||window}function yu(t,e){typeof t=="function"?t(e):t&&(t.current=e)}const WT=typeof window!="undefined"?w.exports.useLayoutEffect:w.exports.useEffect;var Gn=WT;let A1=0;function QT(t){const[e,n]=w.exports.useState(t),r=t||e;return w.exports.useEffect(()=>{e==null&&(A1+=1,n(`mui-${A1}`))},[e]),r}function D1({controlled:t,default:e,name:n,state:r="value"}){const{current:i}=w.exports.useRef(t!==void 0),[o,s]=w.exports.useState(e),l=i?t:o,a=w.exports.useCallback(c=>{i||s(c)},[]);return[l,a]}function Dn(t){const e=w.exports.useRef(t);return Gn(()=>{e.current=t}),w.exports.useCallback((...n)=>(0,e.current)(...n),[])}function Qe(t,e){return w.exports.useMemo(()=>t==null&&e==null?null:n=>{yu(t,n),yu(e,n)},[t,e])}let Ou=!0,Qp=!1,L1;const NT={text:!0,search:!0,url:!0,tel:!0,email:!0,password:!0,number:!0,date:!0,month:!0,week:!0,time:!0,datetime:!0,"datetime-local":!0};function _T(t){const{type:e,tagName:n}=t;return!!(n==="INPUT"&&NT[e]&&!t.readOnly||n==="TEXTAREA"&&!t.readOnly||t.isContentEditable)}function BT(t){t.metaKey||t.altKey||t.ctrlKey||(Ou=!0)}function Np(){Ou=!1}function qT(){this.visibilityState==="hidden"&&Qp&&(Ou=!0)}function zT(t){t.addEventListener("keydown",BT,!0),t.addEventListener("mousedown",Np,!0),t.addEventListener("pointerdown",Np,!0),t.addEventListener("touchstart",Np,!0),t.addEventListener("visibilitychange",qT,!0)}function FT(t){const{target:e}=t;try{return e.matches(":focus-visible")}catch{}return Ou||_T(e)}function _p(){const t=w.exports.useCallback(i=>{i!=null&&zT(i.ownerDocument)},[]),e=w.exports.useRef(!1);function n(){return e.current?(Qp=!0,window.clearTimeout(L1),L1=window.setTimeout(()=>{Qp=!1},100),e.current=!1,!0):!1}function r(i){return FT(i)?(e.current=!0,!0):!1}return{isFocusVisibleRef:e,onFocus:r,onBlur:n,ref:t}}function W1(t){const e=t.documentElement.clientWidth;return Math.abs(window.innerWidth-e)}let Go;function Q1(){if(Go)return Go;const t=document.createElement("div"),e=document.createElement("div");return e.style.width="10px",e.style.height="1px",t.appendChild(e),t.dir="rtl",t.style.fontSize="14px",t.style.width="4px",t.style.height="1px",t.style.position="absolute",t.style.top="-1000px",t.style.overflow="scroll",document.body.appendChild(t),Go="reverse",t.scrollLeft>0?Go="default":(t.scrollLeft=1,t.scrollLeft===0&&(Go="negative")),document.body.removeChild(t),Go}function N1(t,e){const n=t.scrollLeft;if(e!=="rtl")return n;switch(Q1()){case"negative":return t.scrollWidth-t.clientWidth+n;case"reverse":return t.scrollWidth-t.clientWidth-n;default:return n}}function _1(t,e){const n=$({},e);return Object.keys(t).forEach(r=>{n[r]===void 0&&(n[r]=t[r])}),n}function Xl(t,e){return e?vr(t,e,{clone:!1}):t}const Bp={xs:0,sm:600,md:900,lg:1200,xl:1536},B1={keys:["xs","sm","md","lg","xl"],up:t=>`@media (min-width:${Bp[t]}px)`};function Ln(t,e,n){const r=t.theme||{};if(Array.isArray(e)){const o=r.breakpoints||B1;return e.reduce((s,l,a)=>(s[o.up(o.keys[a])]=n(e[a]),s),{})}if(typeof e=="object"){const o=r.breakpoints||B1;return Object.keys(e).reduce((s,l)=>{if(Object.keys(o.values||Bp).indexOf(l)!==-1){const a=o.up(l);s[a]=n(e[l],l)}else{const a=l;s[a]=e[a]}return s},{})}return n(e)}function VT(t={}){var e;return(t==null||(e=t.keys)==null?void 0:e.reduce((r,i)=>{const o=t.up(i);return r[o]={},r},{}))||{}}function UT(t,e){return t.reduce((n,r)=>{const i=n[r];return(!i||Object.keys(i).length===0)&&delete n[r],n},e)}function jT(t,e){if(typeof t!="object")return{};const n={},r=Object.keys(e);return Array.isArray(t)?r.forEach((i,o)=>{o<t.length&&(n[i]=!0)}):r.forEach(i=>{t[i]!=null&&(n[i]=!0)}),n}function vu({values:t,breakpoints:e,base:n}){const r=n||jT(t,e),i=Object.keys(r);if(i.length===0)return t;let o;return i.reduce((s,l,a)=>(Array.isArray(t)?(s[l]=t[a]!=null?t[a]:t[o],o=a):(s[l]=t[l]!=null?t[l]:t[o]||t,o=l),s),{})}function xu(t,e){return!e||typeof e!="string"?null:e.split(".").reduce((n,r)=>n&&n[r]?n[r]:null,t)}function q1(t,e,n,r=n){let i;return typeof t=="function"?i=t(n):Array.isArray(t)?i=t[n]||r:i=xu(t,n)||r,e&&(i=e(i)),i}function K(t){const{prop:e,cssProperty:n=t.prop,themeKey:r,transform:i}=t,o=s=>{if(s[e]==null)return null;const l=s[e],a=s.theme,c=xu(a,r)||{};return Ln(s,l,f=>{let h=q1(c,i,f);return f===h&&typeof f=="string"&&(h=q1(c,i,`${e}${f==="default"?"":V(f)}`,f)),n===!1?h:{[n]:h}})};return o.propTypes={},o.filterProps=[e],o}function si(...t){const e=t.reduce((r,i)=>(i.filterProps.forEach(o=>{r[o]=i}),r),{}),n=r=>Object.keys(r).reduce((i,o)=>e[o]?Xl(i,e[o](r)):i,{});return n.propTypes={},n.filterProps=t.reduce((r,i)=>r.concat(i.filterProps),[]),n}function XT(t){const e={};return n=>(e[n]===void 0&&(e[n]=t(n)),e[n])}const HT={m:"margin",p:"padding"},GT={t:"Top",r:"Right",b:"Bottom",l:"Left",x:["Left","Right"],y:["Top","Bottom"]},z1={marginX:"mx",marginY:"my",paddingX:"px",paddingY:"py"},ZT=XT(t=>{if(t.length>2)if(z1[t])t=z1[t];else return[t];const[e,n]=t.split(""),r=HT[e],i=GT[n]||"";return Array.isArray(i)?i.map(o=>r+o):[r+i]}),YT=["m","mt","mr","mb","ml","mx","my","margin","marginTop","marginRight","marginBottom","marginLeft","marginX","marginY","marginInline","marginInlineStart","marginInlineEnd","marginBlock","marginBlockStart","marginBlockEnd"],KT=["p","pt","pr","pb","pl","px","py","padding","paddingTop","paddingRight","paddingBottom","paddingLeft","paddingX","paddingY","paddingInline","paddingInlineStart","paddingInlineEnd","paddingBlock","paddingBlockStart","paddingBlockEnd"],F1=[...YT,...KT];function Hl(t,e,n,r){const i=xu(t,e)||n;return typeof i=="number"?o=>typeof o=="string"?o:i*o:Array.isArray(i)?o=>typeof o=="string"?o:i[o]:typeof i=="function"?i:()=>{}}function V1(t){return Hl(t,"spacing",8)}function Gl(t,e){if(typeof e=="string"||e==null)return e;const n=Math.abs(e),r=t(n);return e>=0?r:typeof r=="number"?-r:`-${r}`}function JT(t,e){return n=>t.reduce((r,i)=>(r[i]=Gl(e,n),r),{})}function eI(t,e,n,r){if(e.indexOf(n)===-1)return null;const i=ZT(n),o=JT(i,r),s=t[n];return Ln(t,s,o)}function tI(t,e){const n=V1(t.theme);return Object.keys(t).map(r=>eI(t,e,r,n)).reduce(Xl,{})}function wu(t){return tI(t,F1)}wu.propTypes={};wu.filterProps=F1;function Zl(t){return typeof t!="number"?t:`${t}px solid`}const nI=K({prop:"border",themeKey:"borders",transform:Zl}),rI=K({prop:"borderTop",themeKey:"borders",transform:Zl}),iI=K({prop:"borderRight",themeKey:"borders",transform:Zl}),oI=K({prop:"borderBottom",themeKey:"borders",transform:Zl}),sI=K({prop:"borderLeft",themeKey:"borders",transform:Zl}),lI=K({prop:"borderColor",themeKey:"palette"}),aI=K({prop:"borderTopColor",themeKey:"palette"}),cI=K({prop:"borderRightColor",themeKey:"palette"}),uI=K({prop:"borderBottomColor",themeKey:"palette"}),fI=K({prop:"borderLeftColor",themeKey:"palette"}),qp=t=>{if(t.borderRadius!==void 0&&t.borderRadius!==null){const e=Hl(t.theme,"shape.borderRadius",4),n=r=>({borderRadius:Gl(e,r)});return Ln(t,t.borderRadius,n)}return null};qp.propTypes={};qp.filterProps=["borderRadius"];const hI=si(nI,rI,iI,oI,sI,lI,aI,cI,uI,fI,qp);var U1=hI;const dI=K({prop:"displayPrint",cssProperty:!1,transform:t=>({"@media print":{display:t}})}),pI=K({prop:"display"}),mI=K({prop:"overflow"}),gI=K({prop:"textOverflow"}),yI=K({prop:"visibility"}),OI=K({prop:"whiteSpace"});var j1=si(dI,pI,mI,gI,yI,OI);const vI=K({prop:"flexBasis"}),xI=K({prop:"flexDirection"}),wI=K({prop:"flexWrap"}),bI=K({prop:"justifyContent"}),$I=K({prop:"alignItems"}),SI=K({prop:"alignContent"}),kI=K({prop:"order"}),PI=K({prop:"flex"}),CI=K({prop:"flexGrow"}),RI=K({prop:"flexShrink"}),TI=K({prop:"alignSelf"}),II=K({prop:"justifyItems"}),EI=K({prop:"justifySelf"}),MI=si(vI,xI,wI,bI,$I,SI,kI,PI,CI,RI,TI,II,EI);var X1=MI;const zp=t=>{if(t.gap!==void 0&&t.gap!==null){const e=Hl(t.theme,"spacing",8),n=r=>({gap:Gl(e,r)});return Ln(t,t.gap,n)}return null};zp.propTypes={};zp.filterProps=["gap"];const Fp=t=>{if(t.columnGap!==void 0&&t.columnGap!==null){const e=Hl(t.theme,"spacing",8),n=r=>({columnGap:Gl(e,r)});return Ln(t,t.columnGap,n)}return null};Fp.propTypes={};Fp.filterProps=["columnGap"];const Vp=t=>{if(t.rowGap!==void 0&&t.rowGap!==null){const e=Hl(t.theme,"spacing",8),n=r=>({rowGap:Gl(e,r)});return Ln(t,t.rowGap,n)}return null};Vp.propTypes={};Vp.filterProps=["rowGap"];const AI=K({prop:"gridColumn"}),DI=K({prop:"gridRow"}),LI=K({prop:"gridAutoFlow"}),WI=K({prop:"gridAutoColumns"}),QI=K({prop:"gridAutoRows"}),NI=K({prop:"gridTemplateColumns"}),_I=K({prop:"gridTemplateRows"}),BI=K({prop:"gridTemplateAreas"}),qI=K({prop:"gridArea"}),zI=si(zp,Fp,Vp,AI,DI,LI,WI,QI,NI,_I,BI,qI);var H1=zI;const FI=K({prop:"color",themeKey:"palette"}),VI=K({prop:"bgcolor",cssProperty:"backgroundColor",themeKey:"palette"}),UI=K({prop:"backgroundColor",themeKey:"palette"}),jI=si(FI,VI,UI);var G1=jI;const XI=K({prop:"position"}),HI=K({prop:"zIndex",themeKey:"zIndex"}),GI=K({prop:"top"}),ZI=K({prop:"right"}),YI=K({prop:"bottom"}),KI=K({prop:"left"});var Z1=si(XI,HI,GI,ZI,YI,KI);const JI=K({prop:"boxShadow",themeKey:"shadows"});var Y1=JI;function li(t){return t<=1&&t!==0?`${t*100}%`:t}const eE=K({prop:"width",transform:li}),K1=t=>{if(t.maxWidth!==void 0&&t.maxWidth!==null){const e=n=>{var r,i,o;return{maxWidth:((r=t.theme)==null||(i=r.breakpoints)==null||(o=i.values)==null?void 0:o[n])||Bp[n]||li(n)}};return Ln(t,t.maxWidth,e)}return null};K1.filterProps=["maxWidth"];const tE=K({prop:"minWidth",transform:li}),nE=K({prop:"height",transform:li}),rE=K({prop:"maxHeight",transform:li}),iE=K({prop:"minHeight",transform:li});K({prop:"size",cssProperty:"width",transform:li});K({prop:"size",cssProperty:"height",transform:li});const oE=K({prop:"boxSizing"}),sE=si(eE,K1,tE,nE,rE,iE,oE);var J1=sE;const lE=K({prop:"fontFamily",themeKey:"typography"}),aE=K({prop:"fontSize",themeKey:"typography"}),cE=K({prop:"fontStyle",themeKey:"typography"}),uE=K({prop:"fontWeight",themeKey:"typography"}),fE=K({prop:"letterSpacing"}),hE=K({prop:"lineHeight"}),dE=K({prop:"textAlign"}),pE=K({prop:"typography",cssProperty:!1,themeKey:"typography"}),mE=si(pE,lE,aE,cE,uE,fE,hE,dE);var ex=mE;const tx={borders:U1.filterProps,display:j1.filterProps,flexbox:X1.filterProps,grid:H1.filterProps,positions:Z1.filterProps,palette:G1.filterProps,shadows:Y1.filterProps,sizing:J1.filterProps,spacing:wu.filterProps,typography:ex.filterProps},gE={borders:U1,display:j1,flexbox:X1,grid:H1,positions:Z1,palette:G1,shadows:Y1,sizing:J1,spacing:wu,typography:ex},Up=Object.keys(tx).reduce((t,e)=>(tx[e].forEach(n=>{t[n]=gE[e]}),t),{});function nx(t,e,n){const r={[t]:e,theme:n},i=Up[t];return i?i(r):{[t]:e}}function yE(...t){const e=t.reduce((r,i)=>r.concat(Object.keys(i)),[]),n=new Set(e);return t.every(r=>n.size===Object.keys(r).length)}function OE(t,e){return typeof t=="function"?t(e):t}function bu(t){const{sx:e,theme:n={}}=t||{};if(!e)return null;function r(i){let o=i;if(typeof i=="function")o=i(n);else if(typeof i!="object")return i;const s=VT(n.breakpoints),l=Object.keys(s);let a=s;return Object.keys(o).forEach(c=>{const u=OE(o[c],n);if(u!=null)if(typeof u=="object")if(Up[c])a=Xl(a,nx(c,u,n));else{const f=Ln({theme:n},u,h=>({[c]:h}));yE(f,u)?a[c]=bu({sx:u,theme:n}):a=Xl(a,f)}else a=Xl(a,nx(c,u,n))}),UT(l,a)}return Array.isArray(e)?e.map(r):r(e)}bu.filterProps=["sx"];const vE=["sx"],xE=t=>{const e={systemProps:{},otherProps:{}};return Object.keys(t).forEach(n=>{Up[n]?e.systemProps[n]=t[n]:e.otherProps[n]=t[n]}),e};function jp(t){const{sx:e}=t,n=H(t,vE),{systemProps:r,otherProps:i}=xE(n);let o;return Array.isArray(e)?o=[r,...e]:typeof e=="function"?o=(...s)=>{const l=e(...s);return Vl(l)?$({},r,l):r}:o=$({},r,e),$({},i,{sx:o})}function rx(t){var e,n,r="";if(typeof t=="string"||typeof t=="number")r+=t;else if(typeof t=="object")if(Array.isArray(t))for(e=0;e<t.length;e++)t[e]&&(n=rx(t[e]))&&(r&&(r+=" "),r+=n);else for(e in t)t[e]&&(r&&(r+=" "),r+=e);return r}function j(){for(var t=0,e,n,r="";t<arguments.length;)(e=arguments[t++])&&(n=rx(e))&&(r&&(r+=" "),r+=n);return r}const wE=["values","unit","step"];function bE(t){const{values:e={xs:0,sm:600,md:900,lg:1200,xl:1536},unit:n="px",step:r=5}=t,i=H(t,wE),o=Object.keys(e);function s(f){return`@media (min-width:${typeof e[f]=="number"?e[f]:f}${n})`}function l(f){return`@media (max-width:${(typeof e[f]=="number"?e[f]:f)-r/100}${n})`}function a(f,h){const d=o.indexOf(h);return`@media (min-width:${typeof e[f]=="number"?e[f]:f}${n}) and (max-width:${(d!==-1&&typeof e[o[d]]=="number"?e[o[d]]:h)-r/100}${n})`}function c(f){return o.indexOf(f)+1<o.length?a(f,o[o.indexOf(f)+1]):s(f)}function u(f){const h=o.indexOf(f);return h===0?s(o[1]):h===o.length-1?l(o[h]):a(f,o[o.indexOf(f)+1]).replace("@media","@media not all and")}return $({keys:o,values:e,up:s,down:l,between:a,only:c,not:u,unit:n},i)}const $E={borderRadius:4};var SE=$E;function kE(t=8){if(t.mui)return t;const e=V1({spacing:t}),n=(...r)=>(r.length===0?[1]:r).map(o=>{const s=e(o);return typeof s=="number"?`${s}px`:s}).join(" ");return n.mui=!0,n}const PE=["breakpoints","palette","spacing","shape"];function Xp(t={},...e){const{breakpoints:n={},palette:r={},spacing:i,shape:o={}}=t,s=H(t,PE),l=bE(n),a=kE(i);let c=vr({breakpoints:l,direction:"ltr",components:{},palette:$({mode:"light"},r),spacing:a,shape:$({},SE,o)},s);return c=e.reduce((u,f)=>vr(u,f),c),c}const CE=w.exports.createContext(null);var ix=CE;function ox(){return w.exports.useContext(ix)}const RE=typeof Symbol=="function"&&Symbol.for;var TE=RE?Symbol.for("mui.nested"):"__THEME_NESTED__";function IE(t,e){return typeof e=="function"?e(t):$({},t,e)}function EE(t){const{children:e,theme:n}=t,r=ox(),i=w.exports.useMemo(()=>{const o=r===null?n:IE(r,n);return o!=null&&(o[TE]=r!==null),o},[n,r]);return I(ix.Provider,{value:i,children:e})}function ME(t){return Object.keys(t).length===0}function Hp(t=null){const e=ox();return!e||ME(e)?t:e}const AE=Xp();function $u(t=AE){return Hp(t)}const DE=["className","component"];function LE(t={}){const{defaultTheme:e,defaultClassName:n="MuiBox-root",generateClassName:r}=t,i=E1("div")(bu);return w.exports.forwardRef(function(l,a){const c=$u(e),u=jp(l),{className:f,component:h="div"}=u,d=H(u,DE);return I(i,$({as:h,ref:a,className:j(f,r?r(n):n),theme:c},d))})}const WE=["variant"];function sx(t){return t.length===0}function lx(t){const{variant:e}=t,n=H(t,WE);let r=e||"";return Object.keys(n).sort().forEach(i=>{i==="color"?r+=sx(r)?t[i]:V(t[i]):r+=`${sx(r)?i:V(i)}${V(t[i].toString())}`}),r}const QE=["name","slot","skipVariantsResolver","skipSx","overridesResolver"],NE=["theme"],_E=["theme"];function Yl(t){return Object.keys(t).length===0}const BE=(t,e)=>e.components&&e.components[t]&&e.components[t].styleOverrides?e.components[t].styleOverrides:null,qE=(t,e)=>{let n=[];e&&e.components&&e.components[t]&&e.components[t].variants&&(n=e.components[t].variants);const r={};return n.forEach(i=>{const o=lx(i.props);r[o]=i.style}),r},zE=(t,e,n,r)=>{var i,o;const{ownerState:s={}}=t,l=[],a=n==null||(i=n.components)==null||(o=i[r])==null?void 0:o.variants;return a&&a.forEach(c=>{let u=!0;Object.keys(c.props).forEach(f=>{s[f]!==c.props[f]&&t[f]!==c.props[f]&&(u=!1)}),u&&l.push(e[lx(c.props)])}),l};function Su(t){return t!=="ownerState"&&t!=="theme"&&t!=="sx"&&t!=="as"}const FE=Xp();function VE(t={}){const{defaultTheme:e=FE,rootShouldForwardProp:n=Su,slotShouldForwardProp:r=Su}=t;return(i,o={})=>{const{name:s,slot:l,skipVariantsResolver:a,skipSx:c,overridesResolver:u}=o,f=H(o,QE),h=a!==void 0?a:l&&l!=="Root"||!1,d=c||!1;let p,m=Su;l==="Root"?m=n:l&&(m=r);const g=E1(i,$({shouldForwardProp:m,label:p},f));return(O,...v)=>{const x=v?v.map(P=>typeof P=="function"&&P.__emotion_real!==P?C=>{let{theme:R}=C,T=H(C,NE);return P($({theme:Yl(R)?e:R},T))}:P):[];let b=O;s&&u&&x.push(P=>{const C=Yl(P.theme)?e:P.theme,R=BE(s,C);return R?u(P,R):null}),s&&!h&&x.push(P=>{const C=Yl(P.theme)?e:P.theme;return zE(P,qE(s,C),C,s)}),d||x.push(P=>{const C=Yl(P.theme)?e:P.theme;return bu($({},P,{theme:C}))});const S=x.length-v.length;if(Array.isArray(O)&&S>0){const P=new Array(S).fill("");b=[...O,...P],b.raw=[...O.raw,...P]}else typeof O=="function"&&(b=P=>{let{theme:C}=P,R=H(P,_E);return O($({theme:Yl(C)?e:C},R))});return g(b,...x)}}}function ax(t){const{theme:e,name:n,props:r}=t;return!e||!e.components||!e.components[n]||!e.components[n].defaultProps?r:_1(e.components[n].defaultProps,r)}function UE({props:t,name:e,defaultTheme:n}){const r=$u(n);return ax({theme:r,name:e,props:t})}function Gp(t,e=0,n=1){return Math.min(Math.max(e,t),n)}function jE(t){t=t.substr(1);const e=new RegExp(`.{1,${t.length>=6?2:1}}`,"g");let n=t.match(e);return n&&n[0].length===1&&(n=n.map(r=>r+r)),n?`rgb${n.length===4?"a":""}(${n.map((r,i)=>i<3?parseInt(r,16):Math.round(parseInt(r,16)/255*1e3)/1e3).join(", ")})`:""}function qi(t){if(t.type)return t;if(t.charAt(0)==="#")return qi(jE(t));const e=t.indexOf("("),n=t.substring(0,e);if(["rgb","rgba","hsl","hsla","color"].indexOf(n)===-1)throw new Error(Ul(9,t));let r=t.substring(e+1,t.length-1),i;if(n==="color"){if(r=r.split(" "),i=r.shift(),r.length===4&&r[3].charAt(0)==="/"&&(r[3]=r[3].substr(1)),["srgb","display-p3","a98-rgb","prophoto-rgb","rec-2020"].indexOf(i)===-1)throw new Error(Ul(10,i))}else r=r.split(",");return r=r.map(o=>parseFloat(o)),{type:n,values:r,colorSpace:i}}function ku(t){const{type:e,colorSpace:n}=t;let{values:r}=t;return e.indexOf("rgb")!==-1?r=r.map((i,o)=>o<3?parseInt(i,10):i):e.indexOf("hsl")!==-1&&(r[1]=`${r[1]}%`,r[2]=`${r[2]}%`),e.indexOf("color")!==-1?r=`${n} ${r.join(" ")}`:r=`${r.join(", ")}`,`${e}(${r})`}function XE(t){t=qi(t);const{values:e}=t,n=e[0],r=e[1]/100,i=e[2]/100,o=r*Math.min(i,1-i),s=(c,u=(c+n/30)%12)=>i-o*Math.max(Math.min(u-3,9-u,1),-1);let l="rgb";const a=[Math.round(s(0)*255),Math.round(s(8)*255),Math.round(s(4)*255)];return t.type==="hsla"&&(l+="a",a.push(e[3])),ku({type:l,values:a})}function cx(t){t=qi(t);let e=t.type==="hsl"?qi(XE(t)).values:t.values;return e=e.map(n=>(t.type!=="color"&&(n/=255),n<=.03928?n/12.92:((n+.055)/1.055)**2.4)),Number((.2126*e[0]+.7152*e[1]+.0722*e[2]).toFixed(3))}function HE(t,e){const n=cx(t),r=cx(e);return(Math.max(n,r)+.05)/(Math.min(n,r)+.05)}function ge(t,e){return t=qi(t),e=Gp(e),(t.type==="rgb"||t.type==="hsl")&&(t.type+="a"),t.type==="color"?t.values[3]=`/${e}`:t.values[3]=e,ku(t)}function ux(t,e){if(t=qi(t),e=Gp(e),t.type.indexOf("hsl")!==-1)t.values[2]*=1-e;else if(t.type.indexOf("rgb")!==-1||t.type.indexOf("color")!==-1)for(let n=0;n<3;n+=1)t.values[n]*=1-e;return ku(t)}function fx(t,e){if(t=qi(t),e=Gp(e),t.type.indexOf("hsl")!==-1)t.values[2]+=(100-t.values[2])*e;else if(t.type.indexOf("rgb")!==-1)for(let n=0;n<3;n+=1)t.values[n]+=(255-t.values[n])*e;else if(t.type.indexOf("color")!==-1)for(let n=0;n<3;n+=1)t.values[n]+=(1-t.values[n])*e;return ku(t)}function GE(t){const e=$u();return I(Dp.Provider,{value:typeof e=="object"?e:{},children:t.children})}function k7(t){const{children:e,theme:n}=t;return I(EE,{theme:n,children:I(GE,{children:e})})}function zi(t){return typeof t=="string"}function Pu(t,e={},n){return zi(t)?e:$({},e,{ownerState:$({},e.ownerState,n)})}function me(t,e,n){const r={};return Object.keys(t).forEach(i=>{r[i]=t[i].reduce((o,s)=>(s&&(n&&n[s]&&o.push(n[s]),o.push(e(s))),o),[]).join(" ")}),r}const hx=t=>t,ZE=()=>{let t=hx;return{configure(e){t=e},generate(e){return t(e)},reset(){t=hx}}},YE=ZE();var dx=YE;const KE={active:"Mui-active",checked:"Mui-checked",completed:"Mui-completed",disabled:"Mui-disabled",error:"Mui-error",expanded:"Mui-expanded",focused:"Mui-focused",focusVisible:"Mui-focusVisible",required:"Mui-required",selected:"Mui-selected"};function he(t,e){return KE[e]||`${dx.generate(t)}-${e}`}function de(t,e){const n={};return e.forEach(r=>{n[r]=he(t,r)}),n}function JE(t){return he("MuiBackdrop",t)}de("MuiBackdrop",["root","invisible"]);const eM=["classes","className","invisible","component","components","componentsProps","theme"],tM=t=>{const{classes:e,invisible:n}=t;return me({root:["root",n&&"invisible"]},JE,e)},nM=w.exports.forwardRef(function(e,n){const{classes:r,className:i,invisible:o=!1,component:s="div",components:l={},componentsProps:a={},theme:c}=e,u=H(e,eM),f=$({},e,{classes:r,invisible:o}),h=tM(f),d=l.Root||s,p=a.root||{};return I(d,$({"aria-hidden":!0},p,!zi(d)&&{as:s,ownerState:$({},f,p.ownerState),theme:c},{ref:n},u,{className:j(h.root,p.className,i)}))});var rM=nM;function px(t){return t.substring(2).toLowerCase()}function iM(t,e){return e.documentElement.clientWidth<t.clientX||e.documentElement.clientHeight<t.clientY}function P7(t){const{children:e,disableReactTree:n=!1,mouseEvent:r="onClick",onClickAway:i,touchEvent:o="onTouchEnd"}=t,s=w.exports.useRef(!1),l=w.exports.useRef(null),a=w.exports.useRef(!1),c=w.exports.useRef(!1);w.exports.useEffect(()=>(setTimeout(()=>{a.current=!0},0),()=>{a.current=!1}),[]);const u=Qe(e.ref,l),f=Dn(p=>{const m=c.current;c.current=!1;const g=Pt(l.current);if(!a.current||!l.current||"clientX"in p&&iM(p,g))return;if(s.current){s.current=!1;return}let y;p.composedPath?y=p.composedPath().indexOf(l.current)>-1:y=!g.documentElement.contains(p.target)||l.current.contains(p.target),!y&&(n||!m)&&i(p)}),h=p=>m=>{c.current=!0;const g=e.props[p];g&&g(m)},d={ref:u};return o!==!1&&(d[o]=h(o)),w.exports.useEffect(()=>{if(o!==!1){const p=px(o),m=Pt(l.current),g=()=>{s.current=!0};return m.addEventListener(p,f),m.addEventListener("touchmove",g),()=>{m.removeEventListener(p,f),m.removeEventListener("touchmove",g)}}},[f,o]),r!==!1&&(d[r]=h(r)),w.exports.useEffect(()=>{if(r!==!1){const p=px(r),m=Pt(l.current);return m.addEventListener(p,f),()=>{m.removeEventListener(p,f)}}},[f,r]),I(w.exports.Fragment,{children:w.exports.cloneElement(e,d)})}function oM(t){return typeof t=="function"?t():t}const sM=w.exports.forwardRef(function(e,n){const{children:r,container:i,disablePortal:o=!1}=e,[s,l]=w.exports.useState(null),a=Qe(w.exports.isValidElement(r)?r.ref:null,n);return Gn(()=>{o||l(oM(i)||document.body)},[i,o]),Gn(()=>{if(s&&!o)return yu(n,s),()=>{yu(n,null)}},[n,s,o]),o?w.exports.isValidElement(r)?w.exports.cloneElement(r,{ref:a}):r:s&&rc.exports.createPortal(r,s)});var mx=sM;function lM(t){const e=Pt(t);return e.body===t?xr(t).innerWidth>e.documentElement.clientWidth:t.scrollHeight>t.clientHeight}function Kl(t,e){e?t.setAttribute("aria-hidden","true"):t.removeAttribute("aria-hidden")}function gx(t){return parseInt(xr(t).getComputedStyle(t).paddingRight,10)||0}function yx(t,e,n,r=[],i){const o=[e,n,...r],s=["TEMPLATE","SCRIPT","STYLE"];[].forEach.call(t.children,l=>{o.indexOf(l)===-1&&s.indexOf(l.tagName)===-1&&Kl(l,i)})}function Zp(t,e){let n=-1;return t.some((r,i)=>e(r)?(n=i,!0):!1),n}function aM(t,e){const n=[],r=t.container;if(!e.disableScrollLock){if(lM(r)){const a=W1(Pt(r));n.push({value:r.style.paddingRight,property:"padding-right",el:r}),r.style.paddingRight=`${gx(r)+a}px`;const c=Pt(r).querySelectorAll(".mui-fixed");[].forEach.call(c,u=>{n.push({value:u.style.paddingRight,property:"padding-right",el:u}),u.style.paddingRight=`${gx(u)+a}px`})}const o=r.parentElement,s=xr(r),l=(o==null?void 0:o.nodeName)==="HTML"&&s.getComputedStyle(o).overflowY==="scroll"?o:r;n.push({value:l.style.overflow,property:"overflow",el:l},{value:l.style.overflowX,property:"overflow-x",el:l},{value:l.style.overflowY,property:"overflow-y",el:l}),l.style.overflow="hidden"}return()=>{n.forEach(({value:o,el:s,property:l})=>{o?s.style.setProperty(l,o):s.style.removeProperty(l)})}}function cM(t){const e=[];return[].forEach.call(t.children,n=>{n.getAttribute("aria-hidden")==="true"&&e.push(n)}),e}class uM{constructor(){this.containers=void 0,this.modals=void 0,this.modals=[],this.containers=[]}add(e,n){let r=this.modals.indexOf(e);if(r!==-1)return r;r=this.modals.length,this.modals.push(e),e.modalRef&&Kl(e.modalRef,!1);const i=cM(n);yx(n,e.mount,e.modalRef,i,!0);const o=Zp(this.containers,s=>s.container===n);return o!==-1?(this.containers[o].modals.push(e),r):(this.containers.push({modals:[e],container:n,restore:null,hiddenSiblings:i}),r)}mount(e,n){const r=Zp(this.containers,o=>o.modals.indexOf(e)!==-1),i=this.containers[r];i.restore||(i.restore=aM(i,n))}remove(e){const n=this.modals.indexOf(e);if(n===-1)return n;const r=Zp(this.containers,o=>o.modals.indexOf(e)!==-1),i=this.containers[r];if(i.modals.splice(i.modals.indexOf(e),1),this.modals.splice(n,1),i.modals.length===0)i.restore&&i.restore(),e.modalRef&&Kl(e.modalRef,!0),yx(i.container,e.mount,e.modalRef,i.hiddenSiblings,!1),this.containers.splice(r,1);else{const o=i.modals[i.modals.length-1];o.modalRef&&Kl(o.modalRef,!1)}return n}isTopModal(e){return this.modals.length>0&&this.modals[this.modals.length-1]===e}}const fM=["input","select","textarea","a[href]","button","[tabindex]","audio[controls]","video[controls]",'[contenteditable]:not([contenteditable="false"])'].join(",");function hM(t){const e=parseInt(t.getAttribute("tabindex"),10);return Number.isNaN(e)?t.contentEditable==="true"||(t.nodeName==="AUDIO"||t.nodeName==="VIDEO"||t.nodeName==="DETAILS")&&t.getAttribute("tabindex")===null?0:t.tabIndex:e}function dM(t){if(t.tagName!=="INPUT"||t.type!=="radio"||!t.name)return!1;const e=r=>t.ownerDocument.querySelector(`input[type="radio"]${r}`);let n=e(`[name="${t.name}"]:checked`);return n||(n=e(`[name="${t.name}"]`)),n!==t}function pM(t){return!(t.disabled||t.tagName==="INPUT"&&t.type==="hidden"||dM(t))}function mM(t){const e=[],n=[];return Array.from(t.querySelectorAll(fM)).forEach((r,i)=>{const o=hM(r);o===-1||!pM(r)||(o===0?e.push(r):n.push({documentOrder:i,tabIndex:o,node:r}))}),n.sort((r,i)=>r.tabIndex===i.tabIndex?r.documentOrder-i.documentOrder:r.tabIndex-i.tabIndex).map(r=>r.node).concat(e)}function gM(){return!0}function yM(t){const{children:e,disableAutoFocus:n=!1,disableEnforceFocus:r=!1,disableRestoreFocus:i=!1,getTabbable:o=mM,isEnabled:s=gM,open:l}=t,a=w.exports.useRef(),c=w.exports.useRef(null),u=w.exports.useRef(null),f=w.exports.useRef(null),h=w.exports.useRef(null),d=w.exports.useRef(!1),p=w.exports.useRef(null),m=Qe(e.ref,p),g=w.exports.useRef(null);w.exports.useEffect(()=>{!l||!p.current||(d.current=!n)},[n,l]),w.exports.useEffect(()=>{if(!l||!p.current)return;const v=Pt(p.current);return p.current.contains(v.activeElement)||(p.current.hasAttribute("tabIndex")||p.current.setAttribute("tabIndex",-1),d.current&&p.current.focus()),()=>{i||(f.current&&f.current.focus&&(a.current=!0,f.current.focus()),f.current=null)}},[l]),w.exports.useEffect(()=>{if(!l||!p.current)return;const v=Pt(p.current),x=k=>{const{current:P}=p;if(P!==null){if(!v.hasFocus()||r||!s()||a.current){a.current=!1;return}if(!P.contains(v.activeElement)){if(k&&h.current!==k.target||v.activeElement!==h.current)h.current=null;else if(h.current!==null)return;if(!d.current)return;let T=[];if((v.activeElement===c.current||v.activeElement===u.current)&&(T=o(p.current)),T.length>0){var C,R;const L=Boolean(((C=g.current)==null?void 0:C.shiftKey)&&((R=g.current)==null?void 0:R.key)==="Tab"),M=T[0],q=T[T.length-1];L?q.focus():M.focus()}else P.focus()}}},b=k=>{g.current=k,!(r||!s()||k.key!=="Tab")&&v.activeElement===p.current&&k.shiftKey&&(a.current=!0,u.current.focus())};v.addEventListener("focusin",x),v.addEventListener("keydown",b,!0);const S=setInterval(()=>{v.activeElement.tagName==="BODY"&&x()},50);return()=>{clearInterval(S),v.removeEventListener("focusin",x),v.removeEventListener("keydown",b,!0)}},[n,r,i,s,l,o]);const y=v=>{f.current===null&&(f.current=v.relatedTarget),d.current=!0,h.current=v.target;const x=e.props.onFocus;x&&x(v)},O=v=>{f.current===null&&(f.current=v.relatedTarget),d.current=!0};return Ue(w.exports.Fragment,{children:[I("div",{tabIndex:0,onFocus:O,ref:c,"data-test":"sentinelStart"}),w.exports.cloneElement(e,{ref:m,onFocus:y}),I("div",{tabIndex:0,onFocus:O,ref:u,"data-test":"sentinelEnd"})]})}function OM(t){return he("MuiModal",t)}de("MuiModal",["root","hidden"]);const vM=["BackdropComponent","BackdropProps","children","classes","className","closeAfterTransition","component","components","componentsProps","container","disableAutoFocus","disableEnforceFocus","disableEscapeKeyDown","disablePortal","disableRestoreFocus","disableScrollLock","hideBackdrop","keepMounted","manager","onBackdropClick","onClose","onKeyDown","open","theme","onTransitionEnter","onTransitionExited"],xM=t=>{const{open:e,exited:n,classes:r}=t;return me({root:["root",!e&&n&&"hidden"]},OM,r)};function wM(t){return typeof t=="function"?t():t}function bM(t){return t.children?t.children.props.hasOwnProperty("in"):!1}const $M=new uM,SM=w.exports.forwardRef(function(e,n){const{BackdropComponent:r,BackdropProps:i,children:o,classes:s,className:l,closeAfterTransition:a=!1,component:c="div",components:u={},componentsProps:f={},container:h,disableAutoFocus:d=!1,disableEnforceFocus:p=!1,disableEscapeKeyDown:m=!1,disablePortal:g=!1,disableRestoreFocus:y=!1,disableScrollLock:O=!1,hideBackdrop:v=!1,keepMounted:x=!1,manager:b=$M,onBackdropClick:S,onClose:k,onKeyDown:P,open:C,theme:R,onTransitionEnter:T,onTransitionExited:L}=e,M=H(e,vM),[q,F]=w.exports.useState(!0),Q=w.exports.useRef({}),E=w.exports.useRef(null),D=w.exports.useRef(null),X=Qe(D,n),ae=bM(e),W=()=>Pt(E.current),B=()=>(Q.current.modalRef=D.current,Q.current.mountNode=E.current,Q.current),Z=()=>{b.mount(B(),{disableScrollLock:O}),D.current.scrollTop=0},ee=Dn(()=>{const Ee=wM(h)||W().body;b.add(B(),Ee),D.current&&Z()}),pe=w.exports.useCallback(()=>b.isTopModal(B()),[b]),J=Dn(Ee=>{E.current=Ee,!!Ee&&(C&&pe()?Z():Kl(D.current,!0))}),ue=w.exports.useCallback(()=>{b.remove(B())},[b]);w.exports.useEffect(()=>()=>{ue()},[ue]),w.exports.useEffect(()=>{C?ee():(!ae||!a)&&ue()},[C,ue,ae,a,ee]);const se=$({},e,{classes:s,closeAfterTransition:a,disableAutoFocus:d,disableEnforceFocus:p,disableEscapeKeyDown:m,disablePortal:g,disableRestoreFocus:y,disableScrollLock:O,exited:q,hideBackdrop:v,keepMounted:x}),we=xM(se);if(!x&&!C&&(!ae||q))return null;const Be=()=>{F(!1),T&&T()},Ot=()=>{F(!0),L&&L(),a&&ue()},Ht=Ee=>{Ee.target===Ee.currentTarget&&(S&&S(Ee),k&&k(Ee,"backdropClick"))},Gt=Ee=>{P&&P(Ee),!(Ee.key!=="Escape"||!pe())&&(m||(Ee.stopPropagation(),k&&k(Ee,"escapeKeyDown")))},Ge={};o.props.tabIndex===void 0&&(Ge.tabIndex="-1"),ae&&(Ge.onEnter=M1(Be,o.props.onEnter),Ge.onExited=M1(Ot,o.props.onExited));const Et=u.Root||c,ot=f.root||{};return I(mx,{ref:J,container:h,disablePortal:g,children:Ue(Et,$({role:"presentation"},ot,!zi(Et)&&{as:c,ownerState:$({},se,ot.ownerState),theme:R},M,{ref:X,onKeyDown:Gt,className:j(we.root,ot.className,l),children:[!v&&r?I(r,$({open:C,onClick:Ht},i)):null,I(yM,{disableEnforceFocus:p,disableAutoFocus:d,disableRestoreFocus:y,isEnabled:pe,open:C,children:w.exports.cloneElement(o,Ge)})]}))})});var kM=SM,Ft="top",yn="bottom",On="right",Vt="left",Yp="auto",Jl=[Ft,yn,On,Vt],Zo="start",ea="end",PM="clippingParents",Ox="viewport",ta="popper",CM="reference",vx=Jl.reduce(function(t,e){return t.concat([e+"-"+Zo,e+"-"+ea])},[]),xx=[].concat(Jl,[Yp]).reduce(function(t,e){return t.concat([e,e+"-"+Zo,e+"-"+ea])},[]),RM="beforeRead",TM="read",IM="afterRead",EM="beforeMain",MM="main",AM="afterMain",DM="beforeWrite",LM="write",WM="afterWrite",QM=[RM,TM,IM,EM,MM,AM,DM,LM,WM];function Zn(t){return t?(t.nodeName||"").toLowerCase():null}function Wn(t){if(t==null)return window;if(t.toString()!=="[object Window]"){var e=t.ownerDocument;return e&&e.defaultView||window}return t}function Yo(t){var e=Wn(t).Element;return t instanceof e||t instanceof Element}function vn(t){var e=Wn(t).HTMLElement;return t instanceof e||t instanceof HTMLElement}function wx(t){if(typeof ShadowRoot=="undefined")return!1;var e=Wn(t).ShadowRoot;return t instanceof e||t instanceof ShadowRoot}function NM(t){var e=t.state;Object.keys(e.elements).forEach(function(n){var r=e.styles[n]||{},i=e.attributes[n]||{},o=e.elements[n];!vn(o)||!Zn(o)||(Object.assign(o.style,r),Object.keys(i).forEach(function(s){var l=i[s];l===!1?o.removeAttribute(s):o.setAttribute(s,l===!0?"":l)}))})}function _M(t){var e=t.state,n={popper:{position:e.options.strategy,left:"0",top:"0",margin:"0"},arrow:{position:"absolute"},reference:{}};return Object.assign(e.elements.popper.style,n.popper),e.styles=n,e.elements.arrow&&Object.assign(e.elements.arrow.style,n.arrow),function(){Object.keys(e.elements).forEach(function(r){var i=e.elements[r],o=e.attributes[r]||{},s=Object.keys(e.styles.hasOwnProperty(r)?e.styles[r]:n[r]),l=s.reduce(function(a,c){return a[c]="",a},{});!vn(i)||!Zn(i)||(Object.assign(i.style,l),Object.keys(o).forEach(function(a){i.removeAttribute(a)}))})}}var BM={name:"applyStyles",enabled:!0,phase:"write",fn:NM,effect:_M,requires:["computeStyles"]};function Yn(t){return t.split("-")[0]}var Fi=Math.max,Cu=Math.min,Ko=Math.round;function Jo(t,e){e===void 0&&(e=!1);var n=t.getBoundingClientRect(),r=1,i=1;if(vn(t)&&e){var o=t.offsetHeight,s=t.offsetWidth;s>0&&(r=Ko(n.width)/s||1),o>0&&(i=Ko(n.height)/o||1)}return{width:n.width/r,height:n.height/i,top:n.top/i,right:n.right/r,bottom:n.bottom/i,left:n.left/r,x:n.left/r,y:n.top/i}}function Kp(t){var e=Jo(t),n=t.offsetWidth,r=t.offsetHeight;return Math.abs(e.width-n)<=1&&(n=e.width),Math.abs(e.height-r)<=1&&(r=e.height),{x:t.offsetLeft,y:t.offsetTop,width:n,height:r}}function bx(t,e){var n=e.getRootNode&&e.getRootNode();if(t.contains(e))return!0;if(n&&wx(n)){var r=e;do{if(r&&t.isSameNode(r))return!0;r=r.parentNode||r.host}while(r)}return!1}function wr(t){return Wn(t).getComputedStyle(t)}function qM(t){return["table","td","th"].indexOf(Zn(t))>=0}function ai(t){return((Yo(t)?t.ownerDocument:t.document)||window.document).documentElement}function Ru(t){return Zn(t)==="html"?t:t.assignedSlot||t.parentNode||(wx(t)?t.host:null)||ai(t)}function $x(t){return!vn(t)||wr(t).position==="fixed"?null:t.offsetParent}function zM(t){var e=navigator.userAgent.toLowerCase().indexOf("firefox")!==-1,n=navigator.userAgent.indexOf("Trident")!==-1;if(n&&vn(t)){var r=wr(t);if(r.position==="fixed")return null}for(var i=Ru(t);vn(i)&&["html","body"].indexOf(Zn(i))<0;){var o=wr(i);if(o.transform!=="none"||o.perspective!=="none"||o.contain==="paint"||["transform","perspective"].indexOf(o.willChange)!==-1||e&&o.willChange==="filter"||e&&o.filter&&o.filter!=="none")return i;i=i.parentNode}return null}function na(t){for(var e=Wn(t),n=$x(t);n&&qM(n)&&wr(n).position==="static";)n=$x(n);return n&&(Zn(n)==="html"||Zn(n)==="body"&&wr(n).position==="static")?e:n||zM(t)||e}function Jp(t){return["top","bottom"].indexOf(t)>=0?"x":"y"}function ra(t,e,n){return Fi(t,Cu(e,n))}function FM(t,e,n){var r=ra(t,e,n);return r>n?n:r}function Sx(){return{top:0,right:0,bottom:0,left:0}}function kx(t){return Object.assign({},Sx(),t)}function Px(t,e){return e.reduce(function(n,r){return n[r]=t,n},{})}var VM=function(e,n){return e=typeof e=="function"?e(Object.assign({},n.rects,{placement:n.placement})):e,kx(typeof e!="number"?e:Px(e,Jl))};function UM(t){var e,n=t.state,r=t.name,i=t.options,o=n.elements.arrow,s=n.modifiersData.popperOffsets,l=Yn(n.placement),a=Jp(l),c=[Vt,On].indexOf(l)>=0,u=c?"height":"width";if(!(!o||!s)){var f=VM(i.padding,n),h=Kp(o),d=a==="y"?Ft:Vt,p=a==="y"?yn:On,m=n.rects.reference[u]+n.rects.reference[a]-s[a]-n.rects.popper[u],g=s[a]-n.rects.reference[a],y=na(o),O=y?a==="y"?y.clientHeight||0:y.clientWidth||0:0,v=m/2-g/2,x=f[d],b=O-h[u]-f[p],S=O/2-h[u]/2+v,k=ra(x,S,b),P=a;n.modifiersData[r]=(e={},e[P]=k,e.centerOffset=k-S,e)}}function jM(t){var e=t.state,n=t.options,r=n.element,i=r===void 0?"[data-popper-arrow]":r;i!=null&&(typeof i=="string"&&(i=e.elements.popper.querySelector(i),!i)||!bx(e.elements.popper,i)||(e.elements.arrow=i))}var XM={name:"arrow",enabled:!0,phase:"main",fn:UM,effect:jM,requires:["popperOffsets"],requiresIfExists:["preventOverflow"]};function es(t){return t.split("-")[1]}var HM={top:"auto",right:"auto",bottom:"auto",left:"auto"};function GM(t){var e=t.x,n=t.y,r=window,i=r.devicePixelRatio||1;return{x:Ko(e*i)/i||0,y:Ko(n*i)/i||0}}function Cx(t){var e,n=t.popper,r=t.popperRect,i=t.placement,o=t.variation,s=t.offsets,l=t.position,a=t.gpuAcceleration,c=t.adaptive,u=t.roundOffsets,f=t.isFixed,h=s.x,d=h===void 0?0:h,p=s.y,m=p===void 0?0:p,g=typeof u=="function"?u({x:d,y:m}):{x:d,y:m};d=g.x,m=g.y;var y=s.hasOwnProperty("x"),O=s.hasOwnProperty("y"),v=Vt,x=Ft,b=window;if(c){var S=na(n),k="clientHeight",P="clientWidth";if(S===Wn(n)&&(S=ai(n),wr(S).position!=="static"&&l==="absolute"&&(k="scrollHeight",P="scrollWidth")),S=S,i===Ft||(i===Vt||i===On)&&o===ea){x=yn;var C=f&&b.visualViewport?b.visualViewport.height:S[k];m-=C-r.height,m*=a?1:-1}if(i===Vt||(i===Ft||i===yn)&&o===ea){v=On;var R=f&&b.visualViewport?b.visualViewport.width:S[P];d-=R-r.width,d*=a?1:-1}}var T=Object.assign({position:l},c&&HM),L=u===!0?GM({x:d,y:m}):{x:d,y:m};if(d=L.x,m=L.y,a){var M;return Object.assign({},T,(M={},M[x]=O?"0":"",M[v]=y?"0":"",M.transform=(b.devicePixelRatio||1)<=1?"translate("+d+"px, "+m+"px)":"translate3d("+d+"px, "+m+"px, 0)",M))}return Object.assign({},T,(e={},e[x]=O?m+"px":"",e[v]=y?d+"px":"",e.transform="",e))}function ZM(t){var e=t.state,n=t.options,r=n.gpuAcceleration,i=r===void 0?!0:r,o=n.adaptive,s=o===void 0?!0:o,l=n.roundOffsets,a=l===void 0?!0:l,c={placement:Yn(e.placement),variation:es(e.placement),popper:e.elements.popper,popperRect:e.rects.popper,gpuAcceleration:i,isFixed:e.options.strategy==="fixed"};e.modifiersData.popperOffsets!=null&&(e.styles.popper=Object.assign({},e.styles.popper,Cx(Object.assign({},c,{offsets:e.modifiersData.popperOffsets,position:e.options.strategy,adaptive:s,roundOffsets:a})))),e.modifiersData.arrow!=null&&(e.styles.arrow=Object.assign({},e.styles.arrow,Cx(Object.assign({},c,{offsets:e.modifiersData.arrow,position:"absolute",adaptive:!1,roundOffsets:a})))),e.attributes.popper=Object.assign({},e.attributes.popper,{"data-popper-placement":e.placement})}var YM={name:"computeStyles",enabled:!0,phase:"beforeWrite",fn:ZM,data:{}},Tu={passive:!0};function KM(t){var e=t.state,n=t.instance,r=t.options,i=r.scroll,o=i===void 0?!0:i,s=r.resize,l=s===void 0?!0:s,a=Wn(e.elements.popper),c=[].concat(e.scrollParents.reference,e.scrollParents.popper);return o&&c.forEach(function(u){u.addEventListener("scroll",n.update,Tu)}),l&&a.addEventListener("resize",n.update,Tu),function(){o&&c.forEach(function(u){u.removeEventListener("scroll",n.update,Tu)}),l&&a.removeEventListener("resize",n.update,Tu)}}var JM={name:"eventListeners",enabled:!0,phase:"write",fn:function(){},effect:KM,data:{}},eA={left:"right",right:"left",bottom:"top",top:"bottom"};function Iu(t){return t.replace(/left|right|bottom|top/g,function(e){return eA[e]})}var tA={start:"end",end:"start"};function Rx(t){return t.replace(/start|end/g,function(e){return tA[e]})}function em(t){var e=Wn(t),n=e.pageXOffset,r=e.pageYOffset;return{scrollLeft:n,scrollTop:r}}function tm(t){return Jo(ai(t)).left+em(t).scrollLeft}function nA(t){var e=Wn(t),n=ai(t),r=e.visualViewport,i=n.clientWidth,o=n.clientHeight,s=0,l=0;return r&&(i=r.width,o=r.height,/^((?!chrome|android).)*safari/i.test(navigator.userAgent)||(s=r.offsetLeft,l=r.offsetTop)),{width:i,height:o,x:s+tm(t),y:l}}function rA(t){var e,n=ai(t),r=em(t),i=(e=t.ownerDocument)==null?void 0:e.body,o=Fi(n.scrollWidth,n.clientWidth,i?i.scrollWidth:0,i?i.clientWidth:0),s=Fi(n.scrollHeight,n.clientHeight,i?i.scrollHeight:0,i?i.clientHeight:0),l=-r.scrollLeft+tm(t),a=-r.scrollTop;return wr(i||n).direction==="rtl"&&(l+=Fi(n.clientWidth,i?i.clientWidth:0)-o),{width:o,height:s,x:l,y:a}}function nm(t){var e=wr(t),n=e.overflow,r=e.overflowX,i=e.overflowY;return/auto|scroll|overlay|hidden/.test(n+i+r)}function Tx(t){return["html","body","#document"].indexOf(Zn(t))>=0?t.ownerDocument.body:vn(t)&&nm(t)?t:Tx(Ru(t))}function ia(t,e){var n;e===void 0&&(e=[]);var r=Tx(t),i=r===((n=t.ownerDocument)==null?void 0:n.body),o=Wn(r),s=i?[o].concat(o.visualViewport||[],nm(r)?r:[]):r,l=e.concat(s);return i?l:l.concat(ia(Ru(s)))}function rm(t){return Object.assign({},t,{left:t.x,top:t.y,right:t.x+t.width,bottom:t.y+t.height})}function iA(t){var e=Jo(t);return e.top=e.top+t.clientTop,e.left=e.left+t.clientLeft,e.bottom=e.top+t.clientHeight,e.right=e.left+t.clientWidth,e.width=t.clientWidth,e.height=t.clientHeight,e.x=e.left,e.y=e.top,e}function Ix(t,e){return e===Ox?rm(nA(t)):Yo(e)?iA(e):rm(rA(ai(t)))}function oA(t){var e=ia(Ru(t)),n=["absolute","fixed"].indexOf(wr(t).position)>=0,r=n&&vn(t)?na(t):t;return Yo(r)?e.filter(function(i){return Yo(i)&&bx(i,r)&&Zn(i)!=="body"}):[]}function sA(t,e,n){var r=e==="clippingParents"?oA(t):[].concat(e),i=[].concat(r,[n]),o=i[0],s=i.reduce(function(l,a){var c=Ix(t,a);return l.top=Fi(c.top,l.top),l.right=Cu(c.right,l.right),l.bottom=Cu(c.bottom,l.bottom),l.left=Fi(c.left,l.left),l},Ix(t,o));return s.width=s.right-s.left,s.height=s.bottom-s.top,s.x=s.left,s.y=s.top,s}function Ex(t){var e=t.reference,n=t.element,r=t.placement,i=r?Yn(r):null,o=r?es(r):null,s=e.x+e.width/2-n.width/2,l=e.y+e.height/2-n.height/2,a;switch(i){case Ft:a={x:s,y:e.y-n.height};break;case yn:a={x:s,y:e.y+e.height};break;case On:a={x:e.x+e.width,y:l};break;case Vt:a={x:e.x-n.width,y:l};break;default:a={x:e.x,y:e.y}}var c=i?Jp(i):null;if(c!=null){var u=c==="y"?"height":"width";switch(o){case Zo:a[c]=a[c]-(e[u]/2-n[u]/2);break;case ea:a[c]=a[c]+(e[u]/2-n[u]/2);break}}return a}function oa(t,e){e===void 0&&(e={});var n=e,r=n.placement,i=r===void 0?t.placement:r,o=n.boundary,s=o===void 0?PM:o,l=n.rootBoundary,a=l===void 0?Ox:l,c=n.elementContext,u=c===void 0?ta:c,f=n.altBoundary,h=f===void 0?!1:f,d=n.padding,p=d===void 0?0:d,m=kx(typeof p!="number"?p:Px(p,Jl)),g=u===ta?CM:ta,y=t.rects.popper,O=t.elements[h?g:u],v=sA(Yo(O)?O:O.contextElement||ai(t.elements.popper),s,a),x=Jo(t.elements.reference),b=Ex({reference:x,element:y,strategy:"absolute",placement:i}),S=rm(Object.assign({},y,b)),k=u===ta?S:x,P={top:v.top-k.top+m.top,bottom:k.bottom-v.bottom+m.bottom,left:v.left-k.left+m.left,right:k.right-v.right+m.right},C=t.modifiersData.offset;if(u===ta&&C){var R=C[i];Object.keys(P).forEach(function(T){var L=[On,yn].indexOf(T)>=0?1:-1,M=[Ft,yn].indexOf(T)>=0?"y":"x";P[T]+=R[M]*L})}return P}function lA(t,e){e===void 0&&(e={});var n=e,r=n.placement,i=n.boundary,o=n.rootBoundary,s=n.padding,l=n.flipVariations,a=n.allowedAutoPlacements,c=a===void 0?xx:a,u=es(r),f=u?l?vx:vx.filter(function(p){return es(p)===u}):Jl,h=f.filter(function(p){return c.indexOf(p)>=0});h.length===0&&(h=f);var d=h.reduce(function(p,m){return p[m]=oa(t,{placement:m,boundary:i,rootBoundary:o,padding:s})[Yn(m)],p},{});return Object.keys(d).sort(function(p,m){return d[p]-d[m]})}function aA(t){if(Yn(t)===Yp)return[];var e=Iu(t);return[Rx(t),e,Rx(e)]}function cA(t){var e=t.state,n=t.options,r=t.name;if(!e.modifiersData[r]._skip){for(var i=n.mainAxis,o=i===void 0?!0:i,s=n.altAxis,l=s===void 0?!0:s,a=n.fallbackPlacements,c=n.padding,u=n.boundary,f=n.rootBoundary,h=n.altBoundary,d=n.flipVariations,p=d===void 0?!0:d,m=n.allowedAutoPlacements,g=e.options.placement,y=Yn(g),O=y===g,v=a||(O||!p?[Iu(g)]:aA(g)),x=[g].concat(v).reduce(function(ee,pe){return ee.concat(Yn(pe)===Yp?lA(e,{placement:pe,boundary:u,rootBoundary:f,padding:c,flipVariations:p,allowedAutoPlacements:m}):pe)},[]),b=e.rects.reference,S=e.rects.popper,k=new Map,P=!0,C=x[0],R=0;R<x.length;R++){var T=x[R],L=Yn(T),M=es(T)===Zo,q=[Ft,yn].indexOf(L)>=0,F=q?"width":"height",Q=oa(e,{placement:T,boundary:u,rootBoundary:f,altBoundary:h,padding:c}),E=q?M?On:Vt:M?yn:Ft;b[F]>S[F]&&(E=Iu(E));var D=Iu(E),X=[];if(o&&X.push(Q[L]<=0),l&&X.push(Q[E]<=0,Q[D]<=0),X.every(function(ee){return ee})){C=T,P=!1;break}k.set(T,X)}if(P)for(var ae=p?3:1,W=function(pe){var J=x.find(function(ue){var se=k.get(ue);if(se)return se.slice(0,pe).every(function(we){return we})});if(J)return C=J,"break"},B=ae;B>0;B--){var Z=W(B);if(Z==="break")break}e.placement!==C&&(e.modifiersData[r]._skip=!0,e.placement=C,e.reset=!0)}}var uA={name:"flip",enabled:!0,phase:"main",fn:cA,requiresIfExists:["offset"],data:{_skip:!1}};function Mx(t,e,n){return n===void 0&&(n={x:0,y:0}),{top:t.top-e.height-n.y,right:t.right-e.width+n.x,bottom:t.bottom-e.height+n.y,left:t.left-e.width-n.x}}function Ax(t){return[Ft,On,yn,Vt].some(function(e){return t[e]>=0})}function fA(t){var e=t.state,n=t.name,r=e.rects.reference,i=e.rects.popper,o=e.modifiersData.preventOverflow,s=oa(e,{elementContext:"reference"}),l=oa(e,{altBoundary:!0}),a=Mx(s,r),c=Mx(l,i,o),u=Ax(a),f=Ax(c);e.modifiersData[n]={referenceClippingOffsets:a,popperEscapeOffsets:c,isReferenceHidden:u,hasPopperEscaped:f},e.attributes.popper=Object.assign({},e.attributes.popper,{"data-popper-reference-hidden":u,"data-popper-escaped":f})}var hA={name:"hide",enabled:!0,phase:"main",requiresIfExists:["preventOverflow"],fn:fA};function dA(t,e,n){var r=Yn(t),i=[Vt,Ft].indexOf(r)>=0?-1:1,o=typeof n=="function"?n(Object.assign({},e,{placement:t})):n,s=o[0],l=o[1];return s=s||0,l=(l||0)*i,[Vt,On].indexOf(r)>=0?{x:l,y:s}:{x:s,y:l}}function pA(t){var e=t.state,n=t.options,r=t.name,i=n.offset,o=i===void 0?[0,0]:i,s=xx.reduce(function(u,f){return u[f]=dA(f,e.rects,o),u},{}),l=s[e.placement],a=l.x,c=l.y;e.modifiersData.popperOffsets!=null&&(e.modifiersData.popperOffsets.x+=a,e.modifiersData.popperOffsets.y+=c),e.modifiersData[r]=s}var mA={name:"offset",enabled:!0,phase:"main",requires:["popperOffsets"],fn:pA};function gA(t){var e=t.state,n=t.name;e.modifiersData[n]=Ex({reference:e.rects.reference,element:e.rects.popper,strategy:"absolute",placement:e.placement})}var yA={name:"popperOffsets",enabled:!0,phase:"read",fn:gA,data:{}};function OA(t){return t==="x"?"y":"x"}function vA(t){var e=t.state,n=t.options,r=t.name,i=n.mainAxis,o=i===void 0?!0:i,s=n.altAxis,l=s===void 0?!1:s,a=n.boundary,c=n.rootBoundary,u=n.altBoundary,f=n.padding,h=n.tether,d=h===void 0?!0:h,p=n.tetherOffset,m=p===void 0?0:p,g=oa(e,{boundary:a,rootBoundary:c,padding:f,altBoundary:u}),y=Yn(e.placement),O=es(e.placement),v=!O,x=Jp(y),b=OA(x),S=e.modifiersData.popperOffsets,k=e.rects.reference,P=e.rects.popper,C=typeof m=="function"?m(Object.assign({},e.rects,{placement:e.placement})):m,R=typeof C=="number"?{mainAxis:C,altAxis:C}:Object.assign({mainAxis:0,altAxis:0},C),T=e.modifiersData.offset?e.modifiersData.offset[e.placement]:null,L={x:0,y:0};if(!!S){if(o){var M,q=x==="y"?Ft:Vt,F=x==="y"?yn:On,Q=x==="y"?"height":"width",E=S[x],D=E+g[q],X=E-g[F],ae=d?-P[Q]/2:0,W=O===Zo?k[Q]:P[Q],B=O===Zo?-P[Q]:-k[Q],Z=e.elements.arrow,ee=d&&Z?Kp(Z):{width:0,height:0},pe=e.modifiersData["arrow#persistent"]?e.modifiersData["arrow#persistent"].padding:Sx(),J=pe[q],ue=pe[F],se=ra(0,k[Q],ee[Q]),we=v?k[Q]/2-ae-se-J-R.mainAxis:W-se-J-R.mainAxis,Be=v?-k[Q]/2+ae+se+ue+R.mainAxis:B+se+ue+R.mainAxis,Ot=e.elements.arrow&&na(e.elements.arrow),Ht=Ot?x==="y"?Ot.clientTop||0:Ot.clientLeft||0:0,Gt=(M=T==null?void 0:T[x])!=null?M:0,Ge=E+we-Gt-Ht,Et=E+Be-Gt,ot=ra(d?Cu(D,Ge):D,E,d?Fi(X,Et):X);S[x]=ot,L[x]=ot-E}if(l){var Ee,Wr=x==="x"?Ft:Vt,Zt=x==="x"?yn:On,Ke=S[b],ft=b==="y"?"height":"width",Pn=Ke+g[Wr],fr=Ke-g[Zt],un=[Ft,Vt].indexOf(y)!==-1,Qr=(Ee=T==null?void 0:T[b])!=null?Ee:0,_n=un?Pn:Ke-k[ft]-P[ft]-Qr+R.altAxis,Cn=un?Ke+k[ft]+P[ft]-Qr-R.altAxis:fr,N=d&&un?FM(_n,Ke,Cn):ra(d?_n:Pn,Ke,d?Cn:fr);S[b]=N,L[b]=N-Ke}e.modifiersData[r]=L}}var xA={name:"preventOverflow",enabled:!0,phase:"main",fn:vA,requiresIfExists:["offset"]};function wA(t){return{scrollLeft:t.scrollLeft,scrollTop:t.scrollTop}}function bA(t){return t===Wn(t)||!vn(t)?em(t):wA(t)}function $A(t){var e=t.getBoundingClientRect(),n=Ko(e.width)/t.offsetWidth||1,r=Ko(e.height)/t.offsetHeight||1;return n!==1||r!==1}function SA(t,e,n){n===void 0&&(n=!1);var r=vn(e),i=vn(e)&&$A(e),o=ai(e),s=Jo(t,i),l={scrollLeft:0,scrollTop:0},a={x:0,y:0};return(r||!r&&!n)&&((Zn(e)!=="body"||nm(o))&&(l=bA(e)),vn(e)?(a=Jo(e,!0),a.x+=e.clientLeft,a.y+=e.clientTop):o&&(a.x=tm(o))),{x:s.left+l.scrollLeft-a.x,y:s.top+l.scrollTop-a.y,width:s.width,height:s.height}}function kA(t){var e=new Map,n=new Set,r=[];t.forEach(function(o){e.set(o.name,o)});function i(o){n.add(o.name);var s=[].concat(o.requires||[],o.requiresIfExists||[]);s.forEach(function(l){if(!n.has(l)){var a=e.get(l);a&&i(a)}}),r.push(o)}return t.forEach(function(o){n.has(o.name)||i(o)}),r}function PA(t){var e=kA(t);return QM.reduce(function(n,r){return n.concat(e.filter(function(i){return i.phase===r}))},[])}function CA(t){var e;return function(){return e||(e=new Promise(function(n){Promise.resolve().then(function(){e=void 0,n(t())})})),e}}function RA(t){var e=t.reduce(function(n,r){var i=n[r.name];return n[r.name]=i?Object.assign({},i,r,{options:Object.assign({},i.options,r.options),data:Object.assign({},i.data,r.data)}):r,n},{});return Object.keys(e).map(function(n){return e[n]})}var Dx={placement:"bottom",modifiers:[],strategy:"absolute"};function Lx(){for(var t=arguments.length,e=new Array(t),n=0;n<t;n++)e[n]=arguments[n];return!e.some(function(r){return!(r&&typeof r.getBoundingClientRect=="function")})}function TA(t){t===void 0&&(t={});var e=t,n=e.defaultModifiers,r=n===void 0?[]:n,i=e.defaultOptions,o=i===void 0?Dx:i;return function(l,a,c){c===void 0&&(c=o);var u={placement:"bottom",orderedModifiers:[],options:Object.assign({},Dx,o),modifiersData:{},elements:{reference:l,popper:a},attributes:{},styles:{}},f=[],h=!1,d={state:u,setOptions:function(y){var O=typeof y=="function"?y(u.options):y;m(),u.options=Object.assign({},o,u.options,O),u.scrollParents={reference:Yo(l)?ia(l):l.contextElement?ia(l.contextElement):[],popper:ia(a)};var v=PA(RA([].concat(r,u.options.modifiers)));return u.orderedModifiers=v.filter(function(x){return x.enabled}),p(),d.update()},forceUpdate:function(){if(!h){var y=u.elements,O=y.reference,v=y.popper;if(!!Lx(O,v)){u.rects={reference:SA(O,na(v),u.options.strategy==="fixed"),popper:Kp(v)},u.reset=!1,u.placement=u.options.placement,u.orderedModifiers.forEach(function(R){return u.modifiersData[R.name]=Object.assign({},R.data)});for(var x=0;x<u.orderedModifiers.length;x++){if(u.reset===!0){u.reset=!1,x=-1;continue}var b=u.orderedModifiers[x],S=b.fn,k=b.options,P=k===void 0?{}:k,C=b.name;typeof S=="function"&&(u=S({state:u,options:P,name:C,instance:d})||u)}}}},update:CA(function(){return new Promise(function(g){d.forceUpdate(),g(u)})}),destroy:function(){m(),h=!0}};if(!Lx(l,a))return d;d.setOptions(c).then(function(g){!h&&c.onFirstUpdate&&c.onFirstUpdate(g)});function p(){u.orderedModifiers.forEach(function(g){var y=g.name,O=g.options,v=O===void 0?{}:O,x=g.effect;if(typeof x=="function"){var b=x({state:u,name:y,instance:d,options:v}),S=function(){};f.push(b||S)}})}function m(){f.forEach(function(g){return g()}),f=[]}return d}}var IA=[JM,yA,YM,BM,mA,uA,xA,XM,hA],EA=TA({defaultModifiers:IA});const MA=["anchorEl","children","direction","disablePortal","modifiers","open","placement","popperOptions","popperRef","TransitionProps"],AA=["anchorEl","children","container","direction","disablePortal","keepMounted","modifiers","open","placement","popperOptions","popperRef","style","transition"];function DA(t,e){if(e==="ltr")return t;switch(t){case"bottom-end":return"bottom-start";case"bottom-start":return"bottom-end";case"top-end":return"top-start";case"top-start":return"top-end";default:return t}}function im(t){return typeof t=="function"?t():t}const LA={},WA=w.exports.forwardRef(function(e,n){const{anchorEl:r,children:i,direction:o,disablePortal:s,modifiers:l,open:a,placement:c,popperOptions:u,popperRef:f,TransitionProps:h}=e,d=H(e,MA),p=w.exports.useRef(null),m=Qe(p,n),g=w.exports.useRef(null),y=Qe(g,f),O=w.exports.useRef(y);Gn(()=>{O.current=y},[y]),w.exports.useImperativeHandle(f,()=>g.current,[]);const v=DA(c,o),[x,b]=w.exports.useState(v);w.exports.useEffect(()=>{g.current&&g.current.forceUpdate()}),Gn(()=>{if(!r||!a)return;const k=R=>{b(R.placement)};im(r);let P=[{name:"preventOverflow",options:{altBoundary:s}},{name:"flip",options:{altBoundary:s}},{name:"onUpdate",enabled:!0,phase:"afterWrite",fn:({state:R})=>{k(R)}}];l!=null&&(P=P.concat(l)),u&&u.modifiers!=null&&(P=P.concat(u.modifiers));const C=EA(im(r),p.current,$({placement:v},u,{modifiers:P}));return O.current(C),()=>{C.destroy(),O.current(null)}},[r,s,l,a,u,v]);const S={placement:x};return h!==null&&(S.TransitionProps=h),I("div",$({ref:m,role:"tooltip"},d,{children:typeof i=="function"?i(S):i}))}),QA=w.exports.forwardRef(function(e,n){const{anchorEl:r,children:i,container:o,direction:s="ltr",disablePortal:l=!1,keepMounted:a=!1,modifiers:c,open:u,placement:f="bottom",popperOptions:h=LA,popperRef:d,style:p,transition:m=!1}=e,g=H(e,AA),[y,O]=w.exports.useState(!0),v=()=>{O(!1)},x=()=>{O(!0)};if(!a&&!u&&(!m||y))return null;const b=o||(r?Pt(im(r)).body:void 0);return I(mx,{disablePortal:l,container:b,children:I(WA,$({anchorEl:r,direction:s,disablePortal:l,modifiers:c,ref:n,open:m?!y:u,placement:f,popperOptions:h,popperRef:d},g,{style:$({position:"fixed",top:0,left:0,display:!u&&a&&(!m||y)?"none":null},p),TransitionProps:m?{in:u,onEnter:v,onExited:x}:null,children:i}))})});var NA=QA;function _A(t,e,n){return $({toolbar:{minHeight:56,[`${t.up("xs")} and (orientation: landscape)`]:{minHeight:48},[t.up("sm")]:{minHeight:64}}},n)}const BA=["mode","contrastThreshold","tonalOffset"],Wx={text:{primary:"rgba(0, 0, 0, 0.87)",secondary:"rgba(0, 0, 0, 0.6)",disabled:"rgba(0, 0, 0, 0.38)"},divider:"rgba(0, 0, 0, 0.12)",background:{paper:Wl.white,default:Wl.white},action:{active:"rgba(0, 0, 0, 0.54)",hover:"rgba(0, 0, 0, 0.04)",hoverOpacity:.04,selected:"rgba(0, 0, 0, 0.08)",selectedOpacity:.08,disabled:"rgba(0, 0, 0, 0.26)",disabledBackground:"rgba(0, 0, 0, 0.12)",disabledOpacity:.38,focus:"rgba(0, 0, 0, 0.12)",focusOpacity:.12,activatedOpacity:.12}},om={text:{primary:Wl.white,secondary:"rgba(255, 255, 255, 0.7)",disabled:"rgba(255, 255, 255, 0.5)",icon:"rgba(255, 255, 255, 0.5)"},divider:"rgba(255, 255, 255, 0.12)",background:{paper:"#121212",default:"#121212"},action:{active:Wl.white,hover:"rgba(255, 255, 255, 0.08)",hoverOpacity:.08,selected:"rgba(255, 255, 255, 0.16)",selectedOpacity:.16,disabled:"rgba(255, 255, 255, 0.3)",disabledBackground:"rgba(255, 255, 255, 0.12)",disabledOpacity:.38,focus:"rgba(255, 255, 255, 0.12)",focusOpacity:.12,activatedOpacity:.24}};function Qx(t,e,n,r){const i=r.light||r,o=r.dark||r*1.5;t[e]||(t.hasOwnProperty(n)?t[e]=t[n]:e==="light"?t.light=fx(t.main,i):e==="dark"&&(t.dark=ux(t.main,o)))}function qA(t="light"){return t==="dark"?{main:Fo[200],light:Fo[50],dark:Fo[400]}:{main:Fo[700],light:Fo[400],dark:Fo[800]}}function zA(t="light"){return t==="dark"?{main:zo[200],light:zo[50],dark:zo[400]}:{main:zo[500],light:zo[300],dark:zo[700]}}function FA(t="light"){return t==="dark"?{main:qo[500],light:qo[300],dark:qo[700]}:{main:qo[700],light:qo[400],dark:qo[800]}}function VA(t="light"){return t==="dark"?{main:Vo[400],light:Vo[300],dark:Vo[700]}:{main:Vo[700],light:Vo[500],dark:Vo[900]}}function UA(t="light"){return t==="dark"?{main:Uo[400],light:Uo[300],dark:Uo[700]}:{main:Uo[800],light:Uo[500],dark:Uo[900]}}function jA(t="light"){return t==="dark"?{main:Ql[400],light:Ql[300],dark:Ql[700]}:{main:"#ed6c02",light:Ql[500],dark:Ql[900]}}function XA(t){const{mode:e="light",contrastThreshold:n=3,tonalOffset:r=.2}=t,i=H(t,BA),o=t.primary||qA(e),s=t.secondary||zA(e),l=t.error||FA(e),a=t.info||VA(e),c=t.success||UA(e),u=t.warning||jA(e);function f(m){return HE(m,om.text.primary)>=n?om.text.primary:Wx.text.primary}const h=({color:m,name:g,mainShade:y=500,lightShade:O=300,darkShade:v=700})=>{if(m=$({},m),!m.main&&m[y]&&(m.main=m[y]),!m.hasOwnProperty("main"))throw new Error(Ul(11,g?` (${g})`:"",y));if(typeof m.main!="string")throw new Error(Ul(12,g?` (${g})`:"",JSON.stringify(m.main)));return Qx(m,"light",O,r),Qx(m,"dark",v,r),m.contrastText||(m.contrastText=f(m.main)),m},d={dark:om,light:Wx};return vr($({common:Wl,mode:e,primary:h({color:o,name:"primary"}),secondary:h({color:s,name:"secondary",mainShade:"A400",lightShade:"A200",darkShade:"A700"}),error:h({color:l,name:"error"}),warning:h({color:u,name:"warning"}),info:h({color:a,name:"info"}),success:h({color:c,name:"success"}),grey:q2,contrastThreshold:n,getContrastText:f,augmentColor:h,tonalOffset:r},d[e]),i)}const HA=["fontFamily","fontSize","fontWeightLight","fontWeightRegular","fontWeightMedium","fontWeightBold","htmlFontSize","allVariants","pxToRem"];function GA(t){return Math.round(t*1e5)/1e5}const Nx={textTransform:"uppercase"},_x='"Roboto", "Helvetica", "Arial", sans-serif';function ZA(t,e){const n=typeof e=="function"?e(t):e,{fontFamily:r=_x,fontSize:i=14,fontWeightLight:o=300,fontWeightRegular:s=400,fontWeightMedium:l=500,fontWeightBold:a=700,htmlFontSize:c=16,allVariants:u,pxToRem:f}=n,h=H(n,HA),d=i/14,p=f||(y=>`${y/c*d}rem`),m=(y,O,v,x,b)=>$({fontFamily:r,fontWeight:y,fontSize:p(O),lineHeight:v},r===_x?{letterSpacing:`${GA(x/O)}em`}:{},b,u),g={h1:m(o,96,1.167,-1.5),h2:m(o,60,1.2,-.5),h3:m(s,48,1.167,0),h4:m(s,34,1.235,.25),h5:m(s,24,1.334,0),h6:m(l,20,1.6,.15),subtitle1:m(s,16,1.75,.15),subtitle2:m(l,14,1.57,.1),body1:m(s,16,1.5,.15),body2:m(s,14,1.43,.15),button:m(l,14,1.75,.4,Nx),caption:m(s,12,1.66,.4),overline:m(s,12,2.66,1,Nx)};return vr($({htmlFontSize:c,pxToRem:p,fontFamily:r,fontSize:i,fontWeightLight:o,fontWeightRegular:s,fontWeightMedium:l,fontWeightBold:a},g),h,{clone:!1})}const YA=.2,KA=.14,JA=.12;function ze(...t){return[`${t[0]}px ${t[1]}px ${t[2]}px ${t[3]}px rgba(0,0,0,${YA})`,`${t[4]}px ${t[5]}px ${t[6]}px ${t[7]}px rgba(0,0,0,${KA})`,`${t[8]}px ${t[9]}px ${t[10]}px ${t[11]}px rgba(0,0,0,${JA})`].join(",")}const eD=["none",ze(0,2,1,-1,0,1,1,0,0,1,3,0),ze(0,3,1,-2,0,2,2,0,0,1,5,0),ze(0,3,3,-2,0,3,4,0,0,1,8,0),ze(0,2,4,-1,0,4,5,0,0,1,10,0),ze(0,3,5,-1,0,5,8,0,0,1,14,0),ze(0,3,5,-1,0,6,10,0,0,1,18,0),ze(0,4,5,-2,0,7,10,1,0,2,16,1),ze(0,5,5,-3,0,8,10,1,0,3,14,2),ze(0,5,6,-3,0,9,12,1,0,3,16,2),ze(0,6,6,-3,0,10,14,1,0,4,18,3),ze(0,6,7,-4,0,11,15,1,0,4,20,3),ze(0,7,8,-4,0,12,17,2,0,5,22,4),ze(0,7,8,-4,0,13,19,2,0,5,24,4),ze(0,7,9,-4,0,14,21,2,0,5,26,4),ze(0,8,9,-5,0,15,22,2,0,6,28,5),ze(0,8,10,-5,0,16,24,2,0,6,30,5),ze(0,8,11,-5,0,17,26,2,0,6,32,5),ze(0,9,11,-5,0,18,28,2,0,7,34,6),ze(0,9,12,-6,0,19,29,2,0,7,36,6),ze(0,10,13,-6,0,20,31,3,0,8,38,7),ze(0,10,13,-6,0,21,33,3,0,8,40,7),ze(0,10,14,-6,0,22,35,3,0,8,42,7),ze(0,11,14,-7,0,23,36,3,0,9,44,8),ze(0,11,15,-7,0,24,38,3,0,9,46,8)];var tD=eD;const nD=["duration","easing","delay"],sm={easeInOut:"cubic-bezier(0.4, 0, 0.2, 1)",easeOut:"cubic-bezier(0.0, 0, 0.2, 1)",easeIn:"cubic-bezier(0.4, 0, 1, 1)",sharp:"cubic-bezier(0.4, 0, 0.6, 1)"},ci={shortest:150,shorter:200,short:250,standard:300,complex:375,enteringScreen:225,leavingScreen:195};function Bx(t){return`${Math.round(t)}ms`}function rD(t){if(!t)return 0;const e=t/36;return Math.round((4+15*e**.25+e/5)*10)}function iD(t){const e=$({},sm,t.easing),n=$({},ci,t.duration);return $({getAutoHeightDuration:rD,create:(i=["all"],o={})=>{const{duration:s=n.standard,easing:l=e.easeInOut,delay:a=0}=o;return H(o,nD),(Array.isArray(i)?i:[i]).map(c=>`${c} ${typeof s=="string"?s:Bx(s)} ${l} ${typeof a=="string"?a:Bx(a)}`).join(",")}},t,{easing:e,duration:n})}const oD={mobileStepper:1e3,speedDial:1050,appBar:1100,drawer:1200,modal:1300,snackbar:1400,tooltip:1500};var sD=oD;const lD=["breakpoints","mixins","spacing","palette","transitions","typography","shape"];function qx(t={},...e){const{mixins:n={},palette:r={},transitions:i={},typography:o={}}=t,s=H(t,lD),l=XA(r),a=Xp(t);let c=vr(a,{mixins:_A(a.breakpoints,a.spacing,n),palette:l,shadows:tD.slice(),typography:ZA(l,o),transitions:iD(i),zIndex:$({},sD)});return c=vr(c,s),c=e.reduce((u,f)=>vr(u,f),c),c}const aD=qx();var Eu=aD;function br(){return $u(Eu)}function fe({props:t,name:e}){return UE({props:t,name:e,defaultTheme:Eu})}const ts=t=>Su(t)&&t!=="classes",cD=VE({defaultTheme:Eu,rootShouldForwardProp:ts});var G=cD;function uD(t){return he("MuiSvgIcon",t)}de("MuiSvgIcon",["root","colorPrimary","colorSecondary","colorAction","colorError","colorDisabled","fontSizeInherit","fontSizeSmall","fontSizeMedium","fontSizeLarge"]);const fD=["children","className","color","component","fontSize","htmlColor","inheritViewBox","titleAccess","viewBox"],hD=t=>{const{color:e,fontSize:n,classes:r}=t,i={root:["root",e!=="inherit"&&`color${V(e)}`,`fontSize${V(n)}`]};return me(i,uD,r)},dD=G("svg",{name:"MuiSvgIcon",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.color!=="inherit"&&e[`color${V(n.color)}`],e[`fontSize${V(n.fontSize)}`]]}})(({theme:t,ownerState:e})=>{var n,r;return{userSelect:"none",width:"1em",height:"1em",display:"inline-block",fill:"currentColor",flexShrink:0,transition:t.transitions.create("fill",{duration:t.transitions.duration.shorter}),fontSize:{inherit:"inherit",small:t.typography.pxToRem(20),medium:t.typography.pxToRem(24),large:t.typography.pxToRem(35)}[e.fontSize],color:(n=(r=t.palette[e.color])==null?void 0:r.main)!=null?n:{action:t.palette.action.active,disabled:t.palette.action.disabled,inherit:void 0}[e.color]}}),zx=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiSvgIcon"}),{children:i,className:o,color:s="inherit",component:l="svg",fontSize:a="medium",htmlColor:c,inheritViewBox:u=!1,titleAccess:f,viewBox:h="0 0 24 24"}=r,d=H(r,fD),p=$({},r,{color:s,component:l,fontSize:a,inheritViewBox:u,viewBox:h}),m={};u||(m.viewBox=h);const g=hD(p);return Ue(dD,$({as:l,className:j(g.root,o),ownerState:p,focusable:"false",color:c,"aria-hidden":f?void 0:!0,role:f?"img":void 0,ref:n},m,d,{children:[i,f?I("title",{children:f}):null]}))});zx.muiName="SvgIcon";var Fx=zx;function Ne(t,e){const n=(r,i)=>I(Fx,$({"data-testid":`${e}Icon`,ref:i},r,{children:t}));return n.muiName=Fx.muiName,w.exports.memo(w.exports.forwardRef(n))}function lm(t,e){return lm=Object.setPrototypeOf||function(r,i){return r.__proto__=i,r},lm(t,e)}function Vx(t,e){t.prototype=Object.create(e.prototype),t.prototype.constructor=t,lm(t,e)}var Ux={disabled:!1},Mu=In.createContext(null),sa="unmounted",Vi="exited",Ui="entering",ns="entered",am="exiting",$r=function(t){Vx(e,t);function e(r,i){var o;o=t.call(this,r,i)||this;var s=i,l=s&&!s.isMounting?r.enter:r.appear,a;return o.appearStatus=null,r.in?l?(a=Vi,o.appearStatus=Ui):a=ns:r.unmountOnExit||r.mountOnEnter?a=sa:a=Vi,o.state={status:a},o.nextCallback=null,o}e.getDerivedStateFromProps=function(i,o){var s=i.in;return s&&o.status===sa?{status:Vi}:null};var n=e.prototype;return n.componentDidMount=function(){this.updateStatus(!0,this.appearStatus)},n.componentDidUpdate=function(i){var o=null;if(i!==this.props){var s=this.state.status;this.props.in?s!==Ui&&s!==ns&&(o=Ui):(s===Ui||s===ns)&&(o=am)}this.updateStatus(!1,o)},n.componentWillUnmount=function(){this.cancelNextCallback()},n.getTimeouts=function(){var i=this.props.timeout,o,s,l;return o=s=l=i,i!=null&&typeof i!="number"&&(o=i.exit,s=i.enter,l=i.appear!==void 0?i.appear:s),{exit:o,enter:s,appear:l}},n.updateStatus=function(i,o){i===void 0&&(i=!1),o!==null?(this.cancelNextCallback(),o===Ui?this.performEnter(i):this.performExit()):this.props.unmountOnExit&&this.state.status===Vi&&this.setState({status:sa})},n.performEnter=function(i){var o=this,s=this.props.enter,l=this.context?this.context.isMounting:i,a=this.props.nodeRef?[l]:[kp.findDOMNode(this),l],c=a[0],u=a[1],f=this.getTimeouts(),h=l?f.appear:f.enter;if(!i&&!s||Ux.disabled){this.safeSetState({status:ns},function(){o.props.onEntered(c)});return}this.props.onEnter(c,u),this.safeSetState({status:Ui},function(){o.props.onEntering(c,u),o.onTransitionEnd(h,function(){o.safeSetState({status:ns},function(){o.props.onEntered(c,u)})})})},n.performExit=function(){var i=this,o=this.props.exit,s=this.getTimeouts(),l=this.props.nodeRef?void 0:kp.findDOMNode(this);if(!o||Ux.disabled){this.safeSetState({status:Vi},function(){i.props.onExited(l)});return}this.props.onExit(l),this.safeSetState({status:am},function(){i.props.onExiting(l),i.onTransitionEnd(s.exit,function(){i.safeSetState({status:Vi},function(){i.props.onExited(l)})})})},n.cancelNextCallback=function(){this.nextCallback!==null&&(this.nextCallback.cancel(),this.nextCallback=null)},n.safeSetState=function(i,o){o=this.setNextCallback(o),this.setState(i,o)},n.setNextCallback=function(i){var o=this,s=!0;return this.nextCallback=function(l){s&&(s=!1,o.nextCallback=null,i(l))},this.nextCallback.cancel=function(){s=!1},this.nextCallback},n.onTransitionEnd=function(i,o){this.setNextCallback(o);var s=this.props.nodeRef?this.props.nodeRef.current:kp.findDOMNode(this),l=i==null&&!this.props.addEndListener;if(!s||l){setTimeout(this.nextCallback,0);return}if(this.props.addEndListener){var a=this.props.nodeRef?[this.nextCallback]:[s,this.nextCallback],c=a[0],u=a[1];this.props.addEndListener(c,u)}i!=null&&setTimeout(this.nextCallback,i)},n.render=function(){var i=this.state.status;if(i===sa)return null;var o=this.props,s=o.children;o.in,o.mountOnEnter,o.unmountOnExit,o.appear,o.enter,o.exit,o.timeout,o.addEndListener,o.onEnter,o.onEntering,o.onEntered,o.onExit,o.onExiting,o.onExited,o.nodeRef;var l=H(o,["children","in","mountOnEnter","unmountOnExit","appear","enter","exit","timeout","addEndListener","onEnter","onEntering","onEntered","onExit","onExiting","onExited","nodeRef"]);return In.createElement(Mu.Provider,{value:null},typeof s=="function"?s(i,l):In.cloneElement(In.Children.only(s),l))},e}(In.Component);$r.contextType=Mu;$r.propTypes={};function rs(){}$r.defaultProps={in:!1,mountOnEnter:!1,unmountOnExit:!1,appear:!1,enter:!0,exit:!0,onEnter:rs,onEntering:rs,onEntered:rs,onExit:rs,onExiting:rs,onExited:rs};$r.UNMOUNTED=sa;$r.EXITED=Vi;$r.ENTERING=Ui;$r.ENTERED=ns;$r.EXITING=am;var Au=$r;function pD(t){if(t===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}function cm(t,e){var n=function(o){return e&&w.exports.isValidElement(o)?e(o):o},r=Object.create(null);return t&&w.exports.Children.map(t,function(i){return i}).forEach(function(i){r[i.key]=n(i)}),r}function mD(t,e){t=t||{},e=e||{};function n(u){return u in e?e[u]:t[u]}var r=Object.create(null),i=[];for(var o in t)o in e?i.length&&(r[o]=i,i=[]):i.push(o);var s,l={};for(var a in e){if(r[a])for(s=0;s<r[a].length;s++){var c=r[a][s];l[r[a][s]]=n(c)}l[a]=n(a)}for(s=0;s<i.length;s++)l[i[s]]=n(i[s]);return l}function ji(t,e,n){return n[e]!=null?n[e]:t.props[e]}function gD(t,e){return cm(t.children,function(n){return w.exports.cloneElement(n,{onExited:e.bind(null,n),in:!0,appear:ji(n,"appear",t),enter:ji(n,"enter",t),exit:ji(n,"exit",t)})})}function yD(t,e,n){var r=cm(t.children),i=mD(e,r);return Object.keys(i).forEach(function(o){var s=i[o];if(!!w.exports.isValidElement(s)){var l=o in e,a=o in r,c=e[o],u=w.exports.isValidElement(c)&&!c.props.in;a&&(!l||u)?i[o]=w.exports.cloneElement(s,{onExited:n.bind(null,s),in:!0,exit:ji(s,"exit",t),enter:ji(s,"enter",t)}):!a&&l&&!u?i[o]=w.exports.cloneElement(s,{in:!1}):a&&l&&w.exports.isValidElement(c)&&(i[o]=w.exports.cloneElement(s,{onExited:n.bind(null,s),in:c.props.in,exit:ji(s,"exit",t),enter:ji(s,"enter",t)}))}}),i}var OD=Object.values||function(t){return Object.keys(t).map(function(e){return t[e]})},vD={component:"div",childFactory:function(e){return e}},um=function(t){Vx(e,t);function e(r,i){var o;o=t.call(this,r,i)||this;var s=o.handleExited.bind(pD(o));return o.state={contextValue:{isMounting:!0},handleExited:s,firstRender:!0},o}var n=e.prototype;return n.componentDidMount=function(){this.mounted=!0,this.setState({contextValue:{isMounting:!1}})},n.componentWillUnmount=function(){this.mounted=!1},e.getDerivedStateFromProps=function(i,o){var s=o.children,l=o.handleExited,a=o.firstRender;return{children:a?gD(i,l):yD(i,s,l),firstRender:!1}},n.handleExited=function(i,o){var s=cm(this.props.children);i.key in s||(i.props.onExited&&i.props.onExited(o),this.mounted&&this.setState(function(l){var a=$({},l.children);return delete a[i.key],{children:a}}))},n.render=function(){var i=this.props,o=i.component,s=i.childFactory,l=H(i,["component","childFactory"]),a=this.state.contextValue,c=OD(this.state.children).map(s);return delete l.appear,delete l.enter,delete l.exit,o===null?In.createElement(Mu.Provider,{value:a},c):In.createElement(Mu.Provider,{value:a},In.createElement(o,l,c))},e}(In.Component);um.propTypes={};um.defaultProps=vD;var xD=um;const fm=t=>t.scrollTop;function ui(t,e){var n,r;const{timeout:i,easing:o,style:s={}}=t;return{duration:(n=s.transitionDuration)!=null?n:typeof i=="number"?i:i[e.mode]||0,easing:(r=s.transitionTimingFunction)!=null?r:typeof o=="object"?o[e.mode]:o,delay:s.transitionDelay}}function wD(t){return he("MuiCollapse",t)}de("MuiCollapse",["root","horizontal","vertical","entered","hidden","wrapper","wrapperInner"]);const bD=["addEndListener","children","className","collapsedSize","component","easing","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","orientation","style","timeout","TransitionComponent"],$D=t=>{const{orientation:e,classes:n}=t,r={root:["root",`${e}`],entered:["entered"],hidden:["hidden"],wrapper:["wrapper",`${e}`],wrapperInner:["wrapperInner",`${e}`]};return me(r,wD,n)},SD=G("div",{name:"MuiCollapse",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,e[n.orientation],n.state==="entered"&&e.entered,n.state==="exited"&&!n.in&&n.collapsedSize==="0px"&&e.hidden]}})(({theme:t,ownerState:e})=>$({height:0,overflow:"hidden",transition:t.transitions.create("height")},e.orientation==="horizontal"&&{height:"auto",width:0,transition:t.transitions.create("width")},e.state==="entered"&&$({height:"auto",overflow:"visible"},e.orientation==="horizontal"&&{width:"auto"}),e.state==="exited"&&!e.in&&e.collapsedSize==="0px"&&{visibility:"hidden"})),kD=G("div",{name:"MuiCollapse",slot:"Wrapper",overridesResolver:(t,e)=>e.wrapper})(({ownerState:t})=>$({display:"flex",width:"100%"},t.orientation==="horizontal"&&{width:"auto",height:"100%"})),PD=G("div",{name:"MuiCollapse",slot:"WrapperInner",overridesResolver:(t,e)=>e.wrapperInner})(({ownerState:t})=>$({width:"100%"},t.orientation==="horizontal"&&{width:"auto",height:"100%"})),jx=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiCollapse"}),{addEndListener:i,children:o,className:s,collapsedSize:l="0px",component:a,easing:c,in:u,onEnter:f,onEntered:h,onEntering:d,onExit:p,onExited:m,onExiting:g,orientation:y="vertical",style:O,timeout:v=ci.standard,TransitionComponent:x=Au}=r,b=H(r,bD),S=$({},r,{orientation:y,collapsedSize:l}),k=$D(S),P=br(),C=w.exports.useRef(),R=w.exports.useRef(null),T=w.exports.useRef(),L=typeof l=="number"?`${l}px`:l,M=y==="horizontal",q=M?"width":"height";w.exports.useEffect(()=>()=>{clearTimeout(C.current)},[]);const F=w.exports.useRef(null),Q=Qe(n,F),E=J=>ue=>{if(J){const se=F.current;ue===void 0?J(se):J(se,ue)}},D=()=>R.current?R.current[M?"clientWidth":"clientHeight"]:0,X=E((J,ue)=>{R.current&&M&&(R.current.style.position="absolute"),J.style[q]=L,f&&f(J,ue)}),ae=E((J,ue)=>{const se=D();R.current&&M&&(R.current.style.position="");const{duration:we,easing:Be}=ui({style:O,timeout:v,easing:c},{mode:"enter"});if(v==="auto"){const Ot=P.transitions.getAutoHeightDuration(se);J.style.transitionDuration=`${Ot}ms`,T.current=Ot}else J.style.transitionDuration=typeof we=="string"?we:`${we}ms`;J.style[q]=`${se}px`,J.style.transitionTimingFunction=Be,d&&d(J,ue)}),W=E((J,ue)=>{J.style[q]="auto",h&&h(J,ue)}),B=E(J=>{J.style[q]=`${D()}px`,p&&p(J)}),Z=E(m),ee=E(J=>{const ue=D(),{duration:se,easing:we}=ui({style:O,timeout:v,easing:c},{mode:"exit"});if(v==="auto"){const Be=P.transitions.getAutoHeightDuration(ue);J.style.transitionDuration=`${Be}ms`,T.current=Be}else J.style.transitionDuration=typeof se=="string"?se:`${se}ms`;J.style[q]=L,J.style.transitionTimingFunction=we,g&&g(J)});return I(x,$({in:u,onEnter:X,onEntered:W,onEntering:ae,onExit:B,onExited:Z,onExiting:ee,addEndListener:J=>{v==="auto"&&(C.current=setTimeout(J,T.current||0)),i&&i(F.current,J)},nodeRef:F,timeout:v==="auto"?null:v},b,{children:(J,ue)=>I(SD,$({as:a,className:j(k.root,s,{entered:k.entered,exited:!u&&L==="0px"&&k.hidden}[J]),style:$({[M?"minWidth":"minHeight"]:L},O),ownerState:$({},S,{state:J}),ref:Q},ue,{children:I(kD,{ownerState:$({},S,{state:J}),className:k.wrapper,ref:R,children:I(PD,{ownerState:$({},S,{state:J}),className:k.wrapperInner,children:o})})}))}))});jx.muiSupportAuto=!0;var CD=jx;function RD(t){return he("MuiPaper",t)}de("MuiPaper",["root","rounded","outlined","elevation","elevation0","elevation1","elevation2","elevation3","elevation4","elevation5","elevation6","elevation7","elevation8","elevation9","elevation10","elevation11","elevation12","elevation13","elevation14","elevation15","elevation16","elevation17","elevation18","elevation19","elevation20","elevation21","elevation22","elevation23","elevation24"]);const TD=["className","component","elevation","square","variant"],Xx=t=>{let e;return t<1?e=5.11916*t**2:e=4.5*Math.log(t+1)+2,(e/100).toFixed(2)},ID=t=>{const{square:e,elevation:n,variant:r,classes:i}=t,o={root:["root",r,!e&&"rounded",r==="elevation"&&`elevation${n}`]};return me(o,RD,i)},ED=G("div",{name:"MuiPaper",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,e[n.variant],!n.square&&e.rounded,n.variant==="elevation"&&e[`elevation${n.elevation}`]]}})(({theme:t,ownerState:e})=>$({backgroundColor:t.palette.background.paper,color:t.palette.text.primary,transition:t.transitions.create("box-shadow")},!e.square&&{borderRadius:t.shape.borderRadius},e.variant==="outlined"&&{border:`1px solid ${t.palette.divider}`},e.variant==="elevation"&&$({boxShadow:t.shadows[e.elevation]},t.palette.mode==="dark"&&{backgroundImage:`linear-gradient(${ge("#fff",Xx(e.elevation))}, ${ge("#fff",Xx(e.elevation))})`}))),MD=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiPaper"}),{className:i,component:o="div",elevation:s=1,square:l=!1,variant:a="elevation"}=r,c=H(r,TD),u=$({},r,{component:o,elevation:s,square:l,variant:a}),f=ID(u);return I(ED,$({as:o,ownerState:u,className:j(f.root,i),ref:n},c))});var is=MD;const AD=w.exports.createContext({});var Hx=AD;function DD(t){return he("MuiAccordion",t)}const LD=de("MuiAccordion",["root","rounded","expanded","disabled","gutters","region"]);var Du=LD;const WD=["children","className","defaultExpanded","disabled","disableGutters","expanded","onChange","square","TransitionComponent","TransitionProps"],QD=t=>{const{classes:e,square:n,expanded:r,disabled:i,disableGutters:o}=t;return me({root:["root",!n&&"rounded",r&&"expanded",i&&"disabled",!o&&"gutters"],region:["region"]},DD,e)},ND=G(is,{name:"MuiAccordion",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[{[`& .${Du.region}`]:e.region},e.root,!n.square&&e.rounded,!n.disableGutters&&e.gutters]}})(({theme:t})=>{const e={duration:t.transitions.duration.shortest};return{position:"relative",transition:t.transitions.create(["margin"],e),overflowAnchor:"none","&:before":{position:"absolute",left:0,top:-1,right:0,height:1,content:'""',opacity:1,backgroundColor:t.palette.divider,transition:t.transitions.create(["opacity","background-color"],e)},"&:first-of-type":{"&:before":{display:"none"}},[`&.${Du.expanded}`]:{"&:before":{opacity:0},"&:first-of-type":{marginTop:0},"&:last-of-type":{marginBottom:0},"& + &":{"&:before":{display:"none"}}},[`&.${Du.disabled}`]:{backgroundColor:t.palette.action.disabledBackground}}},({theme:t,ownerState:e})=>$({},!e.square&&{borderRadius:0,"&:first-of-type":{borderTopLeftRadius:t.shape.borderRadius,borderTopRightRadius:t.shape.borderRadius},"&:last-of-type":{borderBottomLeftRadius:t.shape.borderRadius,borderBottomRightRadius:t.shape.borderRadius,"@supports (-ms-ime-align: auto)":{borderBottomLeftRadius:0,borderBottomRightRadius:0}}},!e.disableGutters&&{[`&.${Du.expanded}`]:{margin:"16px 0"}})),_D=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiAccordion"}),{children:i,className:o,defaultExpanded:s=!1,disabled:l=!1,disableGutters:a=!1,expanded:c,onChange:u,square:f=!1,TransitionComponent:h=CD,TransitionProps:d}=r,p=H(r,WD),[m,g]=D1({controlled:c,default:s,name:"Accordion",state:"expanded"}),y=w.exports.useCallback(k=>{g(!m),u&&u(k,!m)},[m,u,g]),[O,...v]=w.exports.Children.toArray(i),x=w.exports.useMemo(()=>({expanded:m,disabled:l,disableGutters:a,toggle:y}),[m,l,a,y]),b=$({},r,{square:f,disabled:l,disableGutters:a,expanded:m}),S=QD(b);return Ue(ND,$({className:j(S.root,o),ref:n,ownerState:b,square:f},p,{children:[I(Hx.Provider,{value:x,children:O}),I(h,$({in:m,timeout:"auto"},d,{children:I("div",{"aria-labelledby":O.props.id,id:O.props["aria-controls"],role:"region",className:S.region,children:v})}))]}))});var C7=_D;function BD(t){return he("MuiAccordionDetails",t)}de("MuiAccordionDetails",["root"]);const qD=["className"],zD=t=>{const{classes:e}=t;return me({root:["root"]},BD,e)},FD=G("div",{name:"MuiAccordionDetails",slot:"Root",overridesResolver:(t,e)=>e.root})(({theme:t})=>({padding:t.spacing(1,2,2)})),VD=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiAccordionDetails"}),{className:i}=r,o=H(r,qD),s=r,l=zD(s);return I(FD,$({className:j(l.root,i),ref:n,ownerState:s},o))});var R7=VD;function UD(t){const{className:e,classes:n,pulsate:r=!1,rippleX:i,rippleY:o,rippleSize:s,in:l,onExited:a,timeout:c}=t,[u,f]=w.exports.useState(!1),h=j(e,n.ripple,n.rippleVisible,r&&n.ripplePulsate),d={width:s,height:s,top:-(s/2)+o,left:-(s/2)+i},p=j(n.child,u&&n.childLeaving,r&&n.childPulsate);return!l&&!u&&f(!0),w.exports.useEffect(()=>{if(!l&&a!=null){const m=setTimeout(a,c);return()=>{clearTimeout(m)}}},[a,l,c]),I("span",{className:h,style:d,children:I("span",{className:p})})}const jD=de("MuiTouchRipple",["root","ripple","rippleVisible","ripplePulsate","child","childLeaving","childPulsate"]);var xn=jD;const XD=["center","classes","className"];let Lu=t=>t,Gx,Zx,Yx,Kx;const hm=550,HD=80,GD=Fl(Gx||(Gx=Lu`
  0% {
    transform: scale(0);
    opacity: 0.1;
  }

  100% {
    transform: scale(1);
    opacity: 0.3;
  }
`)),ZD=Fl(Zx||(Zx=Lu`
  0% {
    opacity: 1;
  }

  100% {
    opacity: 0;
  }
`)),YD=Fl(Yx||(Yx=Lu`
  0% {
    transform: scale(1);
  }

  50% {
    transform: scale(0.92);
  }

  100% {
    transform: scale(1);
  }
`)),KD=G("span",{name:"MuiTouchRipple",slot:"Root",skipSx:!0})({overflow:"hidden",pointerEvents:"none",position:"absolute",zIndex:0,top:0,right:0,bottom:0,left:0,borderRadius:"inherit"}),JD=G(UD,{name:"MuiTouchRipple",slot:"Ripple"})(Kx||(Kx=Lu`
  opacity: 0;
  position: absolute;

  &.${0} {
    opacity: 0.3;
    transform: scale(1);
    animation-name: ${0};
    animation-duration: ${0}ms;
    animation-timing-function: ${0};
  }

  &.${0} {
    animation-duration: ${0}ms;
  }

  & .${0} {
    opacity: 1;
    display: block;
    width: 100%;
    height: 100%;
    border-radius: 50%;
    background-color: currentColor;
  }

  & .${0} {
    opacity: 0;
    animation-name: ${0};
    animation-duration: ${0}ms;
    animation-timing-function: ${0};
  }

  & .${0} {
    position: absolute;
    /* @noflip */
    left: 0px;
    top: 0;
    animation-name: ${0};
    animation-duration: 2500ms;
    animation-timing-function: ${0};
    animation-iteration-count: infinite;
    animation-delay: 200ms;
  }
`),xn.rippleVisible,GD,hm,({theme:t})=>t.transitions.easing.easeInOut,xn.ripplePulsate,({theme:t})=>t.transitions.duration.shorter,xn.child,xn.childLeaving,ZD,hm,({theme:t})=>t.transitions.easing.easeInOut,xn.childPulsate,YD,({theme:t})=>t.transitions.easing.easeInOut),eL=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTouchRipple"}),{center:i=!1,classes:o={},className:s}=r,l=H(r,XD),[a,c]=w.exports.useState([]),u=w.exports.useRef(0),f=w.exports.useRef(null);w.exports.useEffect(()=>{f.current&&(f.current(),f.current=null)},[a]);const h=w.exports.useRef(!1),d=w.exports.useRef(null),p=w.exports.useRef(null),m=w.exports.useRef(null);w.exports.useEffect(()=>()=>{clearTimeout(d.current)},[]);const g=w.exports.useCallback(x=>{const{pulsate:b,rippleX:S,rippleY:k,rippleSize:P,cb:C}=x;c(R=>[...R,I(JD,{classes:{ripple:j(o.ripple,xn.ripple),rippleVisible:j(o.rippleVisible,xn.rippleVisible),ripplePulsate:j(o.ripplePulsate,xn.ripplePulsate),child:j(o.child,xn.child),childLeaving:j(o.childLeaving,xn.childLeaving),childPulsate:j(o.childPulsate,xn.childPulsate)},timeout:hm,pulsate:b,rippleX:S,rippleY:k,rippleSize:P},u.current)]),u.current+=1,f.current=C},[o]),y=w.exports.useCallback((x={},b={},S)=>{const{pulsate:k=!1,center:P=i||b.pulsate,fakeElement:C=!1}=b;if(x.type==="mousedown"&&h.current){h.current=!1;return}x.type==="touchstart"&&(h.current=!0);const R=C?null:m.current,T=R?R.getBoundingClientRect():{width:0,height:0,left:0,top:0};let L,M,q;if(P||x.clientX===0&&x.clientY===0||!x.clientX&&!x.touches)L=Math.round(T.width/2),M=Math.round(T.height/2);else{const{clientX:F,clientY:Q}=x.touches?x.touches[0]:x;L=Math.round(F-T.left),M=Math.round(Q-T.top)}if(P)q=Math.sqrt((2*T.width**2+T.height**2)/3),q%2==0&&(q+=1);else{const F=Math.max(Math.abs((R?R.clientWidth:0)-L),L)*2+2,Q=Math.max(Math.abs((R?R.clientHeight:0)-M),M)*2+2;q=Math.sqrt(F**2+Q**2)}x.touches?p.current===null&&(p.current=()=>{g({pulsate:k,rippleX:L,rippleY:M,rippleSize:q,cb:S})},d.current=setTimeout(()=>{p.current&&(p.current(),p.current=null)},HD)):g({pulsate:k,rippleX:L,rippleY:M,rippleSize:q,cb:S})},[i,g]),O=w.exports.useCallback(()=>{y({},{pulsate:!0})},[y]),v=w.exports.useCallback((x,b)=>{if(clearTimeout(d.current),x.type==="touchend"&&p.current){p.current(),p.current=null,d.current=setTimeout(()=>{v(x,b)});return}p.current=null,c(S=>S.length>0?S.slice(1):S),f.current=b},[]);return w.exports.useImperativeHandle(n,()=>({pulsate:O,start:y,stop:v}),[O,y,v]),I(KD,$({className:j(o.root,xn.root,s),ref:m},l,{children:I(xD,{component:null,exit:!0,children:a})}))});var tL=eL;function nL(t){return he("MuiButtonBase",t)}const rL=de("MuiButtonBase",["root","disabled","focusVisible"]);var iL=rL;const oL=["action","centerRipple","children","className","component","disabled","disableRipple","disableTouchRipple","focusRipple","focusVisibleClassName","LinkComponent","onBlur","onClick","onContextMenu","onDragLeave","onFocus","onFocusVisible","onKeyDown","onKeyUp","onMouseDown","onMouseLeave","onMouseUp","onTouchEnd","onTouchMove","onTouchStart","tabIndex","TouchRippleProps","type"],sL=t=>{const{disabled:e,focusVisible:n,focusVisibleClassName:r,classes:i}=t,s=me({root:["root",e&&"disabled",n&&"focusVisible"]},nL,i);return n&&r&&(s.root+=` ${r}`),s},lL=G("button",{name:"MuiButtonBase",slot:"Root",overridesResolver:(t,e)=>e.root})({display:"inline-flex",alignItems:"center",justifyContent:"center",position:"relative",boxSizing:"border-box",WebkitTapHighlightColor:"transparent",backgroundColor:"transparent",outline:0,border:0,margin:0,borderRadius:0,padding:0,cursor:"pointer",userSelect:"none",verticalAlign:"middle",MozAppearance:"none",WebkitAppearance:"none",textDecoration:"none",color:"inherit","&::-moz-focus-inner":{borderStyle:"none"},[`&.${iL.disabled}`]:{pointerEvents:"none",cursor:"default"},"@media print":{colorAdjust:"exact"}}),aL=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiButtonBase"}),{action:i,centerRipple:o=!1,children:s,className:l,component:a="button",disabled:c=!1,disableRipple:u=!1,disableTouchRipple:f=!1,focusRipple:h=!1,LinkComponent:d="a",onBlur:p,onClick:m,onContextMenu:g,onDragLeave:y,onFocus:O,onFocusVisible:v,onKeyDown:x,onKeyUp:b,onMouseDown:S,onMouseLeave:k,onMouseUp:P,onTouchEnd:C,onTouchMove:R,onTouchStart:T,tabIndex:L=0,TouchRippleProps:M,type:q}=r,F=H(r,oL),Q=w.exports.useRef(null),E=w.exports.useRef(null),{isFocusVisibleRef:D,onFocus:X,onBlur:ae,ref:W}=_p(),[B,Z]=w.exports.useState(!1);c&&B&&Z(!1),w.exports.useImperativeHandle(i,()=>({focusVisible:()=>{Z(!0),Q.current.focus()}}),[]),w.exports.useEffect(()=>{B&&h&&!u&&E.current.pulsate()},[u,h,B]);function ee(N,ie,ye=f){return Dn(Oe=>(ie&&ie(Oe),!ye&&E.current&&E.current[N](Oe),!0))}const pe=ee("start",S),J=ee("stop",g),ue=ee("stop",y),se=ee("stop",P),we=ee("stop",N=>{B&&N.preventDefault(),k&&k(N)}),Be=ee("start",T),Ot=ee("stop",C),Ht=ee("stop",R),Gt=ee("stop",N=>{ae(N),D.current===!1&&Z(!1),p&&p(N)},!1),Ge=Dn(N=>{Q.current||(Q.current=N.currentTarget),X(N),D.current===!0&&(Z(!0),v&&v(N)),O&&O(N)}),Et=()=>{const N=Q.current;return a&&a!=="button"&&!(N.tagName==="A"&&N.href)},ot=w.exports.useRef(!1),Ee=Dn(N=>{h&&!ot.current&&B&&E.current&&N.key===" "&&(ot.current=!0,E.current.stop(N,()=>{E.current.start(N)})),N.target===N.currentTarget&&Et()&&N.key===" "&&N.preventDefault(),x&&x(N),N.target===N.currentTarget&&Et()&&N.key==="Enter"&&!c&&(N.preventDefault(),m&&m(N))}),Wr=Dn(N=>{h&&N.key===" "&&E.current&&B&&!N.defaultPrevented&&(ot.current=!1,E.current.stop(N,()=>{E.current.pulsate(N)})),b&&b(N),m&&N.target===N.currentTarget&&Et()&&N.key===" "&&!N.defaultPrevented&&m(N)});let Zt=a;Zt==="button"&&(F.href||F.to)&&(Zt=d);const Ke={};Zt==="button"?(Ke.type=q===void 0?"button":q,Ke.disabled=c):(!F.href&&!F.to&&(Ke.role="button"),c&&(Ke["aria-disabled"]=c));const ft=Qe(W,Q),Pn=Qe(n,ft),[fr,un]=w.exports.useState(!1);w.exports.useEffect(()=>{un(!0)},[]);const Qr=fr&&!u&&!c,_n=$({},r,{centerRipple:o,component:a,disabled:c,disableRipple:u,disableTouchRipple:f,focusRipple:h,tabIndex:L,focusVisible:B}),Cn=sL(_n);return Ue(lL,$({as:Zt,className:j(Cn.root,l),ownerState:_n,onBlur:Gt,onClick:m,onContextMenu:J,onFocus:Ge,onKeyDown:Ee,onKeyUp:Wr,onMouseDown:pe,onMouseLeave:we,onMouseUp:se,onDragLeave:ue,onTouchEnd:Ot,onTouchMove:Ht,onTouchStart:Be,ref:Pn,tabIndex:c?-1:L,type:q},Ke,F,{children:[s,Qr?I(tL,$({ref:E,center:o},M)):null]}))});var Kn=aL;function cL(t){return he("MuiAccordionSummary",t)}const uL=de("MuiAccordionSummary",["root","expanded","focusVisible","disabled","gutters","contentGutters","content","expandIconWrapper"]);var os=uL;const fL=["children","className","expandIcon","focusVisibleClassName","onClick"],hL=t=>{const{classes:e,expanded:n,disabled:r,disableGutters:i}=t;return me({root:["root",n&&"expanded",r&&"disabled",!i&&"gutters"],focusVisible:["focusVisible"],content:["content",n&&"expanded",!i&&"contentGutters"],expandIconWrapper:["expandIconWrapper",n&&"expanded"]},cL,e)},dL=G(Kn,{name:"MuiAccordionSummary",slot:"Root",overridesResolver:(t,e)=>e.root})(({theme:t,ownerState:e})=>{const n={duration:t.transitions.duration.shortest};return $({display:"flex",minHeight:48,padding:t.spacing(0,2),transition:t.transitions.create(["min-height","background-color"],n),[`&.${os.focusVisible}`]:{backgroundColor:t.palette.action.focus},[`&.${os.disabled}`]:{opacity:t.palette.action.disabledOpacity},[`&:hover:not(.${os.disabled})`]:{cursor:"pointer"}},!e.disableGutters&&{[`&.${os.expanded}`]:{minHeight:64}})}),pL=G("div",{name:"MuiAccordionSummary",slot:"Content",overridesResolver:(t,e)=>e.content})(({theme:t,ownerState:e})=>$({display:"flex",flexGrow:1,margin:"12px 0"},!e.disableGutters&&{transition:t.transitions.create(["margin"],{duration:t.transitions.duration.shortest}),[`&.${os.expanded}`]:{margin:"20px 0"}})),mL=G("div",{name:"MuiAccordionSummary",slot:"ExpandIconWrapper",overridesResolver:(t,e)=>e.expandIconWrapper})(({theme:t})=>({display:"flex",color:t.palette.action.active,transform:"rotate(0deg)",transition:t.transitions.create("transform",{duration:t.transitions.duration.shortest}),[`&.${os.expanded}`]:{transform:"rotate(180deg)"}})),gL=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiAccordionSummary"}),{children:i,className:o,expandIcon:s,focusVisibleClassName:l,onClick:a}=r,c=H(r,fL),{disabled:u=!1,disableGutters:f,expanded:h,toggle:d}=w.exports.useContext(Hx),p=y=>{d&&d(y),a&&a(y)},m=$({},r,{expanded:h,disabled:u,disableGutters:f}),g=hL(m);return Ue(dL,$({focusRipple:!1,disableRipple:!0,disabled:u,component:"div","aria-expanded":h,className:j(g.root,o),focusVisibleClassName:j(g.focusVisible,l),onClick:p,ref:n,ownerState:m},c,{children:[I(pL,{className:g.content,ownerState:m,children:i}),s&&I(mL,{className:g.expandIconWrapper,ownerState:m,children:s})]}))});var T7=gL;function yL(t){return he("MuiIconButton",t)}const OL=de("MuiIconButton",["root","disabled","colorInherit","colorPrimary","colorSecondary","edgeStart","edgeEnd","sizeSmall","sizeMedium","sizeLarge"]);var vL=OL;const xL=["edge","children","className","color","disabled","disableFocusRipple","size"],wL=t=>{const{classes:e,disabled:n,color:r,edge:i,size:o}=t,s={root:["root",n&&"disabled",r!=="default"&&`color${V(r)}`,i&&`edge${V(i)}`,`size${V(o)}`]};return me(s,yL,e)},bL=G(Kn,{name:"MuiIconButton",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.color!=="default"&&e[`color${V(n.color)}`],n.edge&&e[`edge${V(n.edge)}`],e[`size${V(n.size)}`]]}})(({theme:t,ownerState:e})=>$({textAlign:"center",flex:"0 0 auto",fontSize:t.typography.pxToRem(24),padding:8,borderRadius:"50%",overflow:"visible",color:t.palette.action.active,transition:t.transitions.create("background-color",{duration:t.transitions.duration.shortest})},!e.disableRipple&&{"&:hover":{backgroundColor:ge(t.palette.action.active,t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}},e.edge==="start"&&{marginLeft:e.size==="small"?-3:-12},e.edge==="end"&&{marginRight:e.size==="small"?-3:-12}),({theme:t,ownerState:e})=>$({},e.color==="inherit"&&{color:"inherit"},e.color!=="inherit"&&e.color!=="default"&&$({color:t.palette[e.color].main},!e.disableRipple&&{"&:hover":{backgroundColor:ge(t.palette[e.color].main,t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}}}),e.size==="small"&&{padding:5,fontSize:t.typography.pxToRem(18)},e.size==="large"&&{padding:12,fontSize:t.typography.pxToRem(28)},{[`&.${vL.disabled}`]:{backgroundColor:"transparent",color:t.palette.action.disabled}})),$L=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiIconButton"}),{edge:i=!1,children:o,className:s,color:l="default",disabled:a=!1,disableFocusRipple:c=!1,size:u="medium"}=r,f=H(r,xL),h=$({},r,{edge:i,color:l,disabled:a,disableFocusRipple:c,size:u}),d=wL(h);return I(bL,$({className:j(d.root,s),centerRipple:!0,focusRipple:!c,disabled:a,ref:n,ownerState:h},f,{children:o}))});var I7=$L;function SL(t){return he("MuiTypography",t)}de("MuiTypography",["root","h1","h2","h3","h4","h5","h6","subtitle1","subtitle2","body1","body2","inherit","button","caption","overline","alignLeft","alignRight","alignCenter","alignJustify","noWrap","gutterBottom","paragraph"]);const kL=["align","className","component","gutterBottom","noWrap","paragraph","variant","variantMapping"],PL=t=>{const{align:e,gutterBottom:n,noWrap:r,paragraph:i,variant:o,classes:s}=t,l={root:["root",o,t.align!=="inherit"&&`align${V(e)}`,n&&"gutterBottom",r&&"noWrap",i&&"paragraph"]};return me(l,SL,s)},CL=G("span",{name:"MuiTypography",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.variant&&e[n.variant],n.align!=="inherit"&&e[`align${V(n.align)}`],n.noWrap&&e.noWrap,n.gutterBottom&&e.gutterBottom,n.paragraph&&e.paragraph]}})(({theme:t,ownerState:e})=>$({margin:0},e.variant&&t.typography[e.variant],e.align!=="inherit"&&{textAlign:e.align},e.noWrap&&{overflow:"hidden",textOverflow:"ellipsis",whiteSpace:"nowrap"},e.gutterBottom&&{marginBottom:"0.35em"},e.paragraph&&{marginBottom:16})),Jx={h1:"h1",h2:"h2",h3:"h3",h4:"h4",h5:"h5",h6:"h6",subtitle1:"h6",subtitle2:"h6",body1:"p",body2:"p",inherit:"p"},RL={primary:"primary.main",textPrimary:"text.primary",secondary:"secondary.main",textSecondary:"text.secondary",error:"error.main"},TL=t=>RL[t]||t,IL=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTypography"}),i=TL(r.color),o=jp($({},r,{color:i})),{align:s="inherit",className:l,component:a,gutterBottom:c=!1,noWrap:u=!1,paragraph:f=!1,variant:h="body1",variantMapping:d=Jx}=o,p=H(o,kL),m=$({},o,{align:s,color:i,className:l,component:a,gutterBottom:c,noWrap:u,paragraph:f,variant:h,variantMapping:d}),g=a||(f?"p":d[h]||Jx[h])||"span",y=PL(m);return I(CL,$({as:g,ref:n,ownerState:m,className:j(y.root,l)},p))});var Sr=IL;function EL(t){return he("MuiAppBar",t)}de("MuiAppBar",["root","positionFixed","positionAbsolute","positionSticky","positionStatic","positionRelative","colorDefault","colorPrimary","colorSecondary","colorInherit","colorTransparent"]);const ML=["className","color","enableColorOnDark","position"],AL=t=>{const{color:e,position:n,classes:r}=t,i={root:["root",`color${V(e)}`,`position${V(n)}`]};return me(i,EL,r)},DL=G(is,{name:"MuiAppBar",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,e[`position${V(n.position)}`],e[`color${V(n.color)}`]]}})(({theme:t,ownerState:e})=>{const n=t.palette.mode==="light"?t.palette.grey[100]:t.palette.grey[900];return $({display:"flex",flexDirection:"column",width:"100%",boxSizing:"border-box",flexShrink:0},e.position==="fixed"&&{position:"fixed",zIndex:t.zIndex.appBar,top:0,left:"auto",right:0,"@media print":{position:"absolute"}},e.position==="absolute"&&{position:"absolute",zIndex:t.zIndex.appBar,top:0,left:"auto",right:0},e.position==="sticky"&&{position:"sticky",zIndex:t.zIndex.appBar,top:0,left:"auto",right:0},e.position==="static"&&{position:"static"},e.position==="relative"&&{position:"relative"},e.color==="default"&&{backgroundColor:n,color:t.palette.getContrastText(n)},e.color&&e.color!=="default"&&e.color!=="inherit"&&e.color!=="transparent"&&{backgroundColor:t.palette[e.color].main,color:t.palette[e.color].contrastText},e.color==="inherit"&&{color:"inherit"},t.palette.mode==="dark"&&!e.enableColorOnDark&&{backgroundColor:null,color:null},e.color==="transparent"&&$({backgroundColor:"transparent",color:"inherit"},t.palette.mode==="dark"&&{backgroundImage:"none"}))}),LL=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiAppBar"}),{className:i,color:o="primary",enableColorOnDark:s=!1,position:l="fixed"}=r,a=H(r,ML),c=$({},r,{color:o,position:l,enableColorOnDark:s}),u=AL(c);return I(DL,$({square:!0,component:"header",ownerState:c,elevation:4,className:j(u.root,i,l==="fixed"&&"mui-fixed"),ref:n},a))});var E7=LL;const WL=w.exports.forwardRef(function(e,n){const r=Hp();return I(NA,$({direction:r==null?void 0:r.direction},e,{ref:n}))});var ew=WL,QL=Ne(I("path",{d:"M12 2C6.47 2 2 6.47 2 12s4.47 10 10 10 10-4.47 10-10S17.53 2 12 2zm5 13.59L15.59 17 12 13.41 8.41 17 7 15.59 10.59 12 7 8.41 8.41 7 12 10.59 15.59 7 17 8.41 13.41 12 17 15.59z"}),"Cancel");function NL(t){return he("MuiChip",t)}const _L=de("MuiChip",["root","sizeSmall","sizeMedium","colorPrimary","colorSecondary","disabled","clickable","clickableColorPrimary","clickableColorSecondary","deletable","deletableColorPrimary","deletableColorSecondary","outlined","filled","outlinedPrimary","outlinedSecondary","avatar","avatarSmall","avatarMedium","avatarColorPrimary","avatarColorSecondary","icon","iconSmall","iconMedium","iconColorPrimary","iconColorSecondary","label","labelSmall","labelMedium","deleteIcon","deleteIconSmall","deleteIconMedium","deleteIconColorPrimary","deleteIconColorSecondary","deleteIconOutlinedColorPrimary","deleteIconOutlinedColorSecondary","focusVisible"]);var $e=_L;const BL=["avatar","className","clickable","color","component","deleteIcon","disabled","icon","label","onClick","onDelete","onKeyDown","onKeyUp","size","variant"],qL=t=>{const{classes:e,disabled:n,size:r,color:i,onDelete:o,clickable:s,variant:l}=t,a={root:["root",l,n&&"disabled",`size${V(r)}`,`color${V(i)}`,s&&"clickable",s&&`clickableColor${V(i)}`,o&&"deletable",o&&`deletableColor${V(i)}`,`${l}${V(i)}`],label:["label",`label${V(r)}`],avatar:["avatar",`avatar${V(r)}`,`avatarColor${V(i)}`],icon:["icon",`icon${V(r)}`,`iconColor${V(i)}`],deleteIcon:["deleteIcon",`deleteIcon${V(r)}`,`deleteIconColor${V(i)}`,`deleteIconOutlinedColor${V(i)}`]};return me(a,NL,e)},zL=G("div",{name:"MuiChip",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t,{color:r,clickable:i,onDelete:o,size:s,variant:l}=n;return[{[`& .${$e.avatar}`]:e.avatar},{[`& .${$e.avatar}`]:e[`avatar${V(s)}`]},{[`& .${$e.avatar}`]:e[`avatarColor${V(r)}`]},{[`& .${$e.icon}`]:e.icon},{[`& .${$e.icon}`]:e[`icon${V(s)}`]},{[`& .${$e.icon}`]:e[`iconColor${V(r)}`]},{[`& .${$e.deleteIcon}`]:e.deleteIcon},{[`& .${$e.deleteIcon}`]:e[`deleteIcon${V(s)}`]},{[`& .${$e.deleteIcon}`]:e[`deleteIconColor${V(r)}`]},{[`& .${$e.deleteIcon}`]:e[`deleteIconOutlinedColor${V(r)}`]},e.root,e[`size${V(s)}`],e[`color${V(r)}`],i&&e.clickable,i&&r!=="default"&&e[`clickableColor${V(r)})`],o&&e.deletable,o&&r!=="default"&&e[`deletableColor${V(r)}`],e[l],l==="outlined"&&e[`outlined${V(r)}`]]}})(({theme:t,ownerState:e})=>{const n=ge(t.palette.text.primary,.26);return $({fontFamily:t.typography.fontFamily,fontSize:t.typography.pxToRem(13),display:"inline-flex",alignItems:"center",justifyContent:"center",height:32,color:t.palette.text.primary,backgroundColor:t.palette.action.selected,borderRadius:32/2,whiteSpace:"nowrap",transition:t.transitions.create(["background-color","box-shadow"]),cursor:"default",outline:0,textDecoration:"none",border:0,padding:0,verticalAlign:"middle",boxSizing:"border-box",[`&.${$e.disabled}`]:{opacity:t.palette.action.disabledOpacity,pointerEvents:"none"},[`& .${$e.avatar}`]:{marginLeft:5,marginRight:-6,width:24,height:24,color:t.palette.mode==="light"?t.palette.grey[700]:t.palette.grey[300],fontSize:t.typography.pxToRem(12)},[`& .${$e.avatarColorPrimary}`]:{color:t.palette.primary.contrastText,backgroundColor:t.palette.primary.dark},[`& .${$e.avatarColorSecondary}`]:{color:t.palette.secondary.contrastText,backgroundColor:t.palette.secondary.dark},[`& .${$e.avatarSmall}`]:{marginLeft:4,marginRight:-4,width:18,height:18,fontSize:t.typography.pxToRem(10)},[`& .${$e.icon}`]:$({color:t.palette.mode==="light"?t.palette.grey[700]:t.palette.grey[300],marginLeft:5,marginRight:-6},e.size==="small"&&{fontSize:18,marginLeft:4,marginRight:-4},e.color!=="default"&&{color:"inherit"}),[`& .${$e.deleteIcon}`]:$({WebkitTapHighlightColor:"transparent",color:n,fontSize:22,cursor:"pointer",margin:"0 5px 0 -6px","&:hover":{color:ge(n,.4)}},e.size==="small"&&{fontSize:16,marginRight:4,marginLeft:-4},e.color!=="default"&&{color:ge(t.palette[e.color].contrastText,.7),"&:hover, &:active":{color:t.palette[e.color].contrastText}})},e.size==="small"&&{height:24},e.color!=="default"&&{backgroundColor:t.palette[e.color].main,color:t.palette[e.color].contrastText},e.onDelete&&{[`&.${$e.focusVisible}`]:{backgroundColor:ge(t.palette.action.selected,t.palette.action.selectedOpacity+t.palette.action.focusOpacity)}},e.onDelete&&e.color!=="default"&&{[`&.${$e.focusVisible}`]:{backgroundColor:t.palette[e.color].dark}})},({theme:t,ownerState:e})=>$({},e.clickable&&{userSelect:"none",WebkitTapHighlightColor:"transparent",cursor:"pointer","&:hover":{backgroundColor:ge(t.palette.action.selected,t.palette.action.selectedOpacity+t.palette.action.hoverOpacity)},[`&.${$e.focusVisible}`]:{backgroundColor:ge(t.palette.action.selected,t.palette.action.selectedOpacity+t.palette.action.focusOpacity)},"&:active":{boxShadow:t.shadows[1]}},e.clickable&&e.color!=="default"&&{[`&:hover, &.${$e.focusVisible}`]:{backgroundColor:t.palette[e.color].dark}}),({theme:t,ownerState:e})=>$({},e.variant==="outlined"&&{backgroundColor:"transparent",border:`1px solid ${t.palette.mode==="light"?t.palette.grey[400]:t.palette.grey[700]}`,[`&.${$e.clickable}:hover`]:{backgroundColor:t.palette.action.hover},[`&.${$e.focusVisible}`]:{backgroundColor:t.palette.action.focus},[`& .${$e.avatar}`]:{marginLeft:4},[`& .${$e.avatarSmall}`]:{marginLeft:2},[`& .${$e.icon}`]:{marginLeft:4},[`& .${$e.iconSmall}`]:{marginLeft:2},[`& .${$e.deleteIcon}`]:{marginRight:5},[`& .${$e.deleteIconSmall}`]:{marginRight:3}},e.variant==="outlined"&&e.color!=="default"&&{color:t.palette[e.color].main,border:`1px solid ${ge(t.palette[e.color].main,.7)}`,[`&.${$e.clickable}:hover`]:{backgroundColor:ge(t.palette[e.color].main,t.palette.action.hoverOpacity)},[`&.${$e.focusVisible}`]:{backgroundColor:ge(t.palette[e.color].main,t.palette.action.focusOpacity)},[`& .${$e.deleteIcon}`]:{color:ge(t.palette[e.color].main,.7),"&:hover, &:active":{color:t.palette[e.color].main}}})),FL=G("span",{name:"MuiChip",slot:"Label",overridesResolver:(t,e)=>{const{ownerState:n}=t,{size:r}=n;return[e.label,e[`label${V(r)}`]]}})(({ownerState:t})=>$({overflow:"hidden",textOverflow:"ellipsis",paddingLeft:12,paddingRight:12,whiteSpace:"nowrap"},t.size==="small"&&{paddingLeft:8,paddingRight:8}));function tw(t){return t.key==="Backspace"||t.key==="Delete"}const VL=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiChip"}),{avatar:i,className:o,clickable:s,color:l="default",component:a,deleteIcon:c,disabled:u=!1,icon:f,label:h,onClick:d,onDelete:p,onKeyDown:m,onKeyUp:g,size:y="medium",variant:O="filled"}=r,v=H(r,BL),x=w.exports.useRef(null),b=Qe(x,n),S=D=>{D.stopPropagation(),p&&p(D)},k=D=>{D.currentTarget===D.target&&tw(D)&&D.preventDefault(),m&&m(D)},P=D=>{D.currentTarget===D.target&&(p&&tw(D)?p(D):D.key==="Escape"&&x.current&&x.current.blur()),g&&g(D)},C=s!==!1&&d?!0:s,R=y==="small",T=C||p?Kn:a||"div",L=$({},r,{component:T,disabled:u,size:y,color:l,onDelete:!!p,clickable:C,variant:O}),M=qL(L),q=T===Kn?$({component:a||"div",focusVisibleClassName:M.focusVisible},p&&{disableRipple:!0}):{};let F=null;if(p){const D=j(l!=="default"&&(O==="outlined"?M[`deleteIconOutlinedColor${V(l)}`]:M[`deleteIconColor${V(l)}`]),R&&M.deleteIconSmall);F=c&&w.exports.isValidElement(c)?w.exports.cloneElement(c,{className:j(c.props.className,M.deleteIcon,D),onClick:S}):I(QL,{className:j(M.deleteIcon,D),onClick:S})}let Q=null;i&&w.exports.isValidElement(i)&&(Q=w.exports.cloneElement(i,{className:j(M.avatar,i.props.className)}));let E=null;return f&&w.exports.isValidElement(f)&&(E=w.exports.cloneElement(f,{className:j(M.icon,f.props.className)})),Ue(zL,$({as:T,className:j(M.root,o),disabled:C&&u?!0:void 0,onClick:d,onKeyDown:k,onKeyUp:P,ref:b,ownerState:L},q,v,{children:[Q||E,I(FL,{className:j(M.label),ownerState:L,children:h}),F]}))});var M7=VL;const UL=["addEndListener","appear","children","easing","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","timeout","TransitionComponent"],jL={entering:{opacity:1},entered:{opacity:1}},XL={enter:ci.enteringScreen,exit:ci.leavingScreen},HL=w.exports.forwardRef(function(e,n){const{addEndListener:r,appear:i=!0,children:o,easing:s,in:l,onEnter:a,onEntered:c,onEntering:u,onExit:f,onExited:h,onExiting:d,style:p,timeout:m=XL,TransitionComponent:g=Au}=e,y=H(e,UL),O=br(),v=w.exports.useRef(null),x=Qe(o.ref,n),b=Qe(v,x),S=q=>F=>{if(q){const Q=v.current;F===void 0?q(Q):q(Q,F)}},k=S(u),P=S((q,F)=>{fm(q);const Q=ui({style:p,timeout:m,easing:s},{mode:"enter"});q.style.webkitTransition=O.transitions.create("opacity",Q),q.style.transition=O.transitions.create("opacity",Q),a&&a(q,F)}),C=S(c),R=S(d),T=S(q=>{const F=ui({style:p,timeout:m,easing:s},{mode:"exit"});q.style.webkitTransition=O.transitions.create("opacity",F),q.style.transition=O.transitions.create("opacity",F),f&&f(q)}),L=S(h);return I(g,$({appear:i,in:l,nodeRef:v,onEnter:P,onEntered:C,onEntering:k,onExit:T,onExited:L,onExiting:R,addEndListener:q=>{r&&r(v.current,q)},timeout:m},y,{children:(q,F)=>w.exports.cloneElement(o,$({style:$({opacity:0,visibility:q==="exited"&&!l?"hidden":void 0},jL[q],p,o.props.style),ref:b},F))}))});var GL=HL;const ZL=["children","components","componentsProps","className","invisible","open","transitionDuration","TransitionComponent"],YL=t=>{const{classes:e}=t;return e},KL=G("div",{name:"MuiBackdrop",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.invisible&&e.invisible]}})(({ownerState:t})=>$({position:"fixed",display:"flex",alignItems:"center",justifyContent:"center",right:0,bottom:0,top:0,left:0,backgroundColor:"rgba(0, 0, 0, 0.5)",WebkitTapHighlightColor:"transparent"},t.invisible&&{backgroundColor:"transparent"})),JL=w.exports.forwardRef(function(e,n){var r;const i=fe({props:e,name:"MuiBackdrop"}),{children:o,components:s={},componentsProps:l={},className:a,invisible:c=!1,open:u,transitionDuration:f,TransitionComponent:h=GL}=i,d=H(i,ZL),p=$({},i,{invisible:c}),m=YL(p);return I(h,$({in:u,timeout:f},d,{children:I(rM,{className:a,invisible:c,components:$({Root:KL},s),componentsProps:{root:$({},l.root,(!s.Root||!zi(s.Root))&&{ownerState:$({},(r=l.root)==null?void 0:r.ownerState)})},classes:m,ref:n,children:o})}))});var e5=JL;const t5=qx(),n5=LE({defaultTheme:t5,defaultClassName:"MuiBox-root",generateClassName:dx.generate});var A7=n5;function r5(t){return he("MuiButton",t)}const i5=de("MuiButton",["root","text","textInherit","textPrimary","textSecondary","outlined","outlinedInherit","outlinedPrimary","outlinedSecondary","contained","containedInherit","containedPrimary","containedSecondary","disableElevation","focusVisible","disabled","colorInherit","textSizeSmall","textSizeMedium","textSizeLarge","outlinedSizeSmall","outlinedSizeMedium","outlinedSizeLarge","containedSizeSmall","containedSizeMedium","containedSizeLarge","sizeMedium","sizeSmall","sizeLarge","fullWidth","startIcon","endIcon","iconSizeSmall","iconSizeMedium","iconSizeLarge"]);var Wu=i5;const o5=w.exports.createContext({});var s5=o5;const l5=["children","color","component","className","disabled","disableElevation","disableFocusRipple","endIcon","focusVisibleClassName","fullWidth","size","startIcon","type","variant"],a5=t=>{const{color:e,disableElevation:n,fullWidth:r,size:i,variant:o,classes:s}=t,l={root:["root",o,`${o}${V(e)}`,`size${V(i)}`,`${o}Size${V(i)}`,e==="inherit"&&"colorInherit",n&&"disableElevation",r&&"fullWidth"],label:["label"],startIcon:["startIcon",`iconSize${V(i)}`],endIcon:["endIcon",`iconSize${V(i)}`]},a=me(l,r5,s);return $({},s,a)},nw=t=>$({},t.size==="small"&&{"& > *:nth-of-type(1)":{fontSize:18}},t.size==="medium"&&{"& > *:nth-of-type(1)":{fontSize:20}},t.size==="large"&&{"& > *:nth-of-type(1)":{fontSize:22}}),c5=G(Kn,{shouldForwardProp:t=>ts(t)||t==="classes",name:"MuiButton",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,e[n.variant],e[`${n.variant}${V(n.color)}`],e[`size${V(n.size)}`],e[`${n.variant}Size${V(n.size)}`],n.color==="inherit"&&e.colorInherit,n.disableElevation&&e.disableElevation,n.fullWidth&&e.fullWidth]}})(({theme:t,ownerState:e})=>$({},t.typography.button,{minWidth:64,padding:"6px 16px",borderRadius:t.shape.borderRadius,transition:t.transitions.create(["background-color","box-shadow","border-color","color"],{duration:t.transitions.duration.short}),"&:hover":$({textDecoration:"none",backgroundColor:ge(t.palette.text.primary,t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}},e.variant==="text"&&e.color!=="inherit"&&{backgroundColor:ge(t.palette[e.color].main,t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}},e.variant==="outlined"&&e.color!=="inherit"&&{border:`1px solid ${t.palette[e.color].main}`,backgroundColor:ge(t.palette[e.color].main,t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:"transparent"}},e.variant==="contained"&&{backgroundColor:t.palette.grey.A100,boxShadow:t.shadows[4],"@media (hover: none)":{boxShadow:t.shadows[2],backgroundColor:t.palette.grey[300]}},e.variant==="contained"&&e.color!=="inherit"&&{backgroundColor:t.palette[e.color].dark,"@media (hover: none)":{backgroundColor:t.palette[e.color].main}}),"&:active":$({},e.variant==="contained"&&{boxShadow:t.shadows[8]}),[`&.${Wu.focusVisible}`]:$({},e.variant==="contained"&&{boxShadow:t.shadows[6]}),[`&.${Wu.disabled}`]:$({color:t.palette.action.disabled},e.variant==="outlined"&&{border:`1px solid ${t.palette.action.disabledBackground}`},e.variant==="outlined"&&e.color==="secondary"&&{border:`1px solid ${t.palette.action.disabled}`},e.variant==="contained"&&{color:t.palette.action.disabled,boxShadow:t.shadows[0],backgroundColor:t.palette.action.disabledBackground})},e.variant==="text"&&{padding:"6px 8px"},e.variant==="text"&&e.color!=="inherit"&&{color:t.palette[e.color].main},e.variant==="outlined"&&{padding:"5px 15px",border:`1px solid ${t.palette.mode==="light"?"rgba(0, 0, 0, 0.23)":"rgba(255, 255, 255, 0.23)"}`},e.variant==="outlined"&&e.color!=="inherit"&&{color:t.palette[e.color].main,border:`1px solid ${ge(t.palette[e.color].main,.5)}`},e.variant==="contained"&&{color:t.palette.getContrastText(t.palette.grey[300]),backgroundColor:t.palette.grey[300],boxShadow:t.shadows[2]},e.variant==="contained"&&e.color!=="inherit"&&{color:t.palette[e.color].contrastText,backgroundColor:t.palette[e.color].main},e.color==="inherit"&&{color:"inherit",borderColor:"currentColor"},e.size==="small"&&e.variant==="text"&&{padding:"4px 5px",fontSize:t.typography.pxToRem(13)},e.size==="large"&&e.variant==="text"&&{padding:"8px 11px",fontSize:t.typography.pxToRem(15)},e.size==="small"&&e.variant==="outlined"&&{padding:"3px 9px",fontSize:t.typography.pxToRem(13)},e.size==="large"&&e.variant==="outlined"&&{padding:"7px 21px",fontSize:t.typography.pxToRem(15)},e.size==="small"&&e.variant==="contained"&&{padding:"4px 10px",fontSize:t.typography.pxToRem(13)},e.size==="large"&&e.variant==="contained"&&{padding:"8px 22px",fontSize:t.typography.pxToRem(15)},e.fullWidth&&{width:"100%"}),({ownerState:t})=>t.disableElevation&&{boxShadow:"none","&:hover":{boxShadow:"none"},[`&.${Wu.focusVisible}`]:{boxShadow:"none"},"&:active":{boxShadow:"none"},[`&.${Wu.disabled}`]:{boxShadow:"none"}}),u5=G("span",{name:"MuiButton",slot:"StartIcon",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.startIcon,e[`iconSize${V(n.size)}`]]}})(({ownerState:t})=>$({display:"inherit",marginRight:8,marginLeft:-4},t.size==="small"&&{marginLeft:-2},nw(t))),f5=G("span",{name:"MuiButton",slot:"EndIcon",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.endIcon,e[`iconSize${V(n.size)}`]]}})(({ownerState:t})=>$({display:"inherit",marginRight:-4,marginLeft:8},t.size==="small"&&{marginRight:-2},nw(t))),h5=w.exports.forwardRef(function(e,n){const r=w.exports.useContext(s5),i=_1(r,e),o=fe({props:i,name:"MuiButton"}),{children:s,color:l="primary",component:a="button",className:c,disabled:u=!1,disableElevation:f=!1,disableFocusRipple:h=!1,endIcon:d,focusVisibleClassName:p,fullWidth:m=!1,size:g="medium",startIcon:y,type:O,variant:v="text"}=o,x=H(o,l5),b=$({},o,{color:l,component:a,disabled:u,disableElevation:f,disableFocusRipple:h,fullWidth:m,size:g,type:O,variant:v}),S=a5(b),k=y&&I(u5,{className:S.startIcon,ownerState:b,children:y}),P=d&&I(f5,{className:S.endIcon,ownerState:b,children:d});return Ue(c5,$({ownerState:b,className:j(c,r.className),component:a,disabled:u,focusRipple:!h,focusVisibleClassName:j(S.focusVisible,p),ref:n,type:O},x,{classes:S,children:[k,s,P]}))});var D7=h5;function d5(t){return he("MuiCard",t)}de("MuiCard",["root"]);const p5=["className","raised"],m5=t=>{const{classes:e}=t;return me({root:["root"]},d5,e)},g5=G(is,{name:"MuiCard",slot:"Root",overridesResolver:(t,e)=>e.root})(()=>({overflow:"hidden"})),y5=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiCard"}),{className:i,raised:o=!1}=r,s=H(r,p5),l=$({},r,{raised:o}),a=m5(l);return I(g5,$({className:j(a.root,i),elevation:o?8:void 0,ref:n,ownerState:l},s))});var L7=y5;function O5(t){return he("MuiCardContent",t)}de("MuiCardContent",["root"]);const v5=["className","component"],x5=t=>{const{classes:e}=t;return me({root:["root"]},O5,e)},w5=G("div",{name:"MuiCardContent",slot:"Root",overridesResolver:(t,e)=>e.root})(()=>({padding:16,"&:last-child":{paddingBottom:24}})),b5=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiCardContent"}),{className:i,component:o="div"}=r,s=H(r,v5),l=$({},r,{component:o}),a=x5(l);return I(w5,$({as:o,className:j(a.root,i),ownerState:l,ref:n},s))});var W7=b5;function $5(t){return he("MuiCardHeader",t)}const S5=de("MuiCardHeader",["root","avatar","action","content","title","subheader"]);var rw=S5;const k5=["action","avatar","className","component","disableTypography","subheader","subheaderTypographyProps","title","titleTypographyProps"],P5=t=>{const{classes:e}=t;return me({root:["root"],avatar:["avatar"],action:["action"],content:["content"],title:["title"],subheader:["subheader"]},$5,e)},C5=G("div",{name:"MuiCardHeader",slot:"Root",overridesResolver:(t,e)=>$({[`& .${rw.title}`]:e.title,[`& .${rw.subheader}`]:e.subheader},e.root)})({display:"flex",alignItems:"center",padding:16}),R5=G("div",{name:"MuiCardHeader",slot:"Avatar",overridesResolver:(t,e)=>e.avatar})({display:"flex",flex:"0 0 auto",marginRight:16}),T5=G("div",{name:"MuiCardHeader",slot:"Action",overridesResolver:(t,e)=>e.action})({flex:"0 0 auto",alignSelf:"flex-start",marginTop:-4,marginRight:-8,marginBottom:-4}),I5=G("div",{name:"MuiCardHeader",slot:"Content",overridesResolver:(t,e)=>e.content})({flex:"1 1 auto"}),E5=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiCardHeader"}),{action:i,avatar:o,className:s,component:l="div",disableTypography:a=!1,subheader:c,subheaderTypographyProps:u,title:f,titleTypographyProps:h}=r,d=H(r,k5),p=$({},r,{component:l,disableTypography:a}),m=P5(p);let g=f;g!=null&&g.type!==Sr&&!a&&(g=I(Sr,$({variant:o?"body2":"h5",className:m.title,component:"span",display:"block"},h,{children:g})));let y=c;return y!=null&&y.type!==Sr&&!a&&(y=I(Sr,$({variant:o?"body2":"body1",className:m.subheader,color:"text.secondary",component:"span",display:"block"},u,{children:y}))),Ue(C5,$({className:j(m.root,s),as:l,ref:n,ownerState:p},d,{children:[o&&I(R5,{className:m.avatar,ownerState:p,children:o}),Ue(I5,{className:m.content,ownerState:p,children:[g,y]}),i&&I(T5,{className:m.action,ownerState:p,children:i})]}))});var Q7=E5;function M5(t){return he("MuiCircularProgress",t)}de("MuiCircularProgress",["root","determinate","indeterminate","colorPrimary","colorSecondary","svg","circle","circleDeterminate","circleIndeterminate","circleDisableShrink"]);const A5=["className","color","disableShrink","size","style","thickness","value","variant"];let Qu=t=>t,iw,ow,sw,lw;const fi=44,D5=Fl(iw||(iw=Qu`
  0% {
    transform: rotate(0deg);
  }

  100% {
    transform: rotate(360deg);
  }
`)),L5=Fl(ow||(ow=Qu`
  0% {
    stroke-dasharray: 1px, 200px;
    stroke-dashoffset: 0;
  }

  50% {
    stroke-dasharray: 100px, 200px;
    stroke-dashoffset: -15px;
  }

  100% {
    stroke-dasharray: 100px, 200px;
    stroke-dashoffset: -125px;
  }
`)),W5=t=>{const{classes:e,variant:n,color:r,disableShrink:i}=t,o={root:["root",n,`color${V(r)}`],svg:["svg"],circle:["circle",`circle${V(n)}`,i&&"circleDisableShrink"]};return me(o,M5,e)},Q5=G("span",{name:"MuiCircularProgress",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,e[n.variant],e[`color${V(n.color)}`]]}})(({ownerState:t,theme:e})=>$({display:"inline-block"},t.variant==="determinate"&&{transition:e.transitions.create("transform")},t.color!=="inherit"&&{color:e.palette[t.color].main}),({ownerState:t})=>t.variant==="indeterminate"&&Lp(sw||(sw=Qu`
      animation: ${0} 1.4s linear infinite;
    `),D5)),N5=G("svg",{name:"MuiCircularProgress",slot:"Svg",overridesResolver:(t,e)=>e.svg})({display:"block"}),_5=G("circle",{name:"MuiCircularProgress",slot:"Circle",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.circle,e[`circle${V(n.variant)}`],n.disableShrink&&e.circleDisableShrink]}})(({ownerState:t,theme:e})=>$({stroke:"currentColor"},t.variant==="determinate"&&{transition:e.transitions.create("stroke-dashoffset")},t.variant==="indeterminate"&&{strokeDasharray:"80px, 200px",strokeDashoffset:0}),({ownerState:t})=>t.variant==="indeterminate"&&!t.disableShrink&&Lp(lw||(lw=Qu`
      animation: ${0} 1.4s ease-in-out infinite;
    `),L5)),B5=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiCircularProgress"}),{className:i,color:o="primary",disableShrink:s=!1,size:l=40,style:a,thickness:c=3.6,value:u=0,variant:f="indeterminate"}=r,h=H(r,A5),d=$({},r,{color:o,disableShrink:s,size:l,thickness:c,value:u,variant:f}),p=W5(d),m={},g={},y={};if(f==="determinate"){const O=2*Math.PI*((fi-c)/2);m.strokeDasharray=O.toFixed(3),y["aria-valuenow"]=Math.round(u),m.strokeDashoffset=`${((100-u)/100*O).toFixed(3)}px`,g.transform="rotate(-90deg)"}return I(Q5,$({className:j(p.root,i),style:$({width:l,height:l},g,a),ownerState:d,ref:n,role:"progressbar"},y,h,{children:I(N5,{className:p.svg,ownerState:d,viewBox:`${fi/2} ${fi/2} ${fi} ${fi}`,children:I(_5,{className:p.circle,style:m,ownerState:d,cx:fi,cy:fi,r:(fi-c)/2,fill:"none",strokeWidth:c})})}))});var N7=B5;function q5(t){return I(DT,$({},t,{defaultTheme:Eu}))}const z5=(t,e)=>$({WebkitFontSmoothing:"antialiased",MozOsxFontSmoothing:"grayscale",boxSizing:"border-box",WebkitTextSizeAdjust:"100%"},e&&{colorScheme:t.palette.mode}),F5=t=>$({color:t.palette.text.primary},t.typography.body1,{backgroundColor:t.palette.background.default,"@media print":{backgroundColor:t.palette.common.white}}),V5=(t,e=!1)=>{var n,r;let i={html:z5(t,e),"*, *::before, *::after":{boxSizing:"inherit"},"strong, b":{fontWeight:t.typography.fontWeightBold},body:$({margin:0},F5(t),{"&::backdrop":{backgroundColor:t.palette.background.default}})};const o=(n=t.components)==null||(r=n.MuiCssBaseline)==null?void 0:r.styleOverrides;return o&&(i=[i,o]),i};function _7(t){const e=fe({props:t,name:"MuiCssBaseline"}),{children:n,enableColorScheme:r=!1}=e;return Ue(w.exports.Fragment,{children:[I(q5,{styles:i=>V5(i,r)}),n]})}const U5=["BackdropComponent","closeAfterTransition","children","components","componentsProps","disableAutoFocus","disableEnforceFocus","disableEscapeKeyDown","disablePortal","disableRestoreFocus","disableScrollLock","hideBackdrop","keepMounted"],j5=t=>t.classes,X5=G("div",{name:"MuiModal",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,!n.open&&n.exited&&e.hidden]}})(({theme:t,ownerState:e})=>$({position:"fixed",zIndex:t.zIndex.modal,right:0,bottom:0,top:0,left:0},!e.open&&e.exited&&{visibility:"hidden"})),H5=G(e5,{name:"MuiModal",slot:"Backdrop",overridesResolver:(t,e)=>e.backdrop})({zIndex:-1}),G5=w.exports.forwardRef(function(e,n){var r;const i=fe({name:"MuiModal",props:e}),{BackdropComponent:o=H5,closeAfterTransition:s=!1,children:l,components:a={},componentsProps:c={},disableAutoFocus:u=!1,disableEnforceFocus:f=!1,disableEscapeKeyDown:h=!1,disablePortal:d=!1,disableRestoreFocus:p=!1,disableScrollLock:m=!1,hideBackdrop:g=!1,keepMounted:y=!1}=i,O=H(i,U5),[v,x]=w.exports.useState(!0),b={closeAfterTransition:s,disableAutoFocus:u,disableEnforceFocus:f,disableEscapeKeyDown:h,disablePortal:d,disableRestoreFocus:p,disableScrollLock:m,hideBackdrop:g,keepMounted:y},S=$({},i,b,{exited:v}),k=j5(S);return I(kM,$({components:$({Root:X5},a),componentsProps:{root:$({},c.root,(!a.Root||!zi(a.Root))&&{ownerState:$({},(r=c.root)==null?void 0:r.ownerState)})},BackdropComponent:o,onTransitionEnter:()=>x(!1),onTransitionExited:()=>x(!0),ref:n},O,{classes:k},b,{children:l}))});var aw=G5;function Z5(t){return he("MuiDivider",t)}const cw=de("MuiDivider",["root","absolute","fullWidth","inset","middle","flexItem","light","vertical","withChildren","withChildrenVertical","textAlignRight","textAlignLeft","wrapper","wrapperVertical"]),Y5=["absolute","children","className","component","flexItem","light","orientation","role","textAlign","variant"],K5=t=>{const{absolute:e,children:n,classes:r,flexItem:i,light:o,orientation:s,textAlign:l,variant:a}=t;return me({root:["root",e&&"absolute",a,o&&"light",s==="vertical"&&"vertical",i&&"flexItem",n&&"withChildren",n&&s==="vertical"&&"withChildrenVertical",l==="right"&&s!=="vertical"&&"textAlignRight",l==="left"&&s!=="vertical"&&"textAlignLeft"],wrapper:["wrapper",s==="vertical"&&"wrapperVertical"]},Z5,r)},J5=G("div",{name:"MuiDivider",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.absolute&&e.absolute,e[n.variant],n.light&&e.light,n.orientation==="vertical"&&e.vertical,n.flexItem&&e.flexItem,n.children&&e.withChildren,n.children&&n.orientation==="vertical"&&e.withChildrenVertical,n.textAlign==="right"&&n.orientation!=="vertical"&&e.textAlignRight,n.textAlign==="left"&&n.orientation!=="vertical"&&e.textAlignLeft]}})(({theme:t,ownerState:e})=>$({margin:0,flexShrink:0,borderWidth:0,borderStyle:"solid",borderColor:t.palette.divider,borderBottomWidth:"thin"},e.absolute&&{position:"absolute",bottom:0,left:0,width:"100%"},e.light&&{borderColor:ge(t.palette.divider,.08)},e.variant==="inset"&&{marginLeft:72},e.variant==="middle"&&e.orientation==="horizontal"&&{marginLeft:t.spacing(2),marginRight:t.spacing(2)},e.variant==="middle"&&e.orientation==="vertical"&&{marginTop:t.spacing(1),marginBottom:t.spacing(1)},e.orientation==="vertical"&&{height:"100%",borderBottomWidth:0,borderRightWidth:"thin"},e.flexItem&&{alignSelf:"stretch",height:"auto"}),({theme:t,ownerState:e})=>$({},e.children&&{display:"flex",whiteSpace:"nowrap",textAlign:"center",border:0,"&::before, &::after":{position:"relative",width:"100%",borderTop:`thin solid ${t.palette.divider}`,top:"50%",content:'""',transform:"translateY(50%)"}}),({theme:t,ownerState:e})=>$({},e.children&&e.orientation==="vertical"&&{flexDirection:"column","&::before, &::after":{height:"100%",top:"0%",left:"50%",borderTop:0,borderLeft:`thin solid ${t.palette.divider}`,transform:"translateX(0%)"}}),({ownerState:t})=>$({},t.textAlign==="right"&&t.orientation!=="vertical"&&{"&::before":{width:"90%"},"&::after":{width:"10%"}},t.textAlign==="left"&&t.orientation!=="vertical"&&{"&::before":{width:"10%"},"&::after":{width:"90%"}})),eW=G("span",{name:"MuiDivider",slot:"Wrapper",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.wrapper,n.orientation==="vertical"&&e.wrapperVertical]}})(({theme:t,ownerState:e})=>$({display:"inline-block",paddingLeft:`calc(${t.spacing(1)} * 1.2)`,paddingRight:`calc(${t.spacing(1)} * 1.2)`},e.orientation==="vertical"&&{paddingTop:`calc(${t.spacing(1)} * 1.2)`,paddingBottom:`calc(${t.spacing(1)} * 1.2)`})),tW=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiDivider"}),{absolute:i=!1,children:o,className:s,component:l=o?"div":"hr",flexItem:a=!1,light:c=!1,orientation:u="horizontal",role:f=l!=="hr"?"separator":void 0,textAlign:h="center",variant:d="fullWidth"}=r,p=H(r,Y5),m=$({},r,{absolute:i,component:l,flexItem:a,light:c,orientation:u,role:f,textAlign:h,variant:d}),g=K5(m);return I(J5,$({as:l,className:j(g.root,s),role:f,ref:n,ownerState:m},p,{children:o?I(eW,{className:g.wrapper,ownerState:m,children:o}):null}))});var B7=tW;const nW=["addEndListener","appear","children","container","direction","easing","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","timeout","TransitionComponent"];function rW(t,e,n){const r=e.getBoundingClientRect(),i=n&&n.getBoundingClientRect(),o=xr(e);let s;if(e.fakeTransform)s=e.fakeTransform;else{const c=o.getComputedStyle(e);s=c.getPropertyValue("-webkit-transform")||c.getPropertyValue("transform")}let l=0,a=0;if(s&&s!=="none"&&typeof s=="string"){const c=s.split("(")[1].split(")")[0].split(",");l=parseInt(c[4],10),a=parseInt(c[5],10)}return t==="left"?i?`translateX(${i.right+l-r.left}px)`:`translateX(${o.innerWidth+l-r.left}px)`:t==="right"?i?`translateX(-${r.right-i.left-l}px)`:`translateX(-${r.left+r.width-l}px)`:t==="up"?i?`translateY(${i.bottom+a-r.top}px)`:`translateY(${o.innerHeight+a-r.top}px)`:i?`translateY(-${r.top-i.top+r.height-a}px)`:`translateY(-${r.top+r.height-a}px)`}function iW(t){return typeof t=="function"?t():t}function Nu(t,e,n){const r=iW(n),i=rW(t,e,r);i&&(e.style.webkitTransform=i,e.style.transform=i)}const oW={enter:sm.easeOut,exit:sm.sharp},sW={enter:ci.enteringScreen,exit:ci.leavingScreen},lW=w.exports.forwardRef(function(e,n){const{addEndListener:r,appear:i=!0,children:o,container:s,direction:l="down",easing:a=oW,in:c,onEnter:u,onEntered:f,onEntering:h,onExit:d,onExited:p,onExiting:m,style:g,timeout:y=sW,TransitionComponent:O=Au}=e,v=H(e,nW),x=br(),b=w.exports.useRef(null),S=Qe(o.ref,b),k=Qe(S,n),P=E=>D=>{E&&(D===void 0?E(b.current):E(b.current,D))},C=P((E,D)=>{Nu(l,E,s),fm(E),u&&u(E,D)}),R=P((E,D)=>{const X=ui({timeout:y,style:g,easing:a},{mode:"enter"});E.style.webkitTransition=x.transitions.create("-webkit-transform",$({},X)),E.style.transition=x.transitions.create("transform",$({},X)),E.style.webkitTransform="none",E.style.transform="none",h&&h(E,D)}),T=P(f),L=P(m),M=P(E=>{const D=ui({timeout:y,style:g,easing:a},{mode:"exit"});E.style.webkitTransition=x.transitions.create("-webkit-transform",D),E.style.transition=x.transitions.create("transform",D),Nu(l,E,s),d&&d(E)}),q=P(E=>{E.style.webkitTransition="",E.style.transition="",p&&p(E)}),F=E=>{r&&r(b.current,E)},Q=w.exports.useCallback(()=>{b.current&&Nu(l,b.current,s)},[l,s]);return w.exports.useEffect(()=>{if(c||l==="down"||l==="right")return;const E=jl(()=>{b.current&&Nu(l,b.current,s)}),D=xr(b.current);return D.addEventListener("resize",E),()=>{E.clear(),D.removeEventListener("resize",E)}},[l,c,s]),w.exports.useEffect(()=>{c||Q()},[c,Q]),I(O,$({nodeRef:b,onEnter:C,onEntered:T,onEntering:R,onExit:M,onExited:q,onExiting:L,addEndListener:F,appear:i,in:c,timeout:y},v,{children:(E,D)=>w.exports.cloneElement(o,$({ref:k,style:$({visibility:E==="exited"&&!c?"hidden":void 0},g,o.props.style)},D))}))});var aW=lW;function cW(t){return he("MuiDrawer",t)}de("MuiDrawer",["root","docked","paper","paperAnchorLeft","paperAnchorRight","paperAnchorTop","paperAnchorBottom","paperAnchorDockedLeft","paperAnchorDockedRight","paperAnchorDockedTop","paperAnchorDockedBottom","modal"]);const uW=["BackdropProps"],fW=["anchor","BackdropProps","children","className","elevation","hideBackdrop","ModalProps","onClose","open","PaperProps","SlideProps","TransitionComponent","transitionDuration","variant"],uw=(t,e)=>{const{ownerState:n}=t;return[e.root,(n.variant==="permanent"||n.variant==="persistent")&&e.docked,e.modal]},hW=t=>{const{classes:e,anchor:n,variant:r}=t,i={root:["root"],docked:[(r==="permanent"||r==="persistent")&&"docked"],modal:["modal"],paper:["paper",`paperAnchor${V(n)}`,r!=="temporary"&&`paperAnchorDocked${V(n)}`]};return me(i,cW,e)},dW=G(aw,{name:"MuiDrawer",slot:"Root",overridesResolver:uw})(({theme:t})=>({zIndex:t.zIndex.drawer})),fw=G("div",{shouldForwardProp:ts,name:"MuiDrawer",slot:"Docked",skipVariantsResolver:!1,overridesResolver:uw})({flex:"0 0 auto"}),pW=G(is,{name:"MuiDrawer",slot:"Paper",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.paper,e[`paperAnchor${V(n.anchor)}`],n.variant!=="temporary"&&e[`paperAnchorDocked${V(n.anchor)}`]]}})(({theme:t,ownerState:e})=>$({overflowY:"auto",display:"flex",flexDirection:"column",height:"100%",flex:"1 0 auto",zIndex:t.zIndex.drawer,WebkitOverflowScrolling:"touch",position:"fixed",top:0,outline:0},e.anchor==="left"&&{left:0},e.anchor==="top"&&{top:0,left:0,right:0,height:"auto",maxHeight:"100%"},e.anchor==="right"&&{right:0},e.anchor==="bottom"&&{top:"auto",left:0,bottom:0,right:0,height:"auto",maxHeight:"100%"},e.anchor==="left"&&e.variant!=="temporary"&&{borderRight:`1px solid ${t.palette.divider}`},e.anchor==="top"&&e.variant!=="temporary"&&{borderBottom:`1px solid ${t.palette.divider}`},e.anchor==="right"&&e.variant!=="temporary"&&{borderLeft:`1px solid ${t.palette.divider}`},e.anchor==="bottom"&&e.variant!=="temporary"&&{borderTop:`1px solid ${t.palette.divider}`})),hw={left:"right",right:"left",top:"down",bottom:"up"};function mW(t){return["left","right"].indexOf(t)!==-1}function gW(t,e){return t.direction==="rtl"&&mW(e)?hw[e]:e}const yW={enter:ci.enteringScreen,exit:ci.leavingScreen},OW=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiDrawer"}),{anchor:i="left",BackdropProps:o,children:s,className:l,elevation:a=16,hideBackdrop:c=!1,ModalProps:{BackdropProps:u}={},onClose:f,open:h=!1,PaperProps:d={},SlideProps:p,TransitionComponent:m=aW,transitionDuration:g=yW,variant:y="temporary"}=r,O=H(r.ModalProps,uW),v=H(r,fW),x=br(),b=w.exports.useRef(!1);w.exports.useEffect(()=>{b.current=!0},[]);const S=gW(x,i),P=$({},r,{anchor:i,elevation:a,open:h,variant:y},v),C=hW(P),R=I(pW,$({elevation:y==="temporary"?a:0,square:!0},d,{className:j(C.paper,d.className),ownerState:P,children:s}));if(y==="permanent")return I(fw,$({className:j(C.root,C.docked,l),ownerState:P,ref:n},v,{children:R}));const T=I(m,$({in:h,direction:hw[S],timeout:g,appear:b.current},p,{children:R}));return y==="persistent"?I(fw,$({className:j(C.root,C.docked,l),ownerState:P,ref:n},v,{children:T})):I(dW,$({BackdropProps:$({},o,u,{transitionDuration:g}),className:j(C.root,C.modal,l),open:h,ownerState:P,onClose:f,hideBackdrop:c,ref:n},v,O,{children:T}))});var q7=OW;const vW=w.exports.createContext();var dw=vW;function xW(t){return he("MuiGrid",t)}const wW=[0,1,2,3,4,5,6,7,8,9,10],bW=["column-reverse","column","row-reverse","row"],$W=["nowrap","wrap-reverse","wrap"],la=["auto",!0,1,2,3,4,5,6,7,8,9,10,11,12],dm=de("MuiGrid",["root","container","item","zeroMinWidth",...wW.map(t=>`spacing-xs-${t}`),...bW.map(t=>`direction-xs-${t}`),...$W.map(t=>`wrap-xs-${t}`),...la.map(t=>`grid-xs-${t}`),...la.map(t=>`grid-sm-${t}`),...la.map(t=>`grid-md-${t}`),...la.map(t=>`grid-lg-${t}`),...la.map(t=>`grid-xl-${t}`)]),SW=["className","columns","columnSpacing","component","container","direction","item","lg","md","rowSpacing","sm","spacing","wrap","xl","xs","zeroMinWidth"];function ss(t){const e=parseFloat(t);return`${e}${String(t).replace(String(e),"")||"px"}`}function kW({theme:t,ownerState:e}){let n;return t.breakpoints.keys.reduce((r,i)=>{let o={};if(e[i]&&(n=e[i]),!n)return r;if(n===!0)o={flexBasis:0,flexGrow:1,maxWidth:"100%"};else if(n==="auto")o={flexBasis:"auto",flexGrow:0,flexShrink:0,maxWidth:"none",width:"auto"};else{const s=vu({values:e.columns,breakpoints:t.breakpoints.values}),l=typeof s=="object"?s[i]:s;if(l==null)return r;const a=`${Math.round(n/l*1e8)/1e6}%`;let c={};if(e.container&&e.item&&e.columnSpacing!==0){const u=t.spacing(e.columnSpacing);if(u!=="0px"){const f=`calc(${a} + ${ss(u)})`;c={flexBasis:f,maxWidth:f}}}o=$({flexBasis:a,flexGrow:0,maxWidth:a},c)}return t.breakpoints.values[i]===0?Object.assign(r,o):r[t.breakpoints.up(i)]=o,r},{})}function PW({theme:t,ownerState:e}){const n=vu({values:e.direction,breakpoints:t.breakpoints.values});return Ln({theme:t},n,r=>{const i={flexDirection:r};return r.indexOf("column")===0&&(i[`& > .${dm.item}`]={maxWidth:"none"}),i})}function CW({theme:t,ownerState:e}){const{container:n,rowSpacing:r}=e;let i={};if(n&&r!==0){const o=vu({values:r,breakpoints:t.breakpoints.values});i=Ln({theme:t},o,s=>{const l=t.spacing(s);return l!=="0px"?{marginTop:`-${ss(l)}`,[`& > .${dm.item}`]:{paddingTop:ss(l)}}:{}})}return i}function RW({theme:t,ownerState:e}){const{container:n,columnSpacing:r}=e;let i={};if(n&&r!==0){const o=vu({values:r,breakpoints:t.breakpoints.values});i=Ln({theme:t},o,s=>{const l=t.spacing(s);return l!=="0px"?{width:`calc(100% + ${ss(l)})`,marginLeft:`-${ss(l)}`,[`& > .${dm.item}`]:{paddingLeft:ss(l)}}:{}})}return i}function pw(t,e,n={}){if(!e||!t||t<=0)return[];if(typeof t=="string"&&!Number.isNaN(Number(t))||typeof t=="number")return[n[`spacing-xs-${String(t)}`]||`spacing-xs-${String(t)}`];const{xs:r,sm:i,md:o,lg:s,xl:l}=t;return[Number(r)>0&&(n[`spacing-xs-${String(r)}`]||`spacing-xs-${String(r)}`),Number(i)>0&&(n[`spacing-sm-${String(i)}`]||`spacing-sm-${String(i)}`),Number(o)>0&&(n[`spacing-md-${String(o)}`]||`spacing-md-${String(o)}`),Number(s)>0&&(n[`spacing-lg-${String(s)}`]||`spacing-lg-${String(s)}`),Number(l)>0&&(n[`spacing-xl-${String(l)}`]||`spacing-xl-${String(l)}`)]}const TW=G("div",{name:"MuiGrid",slot:"Root",overridesResolver:(t,e)=>{const{container:n,direction:r,item:i,lg:o,md:s,sm:l,spacing:a,wrap:c,xl:u,xs:f,zeroMinWidth:h}=t.ownerState;return[e.root,n&&e.container,i&&e.item,h&&e.zeroMinWidth,...pw(a,n,e),r!=="row"&&e[`direction-xs-${String(r)}`],c!=="wrap"&&e[`wrap-xs-${String(c)}`],f!==!1&&e[`grid-xs-${String(f)}`],l!==!1&&e[`grid-sm-${String(l)}`],s!==!1&&e[`grid-md-${String(s)}`],o!==!1&&e[`grid-lg-${String(o)}`],u!==!1&&e[`grid-xl-${String(u)}`]]}})(({ownerState:t})=>$({boxSizing:"border-box"},t.container&&{display:"flex",flexWrap:"wrap",width:"100%"},t.item&&{margin:0},t.zeroMinWidth&&{minWidth:0},t.wrap==="nowrap"&&{flexWrap:"nowrap"},t.wrap==="reverse"&&{flexWrap:"wrap-reverse"}),PW,CW,RW,kW),IW=t=>{const{classes:e,container:n,direction:r,item:i,lg:o,md:s,sm:l,spacing:a,wrap:c,xl:u,xs:f,zeroMinWidth:h}=t,d={root:["root",n&&"container",i&&"item",h&&"zeroMinWidth",...pw(a,n),r!=="row"&&`direction-xs-${String(r)}`,c!=="wrap"&&`wrap-xs-${String(c)}`,f!==!1&&`grid-xs-${String(f)}`,l!==!1&&`grid-sm-${String(l)}`,s!==!1&&`grid-md-${String(s)}`,o!==!1&&`grid-lg-${String(o)}`,u!==!1&&`grid-xl-${String(u)}`]};return me(d,xW,e)},EW=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiGrid"}),i=jp(r),{className:o,columns:s,columnSpacing:l,component:a="div",container:c=!1,direction:u="row",item:f=!1,lg:h=!1,md:d=!1,rowSpacing:p,sm:m=!1,spacing:g=0,wrap:y="wrap",xl:O=!1,xs:v=!1,zeroMinWidth:x=!1}=i,b=H(i,SW),S=p||g,k=l||g,P=w.exports.useContext(dw),C=s||P||12,R=$({},i,{columns:C,container:c,direction:u,item:f,lg:h,md:d,sm:m,rowSpacing:S,columnSpacing:k,wrap:y,xl:O,xs:v,zeroMinWidth:x}),T=IW(R);return(M=>C!==12?I(dw.Provider,{value:C,children:M}):M)(I(TW,$({ownerState:R,className:j(T.root,o),as:a,ref:n},b)))});var z7=EW;const MW=["addEndListener","appear","children","easing","in","onEnter","onEntered","onEntering","onExit","onExited","onExiting","style","timeout","TransitionComponent"];function pm(t){return`scale(${t}, ${t**2})`}const AW={entering:{opacity:1,transform:pm(1)},entered:{opacity:1,transform:"none"}},mw=w.exports.forwardRef(function(e,n){const{addEndListener:r,appear:i=!0,children:o,easing:s,in:l,onEnter:a,onEntered:c,onEntering:u,onExit:f,onExited:h,onExiting:d,style:p,timeout:m="auto",TransitionComponent:g=Au}=e,y=H(e,MW),O=w.exports.useRef(),v=w.exports.useRef(),x=br(),b=w.exports.useRef(null),S=Qe(o.ref,n),k=Qe(b,S),P=Q=>E=>{if(Q){const D=b.current;E===void 0?Q(D):Q(D,E)}},C=P(u),R=P((Q,E)=>{fm(Q);const{duration:D,delay:X,easing:ae}=ui({style:p,timeout:m,easing:s},{mode:"enter"});let W;m==="auto"?(W=x.transitions.getAutoHeightDuration(Q.clientHeight),v.current=W):W=D,Q.style.transition=[x.transitions.create("opacity",{duration:W,delay:X}),x.transitions.create("transform",{duration:W*.666,delay:X,easing:ae})].join(","),a&&a(Q,E)}),T=P(c),L=P(d),M=P(Q=>{const{duration:E,delay:D,easing:X}=ui({style:p,timeout:m,easing:s},{mode:"exit"});let ae;m==="auto"?(ae=x.transitions.getAutoHeightDuration(Q.clientHeight),v.current=ae):ae=E,Q.style.transition=[x.transitions.create("opacity",{duration:ae,delay:D}),x.transitions.create("transform",{duration:ae*.666,delay:D||ae*.333,easing:X})].join(","),Q.style.opacity="0",Q.style.transform=pm(.75),f&&f(Q)}),q=P(h),F=Q=>{m==="auto"&&(O.current=setTimeout(Q,v.current||0)),r&&r(b.current,Q)};return w.exports.useEffect(()=>()=>{clearTimeout(O.current)},[]),I(g,$({appear:i,in:l,nodeRef:b,onEnter:R,onEntered:T,onEntering:C,onExit:M,onExited:q,onExiting:L,addEndListener:F,timeout:m==="auto"?null:m},y,{children:(Q,E)=>w.exports.cloneElement(o,$({style:$({opacity:0,transform:pm(.75),visibility:Q==="exited"&&!l?"hidden":void 0},AW[Q],p,o.props.style),ref:k},E))}))});mw.muiSupportAuto=!0;var mm=mw;function F7(t,e={}){const n=Hp(),r=typeof window!="undefined"&&typeof window.matchMedia!="undefined",{defaultMatches:i=!1,matchMedia:o=r?window.matchMedia:null,noSsr:s=!1,ssrMatchMedia:l=null}=ax({name:"MuiUseMediaQuery",props:e,theme:n});let a=typeof t=="function"?t(n):t;a=a.replace(/^@media( ?)/m,"");const[c,u]=w.exports.useState(()=>s&&r?o(a).matches:l?l(a).matches:i);return Gn(()=>{let f=!0;if(!r)return;const h=o(a),d=()=>{f&&u(h.matches)};return d(),h.addListener(d),()=>{f=!1,h.removeListener(d)}},[a,o,r]),c}function DW(t){return he("MuiLink",t)}const LW=de("MuiLink",["root","underlineNone","underlineHover","underlineAlways","button","focusVisible"]);var WW=LW;const QW=["className","color","component","onBlur","onFocus","TypographyClasses","underline","variant"],NW={primary:"primary.main",textPrimary:"text.primary",secondary:"secondary.main",textSecondary:"text.secondary",error:"error.main"},_W=t=>NW[t]||t,BW=t=>{const{classes:e,component:n,focusVisible:r,underline:i}=t,o={root:["root",`underline${V(i)}`,n==="button"&&"button",r&&"focusVisible"]};return me(o,DW,e)},qW=G(Sr,{name:"MuiLink",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,e[`underline${V(n.underline)}`],n.component==="button"&&e.button]}})(({theme:t,ownerState:e})=>{const n=xu(t,`palette.${_W(e.color)}`)||e.color;return $({},e.underline==="none"&&{textDecoration:"none"},e.underline==="hover"&&{textDecoration:"none","&:hover":{textDecoration:"underline"}},e.underline==="always"&&{textDecoration:"underline",textDecorationColor:n!=="inherit"?ge(n,.4):void 0,"&:hover":{textDecorationColor:"inherit"}},e.component==="button"&&{position:"relative",WebkitTapHighlightColor:"transparent",backgroundColor:"transparent",outline:0,border:0,margin:0,borderRadius:0,padding:0,cursor:"pointer",userSelect:"none",verticalAlign:"middle",MozAppearance:"none",WebkitAppearance:"none","&::-moz-focus-inner":{borderStyle:"none"},[`&.${WW.focusVisible}`]:{outline:"auto"}})}),zW=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiLink"}),{className:i,color:o="primary",component:s="a",onBlur:l,onFocus:a,TypographyClasses:c,underline:u="always",variant:f="inherit"}=r,h=H(r,QW),{isFocusVisibleRef:d,onBlur:p,onFocus:m,ref:g}=_p(),[y,O]=w.exports.useState(!1),v=Qe(n,g),x=P=>{p(P),d.current===!1&&O(!1),l&&l(P)},b=P=>{m(P),d.current===!0&&O(!0),a&&a(P)},S=$({},r,{color:o,component:s,focusVisible:y,underline:u,variant:f}),k=BW(S);return I(qW,$({className:j(k.root,i),classes:c,color:o,component:s,onBlur:x,onFocus:b,ref:v,ownerState:S,variant:f},h))});var V7=zW;const FW=w.exports.createContext({});var Qn=FW;function VW(t){return he("MuiList",t)}de("MuiList",["root","padding","dense","subheader"]);const UW=["children","className","component","dense","disablePadding","subheader"],jW=t=>{const{classes:e,disablePadding:n,dense:r,subheader:i}=t;return me({root:["root",!n&&"padding",r&&"dense",i&&"subheader"]},VW,e)},XW=G("ul",{name:"MuiList",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,!n.disablePadding&&e.padding,n.dense&&e.dense,n.subheader&&e.subheader]}})(({ownerState:t})=>$({listStyle:"none",margin:0,padding:0,position:"relative"},!t.disablePadding&&{paddingTop:8,paddingBottom:8},t.subheader&&{paddingTop:0})),HW=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiList"}),{children:i,className:o,component:s="ul",dense:l=!1,disablePadding:a=!1,subheader:c}=r,u=H(r,UW),f=w.exports.useMemo(()=>({dense:l}),[l]),h=$({},r,{component:s,dense:l,disablePadding:a}),d=jW(h);return I(Qn.Provider,{value:f,children:Ue(XW,$({as:s,className:j(d.root,o),ref:n,ownerState:h},u,{children:[c,i]}))})});var GW=HW;function ZW(t){return he("MuiListItem",t)}const YW=de("MuiListItem",["root","container","focusVisible","dense","alignItemsFlexStart","disabled","divider","gutters","padding","button","secondaryAction","selected"]);var ls=YW;function KW(t){return he("MuiListItemButton",t)}const JW=de("MuiListItemButton",["root","focusVisible","dense","alignItemsFlexStart","disabled","divider","gutters","selected"]);var as=JW;const eQ=["alignItems","autoFocus","component","children","dense","disableGutters","divider","focusVisibleClassName","selected"],tQ=(t,e)=>{const{ownerState:n}=t;return[e.root,n.dense&&e.dense,n.alignItems==="flex-start"&&e.alignItemsFlexStart,n.divider&&e.divider,!n.disableGutters&&e.gutters]},nQ=t=>{const{alignItems:e,classes:n,dense:r,disabled:i,disableGutters:o,divider:s,selected:l}=t,c=me({root:["root",r&&"dense",!o&&"gutters",s&&"divider",i&&"disabled",e==="flex-start"&&"alignItemsFlexStart",l&&"selected"]},KW,n);return $({},n,c)},rQ=G(Kn,{shouldForwardProp:t=>ts(t)||t==="classes",name:"MuiListItemButton",slot:"Root",overridesResolver:tQ})(({theme:t,ownerState:e})=>$({display:"flex",flexGrow:1,justifyContent:"flex-start",alignItems:"center",position:"relative",textDecoration:"none",boxSizing:"border-box",textAlign:"left",paddingTop:8,paddingBottom:8,transition:t.transitions.create("background-color",{duration:t.transitions.duration.shortest}),"&:hover":{textDecoration:"none",backgroundColor:t.palette.action.hover,"@media (hover: none)":{backgroundColor:"transparent"}},[`&.${as.selected}`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity),[`&.${as.focusVisible}`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity+t.palette.action.focusOpacity)}},[`&.${as.selected}:hover`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity+t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity)}},[`&.${as.focusVisible}`]:{backgroundColor:t.palette.action.focus},[`&.${as.disabled}`]:{opacity:t.palette.action.disabledOpacity}},e.divider&&{borderBottom:`1px solid ${t.palette.divider}`,backgroundClip:"padding-box"},e.alignItems==="flex-start"&&{alignItems:"flex-start"},!e.disableGutters&&{paddingLeft:16,paddingRight:16},e.dense&&{paddingTop:4,paddingBottom:4})),iQ=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiListItemButton"}),{alignItems:i="center",autoFocus:o=!1,component:s="div",children:l,dense:a=!1,disableGutters:c=!1,divider:u=!1,focusVisibleClassName:f,selected:h=!1}=r,d=H(r,eQ),p=w.exports.useContext(Qn),m={dense:a||p.dense||!1,alignItems:i,disableGutters:c},g=w.exports.useRef(null);Gn(()=>{o&&g.current&&g.current.focus()},[o]);const y=$({},r,{alignItems:i,dense:m.dense,disableGutters:c,divider:u,selected:h}),O=nQ(y),v=Qe(g,n);return I(Qn.Provider,{value:m,children:I(rQ,$({ref:v,component:s,focusVisibleClassName:j(O.focusVisible,f),ownerState:y},d,{classes:O,children:l}))})});var U7=iQ;function oQ(t){return he("MuiListItemSecondaryAction",t)}de("MuiListItemSecondaryAction",["root","disableGutters"]);const sQ=["className"],lQ=t=>{const{disableGutters:e,classes:n}=t;return me({root:["root",e&&"disableGutters"]},oQ,n)},aQ=G("div",{name:"MuiListItemSecondaryAction",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.disableGutters&&e.disableGutters]}})(({ownerState:t})=>$({position:"absolute",right:16,top:"50%",transform:"translateY(-50%)"},t.disableGutters&&{right:0})),gw=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiListItemSecondaryAction"}),{className:i}=r,o=H(r,sQ),s=w.exports.useContext(Qn),l=$({},r,{disableGutters:s.disableGutters}),a=lQ(l);return I(aQ,$({className:j(a.root,i),ownerState:l,ref:n},o))});gw.muiName="ListItemSecondaryAction";var cQ=gw;const uQ=["className"],fQ=["alignItems","autoFocus","button","children","className","component","components","componentsProps","ContainerComponent","ContainerProps","dense","disabled","disableGutters","disablePadding","divider","focusVisibleClassName","secondaryAction","selected"],hQ=(t,e)=>{const{ownerState:n}=t;return[e.root,n.dense&&e.dense,n.alignItems==="flex-start"&&e.alignItemsFlexStart,n.divider&&e.divider,!n.disableGutters&&e.gutters,!n.disablePadding&&e.padding,n.button&&e.button,n.hasSecondaryAction&&e.secondaryAction]},dQ=t=>{const{alignItems:e,button:n,classes:r,dense:i,disabled:o,disableGutters:s,disablePadding:l,divider:a,hasSecondaryAction:c,selected:u}=t;return me({root:["root",i&&"dense",!s&&"gutters",!l&&"padding",a&&"divider",o&&"disabled",n&&"button",e==="flex-start"&&"alignItemsFlexStart",c&&"secondaryAction",u&&"selected"],container:["container"]},ZW,r)},pQ=G("div",{name:"MuiListItem",slot:"Root",overridesResolver:hQ})(({theme:t,ownerState:e})=>$({display:"flex",justifyContent:"flex-start",alignItems:"center",position:"relative",textDecoration:"none",width:"100%",boxSizing:"border-box",textAlign:"left"},!e.disablePadding&&$({paddingTop:8,paddingBottom:8},e.dense&&{paddingTop:4,paddingBottom:4},!e.disableGutters&&{paddingLeft:16,paddingRight:16},!!e.secondaryAction&&{paddingRight:48}),!!e.secondaryAction&&{[`& > .${as.root}`]:{paddingRight:48}},{[`&.${ls.focusVisible}`]:{backgroundColor:t.palette.action.focus},[`&.${ls.selected}`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity),[`&.${ls.focusVisible}`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity+t.palette.action.focusOpacity)}},[`&.${ls.disabled}`]:{opacity:t.palette.action.disabledOpacity}},e.alignItems==="flex-start"&&{alignItems:"flex-start"},e.divider&&{borderBottom:`1px solid ${t.palette.divider}`,backgroundClip:"padding-box"},e.button&&{transition:t.transitions.create("background-color",{duration:t.transitions.duration.shortest}),"&:hover":{textDecoration:"none",backgroundColor:t.palette.action.hover,"@media (hover: none)":{backgroundColor:"transparent"}},[`&.${ls.selected}:hover`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity+t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity)}}},e.hasSecondaryAction&&{paddingRight:48})),mQ=G("li",{name:"MuiListItem",slot:"Container",overridesResolver:(t,e)=>e.container})({position:"relative"}),gQ=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiListItem"}),{alignItems:i="center",autoFocus:o=!1,button:s=!1,children:l,className:a,component:c,components:u={},componentsProps:f={},ContainerComponent:h="li",ContainerProps:{className:d}={},dense:p=!1,disabled:m=!1,disableGutters:g=!1,disablePadding:y=!1,divider:O=!1,focusVisibleClassName:v,secondaryAction:x,selected:b=!1}=r,S=H(r.ContainerProps,uQ),k=H(r,fQ),P=w.exports.useContext(Qn),C={dense:p||P.dense||!1,alignItems:i,disableGutters:g},R=w.exports.useRef(null);Gn(()=>{o&&R.current&&R.current.focus()},[o]);const T=w.exports.Children.toArray(l),L=T.length&&LT(T[T.length-1],["ListItemSecondaryAction"]),M=$({},r,{alignItems:i,autoFocus:o,button:s,dense:C.dense,disabled:m,disableGutters:g,disablePadding:y,divider:O,hasSecondaryAction:L,selected:b}),q=dQ(M),F=Qe(R,n),Q=u.Root||pQ,E=f.root||{},D=$({className:j(q.root,E.className,a),disabled:m},k);let X=c||"li";return s&&(D.component=c||"div",D.focusVisibleClassName=j(ls.focusVisible,v),X=Kn),L?(X=!D.component&&!c?"div":X,h==="li"&&(X==="li"?X="div":D.component==="li"&&(D.component="div")),I(Qn.Provider,{value:C,children:Ue(mQ,$({as:h,className:j(q.container,d),ref:F,ownerState:M},S,{children:[I(Q,$({},E,!zi(Q)&&{as:X,ownerState:$({},M,E.ownerState)},D,{children:T})),T.pop()]}))})):I(Qn.Provider,{value:C,children:Ue(Q,$({},E,{as:X,ref:F,ownerState:M},!zi(Q)&&{ownerState:$({},M,E.ownerState)},D,{children:[T,x&&I(cQ,{children:x})]}))})});var j7=gQ;function yQ(t){return he("MuiListItemIcon",t)}const OQ=de("MuiListItemIcon",["root","alignItemsFlexStart"]);var yw=OQ;const vQ=["className"],xQ=t=>{const{alignItems:e,classes:n}=t;return me({root:["root",e==="flex-start"&&"alignItemsFlexStart"]},yQ,n)},wQ=G("div",{name:"MuiListItemIcon",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.alignItems==="flex-start"&&e.alignItemsFlexStart]}})(({theme:t,ownerState:e})=>$({minWidth:56,color:t.palette.action.active,flexShrink:0,display:"inline-flex"},e.alignItems==="flex-start"&&{marginTop:8})),bQ=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiListItemIcon"}),{className:i}=r,o=H(r,vQ),s=w.exports.useContext(Qn),l=$({},r,{alignItems:s.alignItems}),a=xQ(l);return I(wQ,$({className:j(a.root,i),ownerState:l,ref:n},o))});var X7=bQ;function $Q(t){return he("MuiListItemText",t)}const SQ=de("MuiListItemText",["root","multiline","dense","inset","primary","secondary"]);var _u=SQ;const kQ=["children","className","disableTypography","inset","primary","primaryTypographyProps","secondary","secondaryTypographyProps"],PQ=t=>{const{classes:e,inset:n,primary:r,secondary:i,dense:o}=t;return me({root:["root",n&&"inset",o&&"dense",r&&i&&"multiline"],primary:["primary"],secondary:["secondary"]},$Q,e)},CQ=G("div",{name:"MuiListItemText",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[{[`& .${_u.primary}`]:e.primary},{[`& .${_u.secondary}`]:e.secondary},e.root,n.inset&&e.inset,n.primary&&n.secondary&&e.multiline,n.dense&&e.dense]}})(({ownerState:t})=>$({flex:"1 1 auto",minWidth:0,marginTop:4,marginBottom:4},t.primary&&t.secondary&&{marginTop:6,marginBottom:6},t.inset&&{paddingLeft:56})),RQ=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiListItemText"}),{children:i,className:o,disableTypography:s=!1,inset:l=!1,primary:a,primaryTypographyProps:c,secondary:u,secondaryTypographyProps:f}=r,h=H(r,kQ),{dense:d}=w.exports.useContext(Qn);let p=a!=null?a:i,m=u;const g=$({},r,{disableTypography:s,inset:l,primary:!!p,secondary:!!m,dense:d}),y=PQ(g);return p!=null&&p.type!==Sr&&!s&&(p=I(Sr,$({variant:d?"body2":"body1",className:y.primary,component:"span",display:"block"},c,{children:p}))),m!=null&&m.type!==Sr&&!s&&(m=I(Sr,$({variant:"body2",className:y.secondary,color:"text.secondary",display:"block"},f,{children:m}))),Ue(CQ,$({className:j(y.root,o),ownerState:g,ref:n},h,{children:[p,m]}))});var H7=RQ;const TQ=["actions","autoFocus","autoFocusItem","children","className","disabledItemsFocusable","disableListWrap","onKeyDown","variant"];function gm(t,e,n){return t===e?t.firstChild:e&&e.nextElementSibling?e.nextElementSibling:n?null:t.firstChild}function Ow(t,e,n){return t===e?n?t.firstChild:t.lastChild:e&&e.previousElementSibling?e.previousElementSibling:n?null:t.lastChild}function vw(t,e){if(e===void 0)return!0;let n=t.innerText;return n===void 0&&(n=t.textContent),n=n.trim().toLowerCase(),n.length===0?!1:e.repeating?n[0]===e.keys[0]:n.indexOf(e.keys.join(""))===0}function aa(t,e,n,r,i,o){let s=!1,l=i(t,e,e?n:!1);for(;l;){if(l===t.firstChild){if(s)return!1;s=!0}const a=r?!1:l.disabled||l.getAttribute("aria-disabled")==="true";if(!l.hasAttribute("tabindex")||!vw(l,o)||a)l=i(t,l,n);else return l.focus(),!0}return!1}const IQ=w.exports.forwardRef(function(e,n){const{actions:r,autoFocus:i=!1,autoFocusItem:o=!1,children:s,className:l,disabledItemsFocusable:a=!1,disableListWrap:c=!1,onKeyDown:u,variant:f="selectedMenu"}=e,h=H(e,TQ),d=w.exports.useRef(null),p=w.exports.useRef({keys:[],repeating:!0,previousKeyMatched:!0,lastTime:null});Gn(()=>{i&&d.current.focus()},[i]),w.exports.useImperativeHandle(r,()=>({adjustStyleForScrollbar:(v,x)=>{const b=!d.current.style.width;if(v.clientHeight<d.current.clientHeight&&b){const S=`${W1(Pt(v))}px`;d.current.style[x.direction==="rtl"?"paddingLeft":"paddingRight"]=S,d.current.style.width=`calc(100% + ${S})`}return d.current}}),[]);const m=v=>{const x=d.current,b=v.key,S=Pt(x).activeElement;if(b==="ArrowDown")v.preventDefault(),aa(x,S,c,a,gm);else if(b==="ArrowUp")v.preventDefault(),aa(x,S,c,a,Ow);else if(b==="Home")v.preventDefault(),aa(x,null,c,a,gm);else if(b==="End")v.preventDefault(),aa(x,null,c,a,Ow);else if(b.length===1){const k=p.current,P=b.toLowerCase(),C=performance.now();k.keys.length>0&&(C-k.lastTime>500?(k.keys=[],k.repeating=!0,k.previousKeyMatched=!0):k.repeating&&P!==k.keys[0]&&(k.repeating=!1)),k.lastTime=C,k.keys.push(P);const R=S&&!k.repeating&&vw(S,k);k.previousKeyMatched&&(R||aa(x,S,!1,a,gm,k))?v.preventDefault():k.previousKeyMatched=!1}u&&u(v)},g=Qe(d,n);let y=-1;w.exports.Children.forEach(s,(v,x)=>{!w.exports.isValidElement(v)||v.props.disabled||(f==="selectedMenu"&&v.props.selected||y===-1)&&(y=x)});const O=w.exports.Children.map(s,(v,x)=>{if(x===y){const b={};return o&&(b.autoFocus=!0),v.props.tabIndex===void 0&&f==="selectedMenu"&&(b.tabIndex=0),w.exports.cloneElement(v,b)}return v});return I(GW,$({role:"menu",ref:g,className:l,onKeyDown:m,tabIndex:i?0:-1},h,{children:O}))});var EQ=IQ;function MQ(t){return he("MuiPopover",t)}de("MuiPopover",["root","paper"]);const AQ=["onEntering"],DQ=["action","anchorEl","anchorOrigin","anchorPosition","anchorReference","children","className","container","elevation","marginThreshold","open","PaperProps","transformOrigin","TransitionComponent","transitionDuration","TransitionProps"];function xw(t,e){let n=0;return typeof e=="number"?n=e:e==="center"?n=t.height/2:e==="bottom"&&(n=t.height),n}function ww(t,e){let n=0;return typeof e=="number"?n=e:e==="center"?n=t.width/2:e==="right"&&(n=t.width),n}function bw(t){return[t.horizontal,t.vertical].map(e=>typeof e=="number"?`${e}px`:e).join(" ")}function ym(t){return typeof t=="function"?t():t}const LQ=t=>{const{classes:e}=t;return me({root:["root"],paper:["paper"]},MQ,e)},WQ=G(aw,{name:"MuiPopover",slot:"Root",overridesResolver:(t,e)=>e.root})({}),QQ=G(is,{name:"MuiPopover",slot:"Paper",overridesResolver:(t,e)=>e.paper})({position:"absolute",overflowY:"auto",overflowX:"hidden",minWidth:16,minHeight:16,maxWidth:"calc(100% - 32px)",maxHeight:"calc(100% - 32px)",outline:0}),NQ=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiPopover"}),{action:i,anchorEl:o,anchorOrigin:s={vertical:"top",horizontal:"left"},anchorPosition:l,anchorReference:a="anchorEl",children:c,className:u,container:f,elevation:h=8,marginThreshold:d=16,open:p,PaperProps:m={},transformOrigin:g={vertical:"top",horizontal:"left"},TransitionComponent:y=mm,transitionDuration:O="auto",TransitionProps:{onEntering:v}={}}=r,x=H(r.TransitionProps,AQ),b=H(r,DQ),S=w.exports.useRef(),k=Qe(S,m.ref),P=$({},r,{anchorOrigin:s,anchorReference:a,elevation:h,marginThreshold:d,PaperProps:m,transformOrigin:g,TransitionComponent:y,transitionDuration:O,TransitionProps:x}),C=LQ(P),R=w.exports.useCallback(()=>{if(a==="anchorPosition")return l;const E=ym(o),X=(E&&E.nodeType===1?E:Pt(S.current).body).getBoundingClientRect();return{top:X.top+xw(X,s.vertical),left:X.left+ww(X,s.horizontal)}},[o,s.horizontal,s.vertical,l,a]),T=w.exports.useCallback(E=>({vertical:xw(E,g.vertical),horizontal:ww(E,g.horizontal)}),[g.horizontal,g.vertical]),L=w.exports.useCallback(E=>{const D={width:E.offsetWidth,height:E.offsetHeight},X=T(D);if(a==="none")return{top:null,left:null,transformOrigin:bw(X)};const ae=R();let W=ae.top-X.vertical,B=ae.left-X.horizontal;const Z=W+D.height,ee=B+D.width,pe=xr(ym(o)),J=pe.innerHeight-d,ue=pe.innerWidth-d;if(W<d){const se=W-d;W-=se,X.vertical+=se}else if(Z>J){const se=Z-J;W-=se,X.vertical+=se}if(B<d){const se=B-d;B-=se,X.horizontal+=se}else if(ee>ue){const se=ee-ue;B-=se,X.horizontal+=se}return{top:`${Math.round(W)}px`,left:`${Math.round(B)}px`,transformOrigin:bw(X)}},[o,a,R,T,d]),M=w.exports.useCallback(()=>{const E=S.current;if(!E)return;const D=L(E);D.top!==null&&(E.style.top=D.top),D.left!==null&&(E.style.left=D.left),E.style.transformOrigin=D.transformOrigin},[L]),q=(E,D)=>{v&&v(E,D),M()};w.exports.useEffect(()=>{p&&M()}),w.exports.useImperativeHandle(i,()=>p?{updatePosition:()=>{M()}}:null,[p,M]),w.exports.useEffect(()=>{if(!p)return;const E=jl(()=>{M()}),D=xr(o);return D.addEventListener("resize",E),()=>{E.clear(),D.removeEventListener("resize",E)}},[o,p,M]);let F=O;O==="auto"&&!y.muiSupportAuto&&(F=void 0);const Q=f||(o?Pt(ym(o)).body:void 0);return I(WQ,$({BackdropProps:{invisible:!0},className:j(C.root,u),container:Q,open:p,ref:n,ownerState:P},b,{children:I(y,$({appear:!0,in:p,onEntering:q,timeout:F},x,{children:I(QQ,$({elevation:h},m,{ref:k,className:j(C.paper,m.className),children:c}))}))}))});var _Q=NQ;function BQ(t){return he("MuiMenu",t)}de("MuiMenu",["root","paper","list"]);const qQ=["onEntering"],zQ=["autoFocus","children","disableAutoFocusItem","MenuListProps","onClose","open","PaperProps","PopoverClasses","transitionDuration","TransitionProps","variant"],FQ={vertical:"top",horizontal:"right"},VQ={vertical:"top",horizontal:"left"},UQ=t=>{const{classes:e}=t;return me({root:["root"],paper:["paper"],list:["list"]},BQ,e)},jQ=G(_Q,{shouldForwardProp:t=>ts(t)||t==="classes",name:"MuiMenu",slot:"Root",overridesResolver:(t,e)=>e.root})({}),XQ=G(is,{name:"MuiMenu",slot:"Paper",overridesResolver:(t,e)=>e.paper})({maxHeight:"calc(100% - 96px)",WebkitOverflowScrolling:"touch"}),HQ=G(EQ,{name:"MuiMenu",slot:"List",overridesResolver:(t,e)=>e.list})({outline:0}),GQ=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiMenu"}),{autoFocus:i=!0,children:o,disableAutoFocusItem:s=!1,MenuListProps:l={},onClose:a,open:c,PaperProps:u={},PopoverClasses:f,transitionDuration:h="auto",TransitionProps:{onEntering:d}={},variant:p="selectedMenu"}=r,m=H(r.TransitionProps,qQ),g=H(r,zQ),y=br(),O=y.direction==="rtl",v=$({},r,{autoFocus:i,disableAutoFocusItem:s,MenuListProps:l,onEntering:d,PaperProps:u,transitionDuration:h,TransitionProps:m,variant:p}),x=UQ(v),b=i&&!s&&c,S=w.exports.useRef(null),k=(R,T)=>{S.current&&S.current.adjustStyleForScrollbar(R,y),d&&d(R,T)},P=R=>{R.key==="Tab"&&(R.preventDefault(),a&&a(R,"tabKeyDown"))};let C=-1;return w.exports.Children.map(o,(R,T)=>{!w.exports.isValidElement(R)||R.props.disabled||(p==="selectedMenu"&&R.props.selected||C===-1)&&(C=T)}),I(jQ,$({classes:f,onClose:a,anchorOrigin:{vertical:"bottom",horizontal:O?"right":"left"},transformOrigin:O?FQ:VQ,PaperProps:$({component:XQ},u,{classes:$({},u.classes,{root:x.paper})}),className:x.root,open:c,ref:n,transitionDuration:h,TransitionProps:$({onEntering:k},m),ownerState:v},g,{children:I(HQ,$({onKeyDown:P,actions:S,autoFocus:i&&(C===-1||s),autoFocusItem:b,variant:p},l,{className:j(x.list,l.className),children:o}))}))});var G7=GQ;function ZQ(t){return he("MuiMenuItem",t)}const YQ=de("MuiMenuItem",["root","focusVisible","dense","disabled","divider","gutters","selected"]);var ca=YQ;const KQ=["autoFocus","component","dense","divider","disableGutters","focusVisibleClassName","role","tabIndex"],JQ=(t,e)=>{const{ownerState:n}=t;return[e.root,n.dense&&e.dense,n.divider&&e.divider,!n.disableGutters&&e.gutters]},eN=t=>{const{disabled:e,dense:n,divider:r,disableGutters:i,selected:o,classes:s}=t,a=me({root:["root",n&&"dense",e&&"disabled",!i&&"gutters",r&&"divider",o&&"selected"]},ZQ,s);return $({},s,a)},tN=G(Kn,{shouldForwardProp:t=>ts(t)||t==="classes",name:"MuiMenuItem",slot:"Root",overridesResolver:JQ})(({theme:t,ownerState:e})=>$({},t.typography.body1,{display:"flex",justifyContent:"flex-start",alignItems:"center",position:"relative",textDecoration:"none",minHeight:48,paddingTop:6,paddingBottom:6,boxSizing:"border-box",whiteSpace:"nowrap"},!e.disableGutters&&{paddingLeft:16,paddingRight:16},e.divider&&{borderBottom:`1px solid ${t.palette.divider}`,backgroundClip:"padding-box"},{"&:hover":{textDecoration:"none",backgroundColor:t.palette.action.hover,"@media (hover: none)":{backgroundColor:"transparent"}},[`&.${ca.selected}`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity),[`&.${ca.focusVisible}`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity+t.palette.action.focusOpacity)}},[`&.${ca.selected}:hover`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity+t.palette.action.hoverOpacity),"@media (hover: none)":{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity)}},[`&.${ca.focusVisible}`]:{backgroundColor:t.palette.action.focus},[`&.${ca.disabled}`]:{opacity:t.palette.action.disabledOpacity},[`& + .${cw.root}`]:{marginTop:t.spacing(1),marginBottom:t.spacing(1)},[`& + .${cw.inset}`]:{marginLeft:52},[`& .${_u.root}`]:{marginTop:0,marginBottom:0},[`& .${_u.inset}`]:{paddingLeft:36},[`& .${yw.root}`]:{minWidth:36}},!e.dense&&{[t.breakpoints.up("sm")]:{minHeight:"auto"}},e.dense&&$({minHeight:32,paddingTop:4,paddingBottom:4},t.typography.body2,{[`& .${yw.root} svg`]:{fontSize:"1.25rem"}}))),nN=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiMenuItem"}),{autoFocus:i=!1,component:o="li",dense:s=!1,divider:l=!1,disableGutters:a=!1,focusVisibleClassName:c,role:u="menuitem",tabIndex:f}=r,h=H(r,KQ),d=w.exports.useContext(Qn),p={dense:s||d.dense||!1,disableGutters:a},m=w.exports.useRef(null);Gn(()=>{i&&m.current&&m.current.focus()},[i]);const g=$({},r,{dense:p.dense,divider:l,disableGutters:a}),y=eN(r),O=Qe(m,n);let v;return r.disabled||(v=f!==void 0?f:-1),I(Qn.Provider,{value:p,children:I(tN,$({ref:O,role:u,tabIndex:v,component:o,focusVisibleClassName:j(y.focusVisible,c)},h,{ownerState:g,classes:y}))})});var Z7=nN;function rN(t){return he("MuiTooltip",t)}const iN=de("MuiTooltip",["popper","popperInteractive","popperArrow","popperClose","tooltip","tooltipArrow","touch","tooltipPlacementLeft","tooltipPlacementRight","tooltipPlacementTop","tooltipPlacementBottom","arrow"]);var hi=iN;const oN=["arrow","children","classes","components","componentsProps","describeChild","disableFocusListener","disableHoverListener","disableInteractive","disableTouchListener","enterDelay","enterNextDelay","enterTouchDelay","followCursor","id","leaveDelay","leaveTouchDelay","onClose","onOpen","open","placement","PopperComponent","PopperProps","title","TransitionComponent","TransitionProps"];function sN(t){return Math.round(t*1e5)/1e5}const lN=t=>{const{classes:e,disableInteractive:n,arrow:r,touch:i,placement:o}=t,s={popper:["popper",!n&&"popperInteractive",r&&"popperArrow"],tooltip:["tooltip",r&&"tooltipArrow",i&&"touch",`tooltipPlacement${V(o.split("-")[0])}`],arrow:["arrow"]};return me(s,rN,e)},aN=G(ew,{name:"MuiTooltip",slot:"Popper",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.popper,!n.disableInteractive&&e.popperInteractive,n.arrow&&e.popperArrow,!n.open&&e.popperClose]}})(({theme:t,ownerState:e,open:n})=>$({zIndex:t.zIndex.tooltip,pointerEvents:"none"},!e.disableInteractive&&{pointerEvents:"auto"},!n&&{pointerEvents:"none"},e.arrow&&{[`&[data-popper-placement*="bottom"] .${hi.arrow}`]:{top:0,marginTop:"-0.71em","&::before":{transformOrigin:"0 100%"}},[`&[data-popper-placement*="top"] .${hi.arrow}`]:{bottom:0,marginBottom:"-0.71em","&::before":{transformOrigin:"100% 0"}},[`&[data-popper-placement*="right"] .${hi.arrow}`]:$({},e.isRtl?{right:0,marginRight:"-0.71em"}:{left:0,marginLeft:"-0.71em"},{height:"1em",width:"0.71em","&::before":{transformOrigin:"100% 100%"}}),[`&[data-popper-placement*="left"] .${hi.arrow}`]:$({},e.isRtl?{left:0,marginLeft:"-0.71em"}:{right:0,marginRight:"-0.71em"},{height:"1em",width:"0.71em","&::before":{transformOrigin:"0 0"}})})),cN=G("div",{name:"MuiTooltip",slot:"Tooltip",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.tooltip,n.touch&&e.touch,n.arrow&&e.tooltipArrow,e[`tooltipPlacement${V(n.placement.split("-")[0])}`]]}})(({theme:t,ownerState:e})=>$({backgroundColor:ge(t.palette.grey[700],.92),borderRadius:t.shape.borderRadius,color:t.palette.common.white,fontFamily:t.typography.fontFamily,padding:"4px 8px",fontSize:t.typography.pxToRem(11),maxWidth:300,margin:2,wordWrap:"break-word",fontWeight:t.typography.fontWeightMedium},e.arrow&&{position:"relative",margin:0},e.touch&&{padding:"8px 16px",fontSize:t.typography.pxToRem(14),lineHeight:`${sN(16/14)}em`,fontWeight:t.typography.fontWeightRegular},{[`.${hi.popper}[data-popper-placement*="left"] &`]:$({transformOrigin:"right center"},e.isRtl?$({marginLeft:"14px"},e.touch&&{marginLeft:"24px"}):$({marginRight:"14px"},e.touch&&{marginRight:"24px"})),[`.${hi.popper}[data-popper-placement*="right"] &`]:$({transformOrigin:"left center"},e.isRtl?$({marginRight:"14px"},e.touch&&{marginRight:"24px"}):$({marginLeft:"14px"},e.touch&&{marginLeft:"24px"})),[`.${hi.popper}[data-popper-placement*="top"] &`]:$({transformOrigin:"center bottom",marginBottom:"14px"},e.touch&&{marginBottom:"24px"}),[`.${hi.popper}[data-popper-placement*="bottom"] &`]:$({transformOrigin:"center top",marginTop:"14px"},e.touch&&{marginTop:"24px"})})),uN=G("span",{name:"MuiTooltip",slot:"Arrow",overridesResolver:(t,e)=>e.arrow})(({theme:t})=>({overflow:"hidden",position:"absolute",width:"1em",height:"0.71em",boxSizing:"border-box",color:ge(t.palette.grey[700],.9),"&::before":{content:'""',margin:"auto",display:"block",width:"100%",height:"100%",backgroundColor:"currentColor",transform:"rotate(45deg)"}}));let Bu=!1,Om=null;function qu(t,e){return n=>{e&&e(n),t(n)}}const fN=w.exports.forwardRef(function(e,n){var r,i,o,s,l,a;const c=fe({props:e,name:"MuiTooltip"}),{arrow:u=!1,children:f,components:h={},componentsProps:d={},describeChild:p=!1,disableFocusListener:m=!1,disableHoverListener:g=!1,disableInteractive:y=!1,disableTouchListener:O=!1,enterDelay:v=100,enterNextDelay:x=0,enterTouchDelay:b=700,followCursor:S=!1,id:k,leaveDelay:P=0,leaveTouchDelay:C=1500,onClose:R,onOpen:T,open:L,placement:M="bottom",PopperComponent:q,PopperProps:F={},title:Q,TransitionComponent:E=mm,TransitionProps:D}=c,X=H(c,oN),ae=br(),W=ae.direction==="rtl",[B,Z]=w.exports.useState(),[ee,pe]=w.exports.useState(null),J=w.exports.useRef(!1),ue=y||S,se=w.exports.useRef(),we=w.exports.useRef(),Be=w.exports.useRef(),Ot=w.exports.useRef(),[Ht,Gt]=D1({controlled:L,default:!1,name:"Tooltip",state:"open"});let Ge=Ht;const Et=QT(k),ot=w.exports.useRef(),Ee=w.exports.useCallback(()=>{ot.current!==void 0&&(document.body.style.WebkitUserSelect=ot.current,ot.current=void 0),clearTimeout(Ot.current)},[]);w.exports.useEffect(()=>()=>{clearTimeout(se.current),clearTimeout(we.current),clearTimeout(Be.current),Ee()},[Ee]);const Wr=ve=>{clearTimeout(Om),Bu=!0,Gt(!0),T&&!Ge&&T(ve)},Zt=Dn(ve=>{clearTimeout(Om),Om=setTimeout(()=>{Bu=!1},800+P),Gt(!1),R&&Ge&&R(ve),clearTimeout(se.current),se.current=setTimeout(()=>{J.current=!1},ae.transitions.duration.shortest)}),Ke=ve=>{J.current&&ve.type!=="touchstart"||(B&&B.removeAttribute("title"),clearTimeout(we.current),clearTimeout(Be.current),v||Bu&&x?we.current=setTimeout(()=>{Wr(ve)},Bu?x:v):Wr(ve))},ft=ve=>{clearTimeout(we.current),clearTimeout(Be.current),Be.current=setTimeout(()=>{Zt(ve)},P)},{isFocusVisibleRef:Pn,onBlur:fr,onFocus:un,ref:Qr}=_p(),[,_n]=w.exports.useState(!1),Cn=ve=>{fr(ve),Pn.current===!1&&(_n(!1),ft(ve))},N=ve=>{B||Z(ve.currentTarget),un(ve),Pn.current===!0&&(_n(!0),Ke(ve))},ie=ve=>{J.current=!0;const vt=f.props;vt.onTouchStart&&vt.onTouchStart(ve)},ye=Ke,Oe=ft,tt=ve=>{ie(ve),clearTimeout(Be.current),clearTimeout(se.current),Ee(),ot.current=document.body.style.WebkitUserSelect,document.body.style.WebkitUserSelect="none",Ot.current=setTimeout(()=>{document.body.style.WebkitUserSelect=ot.current,Ke(ve)},b)},Yt=ve=>{f.props.onTouchEnd&&f.props.onTouchEnd(ve),Ee(),clearTimeout(Be.current),Be.current=setTimeout(()=>{Zt(ve)},C)};w.exports.useEffect(()=>{if(!Ge)return;function ve(vt){(vt.key==="Escape"||vt.key==="Esc")&&Zt(vt)}return document.addEventListener("keydown",ve),()=>{document.removeEventListener("keydown",ve)}},[Zt,Ge]);const Nr=Qe(Z,n),co=Qe(Qr,Nr),MP=Qe(f.ref,co);Q===""&&(Ge=!1);const Cs=w.exports.useRef({x:0,y:0}),fh=w.exports.useRef(),AP=ve=>{const vt=f.props;vt.onMouseMove&&vt.onMouseMove(ve),Cs.current={x:ve.clientX,y:ve.clientY},fh.current&&fh.current.update()},Rs={},hh=typeof Q=="string";p?(Rs.title=!Ge&&hh&&!g?Q:null,Rs["aria-describedby"]=Ge?Et:null):(Rs["aria-label"]=hh?Q:null,Rs["aria-labelledby"]=Ge&&!hh?Et:null);const Rn=$({},Rs,X,f.props,{className:j(X.className,f.props.className),onTouchStart:ie,ref:MP},S?{onMouseMove:AP}:{}),Ts={};O||(Rn.onTouchStart=tt,Rn.onTouchEnd=Yt),g||(Rn.onMouseOver=qu(ye,Rn.onMouseOver),Rn.onMouseLeave=qu(Oe,Rn.onMouseLeave),ue||(Ts.onMouseOver=ye,Ts.onMouseLeave=Oe)),m||(Rn.onFocus=qu(N,Rn.onFocus),Rn.onBlur=qu(Cn,Rn.onBlur),ue||(Ts.onFocus=N,Ts.onBlur=Cn));const DP=w.exports.useMemo(()=>{var ve;let vt=[{name:"arrow",enabled:Boolean(ee),options:{element:ee,padding:4}}];return(ve=F.popperOptions)!=null&&ve.modifiers&&(vt=vt.concat(F.popperOptions.modifiers)),$({},F.popperOptions,{modifiers:vt})},[ee,F]),Is=$({},c,{isRtl:W,arrow:u,disableInteractive:ue,placement:M,PopperComponentProp:q,touch:J.current}),dh=lN(Is),R0=(r=h.Popper)!=null?r:aN,T0=(i=(o=h.Transition)!=null?o:E)!=null?i:mm,I0=(s=h.Tooltip)!=null?s:cN,E0=(l=h.Arrow)!=null?l:uN,LP=Pu(R0,$({},F,d.popper),Is),WP=Pu(T0,$({},D,d.transition),Is),QP=Pu(I0,$({},d.tooltip),Is),NP=Pu(E0,$({},d.arrow),Is);return Ue(w.exports.Fragment,{children:[w.exports.cloneElement(f,Rn),I(R0,$({as:q!=null?q:ew,placement:M,anchorEl:S?{getBoundingClientRect:()=>({top:Cs.current.y,left:Cs.current.x,right:Cs.current.x,bottom:Cs.current.y,width:0,height:0})}:B,popperRef:fh,open:B?Ge:!1,id:Et,transition:!0},Ts,LP,{className:j(dh.popper,F==null?void 0:F.className,(a=d.popper)==null?void 0:a.className),popperOptions:DP,children:({TransitionProps:ve})=>{var vt,M0;return I(T0,$({timeout:ae.transitions.duration.shorter},ve,WP,{children:Ue(I0,$({},QP,{className:j(dh.tooltip,(vt=d.tooltip)==null?void 0:vt.className),children:[Q,u?I(E0,$({},NP,{className:j(dh.arrow,(M0=d.arrow)==null?void 0:M0.className),ref:pe})):null]}))}))}}))]})});var Y7=fN;function hN(t){return he("MuiTab",t)}const dN=de("MuiTab",["root","labelIcon","textColorInherit","textColorPrimary","textColorSecondary","selected","disabled","fullWidth","wrapped","iconWrapper"]);var Xi=dN;const pN=["className","disabled","disableFocusRipple","fullWidth","icon","iconPosition","indicator","label","onChange","onClick","onFocus","selected","selectionFollowsFocus","textColor","value","wrapped"],mN=t=>{const{classes:e,textColor:n,fullWidth:r,wrapped:i,icon:o,label:s,selected:l,disabled:a}=t,c={root:["root",o&&s&&"labelIcon",`textColor${V(n)}`,r&&"fullWidth",i&&"wrapped",l&&"selected",a&&"disabled"],iconWrapper:["iconWrapper"]};return me(c,hN,e)},gN=G(Kn,{name:"MuiTab",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.label&&n.icon&&e.labelIcon,e[`textColor${V(n.textColor)}`],n.fullWidth&&e.fullWidth,n.wrapped&&e.wrapped]}})(({theme:t,ownerState:e})=>$({},t.typography.button,{maxWidth:360,minWidth:90,position:"relative",minHeight:48,flexShrink:0,padding:"12px 16px",overflow:"hidden",whiteSpace:"normal",textAlign:"center"},e.label&&{flexDirection:e.iconPosition==="top"||e.iconPosition==="bottom"?"column":"row"},{lineHeight:1.25},e.icon&&e.label&&{minHeight:72,paddingTop:9,paddingBottom:9,[`& > .${Xi.iconWrapper}`]:$({},e.iconPosition==="top"&&{marginBottom:6},e.iconPosition==="bottom"&&{marginTop:6},e.iconPosition==="start"&&{marginRight:t.spacing(1)},e.iconPosition==="end"&&{marginLeft:t.spacing(1)})},e.textColor==="inherit"&&{color:"inherit",opacity:.6,[`&.${Xi.selected}`]:{opacity:1},[`&.${Xi.disabled}`]:{opacity:t.palette.action.disabledOpacity}},e.textColor==="primary"&&{color:t.palette.text.secondary,[`&.${Xi.selected}`]:{color:t.palette.primary.main},[`&.${Xi.disabled}`]:{color:t.palette.text.disabled}},e.textColor==="secondary"&&{color:t.palette.text.secondary,[`&.${Xi.selected}`]:{color:t.palette.secondary.main},[`&.${Xi.disabled}`]:{color:t.palette.text.disabled}},e.fullWidth&&{flexShrink:1,flexGrow:1,flexBasis:0,maxWidth:"none"},e.wrapped&&{fontSize:t.typography.pxToRem(12)})),yN=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTab"}),{className:i,disabled:o=!1,disableFocusRipple:s=!1,fullWidth:l,icon:a,iconPosition:c="top",indicator:u,label:f,onChange:h,onClick:d,onFocus:p,selected:m,selectionFollowsFocus:g,textColor:y="inherit",value:O,wrapped:v=!1}=r,x=H(r,pN),b=$({},r,{disabled:o,disableFocusRipple:s,selected:m,icon:!!a,iconPosition:c,label:!!f,fullWidth:l,textColor:y,wrapped:v}),S=mN(b),k=a&&f&&w.exports.isValidElement(a)?w.exports.cloneElement(a,{className:j(S.iconWrapper,a.props.className)}):a,P=R=>{!m&&h&&h(R,O),d&&d(R)},C=R=>{g&&!m&&h&&h(R,O),p&&p(R)};return Ue(gN,$({focusRipple:!s,className:j(S.root,i),ref:n,role:"tab","aria-selected":m,disabled:o,onClick:P,onFocus:C,ownerState:b,tabIndex:m?0:-1},x,{children:[c==="top"||c==="start"?Ue(w.exports.Fragment,{children:[k,f]}):Ue(w.exports.Fragment,{children:[f,k]}),u]}))});var K7=yN;const ON=w.exports.createContext();var $w=ON;function vN(t){return he("MuiTable",t)}de("MuiTable",["root","stickyHeader"]);const xN=["className","component","padding","size","stickyHeader"],wN=t=>{const{classes:e,stickyHeader:n}=t;return me({root:["root",n&&"stickyHeader"]},vN,e)},bN=G("table",{name:"MuiTable",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.stickyHeader&&e.stickyHeader]}})(({theme:t,ownerState:e})=>$({display:"table",width:"100%",borderCollapse:"collapse",borderSpacing:0,"& caption":$({},t.typography.body2,{padding:t.spacing(2),color:t.palette.text.secondary,textAlign:"left",captionSide:"bottom"})},e.stickyHeader&&{borderCollapse:"separate"})),Sw="table",$N=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTable"}),{className:i,component:o=Sw,padding:s="normal",size:l="medium",stickyHeader:a=!1}=r,c=H(r,xN),u=$({},r,{component:o,padding:s,size:l,stickyHeader:a}),f=wN(u),h=w.exports.useMemo(()=>({padding:s,size:l,stickyHeader:a}),[s,l,a]);return I($w.Provider,{value:h,children:I(bN,$({as:o,role:o===Sw?null:"table",ref:n,className:j(f.root,i),ownerState:u},c))})});var J7=$N;const SN=w.exports.createContext();var vm=SN;function kN(t){return he("MuiTableBody",t)}de("MuiTableBody",["root"]);const PN=["className","component"],CN=t=>{const{classes:e}=t;return me({root:["root"]},kN,e)},RN=G("tbody",{name:"MuiTableBody",slot:"Root",overridesResolver:(t,e)=>e.root})({display:"table-row-group"}),TN={variant:"body"},kw="tbody",IN=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTableBody"}),{className:i,component:o=kw}=r,s=H(r,PN),l=$({},r,{component:o}),a=CN(l);return I(vm.Provider,{value:TN,children:I(RN,$({className:j(a.root,i),as:o,ref:n,role:o===kw?null:"rowgroup",ownerState:l},s))})});var eF=IN;function EN(t){return he("MuiTableCell",t)}const MN=de("MuiTableCell",["root","head","body","footer","sizeSmall","sizeMedium","paddingCheckbox","paddingNone","alignLeft","alignCenter","alignRight","alignJustify","stickyHeader"]);var AN=MN;const DN=["align","className","component","padding","scope","size","sortDirection","variant"],LN=t=>{const{classes:e,variant:n,align:r,padding:i,size:o,stickyHeader:s}=t,l={root:["root",n,s&&"stickyHeader",r!=="inherit"&&`align${V(r)}`,i!=="normal"&&`padding${V(i)}`,`size${V(o)}`]};return me(l,EN,e)},WN=G("td",{name:"MuiTableCell",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,e[n.variant],e[`size${V(n.size)}`],n.padding!=="normal"&&e[`padding${V(n.padding)}`],n.align!=="inherit"&&e[`align${V(n.align)}`],n.stickyHeader&&e.stickyHeader]}})(({theme:t,ownerState:e})=>$({},t.typography.body2,{display:"table-cell",verticalAlign:"inherit",borderBottom:`1px solid
    ${t.palette.mode==="light"?fx(ge(t.palette.divider,1),.88):ux(ge(t.palette.divider,1),.68)}`,textAlign:"left",padding:16},e.variant==="head"&&{color:t.palette.text.primary,lineHeight:t.typography.pxToRem(24),fontWeight:t.typography.fontWeightMedium},e.variant==="body"&&{color:t.palette.text.primary},e.variant==="footer"&&{color:t.palette.text.secondary,lineHeight:t.typography.pxToRem(21),fontSize:t.typography.pxToRem(12)},e.size==="small"&&{padding:"6px 16px",[`&.${AN.paddingCheckbox}`]:{width:24,padding:"0 12px 0 16px","& > *":{padding:0}}},e.padding==="checkbox"&&{width:48,padding:"0 0 0 4px"},e.padding==="none"&&{padding:0},e.align==="left"&&{textAlign:"left"},e.align==="center"&&{textAlign:"center"},e.align==="right"&&{textAlign:"right",flexDirection:"row-reverse"},e.align==="justify"&&{textAlign:"justify"},e.stickyHeader&&{position:"sticky",top:0,zIndex:2,backgroundColor:t.palette.background.default})),QN=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTableCell"}),{align:i="inherit",className:o,component:s,padding:l,scope:a,size:c,sortDirection:u,variant:f}=r,h=H(r,DN),d=w.exports.useContext($w),p=w.exports.useContext(vm),m=p&&p.variant==="head";let g;s?g=s:g=m?"th":"td";let y=a;!y&&m&&(y="col");const O=f||p&&p.variant,v=$({},r,{align:i,component:g,padding:l||(d&&d.padding?d.padding:"normal"),size:c||(d&&d.size?d.size:"medium"),sortDirection:u,stickyHeader:O==="head"&&d&&d.stickyHeader,variant:O}),x=LN(v);let b=null;return u&&(b=u==="asc"?"ascending":"descending"),I(WN,$({as:g,ref:n,className:j(x.root,o),"aria-sort":b,scope:y,ownerState:v},h))});var tF=QN;function NN(t){return he("MuiToolbar",t)}de("MuiToolbar",["root","gutters","regular","dense"]);const _N=["className","component","disableGutters","variant"],BN=t=>{const{classes:e,disableGutters:n,variant:r}=t;return me({root:["root",!n&&"gutters",r]},NN,e)},qN=G("div",{name:"MuiToolbar",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,!n.disableGutters&&e.gutters,e[n.variant]]}})(({theme:t,ownerState:e})=>$({position:"relative",display:"flex",alignItems:"center"},!e.disableGutters&&{paddingLeft:t.spacing(2),paddingRight:t.spacing(2),[t.breakpoints.up("sm")]:{paddingLeft:t.spacing(3),paddingRight:t.spacing(3)}},e.variant==="dense"&&{minHeight:48}),({theme:t,ownerState:e})=>e.variant==="regular"&&t.mixins.toolbar),zN=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiToolbar"}),{className:i,component:o="div",disableGutters:s=!1,variant:l="regular"}=r,a=H(r,_N),c=$({},r,{component:o,disableGutters:s,variant:l}),u=BN(c);return I(qN,$({as:o,className:j(u.root,i),ref:n,ownerState:c},a))});var nF=zN,FN=Ne(I("path",{d:"M15.41 16.09l-4.58-4.59 4.58-4.59L14 5.5l-6 6 6 6z"}),"KeyboardArrowLeft"),VN=Ne(I("path",{d:"M8.59 16.34l4.58-4.59-4.58-4.59L10 5.75l6 6-6 6z"}),"KeyboardArrowRight");function UN(t){return he("MuiTableRow",t)}const jN=de("MuiTableRow",["root","selected","hover","head","footer"]);var Pw=jN;const XN=["className","component","hover","selected"],HN=t=>{const{classes:e,selected:n,hover:r,head:i,footer:o}=t;return me({root:["root",n&&"selected",r&&"hover",i&&"head",o&&"footer"]},UN,e)},GN=G("tr",{name:"MuiTableRow",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.head&&e.head,n.footer&&e.footer]}})(({theme:t})=>({color:"inherit",display:"table-row",verticalAlign:"middle",outline:0,[`&.${Pw.hover}:hover`]:{backgroundColor:t.palette.action.hover},[`&.${Pw.selected}`]:{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity),"&:hover":{backgroundColor:ge(t.palette.primary.main,t.palette.action.selectedOpacity+t.palette.action.hoverOpacity)}}})),Cw="tr",ZN=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTableRow"}),{className:i,component:o=Cw,hover:s=!1,selected:l=!1}=r,a=H(r,XN),c=w.exports.useContext(vm),u=$({},r,{component:o,hover:s,selected:l,head:c&&c.variant==="head",footer:c&&c.variant==="footer"}),f=HN(u);return I(GN,$({as:o,ref:n,className:j(f.root,i),role:o===Cw?null:"row",ownerState:u},a))});var rF=ZN;function YN(t){return(1+Math.sin(Math.PI*t-Math.PI/2))/2}function KN(t,e,n,r={},i=()=>{}){const{ease:o=YN,duration:s=300}=r;let l=null;const a=e[t];let c=!1;const u=()=>{c=!0},f=h=>{if(c){i(new Error("Animation cancelled"));return}l===null&&(l=h);const d=Math.min(1,(h-l)/s);if(e[t]=o(d)*(n-a)+a,d>=1){requestAnimationFrame(()=>{i(null)});return}requestAnimationFrame(f)};return a===n?(i(new Error("Element already at target position")),u):(requestAnimationFrame(f),u)}const JN=["onChange"],e3={width:99,height:99,position:"absolute",top:-9999,overflow:"scroll"};function t3(t){const{onChange:e}=t,n=H(t,JN),r=w.exports.useRef(),i=w.exports.useRef(null),o=()=>{r.current=i.current.offsetHeight-i.current.clientHeight};return w.exports.useEffect(()=>{const s=jl(()=>{const a=r.current;o(),a!==r.current&&e(r.current)}),l=xr(i.current);return l.addEventListener("resize",s),()=>{s.clear(),l.removeEventListener("resize",s)}},[e]),w.exports.useEffect(()=>{o(),e(r.current)},[e]),I("div",$({style:e3,ref:i},n))}function n3(t){return he("MuiTabScrollButton",t)}const r3=de("MuiTabScrollButton",["root","vertical","horizontal","disabled"]);var i3=r3,Rw,Tw;const o3=["className","direction","orientation","disabled"],s3=t=>{const{classes:e,orientation:n,disabled:r}=t;return me({root:["root",n,r&&"disabled"]},n3,e)},l3=G(Kn,{name:"MuiTabScrollButton",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.root,n.orientation&&e[n.orientation]]}})(({ownerState:t})=>$({width:40,flexShrink:0,opacity:.8,[`&.${i3.disabled}`]:{opacity:0}},t.orientation==="vertical"&&{width:"100%",height:40,"& svg":{transform:`rotate(${t.isRtl?-90:90}deg)`}})),a3=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTabScrollButton"}),{className:i,direction:o}=r,s=H(r,o3),a=br().direction==="rtl",c=$({isRtl:a},r),u=s3(c);return I(l3,$({component:"div",className:j(u.root,i),ref:n,role:null,ownerState:c,tabIndex:null},s,{children:o==="left"?Rw||(Rw=I(FN,{fontSize:"small"})):Tw||(Tw=I(VN,{fontSize:"small"}))}))});var c3=a3;function u3(t){return he("MuiTabs",t)}const f3=de("MuiTabs",["root","vertical","flexContainer","flexContainerVertical","centered","scroller","fixed","scrollableX","scrollableY","hideScrollbar","scrollButtons","scrollButtonsHideMobile","indicator"]);var xm=f3;const h3=["aria-label","aria-labelledby","action","centered","children","className","component","allowScrollButtonsMobile","indicatorColor","onChange","orientation","ScrollButtonComponent","scrollButtons","selectionFollowsFocus","TabIndicatorProps","TabScrollButtonProps","textColor","value","variant","visibleScrollbar"],Iw=(t,e)=>t===e?t.firstChild:e&&e.nextElementSibling?e.nextElementSibling:t.firstChild,Ew=(t,e)=>t===e?t.lastChild:e&&e.previousElementSibling?e.previousElementSibling:t.lastChild,zu=(t,e,n)=>{let r=!1,i=n(t,e);for(;i;){if(i===t.firstChild){if(r)return;r=!0}const o=i.disabled||i.getAttribute("aria-disabled")==="true";if(!i.hasAttribute("tabindex")||o)i=n(t,i);else{i.focus();return}}},d3=t=>{const{vertical:e,fixed:n,hideScrollbar:r,scrollableX:i,scrollableY:o,centered:s,scrollButtonsHideMobile:l,classes:a}=t;return me({root:["root",e&&"vertical"],scroller:["scroller",n&&"fixed",r&&"hideScrollbar",i&&"scrollableX",o&&"scrollableY"],flexContainer:["flexContainer",e&&"flexContainerVertical",s&&"centered"],indicator:["indicator"],scrollButtons:["scrollButtons",l&&"scrollButtonsHideMobile"],scrollableX:[i&&"scrollableX"],hideScrollbar:[r&&"hideScrollbar"]},u3,a)},p3=G("div",{name:"MuiTabs",slot:"Root",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[{[`& .${xm.scrollButtons}`]:e.scrollButtons},{[`& .${xm.scrollButtons}`]:n.scrollButtonsHideMobile&&e.scrollButtonsHideMobile},e.root,n.vertical&&e.vertical]}})(({ownerState:t,theme:e})=>$({overflow:"hidden",minHeight:48,WebkitOverflowScrolling:"touch",display:"flex"},t.vertical&&{flexDirection:"column"},t.scrollButtonsHideMobile&&{[`& .${xm.scrollButtons}`]:{[e.breakpoints.down("sm")]:{display:"none"}}})),m3=G("div",{name:"MuiTabs",slot:"Scroller",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.scroller,n.fixed&&e.fixed,n.hideScrollbar&&e.hideScrollbar,n.scrollableX&&e.scrollableX,n.scrollableY&&e.scrollableY]}})(({ownerState:t})=>$({position:"relative",display:"inline-block",flex:"1 1 auto",whiteSpace:"nowrap"},t.fixed&&{overflowX:"hidden",width:"100%"},t.hideScrollbar&&{scrollbarWidth:"none","&::-webkit-scrollbar":{display:"none"}},t.scrollableX&&{overflowX:"auto",overflowY:"hidden"},t.scrollableY&&{overflowY:"auto",overflowX:"hidden"})),g3=G("div",{name:"MuiTabs",slot:"FlexContainer",overridesResolver:(t,e)=>{const{ownerState:n}=t;return[e.flexContainer,n.vertical&&e.flexContainerVertical,n.centered&&e.centered]}})(({ownerState:t})=>$({display:"flex"},t.vertical&&{flexDirection:"column"},t.centered&&{justifyContent:"center"})),y3=G("span",{name:"MuiTabs",slot:"Indicator",overridesResolver:(t,e)=>e.indicator})(({ownerState:t,theme:e})=>$({position:"absolute",height:2,bottom:0,width:"100%",transition:e.transitions.create()},t.indicatorColor==="primary"&&{backgroundColor:e.palette.primary.main},t.indicatorColor==="secondary"&&{backgroundColor:e.palette.secondary.main},t.vertical&&{height:"100%",width:2,right:0})),O3=G(t3,{name:"MuiTabs",slot:"ScrollbarSize"})({overflowX:"auto",overflowY:"hidden",scrollbarWidth:"none","&::-webkit-scrollbar":{display:"none"}}),Mw={},v3=w.exports.forwardRef(function(e,n){const r=fe({props:e,name:"MuiTabs"}),i=br(),o=i.direction==="rtl",{"aria-label":s,"aria-labelledby":l,action:a,centered:c=!1,children:u,className:f,component:h="div",allowScrollButtonsMobile:d=!1,indicatorColor:p="primary",onChange:m,orientation:g="horizontal",ScrollButtonComponent:y=c3,scrollButtons:O="auto",selectionFollowsFocus:v,TabIndicatorProps:x={},TabScrollButtonProps:b={},textColor:S="primary",value:k,variant:P="standard",visibleScrollbar:C=!1}=r,R=H(r,h3),T=P==="scrollable",L=g==="vertical",M=L?"scrollTop":"scrollLeft",q=L?"top":"left",F=L?"bottom":"right",Q=L?"clientHeight":"clientWidth",E=L?"height":"width",D=$({},r,{component:h,allowScrollButtonsMobile:d,indicatorColor:p,orientation:g,vertical:L,scrollButtons:O,textColor:S,variant:P,visibleScrollbar:C,fixed:!T,hideScrollbar:T&&!C,scrollableX:T&&!L,scrollableY:T&&L,centered:c&&!T,scrollButtonsHideMobile:!d}),X=d3(D),[ae,W]=w.exports.useState(!1),[B,Z]=w.exports.useState(Mw),[ee,pe]=w.exports.useState({start:!1,end:!1}),[J,ue]=w.exports.useState({overflow:"hidden",scrollbarWidth:0}),se=new Map,we=w.exports.useRef(null),Be=w.exports.useRef(null),Ot=()=>{const N=we.current;let ie;if(N){const Oe=N.getBoundingClientRect();ie={clientWidth:N.clientWidth,scrollLeft:N.scrollLeft,scrollTop:N.scrollTop,scrollLeftNormalized:N1(N,i.direction),scrollWidth:N.scrollWidth,top:Oe.top,bottom:Oe.bottom,left:Oe.left,right:Oe.right}}let ye;if(N&&k!==!1){const Oe=Be.current.children;if(Oe.length>0){const tt=Oe[se.get(k)];ye=tt?tt.getBoundingClientRect():null}}return{tabsMeta:ie,tabMeta:ye}},Ht=Dn(()=>{const{tabsMeta:N,tabMeta:ie}=Ot();let ye=0,Oe;if(L)Oe="top",ie&&N&&(ye=ie.top-N.top+N.scrollTop);else if(Oe=o?"right":"left",ie&&N){const Yt=o?N.scrollLeftNormalized+N.clientWidth-N.scrollWidth:N.scrollLeft;ye=(o?-1:1)*(ie[Oe]-N[Oe]+Yt)}const tt={[Oe]:ye,[E]:ie?ie[E]:0};if(isNaN(B[Oe])||isNaN(B[E]))Z(tt);else{const Yt=Math.abs(B[Oe]-tt[Oe]),Nr=Math.abs(B[E]-tt[E]);(Yt>=1||Nr>=1)&&Z(tt)}}),Gt=(N,{animation:ie=!0}={})=>{ie?KN(M,we.current,N,{duration:i.transitions.duration.standard}):we.current[M]=N},Ge=N=>{let ie=we.current[M];L?ie+=N:(ie+=N*(o?-1:1),ie*=o&&Q1()==="reverse"?-1:1),Gt(ie)},Et=()=>{const N=we.current[Q];let ie=0;const ye=Array.from(Be.current.children);for(let Oe=0;Oe<ye.length;Oe+=1){const tt=ye[Oe];if(ie+tt[Q]>N)break;ie+=tt[Q]}return ie},ot=()=>{Ge(-1*Et())},Ee=()=>{Ge(Et())},Wr=w.exports.useCallback(N=>{ue({overflow:null,scrollbarWidth:N})},[]),Zt=()=>{const N={};N.scrollbarSizeListener=T?I(O3,{onChange:Wr,className:j(X.scrollableX,X.hideScrollbar)}):null;const ie=ee.start||ee.end,ye=T&&(O==="auto"&&ie||O===!0);return N.scrollButtonStart=ye?I(y,$({orientation:g,direction:o?"right":"left",onClick:ot,disabled:!ee.start},b,{className:j(X.scrollButtons,b.className)})):null,N.scrollButtonEnd=ye?I(y,$({orientation:g,direction:o?"left":"right",onClick:Ee,disabled:!ee.end},b,{className:j(X.scrollButtons,b.className)})):null,N},Ke=Dn(N=>{const{tabsMeta:ie,tabMeta:ye}=Ot();if(!(!ye||!ie)){if(ye[q]<ie[q]){const Oe=ie[M]+(ye[q]-ie[q]);Gt(Oe,{animation:N})}else if(ye[F]>ie[F]){const Oe=ie[M]+(ye[F]-ie[F]);Gt(Oe,{animation:N})}}}),ft=Dn(()=>{if(T&&O!==!1){const{scrollTop:N,scrollHeight:ie,clientHeight:ye,scrollWidth:Oe,clientWidth:tt}=we.current;let Yt,Nr;if(L)Yt=N>1,Nr=N<ie-ye-1;else{const co=N1(we.current,i.direction);Yt=o?co<Oe-tt-1:co>1,Nr=o?co>1:co<Oe-tt-1}(Yt!==ee.start||Nr!==ee.end)&&pe({start:Yt,end:Nr})}});w.exports.useEffect(()=>{const N=jl(()=>{Ht(),ft()}),ie=xr(we.current);ie.addEventListener("resize",N);let ye;return typeof ResizeObserver!="undefined"&&(ye=new ResizeObserver(N),Array.from(Be.current.children).forEach(Oe=>{ye.observe(Oe)})),()=>{N.clear(),ie.removeEventListener("resize",N),ye&&ye.disconnect()}},[Ht,ft]);const Pn=w.exports.useMemo(()=>jl(()=>{ft()}),[ft]);w.exports.useEffect(()=>()=>{Pn.clear()},[Pn]),w.exports.useEffect(()=>{W(!0)},[]),w.exports.useEffect(()=>{Ht(),ft()}),w.exports.useEffect(()=>{Ke(Mw!==B)},[Ke,B]),w.exports.useImperativeHandle(a,()=>({updateIndicator:Ht,updateScrollButtons:ft}),[Ht,ft]);const fr=I(y3,$({},x,{className:j(X.indicator,x.className),ownerState:D,style:$({},B,x.style)}));let un=0;const Qr=w.exports.Children.map(u,N=>{if(!w.exports.isValidElement(N))return null;const ie=N.props.value===void 0?un:N.props.value;se.set(ie,un);const ye=ie===k;return un+=1,w.exports.cloneElement(N,$({fullWidth:P==="fullWidth",indicator:ye&&!ae&&fr,selected:ye,selectionFollowsFocus:v,onChange:m,textColor:S,value:ie},un===1&&k===!1&&!N.props.tabIndex?{tabIndex:0}:{}))}),_n=N=>{const ie=Be.current,ye=Pt(ie).activeElement;if(ye.getAttribute("role")!=="tab")return;let tt=g==="horizontal"?"ArrowLeft":"ArrowUp",Yt=g==="horizontal"?"ArrowRight":"ArrowDown";switch(g==="horizontal"&&o&&(tt="ArrowRight",Yt="ArrowLeft"),N.key){case tt:N.preventDefault(),zu(ie,ye,Ew);break;case Yt:N.preventDefault(),zu(ie,ye,Iw);break;case"Home":N.preventDefault(),zu(ie,null,Iw);break;case"End":N.preventDefault(),zu(ie,null,Ew);break}},Cn=Zt();return Ue(p3,$({className:j(X.root,f),ownerState:D,ref:n,as:h},R,{children:[Cn.scrollButtonStart,Cn.scrollbarSizeListener,Ue(m3,{className:X.scroller,ownerState:D,style:{overflow:J.overflow,[L?`margin${o?"Left":"Right"}`:"marginBottom"]:C?void 0:-J.scrollbarWidth},ref:we,onScroll:Pn,children:[I(g3,{"aria-label":s,"aria-labelledby":l,"aria-orientation":g==="vertical"?"vertical":null,className:X.flexContainer,ownerState:D,onKeyDown:_n,ref:Be,role:"tablist",children:Qr}),ae&&fr]}),Cn.scrollButtonEnd]}))});var iF=v3;let cs="lc,34,7n,7,7b,19,,,,2,,2,,,20,b,1c,l,g,,2t,7,2,6,2,2,,4,z,,u,r,2j,b,1m,9,9,,o,4,,9,,3,,5,17,3,3b,f,,w,1j,,,,4,8,4,,3,7,a,2,t,,1m,,,,2,4,8,,9,,a,2,q,,2,2,1l,,4,2,4,2,2,3,3,,u,2,3,,b,2,1l,,4,5,,2,4,,k,2,m,6,,,1m,,,2,,4,8,,7,3,a,2,u,,1n,,,,c,,9,,14,,3,,1l,3,5,3,,4,7,2,b,2,t,,1m,,2,,2,,3,,5,2,7,2,b,2,s,2,1l,2,,,2,4,8,,9,,a,2,t,,20,,4,,2,3,,,8,,29,,2,7,c,8,2q,,2,9,b,6,22,2,r,,,,,,1j,e,,5,,2,5,b,,10,9,,2u,4,,6,,2,2,2,p,2,4,3,g,4,d,,2,2,6,,f,,jj,3,qa,3,t,3,t,2,u,2,1s,2,,7,8,,2,b,9,,19,3,3b,2,y,,3a,3,4,2,9,,6,3,63,2,2,,1m,,,7,,,,,2,8,6,a,2,,1c,h,1r,4,1c,7,,,5,,14,9,c,2,w,4,2,2,,3,1k,,,2,3,,,3,1m,8,2,2,48,3,,d,,7,4,,6,,3,2,5i,1m,,5,ek,,5f,x,2da,3,3x,,2o,w,fe,6,2x,2,n9w,4,,a,w,2,28,2,7k,,3,,4,,p,2,5,,47,2,q,i,d,,12,8,p,b,1a,3,1c,,2,4,2,2,13,,1v,6,2,2,2,2,c,,8,,1b,,1f,,,3,2,2,5,2,,,16,2,8,,6m,,2,,4,,fn4,,kh,g,g,g,a6,2,gt,,6a,,45,5,1ae,3,,2,5,4,14,3,4,,4l,2,fx,4,ar,2,49,b,4w,,1i,f,1k,3,1d,4,2,2,1x,3,10,5,,8,1q,,c,2,1g,9,a,4,2,,2n,3,2,,,2,6,,4g,,3,8,l,2,1l,2,,,,,m,,e,7,3,5,5f,8,2,3,,,n,,29,,2,6,,,2,,,2,,2,6j,,2,4,6,2,,2,r,2,2d,8,2,,,2,2y,,,,2,6,,,2t,3,2,4,,5,77,9,,2,6t,,a,2,,,4,,40,4,2,2,4,,w,a,14,6,2,4,8,,9,6,2,3,1a,d,,2,ba,7,,6,,,2a,m,2,7,,2,,2,3e,6,3,,,2,,7,,,20,2,3,,,,9n,2,f0b,5,1n,7,t4,,1r,4,29,,f5k,2,43q,,,3,4,5,8,8,2,7,u,4,44,3,1iz,1j,4,1e,8,,e,,m,5,,f,11s,7,,h,2,7,,2,,5,79,7,c5,4,15s,7,31,7,240,5,gx7k,2o,3k,6o".split(",").map(t=>t?parseInt(t,36):1);for(let t=1;t<cs.length;t++)cs[t]+=cs[t-1];function x3(t){for(let e=1;e<cs.length;e+=2)if(cs[e]>t)return cs[e-1]<=t;return!1}function Aw(t){return t>=127462&&t<=127487}const Dw=8205;function nn(t,e,n=!0,r=!0){return(n?Lw:w3)(t,e,r)}function Lw(t,e,n){if(e==t.length)return e;e&&Ww(t.charCodeAt(e))&&Qw(t.charCodeAt(e-1))&&e--;let r=Ct(t,e);for(e+=wn(r);e<t.length;){let i=Ct(t,e);if(r==Dw||i==Dw||n&&x3(i))e+=wn(i),r=i;else if(Aw(i)){let o=0,s=e-2;for(;s>=0&&Aw(Ct(t,s));)o++,s-=2;if(o%2==0)break;e+=2}else break}return e}function w3(t,e,n){for(;e>0;){let r=Lw(t,e-2,n);if(r<e)return r;e--}return 0}function Ww(t){return t>=56320&&t<57344}function Qw(t){return t>=55296&&t<56320}function Ct(t,e){let n=t.charCodeAt(e);if(!Qw(n)||e+1==t.length)return n;let r=t.charCodeAt(e+1);return Ww(r)?(n-55296<<10)+(r-56320)+65536:n}function wm(t){return t<=65535?String.fromCharCode(t):(t-=65536,String.fromCharCode((t>>10)+55296,(t&1023)+56320))}function wn(t){return t<65536?1:2}function ua(t,e,n=t.length){let r=0;for(let i=0;i<n;)t.charCodeAt(i)==9?(r+=e-r%e,i++):(r++,i=nn(t,i));return r}function bm(t,e,n,r){for(let i=0,o=0;;){if(o>=e)return i;if(i==t.length)break;o+=t.charCodeAt(i)==9?n-o%n:1,i=nn(t,i)}return r===!0?-1:t.length}class Ie{constructor(){}lineAt(e){if(e<0||e>this.length)throw new RangeError(`Invalid position ${e} in document of length ${this.length}`);return this.lineInner(e,!1,1,0)}line(e){if(e<1||e>this.lines)throw new RangeError(`Invalid line number ${e} in ${this.lines}-line document`);return this.lineInner(e,!0,1,0)}replace(e,n,r){let i=[];return this.decompose(0,e,i,2),r.length&&r.decompose(0,r.length,i,1|2),this.decompose(n,this.length,i,1),Jn.from(i,this.length-(n-e)+r.length)}append(e){return this.replace(this.length,this.length,e)}slice(e,n=this.length){let r=[];return this.decompose(e,n,r,0),Jn.from(r,n-e)}eq(e){if(e==this)return!0;if(e.length!=this.length||e.lines!=this.lines)return!1;let n=this.scanIdentical(e,1),r=this.length-this.scanIdentical(e,-1),i=new fa(this),o=new fa(e);for(let s=n,l=n;;){if(i.next(s),o.next(s),s=0,i.lineBreak!=o.lineBreak||i.done!=o.done||i.value!=o.value)return!1;if(l+=i.value.length,i.done||l>=r)return!0}}iter(e=1){return new fa(this,e)}iterRange(e,n=this.length){return new _w(this,e,n)}iterLines(e,n){let r;if(e==null)r=this.iter();else{n==null&&(n=this.lines+1);let i=this.line(e).from;r=this.iterRange(i,Math.max(i,n==this.lines+1?this.length:n<=1?0:this.line(n-1).to))}return new Bw(r)}toString(){return this.sliceString(0)}toJSON(){let e=[];return this.flatten(e),e}static of(e){if(e.length==0)throw new RangeError("A document must have at least one line");return e.length==1&&!e[0]?Ie.empty:e.length<=32?new Ye(e):Jn.from(Ye.split(e,[]))}}class Ye extends Ie{constructor(e,n=b3(e)){super();this.text=e,this.length=n}get lines(){return this.text.length}get children(){return null}lineInner(e,n,r,i){for(let o=0;;o++){let s=this.text[o],l=i+s.length;if((n?r:l)>=e)return new $3(i,l,r,s);i=l+1,r++}}decompose(e,n,r,i){let o=e<=0&&n>=this.length?this:new Ye(Nw(this.text,e,n),Math.min(n,this.length)-Math.max(0,e));if(i&1){let s=r.pop(),l=Fu(o.text,s.text.slice(),0,o.length);if(l.length<=32)r.push(new Ye(l,s.length+o.length));else{let a=l.length>>1;r.push(new Ye(l.slice(0,a)),new Ye(l.slice(a)))}}else r.push(o)}replace(e,n,r){if(!(r instanceof Ye))return super.replace(e,n,r);let i=Fu(this.text,Fu(r.text,Nw(this.text,0,e)),n),o=this.length+r.length-(n-e);return i.length<=32?new Ye(i,o):Jn.from(Ye.split(i,[]),o)}sliceString(e,n=this.length,r=`
`){let i="";for(let o=0,s=0;o<=n&&s<this.text.length;s++){let l=this.text[s],a=o+l.length;o>e&&s&&(i+=r),e<a&&n>o&&(i+=l.slice(Math.max(0,e-o),n-o)),o=a+1}return i}flatten(e){for(let n of this.text)e.push(n)}scanIdentical(){return 0}static split(e,n){let r=[],i=-1;for(let o of e)r.push(o),i+=o.length+1,r.length==32&&(n.push(new Ye(r,i)),r=[],i=-1);return i>-1&&n.push(new Ye(r,i)),n}}class Jn extends Ie{constructor(e,n){super();this.children=e,this.length=n,this.lines=0;for(let r of e)this.lines+=r.lines}lineInner(e,n,r,i){for(let o=0;;o++){let s=this.children[o],l=i+s.length,a=r+s.lines-1;if((n?a:l)>=e)return s.lineInner(e,n,r,i);i=l+1,r=a+1}}decompose(e,n,r,i){for(let o=0,s=0;s<=n&&o<this.children.length;o++){let l=this.children[o],a=s+l.length;if(e<=a&&n>=s){let c=i&((s<=e?1:0)|(a>=n?2:0));s>=e&&a<=n&&!c?r.push(l):l.decompose(e-s,n-s,r,c)}s=a+1}}replace(e,n,r){if(r.lines<this.lines)for(let i=0,o=0;i<this.children.length;i++){let s=this.children[i],l=o+s.length;if(e>=o&&n<=l){let a=s.replace(e-o,n-o,r),c=this.lines-s.lines+a.lines;if(a.lines<c>>5-1&&a.lines>c>>5+1){let u=this.children.slice();return u[i]=a,new Jn(u,this.length-(n-e)+r.length)}return super.replace(o,l,a)}o=l+1}return super.replace(e,n,r)}sliceString(e,n=this.length,r=`
`){let i="";for(let o=0,s=0;o<this.children.length&&s<=n;o++){let l=this.children[o],a=s+l.length;s>e&&o&&(i+=r),e<a&&n>s&&(i+=l.sliceString(e-s,n-s,r)),s=a+1}return i}flatten(e){for(let n of this.children)n.flatten(e)}scanIdentical(e,n){if(!(e instanceof Jn))return 0;let r=0,[i,o,s,l]=n>0?[0,0,this.children.length,e.children.length]:[this.children.length-1,e.children.length-1,-1,-1];for(;;i+=n,o+=n){if(i==s||o==l)return r;let a=this.children[i],c=e.children[o];if(a!=c)return r+a.scanIdentical(c,n);r+=a.length+1}}static from(e,n=e.reduce((r,i)=>r+i.length+1,-1)){let r=0;for(let d of e)r+=d.lines;if(r<32){let d=[];for(let p of e)p.flatten(d);return new Ye(d,n)}let i=Math.max(32,r>>5),o=i<<1,s=i>>1,l=[],a=0,c=-1,u=[];function f(d){let p;if(d.lines>o&&d instanceof Jn)for(let m of d.children)f(m);else d.lines>s&&(a>s||!a)?(h(),l.push(d)):d instanceof Ye&&a&&(p=u[u.length-1])instanceof Ye&&d.lines+p.lines<=32?(a+=d.lines,c+=d.length+1,u[u.length-1]=new Ye(p.text.concat(d.text),p.length+1+d.length)):(a+d.lines>i&&h(),a+=d.lines,c+=d.length+1,u.push(d))}function h(){a!=0&&(l.push(u.length==1?u[0]:Jn.from(u,c)),c=-1,a=u.length=0)}for(let d of e)f(d);return h(),l.length==1?l[0]:new Jn(l,n)}}Ie.empty=new Ye([""],0);function b3(t){let e=-1;for(let n of t)e+=n.length+1;return e}function Fu(t,e,n=0,r=1e9){for(let i=0,o=0,s=!0;o<t.length&&i<=r;o++){let l=t[o],a=i+l.length;a>=n&&(a>r&&(l=l.slice(0,r-i)),i<n&&(l=l.slice(n-i)),s?(e[e.length-1]+=l,s=!1):e.push(l)),i=a+1}return e}function Nw(t,e,n){return Fu(t,[""],e,n)}class fa{constructor(e,n=1){this.dir=n,this.done=!1,this.lineBreak=!1,this.value="",this.nodes=[e],this.offsets=[n>0?1:(e instanceof Ye?e.text.length:e.children.length)<<1]}nextInner(e,n){for(this.done=this.lineBreak=!1;;){let r=this.nodes.length-1,i=this.nodes[r],o=this.offsets[r],s=o>>1,l=i instanceof Ye?i.text.length:i.children.length;if(s==(n>0?l:0)){if(r==0)return this.done=!0,this.value="",this;n>0&&this.offsets[r-1]++,this.nodes.pop(),this.offsets.pop()}else if((o&1)==(n>0?0:1)){if(this.offsets[r]+=n,e==0)return this.lineBreak=!0,this.value=`
`,this;e--}else if(i instanceof Ye){let a=i.text[s+(n<0?-1:0)];if(this.offsets[r]+=n,a.length>Math.max(0,e))return this.value=e==0?a:n>0?a.slice(e):a.slice(0,a.length-e),this;e-=a.length}else{let a=i.children[s+(n<0?-1:0)];e>a.length?(e-=a.length,this.offsets[r]+=n):(n<0&&this.offsets[r]--,this.nodes.push(a),this.offsets.push(n>0?1:(a instanceof Ye?a.text.length:a.children.length)<<1))}}}next(e=0){return e<0&&(this.nextInner(-e,-this.dir),e=this.value.length),this.nextInner(e,this.dir)}}class _w{constructor(e,n,r){this.value="",this.done=!1,this.cursor=new fa(e,n>r?-1:1),this.pos=n>r?e.length:0,this.from=Math.min(n,r),this.to=Math.max(n,r)}nextInner(e,n){if(n<0?this.pos<=this.from:this.pos>=this.to)return this.value="",this.done=!0,this;e+=Math.max(0,n<0?this.pos-this.to:this.from-this.pos);let r=n<0?this.pos-this.from:this.to-this.pos;e>r&&(e=r),r-=e;let{value:i}=this.cursor.next(e);return this.pos+=(i.length+e)*n,this.value=i.length<=r?i:n<0?i.slice(i.length-r):i.slice(0,r),this.done=!this.value,this}next(e=0){return e<0?e=Math.max(e,this.from-this.pos):e>0&&(e=Math.min(e,this.to-this.pos)),this.nextInner(e,this.cursor.dir)}get lineBreak(){return this.cursor.lineBreak&&this.value!=""}}class Bw{constructor(e){this.inner=e,this.afterBreak=!0,this.value="",this.done=!1}next(e=0){let{done:n,lineBreak:r,value:i}=this.inner.next(e);return n?(this.done=!0,this.value=""):r?this.afterBreak?this.value="":(this.afterBreak=!0,this.next()):(this.value=i,this.afterBreak=!1),this}get lineBreak(){return!1}}typeof Symbol!="undefined"&&(Ie.prototype[Symbol.iterator]=function(){return this.iter()},fa.prototype[Symbol.iterator]=_w.prototype[Symbol.iterator]=Bw.prototype[Symbol.iterator]=function(){return this});class $3{constructor(e,n,r,i){this.from=e,this.to=n,this.number=r,this.text=i}get length(){return this.to-this.from}}const $m=/\r\n?|\n/;var Dt=function(t){return t[t.Simple=0]="Simple",t[t.TrackDel=1]="TrackDel",t[t.TrackBefore=2]="TrackBefore",t[t.TrackAfter=3]="TrackAfter",t}(Dt||(Dt={}));class kr{constructor(e){this.sections=e}get length(){let e=0;for(let n=0;n<this.sections.length;n+=2)e+=this.sections[n];return e}get newLength(){let e=0;for(let n=0;n<this.sections.length;n+=2){let r=this.sections[n+1];e+=r<0?this.sections[n]:r}return e}get empty(){return this.sections.length==0||this.sections.length==2&&this.sections[1]<0}iterGaps(e){for(let n=0,r=0,i=0;n<this.sections.length;){let o=this.sections[n++],s=this.sections[n++];s<0?(e(r,i,o),i+=o):i+=s,r+=o}}iterChangedRanges(e,n=!1){Sm(this,e,n)}get invertedDesc(){let e=[];for(let n=0;n<this.sections.length;){let r=this.sections[n++],i=this.sections[n++];i<0?e.push(r,i):e.push(i,r)}return new kr(e)}composeDesc(e){return this.empty?e:e.empty?this:qw(this,e)}mapDesc(e,n=!1){return e.empty?this:km(this,e,n)}mapPos(e,n=-1,r=Dt.Simple){let i=0,o=0;for(let s=0;s<this.sections.length;){let l=this.sections[s++],a=this.sections[s++],c=i+l;if(a<0){if(c>e)return o+(e-i);o+=l}else{if(r!=Dt.Simple&&c>=e&&(r==Dt.TrackDel&&i<e&&c>e||r==Dt.TrackBefore&&i<e||r==Dt.TrackAfter&&c>e))return null;if(c>e||c==e&&n<0&&!l)return e==i||n<0?o:o+a;o+=a}i=c}if(e>i)throw new RangeError(`Position ${e} is out of range for changeset of length ${i}`);return o}touchesRange(e,n=e){for(let r=0,i=0;r<this.sections.length&&i<=n;){let o=this.sections[r++],s=this.sections[r++],l=i+o;if(s>=0&&i<=n&&l>=e)return i<e&&l>n?"cover":!0;i=l}return!1}toString(){let e="";for(let n=0;n<this.sections.length;){let r=this.sections[n++],i=this.sections[n++];e+=(e?" ":"")+r+(i>=0?":"+i:"")}return e}toJSON(){return this.sections}static fromJSON(e){if(!Array.isArray(e)||e.length%2||e.some(n=>typeof n!="number"))throw new RangeError("Invalid JSON representation of ChangeDesc");return new kr(e)}}class ct extends kr{constructor(e,n){super(e);this.inserted=n}apply(e){if(this.length!=e.length)throw new RangeError("Applying change set to a document with the wrong length");return Sm(this,(n,r,i,o,s)=>e=e.replace(i,i+(r-n),s),!1),e}mapDesc(e,n=!1){return km(this,e,n,!0)}invert(e){let n=this.sections.slice(),r=[];for(let i=0,o=0;i<n.length;i+=2){let s=n[i],l=n[i+1];if(l>=0){n[i]=l,n[i+1]=s;let a=i>>1;for(;r.length<a;)r.push(Ie.empty);r.push(s?e.slice(o,o+s):Ie.empty)}o+=s}return new ct(n,r)}compose(e){return this.empty?e:e.empty?this:qw(this,e,!0)}map(e,n=!1){return e.empty?this:km(this,e,n,!0)}iterChanges(e,n=!1){Sm(this,e,n)}get desc(){return new kr(this.sections)}filter(e){let n=[],r=[],i=[],o=new ha(this);e:for(let s=0,l=0;;){let a=s==e.length?1e9:e[s++];for(;l<a||l==a&&o.len==0;){if(o.done)break e;let u=Math.min(o.len,a-l);Lt(i,u,-1);let f=o.ins==-1?-1:o.off==0?o.ins:0;Lt(n,u,f),f>0&&Hi(r,n,o.text),o.forward(u),l+=u}let c=e[s++];for(;l<c;){if(o.done)break e;let u=Math.min(o.len,c-l);Lt(n,u,-1),Lt(i,u,o.ins==-1?-1:o.off==0?o.ins:0),o.forward(u),l+=u}}return{changes:new ct(n,r),filtered:new kr(i)}}toJSON(){let e=[];for(let n=0;n<this.sections.length;n+=2){let r=this.sections[n],i=this.sections[n+1];i<0?e.push(r):i==0?e.push([r]):e.push([r].concat(this.inserted[n>>1].toJSON()))}return e}static of(e,n,r){let i=[],o=[],s=0,l=null;function a(u=!1){if(!u&&!i.length)return;s<n&&Lt(i,n-s,-1);let f=new ct(i,o);l=l?l.compose(f.map(l)):f,i=[],o=[],s=0}function c(u){if(Array.isArray(u))for(let f of u)c(f);else if(u instanceof ct){if(u.length!=n)throw new RangeError(`Mismatched change set length (got ${u.length}, expected ${n})`);a(),l=l?l.compose(u.map(l)):u}else{let{from:f,to:h=f,insert:d}=u;if(f>h||f<0||h>n)throw new RangeError(`Invalid change range ${f} to ${h} (in doc of length ${n})`);let p=d?typeof d=="string"?Ie.of(d.split(r||$m)):d:Ie.empty,m=p.length;if(f==h&&m==0)return;f<s&&a(),f>s&&Lt(i,f-s,-1),Lt(i,h-f,m),Hi(o,i,p),s=h}}return c(e),a(!l),l}static empty(e){return new ct(e?[e,-1]:[],[])}static fromJSON(e){if(!Array.isArray(e))throw new RangeError("Invalid JSON representation of ChangeSet");let n=[],r=[];for(let i=0;i<e.length;i++){let o=e[i];if(typeof o=="number")n.push(o,-1);else{if(!Array.isArray(o)||typeof o[0]!="number"||o.some((s,l)=>l&&typeof s!="string"))throw new RangeError("Invalid JSON representation of ChangeSet");if(o.length==1)n.push(o[0],0);else{for(;r.length<i;)r.push(Ie.empty);r[i]=Ie.of(o.slice(1)),n.push(o[0],r[i].length)}}}return new ct(n,r)}}function Lt(t,e,n,r=!1){if(e==0&&n<=0)return;let i=t.length-2;i>=0&&n<=0&&n==t[i+1]?t[i]+=e:e==0&&t[i]==0?t[i+1]+=n:r?(t[i]+=e,t[i+1]+=n):t.push(e,n)}function Hi(t,e,n){if(n.length==0)return;let r=e.length-2>>1;if(r<t.length)t[t.length-1]=t[t.length-1].append(n);else{for(;t.length<r;)t.push(Ie.empty);t.push(n)}}function Sm(t,e,n){let r=t.inserted;for(let i=0,o=0,s=0;s<t.sections.length;){let l=t.sections[s++],a=t.sections[s++];if(a<0)i+=l,o+=l;else{let c=i,u=o,f=Ie.empty;for(;c+=l,u+=a,a&&r&&(f=f.append(r[s-2>>1])),!(n||s==t.sections.length||t.sections[s+1]<0);)l=t.sections[s++],a=t.sections[s++];e(i,c,o,u,f),i=c,o=u}}}function km(t,e,n,r=!1){let i=[],o=r?[]:null,s=new ha(t),l=new ha(e);for(let a=0,c=0;;)if(s.ins==-1)a+=s.len,s.next();else if(l.ins==-1&&c<a){let u=Math.min(l.len,a-c);l.forward(u),Lt(i,u,-1),c+=u}else if(l.ins>=0&&(s.done||c<a||c==a&&(l.len<s.len||l.len==s.len&&!n))){for(Lt(i,l.ins,-1);a>c&&!s.done&&a+s.len<c+l.len;)a+=s.len,s.next();c+=l.len,l.next()}else if(s.ins>=0){let u=0,f=a+s.len;for(;;)if(l.ins>=0&&c>a&&c+l.len<f)u+=l.ins,c+=l.len,l.next();else if(l.ins==-1&&c<f){let h=Math.min(l.len,f-c);u+=h,l.forward(h),c+=h}else break;Lt(i,u,s.ins),o&&Hi(o,i,s.text),a=f,s.next()}else{if(s.done&&l.done)return o?new ct(i,o):new kr(i);throw new Error("Mismatched change set lengths")}}function qw(t,e,n=!1){let r=[],i=n?[]:null,o=new ha(t),s=new ha(e);for(let l=!1;;){if(o.done&&s.done)return i?new ct(r,i):new kr(r);if(o.ins==0)Lt(r,o.len,0,l),o.next();else if(s.len==0&&!s.done)Lt(r,0,s.ins,l),i&&Hi(i,r,s.text),s.next();else{if(o.done||s.done)throw new Error("Mismatched change set lengths");{let a=Math.min(o.len2,s.len),c=r.length;if(o.ins==-1){let u=s.ins==-1?-1:s.off?0:s.ins;Lt(r,a,u,l),i&&u&&Hi(i,r,s.text)}else s.ins==-1?(Lt(r,o.off?0:o.len,a,l),i&&Hi(i,r,o.textBit(a))):(Lt(r,o.off?0:o.len,s.off?0:s.ins,l),i&&!s.off&&Hi(i,r,s.text));l=(o.ins>a||s.ins>=0&&s.len>a)&&(l||r.length>c),o.forward2(a),s.forward(a)}}}}class ha{constructor(e){this.set=e,this.i=0,this.next()}next(){let{sections:e}=this.set;this.i<e.length?(this.len=e[this.i++],this.ins=e[this.i++]):(this.len=0,this.ins=-2),this.off=0}get done(){return this.ins==-2}get len2(){return this.ins<0?this.len:this.ins}get text(){let{inserted:e}=this.set,n=this.i-2>>1;return n>=e.length?Ie.empty:e[n]}textBit(e){let{inserted:n}=this.set,r=this.i-2>>1;return r>=n.length&&!e?Ie.empty:n[r].slice(this.off,e==null?void 0:this.off+e)}forward(e){e==this.len?this.next():(this.len-=e,this.off+=e)}forward2(e){this.ins==-1?this.forward(e):e==this.ins?this.next():(this.ins-=e,this.off+=e)}}class us{constructor(e,n,r){this.from=e,this.to=n,this.flags=r}get anchor(){return this.flags&16?this.to:this.from}get head(){return this.flags&16?this.from:this.to}get empty(){return this.from==this.to}get assoc(){return this.flags&4?-1:this.flags&8?1:0}get bidiLevel(){let e=this.flags&3;return e==3?null:e}get goalColumn(){let e=this.flags>>5;return e==33554431?void 0:e}map(e,n=-1){let r=e.mapPos(this.from,n),i=e.mapPos(this.to,n);return r==this.from&&i==this.to?this:new us(r,i,this.flags)}extend(e,n=e){if(e<=this.anchor&&n>=this.anchor)return _.range(e,n);let r=Math.abs(e-this.anchor)>Math.abs(n-this.anchor)?e:n;return _.range(this.anchor,r)}eq(e){return this.anchor==e.anchor&&this.head==e.head}toJSON(){return{anchor:this.anchor,head:this.head}}static fromJSON(e){if(!e||typeof e.anchor!="number"||typeof e.head!="number")throw new RangeError("Invalid JSON representation for SelectionRange");return _.range(e.anchor,e.head)}}class _{constructor(e,n=0){this.ranges=e,this.mainIndex=n}map(e,n=-1){return e.empty?this:_.create(this.ranges.map(r=>r.map(e,n)),this.mainIndex)}eq(e){if(this.ranges.length!=e.ranges.length||this.mainIndex!=e.mainIndex)return!1;for(let n=0;n<this.ranges.length;n++)if(!this.ranges[n].eq(e.ranges[n]))return!1;return!0}get main(){return this.ranges[this.mainIndex]}asSingle(){return this.ranges.length==1?this:new _([this.main])}addRange(e,n=!0){return _.create([e].concat(this.ranges),n?0:this.mainIndex+1)}replaceRange(e,n=this.mainIndex){let r=this.ranges.slice();return r[n]=e,_.create(r,this.mainIndex)}toJSON(){return{ranges:this.ranges.map(e=>e.toJSON()),main:this.mainIndex}}static fromJSON(e){if(!e||!Array.isArray(e.ranges)||typeof e.main!="number"||e.main>=e.ranges.length)throw new RangeError("Invalid JSON representation for EditorSelection");return new _(e.ranges.map(n=>us.fromJSON(n)),e.main)}static single(e,n=e){return new _([_.range(e,n)],0)}static create(e,n=0){if(e.length==0)throw new RangeError("A selection needs at least one range");for(let r=0,i=0;i<e.length;i++){let o=e[i];if(o.empty?o.from<=r:o.from<r)return S3(e.slice(),n);r=o.to}return new _(e,n)}static cursor(e,n=0,r,i){return new us(e,e,(n==0?0:n<0?4:8)|(r==null?3:Math.min(2,r))|(i!=null?i:33554431)<<5)}static range(e,n,r){let i=(r!=null?r:33554431)<<5;return n<e?new us(n,e,16|i):new us(e,n,i)}}function S3(t,e=0){let n=t[e];t.sort((r,i)=>r.from-i.from),e=t.indexOf(n);for(let r=1;r<t.length;r++){let i=t[r],o=t[r-1];if(i.empty?i.from<=o.to:i.from<o.to){let s=o.from,l=Math.max(i.to,o.to);r<=e&&e--,t.splice(--r,2,i.anchor>i.head?_.range(l,s):_.range(s,l))}}return new _(t,e)}function zw(t,e){for(let n of t.ranges)if(n.to>e)throw new RangeError("Selection points outside of document")}let Pm=0;class ne{constructor(e,n,r,i,o){this.combine=e,this.compareInput=n,this.compare=r,this.isStatic=i,this.extensions=o,this.id=Pm++,this.default=e([])}static define(e={}){return new ne(e.combine||(n=>n),e.compareInput||((n,r)=>n===r),e.compare||(e.combine?(n,r)=>n===r:k3),!!e.static,e.enables)}of(e){return new Vu([],this,0,e)}compute(e,n){if(this.isStatic)throw new Error("Can't compute a static facet");return new Vu(e,this,1,n)}computeN(e,n){if(this.isStatic)throw new Error("Can't compute a static facet");return new Vu(e,this,2,n)}from(e,n){return n||(n=r=>r),this.compute([e],r=>n(r.field(e)))}}function k3(t,e){return t==e||t.length==e.length&&t.every((n,r)=>n===e[r])}class Vu{constructor(e,n,r,i){this.dependencies=e,this.facet=n,this.type=r,this.value=i,this.id=Pm++}dynamicSlot(e){var n;let r=this.value,i=this.facet.compareInput,o=e[this.id]>>1,s=this.type==2,l=!1,a=!1,c=[];for(let u of this.dependencies)u=="doc"?l=!0:u=="selection"?a=!0:(((n=e[u.id])!==null&&n!==void 0?n:1)&1)==0&&c.push(e[u.id]);return(u,f)=>{let h=u.values[o];if(h===pa)return u.values[o]=r(u),1;if(f&&(l&&f.docChanged||a&&(f.docChanged||f.selection)||c.some(p=>(ma(u,p)&1)>0))){let p=r(u);if(s?!P3(p,h,i):!i(p,h))return u.values[o]=p,1}return 0}}}function P3(t,e,n){if(t.length!=e.length)return!1;for(let r=0;r<t.length;r++)if(!n(t[r],e[r]))return!1;return!0}function C3(t,e,n){let r=n.map(l=>t[l.id]),i=n.map(l=>l.type),o=r.filter(l=>!(l&1)),s=t[e.id]>>1;return(l,a)=>{let c=l.values[s],u=c===pa||!a;for(let d of o)ma(l,d)&1&&(u=!0);if(!u)return 0;let f=[];for(let d=0;d<r.length;d++){let p=fs(l,r[d]);if(i[d]==2)for(let m of p)f.push(m);else f.push(p)}let h=e.combine(f);return c!==pa&&e.compare(h,c)?0:(l.values[s]=h,1)}}const Fw=ne.define({static:!0});class Rt{constructor(e,n,r,i,o){this.id=e,this.createF=n,this.updateF=r,this.compareF=i,this.spec=o,this.provides=void 0}static define(e){let n=new Rt(Pm++,e.create,e.update,e.compare||((r,i)=>r===i),e);return e.provide&&(n.provides=e.provide(n)),n}create(e){let n=e.facet(Fw).find(r=>r.field==this);return((n==null?void 0:n.create)||this.createF)(e)}slot(e){let n=e[this.id]>>1;return(r,i)=>{let o=r.values[n];if(o===pa)return r.values[n]=this.create(r),1;if(i){let s=this.updateF(o,i);if(!this.compareF(o,s))return r.values[n]=s,1}return 0}}init(e){return[this,Fw.of({field:this,create:e})]}get extension(){return this}}const Pr={lowest:4,low:3,default:2,high:1,highest:0};function di(t){return e=>new Vw(e,t)}const da={lowest:di(Pr.lowest),low:di(Pr.low),default:di(Pr.default),high:di(Pr.high),highest:di(Pr.highest),fallback:di(Pr.lowest),extend:di(Pr.high),override:di(Pr.highest)};class Vw{constructor(e,n){this.inner=e,this.prec=n}}class Uu{of(e){return new Cm(this,e)}reconfigure(e){return Uu.reconfigure.of({compartment:this,extension:e})}get(e){return e.config.compartments.get(this)}}class Cm{constructor(e,n){this.compartment=e,this.inner=n}}class ju{constructor(e,n,r,i,o){for(this.base=e,this.compartments=n,this.dynamicSlots=r,this.address=i,this.staticValues=o,this.statusTemplate=[];this.statusTemplate.length<r.length;)this.statusTemplate.push(0)}staticFacet(e){let n=this.address[e.id];return n==null?e.default:this.staticValues[n>>1]}static resolve(e,n,r){let i=[],o=Object.create(null),s=new Map;for(let h of R3(e,n,s))h instanceof Rt?i.push(h):(o[h.facet.id]||(o[h.facet.id]=[])).push(h);let l=Object.create(null),a=[],c=[],u=[];for(let h of i)l[h.id]=c.length<<1,c.push(d=>h.slot(d)),u.push([]);for(let h in o){let d=o[h],p=d[0].facet;if(d.every(m=>m.type==0)){l[p.id]=a.length<<1|1;let m=p.combine(d.map(y=>y.value)),g=r?r.config.address[p.id]:null;if(g!=null){let y=fs(r,g);p.compare(m,y)&&(m=y)}a.push(m)}else{for(let m of d)m.type==0?(l[m.id]=a.length<<1|1,a.push(m.value)):(l[m.id]=c.length<<1,c.push(g=>m.dynamicSlot(g)),u.push(m.dependencies.filter(g=>typeof g!="string").map(g=>g.id)));l[p.id]=c.length<<1,c.push(m=>C3(m,p,d)),u.push(d.filter(m=>m.type!=0).map(m=>m.id))}}let f=c.map(h=>pa);if(r){let h=(d,p)=>{if(p>7)return!1;let m=l[d];if(!(m&1))return u[m>>1].every(y=>h(y,p+1));let g=r.config.address[d];return g!=null&&fs(r,g)==a[m>>1]};for(let d in l){let p=l[d],m=r.config.address[d];m!=null&&(p&1)==0&&h(+d,0)&&(f[p>>1]=fs(r,m))}}return{configuration:new ju(e,s,c.map(h=>h(l)),l,a),values:f}}}function R3(t,e,n){let r=[[],[],[],[],[]],i=new Map;function o(s,l){let a=i.get(s);if(a!=null){if(a>=l)return;let c=r[a].indexOf(s);c>-1&&r[a].splice(c,1),s instanceof Cm&&n.delete(s.compartment)}if(i.set(s,l),Array.isArray(s))for(let c of s)o(c,l);else if(s instanceof Cm){if(n.has(s.compartment))throw new RangeError("Duplicate use of compartment in extensions");let c=e.get(s.compartment)||s.inner;n.set(s.compartment,c),o(c,l)}else if(s instanceof Vw)o(s.inner,s.prec);else if(s instanceof Rt)r[l].push(s),s.provides&&o(s.provides,l);else if(s instanceof Vu)r[l].push(s),s.facet.extensions&&o(s.facet.extensions,l);else{let c=s.extension;if(!c)throw new Error(`Unrecognized extension value in extension set (${s}). This sometimes happens because multiple instances of @codemirror/state are loaded, breaking instanceof checks.`);o(c,l)}}return o(t,Pr.default),r.reduce((s,l)=>s.concat(l))}const pa={};function ma(t,e){if(e&1)return 2;let n=e>>1,r=t.status[n];if(r==4)throw new Error("Cyclic dependency between fields and/or facets");if(r&2)return r;t.status[n]=4;let i=t.config.dynamicSlots[n](t,t.applying);return t.status[n]=2|i}function fs(t,e){return e&1?t.config.staticValues[e>>1]:t.values[e>>1]}const Uw=ne.define(),jw=ne.define({combine:t=>t.some(e=>e),static:!0}),Xw=ne.define({combine:t=>t.length?t[0]:void 0,static:!0}),Hw=ne.define(),Gw=ne.define(),Zw=ne.define(),Yw=ne.define({combine:t=>t.length?t[0]:!1});class pi{constructor(e,n){this.type=e,this.value=n}static define(){return new T3}}class T3{of(e){return new pi(this,e)}}class I3{constructor(e){this.map=e}of(e){return new ce(this,e)}}class ce{constructor(e,n){this.type=e,this.value=n}map(e){let n=this.type.map(this.value,e);return n===void 0?void 0:n==this.value?this:new ce(this.type,n)}is(e){return this.type==e}static define(e={}){return new I3(e.map||(n=>n))}static mapEffects(e,n){if(!e.length)return e;let r=[];for(let i of e){let o=i.map(n);o&&r.push(o)}return r}}ce.reconfigure=ce.define();ce.appendConfig=ce.define();class ut{constructor(e,n,r,i,o,s){this.startState=e,this.changes=n,this.selection=r,this.effects=i,this.annotations=o,this.scrollIntoView=s,this._doc=null,this._state=null,r&&zw(r,n.newLength),o.some(l=>l.type==ut.time)||(this.annotations=o.concat(ut.time.of(Date.now())))}get newDoc(){return this._doc||(this._doc=this.changes.apply(this.startState.doc))}get newSelection(){return this.selection||this.startState.selection.map(this.changes)}get state(){return this._state||this.startState.applyTransaction(this),this._state}annotation(e){for(let n of this.annotations)if(n.type==e)return n.value}get docChanged(){return!this.changes.empty}get reconfigured(){return this.startState.config!=this.state.config}isUserEvent(e){let n=this.annotation(ut.userEvent);return!!(n&&(n==e||n.length>e.length&&n.slice(0,e.length)==e&&n[e.length]=="."))}}ut.time=pi.define();ut.userEvent=pi.define();ut.addToHistory=pi.define();ut.remote=pi.define();function E3(t,e){let n=[];for(let r=0,i=0;;){let o,s;if(r<t.length&&(i==e.length||e[i]>=t[r]))o=t[r++],s=t[r++];else if(i<e.length)o=e[i++],s=e[i++];else return n;!n.length||n[n.length-1]<o?n.push(o,s):n[n.length-1]<s&&(n[n.length-1]=s)}}function Kw(t,e,n){var r;let i,o,s;return n?(i=e.changes,o=ct.empty(e.changes.length),s=t.changes.compose(e.changes)):(i=e.changes.map(t.changes),o=t.changes.mapDesc(e.changes,!0),s=t.changes.compose(i)),{changes:s,selection:e.selection?e.selection.map(o):(r=t.selection)===null||r===void 0?void 0:r.map(i),effects:ce.mapEffects(t.effects,i).concat(ce.mapEffects(e.effects,o)),annotations:t.annotations.length?t.annotations.concat(e.annotations):e.annotations,scrollIntoView:t.scrollIntoView||e.scrollIntoView}}function Rm(t,e,n){let r=e.selection,i=hs(e.annotations);return e.userEvent&&(i=i.concat(ut.userEvent.of(e.userEvent))),{changes:e.changes instanceof ct?e.changes:ct.of(e.changes||[],n,t.facet(Xw)),selection:r&&(r instanceof _?r:_.single(r.anchor,r.head)),effects:hs(e.effects),annotations:i,scrollIntoView:!!e.scrollIntoView}}function Jw(t,e,n){let r=Rm(t,e.length?e[0]:{},t.doc.length);e.length&&e[0].filter===!1&&(n=!1);for(let o=1;o<e.length;o++){e[o].filter===!1&&(n=!1);let s=!!e[o].sequential;r=Kw(r,Rm(t,e[o],s?r.changes.newLength:t.doc.length),s)}let i=new ut(t,r.changes,r.selection,r.effects,r.annotations,r.scrollIntoView);return A3(n?M3(i):i)}function M3(t){let e=t.startState,n=!0;for(let i of e.facet(Hw)){let o=i(t);if(o===!1){n=!1;break}Array.isArray(o)&&(n=n===!0?o:E3(n,o))}if(n!==!0){let i,o;if(n===!1)o=t.changes.invertedDesc,i=ct.empty(e.doc.length);else{let s=t.changes.filter(n);i=s.changes,o=s.filtered.invertedDesc}t=new ut(e,i,t.selection&&t.selection.map(o),ce.mapEffects(t.effects,o),t.annotations,t.scrollIntoView)}let r=e.facet(Gw);for(let i=r.length-1;i>=0;i--){let o=r[i](t);o instanceof ut?t=o:Array.isArray(o)&&o.length==1&&o[0]instanceof ut?t=o[0]:t=Jw(e,hs(o),!1)}return t}function A3(t){let e=t.startState,n=e.facet(Zw),r=t;for(let i=n.length-1;i>=0;i--){let o=n[i](t);o&&Object.keys(o).length&&(r=Kw(t,Rm(e,o,t.changes.newLength),!0))}return r==t?t:new ut(e,t.changes,t.selection,r.effects,r.annotations,r.scrollIntoView)}const D3=[];function hs(t){return t==null?D3:Array.isArray(t)?t:[t]}var rn=function(t){return t[t.Word=0]="Word",t[t.Space=1]="Space",t[t.Other=2]="Other",t}(rn||(rn={}));const L3=/[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;let Tm;try{Tm=new RegExp("[\\p{Alphabetic}\\p{Number}_]","u")}catch{}function W3(t){if(Tm)return Tm.test(t);for(let e=0;e<t.length;e++){let n=t[e];if(/\w/.test(n)||n>"\x80"&&(n.toUpperCase()!=n.toLowerCase()||L3.test(n)))return!0}return!1}function Q3(t){return e=>{if(!/\S/.test(e))return rn.Space;if(W3(e))return rn.Word;for(let n=0;n<t.length;n++)if(e.indexOf(t[n])>-1)return rn.Word;return rn.Other}}class Pe{constructor(e,n,r,i,o=null){this.config=e,this.doc=n,this.selection=r,this.values=i,this.applying=null,this.status=e.statusTemplate.slice(),this.applying=o,o&&(o._state=this);for(let s=0;s<this.config.dynamicSlots.length;s++)ma(this,s<<1);this.applying=null}field(e,n=!0){let r=this.config.address[e.id];if(r==null){if(n)throw new RangeError("Field is not present in this state");return}return ma(this,r),fs(this,r)}update(...e){return Jw(this,e,!0)}applyTransaction(e){let n=this.config,{base:r,compartments:i}=n;for(let s of e.effects)s.is(Uu.reconfigure)?(n&&(i=new Map,n.compartments.forEach((l,a)=>i.set(a,l)),n=null),i.set(s.value.compartment,s.value.extension)):s.is(ce.reconfigure)?(n=null,r=s.value):s.is(ce.appendConfig)&&(n=null,r=hs(r).concat(s.value));let o;if(n)o=e.startState.values.slice();else{let s=ju.resolve(r,i,this);n=s.configuration,o=new Pe(n,this.doc,this.selection,s.values,null).values}new Pe(n,e.newDoc,e.newSelection,o,e)}replaceSelection(e){return typeof e=="string"&&(e=this.toText(e)),this.changeByRange(n=>({changes:{from:n.from,to:n.to,insert:e},range:_.cursor(n.from+e.length)}))}changeByRange(e){let n=this.selection,r=e(n.ranges[0]),i=this.changes(r.changes),o=[r.range],s=hs(r.effects);for(let l=1;l<n.ranges.length;l++){let a=e(n.ranges[l]),c=this.changes(a.changes),u=c.map(i);for(let h=0;h<l;h++)o[h]=o[h].map(u);let f=i.mapDesc(c,!0);o.push(a.range.map(f)),i=i.compose(u),s=ce.mapEffects(s,u).concat(ce.mapEffects(hs(a.effects),f))}return{changes:i,selection:_.create(o,n.mainIndex),effects:s}}changes(e=[]){return e instanceof ct?e:ct.of(e,this.doc.length,this.facet(Pe.lineSeparator))}toText(e){return Ie.of(e.split(this.facet(Pe.lineSeparator)||$m))}sliceDoc(e=0,n=this.doc.length){return this.doc.sliceString(e,n,this.lineBreak)}facet(e){let n=this.config.address[e.id];return n==null?e.default:(ma(this,n),fs(this,n))}toJSON(e){let n={doc:this.sliceDoc(),selection:this.selection.toJSON()};if(e)for(let r in e){let i=e[r];i instanceof Rt&&(n[r]=i.spec.toJSON(this.field(e[r]),this))}return n}static fromJSON(e,n={},r){if(!e||typeof e.doc!="string")throw new RangeError("Invalid JSON representation for EditorState");let i=[];if(r)for(let o in r){let s=r[o],l=e[o];i.push(s.init(a=>s.spec.fromJSON(l,a)))}return Pe.create({doc:e.doc,selection:_.fromJSON(e.selection),extensions:n.extensions?i.concat([n.extensions]):i})}static create(e={}){let{configuration:n,values:r}=ju.resolve(e.extensions||[],new Map),i=e.doc instanceof Ie?e.doc:Ie.of((e.doc||"").split(n.staticFacet(Pe.lineSeparator)||$m)),o=e.selection?e.selection instanceof _?e.selection:_.single(e.selection.anchor,e.selection.head):_.single(0);return zw(o,i.length),n.staticFacet(jw)||(o=o.asSingle()),new Pe(n,i,o,r)}get tabSize(){return this.facet(Pe.tabSize)}get lineBreak(){return this.facet(Pe.lineSeparator)||`
`}get readOnly(){return this.facet(Yw)}phrase(e){for(let n of this.facet(Pe.phrases))if(Object.prototype.hasOwnProperty.call(n,e))return n[e];return e}languageDataAt(e,n,r=-1){let i=[];for(let o of this.facet(Uw))for(let s of o(this,n,r))Object.prototype.hasOwnProperty.call(s,e)&&i.push(s[e]);return i}charCategorizer(e){return Q3(this.languageDataAt("wordChars",e).join(""))}wordAt(e){let{text:n,from:r,length:i}=this.doc.lineAt(e),o=this.charCategorizer(e),s=e-r,l=e-r;for(;s>0;){let a=nn(n,s,!1);if(o(n.slice(a,s))!=rn.Word)break;s=a}for(;l<i;){let a=nn(n,l);if(o(n.slice(l,a))!=rn.Word)break;l=a}return s==l?null:_.range(s+r,l+r)}}Pe.allowMultipleSelections=jw;Pe.tabSize=ne.define({combine:t=>t.length?t[0]:4});Pe.lineSeparator=Xw;Pe.readOnly=Yw;Pe.phrases=ne.define();Pe.languageData=Uw;Pe.changeFilter=Hw;Pe.transactionFilter=Gw;Pe.transactionExtender=Zw;Uu.reconfigure=ce.define();function mi(t,e,n={}){let r={};for(let i of t)for(let o of Object.keys(i)){let s=i[o],l=r[o];if(l===void 0)r[o]=s;else if(!(l===s||s===void 0))if(Object.hasOwnProperty.call(n,o))r[o]=n[o](l,s);else throw new Error("Config merge conflict for field "+o)}for(let i in e)r[i]===void 0&&(r[i]=e[i]);return r}const Im="\u037C",eb=typeof Symbol=="undefined"?"__"+Im:Symbol.for(Im),Em=typeof Symbol=="undefined"?"__styleSet"+Math.floor(Math.random()*1e8):Symbol("styleSet"),tb=typeof globalThis!="undefined"?globalThis:typeof window!="undefined"?window:{};class gi{constructor(e,n){this.rules=[];let{finish:r}=n||{};function i(s){return/^@/.test(s)?[s]:s.split(/,\s*/)}function o(s,l,a,c){let u=[],f=/^@(\w+)\b/.exec(s[0]),h=f&&f[1]=="keyframes";if(f&&l==null)return a.push(s[0]+";");for(let d in l){let p=l[d];if(/&/.test(d))o(d.split(/,\s*/).map(m=>s.map(g=>m.replace(/&/,g))).reduce((m,g)=>m.concat(g)),p,a);else if(p&&typeof p=="object"){if(!f)throw new RangeError("The value of a property ("+d+") should be a primitive value.");o(i(d),p,u,h)}else p!=null&&u.push(d.replace(/_.*/,"").replace(/[A-Z]/g,m=>"-"+m.toLowerCase())+": "+p+";")}(u.length||h)&&a.push((r&&!f&&!c?s.map(r):s).join(", ")+" {"+u.join(" ")+"}")}for(let s in e)o(i(s),e[s],this.rules)}getRules(){return this.rules.join(`
`)}static newName(){let e=tb[eb]||1;return tb[eb]=e+1,Im+e.toString(36)}static mount(e,n){(e[Em]||new N3(e)).mount(Array.isArray(n)?n:[n])}}let Xu=null;class N3{constructor(e){if(!e.head&&e.adoptedStyleSheets&&typeof CSSStyleSheet!="undefined"){if(Xu)return e.adoptedStyleSheets=[Xu.sheet].concat(e.adoptedStyleSheets),e[Em]=Xu;this.sheet=new CSSStyleSheet,e.adoptedStyleSheets=[this.sheet].concat(e.adoptedStyleSheets),Xu=this}else{this.styleTag=(e.ownerDocument||e).createElement("style");let n=e.head||e;n.insertBefore(this.styleTag,n.firstChild)}this.modules=[],e[Em]=this}mount(e){let n=this.sheet,r=0,i=0;for(let o=0;o<e.length;o++){let s=e[o],l=this.modules.indexOf(s);if(l<i&&l>-1&&(this.modules.splice(l,1),i--,l=-1),l==-1){if(this.modules.splice(i++,0,s),n)for(let a=0;a<s.rules.length;a++)n.insertRule(s.rules[a],r++)}else{for(;i<l;)r+=this.modules[i++].rules.length;r+=s.rules.length,i++}}if(!n){let o="";for(let s=0;s<this.modules.length;s++)o+=this.modules[s].getRules()+`
`;this.styleTag.textContent=o}}}class Gi{eq(e){return this==e}range(e,n=e){return new Mm(e,n,this)}}Gi.prototype.startSide=Gi.prototype.endSide=0;Gi.prototype.point=!1;Gi.prototype.mapMode=Dt.TrackDel;class Mm{constructor(e,n,r){this.from=e,this.to=n,this.value=r}}function Am(t,e){return t.from-e.from||t.value.startSide-e.value.startSide}class Dm{constructor(e,n,r,i){this.from=e,this.to=n,this.value=r,this.maxPoint=i}get length(){return this.to[this.to.length-1]}findIndex(e,n,r,i=0){let o=r?this.to:this.from;for(let s=i,l=o.length;;){if(s==l)return s;let a=s+l>>1,c=o[a]-e||(r?this.value[a].endSide:this.value[a].startSide)-n;if(a==s)return c>=0?s:l;c>=0?l=a:s=a+1}}between(e,n,r,i){for(let o=this.findIndex(n,-1e9,!0),s=this.findIndex(r,1e9,!1,o);o<s;o++)if(i(this.from[o]+e,this.to[o]+e,this.value[o])===!1)return!1}map(e,n){let r=[],i=[],o=[],s=-1,l=-1;for(let a=0;a<this.value.length;a++){let c=this.value[a],u=this.from[a]+e,f=this.to[a]+e,h,d;if(u==f){let p=n.mapPos(u,c.startSide,c.mapMode);if(p==null||(h=d=p,c.startSide!=c.endSide&&(d=n.mapPos(u,c.endSide),d<h)))continue}else if(h=n.mapPos(u,c.startSide),d=n.mapPos(f,c.endSide),h>d||h==d&&c.startSide>0&&c.endSide<=0)continue;(d-h||c.endSide-c.startSide)<0||(s<0&&(s=h),c.point&&(l=Math.max(l,d-h)),r.push(c),i.push(h-s),o.push(d-s))}return{mapped:r.length?new Dm(i,o,r,l):null,pos:s}}}class Te{constructor(e,n,r=Te.empty,i){this.chunkPos=e,this.chunk=n,this.nextLayer=r,this.maxPoint=i}get length(){let e=this.chunk.length-1;return e<0?0:Math.max(this.chunkEnd(e),this.nextLayer.length)}get size(){if(this.isEmpty)return 0;let e=this.nextLayer.size;for(let n of this.chunk)e+=n.value.length;return e}chunkEnd(e){return this.chunkPos[e]+this.chunk[e].length}update(e){let{add:n=[],sort:r=!1,filterFrom:i=0,filterTo:o=this.length}=e,s=e.filter;if(n.length==0&&!s)return this;if(r&&n.slice().sort(Am),this.isEmpty)return n.length?Te.of(n):this;let l=new rb(this,null,-1).goto(0),a=0,c=[],u=new yi;for(;l.value||a<n.length;)if(a<n.length&&(l.from-n[a].from||l.startSide-n[a].value.startSide)>=0){let f=n[a++];u.addInner(f.from,f.to,f.value)||c.push(f)}else l.rangeIndex==1&&l.chunkIndex<this.chunk.length&&(a==n.length||this.chunkEnd(l.chunkIndex)<n[a].from)&&(!s||i>this.chunkEnd(l.chunkIndex)||o<this.chunkPos[l.chunkIndex])&&u.addChunk(this.chunkPos[l.chunkIndex],this.chunk[l.chunkIndex])?l.nextChunk():((!s||i>l.to||o<l.from||s(l.from,l.to,l.value))&&(u.addInner(l.from,l.to,l.value)||c.push(new Mm(l.from,l.to,l.value))),l.next());return u.finishInner(this.nextLayer.isEmpty&&!c.length?Te.empty:this.nextLayer.update({add:c,filter:s,filterFrom:i,filterTo:o}))}map(e){if(e.length==0||this.isEmpty)return this;let n=[],r=[],i=-1;for(let s=0;s<this.chunk.length;s++){let l=this.chunkPos[s],a=this.chunk[s],c=e.touchesRange(l,l+a.length);if(c===!1)i=Math.max(i,a.maxPoint),n.push(a),r.push(e.mapPos(l));else if(c===!0){let{mapped:u,pos:f}=a.map(l,e);u&&(i=Math.max(i,u.maxPoint),n.push(u),r.push(f))}}let o=this.nextLayer.map(e);return n.length==0?o:new Te(r,n,o,i)}between(e,n,r){if(!this.isEmpty){for(let i=0;i<this.chunk.length;i++){let o=this.chunkPos[i],s=this.chunk[i];if(n>=o&&e<=o+s.length&&s.between(o,e-o,n-o,r)===!1)return}this.nextLayer.between(e,n,r)}}iter(e=0){return ga.from([this]).goto(e)}get isEmpty(){return this.nextLayer==this}static iter(e,n=0){return ga.from(e).goto(n)}static compare(e,n,r,i,o=-1){let s=e.filter(f=>f.maxPoint>0||!f.isEmpty&&f.maxPoint>=o),l=n.filter(f=>f.maxPoint>0||!f.isEmpty&&f.maxPoint>=o),a=nb(s,l,r),c=new ya(s,a,o),u=new ya(l,a,o);r.iterGaps((f,h,d)=>ib(c,f,u,h,d,i)),r.empty&&r.length==0&&ib(c,0,u,0,0,i)}static eq(e,n,r=0,i){i==null&&(i=1e9);let o=e.filter(u=>!u.isEmpty&&n.indexOf(u)<0),s=n.filter(u=>!u.isEmpty&&e.indexOf(u)<0);if(o.length!=s.length)return!1;if(!o.length)return!0;let l=nb(o,s),a=new ya(o,l,0).goto(r),c=new ya(s,l,0).goto(r);for(;;){if(a.to!=c.to||!Wm(a.active,c.active)||a.point&&(!c.point||!a.point.eq(c.point)))return!1;if(a.to>i)return!0;a.next(),c.next()}}static spans(e,n,r,i,o=-1){var s;let l=new ya(e,null,o,(s=i.filterPoint)===null||s===void 0?void 0:s.bind(i)).goto(n),a=n,c=l.openStart;for(;;){let u=Math.min(l.to,r);if(l.point?(i.point(a,u,l.point,l.activeForPoint(l.to),c),c=l.openEnd(u)+(l.to>u?1:0)):u>a&&(i.span(a,u,l.active,c),c=l.openEnd(u)),l.to>r)break;a=l.to,l.next()}return c}static of(e,n=!1){let r=new yi;for(let i of e instanceof Mm?[e]:n?_3(e):e)r.add(i.from,i.to,i.value);return r.finish()}}Te.empty=new Te([],[],null,-1);function _3(t){if(t.length>1)for(let e=t[0],n=1;n<t.length;n++){let r=t[n];if(Am(e,r)>0)return t.slice().sort(Am);e=r}return t}Te.empty.nextLayer=Te.empty;class yi{constructor(){this.chunks=[],this.chunkPos=[],this.chunkStart=-1,this.last=null,this.lastFrom=-1e9,this.lastTo=-1e9,this.from=[],this.to=[],this.value=[],this.maxPoint=-1,this.setMaxPoint=-1,this.nextLayer=null}finishChunk(e){this.chunks.push(new Dm(this.from,this.to,this.value,this.maxPoint)),this.chunkPos.push(this.chunkStart),this.chunkStart=-1,this.setMaxPoint=Math.max(this.setMaxPoint,this.maxPoint),this.maxPoint=-1,e&&(this.from=[],this.to=[],this.value=[])}add(e,n,r){this.addInner(e,n,r)||(this.nextLayer||(this.nextLayer=new yi)).add(e,n,r)}addInner(e,n,r){let i=e-this.lastTo||r.startSide-this.last.endSide;if(i<=0&&(e-this.lastFrom||r.startSide-this.last.startSide)<0)throw new Error("Ranges must be added sorted by `from` position and `startSide`");return i<0?!1:(this.from.length==250&&this.finishChunk(!0),this.chunkStart<0&&(this.chunkStart=e),this.from.push(e-this.chunkStart),this.to.push(n-this.chunkStart),this.last=r,this.lastFrom=e,this.lastTo=n,this.value.push(r),r.point&&(this.maxPoint=Math.max(this.maxPoint,n-e)),!0)}addChunk(e,n){if((e-this.lastTo||n.value[0].startSide-this.last.endSide)<0)return!1;this.from.length&&this.finishChunk(!0),this.setMaxPoint=Math.max(this.setMaxPoint,n.maxPoint),this.chunks.push(n),this.chunkPos.push(e);let r=n.value.length-1;return this.last=n.value[r],this.lastFrom=n.from[r]+e,this.lastTo=n.to[r]+e,!0}finish(){return this.finishInner(Te.empty)}finishInner(e){if(this.from.length&&this.finishChunk(!1),this.chunks.length==0)return e;let n=new Te(this.chunkPos,this.chunks,this.nextLayer?this.nextLayer.finishInner(e):e,this.setMaxPoint);return this.from=null,n}}function nb(t,e,n){let r=new Map;for(let o of t)for(let s=0;s<o.chunk.length;s++)o.chunk[s].maxPoint<=0&&r.set(o.chunk[s],o.chunkPos[s]);let i=new Set;for(let o of e)for(let s=0;s<o.chunk.length;s++){let l=r.get(o.chunk[s]);l!=null&&(n?n.mapPos(l):l)==o.chunkPos[s]&&!(n==null?void 0:n.touchesRange(l,l+o.chunk[s].length))&&i.add(o.chunk[s])}return i}class rb{constructor(e,n,r,i=0){this.layer=e,this.skip=n,this.minPoint=r,this.rank=i}get startSide(){return this.value?this.value.startSide:0}get endSide(){return this.value?this.value.endSide:0}goto(e,n=-1e9){return this.chunkIndex=this.rangeIndex=0,this.gotoInner(e,n,!1),this}gotoInner(e,n,r){for(;this.chunkIndex<this.layer.chunk.length;){let i=this.layer.chunk[this.chunkIndex];if(!(this.skip&&this.skip.has(i)||this.layer.chunkEnd(this.chunkIndex)<e||i.maxPoint<this.minPoint))break;this.chunkIndex++,r=!1}if(this.chunkIndex<this.layer.chunk.length){let i=this.layer.chunk[this.chunkIndex].findIndex(e-this.layer.chunkPos[this.chunkIndex],n,!0);(!r||this.rangeIndex<i)&&this.setRangeIndex(i)}this.next()}forward(e,n){(this.to-e||this.endSide-n)<0&&this.gotoInner(e,n,!0)}next(){for(;;)if(this.chunkIndex==this.layer.chunk.length){this.from=this.to=1e9,this.value=null;break}else{let e=this.layer.chunkPos[this.chunkIndex],n=this.layer.chunk[this.chunkIndex],r=e+n.from[this.rangeIndex];if(this.from=r,this.to=e+n.to[this.rangeIndex],this.value=n.value[this.rangeIndex],this.setRangeIndex(this.rangeIndex+1),this.minPoint<0||this.value.point&&this.to-this.from>=this.minPoint)break}}setRangeIndex(e){if(e==this.layer.chunk[this.chunkIndex].value.length){if(this.chunkIndex++,this.skip)for(;this.chunkIndex<this.layer.chunk.length&&this.skip.has(this.layer.chunk[this.chunkIndex]);)this.chunkIndex++;this.rangeIndex=0}else this.rangeIndex=e}nextChunk(){this.chunkIndex++,this.rangeIndex=0,this.next()}compare(e){return this.from-e.from||this.startSide-e.startSide||this.to-e.to||this.endSide-e.endSide}}class ga{constructor(e){this.heap=e}static from(e,n=null,r=-1){let i=[];for(let o=0;o<e.length;o++)for(let s=e[o];!s.isEmpty;s=s.nextLayer)s.maxPoint>=r&&i.push(new rb(s,n,r,o));return i.length==1?i[0]:new ga(i)}get startSide(){return this.value?this.value.startSide:0}goto(e,n=-1e9){for(let r of this.heap)r.goto(e,n);for(let r=this.heap.length>>1;r>=0;r--)Lm(this.heap,r);return this.next(),this}forward(e,n){for(let r of this.heap)r.forward(e,n);for(let r=this.heap.length>>1;r>=0;r--)Lm(this.heap,r);(this.to-e||this.value.endSide-n)<0&&this.next()}next(){if(this.heap.length==0)this.from=this.to=1e9,this.value=null,this.rank=-1;else{let e=this.heap[0];this.from=e.from,this.to=e.to,this.value=e.value,this.rank=e.rank,e.value&&e.next(),Lm(this.heap,0)}}}function Lm(t,e){for(let n=t[e];;){let r=(e<<1)+1;if(r>=t.length)break;let i=t[r];if(r+1<t.length&&i.compare(t[r+1])>=0&&(i=t[r+1],r++),n.compare(i)<0)break;t[r]=n,t[e]=i,e=r}}class ya{constructor(e,n,r,i=()=>!0){this.minPoint=r,this.filterPoint=i,this.active=[],this.activeTo=[],this.activeRank=[],this.minActive=-1,this.point=null,this.pointFrom=0,this.pointRank=0,this.to=-1e9,this.endSide=0,this.openStart=-1,this.cursor=ga.from(e,n,r)}goto(e,n=-1e9){return this.cursor.goto(e,n),this.active.length=this.activeTo.length=this.activeRank.length=0,this.minActive=-1,this.to=e,this.endSide=n,this.openStart=-1,this.next(),this}forward(e,n){for(;this.minActive>-1&&(this.activeTo[this.minActive]-e||this.active[this.minActive].endSide-n)<0;)this.removeActive(this.minActive);this.cursor.forward(e,n)}removeActive(e){Hu(this.active,e),Hu(this.activeTo,e),Hu(this.activeRank,e),this.minActive=ob(this.active,this.activeTo)}addActive(e){let n=0,{value:r,to:i,rank:o}=this.cursor;for(;n<this.activeRank.length&&this.activeRank[n]<=o;)n++;Gu(this.active,n,r),Gu(this.activeTo,n,i),Gu(this.activeRank,n,o),e&&Gu(e,n,this.cursor.from),this.minActive=ob(this.active,this.activeTo)}next(){let e=this.to,n=this.point;this.point=null;let r=this.openStart<0?[]:null,i=0;for(;;){let o=this.minActive;if(o>-1&&(this.activeTo[o]-this.cursor.from||this.active[o].endSide-this.cursor.startSide)<0){if(this.activeTo[o]>e){this.to=this.activeTo[o],this.endSide=this.active[o].endSide;break}this.removeActive(o),r&&Hu(r,o)}else if(this.cursor.value)if(this.cursor.from>e){this.to=this.cursor.from,this.endSide=this.cursor.startSide;break}else{let s=this.cursor.value;if(!s.point)this.addActive(r),this.cursor.next();else if(n&&this.cursor.to==this.to&&this.cursor.from<this.cursor.to)this.cursor.next();else if(!this.filterPoint(this.cursor.from,this.cursor.to,this.cursor.value,this.cursor.rank))this.cursor.next();else{this.point=s,this.pointFrom=this.cursor.from,this.pointRank=this.cursor.rank,this.to=this.cursor.to,this.endSide=s.endSide,this.cursor.from<e&&(i=1),this.cursor.next(),this.to>e&&this.forward(this.to,this.endSide);break}}else{this.to=this.endSide=1e9;break}}if(r){let o=0;for(;o<r.length&&r[o]<e;)o++;this.openStart=o+i}}activeForPoint(e){if(!this.active.length)return this.active;let n=[];for(let r=this.active.length-1;r>=0&&!(this.activeRank[r]<this.pointRank);r--)(this.activeTo[r]>e||this.activeTo[r]==e&&this.active[r].endSide>=this.point.endSide)&&n.push(this.active[r]);return n.reverse()}openEnd(e){let n=0;for(let r=this.activeTo.length-1;r>=0&&this.activeTo[r]>e;r--)n++;return n}}function ib(t,e,n,r,i,o){t.goto(e),n.goto(r);let s=r+i,l=r,a=r-e;for(;;){let c=t.to+a-n.to||t.endSide-n.endSide,u=c<0?t.to+a:n.to,f=Math.min(u,s);if(t.point||n.point?t.point&&n.point&&(t.point==n.point||t.point.eq(n.point))&&Wm(t.activeForPoint(t.to+a),n.activeForPoint(n.to))||o.comparePoint(l,f,t.point,n.point):f>l&&!Wm(t.active,n.active)&&o.compareRange(l,f,t.active,n.active),u>s)break;l=u,c<=0&&t.next(),c>=0&&n.next()}}function Wm(t,e){if(t.length!=e.length)return!1;for(let n=0;n<t.length;n++)if(t[n]!=e[n]&&!t[n].eq(e[n]))return!1;return!0}function Hu(t,e){for(let n=e,r=t.length-1;n<r;n++)t[n]=t[n+1];t.pop()}function Gu(t,e,n){for(let r=t.length-1;r>=e;r--)t[r+1]=t[r];t[e]=n}function ob(t,e){let n=-1,r=1e9;for(let i=0;i<e.length;i++)(e[i]-r||t[i].endSide-t[n].endSide)<0&&(n=i,r=e[i]);return n}var Oi={8:"Backspace",9:"Tab",10:"Enter",12:"NumLock",13:"Enter",16:"Shift",17:"Control",18:"Alt",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",44:"PrintScreen",45:"Insert",46:"Delete",59:";",61:"=",91:"Meta",92:"Meta",106:"*",107:"+",108:",",109:"-",110:".",111:"/",144:"NumLock",145:"ScrollLock",160:"Shift",161:"Shift",162:"Control",163:"Control",164:"Alt",165:"Alt",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'",229:"q"},Zu={48:")",49:"!",50:"@",51:"#",52:"$",53:"%",54:"^",55:"&",56:"*",57:"(",59:":",61:"+",173:"_",186:":",187:"+",188:"<",189:"_",190:">",191:"?",192:"~",219:"{",220:"|",221:"}",222:'"',229:"Q"},sb=typeof navigator!="undefined"&&/Chrome\/(\d+)/.exec(navigator.userAgent),B3=typeof navigator!="undefined"&&/Apple Computer/.test(navigator.vendor),q3=typeof navigator!="undefined"&&/Gecko\/\d+/.test(navigator.userAgent),lb=typeof navigator!="undefined"&&/Mac/.test(navigator.platform),z3=typeof navigator!="undefined"&&/MSIE \d|Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(navigator.userAgent),F3=sb&&(lb||+sb[1]<57)||q3&&lb;for(var gt=0;gt<10;gt++)Oi[48+gt]=Oi[96+gt]=String(gt);for(var gt=1;gt<=24;gt++)Oi[gt+111]="F"+gt;for(var gt=65;gt<=90;gt++)Oi[gt]=String.fromCharCode(gt+32),Zu[gt]=String.fromCharCode(gt);for(var Qm in Oi)Zu.hasOwnProperty(Qm)||(Zu[Qm]=Oi[Qm]);function V3(t){var e=F3&&(t.ctrlKey||t.altKey||t.metaKey)||(B3||z3)&&t.shiftKey&&t.key&&t.key.length==1,n=!e&&t.key||(t.shiftKey?Zu:Oi)[t.keyCode]||t.key||"Unidentified";return n=="Esc"&&(n="Escape"),n=="Del"&&(n="Delete"),n=="Left"&&(n="ArrowLeft"),n=="Up"&&(n="ArrowUp"),n=="Right"&&(n="ArrowRight"),n=="Down"&&(n="ArrowDown"),n}function Yu(t){let e;return t.nodeType==11?e=t.getSelection?t:t.ownerDocument:e=t,e.getSelection()}function Nm(t,e){return e?t.contains(e.nodeType!=1?e.parentNode:e):!1}function U3(){let t=document.activeElement;for(;t&&t.shadowRoot;)t=t.shadowRoot.activeElement;return t}function _m(t,e){if(!e.anchorNode)return!1;try{return Nm(t,e.anchorNode)}catch{return!1}}function ds(t){return t.nodeType==3?Oa(t,0,t.nodeValue.length).getClientRects():t.nodeType==1?t.getClientRects():[]}function Ku(t,e,n,r){return n?ab(t,e,n,r,-1)||ab(t,e,n,r,1):!1}function Bm(t){for(var e=0;;e++)if(t=t.previousSibling,!t)return e}function ab(t,e,n,r,i){for(;;){if(t==n&&e==r)return!0;if(e==(i<0?0:Ju(t))){if(t.nodeName=="DIV")return!1;let o=t.parentNode;if(!o||o.nodeType!=1)return!1;e=Bm(t)+(i<0?0:1),t=o}else if(t.nodeType==1){if(t=t.childNodes[e+(i<0?-1:0)],t.nodeType==1&&t.contentEditable=="false")return!1;e=i<0?Ju(t):0}else return!1}}function Ju(t){return t.nodeType==3?t.nodeValue.length:t.childNodes.length}const cb={left:0,right:0,top:0,bottom:0};function ef(t,e){let n=e?t.left:t.right;return{left:n,right:n,top:t.top,bottom:t.bottom}}function j3(t){return{left:0,right:t.innerWidth,top:0,bottom:t.innerHeight}}function X3(t,e,n,r,i,o,s,l){let a=t.ownerDocument,c=a.defaultView;for(let u=t;u;)if(u.nodeType==1){let f,h=u==a.body;if(h)f=j3(c);else{if(u.scrollHeight<=u.clientHeight&&u.scrollWidth<=u.clientWidth){u=u.parentNode;continue}let m=u.getBoundingClientRect();f={left:m.left,right:m.left+u.clientWidth,top:m.top,bottom:m.top+u.clientHeight}}let d=0,p=0;if(i=="nearest")e.top<f.top?(p=-(f.top-e.top+s),n>0&&e.bottom>f.bottom+p&&(p=e.bottom-f.bottom+p+s)):e.bottom>f.bottom&&(p=e.bottom-f.bottom+s,n<0&&e.top-p<f.top&&(p=-(f.top+p-e.top+s)));else{let m=e.bottom-e.top,g=f.bottom-f.top;p=(i=="center"&&m<=g?e.top+m/2-g/2:i=="start"||i=="center"&&n<0?e.top-s:e.bottom-g+s)-f.top}if(r=="nearest"?e.left<f.left?(d=-(f.left-e.left+o),n>0&&e.right>f.right+d&&(d=e.right-f.right+d+o)):e.right>f.right&&(d=e.right-f.right+o,n<0&&e.left<f.left+d&&(d=-(f.left+d-e.left+o))):d=(r=="center"?e.left+(e.right-e.left)/2-(f.right-f.left)/2:r=="start"==l?e.left-o:e.right-(f.right-f.left)+o)-f.left,d||p)if(h)c.scrollBy(d,p);else{if(p){let m=u.scrollTop;u.scrollTop+=p,p=u.scrollTop-m}if(d){let m=u.scrollLeft;u.scrollLeft+=d,d=u.scrollLeft-m}e={left:e.left-d,top:e.top-p,right:e.right-d,bottom:e.bottom-p}}if(h)break;u=u.assignedSlot||u.parentNode,r=i="nearest"}else if(u.nodeType==11)u=u.host;else break}class H3{constructor(){this.anchorNode=null,this.anchorOffset=0,this.focusNode=null,this.focusOffset=0}eq(e){return this.anchorNode==e.anchorNode&&this.anchorOffset==e.anchorOffset&&this.focusNode==e.focusNode&&this.focusOffset==e.focusOffset}setRange(e){this.set(e.anchorNode,e.anchorOffset,e.focusNode,e.focusOffset)}set(e,n,r,i){this.anchorNode=e,this.anchorOffset=n,this.focusNode=r,this.focusOffset=i}}let ps=null;function ub(t){if(t.setActive)return t.setActive();if(ps)return t.focus(ps);let e=[];for(let n=t;n&&(e.push(n,n.scrollTop,n.scrollLeft),n!=n.ownerDocument);n=n.parentNode);if(t.focus(ps==null?{get preventScroll(){return ps={preventScroll:!0},!0}}:void 0),!ps){ps=!1;for(let n=0;n<e.length;){let r=e[n++],i=e[n++],o=e[n++];r.scrollTop!=i&&(r.scrollTop=i),r.scrollLeft!=o&&(r.scrollLeft=o)}}}let fb;function Oa(t,e,n=e){let r=fb||(fb=document.createRange());return r.setEnd(t,n),r.setStart(t,e),r}function va(t,e,n){let r={key:e,code:e,keyCode:n,which:n,cancelable:!0},i=new KeyboardEvent("keydown",r);i.synthetic=!0,t.dispatchEvent(i);let o=new KeyboardEvent("keyup",r);return o.synthetic=!0,t.dispatchEvent(o),i.defaultPrevented||o.defaultPrevented}function G3(t){for(;t;){if(t&&(t.nodeType==9||t.nodeType==11&&t.host))return t;t=t.assignedSlot||t.parentNode}return null}function hb(t){for(;t.attributes.length;)t.removeAttributeNode(t.attributes[0])}class Tt{constructor(e,n,r=!0){this.node=e,this.offset=n,this.precise=r}static before(e,n){return new Tt(e.parentNode,Bm(e),n)}static after(e,n){return new Tt(e.parentNode,Bm(e)+1,n)}}const qm=[];class He{constructor(){this.parent=null,this.dom=null,this.dirty=2}get editorView(){if(!this.parent)throw new Error("Accessing view in orphan content view");return this.parent.editorView}get overrideDOMText(){return null}get posAtStart(){return this.parent?this.parent.posBefore(this):0}get posAtEnd(){return this.posAtStart+this.length}posBefore(e){let n=this.posAtStart;for(let r of this.children){if(r==e)return n;n+=r.length+r.breakAfter}throw new RangeError("Invalid child in posBefore")}posAfter(e){return this.posBefore(e)+e.length}coordsAt(e,n){return null}sync(e){if(this.dirty&2){let n=this.dom,r=n.firstChild;for(let i of this.children){if(i.dirty){if(!i.dom&&r){let o=He.get(r);(!o||!o.parent&&o.constructor==i.constructor)&&i.reuseDOM(r)}i.sync(e),i.dirty=0}if(e&&!e.written&&e.node==n&&r!=i.dom&&(e.written=!0),i.dom.parentNode==n){for(;r&&r!=i.dom;)r=db(r);r=i.dom.nextSibling}else n.insertBefore(i.dom,r)}for(r&&e&&e.node==n&&(e.written=!0);r;)r=db(r)}else if(this.dirty&1)for(let n of this.children)n.dirty&&(n.sync(e),n.dirty=0)}reuseDOM(e){}localPosFromDOM(e,n){let r;if(e==this.dom)r=this.dom.childNodes[n];else{let i=Ju(e)==0?0:n==0?-1:1;for(;;){let o=e.parentNode;if(o==this.dom)break;i==0&&o.firstChild!=o.lastChild&&(e==o.firstChild?i=-1:i=1),e=o}i<0?r=e:r=e.nextSibling}if(r==this.dom.firstChild)return 0;for(;r&&!He.get(r);)r=r.nextSibling;if(!r)return this.length;for(let i=0,o=0;;i++){let s=this.children[i];if(s.dom==r)return o;o+=s.length+s.breakAfter}}domBoundsAround(e,n,r=0){let i=-1,o=-1,s=-1,l=-1;for(let a=0,c=r,u=r;a<this.children.length;a++){let f=this.children[a],h=c+f.length;if(c<e&&h>n)return f.domBoundsAround(e,n,c);if(h>=e&&i==-1&&(i=a,o=c),c>n&&f.dom.parentNode==this.dom){s=a,l=u;break}u=h,c=h+f.breakAfter}return{from:o,to:l<0?r+this.length:l,startDOM:(i?this.children[i-1].dom.nextSibling:null)||this.dom.firstChild,endDOM:s<this.children.length&&s>=0?this.children[s].dom:null}}markDirty(e=!1){this.dirty|=2,this.markParentsDirty(e)}markParentsDirty(e){for(let n=this.parent;n;n=n.parent){if(e&&(n.dirty|=2),n.dirty&1)return;n.dirty|=1,e=!1}}setParent(e){this.parent!=e&&(this.parent=e,this.dirty&&this.markParentsDirty(!0))}setDOM(e){this.dom&&(this.dom.cmView=null),this.dom=e,e.cmView=this}get rootView(){for(let e=this;;){let n=e.parent;if(!n)return e;e=n}}replaceChildren(e,n,r=qm){this.markDirty();for(let i=e;i<n;i++){let o=this.children[i];o.parent==this&&o.destroy()}this.children.splice(e,n-e,...r);for(let i=0;i<r.length;i++)r[i].setParent(this)}ignoreMutation(e){return!1}ignoreEvent(e){return!1}childCursor(e=this.length){return new pb(this.children,e,this.children.length)}childPos(e,n=1){return this.childCursor().findPos(e,n)}toString(){let e=this.constructor.name.replace("View","");return e+(this.children.length?"("+this.children.join()+")":this.length?"["+(e=="Text"?this.text:this.length)+"]":"")+(this.breakAfter?"#":"")}static get(e){return e.cmView}get isEditable(){return!0}merge(e,n,r,i,o,s){return!1}become(e){return!1}getSide(){return 0}destroy(){this.parent=null}}He.prototype.breakAfter=0;function db(t){let e=t.nextSibling;return t.parentNode.removeChild(t),e}class pb{constructor(e,n,r){this.children=e,this.pos=n,this.i=r,this.off=0}findPos(e,n=1){for(;;){if(e>this.pos||e==this.pos&&(n>0||this.i==0||this.children[this.i-1].breakAfter))return this.off=e-this.pos,this;let r=this.children[--this.i];this.pos-=r.length+r.breakAfter}}}function mb(t,e,n,r,i,o,s,l,a){let{children:c}=t,u=c.length?c[e]:null,f=o.length?o[o.length-1]:null,h=f?f.breakAfter:s;if(!(e==r&&u&&!s&&!h&&o.length<2&&u.merge(n,i,o.length?f:null,n==0,l,a))){if(r<c.length){let d=c[r];d&&i<d.length?(e==r&&(d=d.split(i),i=0),!h&&f&&d.merge(0,i,f,!0,0,a)?o[o.length-1]=d:(i&&d.merge(0,i,null,!1,0,a),o.push(d))):(d==null?void 0:d.breakAfter)&&(f?f.breakAfter=1:s=1),r++}for(u&&(u.breakAfter=s,n>0&&(!s&&o.length&&u.merge(n,u.length,o[0],!1,l,0)?u.breakAfter=o.shift().breakAfter:(n<u.length||u.children.length&&u.children[u.children.length-1].length==0)&&u.merge(n,u.length,null,!1,l,0),e++));e<r&&o.length;)if(c[r-1].become(o[o.length-1]))r--,o.pop(),a=o.length?0:l;else if(c[e].become(o[0]))e++,o.shift(),l=o.length?0:a;else break;!o.length&&e&&r<c.length&&!c[e-1].breakAfter&&c[r].merge(0,0,c[e-1],!1,l,a)&&e--,(e<r||o.length)&&t.replaceChildren(e,r,o)}}function gb(t,e,n,r,i,o){let s=t.childCursor(),{i:l,off:a}=s.findPos(n,1),{i:c,off:u}=s.findPos(e,-1),f=e-n;for(let h of r)f+=h.length;t.length+=f,mb(t,c,u,l,a,r,0,i,o)}let[on,zm]=typeof navigator!="undefined"?[navigator,document]:[{userAgent:"",vendor:"",platform:""},{documentElement:{style:{}}}];const Fm=/Edge\/(\d+)/.exec(on.userAgent),yb=/MSIE \d/.test(on.userAgent),Vm=/Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(on.userAgent),tf=!!(yb||Vm||Fm),Ob=!tf&&/gecko\/(\d+)/i.test(on.userAgent),Um=!tf&&/Chrome\/(\d+)/.exec(on.userAgent),vb="webkitFontSmoothing"in zm.documentElement.style,xb=!tf&&/Apple Computer/.test(on.vendor),wb=xb&&(/Mobile\/\w+/.test(on.userAgent)||on.maxTouchPoints>2);var re={mac:wb||/Mac/.test(on.platform),windows:/Win/.test(on.platform),linux:/Linux|X11/.test(on.platform),ie:tf,ie_version:yb?zm.documentMode||6:Vm?+Vm[1]:Fm?+Fm[1]:0,gecko:Ob,gecko_version:Ob?+(/Firefox\/(\d+)/.exec(on.userAgent)||[0,0])[1]:0,chrome:!!Um,chrome_version:Um?+Um[1]:0,ios:wb,android:/Android\b/.test(on.userAgent),webkit:vb,safari:xb,webkit_version:vb?+(/\bAppleWebKit\/(\d+)/.exec(navigator.userAgent)||[0,0])[1]:0,tabSize:zm.documentElement.style.tabSize!=null?"tab-size":"-moz-tab-size"};const Z3=256;class Zi extends He{constructor(e){super();this.text=e}get length(){return this.text.length}createDOM(e){this.setDOM(e||document.createTextNode(this.text))}sync(e){this.dom||this.createDOM(),this.dom.nodeValue!=this.text&&(e&&e.node==this.dom&&(e.written=!0),this.dom.nodeValue=this.text)}reuseDOM(e){e.nodeType==3&&this.createDOM(e)}merge(e,n,r){return r&&(!(r instanceof Zi)||this.length-(n-e)+r.length>Z3)?!1:(this.text=this.text.slice(0,e)+(r?r.text:"")+this.text.slice(n),this.markDirty(),!0)}split(e){let n=new Zi(this.text.slice(e));return this.text=this.text.slice(0,e),this.markDirty(),n}localPosFromDOM(e,n){return e==this.dom?n:n?this.text.length:0}domAtPos(e){return new Tt(this.dom,e)}domBoundsAround(e,n,r){return{from:r,to:r+this.length,startDOM:this.dom,endDOM:this.dom.nextSibling}}coordsAt(e,n){return bb(this.dom,e,n)}}class Yi extends He{constructor(e,n=[],r=0){super();this.mark=e,this.children=n,this.length=r;for(let i of n)i.setParent(this)}setAttrs(e){if(hb(e),this.mark.class&&(e.className=this.mark.class),this.mark.attrs)for(let n in this.mark.attrs)e.setAttribute(n,this.mark.attrs[n]);return e}reuseDOM(e){e.nodeName==this.mark.tagName.toUpperCase()&&(this.setDOM(e),this.dirty|=4|2)}sync(e){this.dom?this.dirty&4&&this.setAttrs(this.dom):this.setDOM(this.setAttrs(document.createElement(this.mark.tagName))),super.sync(e)}merge(e,n,r,i,o,s){return r&&(!(r instanceof Yi&&r.mark.eq(this.mark))||e&&o<=0||n<this.length&&s<=0)?!1:(gb(this,e,n,r?r.children:[],o-1,s-1),this.markDirty(),!0)}split(e){let n=[],r=0,i=-1,o=0;for(let l of this.children){let a=r+l.length;a>e&&n.push(r<e?l.split(e-r):l),i<0&&r>=e&&(i=o),r=a,o++}let s=this.length-e;return this.length=e,i>-1&&(this.children.length=i,this.markDirty()),new Yi(this.mark,n,s)}domAtPos(e){return $b(this.dom,this.children,e)}coordsAt(e,n){return kb(this,e,n)}}function bb(t,e,n){let r=t.nodeValue.length;e>r&&(e=r);let i=e,o=e,s=0;e==0&&n<0||e==r&&n>=0?re.chrome||re.gecko||(e?(i--,s=1):(o++,s=-1)):n<0?i--:o++;let l=Oa(t,i,o).getClientRects();if(!l.length)return cb;let a=l[(s?s<0:n>=0)?0:l.length-1];return re.safari&&!s&&a.width==0&&(a=Array.prototype.find.call(l,c=>c.width)||a),s?ef(a,s<0):a||null}class vi extends He{constructor(e,n,r){super();this.widget=e,this.length=n,this.side=r}static create(e,n,r){return new(e.customView||vi)(e,n,r)}split(e){let n=vi.create(this.widget,this.length-e,this.side);return this.length-=e,n}sync(){(!this.dom||!this.widget.updateDOM(this.dom))&&(this.setDOM(this.widget.toDOM(this.editorView)),this.dom.contentEditable="false")}getSide(){return this.side}merge(e,n,r,i,o,s){return r&&(!(r instanceof vi)||!this.widget.compare(r.widget)||e>0&&o<=0||n<this.length&&s<=0)?!1:(this.length=e+(r?r.length:0)+(this.length-n),!0)}become(e){return e.length==this.length&&e instanceof vi&&e.side==this.side&&this.widget.constructor==e.widget.constructor?(this.widget.eq(e.widget)||this.markDirty(!0),this.widget=e.widget,!0):!1}ignoreMutation(){return!0}ignoreEvent(e){return this.widget.ignoreEvent(e)}get overrideDOMText(){if(this.length==0)return Ie.empty;let e=this;for(;e.parent;)e=e.parent;let n=e.editorView,r=n&&n.state.doc,i=this.posAtStart;return r?r.slice(i,i+this.length):Ie.empty}domAtPos(e){return e==0?Tt.before(this.dom):Tt.after(this.dom,e==this.length)}domBoundsAround(){return null}coordsAt(e,n){let r=this.dom.getClientRects(),i=null;if(!r.length)return cb;for(let o=e>0?r.length-1:0;i=r[o],!(e>0?o==0:o==r.length-1||i.top<i.bottom);o+=e>0?-1:1);return e==0&&n>0||e==this.length&&n<=0?i:ef(i,e==0)}get isEditable(){return!1}destroy(){super.destroy(),this.dom&&this.widget.destroy(this.dom)}}class Y3 extends vi{domAtPos(e){return new Tt(this.widget.text,e)}sync(){this.setDOM(this.widget.toDOM())}localPosFromDOM(e,n){return n?e.nodeType==3?Math.min(n,this.length):this.length:0}ignoreMutation(){return!1}get overrideDOMText(){return null}coordsAt(e,n){return bb(this.widget.text,e,n)}get isEditable(){return!0}}const jm=re.android?"\u200B\u200B":"\u200B";class ms extends He{constructor(e){super();this.side=e}get length(){return 0}merge(){return!1}become(e){return e instanceof ms&&e.side==this.side}split(){return new ms(this.side)}sync(){this.dom?this.dirty&&this.dom.nodeValue!=jm&&(this.dom.nodeValue=jm):this.setDOM(document.createTextNode(jm))}getSide(){return this.side}domAtPos(e){return Tt.before(this.dom)}localPosFromDOM(){return 0}domBoundsAround(){return null}coordsAt(e){let n=ds(this.dom);return n[n.length-1]||null}get overrideDOMText(){return Ie.of([this.dom.nodeValue.replace(/\u200b/g,"")])}}Zi.prototype.children=vi.prototype.children=ms.prototype.children=qm;function $b(t,e,n){let r=0;for(let i=0;r<e.length;r++){let o=e[r],s=i+o.length;if(!(s==i&&o.getSide()<=0)){if(n>i&&n<s&&o.dom.parentNode==t)return o.domAtPos(n-i);if(n<=i)break;i=s}}for(;r>0;r--){let i=e[r-1].dom;if(i.parentNode==t)return Tt.after(i)}return new Tt(t,0)}function Sb(t,e,n){let r,{children:i}=t;n>0&&e instanceof Yi&&i.length&&(r=i[i.length-1])instanceof Yi&&r.mark.eq(e.mark)?Sb(r,e.children[0],n-1):(i.push(e),e.setParent(t)),t.length+=e.length}function kb(t,e,n){for(let o=0,s=0;s<t.children.length;s++){let l=t.children[s],a=o+l.length,c;if((n<=0||a==t.length||l.getSide()>0?a>=e:a>e)&&(e<a||s+1==t.children.length||(c=t.children[s+1]).length||c.getSide()>0)){let u=0;if(a==o){if(l.getSide()<=0)continue;u=n=-l.getSide()}let f=l.coordsAt(e-o,n);return u&&f?ef(f,n<0):f}o=a}let r=t.dom.lastChild;if(!r)return t.dom.getBoundingClientRect();let i=ds(r);return i[i.length-1]||null}function Xm(t,e){for(let n in t)n=="class"&&e.class?e.class+=" "+t.class:n=="style"&&e.style?e.style+=";"+t.style:e[n]=t[n];return e}function Hm(t,e){if(t==e)return!0;if(!t||!e)return!1;let n=Object.keys(t),r=Object.keys(e);if(n.length!=r.length)return!1;for(let i of n)if(r.indexOf(i)==-1||t[i]!==e[i])return!1;return!0}function Gm(t,e,n){if(e)for(let r in e)n&&r in n||t.removeAttribute(r);if(n)for(let r in n)e&&e[r]==n[r]||t.setAttribute(r,n[r])}class Cr{eq(e){return!1}updateDOM(e){return!1}compare(e){return this==e||this.constructor==e.constructor&&this.eq(e)}get estimatedHeight(){return-1}ignoreEvent(e){return!0}get customView(){return null}destroy(e){}}var De=function(t){return t[t.Text=0]="Text",t[t.WidgetBefore=1]="WidgetBefore",t[t.WidgetAfter=2]="WidgetAfter",t[t.WidgetRange=3]="WidgetRange",t}(De||(De={}));class te extends Gi{constructor(e,n,r,i){super();this.startSide=e,this.endSide=n,this.widget=r,this.spec=i}get heightRelevant(){return!1}static mark(e){return new nf(e)}static widget(e){let n=e.side||0,r=!!e.block;return n+=r?n>0?3e8:-4e8:n>0?1e8:-1e8,new Ki(e,n,n,r,e.widget||null,!1)}static replace(e){let n=!!e.block,{start:r,end:i}=Pb(e,n),o=n?r?-3e8:-1:4e8,s=n?i?2e8:1:-5e8;return new Ki(e,o,s,n,e.widget||null,!0)}static line(e){return new xa(e)}static set(e,n=!1){return Te.of(e,n)}hasHeight(){return this.widget?this.widget.estimatedHeight>-1:!1}}te.none=Te.empty;class nf extends te{constructor(e){let{start:n,end:r}=Pb(e);super(n?-1:4e8,r?1:-5e8,null,e);this.tagName=e.tagName||"span",this.class=e.class||"",this.attrs=e.attributes||null}eq(e){return this==e||e instanceof nf&&this.tagName==e.tagName&&this.class==e.class&&Hm(this.attrs,e.attrs)}range(e,n=e){if(e>=n)throw new RangeError("Mark decorations may not be empty");return super.range(e,n)}}nf.prototype.point=!1;class xa extends te{constructor(e){super(-2e8,-2e8,null,e)}eq(e){return e instanceof xa&&Hm(this.spec.attributes,e.spec.attributes)}range(e,n=e){if(n!=e)throw new RangeError("Line decoration ranges must be zero-length");return super.range(e,n)}}xa.prototype.mapMode=Dt.TrackBefore;xa.prototype.point=!0;class Ki extends te{constructor(e,n,r,i,o,s){super(n,r,o,e);this.block=i,this.isReplace=s,this.mapMode=i?n<=0?Dt.TrackBefore:Dt.TrackAfter:Dt.TrackDel}get type(){return this.startSide<this.endSide?De.WidgetRange:this.startSide<=0?De.WidgetBefore:De.WidgetAfter}get heightRelevant(){return this.block||!!this.widget&&this.widget.estimatedHeight>=5}eq(e){return e instanceof Ki&&K3(this.widget,e.widget)&&this.block==e.block&&this.startSide==e.startSide&&this.endSide==e.endSide}range(e,n=e){if(this.isReplace&&(e>n||e==n&&this.startSide>0&&this.endSide<=0))throw new RangeError("Invalid range for replacement decoration");if(!this.isReplace&&n!=e)throw new RangeError("Widget decorations can only have zero-length ranges");return super.range(e,n)}}Ki.prototype.point=!0;function Pb(t,e=!1){let{inclusiveStart:n,inclusiveEnd:r}=t;return n==null&&(n=t.inclusive),r==null&&(r=t.inclusive),{start:n!=null?n:e,end:r!=null?r:e}}function K3(t,e){return t==e||!!(t&&e&&t.compare(e))}function Zm(t,e,n,r=0){let i=n.length-1;i>=0&&n[i]+r>t?n[i]=Math.max(n[i],e):n.push(t,e)}class Wt extends He{constructor(){super(...arguments);this.children=[],this.length=0,this.prevAttrs=void 0,this.attrs=null,this.breakAfter=0}merge(e,n,r,i,o,s){if(r){if(!(r instanceof Wt))return!1;this.dom||r.transferDOM(this)}return i&&this.setDeco(r?r.attrs:null),gb(this,e,n,r?r.children:[],o,s),!0}split(e){let n=new Wt;if(n.breakAfter=this.breakAfter,this.length==0)return n;let{i:r,off:i}=this.childPos(e);i&&(n.append(this.children[r].split(i),0),this.children[r].merge(i,this.children[r].length,null,!1,0,0),r++);for(let o=r;o<this.children.length;o++)n.append(this.children[o],0);for(;r>0&&this.children[r-1].length==0;)this.children[--r].destroy();return this.children.length=r,this.markDirty(),this.length=e,n}transferDOM(e){!this.dom||(e.setDOM(this.dom),e.prevAttrs=this.prevAttrs===void 0?this.attrs:this.prevAttrs,this.prevAttrs=void 0,this.dom=null)}setDeco(e){Hm(this.attrs,e)||(this.dom&&(this.prevAttrs=this.attrs,this.markDirty()),this.attrs=e)}append(e,n){Sb(this,e,n)}addLineDeco(e){let n=e.spec.attributes,r=e.spec.class;n&&(this.attrs=Xm(n,this.attrs||{})),r&&(this.attrs=Xm({class:r},this.attrs||{}))}domAtPos(e){return $b(this.dom,this.children,e)}reuseDOM(e){e.nodeName=="DIV"&&(this.setDOM(e),this.dirty|=4|2)}sync(e){var n;this.dom?this.dirty&4&&(hb(this.dom),this.dom.className="cm-line",this.prevAttrs=this.attrs?null:void 0):(this.setDOM(document.createElement("div")),this.dom.className="cm-line",this.prevAttrs=this.attrs?null:void 0),this.prevAttrs!==void 0&&(Gm(this.dom,this.prevAttrs,this.attrs),this.dom.classList.add("cm-line"),this.prevAttrs=void 0),super.sync(e);let r=this.dom.lastChild;for(;r&&He.get(r)instanceof Yi;)r=r.lastChild;if(!r||r.nodeName!="BR"&&((n=He.get(r))===null||n===void 0?void 0:n.isEditable)==!1&&(!re.ios||!this.children.some(i=>i instanceof Zi))){let i=document.createElement("BR");i.cmIgnore=!0,this.dom.appendChild(i)}}measureTextSize(){if(this.children.length==0||this.length>20)return null;let e=0;for(let n of this.children){if(!(n instanceof Zi))return null;let r=ds(n.dom);if(r.length!=1)return null;e+=r[0].width}return{lineHeight:this.dom.getBoundingClientRect().height,charWidth:e/this.length}}coordsAt(e,n){return kb(this,e,n)}become(e){return!1}get type(){return De.Text}static find(e,n){for(let r=0,i=0;r<e.children.length;r++){let o=e.children[r],s=i+o.length;if(s>=n){if(o instanceof Wt)return o;if(s>n)break}i=s+o.breakAfter}return null}}class Ji extends He{constructor(e,n,r){super();this.widget=e,this.length=n,this.type=r,this.breakAfter=0}merge(e,n,r,i,o,s){return r&&(!(r instanceof Ji)||!this.widget.compare(r.widget)||e>0&&o<=0||n<this.length&&s<=0)?!1:(this.length=e+(r?r.length:0)+(this.length-n),!0)}domAtPos(e){return e==0?Tt.before(this.dom):Tt.after(this.dom,e==this.length)}split(e){let n=this.length-e;this.length=e;let r=new Ji(this.widget,n,this.type);return r.breakAfter=this.breakAfter,r}get children(){return qm}sync(){(!this.dom||!this.widget.updateDOM(this.dom))&&(this.setDOM(this.widget.toDOM(this.editorView)),this.dom.contentEditable="false")}get overrideDOMText(){return this.parent?this.parent.view.state.doc.slice(this.posAtStart,this.posAtEnd):Ie.empty}domBoundsAround(){return null}become(e){return e instanceof Ji&&e.type==this.type&&e.widget.constructor==this.widget.constructor?(e.widget.eq(this.widget)||this.markDirty(!0),this.widget=e.widget,this.length=e.length,this.breakAfter=e.breakAfter,!0):!1}ignoreMutation(){return!0}ignoreEvent(e){return this.widget.ignoreEvent(e)}destroy(){super.destroy(),this.dom&&this.widget.destroy(this.dom)}}class Ym{constructor(e,n,r,i){this.doc=e,this.pos=n,this.end=r,this.disallowBlockEffectsBelow=i,this.content=[],this.curLine=null,this.breakAtStart=0,this.pendingBuffer=0,this.atCursorPos=!0,this.openStart=-1,this.openEnd=-1,this.text="",this.textOff=0,this.cursor=e.iter(),this.skip=n}posCovered(){if(this.content.length==0)return!this.breakAtStart&&this.doc.lineAt(this.pos).from!=this.pos;let e=this.content[this.content.length-1];return!e.breakAfter&&!(e instanceof Ji&&e.type==De.WidgetBefore)}getLine(){return this.curLine||(this.content.push(this.curLine=new Wt),this.atCursorPos=!0),this.curLine}flushBuffer(e){this.pendingBuffer&&(this.curLine.append(rf(new ms(-1),e),e.length),this.pendingBuffer=0)}addBlockWidget(e){this.flushBuffer([]),this.curLine=null,this.content.push(e)}finish(e){e?this.pendingBuffer=0:this.flushBuffer([]),this.posCovered()||this.getLine()}buildText(e,n,r){for(;e>0;){if(this.textOff==this.text.length){let{value:o,lineBreak:s,done:l}=this.cursor.next(this.skip);if(this.skip=0,l)throw new Error("Ran out of text content when drawing inline views");if(s){this.posCovered()||this.getLine(),this.content.length?this.content[this.content.length-1].breakAfter=1:this.breakAtStart=1,this.flushBuffer([]),this.curLine=null,e--;continue}else this.text=o,this.textOff=0}let i=Math.min(this.text.length-this.textOff,e,512);this.flushBuffer(n),this.getLine().append(rf(new Zi(this.text.slice(this.textOff,this.textOff+i)),n),r),this.atCursorPos=!0,this.textOff+=i,e-=i,r=0}}span(e,n,r,i){this.buildText(n-e,r,i),this.pos=n,this.openStart<0&&(this.openStart=i)}point(e,n,r,i,o){let s=n-e;if(r instanceof Ki)if(r.block){let{type:l}=r;l==De.WidgetAfter&&!this.posCovered()&&this.getLine(),this.addBlockWidget(new Ji(r.widget||new Cb("div"),s,l))}else{let l=vi.create(r.widget||new Cb("span"),s,r.startSide),a=this.atCursorPos&&!l.isEditable&&o<=i.length&&(e<n||r.startSide>0),c=!l.isEditable&&(e<n||r.startSide<=0),u=this.getLine();this.pendingBuffer==2&&!a&&(this.pendingBuffer=0),this.flushBuffer(i),a&&(u.append(rf(new ms(1),i),o),o=i.length+Math.max(0,o-i.length)),u.append(rf(l,i),o),this.atCursorPos=c,this.pendingBuffer=c?e<n?1:2:0}else this.doc.lineAt(this.pos).from==this.pos&&this.getLine().addLineDeco(r);s&&(this.textOff+s<=this.text.length?this.textOff+=s:(this.skip+=s-(this.text.length-this.textOff),this.text="",this.textOff=0),this.pos=n),this.openStart<0&&(this.openStart=o)}filterPoint(e,n,r,i){if(i>=this.disallowBlockEffectsBelow||!(r instanceof Ki))return!0;if(r.block)throw new RangeError("Block decorations may not be specified via plugins");return n<=this.doc.lineAt(this.pos).to}static build(e,n,r,i,o){let s=new Ym(e,n,r,o);return s.openEnd=Te.spans(i,n,r,s),s.openStart<0&&(s.openStart=s.openEnd),s.finish(s.openEnd),s}}function rf(t,e){for(let n of e)t=new Yi(n,[t],t.length);return t}class Cb extends Cr{constructor(e){super();this.tag=e}eq(e){return e.tag==this.tag}toDOM(){return document.createElement(this.tag)}updateDOM(e){return e.nodeName.toLowerCase()==this.tag}}const J3=[],Rb=ne.define(),Tb=ne.define(),Ib=ne.define(),Eb=ne.define(),Km=ne.define(),Mb=ne.define(),Jm=ce.define({map:(t,e)=>t.map(e)}),Ab=ce.define({map:(t,e)=>t.map(e)});class gs{constructor(e,n="nearest",r="nearest",i=5,o=5){this.range=e,this.y=n,this.x=r,this.yMargin=i,this.xMargin=o}map(e){return e.empty?this:new gs(this.range.map(e),this.y,this.x,this.yMargin,this.xMargin)}}const Db=ce.define({map:(t,e)=>t.map(e)});function bn(t,e,n){let r=t.facet(Eb);r.length?r[0](e):window.onerror?window.onerror(String(e),n,void 0,void 0,e):n?console.error(n+":",e):console.error(e)}const wa=ne.define({combine:t=>t.length?t[0]:!0});class e4{constructor(e,n){this.field=e,this.get=n}}class It{from(e){return new e4(this,e)}static define(){return new It}}It.decorations=It.define();It.atomicRanges=It.define();It.scrollMargins=It.define();let t4=0;const ba=ne.define();class rt{constructor(e,n,r){this.id=e,this.create=n,this.fields=r,this.extension=ba.of(this)}static define(e,n){let{eventHandlers:r,provide:i,decorations:o}=n||{},s=[];if(i)for(let l of Array.isArray(i)?i:[i])s.push(l);return r&&s.push(Lb.from(l=>({plugin:l,handlers:r}))),o&&s.push(It.decorations.from(o)),new rt(t4++,e,s)}static fromClass(e,n){return rt.define(r=>new e(r),n)}}const Lb=It.define();class eg{constructor(e){this.spec=e,this.mustUpdate=null,this.value=null}takeField(e,n){if(this.spec)for(let{field:r,get:i}of this.spec.fields)r==e&&n.push(i(this.value))}update(e){if(this.value){if(this.mustUpdate){let n=this.mustUpdate;if(this.mustUpdate=null,this.value.update)try{this.value.update(n)}catch(r){if(bn(n.state,r,"CodeMirror plugin crashed"),this.value.destroy)try{this.value.destroy()}catch{}this.deactivate()}}}else if(this.spec)try{this.value=this.spec.create(e)}catch(n){bn(e.state,n,"CodeMirror plugin crashed"),this.deactivate()}return this}destroy(e){var n;if((n=this.value)===null||n===void 0?void 0:n.destroy)try{this.value.destroy()}catch(r){bn(e.state,r,"CodeMirror plugin crashed")}}deactivate(){this.spec=this.value=null}}const Wb=ne.define(),Qb=ne.define(),eo=ne.define(),$a=ne.define();class Rr{constructor(e,n,r,i){this.fromA=e,this.toA=n,this.fromB=r,this.toB=i}join(e){return new Rr(Math.min(this.fromA,e.fromA),Math.max(this.toA,e.toA),Math.min(this.fromB,e.fromB),Math.max(this.toB,e.toB))}addToSet(e){let n=e.length,r=this;for(;n>0;n--){let i=e[n-1];if(!(i.fromA>r.toA)){if(i.toA<r.fromA)break;r=r.join(i),e.splice(n-1,1)}}return e.splice(n,0,r),e}static extendWithRanges(e,n){if(n.length==0)return e;let r=[];for(let i=0,o=0,s=0,l=0;;i++){let a=i==e.length?null:e[i],c=s-l,u=a?a.fromB:1e9;for(;o<n.length&&n[o]<u;){let f=n[o],h=n[o+1],d=Math.max(l,f),p=Math.min(u,h);if(d<=p&&new Rr(d+c,p+c,d,p).addToSet(r),h>u)break;o+=2}if(!a)return r;new Rr(a.fromA,a.toA,a.fromB,a.toB).addToSet(r),s=a.toA,l=a.toB}}}class Nb{constructor(e,n,r=J3){this.view=e,this.state=n,this.transactions=r,this.flags=0,this.startState=e.state,this.changes=ct.empty(this.startState.doc.length);for(let s of r)this.changes=this.changes.compose(s.changes);let i=[];this.changes.iterChangedRanges((s,l,a,c)=>i.push(new Rr(s,l,a,c))),this.changedRanges=i;let o=e.hasFocus;o!=e.inputState.notifiedFocused&&(e.inputState.notifiedFocused=o,this.flags|=1)}get viewportChanged(){return(this.flags&4)>0}get heightChanged(){return(this.flags&2)>0}get geometryChanged(){return this.docChanged||(this.flags&(8|2))>0}get focusChanged(){return(this.flags&1)>0}get docChanged(){return!this.changes.empty}get selectionSet(){return this.transactions.some(e=>e.selection)}get empty(){return this.flags==0&&this.transactions.length==0}}var Se=function(t){return t[t.LTR=0]="LTR",t[t.RTL=1]="RTL",t}(Se||(Se={}));const tg=Se.LTR,n4=Se.RTL;function _b(t){let e=[];for(let n=0;n<t.length;n++)e.push(1<<+t[n]);return e}const r4=_b("88888888888888888888888888888888888666888888787833333333337888888000000000000000000000000008888880000000000000000000000000088888888888888888888888888888888888887866668888088888663380888308888800000000000000000000000800000000000000000000000000000008"),i4=_b("4444448826627288999999999992222222222222222222222222222222222222222222222229999999999999999999994444444444644222822222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222999999949999999229989999223333333333"),ng=Object.create(null),er=[];for(let t of["()","[]","{}"]){let e=t.charCodeAt(0),n=t.charCodeAt(1);ng[e]=n,ng[n]=-e}function o4(t){return t<=247?r4[t]:1424<=t&&t<=1524?2:1536<=t&&t<=1785?i4[t-1536]:1774<=t&&t<=2220?4:8192<=t&&t<=8203||t==8204?256:1}const s4=/[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac]/;class ys{constructor(e,n,r){this.from=e,this.to=n,this.level=r}get dir(){return this.level%2?n4:tg}side(e,n){return this.dir==n==e?this.to:this.from}static find(e,n,r,i){let o=-1;for(let s=0;s<e.length;s++){let l=e[s];if(l.from<=n&&l.to>=n){if(l.level==r)return s;(o<0||(i!=0?i<0?l.from<n:l.to>n:e[o].level>l.level))&&(o=s)}}if(o<0)throw new RangeError("Index out of range");return o}}const Ve=[];function l4(t,e){let n=t.length,r=e==tg?1:2,i=e==tg?2:1;if(!t||r==1&&!s4.test(t))return Bb(n);for(let s=0,l=r,a=r;s<n;s++){let c=o4(t.charCodeAt(s));c==512?c=l:c==8&&a==4&&(c=16),Ve[s]=c==4?2:c,c&7&&(a=c),l=c}for(let s=0,l=r,a=r;s<n;s++){let c=Ve[s];if(c==128)s<n-1&&l==Ve[s+1]&&l&24?c=Ve[s]=l:Ve[s]=256;else if(c==64){let u=s+1;for(;u<n&&Ve[u]==64;)u++;let f=s&&l==8||u<n&&Ve[u]==8?a==1?1:8:256;for(let h=s;h<u;h++)Ve[h]=f;s=u-1}else c==8&&a==1&&(Ve[s]=1);l=c,c&7&&(a=c)}for(let s=0,l=0,a=0,c,u,f;s<n;s++)if(u=ng[c=t.charCodeAt(s)])if(u<0){for(let h=l-3;h>=0;h-=3)if(er[h+1]==-u){let d=er[h+2],p=d&2?r:d&4?d&1?i:r:0;p&&(Ve[s]=Ve[er[h]]=p),l=h;break}}else{if(er.length==189)break;er[l++]=s,er[l++]=c,er[l++]=a}else if((f=Ve[s])==2||f==1){let h=f==r;a=h?0:1;for(let d=l-3;d>=0;d-=3){let p=er[d+2];if(p&2)break;if(h)er[d+2]|=2;else{if(p&4)break;er[d+2]|=4}}}for(let s=0;s<n;s++)if(Ve[s]==256){let l=s+1;for(;l<n&&Ve[l]==256;)l++;let a=(s?Ve[s-1]:r)==1,c=(l<n?Ve[l]:r)==1,u=a==c?a?1:2:r;for(let f=s;f<l;f++)Ve[f]=u;s=l-1}let o=[];if(r==1)for(let s=0;s<n;){let l=s,a=Ve[s++]!=1;for(;s<n&&a==(Ve[s]!=1);)s++;if(a)for(let c=s;c>l;){let u=c,f=Ve[--c]!=2;for(;c>l&&f==(Ve[c-1]!=2);)c--;o.push(new ys(c,u,f?2:1))}else o.push(new ys(l,s,0))}else for(let s=0;s<n;){let l=s,a=Ve[s++]==2;for(;s<n&&a==(Ve[s]==2);)s++;o.push(new ys(l,s,a?1:2))}return o}function Bb(t){return[new ys(0,t,0)]}let qb="";function a4(t,e,n,r,i){var o;let s=r.head-t.from,l=-1;if(s==0){if(!i||!t.length)return null;e[0].level!=n&&(s=e[0].side(!1,n),l=0)}else if(s==t.length){if(i)return null;let h=e[e.length-1];h.level!=n&&(s=h.side(!0,n),l=e.length-1)}l<0&&(l=ys.find(e,s,(o=r.bidiLevel)!==null&&o!==void 0?o:-1,r.assoc));let a=e[l];s==a.side(i,n)&&(a=e[l+=i?1:-1],s=a.side(!i,n));let c=i==(a.dir==n),u=nn(t.text,s,c);if(qb=t.text.slice(Math.min(s,u),Math.max(s,u)),u!=a.side(i,n))return _.cursor(u+t.from,c?-1:1,a.level);let f=l==(i?e.length-1:0)?null:e[l+(i?1:-1)];return!f&&a.level!=n?_.cursor(i?t.to:t.from,i?-1:1,n):f&&f.level<a.level?_.cursor(f.side(!i,n)+t.from,i?1:-1,f.level):_.cursor(u+t.from,i?-1:1,a.level)}class zb{constructor(e,n){this.points=e,this.view=n,this.text="",this.lineBreak=n.state.lineBreak}readRange(e,n){if(!e)return this;let r=e.parentNode;for(let i=e;;){this.findPointBefore(r,i),this.readNode(i);let o=i.nextSibling;if(o==n)break;let s=He.get(i),l=He.get(o);(s&&l?s.breakAfter:(s?s.breakAfter:Fb(i))||Fb(o)&&(i.nodeName!="BR"||i.cmIgnore))&&(this.text+=this.lineBreak),i=o}return this.findPointBefore(r,n),this}readNode(e){if(e.cmIgnore)return;let n=He.get(e),r=n&&n.overrideDOMText,i;r!=null?i=r.sliceString(0,void 0,this.lineBreak):e.nodeType==3?i=e.nodeValue:e.nodeName=="BR"?i=e.nextSibling?this.lineBreak:"":e.nodeType==1&&this.readRange(e.firstChild,null),i!=null&&(this.findPointIn(e,i.length),this.text+=i,re.chrome&&this.view.inputState.lastKeyCode==13&&!e.nextSibling&&/\n\n$/.test(this.text)&&(this.text=this.text.slice(0,-1)))}findPointBefore(e,n){for(let r of this.points)r.node==e&&e.childNodes[r.offset]==n&&(r.pos=this.text.length)}findPointIn(e,n){for(let r of this.points)r.node==e&&(r.pos=this.text.length+Math.min(r.offset,n))}}function Fb(t){return t.nodeType==1&&/^(DIV|P|LI|UL|OL|BLOCKQUOTE|DD|DT|H\d|SECTION|PRE)$/.test(t.nodeName)}class Vb{constructor(e,n){this.node=e,this.offset=n,this.pos=-1}}class Ub extends He{constructor(e){super();this.view=e,this.compositionDeco=te.none,this.decorations=[],this.pluginDecorationLength=0,this.minWidth=0,this.minWidthFrom=0,this.minWidthTo=0,this.impreciseAnchor=null,this.impreciseHead=null,this.forceSelection=!1,this.lastUpdate=Date.now(),this.setDOM(e.contentDOM),this.children=[new Wt],this.children[0].setParent(this),this.updateDeco(),this.updateInner([new Rr(0,0,0,e.state.doc.length)],0)}get root(){return this.view.root}get editorView(){return this.view}get length(){return this.view.state.doc.length}update(e){let n=e.changedRanges;this.minWidth>0&&n.length&&(n.every(({fromA:s,toA:l})=>l<this.minWidthFrom||s>this.minWidthTo)?(this.minWidthFrom=e.changes.mapPos(this.minWidthFrom,1),this.minWidthTo=e.changes.mapPos(this.minWidthTo,1)):this.minWidth=this.minWidthFrom=this.minWidthTo=0),this.view.inputState.composing<0?this.compositionDeco=te.none:(e.transactions.length||this.dirty)&&(this.compositionDeco=u4(this.view,e.changes)),(re.ie||re.chrome)&&!this.compositionDeco.size&&e&&e.state.doc.lines!=e.startState.doc.lines&&(this.forceSelection=!0);let r=this.decorations,i=this.updateDeco(),o=p4(r,i,e.changes);return n=Rr.extendWithRanges(n,o),this.dirty==0&&n.length==0?!1:(this.updateInner(n,e.startState.doc.length),e.transactions.length&&(this.lastUpdate=Date.now()),!0)}updateInner(e,n){this.view.viewState.mustMeasureContent=!0,this.updateChildren(e,n);let{observer:r}=this.view;r.ignore(()=>{this.dom.style.height=this.view.viewState.contentHeight+"px",this.dom.style.minWidth=this.minWidth?this.minWidth+"px":"";let o=re.chrome||re.ios?{node:r.selectionRange.focusNode,written:!1}:void 0;this.sync(o),this.dirty=0,o&&(o.written||r.selectionRange.focusNode!=o.node)&&(this.forceSelection=!0),this.dom.style.height=""});let i=[];if(this.view.viewport.from||this.view.viewport.to<this.view.state.doc.length)for(let o of this.children)o instanceof Ji&&o.widget instanceof jb&&i.push(o.dom);r.updateGaps(i)}updateChildren(e,n){let r=this.childCursor(n);for(let i=e.length-1;;i--){let o=i>=0?e[i]:null;if(!o)break;let{fromA:s,toA:l,fromB:a,toB:c}=o,{content:u,breakAtStart:f,openStart:h,openEnd:d}=Ym.build(this.view.state.doc,a,c,this.decorations,this.pluginDecorationLength),{i:p,off:m}=r.findPos(l,1),{i:g,off:y}=r.findPos(s,-1);mb(this,g,y,p,m,u,f,h,d)}}updateSelection(e=!1,n=!1){if(e&&this.view.observer.readSelectionRange(),!(n||this.mayControlSelection())||re.ios&&this.view.inputState.rapidCompositionStart)return;let r=this.forceSelection;this.forceSelection=!1;let i=this.view.state.selection.main,o=this.domAtPos(i.anchor),s=i.empty?o:this.domAtPos(i.head);if(re.gecko&&i.empty&&c4(o)){let a=document.createTextNode("");this.view.observer.ignore(()=>o.node.insertBefore(a,o.node.childNodes[o.offset]||null)),o=s=new Tt(a,0),r=!0}let l=this.view.observer.selectionRange;(r||!l.focusNode||!Ku(o.node,o.offset,l.anchorNode,l.anchorOffset)||!Ku(s.node,s.offset,l.focusNode,l.focusOffset))&&(this.view.observer.ignore(()=>{re.android&&re.chrome&&this.dom.contains(l.focusNode)&&m4(l.focusNode,this.dom)&&(this.dom.blur(),this.dom.focus({preventScroll:!0}));let a=Yu(this.root);if(i.empty){if(re.gecko){let c=h4(o.node,o.offset);if(c&&c!=(1|2)){let u=Xb(o.node,o.offset,c==1?1:-1);u&&(o=new Tt(u,c==1?0:u.nodeValue.length))}}a.collapse(o.node,o.offset),i.bidiLevel!=null&&l.cursorBidiLevel!=null&&(l.cursorBidiLevel=i.bidiLevel)}else if(a.extend)a.collapse(o.node,o.offset),a.extend(s.node,s.offset);else{let c=document.createRange();i.anchor>i.head&&([o,s]=[s,o]),c.setEnd(s.node,s.offset),c.setStart(o.node,o.offset),a.removeAllRanges(),a.addRange(c)}}),this.view.observer.setSelectionRange(o,s)),this.impreciseAnchor=o.precise?null:new Tt(l.anchorNode,l.anchorOffset),this.impreciseHead=s.precise?null:new Tt(l.focusNode,l.focusOffset)}enforceCursorAssoc(){if(this.compositionDeco.size)return;let e=this.view.state.selection.main,n=Yu(this.root);if(!e.empty||!e.assoc||!n.modify)return;let r=Wt.find(this,e.head);if(!r)return;let i=r.posAtStart;if(e.head==i||e.head==i+r.length)return;let o=this.coordsAt(e.head,-1),s=this.coordsAt(e.head,1);if(!o||!s||o.bottom>s.top)return;let l=this.domAtPos(e.head+e.assoc);n.collapse(l.node,l.offset),n.modify("move",e.assoc<0?"forward":"backward","lineboundary")}mayControlSelection(){return this.view.state.facet(wa)?this.root.activeElement==this.dom:_m(this.dom,this.view.observer.selectionRange)}nearest(e){for(let n=e;n;){let r=He.get(n);if(r&&r.rootView==this)return r;n=n.parentNode}return null}posFromDOM(e,n){let r=this.nearest(e);if(!r)throw new RangeError("Trying to find position for a DOM position outside of the document");return r.localPosFromDOM(e,n)+r.posAtStart}domAtPos(e){let{i:n,off:r}=this.childCursor().findPos(e,-1);for(;n<this.children.length-1;){let i=this.children[n];if(r<i.length||i instanceof Wt)break;n++,r=0}return this.children[n].domAtPos(r)}coordsAt(e,n){for(let r=this.length,i=this.children.length-1;;i--){let o=this.children[i],s=r-o.breakAfter-o.length;if(e>s||e==s&&o.type!=De.WidgetBefore&&o.type!=De.WidgetAfter&&(!i||n==2||this.children[i-1].breakAfter||this.children[i-1].type==De.WidgetBefore&&n>-2))return o.coordsAt(e-s,n);r=s}}measureVisibleLineHeights(){let e=[],{from:n,to:r}=this.view.viewState.viewport,i=this.view.contentDOM.clientWidth,o=i>Math.max(this.view.scrollDOM.clientWidth,this.minWidth)+1,s=-1;for(let l=0,a=0;a<this.children.length;a++){let c=this.children[a],u=l+c.length;if(u>r)break;if(l>=n){let f=c.dom.getBoundingClientRect();if(e.push(f.height),o){let h=c.dom.lastChild,d=h?ds(h):[];if(d.length){let p=d[d.length-1],m=this.view.textDirection==Se.LTR?p.right-f.left:f.right-p.left;m>s&&(s=m,this.minWidth=i,this.minWidthFrom=l,this.minWidthTo=u)}}}l=u+c.breakAfter}return e}measureTextSize(){for(let i of this.children)if(i instanceof Wt){let o=i.measureTextSize();if(o)return o}let e=document.createElement("div"),n,r;return e.className="cm-line",e.textContent="abc def ghi jkl mno pqr stu",this.view.observer.ignore(()=>{this.dom.appendChild(e);let i=ds(e.firstChild)[0];n=e.getBoundingClientRect().height,r=i?i.width/27:7,e.remove()}),{lineHeight:n,charWidth:r}}childCursor(e=this.length){let n=this.children.length;return n&&(e-=this.children[--n].length),new pb(this.children,e,n)}computeBlockGapDeco(){let e=[],n=this.view.viewState;for(let r=0,i=0;;i++){let o=i==n.viewports.length?null:n.viewports[i],s=o?o.from-1:this.length;if(s>r){let l=n.lineBlockAt(s).bottom-n.lineBlockAt(r).top;e.push(te.replace({widget:new jb(l),block:!0,inclusive:!0}).range(r,s))}if(!o)break;r=o.to+1}return te.set(e)}updateDeco(){let e=this.view.pluginField(It.decorations);return this.pluginDecorationLength=e.length,this.decorations=[...e,...this.view.state.facet(eo),this.compositionDeco,this.computeBlockGapDeco(),this.view.viewState.lineGapDeco]}scrollIntoView(e){let{range:n}=e,r=this.coordsAt(n.head,n.empty?n.assoc:n.head>n.anchor?-1:1),i;if(!r)return;!n.empty&&(i=this.coordsAt(n.anchor,n.anchor>n.head?-1:1))&&(r={left:Math.min(r.left,i.left),top:Math.min(r.top,i.top),right:Math.max(r.right,i.right),bottom:Math.max(r.bottom,i.bottom)});let o=0,s=0,l=0,a=0;for(let u of this.view.pluginField(It.scrollMargins))if(u){let{left:f,right:h,top:d,bottom:p}=u;f!=null&&(o=Math.max(o,f)),h!=null&&(s=Math.max(s,h)),d!=null&&(l=Math.max(l,d)),p!=null&&(a=Math.max(a,p))}let c={left:r.left-o,top:r.top-l,right:r.right+s,bottom:r.bottom+a};X3(this.view.scrollDOM,c,n.head<n.anchor?-1:1,e.x,e.y,e.xMargin,e.yMargin,this.view.textDirection==Se.LTR)}}function c4(t){return t.node.nodeType==1&&t.node.firstChild&&(t.offset==0||t.node.childNodes[t.offset-1].contentEditable=="false")&&(t.offset==t.node.childNodes.length||t.node.childNodes[t.offset].contentEditable=="false")}class jb extends Cr{constructor(e){super();this.height=e}toDOM(){let e=document.createElement("div");return this.updateDOM(e),e}eq(e){return e.height==this.height}updateDOM(e){return e.style.height=this.height+"px",!0}get estimatedHeight(){return this.height}}function u4(t,e){let n=t.observer.selectionRange,r=n.focusNode&&Xb(n.focusNode,n.focusOffset,0);if(!r)return te.none;let i=t.docView.nearest(r);if(!i)return te.none;let o,s,l=r;if(i instanceof Wt){for(;l.parentNode!=i.dom;)l=l.parentNode;let h=l.previousSibling;for(;h&&!He.get(h);)h=h.previousSibling;o=s=h?He.get(h).posAtEnd:i.posAtStart}else{for(;;){let{parent:h}=i;if(!h)return te.none;if(h instanceof Wt)break;i=h}o=i.posAtStart,s=o+i.length,l=i.dom}let a=e.mapPos(o,1),c=Math.max(a,e.mapPos(s,-1)),{state:u}=t,f=l.nodeType==3?l.nodeValue:new zb([],t).readRange(l.firstChild,null).text;if(c-a<f.length)if(u.sliceDoc(a,Math.min(u.doc.length,a+f.length))==f)c=a+f.length;else if(u.sliceDoc(Math.max(0,c-f.length),c)==f)a=c-f.length;else return te.none;else if(u.sliceDoc(a,c)!=f)return te.none;return te.set(te.replace({widget:new f4(l,r)}).range(a,c))}class f4 extends Cr{constructor(e,n){super();this.top=e,this.text=n}eq(e){return this.top==e.top&&this.text==e.text}toDOM(){return this.top}ignoreEvent(){return!1}get customView(){return Y3}}function Xb(t,e,n){for(;;){if(t.nodeType==3)return t;if(t.nodeType==1&&e>0&&n<=0)t=t.childNodes[e-1],e=Ju(t);else if(t.nodeType==1&&e<t.childNodes.length&&n>=0)t=t.childNodes[e],e=0;else return null}}function h4(t,e){return t.nodeType!=1?0:(e&&t.childNodes[e-1].contentEditable=="false"?1:0)|(e<t.childNodes.length&&t.childNodes[e].contentEditable=="false"?2:0)}class d4{constructor(){this.changes=[]}compareRange(e,n){Zm(e,n,this.changes)}comparePoint(e,n){Zm(e,n,this.changes)}}function p4(t,e,n){let r=new d4;return Te.compare(t,e,n,r),r.changes}function m4(t,e){for(let n=t;n&&n!=e;n=n.assignedSlot||n.parentNode)if(n.nodeType==1&&n.contentEditable=="false")return!0;return!1}function g4(t,e,n=1){let r=t.charCategorizer(e),i=t.doc.lineAt(e),o=e-i.from;if(i.length==0)return _.cursor(e);o==0?n=1:o==i.length&&(n=-1);let s=o,l=o;n<0?s=nn(i.text,o,!1):l=nn(i.text,o);let a=r(i.text.slice(s,l));for(;s>0;){let c=nn(i.text,s,!1);if(r(i.text.slice(c,s))!=a)break;s=c}for(;l<i.length;){let c=nn(i.text,l);if(r(i.text.slice(l,c))!=a)break;l=c}return _.range(s+i.from,l+i.from)}function y4(t,e){return e.left>t?e.left-t:Math.max(0,t-e.right)}function O4(t,e){return e.top>t?e.top-t:Math.max(0,t-e.bottom)}function rg(t,e){return t.top<e.bottom-1&&t.bottom>e.top+1}function Hb(t,e){return e<t.top?{top:e,left:t.left,right:t.right,bottom:t.bottom}:t}function Gb(t,e){return e>t.bottom?{top:t.top,left:t.left,right:t.right,bottom:e}:t}function ig(t,e,n){let r,i,o,s,l,a,c,u;for(let d=t.firstChild;d;d=d.nextSibling){let p=ds(d);for(let m=0;m<p.length;m++){let g=p[m];i&&rg(i,g)&&(g=Hb(Gb(g,i.bottom),i.top));let y=y4(e,g),O=O4(n,g);if(y==0&&O==0)return d.nodeType==3?Zb(d,e,n):ig(d,e,n);(!r||s>O||s==O&&o>y)&&(r=d,i=g,o=y,s=O),y==0?n>g.bottom&&(!c||c.bottom<g.bottom)?(l=d,c=g):n<g.top&&(!u||u.top>g.top)&&(a=d,u=g):c&&rg(c,g)?c=Gb(c,g.bottom):u&&rg(u,g)&&(u=Hb(u,g.top))}}if(c&&c.bottom>=n?(r=l,i=c):u&&u.top<=n&&(r=a,i=u),!r)return{node:t,offset:0};let f=Math.max(i.left,Math.min(i.right,e));if(r.nodeType==3)return Zb(r,f,n);if(!o&&r.contentEditable=="true")return ig(r,f,n);let h=Array.prototype.indexOf.call(t.childNodes,r)+(e>=(i.left+i.right)/2?1:0);return{node:t,offset:h}}function Zb(t,e,n){let r=t.nodeValue.length,i=-1,o=1e9,s=0;for(let l=0;l<r;l++){let a=Oa(t,l,l+1).getClientRects();for(let c=0;c<a.length;c++){let u=a[c];if(u.top==u.bottom)continue;s||(s=e-u.left);let f=(u.top>n?u.top-n:n-u.bottom)-1;if(u.left-1<=e&&u.right+1>=e&&f<o){let h=e>=(u.left+u.right)/2,d=h;if((re.chrome||re.gecko)&&Oa(t,l).getBoundingClientRect().left==u.right&&(d=!h),f<=0)return{node:t,offset:l+(d?1:0)};i=l+(d?1:0),o=f}}}return{node:t,offset:i>-1?i:s>0?t.nodeValue.length:0}}function Yb(t,{x:e,y:n},r,i=-1){var o;let s=t.contentDOM.getBoundingClientRect(),l=s.top+t.viewState.paddingTop,a,{docHeight:c}=t.viewState,u=n-l;if(u<0)return 0;if(u>c)return t.state.doc.length;for(let y=t.defaultLineHeight/2,O=!1;a=t.elementAtHeight(u),a.type!=De.Text;)for(;u=i>0?a.bottom+y:a.top-y,!(u>=0&&u<=c);){if(O)return r?null:0;O=!0,i=-i}n=l+u;let f=a.from;if(f<t.viewport.from)return t.viewport.from==0?0:r?null:Kb(t,s,a,e,n);if(f>t.viewport.to)return t.viewport.to==t.state.doc.length?t.state.doc.length:r?null:Kb(t,s,a,e,n);let h=t.dom.ownerDocument,d=t.root.elementFromPoint?t.root:h,p=d.elementFromPoint(e,n);p&&!t.contentDOM.contains(p)&&(p=null),p||(e=Math.max(s.left+1,Math.min(s.right-1,e)),p=d.elementFromPoint(e,n),p&&!t.contentDOM.contains(p)&&(p=null));let m,g=-1;if(p&&((o=t.docView.nearest(p))===null||o===void 0?void 0:o.isEditable)!=!1){if(h.caretPositionFromPoint){let y=h.caretPositionFromPoint(e,n);y&&({offsetNode:m,offset:g}=y)}else if(h.caretRangeFromPoint){let y=h.caretRangeFromPoint(e,n);y&&({startContainer:m,startOffset:g}=y,re.safari&&v4(m,g,e)&&(m=void 0))}}if(!m||!t.docView.dom.contains(m)){let y=Wt.find(t.docView,f);if(!y)return u>a.top+a.height/2?a.to:a.from;({node:m,offset:g}=ig(y.dom,e,n))}return t.docView.posFromDOM(m,g)}function Kb(t,e,n,r,i){let o=Math.round((r-e.left)*t.defaultCharacterWidth);t.lineWrapping&&n.height>t.defaultLineHeight*1.5&&(o+=Math.floor((i-n.top)/t.defaultLineHeight)*t.viewState.heightOracle.lineLength);let s=t.state.sliceDoc(n.from,n.to);return n.from+bm(s,o,t.state.tabSize)}function v4(t,e,n){let r;if(t.nodeType!=3||e!=(r=t.nodeValue.length))return!1;for(let i=t.nextSibling;i;i=i.nextSibling)if(i.nodeType!=1||i.nodeName!="BR")return!1;return Oa(t,r-1,r).getBoundingClientRect().left>n}function x4(t,e,n,r){let i=t.state.doc.lineAt(e.head),o=!r||!t.lineWrapping?null:t.coordsAtPos(e.assoc<0&&e.head>i.from?e.head-1:e.head);if(o){let a=t.dom.getBoundingClientRect(),c=t.posAtCoords({x:n==(t.textDirection==Se.LTR)?a.right-1:a.left+1,y:(o.top+o.bottom)/2});if(c!=null)return _.cursor(c,n?-1:1)}let s=Wt.find(t.docView,e.head),l=s?n?s.posAtEnd:s.posAtStart:n?i.to:i.from;return _.cursor(l,n?-1:1)}function Jb(t,e,n,r){let i=t.state.doc.lineAt(e.head),o=t.bidiSpans(i);for(let s=e,l=null;;){let a=a4(i,o,t.textDirection,s,n),c=qb;if(!a){if(i.number==(n?t.state.doc.lines:1))return s;c=`
`,i=t.state.doc.line(i.number+(n?1:-1)),o=t.bidiSpans(i),a=_.cursor(n?i.from:i.to)}if(l){if(!l(c))return s}else{if(!r)return a;l=r(c)}s=a}}function w4(t,e,n){let r=t.state.charCategorizer(e),i=r(n);return o=>{let s=r(o);return i==rn.Space&&(i=s),i==s}}function b4(t,e,n,r){let i=e.head,o=n?1:-1;if(i==(n?t.state.doc.length:0))return _.cursor(i);let s=e.goalColumn,l,a=t.contentDOM.getBoundingClientRect(),c=t.coordsAtPos(i),u=t.documentTop;if(c)s==null&&(s=c.left-a.left),l=o<0?c.top:c.bottom;else{let d=t.viewState.lineBlockAt(i-u);s==null&&(s=Math.min(a.right-a.left,t.defaultCharacterWidth*(i-d.from))),l=(o<0?d.top:d.bottom)+u}let f=a.left+s,h=r!=null?r:t.defaultLineHeight>>1;for(let d=0;;d+=10){let p=l+(h+d)*o,m=Yb(t,{x:f,y:p},!1,o);if(p<a.top||p>a.bottom||(o<0?m<i:m>i))return _.cursor(m,void 0,void 0,s)}}function og(t,e,n){let r=t.pluginField(It.atomicRanges);for(;;){let i=!1;for(let o of r)o.between(n.from-1,n.from+1,(s,l,a)=>{n.from>s&&n.from<l&&(n=e.from>n.from?_.cursor(s,1):_.cursor(l,-1),i=!0)});if(!i)return n}}class $4{constructor(e){this.lastKeyCode=0,this.lastKeyTime=0,this.pendingIOSKey=void 0,this.lastSelectionOrigin=null,this.lastSelectionTime=0,this.lastEscPress=0,this.lastContextMenu=0,this.scrollHandlers=[],this.registeredEvents=[],this.customHandlers=[],this.composing=-1,this.compositionFirstChange=null,this.compositionEndedAt=0,this.rapidCompositionStart=!1,this.mouseSelection=null;for(let n in it){let r=it[n];e.contentDOM.addEventListener(n,i=>{n=="keydown"&&this.keydown(e,i)||!n$(e,i)||this.ignoreDuringComposition(i)||(this.mustFlushObserver(i)&&e.observer.forceFlush(),this.runCustomHandlers(n,e,i)?i.preventDefault():r(e,i))}),this.registeredEvents.push(n)}this.notifiedFocused=e.hasFocus,this.ensureHandlers(e),re.safari&&e.contentDOM.addEventListener("input",()=>null)}setSelectionOrigin(e){this.lastSelectionOrigin=e,this.lastSelectionTime=Date.now()}ensureHandlers(e){let n=this.customHandlers=e.pluginField(Lb);for(let r of n)for(let i in r.handlers)this.registeredEvents.indexOf(i)<0&&i!="scroll"&&(this.registeredEvents.push(i),e.contentDOM.addEventListener(i,o=>{!n$(e,o)||this.runCustomHandlers(i,e,o)&&o.preventDefault()}))}runCustomHandlers(e,n,r){for(let i of this.customHandlers){let o=i.handlers[e];if(o)try{if(o.call(i.plugin,r,n)||r.defaultPrevented)return!0}catch(s){bn(n.state,s)}}return!1}runScrollHandlers(e,n){for(let r of this.customHandlers){let i=r.handlers.scroll;if(i)try{i.call(r.plugin,n,e)}catch(o){bn(e.state,o)}}}keydown(e,n){if(this.lastKeyCode=n.keyCode,this.lastKeyTime=Date.now(),this.screenKeyEvent(e,n))return!0;if(re.android&&re.chrome&&!n.synthetic&&(n.keyCode==13||n.keyCode==8))return e.observer.delayAndroidKey(n.key,n.keyCode),!0;let r;return re.ios&&(r=e$.find(i=>i.keyCode==n.keyCode))&&!(n.ctrlKey||n.altKey||n.metaKey)&&!n.synthetic?(this.pendingIOSKey=r,setTimeout(()=>this.flushIOSKey(e),250),!0):!1}flushIOSKey(e){let n=this.pendingIOSKey;return n?(this.pendingIOSKey=void 0,va(e.contentDOM,n.key,n.keyCode)):!1}ignoreDuringComposition(e){return/^key/.test(e.type)?this.composing>0?!0:re.safari&&Date.now()-this.compositionEndedAt<500?(this.compositionEndedAt=0,!0):!1:!1}screenKeyEvent(e,n){let r=n.keyCode==9&&Date.now()<this.lastEscPress+2e3;return n.keyCode==27?this.lastEscPress=Date.now():t$.indexOf(n.keyCode)<0&&(this.lastEscPress=0),r}mustFlushObserver(e){return e.type=="keydown"&&e.keyCode!=229||e.type=="compositionend"&&!re.ios}startMouseSelection(e){this.mouseSelection&&this.mouseSelection.destroy(),this.mouseSelection=e}update(e){this.mouseSelection&&this.mouseSelection.update(e),e.transactions.length&&(this.lastKeyCode=this.lastSelectionTime=0)}destroy(){this.mouseSelection&&this.mouseSelection.destroy()}}const e$=[{key:"Backspace",keyCode:8,inputType:"deleteContentBackward"},{key:"Enter",keyCode:13,inputType:"insertParagraph"},{key:"Delete",keyCode:46,inputType:"deleteContentForward"}],t$=[16,17,18,20,91,92,224,225];class S4{constructor(e,n,r,i){this.view=e,this.style=r,this.mustSelect=i,this.lastEvent=n;let o=e.contentDOM.ownerDocument;o.addEventListener("mousemove",this.move=this.move.bind(this)),o.addEventListener("mouseup",this.up=this.up.bind(this)),this.extend=n.shiftKey,this.multiple=e.state.facet(Pe.allowMultipleSelections)&&k4(e,n),this.dragMove=P4(e,n),this.dragging=C4(e,n)&&sg(n)==1?null:!1,this.dragging===!1&&(n.preventDefault(),this.select(n))}move(e){if(e.buttons==0)return this.destroy();this.dragging===!1&&this.select(this.lastEvent=e)}up(e){this.dragging==null&&this.select(this.lastEvent),this.dragging||e.preventDefault(),this.destroy()}destroy(){let e=this.view.contentDOM.ownerDocument;e.removeEventListener("mousemove",this.move),e.removeEventListener("mouseup",this.up),this.view.inputState.mouseSelection=null}select(e){let n=this.style.get(e,this.extend,this.multiple);(this.mustSelect||!n.eq(this.view.state.selection)||n.main.assoc!=this.view.state.selection.main.assoc)&&this.view.dispatch({selection:n,userEvent:"select.pointer",scrollIntoView:!0}),this.mustSelect=!1}update(e){e.docChanged&&this.dragging&&(this.dragging=this.dragging.map(e.changes)),this.style.update(e)&&setTimeout(()=>this.select(this.lastEvent),20)}}function k4(t,e){let n=t.state.facet(Rb);return n.length?n[0](e):re.mac?e.metaKey:e.ctrlKey}function P4(t,e){let n=t.state.facet(Tb);return n.length?n[0](e):re.mac?!e.altKey:!e.ctrlKey}function C4(t,e){let{main:n}=t.state.selection;if(n.empty)return!1;let r=Yu(t.root);if(r.rangeCount==0)return!0;let i=r.getRangeAt(0).getClientRects();for(let o=0;o<i.length;o++){let s=i[o];if(s.left<=e.clientX&&s.right>=e.clientX&&s.top<=e.clientY&&s.bottom>=e.clientY)return!0}return!1}function n$(t,e){if(!e.bubbles)return!0;if(e.defaultPrevented)return!1;for(let n=e.target,r;n!=t.contentDOM;n=n.parentNode)if(!n||n.nodeType==11||(r=He.get(n))&&r.ignoreEvent(e))return!1;return!0}const it=Object.create(null),r$=re.ie&&re.ie_version<15||re.ios&&re.webkit_version<604;function R4(t){let e=t.dom.parentNode;if(!e)return;let n=e.appendChild(document.createElement("textarea"));n.style.cssText="position: fixed; left: -10000px; top: 10px",n.focus(),setTimeout(()=>{t.focus(),n.remove(),i$(t,n.value)},50)}function i$(t,e){let{state:n}=t,r,i=1,o=n.toText(e),s=o.lines==n.selection.ranges.length;if(lg!=null&&n.selection.ranges.every(a=>a.empty)&&lg==o.toString()){let a=-1;r=n.changeByRange(c=>{let u=n.doc.lineAt(c.from);if(u.from==a)return{range:c};a=u.from;let f=n.toText((s?o.line(i++).text:e)+n.lineBreak);return{changes:{from:u.from,insert:f},range:_.cursor(c.from+f.length)}})}else s?r=n.changeByRange(a=>{let c=o.line(i++);return{changes:{from:a.from,to:a.to,insert:c.text},range:_.cursor(a.from+c.length)}}):r=n.replaceSelection(o);t.dispatch(r,{userEvent:"input.paste",scrollIntoView:!0})}it.keydown=(t,e)=>{t.inputState.setSelectionOrigin("select")};let o$=0;it.touchstart=(t,e)=>{o$=Date.now(),t.inputState.setSelectionOrigin("select.pointer")};it.touchmove=t=>{t.inputState.setSelectionOrigin("select.pointer")};it.mousedown=(t,e)=>{if(t.observer.flush(),o$>Date.now()-2e3&&sg(e)==1)return;let n=null;for(let r of t.state.facet(Ib))if(n=r(t,e),n)break;if(!n&&e.button==0&&(n=E4(t,e)),n){let r=t.root.activeElement!=t.contentDOM;r&&t.observer.ignore(()=>ub(t.contentDOM)),t.inputState.startMouseSelection(new S4(t,e,n,r))}};function s$(t,e,n,r){if(r==1)return _.cursor(e,n);if(r==2)return g4(t.state,e,n);{let i=Wt.find(t.docView,e),o=t.state.doc.lineAt(i?i.posAtEnd:e),s=i?i.posAtStart:o.from,l=i?i.posAtEnd:o.to;return l<t.state.doc.length&&l==o.to&&l++,_.range(s,l)}}let l$=(t,e)=>t>=e.top&&t<=e.bottom,a$=(t,e,n)=>l$(e,n)&&t>=n.left&&t<=n.right;function T4(t,e,n,r){let i=Wt.find(t.docView,e);if(!i)return 1;let o=e-i.posAtStart;if(o==0)return 1;if(o==i.length)return-1;let s=i.coordsAt(o,-1);if(s&&a$(n,r,s))return-1;let l=i.coordsAt(o,1);return l&&a$(n,r,l)?1:s&&l$(r,s)?-1:1}function c$(t,e){let n=t.posAtCoords({x:e.clientX,y:e.clientY},!1);return{pos:n,bias:T4(t,n,e.clientX,e.clientY)}}const I4=re.ie&&re.ie_version<=11;let u$=null,f$=0,h$=0;function sg(t){if(!I4)return t.detail;let e=u$,n=h$;return u$=t,h$=Date.now(),f$=!e||n>Date.now()-400&&Math.abs(e.clientX-t.clientX)<2&&Math.abs(e.clientY-t.clientY)<2?(f$+1)%3:1}function E4(t,e){let n=c$(t,e),r=sg(e),i=t.state.selection,o=n,s=e;return{update(l){l.docChanged&&(n&&(n.pos=l.changes.mapPos(n.pos)),i=i.map(l.changes),s=null)},get(l,a,c){let u;if(s&&l.clientX==s.clientX&&l.clientY==s.clientY?u=o:(u=o=c$(t,l),s=l),!u||!n)return i;let f=s$(t,u.pos,u.bias,r);if(n.pos!=u.pos&&!a){let h=s$(t,n.pos,n.bias,r),d=Math.min(h.from,f.from),p=Math.max(h.to,f.to);f=d<f.from?_.range(d,p):_.range(p,d)}return a?i.replaceRange(i.main.extend(f.from,f.to)):c?i.addRange(f):_.create([f])}}}it.dragstart=(t,e)=>{let{selection:{main:n}}=t.state,{mouseSelection:r}=t.inputState;r&&(r.dragging=n),e.dataTransfer&&(e.dataTransfer.setData("Text",t.state.sliceDoc(n.from,n.to)),e.dataTransfer.effectAllowed="copyMove")};function d$(t,e,n,r){if(!n)return;let i=t.posAtCoords({x:e.clientX,y:e.clientY},!1);e.preventDefault();let{mouseSelection:o}=t.inputState,s=r&&o&&o.dragging&&o.dragMove?{from:o.dragging.from,to:o.dragging.to}:null,l={from:i,insert:n},a=t.state.changes(s?[s,l]:l);t.focus(),t.dispatch({changes:a,selection:{anchor:a.mapPos(i,-1),head:a.mapPos(i,1)},userEvent:s?"move.drop":"input.drop"})}it.drop=(t,e)=>{if(!e.dataTransfer)return;if(t.state.readOnly)return e.preventDefault();let n=e.dataTransfer.files;if(n&&n.length){e.preventDefault();let r=Array(n.length),i=0,o=()=>{++i==n.length&&d$(t,e,r.filter(s=>s!=null).join(t.state.lineBreak),!1)};for(let s=0;s<n.length;s++){let l=new FileReader;l.onerror=o,l.onload=()=>{/[\x00-\x08\x0e-\x1f]{2}/.test(l.result)||(r[s]=l.result),o()},l.readAsText(n[s])}}else d$(t,e,e.dataTransfer.getData("Text"),!0)};it.paste=(t,e)=>{if(t.state.readOnly)return e.preventDefault();t.observer.flush();let n=r$?null:e.clipboardData;n?(i$(t,n.getData("text/plain")),e.preventDefault()):R4(t)};function M4(t,e){let n=t.dom.parentNode;if(!n)return;let r=n.appendChild(document.createElement("textarea"));r.style.cssText="position: fixed; left: -10000px; top: 10px",r.value=e,r.focus(),r.selectionEnd=e.length,r.selectionStart=0,setTimeout(()=>{r.remove(),t.focus()},50)}function A4(t){let e=[],n=[],r=!1;for(let i of t.selection.ranges)i.empty||(e.push(t.sliceDoc(i.from,i.to)),n.push(i));if(!e.length){let i=-1;for(let{from:o}of t.selection.ranges){let s=t.doc.lineAt(o);s.number>i&&(e.push(s.text),n.push({from:s.from,to:Math.min(t.doc.length,s.to+1)})),i=s.number}r=!0}return{text:e.join(t.lineBreak),ranges:n,linewise:r}}let lg=null;it.copy=it.cut=(t,e)=>{let{text:n,ranges:r,linewise:i}=A4(t.state);if(!n&&!i)return;lg=i?n:null;let o=r$?null:e.clipboardData;o?(e.preventDefault(),o.clearData(),o.setData("text/plain",n)):M4(t,n),e.type=="cut"&&!t.state.readOnly&&t.dispatch({changes:r,scrollIntoView:!0,userEvent:"delete.cut"})};it.focus=it.blur=t=>{setTimeout(()=>{t.hasFocus!=t.inputState.notifiedFocused&&t.update([])},10)};it.beforeprint=t=>{t.viewState.printing=!0,t.requestMeasure(),setTimeout(()=>{t.viewState.printing=!1,t.requestMeasure()},2e3)};function p$(t,e){if(t.docView.compositionDeco.size){t.inputState.rapidCompositionStart=e;try{t.update([])}finally{t.inputState.rapidCompositionStart=!1}}}it.compositionstart=it.compositionupdate=t=>{t.inputState.compositionFirstChange==null&&(t.inputState.compositionFirstChange=!0),t.inputState.composing<0&&(t.inputState.composing=0,t.docView.compositionDeco.size&&(t.observer.flush(),p$(t,!0)))};it.compositionend=t=>{t.inputState.composing=-1,t.inputState.compositionEndedAt=Date.now(),t.inputState.compositionFirstChange=null,setTimeout(()=>{t.inputState.composing<0&&p$(t,!1)},50)};it.contextmenu=t=>{t.inputState.lastContextMenu=Date.now()};it.beforeinput=(t,e)=>{var n;let r;if(re.chrome&&re.android&&(r=e$.find(i=>i.inputType==e.inputType))&&(t.observer.delayAndroidKey(r.key,r.keyCode),r.key=="Backspace"||r.key=="Delete")){let i=((n=window.visualViewport)===null||n===void 0?void 0:n.height)||0;setTimeout(()=>{var o;(((o=window.visualViewport)===null||o===void 0?void 0:o.height)||0)>i+10&&t.hasFocus&&(t.contentDOM.blur(),t.focus())},100)}};const m$=["pre-wrap","normal","pre-line","break-spaces"];class D4{constructor(){this.doc=Ie.empty,this.lineWrapping=!1,this.direction=Se.LTR,this.heightSamples={},this.lineHeight=14,this.charWidth=7,this.lineLength=30,this.heightChanged=!1}heightForGap(e,n){let r=this.doc.lineAt(n).number-this.doc.lineAt(e).number+1;return this.lineWrapping&&(r+=Math.ceil((n-e-r*this.lineLength*.5)/this.lineLength)),this.lineHeight*r}heightForLine(e){return this.lineWrapping?(1+Math.max(0,Math.ceil((e-this.lineLength)/(this.lineLength-5))))*this.lineHeight:this.lineHeight}setDoc(e){return this.doc=e,this}mustRefreshForStyle(e,n){return m$.indexOf(e)>-1!=this.lineWrapping||this.direction!=n}mustRefreshForHeights(e){let n=!1;for(let r=0;r<e.length;r++){let i=e[r];i<0?r++:this.heightSamples[Math.floor(i*10)]||(n=!0,this.heightSamples[Math.floor(i*10)]=!0)}return n}refresh(e,n,r,i,o,s){let l=m$.indexOf(e)>-1,a=Math.round(r)!=Math.round(this.lineHeight)||this.lineWrapping!=l||this.direction!=n;if(this.lineWrapping=l,this.direction=n,this.lineHeight=r,this.charWidth=i,this.lineLength=o,a){this.heightSamples={};for(let c=0;c<s.length;c++){let u=s[c];u<0?c++:this.heightSamples[Math.floor(u*10)]=!0}}return a}}class L4{constructor(e,n){this.from=e,this.heights=n,this.index=0}get more(){return this.index<this.heights.length}}class Tr{constructor(e,n,r,i,o){this.from=e,this.length=n,this.top=r,this.height=i,this.type=o}get to(){return this.from+this.length}get bottom(){return this.top+this.height}join(e){let n=(Array.isArray(this.type)?this.type:[this]).concat(Array.isArray(e.type)?e.type:[e]);return new Tr(this.from,this.length+e.length,this.top,this.height+e.height,n)}moveY(e){return e?new Tr(this.from,this.length,this.top+e,this.height,Array.isArray(this.type)?this.type.map(n=>n.moveY(e)):this.type):this}}var _e=function(t){return t[t.ByPos=0]="ByPos",t[t.ByHeight=1]="ByHeight",t[t.ByPosNoHeight=2]="ByPosNoHeight",t}(_e||(_e={}));const of=.001;class Ut{constructor(e,n,r=2){this.length=e,this.height=n,this.flags=r}get outdated(){return(this.flags&2)>0}set outdated(e){this.flags=(e?2:0)|this.flags&~2}setHeight(e,n){this.height!=n&&(Math.abs(this.height-n)>of&&(e.heightChanged=!0),this.height=n)}replace(e,n,r){return Ut.of(r)}decomposeLeft(e,n){n.push(this)}decomposeRight(e,n){n.push(this)}applyChanges(e,n,r,i){let o=this;for(let s=i.length-1;s>=0;s--){let{fromA:l,toA:a,fromB:c,toB:u}=i[s],f=o.lineAt(l,_e.ByPosNoHeight,n,0,0),h=f.to>=a?f:o.lineAt(a,_e.ByPosNoHeight,n,0,0);for(u+=h.to-a,a=h.to;s>0&&f.from<=i[s-1].toA;)l=i[s-1].fromA,c=i[s-1].fromB,s--,l<f.from&&(f=o.lineAt(l,_e.ByPosNoHeight,n,0,0));c+=f.from-l,l=f.from;let d=ag.build(r,e,c,u);o=o.replace(l,a,d)}return o.updateHeight(r,0)}static empty(){return new sn(0,0)}static of(e){if(e.length==1)return e[0];let n=0,r=e.length,i=0,o=0;for(;;)if(n==r)if(i>o*2){let l=e[n-1];l.break?e.splice(--n,1,l.left,null,l.right):e.splice(--n,1,l.left,l.right),r+=1+l.break,i-=l.size}else if(o>i*2){let l=e[r];l.break?e.splice(r,1,l.left,null,l.right):e.splice(r,1,l.left,l.right),r+=2+l.break,o-=l.size}else break;else if(i<o){let l=e[n++];l&&(i+=l.size)}else{let l=e[--r];l&&(o+=l.size)}let s=0;return e[n-1]==null?(s=1,n--):e[n]==null&&(s=1,r++),new W4(Ut.of(e.slice(0,n)),s,Ut.of(e.slice(r)))}}Ut.prototype.size=1;class g$ extends Ut{constructor(e,n,r){super(e,n);this.type=r}blockAt(e,n,r,i){return new Tr(i,this.length,r,this.height,this.type)}lineAt(e,n,r,i,o){return this.blockAt(0,r,i,o)}forEachLine(e,n,r,i,o,s){s(this.blockAt(0,r,i,o))}updateHeight(e,n=0,r=!1,i){return i&&i.from<=n&&i.more&&this.setHeight(e,i.heights[i.index++]),this.outdated=!1,this}toString(){return`block(${this.length})`}}class sn extends g${constructor(e,n){super(e,n,De.Text);this.collapsed=0,this.widgetHeight=0}replace(e,n,r){let i=r[0];return r.length==1&&(i instanceof sn||i instanceof yt&&i.flags&4)&&Math.abs(this.length-i.length)<10?(i instanceof yt?i=new sn(i.length,this.height):i.height=this.height,this.outdated||(i.outdated=!1),i):Ut.of(r)}updateHeight(e,n=0,r=!1,i){return i&&i.from<=n&&i.more?this.setHeight(e,i.heights[i.index++]):(r||this.outdated)&&this.setHeight(e,Math.max(this.widgetHeight,e.heightForLine(this.length-this.collapsed))),this.outdated=!1,this}toString(){return`line(${this.length}${this.collapsed?-this.collapsed:""}${this.widgetHeight?":"+this.widgetHeight:""})`}}class yt extends Ut{constructor(e){super(e,0)}lines(e,n){let r=e.lineAt(n).number,i=e.lineAt(n+this.length).number;return{firstLine:r,lastLine:i,lineHeight:this.height/(i-r+1)}}blockAt(e,n,r,i){let{firstLine:o,lastLine:s,lineHeight:l}=this.lines(n,i),a=Math.max(0,Math.min(s-o,Math.floor((e-r)/l))),{from:c,length:u}=n.line(o+a);return new Tr(c,u,r+l*a,l,De.Text)}lineAt(e,n,r,i,o){if(n==_e.ByHeight)return this.blockAt(e,r,i,o);if(n==_e.ByPosNoHeight){let{from:f,to:h}=r.lineAt(e);return new Tr(f,h-f,0,0,De.Text)}let{firstLine:s,lineHeight:l}=this.lines(r,o),{from:a,length:c,number:u}=r.lineAt(e);return new Tr(a,c,i+l*(u-s),l,De.Text)}forEachLine(e,n,r,i,o,s){let{firstLine:l,lineHeight:a}=this.lines(r,o);for(let c=Math.max(e,o),u=Math.min(o+this.length,n);c<=u;){let f=r.lineAt(c);c==e&&(i+=a*(f.number-l)),s(new Tr(f.from,f.length,i,a,De.Text)),i+=a,c=f.to+1}}replace(e,n,r){let i=this.length-n;if(i>0){let o=r[r.length-1];o instanceof yt?r[r.length-1]=new yt(o.length+i):r.push(null,new yt(i-1))}if(e>0){let o=r[0];o instanceof yt?r[0]=new yt(e+o.length):r.unshift(new yt(e-1),null)}return Ut.of(r)}decomposeLeft(e,n){n.push(new yt(e-1),null)}decomposeRight(e,n){n.push(null,new yt(this.length-e-1))}updateHeight(e,n=0,r=!1,i){let o=n+this.length;if(i&&i.from<=n+this.length&&i.more){let s=[],l=Math.max(n,i.from),a=-1,c=e.heightChanged;for(i.from>n&&s.push(new yt(i.from-n-1).updateHeight(e,n));l<=o&&i.more;){let f=e.doc.lineAt(l).length;s.length&&s.push(null);let h=i.heights[i.index++];a==-1?a=h:Math.abs(h-a)>=of&&(a=-2);let d=new sn(f,h);d.outdated=!1,s.push(d),l+=f+1}l<=o&&s.push(null,new yt(o-l).updateHeight(e,l));let u=Ut.of(s);return e.heightChanged=c||a<0||Math.abs(u.height-this.height)>=of||Math.abs(a-this.lines(e.doc,n).lineHeight)>=of,u}else(r||this.outdated)&&(this.setHeight(e,e.heightForGap(n,n+this.length)),this.outdated=!1);return this}toString(){return`gap(${this.length})`}}class W4 extends Ut{constructor(e,n,r){super(e.length+n+r.length,e.height+r.height,n|(e.outdated||r.outdated?2:0));this.left=e,this.right=r,this.size=e.size+r.size}get break(){return this.flags&1}blockAt(e,n,r,i){let o=r+this.left.height;return e<o?this.left.blockAt(e,n,r,i):this.right.blockAt(e,n,o,i+this.left.length+this.break)}lineAt(e,n,r,i,o){let s=i+this.left.height,l=o+this.left.length+this.break,a=n==_e.ByHeight?e<s:e<l,c=a?this.left.lineAt(e,n,r,i,o):this.right.lineAt(e,n,r,s,l);if(this.break||(a?c.to<l:c.from>l))return c;let u=n==_e.ByPosNoHeight?_e.ByPosNoHeight:_e.ByPos;return a?c.join(this.right.lineAt(l,u,r,s,l)):this.left.lineAt(l,u,r,i,o).join(c)}forEachLine(e,n,r,i,o,s){let l=i+this.left.height,a=o+this.left.length+this.break;if(this.break)e<a&&this.left.forEachLine(e,n,r,i,o,s),n>=a&&this.right.forEachLine(e,n,r,l,a,s);else{let c=this.lineAt(a,_e.ByPos,r,i,o);e<c.from&&this.left.forEachLine(e,c.from-1,r,i,o,s),c.to>=e&&c.from<=n&&s(c),n>c.to&&this.right.forEachLine(c.to+1,n,r,l,a,s)}}replace(e,n,r){let i=this.left.length+this.break;if(n<i)return this.balanced(this.left.replace(e,n,r),this.right);if(e>this.left.length)return this.balanced(this.left,this.right.replace(e-i,n-i,r));let o=[];e>0&&this.decomposeLeft(e,o);let s=o.length;for(let l of r)o.push(l);if(e>0&&y$(o,s-1),n<this.length){let l=o.length;this.decomposeRight(n,o),y$(o,l)}return Ut.of(o)}decomposeLeft(e,n){let r=this.left.length;if(e<=r)return this.left.decomposeLeft(e,n);n.push(this.left),this.break&&(r++,e>=r&&n.push(null)),e>r&&this.right.decomposeLeft(e-r,n)}decomposeRight(e,n){let r=this.left.length,i=r+this.break;if(e>=i)return this.right.decomposeRight(e-i,n);e<r&&this.left.decomposeRight(e,n),this.break&&e<i&&n.push(null),n.push(this.right)}balanced(e,n){return e.size>2*n.size||n.size>2*e.size?Ut.of(this.break?[e,null,n]:[e,n]):(this.left=e,this.right=n,this.height=e.height+n.height,this.outdated=e.outdated||n.outdated,this.size=e.size+n.size,this.length=e.length+this.break+n.length,this)}updateHeight(e,n=0,r=!1,i){let{left:o,right:s}=this,l=n+o.length+this.break,a=null;return i&&i.from<=n+o.length&&i.more?a=o=o.updateHeight(e,n,r,i):o.updateHeight(e,n,r),i&&i.from<=l+s.length&&i.more?a=s=s.updateHeight(e,l,r,i):s.updateHeight(e,l,r),a?this.balanced(o,s):(this.height=this.left.height+this.right.height,this.outdated=!1,this)}toString(){return this.left+(this.break?" ":"-")+this.right}}function y$(t,e){let n,r;t[e]==null&&(n=t[e-1])instanceof yt&&(r=t[e+1])instanceof yt&&t.splice(e-1,3,new yt(n.length+1+r.length))}const Q4=5;class ag{constructor(e,n){this.pos=e,this.oracle=n,this.nodes=[],this.lineStart=-1,this.lineEnd=-1,this.covering=null,this.writtenTo=e}get isCovered(){return this.covering&&this.nodes[this.nodes.length-1]==this.covering}span(e,n){if(this.lineStart>-1){let r=Math.min(n,this.lineEnd),i=this.nodes[this.nodes.length-1];i instanceof sn?i.length+=r-this.pos:(r>this.pos||!this.isCovered)&&this.nodes.push(new sn(r-this.pos,-1)),this.writtenTo=r,n>r&&(this.nodes.push(null),this.writtenTo++,this.lineStart=-1)}this.pos=n}point(e,n,r){if(e<n||r.heightRelevant){let i=r.widget?r.widget.estimatedHeight:0;i<0&&(i=this.oracle.lineHeight);let o=n-e;r.block?this.addBlock(new g$(o,i,r.type)):(o||i>=Q4)&&this.addLineDeco(i,o)}else n>e&&this.span(e,n);this.lineEnd>-1&&this.lineEnd<this.pos&&(this.lineEnd=this.oracle.doc.lineAt(this.pos).to)}enterLine(){if(this.lineStart>-1)return;let{from:e,to:n}=this.oracle.doc.lineAt(this.pos);this.lineStart=e,this.lineEnd=n,this.writtenTo<e&&((this.writtenTo<e-1||this.nodes[this.nodes.length-1]==null)&&this.nodes.push(this.blankContent(this.writtenTo,e-1)),this.nodes.push(null)),this.pos>e&&this.nodes.push(new sn(this.pos-e,-1)),this.writtenTo=this.pos}blankContent(e,n){let r=new yt(n-e);return this.oracle.doc.lineAt(e).to==n&&(r.flags|=4),r}ensureLine(){this.enterLine();let e=this.nodes.length?this.nodes[this.nodes.length-1]:null;if(e instanceof sn)return e;let n=new sn(0,-1);return this.nodes.push(n),n}addBlock(e){this.enterLine(),e.type==De.WidgetAfter&&!this.isCovered&&this.ensureLine(),this.nodes.push(e),this.writtenTo=this.pos=this.pos+e.length,e.type!=De.WidgetBefore&&(this.covering=e)}addLineDeco(e,n){let r=this.ensureLine();r.length+=n,r.collapsed+=n,r.widgetHeight=Math.max(r.widgetHeight,e),this.writtenTo=this.pos=this.pos+n}finish(e){let n=this.nodes.length==0?null:this.nodes[this.nodes.length-1];this.lineStart>-1&&!(n instanceof sn)&&!this.isCovered?this.nodes.push(new sn(0,-1)):(this.writtenTo<this.pos||n==null)&&this.nodes.push(this.blankContent(this.writtenTo,this.pos));let r=e;for(let i of this.nodes)i instanceof sn&&i.updateHeight(this.oracle,r),r+=i?i.length:1;return this.nodes}static build(e,n,r,i){let o=new ag(r,e);return Te.spans(n,r,i,o,0),o.finish(r)}}function N4(t,e,n){let r=new _4;return Te.compare(t,e,n,r,0),r.changes}class _4{constructor(){this.changes=[]}compareRange(){}comparePoint(e,n,r,i){(e<n||r&&r.heightRelevant||i&&i.heightRelevant)&&Zm(e,n,this.changes,5)}}function B4(t,e){let n=t.getBoundingClientRect(),r=Math.max(0,n.left),i=Math.min(innerWidth,n.right),o=Math.max(0,n.top),s=Math.min(innerHeight,n.bottom),l=t.ownerDocument.body;for(let a=t.parentNode;a&&a!=l;)if(a.nodeType==1){let c=a,u=window.getComputedStyle(c);if((c.scrollHeight>c.clientHeight||c.scrollWidth>c.clientWidth)&&u.overflow!="visible"){let f=c.getBoundingClientRect();r=Math.max(r,f.left),i=Math.min(i,f.right),o=Math.max(o,f.top),s=Math.min(s,f.bottom)}a=u.position=="absolute"||u.position=="fixed"?c.offsetParent:c.parentNode}else if(a.nodeType==11)a=a.host;else break;return{left:r-n.left,right:Math.max(r,i)-n.left,top:o-(n.top+e),bottom:Math.max(o,s)-(n.top+e)}}class cg{constructor(e,n,r){this.from=e,this.to=n,this.size=r}static same(e,n){if(e.length!=n.length)return!1;for(let r=0;r<e.length;r++){let i=e[r],o=n[r];if(i.from!=o.from||i.to!=o.to||i.size!=o.size)return!1}return!0}draw(e){return te.replace({widget:new q4(this.size,e)}).range(this.from,this.to)}}class q4 extends Cr{constructor(e,n){super();this.size=e,this.vertical=n}eq(e){return e.size==this.size&&e.vertical==this.vertical}toDOM(){let e=document.createElement("div");return this.vertical?e.style.height=this.size+"px":(e.style.width=this.size+"px",e.style.height="2px",e.style.display="inline-block"),e}get estimatedHeight(){return this.vertical?this.size:-1}}class O${constructor(e){this.state=e,this.pixelViewport={left:0,right:window.innerWidth,top:0,bottom:0},this.inView=!0,this.paddingTop=0,this.paddingBottom=0,this.contentDOMWidth=0,this.contentDOMHeight=0,this.editorHeight=0,this.editorWidth=0,this.heightOracle=new D4,this.scaler=w$,this.scrollTarget=null,this.printing=!1,this.mustMeasureContent=!0,this.visibleRanges=[],this.mustEnforceCursorAssoc=!1,this.heightMap=Ut.empty().applyChanges(e.facet(eo),Ie.empty,this.heightOracle.setDoc(e.doc),[new Rr(0,0,0,e.doc.length)]),this.viewport=this.getViewport(0,null),this.updateViewportLines(),this.updateForViewport(),this.lineGaps=this.ensureLineGaps([]),this.lineGapDeco=te.set(this.lineGaps.map(n=>n.draw(!1))),this.computeVisibleRanges()}updateForViewport(){let e=[this.viewport],{main:n}=this.state.selection;for(let r=0;r<=1;r++){let i=r?n.head:n.anchor;if(!e.some(({from:o,to:s})=>i>=o&&i<=s)){let{from:o,to:s}=this.lineBlockAt(i);e.push(new sf(o,s))}}this.viewports=e.sort((r,i)=>r.from-i.from),this.scaler=this.heightMap.height<=7e6?w$:new V4(this.heightOracle.doc,this.heightMap,this.viewports)}updateViewportLines(){this.viewportLines=[],this.heightMap.forEachLine(this.viewport.from,this.viewport.to,this.state.doc,0,0,e=>{this.viewportLines.push(this.scaler.scale==1?e:Sa(e,this.scaler))})}update(e,n=null){let r=this.state;this.state=e.state;let i=this.state.facet(eo),o=e.changedRanges,s=Rr.extendWithRanges(o,N4(e.startState.facet(eo),i,e?e.changes:ct.empty(this.state.doc.length))),l=this.heightMap.height;this.heightMap=this.heightMap.applyChanges(i,r.doc,this.heightOracle.setDoc(this.state.doc),s),this.heightMap.height!=l&&(e.flags|=2);let a=s.length?this.mapViewport(this.viewport,e.changes):this.viewport;(n&&(n.range.head<a.from||n.range.head>a.to)||!this.viewportIsAppropriate(a))&&(a=this.getViewport(0,n));let c=!e.changes.empty||e.flags&2||a.from!=this.viewport.from||a.to!=this.viewport.to;this.viewport=a,this.updateForViewport(),c&&this.updateViewportLines(),(this.lineGaps.length||this.viewport.to-this.viewport.from>4e3)&&this.updateLineGaps(this.ensureLineGaps(this.mapLineGaps(this.lineGaps,e.changes))),e.flags|=this.computeVisibleRanges(),n&&(this.scrollTarget=n),!this.mustEnforceCursorAssoc&&e.selectionSet&&e.view.lineWrapping&&e.state.selection.main.empty&&e.state.selection.main.assoc&&(this.mustEnforceCursorAssoc=!0)}measure(e){let n=e.contentDOM,r=window.getComputedStyle(n),i=this.heightOracle,o=r.whiteSpace,s=r.direction=="rtl"?Se.RTL:Se.LTR,l=this.heightOracle.mustRefreshForStyle(o,s),a=l||this.mustMeasureContent||this.contentDOMHeight!=n.clientHeight,c=0,u=0;if(a){this.mustMeasureContent=!1,this.contentDOMHeight=n.clientHeight;let y=parseInt(r.paddingTop)||0,O=parseInt(r.paddingBottom)||0;(this.paddingTop!=y||this.paddingBottom!=O)&&(c|=8,this.paddingTop=y,this.paddingBottom=O)}let f=this.printing?{top:-1e8,bottom:1e8,left:-1e8,right:1e8}:B4(n,this.paddingTop),h=f.top-this.pixelViewport.top,d=f.bottom-this.pixelViewport.bottom;this.pixelViewport=f;let p=this.pixelViewport.bottom>this.pixelViewport.top&&this.pixelViewport.right>this.pixelViewport.left;if(p!=this.inView&&(this.inView=p,p&&(a=!0)),!this.inView)return 0;let m=n.clientWidth;if((this.contentDOMWidth!=m||this.editorHeight!=e.scrollDOM.clientHeight||this.editorWidth!=e.scrollDOM.clientWidth)&&(this.contentDOMWidth=m,this.editorHeight=e.scrollDOM.clientHeight,this.editorWidth=e.scrollDOM.clientWidth,c|=8),a){let y=e.docView.measureVisibleLineHeights();if(i.mustRefreshForHeights(y)&&(l=!0),l||i.lineWrapping&&Math.abs(m-this.contentDOMWidth)>i.charWidth){let{lineHeight:O,charWidth:v}=e.docView.measureTextSize();l=i.refresh(o,s,O,v,m/v,y),l&&(e.docView.minWidth=0,c|=8)}h>0&&d>0?u=Math.max(h,d):h<0&&d<0&&(u=Math.min(h,d)),i.heightChanged=!1,this.heightMap=this.heightMap.updateHeight(i,0,l,new L4(this.viewport.from,y)),i.heightChanged&&(c|=2)}let g=!this.viewportIsAppropriate(this.viewport,u)||this.scrollTarget&&(this.scrollTarget.range.head<this.viewport.from||this.scrollTarget.range.head>this.viewport.to);return g&&(this.viewport=this.getViewport(u,this.scrollTarget)),this.updateForViewport(),(c&2||g)&&this.updateViewportLines(),(this.lineGaps.length||this.viewport.to-this.viewport.from>4e3)&&this.updateLineGaps(this.ensureLineGaps(l?[]:this.lineGaps)),c|=this.computeVisibleRanges(),this.mustEnforceCursorAssoc&&(this.mustEnforceCursorAssoc=!1,e.docView.enforceCursorAssoc()),c}get visibleTop(){return this.scaler.fromDOM(this.pixelViewport.top)}get visibleBottom(){return this.scaler.fromDOM(this.pixelViewport.bottom)}getViewport(e,n){let r=.5-Math.max(-.5,Math.min(.5,e/1e3/2)),i=this.heightMap,o=this.state.doc,{visibleTop:s,visibleBottom:l}=this,a=new sf(i.lineAt(s-r*1e3,_e.ByHeight,o,0,0).from,i.lineAt(l+(1-r)*1e3,_e.ByHeight,o,0,0).to);if(n){let{head:c}=n.range,u=this.editorHeight;if(c<a.from||c>a.to){let f=i.lineAt(c,_e.ByPos,o,0,0),h;n.y=="center"?h=(f.top+f.bottom)/2-u/2:n.y=="start"||n.y=="nearest"&&c<a.from?h=f.top:h=f.bottom-u,a=new sf(i.lineAt(h-1e3/2,_e.ByHeight,o,0,0).from,i.lineAt(h+u+1e3/2,_e.ByHeight,o,0,0).to)}}return a}mapViewport(e,n){let r=n.mapPos(e.from,-1),i=n.mapPos(e.to,1);return new sf(this.heightMap.lineAt(r,_e.ByPos,this.state.doc,0,0).from,this.heightMap.lineAt(i,_e.ByPos,this.state.doc,0,0).to)}viewportIsAppropriate({from:e,to:n},r=0){if(!this.inView)return!0;let{top:i}=this.heightMap.lineAt(e,_e.ByPos,this.state.doc,0,0),{bottom:o}=this.heightMap.lineAt(n,_e.ByPos,this.state.doc,0,0),{visibleTop:s,visibleBottom:l}=this;return(e==0||i<=s-Math.max(10,Math.min(-r,250)))&&(n==this.state.doc.length||o>=l+Math.max(10,Math.min(r,250)))&&i>s-2*1e3&&o<l+2*1e3}mapLineGaps(e,n){if(!e.length||n.empty)return e;let r=[];for(let i of e)n.touchesRange(i.from,i.to)||r.push(new cg(n.mapPos(i.from),n.mapPos(i.to),i.size));return r}ensureLineGaps(e){let n=[];if(this.heightOracle.direction!=Se.LTR)return n;for(let r of this.viewportLines){if(r.length<4e3)continue;let i=z4(r.from,r.to,this.state);if(i.total<4e3)continue;let o,s;if(this.heightOracle.lineWrapping){let c=2e3/this.heightOracle.lineLength*this.heightOracle.lineHeight;o=lf(i,(this.visibleTop-r.top-c)/r.height),s=lf(i,(this.visibleBottom-r.top+c)/r.height)}else{let c=i.total*this.heightOracle.charWidth,u=2e3*this.heightOracle.charWidth;o=lf(i,(this.pixelViewport.left-u)/c),s=lf(i,(this.pixelViewport.right+u)/c)}let l=[];o>r.from&&l.push({from:r.from,to:o}),s<r.to&&l.push({from:s,to:r.to});let a=this.state.selection.main;a.from>=r.from&&a.from<=r.to&&x$(l,a.from-10,a.from+10),!a.empty&&a.to>=r.from&&a.to<=r.to&&x$(l,a.to-10,a.to+10);for(let{from:c,to:u}of l)u-c>1e3&&n.push(F4(e,f=>f.from>=r.from&&f.to<=r.to&&Math.abs(f.from-c)<1e3&&Math.abs(f.to-u)<1e3)||new cg(c,u,this.gapSize(r,c,u,i)))}return n}gapSize(e,n,r,i){let o=v$(i,r)-v$(i,n);return this.heightOracle.lineWrapping?e.height*o:i.total*this.heightOracle.charWidth*o}updateLineGaps(e){cg.same(e,this.lineGaps)||(this.lineGaps=e,this.lineGapDeco=te.set(e.map(n=>n.draw(this.heightOracle.lineWrapping))))}computeVisibleRanges(){let e=this.state.facet(eo);this.lineGaps.length&&(e=e.concat(this.lineGapDeco));let n=[];Te.spans(e,this.viewport.from,this.viewport.to,{span(i,o){n.push({from:i,to:o})},point(){}},20);let r=n.length!=this.visibleRanges.length||this.visibleRanges.some((i,o)=>i.from!=n[o].from||i.to!=n[o].to);return this.visibleRanges=n,r?4:0}lineBlockAt(e){return e>=this.viewport.from&&e<=this.viewport.to&&this.viewportLines.find(n=>n.from<=e&&n.to>=e)||Sa(this.heightMap.lineAt(e,_e.ByPos,this.state.doc,0,0),this.scaler)}lineBlockAtHeight(e){return Sa(this.heightMap.lineAt(this.scaler.fromDOM(e),_e.ByHeight,this.state.doc,0,0),this.scaler)}elementAtHeight(e){return Sa(this.heightMap.blockAt(this.scaler.fromDOM(e),this.state.doc,0,0),this.scaler)}get docHeight(){return this.scaler.toDOM(this.heightMap.height)}get contentHeight(){return this.docHeight+this.paddingTop+this.paddingBottom}}class sf{constructor(e,n){this.from=e,this.to=n}}function z4(t,e,n){let r=[],i=t,o=0;return Te.spans(n.facet(eo),t,e,{span(){},point(s,l){s>i&&(r.push({from:i,to:s}),o+=s-i),i=l}},20),i<e&&(r.push({from:i,to:e}),o+=e-i),{total:o,ranges:r}}function lf({total:t,ranges:e},n){if(n<=0)return e[0].from;if(n>=1)return e[e.length-1].to;let r=Math.floor(t*n);for(let i=0;;i++){let{from:o,to:s}=e[i],l=s-o;if(r<=l)return o+r;r-=l}}function v$(t,e){let n=0;for(let{from:r,to:i}of t.ranges){if(e<=i){n+=e-r;break}n+=i-r}return n/t.total}function x$(t,e,n){for(let r=0;r<t.length;r++){let i=t[r];if(i.from<n&&i.to>e){let o=[];i.from<e&&o.push({from:i.from,to:e}),i.to>n&&o.push({from:n,to:i.to}),t.splice(r,1,...o),r+=o.length-1}}}function F4(t,e){for(let n of t)if(e(n))return n}const w$={toDOM(t){return t},fromDOM(t){return t},scale:1};class V4{constructor(e,n,r){let i=0,o=0,s=0;this.viewports=r.map(({from:l,to:a})=>{let c=n.lineAt(l,_e.ByPos,e,0,0).top,u=n.lineAt(a,_e.ByPos,e,0,0).bottom;return i+=u-c,{from:l,to:a,top:c,bottom:u,domTop:0,domBottom:0}}),this.scale=(7e6-i)/(n.height-i);for(let l of this.viewports)l.domTop=s+(l.top-o)*this.scale,s=l.domBottom=l.domTop+(l.bottom-l.top),o=l.bottom}toDOM(e){for(let n=0,r=0,i=0;;n++){let o=n<this.viewports.length?this.viewports[n]:null;if(!o||e<o.top)return i+(e-r)*this.scale;if(e<=o.bottom)return o.domTop+(e-o.top);r=o.bottom,i=o.domBottom}}fromDOM(e){for(let n=0,r=0,i=0;;n++){let o=n<this.viewports.length?this.viewports[n]:null;if(!o||e<o.domTop)return r+(e-i)/this.scale;if(e<=o.domBottom)return o.top+(e-o.domTop);r=o.bottom,i=o.domBottom}}}function Sa(t,e){if(e.scale==1)return t;let n=e.toDOM(t.top),r=e.toDOM(t.bottom);return new Tr(t.from,t.length,n,r-n,Array.isArray(t.type)?t.type.map(i=>Sa(i,e)):t.type)}const b$=ne.define({combine:t=>t.join(" ")}),ug=ne.define({combine:t=>t.indexOf(!0)>-1}),fg=gi.newName(),$$=gi.newName(),S$=gi.newName(),k$={"&light":"."+$$,"&dark":"."+S$};function hg(t,e,n){return new gi(e,{finish(r){return/&/.test(r)?r.replace(/&\w*/,i=>{if(i=="&")return t;if(!n||!n[i])throw new RangeError(`Unsupported selector: ${i}`);return n[i]}):t+" "+r}})}const U4=hg("."+fg,{"&.cm-editor":{position:"relative !important",boxSizing:"border-box","&.cm-focused":{outline:"1px dotted #212121"},display:"flex !important",flexDirection:"column"},".cm-scroller":{display:"flex !important",alignItems:"flex-start !important",fontFamily:"monospace",lineHeight:1.4,height:"100%",overflowX:"auto",position:"relative",zIndex:0},".cm-content":{margin:0,flexGrow:2,minHeight:"100%",display:"block",whiteSpace:"pre",wordWrap:"normal",boxSizing:"border-box",padding:"4px 0",outline:"none","&[contenteditable=true]":{WebkitUserModify:"read-write-plaintext-only"}},".cm-lineWrapping":{whiteSpace_fallback:"pre-wrap",whiteSpace:"break-spaces",wordBreak:"break-word",overflowWrap:"anywhere"},"&light .cm-content":{caretColor:"black"},"&dark .cm-content":{caretColor:"white"},".cm-line":{display:"block",padding:"0 2px 0 4px"},".cm-selectionLayer":{zIndex:-1,contain:"size style"},".cm-selectionBackground":{position:"absolute"},"&light .cm-selectionBackground":{background:"#d9d9d9"},"&dark .cm-selectionBackground":{background:"#222"},"&light.cm-focused .cm-selectionBackground":{background:"#d7d4f0"},"&dark.cm-focused .cm-selectionBackground":{background:"#233"},".cm-cursorLayer":{zIndex:100,contain:"size style",pointerEvents:"none"},"&.cm-focused .cm-cursorLayer":{animation:"steps(1) cm-blink 1.2s infinite"},"@keyframes cm-blink":{"0%":{},"50%":{visibility:"hidden"},"100%":{}},"@keyframes cm-blink2":{"0%":{},"50%":{visibility:"hidden"},"100%":{}},".cm-cursor, .cm-dropCursor":{position:"absolute",borderLeft:"1.2px solid black",marginLeft:"-0.6px",pointerEvents:"none"},".cm-cursor":{display:"none"},"&dark .cm-cursor":{borderLeftColor:"#444"},"&.cm-focused .cm-cursor":{display:"block"},"&light .cm-activeLine":{backgroundColor:"#f3f9ff"},"&dark .cm-activeLine":{backgroundColor:"#223039"},"&light .cm-specialChar":{color:"red"},"&dark .cm-specialChar":{color:"#f78"},".cm-tab":{display:"inline-block",overflow:"hidden",verticalAlign:"bottom"},".cm-placeholder":{color:"#888",display:"inline-block",verticalAlign:"top"},".cm-button":{verticalAlign:"middle",color:"inherit",fontSize:"70%",padding:".2em 1em",borderRadius:"1px"},"&light .cm-button":{backgroundImage:"linear-gradient(#eff1f5, #d9d9df)",border:"1px solid #888","&:active":{backgroundImage:"linear-gradient(#b4b4b4, #d0d3d6)"}},"&dark .cm-button":{backgroundImage:"linear-gradient(#393939, #111)",border:"1px solid #888","&:active":{backgroundImage:"linear-gradient(#111, #333)"}},".cm-textfield":{verticalAlign:"middle",color:"inherit",fontSize:"70%",border:"1px solid silver",padding:".2em .5em"},"&light .cm-textfield":{backgroundColor:"white"},"&dark .cm-textfield":{border:"1px solid #555",backgroundColor:"inherit"}},k$),j4={childList:!0,characterData:!0,subtree:!0,attributes:!0,characterDataOldValue:!0},dg=re.ie&&re.ie_version<=11;class X4{constructor(e,n,r){this.view=e,this.onChange=n,this.onScrollChanged=r,this.active=!1,this.selectionRange=new H3,this.selectionChanged=!1,this.delayedFlush=-1,this.resizeTimeout=-1,this.queue=[],this.delayedAndroidKey=null,this.scrollTargets=[],this.intersection=null,this.resize=null,this.intersecting=!1,this.gapIntersection=null,this.gaps=[],this.parentCheck=-1,this.dom=e.contentDOM,this.observer=new MutationObserver(i=>{for(let o of i)this.queue.push(o);(re.ie&&re.ie_version<=11||re.ios&&e.composing)&&i.some(o=>o.type=="childList"&&o.removedNodes.length||o.type=="characterData"&&o.oldValue.length>o.target.nodeValue.length)?this.flushSoon():this.flush()}),dg&&(this.onCharData=i=>{this.queue.push({target:i.target,type:"characterData",oldValue:i.prevValue}),this.flushSoon()}),this.onSelectionChange=this.onSelectionChange.bind(this),typeof ResizeObserver=="function"&&(this.resize=new ResizeObserver(()=>{this.view.docView.lastUpdate<Date.now()-75&&this.resizeTimeout<0&&(this.resizeTimeout=setTimeout(()=>{this.resizeTimeout=-1,this.view.requestMeasure()},50))}),this.resize.observe(e.scrollDOM)),this.start(),this.onScroll=this.onScroll.bind(this),window.addEventListener("scroll",this.onScroll),typeof IntersectionObserver=="function"&&(this.intersection=new IntersectionObserver(i=>{this.parentCheck<0&&(this.parentCheck=setTimeout(this.listenForScroll.bind(this),1e3)),i.length>0&&i[i.length-1].intersectionRatio>0!=this.intersecting&&(this.intersecting=!this.intersecting,this.intersecting!=this.view.inView&&this.onScrollChanged(document.createEvent("Event")))},{}),this.intersection.observe(this.dom),this.gapIntersection=new IntersectionObserver(i=>{i.length>0&&i[i.length-1].intersectionRatio>0&&this.onScrollChanged(document.createEvent("Event"))},{})),this.listenForScroll(),this.readSelectionRange(),this.dom.ownerDocument.addEventListener("selectionchange",this.onSelectionChange)}onScroll(e){this.intersecting&&this.flush(!1),this.onScrollChanged(e)}updateGaps(e){if(this.gapIntersection&&(e.length!=this.gaps.length||this.gaps.some((n,r)=>n!=e[r]))){this.gapIntersection.disconnect();for(let n of e)this.gapIntersection.observe(n);this.gaps=e}}onSelectionChange(e){if(!this.readSelectionRange()||this.delayedAndroidKey)return;let{view:n}=this,r=this.selectionRange;if(n.state.facet(wa)?n.root.activeElement!=this.dom:!_m(n.dom,r))return;let i=r.anchorNode&&n.docView.nearest(r.anchorNode);i&&i.ignoreEvent(e)||((re.ie&&re.ie_version<=11||re.android&&re.chrome)&&!n.state.selection.main.empty&&r.focusNode&&Ku(r.focusNode,r.focusOffset,r.anchorNode,r.anchorOffset)?this.flushSoon():this.flush(!1))}readSelectionRange(){let{root:e}=this.view,n=Yu(e),r=re.safari&&e.nodeType==11&&U3()==this.view.contentDOM&&H4(this.view)||n;return this.selectionRange.eq(r)?!1:(this.selectionRange.setRange(r),this.selectionChanged=!0)}setSelectionRange(e,n){this.selectionRange.set(e.node,e.offset,n.node,n.offset),this.selectionChanged=!1}listenForScroll(){this.parentCheck=-1;let e=0,n=null;for(let r=this.dom;r;)if(r.nodeType==1)!n&&e<this.scrollTargets.length&&this.scrollTargets[e]==r?e++:n||(n=this.scrollTargets.slice(0,e)),n&&n.push(r),r=r.assignedSlot||r.parentNode;else if(r.nodeType==11)r=r.host;else break;if(e<this.scrollTargets.length&&!n&&(n=this.scrollTargets.slice(0,e)),n){for(let r of this.scrollTargets)r.removeEventListener("scroll",this.onScroll);for(let r of this.scrollTargets=n)r.addEventListener("scroll",this.onScroll)}}ignore(e){if(!this.active)return e();try{return this.stop(),e()}finally{this.start(),this.clear()}}start(){this.active||(this.observer.observe(this.dom,j4),dg&&this.dom.addEventListener("DOMCharacterDataModified",this.onCharData),this.active=!0)}stop(){!this.active||(this.active=!1,this.observer.disconnect(),dg&&this.dom.removeEventListener("DOMCharacterDataModified",this.onCharData))}clear(){this.observer.takeRecords(),this.queue.length=0,this.selectionChanged=!1}delayAndroidKey(e,n){this.delayedAndroidKey||requestAnimationFrame(()=>{let r=this.delayedAndroidKey;this.delayedAndroidKey=null;let i=this.view.state;va(this.view.contentDOM,r.key,r.keyCode)?this.processRecords():this.flush(),this.view.state==i&&this.view.update([])}),(!this.delayedAndroidKey||e=="Enter")&&(this.delayedAndroidKey={key:e,keyCode:n})}flushSoon(){this.delayedFlush<0&&(this.delayedFlush=window.setTimeout(()=>{this.delayedFlush=-1,this.flush()},20))}forceFlush(){this.delayedFlush>=0&&(window.clearTimeout(this.delayedFlush),this.delayedFlush=-1,this.flush())}processRecords(){let e=this.queue;for(let o of this.observer.takeRecords())e.push(o);e.length&&(this.queue=[]);let n=-1,r=-1,i=!1;for(let o of e){let s=this.readMutation(o);!s||(s.typeOver&&(i=!0),n==-1?{from:n,to:r}=s:(n=Math.min(s.from,n),r=Math.max(s.to,r)))}return{from:n,to:r,typeOver:i}}flush(e=!0){if(this.delayedFlush>=0||this.delayedAndroidKey)return;e&&this.readSelectionRange();let{from:n,to:r,typeOver:i}=this.processRecords(),o=this.selectionChanged&&_m(this.dom,this.selectionRange);if(n<0&&!o)return;this.selectionChanged=!1;let s=this.view.state;this.onChange(n,r,i),this.view.state==s&&this.view.update([])}readMutation(e){let n=this.view.docView.nearest(e.target);if(!n||n.ignoreMutation(e))return null;if(n.markDirty(e.type=="attributes"),e.type=="attributes"&&(n.dirty|=4),e.type=="childList"){let r=P$(n,e.previousSibling||e.target.previousSibling,-1),i=P$(n,e.nextSibling||e.target.nextSibling,1);return{from:r?n.posAfter(r):n.posAtStart,to:i?n.posBefore(i):n.posAtEnd,typeOver:!1}}else return e.type=="characterData"?{from:n.posAtStart,to:n.posAtEnd,typeOver:e.target.nodeValue==e.oldValue}:null}destroy(){var e,n,r;this.stop(),(e=this.intersection)===null||e===void 0||e.disconnect(),(n=this.gapIntersection)===null||n===void 0||n.disconnect(),(r=this.resize)===null||r===void 0||r.disconnect();for(let i of this.scrollTargets)i.removeEventListener("scroll",this.onScroll);window.removeEventListener("scroll",this.onScroll),this.dom.ownerDocument.removeEventListener("selectionchange",this.onSelectionChange),clearTimeout(this.parentCheck),clearTimeout(this.resizeTimeout)}}function P$(t,e,n){for(;e;){let r=He.get(e);if(r&&r.parent==t)return r;let i=e.parentNode;e=i!=t.dom?i:n>0?e.nextSibling:e.previousSibling}return null}function H4(t){let e=null;function n(a){a.preventDefault(),a.stopImmediatePropagation(),e=a.getTargetRanges()[0]}if(t.contentDOM.addEventListener("beforeinput",n,!0),document.execCommand("indent"),t.contentDOM.removeEventListener("beforeinput",n,!0),!e)return null;let r=e.startContainer,i=e.startOffset,o=e.endContainer,s=e.endOffset,l=t.docView.domAtPos(t.state.selection.main.anchor);return Ku(l.node,l.offset,o,s)&&([r,i,o,s]=[o,s,r,i]),{anchorNode:r,anchorOffset:i,focusNode:o,focusOffset:s}}function G4(t,e,n,r){let i,o,s=t.state.selection.main;if(e>-1){let l=t.docView.domBoundsAround(e,n,0);if(!l||t.state.readOnly)return;let{from:a,to:c}=l,u=t.docView.impreciseHead||t.docView.impreciseAnchor?[]:Y4(t),f=new zb(u,t);f.readRange(l.startDOM,l.endDOM),o=K4(u,a);let h=s.from,d=null;(t.inputState.lastKeyCode===8&&t.inputState.lastKeyTime>Date.now()-100||re.android&&f.text.length<c-a)&&(h=s.to,d="end");let p=Z4(t.state.sliceDoc(a,c),f.text,h-a,d);p&&(i={from:a+p.from,to:a+p.toA,insert:t.state.toText(f.text.slice(p.from,p.toB))})}else if(t.hasFocus||!t.state.facet(wa)){let l=t.observer.selectionRange,{impreciseHead:a,impreciseAnchor:c}=t.docView,u=a&&a.node==l.focusNode&&a.offset==l.focusOffset||!Nm(t.contentDOM,l.focusNode)?t.state.selection.main.head:t.docView.posFromDOM(l.focusNode,l.focusOffset),f=c&&c.node==l.anchorNode&&c.offset==l.anchorOffset||!Nm(t.contentDOM,l.anchorNode)?t.state.selection.main.anchor:t.docView.posFromDOM(l.anchorNode,l.anchorOffset);(u!=s.head||f!=s.anchor)&&(o=_.single(f,u))}if(!(!i&&!o)){if(!i&&r&&!s.empty&&o&&o.main.empty?i={from:s.from,to:s.to,insert:t.state.doc.slice(s.from,s.to)}:i&&i.from>=s.from&&i.to<=s.to&&(i.from!=s.from||i.to!=s.to)&&s.to-s.from-(i.to-i.from)<=4&&(i={from:s.from,to:s.to,insert:t.state.doc.slice(s.from,i.from).append(i.insert).append(t.state.doc.slice(i.to,s.to))}),i){let l=t.state;if(re.ios&&t.inputState.flushIOSKey(t)||re.android&&(i.from==s.from&&i.to==s.to&&i.insert.length==1&&i.insert.lines==2&&va(t.contentDOM,"Enter",13)||i.from==s.from-1&&i.to==s.to&&i.insert.length==0&&va(t.contentDOM,"Backspace",8)||i.from==s.from&&i.to==s.to+1&&i.insert.length==0&&va(t.contentDOM,"Delete",46)))return;let a=i.insert.toString();if(t.state.facet(Mb).some(f=>f(t,i.from,i.to,a)))return;t.inputState.composing>=0&&t.inputState.composing++;let c;if(i.from>=s.from&&i.to<=s.to&&i.to-i.from>=(s.to-s.from)/3&&(!o||o.main.empty&&o.main.from==i.from+i.insert.length)){let f=s.from<i.from?l.sliceDoc(s.from,i.from):"",h=s.to>i.to?l.sliceDoc(i.to,s.to):"";c=l.replaceSelection(t.state.toText(f+i.insert.sliceString(0,void 0,t.state.lineBreak)+h))}else{let f=l.changes(i);c={changes:f,selection:o&&!l.selection.main.eq(o.main)&&o.main.to<=f.newLength?l.selection.replaceRange(o.main):void 0}}let u="input.type";t.composing&&(u+=".compose",t.inputState.compositionFirstChange&&(u+=".start",t.inputState.compositionFirstChange=!1)),t.dispatch(c,{scrollIntoView:!0,userEvent:u})}else if(o&&!o.main.eq(s)){let l=!1,a="select";t.inputState.lastSelectionTime>Date.now()-50&&(t.inputState.lastSelectionOrigin=="select"&&(l=!0),a=t.inputState.lastSelectionOrigin),t.dispatch({selection:o,scrollIntoView:l,userEvent:a})}}}function Z4(t,e,n,r){let i=Math.min(t.length,e.length),o=0;for(;o<i&&t.charCodeAt(o)==e.charCodeAt(o);)o++;if(o==i&&t.length==e.length)return null;let s=t.length,l=e.length;for(;s>0&&l>0&&t.charCodeAt(s-1)==e.charCodeAt(l-1);)s--,l--;if(r=="end"){let a=Math.max(0,o-Math.min(s,l));n-=s+a-o}return s<o&&t.length<e.length?(o-=n<=o&&n>=s?o-n:0,l=o+(l-s),s=o):l<o&&(o-=n<=o&&n>=l?o-n:0,s=o+(s-l),l=o),{from:o,toA:s,toB:l}}function Y4(t){let e=[];if(t.root.activeElement!=t.contentDOM)return e;let{anchorNode:n,anchorOffset:r,focusNode:i,focusOffset:o}=t.observer.selectionRange;return n&&(e.push(new Vb(n,r)),(i!=n||o!=r)&&e.push(new Vb(i,o))),e}function K4(t,e){if(t.length==0)return null;let n=t[0].pos,r=t.length==2?t[1].pos:n;return n>-1&&r>-1?_.single(n+e,r+e):null}class oe{constructor(e={}){this.plugins=[],this.pluginMap=new Map,this.editorAttrs={},this.contentAttrs={},this.bidiCache=[],this.destroyed=!1,this.updateState=2,this.measureScheduled=-1,this.measureRequests=[],this.contentDOM=document.createElement("div"),this.scrollDOM=document.createElement("div"),this.scrollDOM.tabIndex=-1,this.scrollDOM.className="cm-scroller",this.scrollDOM.appendChild(this.contentDOM),this.announceDOM=document.createElement("div"),this.announceDOM.style.cssText="position: absolute; top: -10000px",this.announceDOM.setAttribute("aria-live","polite"),this.dom=document.createElement("div"),this.dom.appendChild(this.announceDOM),this.dom.appendChild(this.scrollDOM),this._dispatch=e.dispatch||(n=>this.update([n])),this.dispatch=this.dispatch.bind(this),this.root=e.root||G3(e.parent)||document,this.viewState=new O$(e.state||Pe.create()),this.plugins=this.state.facet(ba).map(n=>new eg(n));for(let n of this.plugins)n.update(this);this.observer=new X4(this,(n,r,i)=>{G4(this,n,r,i)},n=>{this.inputState.runScrollHandlers(this,n),this.observer.intersecting&&this.measure()}),this.inputState=new $4(this),this.docView=new Ub(this),this.mountStyles(),this.updateAttrs(),this.updateState=0,e_(),this.requestMeasure(),e.parent&&e.parent.appendChild(this.dom)}get state(){return this.viewState.state}get viewport(){return this.viewState.viewport}get visibleRanges(){return this.viewState.visibleRanges}get inView(){return this.viewState.inView}get composing(){return this.inputState.composing>0}dispatch(...e){this._dispatch(e.length==1&&e[0]instanceof ut?e[0]:this.state.update(...e))}update(e){if(this.updateState!=0)throw new Error("Calls to EditorView.update are not allowed while an update is in progress");let n=!1,r,i=this.state;for(let s of e){if(s.startState!=i)throw new RangeError("Trying to update state with a transaction that doesn't start from the previous state.");i=s.state}if(this.destroyed){this.viewState.state=i;return}if(i.facet(Pe.phrases)!=this.state.facet(Pe.phrases))return this.setState(i);r=new Nb(this,i,e);let o=this.viewState.scrollTarget;try{this.updateState=2;for(let s of e){if(o&&(o=o.map(s.changes)),s.scrollIntoView){let{main:l}=s.state.selection;o=new gs(l.empty?l:_.cursor(l.head,l.head>l.anchor?-1:1))}for(let l of s.effects)l.is(Jm)?o=new gs(l.value):l.is(Ab)?o=new gs(l.value,"center"):l.is(Db)&&(o=l.value)}this.viewState.update(r,o),this.bidiCache=af.update(this.bidiCache,r.changes),r.empty||(this.updatePlugins(r),this.inputState.update(r)),n=this.docView.update(r),this.state.facet($a)!=this.styleModules&&this.mountStyles(),this.updateAttrs(),this.showAnnouncements(e),this.docView.updateSelection(n,e.some(s=>s.isUserEvent("select.pointer")))}finally{this.updateState=0}if((n||o||this.viewState.mustEnforceCursorAssoc)&&this.requestMeasure(),!r.empty)for(let s of this.state.facet(Km))s(r)}setState(e){if(this.updateState!=0)throw new Error("Calls to EditorView.setState are not allowed while an update is in progress");if(this.destroyed){this.viewState.state=e;return}this.updateState=2;try{for(let n of this.plugins)n.destroy(this);this.viewState=new O$(e),this.plugins=e.facet(ba).map(n=>new eg(n)),this.pluginMap.clear();for(let n of this.plugins)n.update(this);this.docView=new Ub(this),this.inputState.ensureHandlers(this),this.mountStyles(),this.updateAttrs(),this.bidiCache=[]}finally{this.updateState=0}this.requestMeasure()}updatePlugins(e){let n=e.startState.facet(ba),r=e.state.facet(ba);if(n!=r){let i=[];for(let o of r){let s=n.indexOf(o);if(s<0)i.push(new eg(o));else{let l=this.plugins[s];l.mustUpdate=e,i.push(l)}}for(let o of this.plugins)o.mustUpdate!=e&&o.destroy(this);this.plugins=i,this.pluginMap.clear(),this.inputState.ensureHandlers(this)}else for(let i of this.plugins)i.mustUpdate=e;for(let i=0;i<this.plugins.length;i++)this.plugins[i].update(this)}measure(e=!0){if(this.destroyed)return;this.measureScheduled>-1&&cancelAnimationFrame(this.measureScheduled),this.measureScheduled=0,e&&this.observer.flush();let n=null;try{for(let r=0;;r++){this.updateState=1;let i=this.viewport,o=this.viewState.measure(this);if(!o&&!this.measureRequests.length&&this.viewState.scrollTarget==null)break;if(r>5){console.warn(this.measureRequests.length?"Measure loop restarted more than 5 times":"Viewport failed to stabilize");break}let s=[];o&4||([this.measureRequests,s]=[s,this.measureRequests]);let l=s.map(u=>{try{return u.read(this)}catch(f){return bn(this.state,f),C$}}),a=new Nb(this,this.state),c=!1;a.flags|=o,n?n.flags|=o:n=a,this.updateState=2,a.empty||(this.updatePlugins(a),this.inputState.update(a),this.updateAttrs(),c=this.docView.update(a));for(let u=0;u<s.length;u++)if(l[u]!=C$)try{let f=s[u];f.write&&f.write(l[u],this)}catch(f){bn(this.state,f)}if(this.viewState.scrollTarget&&(this.docView.scrollIntoView(this.viewState.scrollTarget),this.viewState.scrollTarget=null),c&&this.docView.updateSelection(!0),this.viewport.from==i.from&&this.viewport.to==i.to&&this.measureRequests.length==0)break}}finally{this.updateState=0,this.measureScheduled=-1}if(n&&!n.empty)for(let r of this.state.facet(Km))r(n)}get themeClasses(){return fg+" "+(this.state.facet(ug)?S$:$$)+" "+this.state.facet(b$)}updateAttrs(){let e=R$(this,Wb,{class:"cm-editor"+(this.hasFocus?" cm-focused ":" ")+this.themeClasses}),n={spellcheck:"false",autocorrect:"off",autocapitalize:"off",translate:"no",contenteditable:this.state.facet(wa)?"true":"false",class:"cm-content",style:`${re.tabSize}: ${this.state.tabSize}`,role:"textbox","aria-multiline":"true"};this.state.readOnly&&(n["aria-readonly"]="true"),R$(this,Qb,n),this.observer.ignore(()=>{Gm(this.contentDOM,this.contentAttrs,n),Gm(this.dom,this.editorAttrs,e)}),this.editorAttrs=e,this.contentAttrs=n}showAnnouncements(e){let n=!0;for(let r of e)for(let i of r.effects)if(i.is(oe.announce)){n&&(this.announceDOM.textContent=""),n=!1;let o=this.announceDOM.appendChild(document.createElement("div"));o.textContent=i.value}}mountStyles(){this.styleModules=this.state.facet($a),gi.mount(this.root,this.styleModules.concat(U4).reverse())}readMeasured(){if(this.updateState==2)throw new Error("Reading the editor layout isn't allowed during an update");this.updateState==0&&this.measureScheduled>-1&&this.measure(!1)}requestMeasure(e){if(this.measureScheduled<0&&(this.measureScheduled=requestAnimationFrame(()=>this.measure())),e){if(e.key!=null){for(let n=0;n<this.measureRequests.length;n++)if(this.measureRequests[n].key===e.key){this.measureRequests[n]=e;return}}this.measureRequests.push(e)}}pluginField(e){let n=[];for(let r of this.plugins)r.update(this).takeField(e,n);return n}plugin(e){let n=this.pluginMap.get(e);return(n===void 0||n&&n.spec!=e)&&this.pluginMap.set(e,n=this.plugins.find(r=>r.spec==e)||null),n&&n.update(this).value}get documentTop(){return this.contentDOM.getBoundingClientRect().top+this.viewState.paddingTop}get documentPadding(){return{top:this.viewState.paddingTop,bottom:this.viewState.paddingBottom}}blockAtHeight(e,n){let r=pg(n,this);return this.elementAtHeight(e-r).moveY(r)}elementAtHeight(e){return this.readMeasured(),this.viewState.elementAtHeight(e)}visualLineAtHeight(e,n){let r=pg(n,this);return this.lineBlockAtHeight(e-r).moveY(r)}lineBlockAtHeight(e){return this.readMeasured(),this.viewState.lineBlockAtHeight(e)}viewportLines(e,n){let r=pg(n,this);for(let i of this.viewportLineBlocks)e(i.moveY(r))}get viewportLineBlocks(){return this.viewState.viewportLines}visualLineAt(e,n=0){return this.lineBlockAt(e).moveY(n+this.viewState.paddingTop)}lineBlockAt(e){return this.viewState.lineBlockAt(e)}get contentHeight(){return this.viewState.contentHeight}moveByChar(e,n,r){return og(this,e,Jb(this,e,n,r))}moveByGroup(e,n){return og(this,e,Jb(this,e,n,r=>w4(this,e.head,r)))}moveToLineBoundary(e,n,r=!0){return x4(this,e,n,r)}moveVertically(e,n,r){return og(this,e,b4(this,e,n,r))}scrollPosIntoView(e){this.dispatch({effects:Jm.of(_.cursor(e))})}domAtPos(e){return this.docView.domAtPos(e)}posAtDOM(e,n=0){return this.docView.posFromDOM(e,n)}posAtCoords(e,n=!0){return this.readMeasured(),Yb(this,e,n)}coordsAtPos(e,n=1){this.readMeasured();let r=this.docView.coordsAt(e,n);if(!r||r.left==r.right)return r;let i=this.state.doc.lineAt(e),o=this.bidiSpans(i),s=o[ys.find(o,e-i.from,-1,n)];return ef(r,s.dir==Se.LTR==n>0)}get defaultCharacterWidth(){return this.viewState.heightOracle.charWidth}get defaultLineHeight(){return this.viewState.heightOracle.lineHeight}get textDirection(){return this.viewState.heightOracle.direction}get lineWrapping(){return this.viewState.heightOracle.lineWrapping}bidiSpans(e){if(e.length>J4)return Bb(e.length);let n=this.textDirection;for(let i of this.bidiCache)if(i.from==e.from&&i.dir==n)return i.order;let r=l4(e.text,this.textDirection);return this.bidiCache.push(new af(e.from,e.to,n,r)),r}get hasFocus(){var e;return(document.hasFocus()||re.safari&&((e=this.inputState)===null||e===void 0?void 0:e.lastContextMenu)>Date.now()-3e4)&&this.root.activeElement==this.contentDOM}focus(){this.observer.ignore(()=>{ub(this.contentDOM),this.docView.updateSelection()})}destroy(){for(let e of this.plugins)e.destroy(this);this.plugins=[],this.inputState.destroy(),this.dom.remove(),this.observer.destroy(),this.measureScheduled>-1&&cancelAnimationFrame(this.measureScheduled),this.destroyed=!0}static scrollIntoView(e,n={}){return Db.of(new gs(typeof e=="number"?_.cursor(e):e,n.y,n.x,n.yMargin,n.xMargin))}static domEventHandlers(e){return rt.define(()=>({}),{eventHandlers:e})}static theme(e,n){let r=gi.newName(),i=[b$.of(r),$a.of(hg(`.${r}`,e))];return n&&n.dark&&i.push(ug.of(!0)),i}static baseTheme(e){return da.lowest($a.of(hg("."+fg,e,k$)))}}oe.scrollTo=Jm;oe.centerOn=Ab;oe.styleModule=$a;oe.inputHandler=Mb;oe.exceptionSink=Eb;oe.updateListener=Km;oe.editable=wa;oe.mouseSelectionStyle=Ib;oe.dragMovesSelection=Tb;oe.clickAddsSelectionRange=Rb;oe.decorations=eo;oe.darkTheme=ug;oe.contentAttributes=Qb;oe.editorAttributes=Wb;oe.lineWrapping=oe.contentAttributes.of({class:"cm-lineWrapping"});oe.announce=ce.define();const J4=4096;function pg(t,e){return(t==null?e.contentDOM.getBoundingClientRect().top:t)+e.viewState.paddingTop}let mg=-1;function e_(){window.addEventListener("resize",()=>{mg==-1&&(mg=setTimeout(t_,50))})}function t_(){mg=-1;let t=document.querySelectorAll(".cm-content");for(let e=0;e<t.length;e++){let n=He.get(t[e]);n&&n.editorView.requestMeasure()}}const C$={};class af{constructor(e,n,r,i){this.from=e,this.to=n,this.dir=r,this.order=i}static update(e,n){if(n.empty)return e;let r=[],i=e.length?e[e.length-1].dir:Se.LTR;for(let o=Math.max(0,e.length-10);o<e.length;o++){let s=e[o];s.dir==i&&!n.touchesRange(s.from,s.to)&&r.push(new af(n.mapPos(s.from,1),n.mapPos(s.to,-1),s.dir,s.order))}return r}}function R$(t,e,n){for(let r=t.state.facet(e),i=r.length-1;i>=0;i--){let o=r[i],s=typeof o=="function"?o(t):o;s&&Xm(s,n)}return n}const n_=re.mac?"mac":re.windows?"win":re.linux?"linux":"key";function r_(t,e){const n=t.split(/-(?!$)/);let r=n[n.length-1];r=="Space"&&(r=" ");let i,o,s,l;for(let a=0;a<n.length-1;++a){const c=n[a];if(/^(cmd|meta|m)$/i.test(c))l=!0;else if(/^a(lt)?$/i.test(c))i=!0;else if(/^(c|ctrl|control)$/i.test(c))o=!0;else if(/^s(hift)?$/i.test(c))s=!0;else if(/^mod$/i.test(c))e=="mac"?l=!0:o=!0;else throw new Error("Unrecognized modifier name: "+c)}return i&&(r="Alt-"+r),o&&(r="Ctrl-"+r),l&&(r="Meta-"+r),s&&(r="Shift-"+r),r}function gg(t,e,n){return e.altKey&&(t="Alt-"+t),e.ctrlKey&&(t="Ctrl-"+t),e.metaKey&&(t="Meta-"+t),n!==!1&&e.shiftKey&&(t="Shift-"+t),t}const i_=oe.domEventHandlers({keydown(t,e){return E$(I$(e.state),t,e,"editor")}}),cf=ne.define({enables:i_}),T$=new WeakMap;function I$(t){let e=t.facet(cf),n=T$.get(e);return n||T$.set(e,n=l_(e.reduce((r,i)=>r.concat(i),[]))),n}function o_(t,e,n){return E$(I$(t.state),e,t,n)}let xi=null;const s_=4e3;function l_(t,e=n_){let n=Object.create(null),r=Object.create(null),i=(s,l)=>{let a=r[s];if(a==null)r[s]=l;else if(a!=l)throw new Error("Key binding "+s+" is used both as a regular binding and as a multi-stroke prefix")},o=(s,l,a,c)=>{let u=n[s]||(n[s]=Object.create(null)),f=l.split(/ (?!$)/).map(p=>r_(p,e));for(let p=1;p<f.length;p++){let m=f.slice(0,p).join(" ");i(m,!0),u[m]||(u[m]={preventDefault:!0,commands:[g=>{let y=xi={view:g,prefix:m,scope:s};return setTimeout(()=>{xi==y&&(xi=null)},s_),!0}]})}let h=f.join(" ");i(h,!1);let d=u[h]||(u[h]={preventDefault:!1,commands:[]});d.commands.push(a),c&&(d.preventDefault=!0)};for(let s of t){let l=s[e]||s.key;if(!!l)for(let a of s.scope?s.scope.split(" "):["editor"])o(a,l,s.run,s.preventDefault),s.shift&&o(a,"Shift-"+l,s.shift,s.preventDefault)}return n}function E$(t,e,n,r){let i=V3(e),o=i.length==1&&i!=" ",s="",l=!1;xi&&xi.view==n&&xi.scope==r&&(s=xi.prefix+" ",(l=t$.indexOf(e.keyCode)<0)&&(xi=null));let a=f=>{if(f){for(let h of f.commands)if(h(n))return!0;f.preventDefault&&(l=!0)}return!1},c=t[r],u;if(c){if(a(c[s+gg(i,e,!o)]))return!0;if(o&&(e.shiftKey||e.altKey||e.metaKey)&&(u=Oi[e.keyCode])&&u!=i){if(a(c[s+gg(u,e,!0)]))return!0}else if(o&&e.shiftKey&&a(c[s+gg(i,e,!0)]))return!0}return l}const M$=!re.ios,ka=ne.define({combine(t){return mi(t,{cursorBlinkRate:1200,drawRangeCursor:!0},{cursorBlinkRate:(e,n)=>Math.min(e,n),drawRangeCursor:(e,n)=>e||n})}});function a_(t={}){return[ka.of(t),c_,u_]}class A${constructor(e,n,r,i,o){this.left=e,this.top=n,this.width=r,this.height=i,this.className=o}draw(){let e=document.createElement("div");return e.className=this.className,this.adjust(e),e}adjust(e){e.style.left=this.left+"px",e.style.top=this.top+"px",this.width>=0&&(e.style.width=this.width+"px"),e.style.height=this.height+"px"}eq(e){return this.left==e.left&&this.top==e.top&&this.width==e.width&&this.height==e.height&&this.className==e.className}}const c_=rt.fromClass(class{constructor(t){this.view=t,this.rangePieces=[],this.cursors=[],this.measureReq={read:this.readPos.bind(this),write:this.drawSel.bind(this)},this.selectionLayer=t.scrollDOM.appendChild(document.createElement("div")),this.selectionLayer.className="cm-selectionLayer",this.selectionLayer.setAttribute("aria-hidden","true"),this.cursorLayer=t.scrollDOM.appendChild(document.createElement("div")),this.cursorLayer.className="cm-cursorLayer",this.cursorLayer.setAttribute("aria-hidden","true"),t.requestMeasure(this.measureReq),this.setBlinkRate()}setBlinkRate(){this.cursorLayer.style.animationDuration=this.view.state.facet(ka).cursorBlinkRate+"ms"}update(t){let e=t.startState.facet(ka)!=t.state.facet(ka);(e||t.selectionSet||t.geometryChanged||t.viewportChanged)&&this.view.requestMeasure(this.measureReq),t.transactions.some(n=>n.scrollIntoView)&&(this.cursorLayer.style.animationName=this.cursorLayer.style.animationName=="cm-blink"?"cm-blink2":"cm-blink"),e&&this.setBlinkRate()}readPos(){let{state:t}=this.view,e=t.facet(ka),n=t.selection.ranges.map(i=>i.empty?[]:f_(this.view,i)).reduce((i,o)=>i.concat(o)),r=[];for(let i of t.selection.ranges){let o=i==t.selection.main;if(i.empty?!o||M$:e.drawRangeCursor){let s=h_(this.view,i,o);s&&r.push(s)}}return{rangePieces:n,cursors:r}}drawSel({rangePieces:t,cursors:e}){if(t.length!=this.rangePieces.length||t.some((n,r)=>!n.eq(this.rangePieces[r]))){this.selectionLayer.textContent="";for(let n of t)this.selectionLayer.appendChild(n.draw());this.rangePieces=t}if(e.length!=this.cursors.length||e.some((n,r)=>!n.eq(this.cursors[r]))){let n=this.cursorLayer.children;if(n.length!==e.length){this.cursorLayer.textContent="";for(const r of e)this.cursorLayer.appendChild(r.draw())}else e.forEach((r,i)=>r.adjust(n[i]));this.cursors=e}}destroy(){this.selectionLayer.remove(),this.cursorLayer.remove()}}),D$={".cm-line":{"& ::selection":{backgroundColor:"transparent !important"},"&::selection":{backgroundColor:"transparent !important"}}};M$&&(D$[".cm-line"].caretColor="transparent !important");const u_=da.highest(oe.theme(D$));function L$(t){let e=t.scrollDOM.getBoundingClientRect();return{left:(t.textDirection==Se.LTR?e.left:e.right-t.scrollDOM.clientWidth)-t.scrollDOM.scrollLeft,top:e.top-t.scrollDOM.scrollTop}}function W$(t,e,n){let r=_.cursor(e);return{from:Math.max(n.from,t.moveToLineBoundary(r,!1,!0).from),to:Math.min(n.to,t.moveToLineBoundary(r,!0,!0).from),type:De.Text}}function Q$(t,e){let n=t.lineBlockAt(e);if(Array.isArray(n.type)){for(let r of n.type)if(r.to>e||r.to==e&&(r.to==n.to||r.type==De.Text))return r}return n}function f_(t,e){if(e.to<=t.viewport.from||e.from>=t.viewport.to)return[];let n=Math.max(e.from,t.viewport.from),r=Math.min(e.to,t.viewport.to),i=t.textDirection==Se.LTR,o=t.contentDOM,s=o.getBoundingClientRect(),l=L$(t),a=window.getComputedStyle(o.firstChild),c=s.left+parseInt(a.paddingLeft)+Math.min(0,parseInt(a.textIndent)),u=s.right-parseInt(a.paddingRight),f=Q$(t,n),h=Q$(t,r),d=f.type==De.Text?f:null,p=h.type==De.Text?h:null;if(t.lineWrapping&&(d&&(d=W$(t,n,d)),p&&(p=W$(t,r,p))),d&&p&&d.from==p.from)return g(y(e.from,e.to,d));{let v=d?y(e.from,null,d):O(f,!1),x=p?y(null,e.to,p):O(h,!0),b=[];return(d||f).to<(p||h).from-1?b.push(m(c,v.bottom,u,x.top)):v.bottom<x.top&&t.elementAtHeight((v.bottom+x.top)/2).type==De.Text&&(v.bottom=x.top=(v.bottom+x.top)/2),g(v).concat(b).concat(g(x))}function m(v,x,b,S){return new A$(v-l.left,x-l.top,b-v,S-x,"cm-selectionBackground")}function g({top:v,bottom:x,horizontal:b}){let S=[];for(let k=0;k<b.length;k+=2)S.push(m(b[k],v,b[k+1],x));return S}function y(v,x,b){let S=1e9,k=-1e9,P=[];function C(L,M,q,F,Q){let E=t.coordsAtPos(L,L==b.to?-2:2),D=t.coordsAtPos(q,q==b.from?2:-2);S=Math.min(E.top,D.top,S),k=Math.max(E.bottom,D.bottom,k),Q==Se.LTR?P.push(i&&M?c:E.left,i&&F?u:D.right):P.push(!i&&F?c:D.left,!i&&M?u:E.right)}let R=v!=null?v:b.from,T=x!=null?x:b.to;for(let L of t.visibleRanges)if(L.to>R&&L.from<T)for(let M=Math.max(L.from,R),q=Math.min(L.to,T);;){let F=t.state.doc.lineAt(M);for(let Q of t.bidiSpans(F)){let E=Q.from+F.from,D=Q.to+F.from;if(E>=q)break;D>M&&C(Math.max(E,M),v==null&&E<=R,Math.min(D,q),x==null&&D>=T,Q.dir)}if(M=F.to+1,M>=q)break}return P.length==0&&C(R,v==null,T,x==null,t.textDirection),{top:S,bottom:k,horizontal:P}}function O(v,x){let b=s.top+(x?v.top:v.bottom);return{top:b,bottom:b,horizontal:[]}}}function h_(t,e,n){let r=t.coordsAtPos(e.head,e.assoc||1);if(!r)return null;let i=L$(t);return new A$(r.left-i.left,r.top-i.top,-1,r.bottom-r.top,n?"cm-cursor cm-cursor-primary":"cm-cursor cm-cursor-secondary")}const N$=ce.define({map(t,e){return t==null?null:e.mapPos(t)}}),Pa=Rt.define({create(){return null},update(t,e){return t!=null&&(t=e.changes.mapPos(t)),e.effects.reduce((n,r)=>r.is(N$)?r.value:n,t)}}),d_=rt.fromClass(class{constructor(t){this.view=t,this.cursor=null,this.measureReq={read:this.readPos.bind(this),write:this.drawCursor.bind(this)}}update(t){var e;let n=t.state.field(Pa);n==null?this.cursor!=null&&((e=this.cursor)===null||e===void 0||e.remove(),this.cursor=null):(this.cursor||(this.cursor=this.view.scrollDOM.appendChild(document.createElement("div")),this.cursor.className="cm-dropCursor"),(t.startState.field(Pa)!=n||t.docChanged||t.geometryChanged)&&this.view.requestMeasure(this.measureReq))}readPos(){let t=this.view.state.field(Pa),e=t!=null&&this.view.coordsAtPos(t);if(!e)return null;let n=this.view.scrollDOM.getBoundingClientRect();return{left:e.left-n.left+this.view.scrollDOM.scrollLeft,top:e.top-n.top+this.view.scrollDOM.scrollTop,height:e.bottom-e.top}}drawCursor(t){this.cursor&&(t?(this.cursor.style.left=t.left+"px",this.cursor.style.top=t.top+"px",this.cursor.style.height=t.height+"px"):this.cursor.style.left="-100000px")}destroy(){this.cursor&&this.cursor.remove()}setDropPos(t){this.view.state.field(Pa)!=t&&this.view.dispatch({effects:N$.of(t)})}},{eventHandlers:{dragover(t){this.setDropPos(this.view.posAtCoords({x:t.clientX,y:t.clientY}))},dragleave(t){(t.target==this.view.contentDOM||!this.view.contentDOM.contains(t.relatedTarget))&&this.setDropPos(null)},dragend(){this.setDropPos(null)},drop(){this.setDropPos(null)}}});function p_(){return[Pa,d_]}function _$(t,e,n,r,i){e.lastIndex=0;for(let o=t.iterRange(n,r),s=n,l;!o.next().done;s+=o.value.length)if(!o.lineBreak)for(;l=e.exec(o.value);)i(s+l.index,s+l.index+l[0].length,l)}function m_(t,e){let n=t.visibleRanges;if(n.length==1&&n[0].from==t.viewport.from&&n[0].to==t.viewport.to)return n;let r=[];for(let{from:i,to:o}of n)i=Math.max(t.state.doc.lineAt(i).from,i-e),o=Math.min(t.state.doc.lineAt(o).to,o+e),r.length&&r[r.length-1].to>=i?r[r.length-1].to=o:r.push({from:i,to:o});return r}class g_{constructor(e){let{regexp:n,decoration:r,boundary:i,maxLength:o=1e3}=e;if(!n.global)throw new RangeError("The regular expression given to MatchDecorator should have its 'g' flag set");this.regexp=n,this.getDeco=typeof r=="function"?r:()=>r,this.boundary=i,this.maxLength=o}createDeco(e){let n=new yi;for(let{from:r,to:i}of m_(e,this.maxLength))_$(e.state.doc,this.regexp,r,i,(o,s,l)=>n.add(o,s,this.getDeco(l,e,o)));return n.finish()}updateDeco(e,n){let r=1e9,i=-1;return e.docChanged&&e.changes.iterChanges((o,s,l,a)=>{a>e.view.viewport.from&&l<e.view.viewport.to&&(r=Math.min(l,r),i=Math.max(a,i))}),e.viewportChanged||i-r>1e3?this.createDeco(e.view):i>-1?this.updateRange(e.view,n.map(e.changes),r,i):n}updateRange(e,n,r,i){for(let o of e.visibleRanges){let s=Math.max(o.from,r),l=Math.min(o.to,i);if(l>s){let a=e.state.doc.lineAt(s),c=a.to<l?e.state.doc.lineAt(l):a,u=Math.max(o.from,a.from),f=Math.min(o.to,c.to);if(this.boundary){for(;s>a.from;s--)if(this.boundary.test(a.text[s-1-a.from])){u=s;break}for(;l<c.to;l++)if(this.boundary.test(c.text[l-c.from])){f=l;break}}let h=[],d;if(a==c)for(this.regexp.lastIndex=u-a.from;(d=this.regexp.exec(a.text))&&d.index<f-a.from;){let p=d.index+a.from;h.push(this.getDeco(d,e,p).range(p,p+d[0].length))}else _$(e.state.doc,this.regexp,u,f,(p,m,g)=>h.push(this.getDeco(g,e,p).range(p,m)));n=n.update({filterFrom:u,filterTo:f,filter:(p,m)=>p<u||m>f,add:h})}}return n}}const yg=/x/.unicode!=null?"gu":"g",y_=new RegExp(`[\0-\b
-\x7F-\x9F\xAD\u061C\u200B\u200E\u200F\u2028\u2029\u202D\u202E\uFEFF\uFFF9-\uFFFC]`,yg),O_={0:"null",7:"bell",8:"backspace",10:"newline",11:"vertical tab",13:"carriage return",27:"escape",8203:"zero width space",8204:"zero width non-joiner",8205:"zero width joiner",8206:"left-to-right mark",8207:"right-to-left mark",8232:"line separator",8237:"left-to-right override",8238:"right-to-left override",8233:"paragraph separator",65279:"zero width no-break space",65532:"object replacement"};let Og=null;function v_(){var t;if(Og==null&&typeof document!="undefined"&&document.body){let e=document.body.style;Og=((t=e.tabSize)!==null&&t!==void 0?t:e.MozTabSize)!=null}return Og||!1}const uf=ne.define({combine(t){let e=mi(t,{render:null,specialChars:y_,addSpecialChars:null});return(e.replaceTabs=!v_())&&(e.specialChars=new RegExp("	|"+e.specialChars.source,yg)),e.addSpecialChars&&(e.specialChars=new RegExp(e.specialChars.source+"|"+e.addSpecialChars.source,yg)),e}});function x_(t={}){return[uf.of(t),w_()]}let B$=null;function w_(){return B$||(B$=rt.fromClass(class{constructor(t){this.view=t,this.decorations=te.none,this.decorationCache=Object.create(null),this.decorator=this.makeDecorator(t.state.facet(uf)),this.decorations=this.decorator.createDeco(t)}makeDecorator(t){return new g_({regexp:t.specialChars,decoration:(e,n,r)=>{let{doc:i}=n.state,o=Ct(e[0],0);if(o==9){let s=i.lineAt(r),l=n.state.tabSize,a=ua(s.text,l,r-s.from);return te.replace({widget:new k_((l-a%l)*this.view.defaultCharacterWidth)})}return this.decorationCache[o]||(this.decorationCache[o]=te.replace({widget:new S_(t,o)}))},boundary:t.replaceTabs?void 0:/[^]/})}update(t){let e=t.state.facet(uf);t.startState.facet(uf)!=e?(this.decorator=this.makeDecorator(e),this.decorations=this.decorator.createDeco(t.view)):this.decorations=this.decorator.updateDeco(t,this.decorations)}},{decorations:t=>t.decorations}))}const b_="\u2022";function $_(t){return t>=32?b_:t==10?"\u2424":String.fromCharCode(9216+t)}class S_ extends Cr{constructor(e,n){super();this.options=e,this.code=n}eq(e){return e.code==this.code}toDOM(e){let n=$_(this.code),r=e.state.phrase("Control character")+" "+(O_[this.code]||"0x"+this.code.toString(16)),i=this.options.render&&this.options.render(this.code,r,n);if(i)return i;let o=document.createElement("span");return o.textContent=n,o.title=r,o.setAttribute("aria-label",r),o.className="cm-specialChar",o}ignoreEvent(){return!1}}class k_ extends Cr{constructor(e){super();this.width=e}eq(e){return e.width==this.width}toDOM(){let e=document.createElement("span");return e.textContent="	",e.className="cm-tab",e.style.width=this.width+"px",e}ignoreEvent(){return!1}}function P_(){return R_}const C_=te.line({class:"cm-activeLine"}),R_=rt.fromClass(class{constructor(t){this.decorations=this.getDeco(t)}update(t){(t.docChanged||t.selectionSet)&&(this.decorations=this.getDeco(t.view))}getDeco(t){let e=-1,n=[];for(let r of t.state.selection.ranges){if(!r.empty)return te.none;let i=t.lineBlockAt(r.head);i.from>e&&(n.push(C_.range(i.from)),e=i.from)}return te.set(n)}},{decorations:t=>t.decorations});class T_ extends Cr{constructor(e){super();this.content=e}toDOM(){let e=document.createElement("span");return e.className="cm-placeholder",e.style.pointerEvents="none",e.appendChild(typeof this.content=="string"?document.createTextNode(this.content):this.content),typeof this.content=="string"?e.setAttribute("aria-label","placeholder "+this.content):e.setAttribute("aria-hidden","true"),e}ignoreEvent(){return!1}}function I_(t){return rt.fromClass(class{constructor(e){this.view=e,this.placeholder=te.set([te.widget({widget:new T_(t),side:1}).range(0)])}get decorations(){return this.view.state.doc.length?te.none:this.placeholder}},{decorations:e=>e.decorations})}const E_=typeof navigator!="undefined"&&!/Edge\/(\d+)/.exec(navigator.userAgent)&&/Apple Computer/.test(navigator.vendor)&&(/Mobile\/\w+/.test(navigator.userAgent)||navigator.maxTouchPoints>2),vg="-10000px";class q${constructor(e,n,r){this.facet=n,this.createTooltipView=r,this.input=e.state.facet(n),this.tooltips=this.input.filter(i=>i),this.tooltipViews=this.tooltips.map(r)}update(e){let n=e.state.facet(this.facet),r=n.filter(o=>o);if(n===this.input){for(let o of this.tooltipViews)o.update&&o.update(e);return!1}let i=[];for(let o=0;o<r.length;o++){let s=r[o],l=-1;if(!!s){for(let a=0;a<this.tooltips.length;a++){let c=this.tooltips[a];c&&c.create==s.create&&(l=a)}if(l<0)i[o]=this.createTooltipView(s);else{let a=i[o]=this.tooltipViews[l];a.update&&a.update(e)}}}for(let o of this.tooltipViews)i.indexOf(o)<0&&o.dom.remove();return this.input=n,this.tooltips=r,this.tooltipViews=i,!0}}function M_(){return{top:0,left:0,bottom:innerHeight,right:innerWidth}}const xg=ne.define({combine:t=>{var e,n,r;return{position:E_?"absolute":((e=t.find(i=>i.position))===null||e===void 0?void 0:e.position)||"fixed",parent:((n=t.find(i=>i.parent))===null||n===void 0?void 0:n.parent)||null,tooltipSpace:((r=t.find(i=>i.tooltipSpace))===null||r===void 0?void 0:r.tooltipSpace)||M_}}}),z$=rt.fromClass(class{constructor(t){var e;this.view=t,this.inView=!0,this.lastTransaction=0,this.measureTimeout=-1;let n=t.state.facet(xg);this.position=n.position,this.parent=n.parent,this.classes=t.themeClasses,this.createContainer(),this.measureReq={read:this.readMeasure.bind(this),write:this.writeMeasure.bind(this),key:this},this.manager=new q$(t,wg,r=>this.createTooltip(r)),this.intersectionObserver=typeof IntersectionObserver=="function"?new IntersectionObserver(r=>{Date.now()>this.lastTransaction-50&&r.length>0&&r[r.length-1].intersectionRatio<1&&this.measureSoon()},{threshold:[1]}):null,this.observeIntersection(),(e=t.dom.ownerDocument.defaultView)===null||e===void 0||e.addEventListener("resize",this.measureSoon=this.measureSoon.bind(this)),this.maybeMeasure()}createContainer(){this.parent?(this.container=document.createElement("div"),this.container.style.position="relative",this.container.className=this.view.themeClasses,this.parent.appendChild(this.container)):this.container=this.view.dom}observeIntersection(){if(this.intersectionObserver){this.intersectionObserver.disconnect();for(let t of this.manager.tooltipViews)this.intersectionObserver.observe(t.dom)}}measureSoon(){this.measureTimeout<0&&(this.measureTimeout=setTimeout(()=>{this.measureTimeout=-1,this.maybeMeasure()},50))}update(t){t.transactions.length&&(this.lastTransaction=Date.now());let e=this.manager.update(t);e&&this.observeIntersection();let n=e||t.geometryChanged,r=t.state.facet(xg);if(r.position!=this.position){this.position=r.position;for(let i of this.manager.tooltipViews)i.dom.style.position=this.position;n=!0}if(r.parent!=this.parent){this.parent&&this.container.remove(),this.parent=r.parent,this.createContainer();for(let i of this.manager.tooltipViews)this.container.appendChild(i.dom);n=!0}else this.parent&&this.view.themeClasses!=this.classes&&(this.classes=this.container.className=this.view.themeClasses);n&&this.maybeMeasure()}createTooltip(t){let e=t.create(this.view);if(e.dom.classList.add("cm-tooltip"),t.arrow&&!e.dom.querySelector("cm-tooltip > cm-tooltip-arrow")){let n=document.createElement("div");n.className="cm-tooltip-arrow",e.dom.appendChild(n)}return e.dom.style.position=this.position,e.dom.style.top=vg,this.container.appendChild(e.dom),e.mount&&e.mount(this.view),e}destroy(){var t,e;(t=this.view.dom.ownerDocument.defaultView)===null||t===void 0||t.removeEventListener("resize",this.measureSoon);for(let{dom:n}of this.manager.tooltipViews)n.remove();(e=this.intersectionObserver)===null||e===void 0||e.disconnect(),clearTimeout(this.measureTimeout)}readMeasure(){let t=this.view.dom.getBoundingClientRect();return{editor:t,parent:this.parent?this.container.getBoundingClientRect():t,pos:this.manager.tooltips.map(e=>this.view.coordsAtPos(e.pos)),size:this.manager.tooltipViews.map(({dom:e})=>e.getBoundingClientRect()),space:this.view.state.facet(xg).tooltipSpace(this.view)}}writeMeasure(t){let{editor:e,space:n}=t,r=[];for(let i=0;i<this.manager.tooltips.length;i++){let o=this.manager.tooltips[i],s=this.manager.tooltipViews[i],{dom:l}=s,a=t.pos[i],c=t.size[i];if(!a||a.bottom<=Math.max(e.top,n.top)||a.top>=Math.min(e.bottom,n.bottom)||a.right<=Math.max(e.left,n.left)||a.left>=Math.min(e.right,n.right)){l.style.top=vg;continue}let u=o.arrow?s.dom.querySelector(".cm-tooltip-arrow"):null,f=u?7:0,h=c.right-c.left,d=c.bottom-c.top,p=s.offset||D_,m=this.view.textDirection==Se.LTR,g=c.width>n.right-n.left?m?n.left:n.right-c.width:m?Math.min(a.left-(u?14:0)+p.x,n.right-h):Math.max(n.left,a.left-h+(u?14:0)-p.x),y=!!o.above;!o.strictSide&&(y?a.top-(c.bottom-c.top)-p.y<n.top:a.bottom+(c.bottom-c.top)+p.y>n.bottom)&&y==n.bottom-a.bottom>a.top-n.top&&(y=!y);let O=y?a.top-d-f-p.y:a.bottom+f+p.y,v=g+h;for(let x of r)x.left<v&&x.right>g&&x.top<O+d&&x.bottom>O&&(O=y?x.top-d-2-f:x.bottom+f+2);this.position=="absolute"?(l.style.top=O-t.parent.top+"px",l.style.left=g-t.parent.left+"px"):(l.style.top=O+"px",l.style.left=g+"px"),u&&(u.style.left=`${a.left+(m?p.x:-p.x)-(g+14-7)}px`),r.push({left:g,top:O,right:v,bottom:O+d}),l.classList.toggle("cm-tooltip-above",y),l.classList.toggle("cm-tooltip-below",!y),s.positioned&&s.positioned()}}maybeMeasure(){if(this.manager.tooltips.length&&(this.view.inView&&this.view.requestMeasure(this.measureReq),this.inView!=this.view.inView&&(this.inView=this.view.inView,!this.inView)))for(let t of this.manager.tooltipViews)t.dom.style.top=vg}},{eventHandlers:{scroll(){this.maybeMeasure()}}}),A_=oe.baseTheme({".cm-tooltip":{zIndex:100},"&light .cm-tooltip":{border:"1px solid #bbb",backgroundColor:"#f5f5f5"},"&light .cm-tooltip-section:not(:first-child)":{borderTop:"1px solid #bbb"},"&dark .cm-tooltip":{backgroundColor:"#333338",color:"white"},".cm-tooltip-arrow":{height:`${7}px`,width:`${7*2}px`,position:"absolute",zIndex:-1,overflow:"hidden","&:before, &:after":{content:"''",position:"absolute",width:0,height:0,borderLeft:`${7}px solid transparent`,borderRight:`${7}px solid transparent`},".cm-tooltip-above &":{bottom:`-${7}px`,"&:before":{borderTop:`${7}px solid #bbb`},"&:after":{borderTop:`${7}px solid #f5f5f5`,bottom:"1px"}},".cm-tooltip-below &":{top:`-${7}px`,"&:before":{borderBottom:`${7}px solid #bbb`},"&:after":{borderBottom:`${7}px solid #f5f5f5`,top:"1px"}}},"&dark .cm-tooltip .cm-tooltip-arrow":{"&:before":{borderTopColor:"#333338",borderBottomColor:"#333338"},"&:after":{borderTopColor:"transparent",borderBottomColor:"transparent"}}}),D_={x:0,y:0},wg=ne.define({enables:[z$,A_]}),ff=ne.define();class bg{constructor(e){this.view=e,this.mounted=!1,this.dom=document.createElement("div"),this.dom.classList.add("cm-tooltip-hover"),this.manager=new q$(e,ff,n=>this.createHostedView(n))}static create(e){return new bg(e)}createHostedView(e){let n=e.create(this.view);return n.dom.classList.add("cm-tooltip-section"),this.dom.appendChild(n.dom),this.mounted&&n.mount&&n.mount(this.view),n}mount(e){for(let n of this.manager.tooltipViews)n.mount&&n.mount(e);this.mounted=!0}positioned(){for(let e of this.manager.tooltipViews)e.positioned&&e.positioned()}update(e){this.manager.update(e)}}const L_=wg.compute([ff],t=>{let e=t.facet(ff).filter(n=>n);return e.length===0?null:{pos:Math.min(...e.map(n=>n.pos)),end:Math.max(...e.filter(n=>n.end!=null).map(n=>n.end)),create:bg.create,above:e[0].above,arrow:e.some(n=>n.arrow)}});class W_{constructor(e,n,r,i,o){this.view=e,this.source=n,this.field=r,this.setHover=i,this.hoverTime=o,this.hoverTimeout=-1,this.restartTimeout=-1,this.pending=null,this.lastMove={x:0,y:0,target:e.dom,time:0},this.checkHover=this.checkHover.bind(this),e.dom.addEventListener("mouseleave",this.mouseleave=this.mouseleave.bind(this)),e.dom.addEventListener("mousemove",this.mousemove=this.mousemove.bind(this))}update(){this.pending&&(this.pending=null,clearTimeout(this.restartTimeout),this.restartTimeout=setTimeout(()=>this.startHover(),20))}get active(){return this.view.state.field(this.field)}checkHover(){if(this.hoverTimeout=-1,this.active)return;let e=Date.now()-this.lastMove.time;e<this.hoverTime?this.hoverTimeout=setTimeout(this.checkHover,this.hoverTime-e):this.startHover()}startHover(){var e;clearTimeout(this.restartTimeout);let{lastMove:n}=this,r=this.view.contentDOM.contains(n.target)?this.view.posAtCoords(n):null;if(r==null)return;let i=this.view.coordsAtPos(r);if(i==null||n.y<i.top||n.y>i.bottom||n.x<i.left-this.view.defaultCharacterWidth||n.x>i.right+this.view.defaultCharacterWidth)return;let o=this.view.bidiSpans(this.view.state.doc.lineAt(r)).find(a=>a.from<=r&&a.to>=r),s=o&&o.dir==Se.RTL?-1:1,l=this.source(this.view,r,n.x<i.left?-s:s);if((e=l)===null||e===void 0?void 0:e.then){let a=this.pending={pos:r};l.then(c=>{this.pending==a&&(this.pending=null,c&&this.view.dispatch({effects:this.setHover.of(c)}))},c=>bn(this.view.state,c,"hover tooltip"))}else l&&this.view.dispatch({effects:this.setHover.of(l)})}mousemove(e){var n;this.lastMove={x:e.clientX,y:e.clientY,target:e.target,time:Date.now()},this.hoverTimeout<0&&(this.hoverTimeout=setTimeout(this.checkHover,this.hoverTime));let r=this.active;if(r&&!Q_(this.lastMove.target)||this.pending){let{pos:i}=r||this.pending,o=(n=r==null?void 0:r.end)!==null&&n!==void 0?n:i;(i==o?this.view.posAtCoords(this.lastMove)!=i:!N_(this.view,i,o,e.clientX,e.clientY,6))&&(this.view.dispatch({effects:this.setHover.of(null)}),this.pending=null)}}mouseleave(){clearTimeout(this.hoverTimeout),this.hoverTimeout=-1,this.active&&this.view.dispatch({effects:this.setHover.of(null)})}destroy(){clearTimeout(this.hoverTimeout),this.view.dom.removeEventListener("mouseleave",this.mouseleave),this.view.dom.removeEventListener("mousemove",this.mousemove)}}function Q_(t){for(let e=t;e;e=e.parentNode)if(e.nodeType==1&&e.classList.contains("cm-tooltip"))return!0;return!1}function N_(t,e,n,r,i,o){let s=document.createRange(),l=t.domAtPos(e),a=t.domAtPos(n);s.setEnd(a.node,a.offset),s.setStart(l.node,l.offset);let c=s.getClientRects();s.detach();for(let u=0;u<c.length;u++){let f=c[u];if(Math.max(f.top-i,i-f.bottom,f.left-r,r-f.right)<=o)return!0}return!1}function __(t,e={}){let n=ce.define(),r=Rt.define({create(){return null},update(o,s){if(o&&e.hideOnChange&&(s.docChanged||s.selection))return null;for(let l of s.effects){if(l.is(n))return l.value;if(l.is(q_))return null}if(o&&s.docChanged){let l=s.changes.mapPos(o.pos,-1,Dt.TrackDel);if(l==null)return null;let a=Object.assign(Object.create(null),o);return a.pos=l,o.end!=null&&(a.end=s.changes.mapPos(o.end)),a}return o},provide:o=>ff.from(o)}),i=e.hoverTime||600;return[r,rt.define(o=>new W_(o,t,r,n,i)),L_]}function B_(t,e){let n=t.plugin(z$);if(!n)return null;let r=n.manager.tooltips.indexOf(e);return r<0?null:n.manager.tooltipViews[r]}const q_=ce.define(),F$=1024;let z_=0;class $g{constructor(e,n){this.from=e,this.to=n}}class xe{constructor(e={}){this.id=z_++,this.perNode=!!e.perNode,this.deserialize=e.deserialize||(()=>{throw new Error("This node type doesn't define a deserialize function")})}add(e){if(this.perNode)throw new RangeError("Can't add per-node props to node types");return typeof e!="function"&&(e=ln.match(e)),n=>{let r=e(n);return r===void 0?null:[this,r]}}}xe.closedBy=new xe({deserialize:t=>t.split(" ")});xe.openedBy=new xe({deserialize:t=>t.split(" ")});xe.group=new xe({deserialize:t=>t.split(" ")});xe.contextHash=new xe({perNode:!0});xe.lookAhead=new xe({perNode:!0});xe.mounted=new xe({perNode:!0});const F_=Object.create(null);class ln{constructor(e,n,r,i=0){this.name=e,this.props=n,this.id=r,this.flags=i}static define(e){let n=e.props&&e.props.length?Object.create(null):F_,r=(e.top?1:0)|(e.skipped?2:0)|(e.error?4:0)|(e.name==null?8:0),i=new ln(e.name||"",n,e.id,r);if(e.props){for(let o of e.props)if(Array.isArray(o)||(o=o(i)),o){if(o[0].perNode)throw new RangeError("Can't store a per-node prop on a node type");n[o[0].id]=o[1]}}return i}prop(e){return this.props[e.id]}get isTop(){return(this.flags&1)>0}get isSkipped(){return(this.flags&2)>0}get isError(){return(this.flags&4)>0}get isAnonymous(){return(this.flags&8)>0}is(e){if(typeof e=="string"){if(this.name==e)return!0;let n=this.prop(xe.group);return n?n.indexOf(e)>-1:!1}return this.id==e}static match(e){let n=Object.create(null);for(let r in e)for(let i of r.split(" "))n[i]=e[r];return r=>{for(let i=r.prop(xe.group),o=-1;o<(i?i.length:0);o++){let s=n[o<0?r.name:i[o]];if(s)return s}}}}ln.none=new ln("",Object.create(null),0,8);class Sg{constructor(e){this.types=e;for(let n=0;n<e.length;n++)if(e[n].id!=n)throw new RangeError("Node type ids should correspond to array positions when creating a node set")}extend(...e){let n=[];for(let r of this.types){let i=null;for(let o of e){let s=o(r);s&&(i||(i=Object.assign({},r.props)),i[s[0].id]=s[1])}n.push(i?new ln(r.name,i,r.id,r.flags):r)}return new Sg(n)}}const hf=new WeakMap,V$=new WeakMap;class et{constructor(e,n,r,i,o){if(this.type=e,this.children=n,this.positions=r,this.length=i,this.props=null,o&&o.length){this.props=Object.create(null);for(let[s,l]of o)this.props[typeof s=="number"?s:s.id]=l}}toString(){let e=this.prop(xe.mounted);if(e&&!e.overlay)return e.tree.toString();let n="";for(let r of this.children){let i=r.toString();i&&(n&&(n+=","),n+=i)}return this.type.name?(/\W/.test(this.type.name)&&!this.type.isError?JSON.stringify(this.type.name):this.type.name)+(n.length?"("+n+")":""):n}cursor(e,n=0){let r=e!=null&&hf.get(this)||this.topNode,i=new pf(r);return e!=null&&(i.moveTo(e,n),hf.set(this,i._tree)),i}fullCursor(){return new pf(this.topNode,1)}get topNode(){return new Ir(this,0,0,null)}resolve(e,n=0){let r=Os(hf.get(this)||this.topNode,e,n,!1);return hf.set(this,r),r}resolveInner(e,n=0){let r=Os(V$.get(this)||this.topNode,e,n,!0);return V$.set(this,r),r}iterate(e){let{enter:n,leave:r,from:i=0,to:o=this.length}=e;for(let s=this.cursor(),l=()=>s.node;;){let a=!1;if(s.from<=o&&s.to>=i&&(s.type.isAnonymous||n(s.type,s.from,s.to,l)!==!1)){if(s.firstChild())continue;s.type.isAnonymous||(a=!0)}for(;a&&r&&r(s.type,s.from,s.to,l),a=s.type.isAnonymous,!s.nextSibling();){if(!s.parent())return;a=!0}}}prop(e){return e.perNode?this.props?this.props[e.id]:void 0:this.type.prop(e)}get propValues(){let e=[];if(this.props)for(let n in this.props)e.push([+n,this.props[n]]);return e}balance(e={}){return this.children.length<=8?this:Cg(ln.none,this.children,this.positions,0,this.children.length,0,this.length,(n,r,i)=>new et(this.type,n,r,i,this.propValues),e.makeTree||((n,r,i)=>new et(ln.none,n,r,i)))}static build(e){return U_(e)}}et.empty=new et(ln.none,[],[],0);class kg{constructor(e,n){this.buffer=e,this.index=n}get id(){return this.buffer[this.index-4]}get start(){return this.buffer[this.index-3]}get end(){return this.buffer[this.index-2]}get size(){return this.buffer[this.index-1]}get pos(){return this.index}next(){this.index-=4}fork(){return new kg(this.buffer,this.index)}}class to{constructor(e,n,r){this.buffer=e,this.length=n,this.set=r}get type(){return ln.none}toString(){let e=[];for(let n=0;n<this.buffer.length;)e.push(this.childString(n)),n=this.buffer[n+3];return e.join(",")}childString(e){let n=this.buffer[e],r=this.buffer[e+3],i=this.set.types[n],o=i.name;if(/\W/.test(o)&&!i.isError&&(o=JSON.stringify(o)),e+=4,r==e)return o;let s=[];for(;e<r;)s.push(this.childString(e)),e=this.buffer[e+3];return o+"("+s.join(",")+")"}findChild(e,n,r,i,o){let{buffer:s}=this,l=-1;for(let a=e;a!=n&&!(U$(o,i,s[a+1],s[a+2])&&(l=a,r>0));a=s[a+3]);return l}slice(e,n,r,i){let o=this.buffer,s=new Uint16Array(n-e);for(let l=e,a=0;l<n;)s[a++]=o[l++],s[a++]=o[l++]-r,s[a++]=o[l++]-r,s[a++]=o[l++]-e;return new to(s,i-r,this.set)}}function U$(t,e,n,r){switch(t){case-2:return n<e;case-1:return r>=e&&n<e;case 0:return n<e&&r>e;case 1:return n<=e&&r>e;case 2:return r>e;case 4:return!0}}function j$(t,e){let n=t.childBefore(e);for(;n;){let r=n.lastChild;if(!r||r.to!=n.to)break;r.type.isError&&r.from==r.to?(t=n,n=r.prevSibling):n=r}return t}function Os(t,e,n,r){for(var i;t.from==t.to||(n<1?t.from>=e:t.from>e)||(n>-1?t.to<=e:t.to<e);){let o=!r&&t instanceof Ir&&t.index<0?null:t.parent;if(!o)return t;t=o}if(r)for(let o=t,s=o.parent;s;o=s,s=o.parent)o instanceof Ir&&o.index<0&&((i=s.enter(e,n,!0))===null||i===void 0?void 0:i.from)!=o.from&&(t=s);for(;;){let o=t.enter(e,n,r);if(!o)return t;t=o}}class Ir{constructor(e,n,r,i){this.node=e,this._from=n,this.index=r,this._parent=i}get type(){return this.node.type}get name(){return this.node.type.name}get from(){return this._from}get to(){return this._from+this.node.length}nextChild(e,n,r,i,o=0){for(let s=this;;){for(let{children:l,positions:a}=s.node,c=n>0?l.length:-1;e!=c;e+=n){let u=l[e],f=a[e]+s._from;if(!!U$(i,r,f,f+u.length)){if(u instanceof to){if(o&2)continue;let h=u.findChild(0,u.buffer.length,n,r-f,i);if(h>-1)return new wi(new V_(s,u,e,f),null,h)}else if(o&1||!u.type.isAnonymous||Pg(u)){let h;if(!(o&1)&&u.props&&(h=u.prop(xe.mounted))&&!h.overlay)return new Ir(h.tree,f,e,s);let d=new Ir(u,f,e,s);return o&1||!d.type.isAnonymous?d:d.nextChild(n<0?u.children.length-1:0,n,r,i)}}}if(o&1||!s.type.isAnonymous||(s.index>=0?e=s.index+n:e=n<0?-1:s._parent.node.children.length,s=s._parent,!s))return null}}get firstChild(){return this.nextChild(0,1,0,4)}get lastChild(){return this.nextChild(this.node.children.length-1,-1,0,4)}childAfter(e){return this.nextChild(0,1,e,2)}childBefore(e){return this.nextChild(this.node.children.length-1,-1,e,-2)}enter(e,n,r=!0,i=!0){let o;if(r&&(o=this.node.prop(xe.mounted))&&o.overlay){let s=e-this.from;for(let{from:l,to:a}of o.overlay)if((n>0?l<=s:l<s)&&(n<0?a>=s:a>s))return new Ir(o.tree,o.overlay[0].from+this.from,-1,this)}return this.nextChild(0,1,e,n,i?0:2)}nextSignificantParent(){let e=this;for(;e.type.isAnonymous&&e._parent;)e=e._parent;return e}get parent(){return this._parent?this._parent.nextSignificantParent():null}get nextSibling(){return this._parent&&this.index>=0?this._parent.nextChild(this.index+1,1,0,4):null}get prevSibling(){return this._parent&&this.index>=0?this._parent.nextChild(this.index-1,-1,0,4):null}get cursor(){return new pf(this)}get tree(){return this.node}toTree(){return this.node}resolve(e,n=0){return Os(this,e,n,!1)}resolveInner(e,n=0){return Os(this,e,n,!0)}enterUnfinishedNodesBefore(e){return j$(this,e)}getChild(e,n=null,r=null){let i=df(this,e,n,r);return i.length?i[0]:null}getChildren(e,n=null,r=null){return df(this,e,n,r)}toString(){return this.node.toString()}}function df(t,e,n,r){let i=t.cursor,o=[];if(!i.firstChild())return o;if(n!=null){for(;!i.type.is(n);)if(!i.nextSibling())return o}for(;;){if(r!=null&&i.type.is(r))return o;if(i.type.is(e)&&o.push(i.node),!i.nextSibling())return r==null?o:[]}}class V_{constructor(e,n,r,i){this.parent=e,this.buffer=n,this.index=r,this.start=i}}class wi{constructor(e,n,r){this.context=e,this._parent=n,this.index=r,this.type=e.buffer.set.types[e.buffer.buffer[r]]}get name(){return this.type.name}get from(){return this.context.start+this.context.buffer.buffer[this.index+1]}get to(){return this.context.start+this.context.buffer.buffer[this.index+2]}child(e,n,r){let{buffer:i}=this.context,o=i.findChild(this.index+4,i.buffer[this.index+3],e,n-this.context.start,r);return o<0?null:new wi(this.context,this,o)}get firstChild(){return this.child(1,0,4)}get lastChild(){return this.child(-1,0,4)}childAfter(e){return this.child(1,e,2)}childBefore(e){return this.child(-1,e,-2)}enter(e,n,r,i=!0){if(!i)return null;let{buffer:o}=this.context,s=o.findChild(this.index+4,o.buffer[this.index+3],n>0?1:-1,e-this.context.start,n);return s<0?null:new wi(this.context,this,s)}get parent(){return this._parent||this.context.parent.nextSignificantParent()}externalSibling(e){return this._parent?null:this.context.parent.nextChild(this.context.index+e,e,0,4)}get nextSibling(){let{buffer:e}=this.context,n=e.buffer[this.index+3];return n<(this._parent?e.buffer[this._parent.index+3]:e.buffer.length)?new wi(this.context,this._parent,n):this.externalSibling(1)}get prevSibling(){let{buffer:e}=this.context,n=this._parent?this._parent.index+4:0;return this.index==n?this.externalSibling(-1):new wi(this.context,this._parent,e.findChild(n,this.index,-1,0,4))}get cursor(){return new pf(this)}get tree(){return null}toTree(){let e=[],n=[],{buffer:r}=this.context,i=this.index+4,o=r.buffer[this.index+3];if(o>i){let s=r.buffer[this.index+1],l=r.buffer[this.index+2];e.push(r.slice(i,o,s,l)),n.push(0)}return new et(this.type,e,n,this.to-this.from)}resolve(e,n=0){return Os(this,e,n,!1)}resolveInner(e,n=0){return Os(this,e,n,!0)}enterUnfinishedNodesBefore(e){return j$(this,e)}toString(){return this.context.buffer.childString(this.index)}getChild(e,n=null,r=null){let i=df(this,e,n,r);return i.length?i[0]:null}getChildren(e,n=null,r=null){return df(this,e,n,r)}}class pf{constructor(e,n=0){if(this.mode=n,this.buffer=null,this.stack=[],this.index=0,this.bufferNode=null,e instanceof Ir)this.yieldNode(e);else{this._tree=e.context.parent,this.buffer=e.context;for(let r=e._parent;r;r=r._parent)this.stack.unshift(r.index);this.bufferNode=e,this.yieldBuf(e.index)}}get name(){return this.type.name}yieldNode(e){return e?(this._tree=e,this.type=e.type,this.from=e.from,this.to=e.to,!0):!1}yieldBuf(e,n){this.index=e;let{start:r,buffer:i}=this.buffer;return this.type=n||i.set.types[i.buffer[e]],this.from=r+i.buffer[e+1],this.to=r+i.buffer[e+2],!0}yield(e){return e?e instanceof Ir?(this.buffer=null,this.yieldNode(e)):(this.buffer=e.context,this.yieldBuf(e.index,e.type)):!1}toString(){return this.buffer?this.buffer.buffer.childString(this.index):this._tree.toString()}enterChild(e,n,r){if(!this.buffer)return this.yield(this._tree.nextChild(e<0?this._tree.node.children.length-1:0,e,n,r,this.mode));let{buffer:i}=this.buffer,o=i.findChild(this.index+4,i.buffer[this.index+3],e,n-this.buffer.start,r);return o<0?!1:(this.stack.push(this.index),this.yieldBuf(o))}firstChild(){return this.enterChild(1,0,4)}lastChild(){return this.enterChild(-1,0,4)}childAfter(e){return this.enterChild(1,e,2)}childBefore(e){return this.enterChild(-1,e,-2)}enter(e,n,r=!0,i=!0){return this.buffer?i?this.enterChild(1,e,n):!1:this.yield(this._tree.enter(e,n,r&&!(this.mode&1),i))}parent(){if(!this.buffer)return this.yieldNode(this.mode&1?this._tree._parent:this._tree.parent);if(this.stack.length)return this.yieldBuf(this.stack.pop());let e=this.mode&1?this.buffer.parent:this.buffer.parent.nextSignificantParent();return this.buffer=null,this.yieldNode(e)}sibling(e){if(!this.buffer)return this._tree._parent?this.yield(this._tree.index<0?null:this._tree._parent.nextChild(this._tree.index+e,e,0,4,this.mode)):!1;let{buffer:n}=this.buffer,r=this.stack.length-1;if(e<0){let i=r<0?0:this.stack[r]+4;if(this.index!=i)return this.yieldBuf(n.findChild(i,this.index,-1,0,4))}else{let i=n.buffer[this.index+3];if(i<(r<0?n.buffer.length:n.buffer[this.stack[r]+3]))return this.yieldBuf(i)}return r<0?this.yield(this.buffer.parent.nextChild(this.buffer.index+e,e,0,4,this.mode)):!1}nextSibling(){return this.sibling(1)}prevSibling(){return this.sibling(-1)}atLastNode(e){let n,r,{buffer:i}=this;if(i){if(e>0){if(this.index<i.buffer.buffer.length)return!1}else for(let o=0;o<this.index;o++)if(i.buffer.buffer[o+3]<this.index)return!1;({index:n,parent:r}=i)}else({index:n,_parent:r}=this._tree);for(;r;{index:n,_parent:r}=r)if(n>-1)for(let o=n+e,s=e<0?-1:r.node.children.length;o!=s;o+=e){let l=r.node.children[o];if(this.mode&1||l instanceof to||!l.type.isAnonymous||Pg(l))return!1}return!0}move(e,n){if(n&&this.enterChild(e,0,4))return!0;for(;;){if(this.sibling(e))return!0;if(this.atLastNode(e)||!this.parent())return!1}}next(e=!0){return this.move(1,e)}prev(e=!0){return this.move(-1,e)}moveTo(e,n=0){for(;(this.from==this.to||(n<1?this.from>=e:this.from>e)||(n>-1?this.to<=e:this.to<e))&&this.parent(););for(;this.enterChild(1,e,n););return this}get node(){if(!this.buffer)return this._tree;let e=this.bufferNode,n=null,r=0;if(e&&e.context==this.buffer){e:for(let i=this.index,o=this.stack.length;o>=0;){for(let s=e;s;s=s._parent)if(s.index==i){if(i==this.index)return s;n=s,r=o+1;break e}i=this.stack[--o]}}for(let i=r;i<this.stack.length;i++)n=new wi(this.buffer,n,this.stack[i]);return this.bufferNode=new wi(this.buffer,n,this.index)}get tree(){return this.buffer?null:this._tree.node}}function Pg(t){return t.children.some(e=>e instanceof to||!e.type.isAnonymous||Pg(e))}function U_(t){var e;let{buffer:n,nodeSet:r,maxBufferLength:i=F$,reused:o=[],minRepeatType:s=r.types.length}=t,l=Array.isArray(n)?new kg(n,n.length):n,a=r.types,c=0,u=0;function f(x,b,S,k,P){let{id:C,start:R,end:T,size:L}=l,M=u;for(;L<0;)if(l.next(),L==-1){let D=o[C];S.push(D),k.push(R-x);return}else if(L==-3){c=C;return}else if(L==-4){u=C;return}else throw new RangeError(`Unrecognized record size: ${L}`);let q=a[C],F,Q,E=R-x;if(T-R<=i&&(Q=m(l.pos-b,P))){let D=new Uint16Array(Q.size-Q.skip),X=l.pos-Q.size,ae=D.length;for(;l.pos>X;)ae=g(Q.start,D,ae);F=new to(D,T-Q.start,r),E=Q.start-x}else{let D=l.pos-L;l.next();let X=[],ae=[],W=C>=s?C:-1,B=0,Z=T;for(;l.pos>D;)W>=0&&l.id==W&&l.size>=0?(l.end<=Z-i&&(d(X,ae,R,B,l.end,Z,W,M),B=X.length,Z=l.end),l.next()):f(R,D,X,ae,W);if(W>=0&&B>0&&B<X.length&&d(X,ae,R,B,R,Z,W,M),X.reverse(),ae.reverse(),W>-1&&B>0){let ee=h(q);F=Cg(q,X,ae,0,X.length,0,T-R,ee,ee)}else F=p(q,X,ae,T-R,M-T)}S.push(F),k.push(E)}function h(x){return(b,S,k)=>{let P=0,C=b.length-1,R,T;if(C>=0&&(R=b[C])instanceof et){if(!C&&R.type==x&&R.length==k)return R;(T=R.prop(xe.lookAhead))&&(P=S[C]+R.length+T)}return p(x,b,S,k,P)}}function d(x,b,S,k,P,C,R,T){let L=[],M=[];for(;x.length>k;)L.push(x.pop()),M.push(b.pop()+S-P);x.push(p(r.types[R],L,M,C-P,T-C)),b.push(P-S)}function p(x,b,S,k,P=0,C){if(c){let R=[xe.contextHash,c];C=C?[R].concat(C):[R]}if(P>25){let R=[xe.lookAhead,P];C=C?[R].concat(C):[R]}return new et(x,b,S,k,C)}function m(x,b){let S=l.fork(),k=0,P=0,C=0,R=S.end-i,T={size:0,start:0,skip:0};e:for(let L=S.pos-x;S.pos>L;){let M=S.size;if(S.id==b&&M>=0){T.size=k,T.start=P,T.skip=C,C+=4,k+=4,S.next();continue}let q=S.pos-M;if(M<0||q<L||S.start<R)break;let F=S.id>=s?4:0,Q=S.start;for(S.next();S.pos>q;){if(S.size<0)if(S.size==-3)F+=4;else break e;else S.id>=s&&(F+=4);S.next()}P=Q,k+=M,C+=F}return(b<0||k==x)&&(T.size=k,T.start=P,T.skip=C),T.size>4?T:void 0}function g(x,b,S){let{id:k,start:P,end:C,size:R}=l;if(l.next(),R>=0&&k<s){let T=S;if(R>4){let L=l.pos-(R-4);for(;l.pos>L;)S=g(x,b,S)}b[--S]=T,b[--S]=C-x,b[--S]=P-x,b[--S]=k}else R==-3?c=k:R==-4&&(u=k);return S}let y=[],O=[];for(;l.pos>0;)f(t.start||0,t.bufferStart||0,y,O,-1);let v=(e=t.length)!==null&&e!==void 0?e:y.length?O[0]+y[0].length:0;return new et(a[t.topID],y.reverse(),O.reverse(),v)}const X$=new WeakMap;function mf(t,e){if(!t.isAnonymous||e instanceof to||e.type!=t)return 1;let n=X$.get(e);if(n==null){n=1;for(let r of e.children){if(r.type!=t||!(r instanceof et)){n=1;break}n+=mf(t,r)}X$.set(e,n)}return n}function Cg(t,e,n,r,i,o,s,l,a){let c=0;for(let p=r;p<i;p++)c+=mf(t,e[p]);let u=Math.ceil(c*1.5/8),f=[],h=[];function d(p,m,g,y,O){for(let v=g;v<y;){let x=v,b=m[v],S=mf(t,p[v]);for(v++;v<y;v++){let k=mf(t,p[v]);if(S+k>=u)break;S+=k}if(v==x+1){if(S>u){let k=p[x];d(k.children,k.positions,0,k.children.length,m[x]+O);continue}f.push(p[x])}else{let k=m[v-1]+p[v-1].length-b;f.push(Cg(t,p,m,x,v,b,k,null,a))}h.push(b+O-o)}}return d(e,n,r,i,0),(l||a)(f,h,s)}class no{constructor(e,n,r,i,o=!1,s=!1){this.from=e,this.to=n,this.tree=r,this.offset=i,this.open=(o?1:0)|(s?2:0)}get openStart(){return(this.open&1)>0}get openEnd(){return(this.open&2)>0}static addTree(e,n=[],r=!1){let i=[new no(0,e.length,e,0,!1,r)];for(let o of n)o.to>e.length&&i.push(o);return i}static applyChanges(e,n,r=128){if(!n.length)return e;let i=[],o=1,s=e.length?e[0]:null;for(let l=0,a=0,c=0;;l++){let u=l<n.length?n[l]:null,f=u?u.fromA:1e9;if(f-a>=r)for(;s&&s.from<f;){let h=s;if(a>=h.from||f<=h.to||c){let d=Math.max(h.from,a)-c,p=Math.min(h.to,f)-c;h=d>=p?null:new no(d,p,h.tree,h.offset+c,l>0,!!u)}if(h&&i.push(h),s.to>f)break;s=o<e.length?e[o++]:null}if(!u)break;a=u.toA,c=u.toA-u.toB}return i}}class H${startParse(e,n,r){return typeof e=="string"&&(e=new j_(e)),r=r?r.length?r.map(i=>new $g(i.from,i.to)):[new $g(0,0)]:[new $g(0,e.length)],this.createParse(e,n||[],r)}parse(e,n,r){let i=this.startParse(e,n,r);for(;;){let o=i.advance();if(o)return o}}}class j_{constructor(e){this.string=e}get length(){return this.string.length}chunk(e){return this.string.slice(e)}get lineChunks(){return!1}read(e,n){return this.string.slice(e,n)}}new xe({perNode:!0});const gf=new xe;function X_(t){return ne.define({combine:t?e=>e.concat(t):void 0})}class tr{constructor(e,n,r,i=[]){this.data=e,this.topNode=r,Pe.prototype.hasOwnProperty("tree")||Object.defineProperty(Pe.prototype,"tree",{get(){return an(this)}}),this.parser=n,this.extension=[bi.of(this),Pe.languageData.of((o,s,l)=>o.facet(G$(o,s,l)))].concat(i)}isActiveAt(e,n,r=-1){return G$(e,n,r)==this.data}findRegions(e){let n=e.facet(bi);if((n==null?void 0:n.data)==this.data)return[{from:0,to:e.doc.length}];if(!n||!n.allowsNesting)return[];let r=[],i=(o,s)=>{if(o.prop(gf)==this.data){r.push({from:s,to:s+o.length});return}let l=o.prop(xe.mounted);if(l){if(l.tree.prop(gf)==this.data){if(l.overlay)for(let a of l.overlay)r.push({from:a.from+s,to:a.to+s});else r.push({from:s,to:s+o.length});return}else if(l.overlay){let a=r.length;if(i(l.tree,l.overlay[0].from+s),r.length>a)return}}for(let a=0;a<o.children.length;a++){let c=o.children[a];c instanceof et&&i(c,o.positions[a]+s)}};return i(an(e),0),r}get allowsNesting(){return!0}}tr.setState=ce.define();function G$(t,e,n){let r=t.facet(bi);if(!r)return null;let i=r.data;if(r.allowsNesting)for(let o=an(t).topNode;o;o=o.enter(e,n,!0,!1))i=o.type.prop(gf)||i;return i}class yf extends tr{constructor(e,n){super(e,n,n.topNode);this.parser=n}static define(e){let n=X_(e.languageData);return new yf(n,e.parser.configure({props:[gf.add(r=>r.isTop?n:void 0)]}))}configure(e){return new yf(this.data,this.parser.configure(e))}get allowsNesting(){return this.parser.wrappers.length>0}}function an(t){let e=t.field(tr.state,!1);return e?e.tree:et.empty}class H_{constructor(e,n=e.length){this.doc=e,this.length=n,this.cursorPos=0,this.string="",this.cursor=e.iter()}syncTo(e){return this.string=this.cursor.next(e-this.cursorPos).value,this.cursorPos=e+this.string.length,this.cursorPos-this.string.length}chunk(e){return this.syncTo(e),this.string}get lineChunks(){return!0}read(e,n){let r=this.cursorPos-this.string.length;return e<r||n>=this.cursorPos?this.doc.sliceString(e,n):this.string.slice(e-r,n-r)}}let Ca=null;class Rg{constructor(e,n,r=[],i,o,s,l,a){this.parser=e,this.state=n,this.fragments=r,this.tree=i,this.treeLen=o,this.viewport=s,this.skipped=l,this.scheduleOn=a,this.parse=null,this.tempSkipped=[]}startParse(){return this.parser.startParse(new H_(this.state.doc),this.fragments)}work(e,n){return n!=null&&n>=this.state.doc.length&&(n=void 0),this.tree!=et.empty&&this.isDone(n!=null?n:this.state.doc.length)?(this.takeTree(),!0):this.withContext(()=>{var r;let i=Date.now()+e;for(this.parse||(this.parse=this.startParse()),n!=null&&(this.parse.stoppedAt==null||this.parse.stoppedAt>n)&&n<this.state.doc.length&&this.parse.stopAt(n);;){let o=this.parse.advance();if(o)if(this.fragments=this.withoutTempSkipped(no.addTree(o,this.fragments,this.parse.stoppedAt!=null)),this.treeLen=(r=this.parse.stoppedAt)!==null&&r!==void 0?r:this.state.doc.length,this.tree=o,this.parse=null,this.treeLen<(n!=null?n:this.state.doc.length))this.parse=this.startParse();else return!0;if(Date.now()>i)return!1}})}takeTree(){let e,n;this.parse&&(e=this.parse.parsedPos)>=this.treeLen&&((this.parse.stoppedAt==null||this.parse.stoppedAt>e)&&this.parse.stopAt(e),this.withContext(()=>{for(;!(n=this.parse.advance()););}),this.treeLen=e,this.tree=n,this.fragments=this.withoutTempSkipped(no.addTree(this.tree,this.fragments,!0)),this.parse=null)}withContext(e){let n=Ca;Ca=this;try{return e()}finally{Ca=n}}withoutTempSkipped(e){for(let n;n=this.tempSkipped.pop();)e=Z$(e,n.from,n.to);return e}changes(e,n){let{fragments:r,tree:i,treeLen:o,viewport:s,skipped:l}=this;if(this.takeTree(),!e.empty){let a=[];if(e.iterChangedRanges((c,u,f,h)=>a.push({fromA:c,toA:u,fromB:f,toB:h})),r=no.applyChanges(r,a),i=et.empty,o=0,s={from:e.mapPos(s.from,-1),to:e.mapPos(s.to,1)},this.skipped.length){l=[];for(let c of this.skipped){let u=e.mapPos(c.from,1),f=e.mapPos(c.to,-1);u<f&&l.push({from:u,to:f})}}}return new Rg(this.parser,n,r,i,o,s,l,this.scheduleOn)}updateViewport(e){if(this.viewport.from==e.from&&this.viewport.to==e.to)return!1;this.viewport=e;let n=this.skipped.length;for(let r=0;r<this.skipped.length;r++){let{from:i,to:o}=this.skipped[r];i<e.to&&o>e.from&&(this.fragments=Z$(this.fragments,i,o),this.skipped.splice(r--,1))}return this.skipped.length>=n?!1:(this.reset(),!0)}reset(){this.parse&&(this.takeTree(),this.parse=null)}skipUntilInView(e,n){this.skipped.push({from:e,to:n})}static getSkippingParser(e){return new class extends H${createParse(n,r,i){let o=i[0].from,s=i[i.length-1].to;return{parsedPos:o,advance(){let a=Ca;if(a){for(let c of i)a.tempSkipped.push(c);e&&(a.scheduleOn=a.scheduleOn?Promise.all([a.scheduleOn,e]):e)}return this.parsedPos=s,new et(ln.none,[],[],s-o)},stoppedAt:null,stopAt(){}}}}}isDone(e){e=Math.min(e,this.state.doc.length);let n=this.fragments;return this.treeLen>=e&&n.length&&n[0].from==0&&n[0].to>=e}static get(){return Ca}}function Z$(t,e,n){return no.applyChanges(t,[{fromA:e,toA:n,fromB:e,toB:n}])}class vs{constructor(e){this.context=e,this.tree=e.tree}apply(e){if(!e.docChanged)return this;let n=this.context.changes(e.changes,e.state),r=this.context.treeLen==e.startState.doc.length?void 0:Math.max(e.changes.mapPos(this.context.treeLen),n.viewport.to);return n.work(20,r)||n.takeTree(),new vs(n)}static init(e){let n=Math.min(3e3,e.doc.length),r=new Rg(e.facet(bi).parser,e,[],et.empty,0,{from:0,to:n},[],null);return r.work(20,n)||r.takeTree(),new vs(r)}}tr.state=Rt.define({create:vs.init,update(t,e){for(let n of e.effects)if(n.is(tr.setState))return n.value;return e.startState.facet(bi)!=e.state.facet(bi)?vs.init(e.state):t.apply(e)}});let Y$=t=>{let e=setTimeout(()=>t(),500);return()=>clearTimeout(e)};typeof requestIdleCallback!="undefined"&&(Y$=t=>{let e=-1,n=setTimeout(()=>{e=requestIdleCallback(t,{timeout:500-100})},100);return()=>e<0?clearTimeout(n):cancelIdleCallback(e)});const G_=rt.fromClass(class{constructor(e){this.view=e,this.working=null,this.workScheduled=0,this.chunkEnd=-1,this.chunkBudget=-1,this.work=this.work.bind(this),this.scheduleWork()}update(e){let n=this.view.state.field(tr.state).context;(n.updateViewport(e.view.viewport)||this.view.viewport.to>n.treeLen)&&this.scheduleWork(),e.docChanged&&(this.view.hasFocus&&(this.chunkBudget+=50),this.scheduleWork()),this.checkAsyncSchedule(n)}scheduleWork(){if(this.working)return;let{state:e}=this.view,n=e.field(tr.state);(n.tree!=n.context.tree||!n.context.isDone(e.doc.length))&&(this.working=Y$(this.work))}work(e){this.working=null;let n=Date.now();if(this.chunkEnd<n&&(this.chunkEnd<0||this.view.hasFocus)&&(this.chunkEnd=n+3e4,this.chunkBudget=3e3),this.chunkBudget<=0)return;let{state:r,viewport:{to:i}}=this.view,o=r.field(tr.state);if(o.tree==o.context.tree&&o.context.isDone(i+1e5))return;let s=Math.min(this.chunkBudget,100,e?Math.max(25,e.timeRemaining()-5):1e9),l=o.context.treeLen<i&&r.doc.length>i+1e3,a=o.context.work(s,i+(l?0:1e5));this.chunkBudget-=Date.now()-n,(a||this.chunkBudget<=0)&&(o.context.takeTree(),this.view.dispatch({effects:tr.setState.of(new vs(o.context))})),this.chunkBudget>0&&!(a&&!l)&&this.scheduleWork(),this.checkAsyncSchedule(o.context)}checkAsyncSchedule(e){e.scheduleOn&&(this.workScheduled++,e.scheduleOn.then(()=>this.scheduleWork()).catch(n=>bn(this.view.state,n)).then(()=>this.workScheduled--),e.scheduleOn=null)}destroy(){this.working&&this.working()}isWorking(){return this.working||this.workScheduled>0}},{eventHandlers:{focus(){this.scheduleWork()}}}),bi=ne.define({combine(t){return t.length?t[0]:null},enables:[tr.state,G_]});class Z_{constructor(e,n=[]){this.language=e,this.support=n,this.extension=[e,n]}}const Y_=ne.define(),Tg=ne.define({combine:t=>{if(!t.length)return"  ";if(!/^(?: +|\t+)$/.test(t[0]))throw new Error("Invalid indent unit: "+JSON.stringify(t[0]));return t[0]}});function Of(t){let e=t.facet(Tg);return e.charCodeAt(0)==9?t.tabSize*e.length:e.length}function Ra(t,e){let n="",r=t.tabSize;if(t.facet(Tg).charCodeAt(0)==9)for(;e>=r;)n+="	",e-=r;for(let i=0;i<e;i++)n+=" ";return n}function Ig(t,e){t instanceof Pe&&(t=new vf(t));for(let r of t.state.facet(Y_)){let i=r(t,e);if(i!=null)return i}let n=an(t.state);return n?K_(t,n,e):null}class vf{constructor(e,n={}){this.state=e,this.options=n,this.unit=Of(e)}lineAt(e,n=1){let r=this.state.doc.lineAt(e),{simulateBreak:i}=this.options;return i!=null&&i>=r.from&&i<=r.to?(n<0?i<e:i<=e)?{text:r.text.slice(i-r.from),from:i}:{text:r.text.slice(0,i-r.from),from:r.from}:r}textAfterPos(e,n=1){if(this.options.simulateDoubleBreak&&e==this.options.simulateBreak)return"";let{text:r,from:i}=this.lineAt(e,n);return r.slice(e-i,Math.min(r.length,e+100-i))}column(e,n=1){let{text:r,from:i}=this.lineAt(e,n),o=this.countColumn(r,e-i),s=this.options.overrideIndentation?this.options.overrideIndentation(i):-1;return s>-1&&(o+=s-this.countColumn(r,r.search(/\S|$/))),o}countColumn(e,n=e.length){return ua(e,this.state.tabSize,n)}lineIndent(e,n=1){let{text:r,from:i}=this.lineAt(e,n),o=this.options.overrideIndentation;if(o){let s=o(i);if(s>-1)return s}return this.countColumn(r,r.search(/\S|$/))}get simulatedBreak(){return this.options.simulateBreak||null}}const K$=new xe;function K_(t,e,n){return J$(e.resolveInner(n).enterUnfinishedNodesBefore(n),n,t)}function J_(t){return t.pos==t.options.simulateBreak&&t.options.simulateDoubleBreak}function eB(t){let e=t.type.prop(K$);if(e)return e;let n=t.firstChild,r;if(n&&(r=n.type.prop(xe.closedBy))){let i=t.lastChild,o=i&&r.indexOf(i.name)>-1;return s=>eS(s,!0,1,void 0,o&&!J_(s)?i.from:void 0)}return t.parent==null?tB:null}function J$(t,e,n){for(;t;t=t.parent){let r=eB(t);if(r)return r(new nB(n,e,t))}return null}function tB(){return 0}class nB extends vf{constructor(e,n,r){super(e.state,e.options);this.base=e,this.pos=n,this.node=r}get textAfter(){return this.textAfterPos(this.pos)}get baseIndent(){let e=this.state.doc.lineAt(this.node.from);for(;;){let n=this.node.resolve(e.from);for(;n.parent&&n.parent.from==n.from;)n=n.parent;if(rB(n,this.node))break;e=this.state.doc.lineAt(n.from)}return this.lineIndent(e.from)}continue(){let e=this.node.parent;return e?J$(e,this.pos,this.base):0}}function rB(t,e){for(let n=e;n;n=n.parent)if(t==n)return!0;return!1}function iB(t){let e=t.node,n=e.childAfter(e.from),r=e.lastChild;if(!n)return null;let i=t.options.simulateBreak,o=t.state.doc.lineAt(n.from),s=i==null||i<=o.from?o.to:Math.min(o.to,i);for(let l=n.to;;){let a=e.childAfter(l);if(!a||a==r)return null;if(!a.type.isSkipped)return a.from<s?n:null;l=a.to}}function Eg({closing:t,align:e=!0,units:n=1}){return r=>eS(r,e,n,t)}function eS(t,e,n,r,i){let o=t.textAfter,s=o.match(/^\s*/)[0].length,l=r&&o.slice(s,s+r.length)==r||i==t.pos+s,a=e?iB(t):null;return a?l?t.column(a.from):t.column(a.to):t.baseIndent+(l?0:t.unit*n)}const oB=200;function sB(){return Pe.transactionFilter.of(t=>{if(!t.docChanged||!t.isUserEvent("input.type"))return t;let e=t.startState.languageDataAt("indentOnInput",t.startState.selection.main.head);if(!e.length)return t;let n=t.newDoc,{head:r}=t.newSelection.main,i=n.lineAt(r);if(r>i.from+oB)return t;let o=n.sliceString(i.from,r);if(!e.some(c=>c.test(o)))return t;let{state:s}=t,l=-1,a=[];for(let{head:c}of s.selection.ranges){let u=s.doc.lineAt(c);if(u.from==l)continue;l=u.from;let f=Ig(s,u.from);if(f==null)continue;let h=/^\s*/.exec(u.text)[0],d=Ra(s,f);h!=d&&a.push({from:u.from,to:u.from+h.length,insert:d})}return a.length?[t,{changes:a,sequential:!0}]:t})}const lB=ne.define(),tS=new xe;function aB(t){let e=t.firstChild,n=t.lastChild;return e&&e.to<n.from?{from:e.to,to:n.type.isError?t.to:n.from}:null}function cB(t,e,n){let r=an(t);if(r.length==0)return null;let i=r.resolveInner(n),o=null;for(let s=i;s;s=s.parent){if(s.to<=n||s.from>n)continue;if(o&&s.from<e)break;let l=s.type.prop(tS);if(l){let a=l(s,t);a&&a.from<=n&&a.from>=e&&a.to>n&&(o=a)}}return o}function xf(t,e,n){for(let r of t.facet(lB)){let i=r(t,e,n);if(i)return i}return cB(t,e,n)}class uB{constructor(e,n,r){this.state=e,this.pos=n,this.explicit=r,this.abortListeners=[]}tokenBefore(e){let n=an(this.state).resolveInner(this.pos,-1);for(;n&&e.indexOf(n.name)<0;)n=n.parent;return n?{from:n.from,to:this.pos,text:this.state.sliceDoc(n.from,this.pos),type:n.type}:null}matchBefore(e){let n=this.state.doc.lineAt(this.pos),r=Math.max(n.from,this.pos-250),i=n.text.slice(r-n.from,this.pos-n.from),o=i.search(iS(e,!1));return o<0?null:{from:r+o,to:this.pos,text:i.slice(o)}}get aborted(){return this.abortListeners==null}addEventListener(e,n){e=="abort"&&this.abortListeners&&this.abortListeners.push(n)}}function nS(t){let e=Object.keys(t).join(""),n=/\w/.test(e);return n&&(e=e.replace(/\w/g,"")),`[${n?"\\w":""}${e.replace(/[^\w\s]/g,"\\$&")}]`}function fB(t){let e=Object.create(null),n=Object.create(null);for(let{label:i}of t){e[i[0]]=!0;for(let o=1;o<i.length;o++)n[i[o]]=!0}let r=nS(e)+nS(n)+"*$";return[new RegExp("^"+r),new RegExp(r)]}function hB(t){let e=t.map(i=>typeof i=="string"?{label:i}:i),[n,r]=e.every(i=>/^\w+$/.test(i.label))?[/\w*$/,/\w+$/]:fB(e);return i=>{let o=i.matchBefore(r);return o||i.explicit?{from:o?o.from:i.pos,options:e,span:n}:null}}class rS{constructor(e,n,r){this.completion=e,this.source=n,this.match=r}}function xs(t){return t.selection.main.head}function iS(t,e){var n;let{source:r}=t,i=e&&r[0]!="^",o=r[r.length-1]!="$";return!i&&!o?t:new RegExp(`${i?"^":""}(?:${r})${o?"$":""}`,(n=t.flags)!==null&&n!==void 0?n:t.ignoreCase?"i":"")}const dB=pi.define();function oS(t,e){let n=e.completion.apply||e.completion.label,r=e.source;typeof n=="string"?t.dispatch({changes:{from:r.from,to:r.to,insert:n},selection:{anchor:r.from+n.length},userEvent:"input.complete",annotations:dB.of(e.completion)}):n(t,e.completion,r.from,r.to)}const sS=new WeakMap;function pB(t){if(!Array.isArray(t))return t;let e=sS.get(t);return e||sS.set(t,e=hB(t)),e}class mB{constructor(e){this.pattern=e,this.chars=[],this.folded=[],this.any=[],this.precise=[],this.byWord=[];for(let n=0;n<e.length;){let r=Ct(e,n),i=wn(r);this.chars.push(r);let o=e.slice(n,n+i),s=o.toUpperCase();this.folded.push(Ct(s==o?o.toLowerCase():s,0)),n+=i}this.astral=e.length!=this.chars.length}match(e){if(this.pattern.length==0)return[0];if(e.length<this.pattern.length)return null;let{chars:n,folded:r,any:i,precise:o,byWord:s}=this;if(n.length==1){let O=Ct(e,0);return O==n[0]?[0,0,wn(O)]:O==r[0]?[-200,0,wn(O)]:null}let l=e.indexOf(this.pattern);if(l==0)return[0,0,this.pattern.length];let a=n.length,c=0;if(l<0){for(let O=0,v=Math.min(e.length,200);O<v&&c<a;){let x=Ct(e,O);(x==n[c]||x==r[c])&&(i[c++]=O),O+=wn(x)}if(c<a)return null}let u=0,f=0,h=!1,d=0,p=-1,m=-1,g=/[a-z]/.test(e),y=!0;for(let O=0,v=Math.min(e.length,200),x=0;O<v&&f<a;){let b=Ct(e,O);l<0&&(u<a&&b==n[u]&&(o[u++]=O),d<a&&(b==n[d]||b==r[d]?(d==0&&(p=O),m=O+1,d++):d=0));let S,k=b<255?b>=48&&b<=57||b>=97&&b<=122?2:b>=65&&b<=90?1:0:(S=wm(b))!=S.toLowerCase()?1:S!=S.toUpperCase()?2:0;(!O||k==1&&g||x==0&&k!=0)&&(n[f]==b||r[f]==b&&(h=!0)?s[f++]=O:s.length&&(y=!1)),x=k,O+=wn(b)}return f==a&&s[0]==0&&y?this.result(-100+(h?-200:0),s,e):d==a&&p==0?[-200-e.length,0,m]:l>-1?[-700-e.length,l,l+this.pattern.length]:d==a?[-200+-700-e.length,p,m]:f==a?this.result(-100+(h?-200:0)+-700+(y?0:-1100),s,e):n.length==2?null:this.result((i[0]?-700:0)+-200+-1100,i,e)}result(e,n,r){let i=[e-r.length],o=1;for(let s of n){let l=s+(this.astral?wn(Ct(r,s)):1);o>1&&i[o-1]==s?i[o-1]=l:(i[o++]=s,i[o++]=l)}return i}}const ro=ne.define({combine(t){return mi(t,{activateOnTyping:!0,override:null,maxRenderedOptions:100,defaultKeymap:!0,optionClass:()=>"",aboveCursor:!1,icons:!0,addToOptions:[]},{defaultKeymap:(e,n)=>e&&n,icons:(e,n)=>e&&n,optionClass:(e,n)=>r=>gB(e(r),n(r)),addToOptions:(e,n)=>e.concat(n)})}});function gB(t,e){return t?e?t+" "+e:t:e}function yB(t){let e=t.addToOptions.slice();return t.icons&&e.push({render(n){let r=document.createElement("div");return r.classList.add("cm-completionIcon"),n.type&&r.classList.add(...n.type.split(/\s+/g).map(i=>"cm-completionIcon-"+i)),r.setAttribute("aria-hidden","true"),r},position:20}),e.push({render(n,r,i){let o=document.createElement("span");o.className="cm-completionLabel";let{label:s}=n,l=0;for(let a=1;a<i.length;){let c=i[a++],u=i[a++];c>l&&o.appendChild(document.createTextNode(s.slice(l,c)));let f=o.appendChild(document.createElement("span"));f.appendChild(document.createTextNode(s.slice(c,u))),f.className="cm-completionMatchedText",l=u}return l<s.length&&o.appendChild(document.createTextNode(s.slice(l))),o},position:50},{render(n){if(!n.detail)return null;let r=document.createElement("span");return r.className="cm-completionDetail",r.textContent=n.detail,r},position:80}),e.sort((n,r)=>n.position-r.position).map(n=>n.render)}function OB(t,e){let n=document.createElement("div");n.className="cm-tooltip cm-completionInfo";let{info:r}=t.completion;if(typeof r=="string")n.textContent=r;else{let i=r(t.completion);i.then?i.then(o=>n.appendChild(o),o=>bn(e.state,o,"completion info")):n.appendChild(i)}return n}function lS(t,e,n){if(t<=n)return{from:0,to:t};if(e<=t>>1){let i=Math.floor(e/n);return{from:i*n,to:(i+1)*n}}let r=Math.floor((t-e)/n);return{from:t-(r+1)*n,to:t-r*n}}class vB{constructor(e,n){this.view=e,this.stateField=n,this.info=null,this.placeInfo={read:()=>this.measureInfo(),write:l=>this.positionInfo(l),key:this};let r=e.state.field(n),{options:i,selected:o}=r.open,s=e.state.facet(ro);this.optionContent=yB(s),this.optionClass=s.optionClass,this.range=lS(i.length,o,s.maxRenderedOptions),this.dom=document.createElement("div"),this.dom.className="cm-tooltip-autocomplete",this.dom.addEventListener("mousedown",l=>{for(let a=l.target,c;a&&a!=this.dom;a=a.parentNode)if(a.nodeName=="LI"&&(c=/-(\d+)$/.exec(a.id))&&+c[1]<i.length){oS(e,i[+c[1]]),l.preventDefault();return}}),this.list=this.dom.appendChild(this.createListBox(i,r.id,this.range)),this.list.addEventListener("scroll",()=>{this.info&&this.view.requestMeasure(this.placeInfo)})}mount(){this.updateSel()}update(e){e.state.field(this.stateField)!=e.startState.field(this.stateField)&&this.updateSel()}positioned(){this.info&&this.view.requestMeasure(this.placeInfo)}updateSel(){let e=this.view.state.field(this.stateField),n=e.open;if((n.selected<this.range.from||n.selected>=this.range.to)&&(this.range=lS(n.options.length,n.selected,this.view.state.facet(ro).maxRenderedOptions),this.list.remove(),this.list=this.dom.appendChild(this.createListBox(n.options,e.id,this.range)),this.list.addEventListener("scroll",()=>{this.info&&this.view.requestMeasure(this.placeInfo)})),this.updateSelectedOption(n.selected)){this.info&&(this.info.remove(),this.info=null);let r=n.options[n.selected];r.completion.info&&(this.info=this.dom.appendChild(OB(r,this.view)),this.view.requestMeasure(this.placeInfo))}}updateSelectedOption(e){let n=null;for(let r=this.list.firstChild,i=this.range.from;r;r=r.nextSibling,i++)i==e?r.hasAttribute("aria-selected")||(r.setAttribute("aria-selected","true"),n=r):r.hasAttribute("aria-selected")&&r.removeAttribute("aria-selected");return n&&wB(this.list,n),n}measureInfo(){let e=this.dom.querySelector("[aria-selected]");if(!e||!this.info)return null;let n=this.dom.getBoundingClientRect(),r=this.info.getBoundingClientRect(),i=e.getBoundingClientRect();if(i.top>Math.min(innerHeight,n.bottom)-10||i.bottom<Math.max(0,n.top)+10)return null;let o=Math.max(0,Math.min(i.top,innerHeight-r.height))-n.top,s=this.view.textDirection==Se.RTL,l=n.left,a=innerWidth-n.right;return s&&l<Math.min(r.width,a)?s=!1:!s&&a<Math.min(r.width,l)&&(s=!0),{top:o,left:s}}positionInfo(e){this.info&&(this.info.style.top=(e?e.top:-1e6)+"px",e&&(this.info.classList.toggle("cm-completionInfo-left",e.left),this.info.classList.toggle("cm-completionInfo-right",!e.left)))}createListBox(e,n,r){const i=document.createElement("ul");i.id=n,i.setAttribute("role","listbox");for(let o=r.from;o<r.to;o++){let{completion:s,match:l}=e[o];const a=i.appendChild(document.createElement("li"));a.id=n+"-"+o,a.setAttribute("role","option");let c=this.optionClass(s);c&&(a.className=c);for(let u of this.optionContent){let f=u(s,this.view.state,l);f&&a.appendChild(f)}}return r.from&&i.classList.add("cm-completionListIncompleteTop"),r.to<e.length&&i.classList.add("cm-completionListIncompleteBottom"),i}}function xB(t){return e=>new vB(e,t)}function wB(t,e){let n=t.getBoundingClientRect(),r=e.getBoundingClientRect();r.top<n.top?t.scrollTop-=n.top-r.top:r.bottom>n.bottom&&(t.scrollTop+=r.bottom-n.bottom)}const bB=300;function aS(t){return(t.boost||0)*100+(t.apply?10:0)+(t.info?5:0)+(t.type?1:0)}function $B(t,e){let n=[],r=0;for(let s of t)if(s.hasResult())if(s.result.filter===!1)for(let l of s.result.options)n.push(new rS(l,s,[1e9-r++]));else{let l=new mB(e.sliceDoc(s.from,s.to)),a;for(let c of s.result.options)(a=l.match(c.label))&&(c.boost!=null&&(a[0]+=c.boost),n.push(new rS(c,s,a)))}n.sort(uS);let i=[],o=null;for(let s of n.sort(uS)){if(i.length==bB)break;!o||o.label!=s.completion.label||o.detail!=s.completion.detail||o.type!=s.completion.type||o.apply!=s.completion.apply?i.push(s):aS(s.completion)>aS(o)&&(i[i.length-1]=s),o=s.completion}return i}class Ta{constructor(e,n,r,i,o){this.options=e,this.attrs=n,this.tooltip=r,this.timestamp=i,this.selected=o}setSelected(e,n){return e==this.selected||e>=this.options.length?this:new Ta(this.options,cS(n,e),this.tooltip,this.timestamp,e)}static build(e,n,r,i,o){let s=$B(e,n);if(!s.length)return null;let l=0;if(i&&i.selected){let a=i.options[i.selected].completion;for(let c=0;c<s.length&&!l;c++)s[c].completion==a&&(l=c)}return new Ta(s,cS(r,l),{pos:e.reduce((a,c)=>c.hasResult()?Math.min(a,c.from):a,1e8),create:xB($n),above:o.aboveCursor},i?i.timestamp:Date.now(),l)}map(e){return new Ta(this.options,this.attrs,Object.assign(Object.assign({},this.tooltip),{pos:e.mapPos(this.tooltip.pos)}),this.timestamp,this.selected)}}class wf{constructor(e,n,r){this.active=e,this.id=n,this.open=r}static start(){return new wf(PB,"cm-ac-"+Math.floor(Math.random()*2e6).toString(36),null)}update(e){let{state:n}=e,r=n.facet(ro),o=(r.override||n.languageDataAt("autocomplete",xs(n)).map(pB)).map(l=>(this.active.find(c=>c.source==l)||new jt(l,this.active.some(c=>c.state!=0)?1:0)).update(e,r));o.length==this.active.length&&o.every((l,a)=>l==this.active[a])&&(o=this.active);let s=e.selection||o.some(l=>l.hasResult()&&e.changes.touchesRange(l.from,l.to))||!SB(o,this.active)?Ta.build(o,n,this.id,this.open,r):this.open&&e.docChanged?this.open.map(e.changes):this.open;!s&&o.every(l=>l.state!=1)&&o.some(l=>l.hasResult())&&(o=o.map(l=>l.hasResult()?new jt(l.source,0):l));for(let l of e.effects)l.is(hS)&&(s=s&&s.setSelected(l.value,this.id));return o==this.active&&s==this.open?this:new wf(o,this.id,s)}get tooltip(){return this.open?this.open.tooltip:null}get attrs(){return this.open?this.open.attrs:kB}}function SB(t,e){if(t==e)return!0;for(let n=0,r=0;;){for(;n<t.length&&!t[n].hasResult;)n++;for(;r<e.length&&!e[r].hasResult;)r++;let i=n==t.length,o=r==e.length;if(i||o)return i==o;if(t[n++].result!=e[r++].result)return!1}}const kB={"aria-autocomplete":"list","aria-expanded":"false"};function cS(t,e){return{"aria-autocomplete":"list","aria-expanded":"true","aria-activedescendant":t+"-"+e,"aria-controls":t}}const PB=[];function uS(t,e){let n=e.match[0]-t.match[0];return n||t.completion.label.localeCompare(e.completion.label)}function Mg(t){return t.isUserEvent("input.type")?"input":t.isUserEvent("delete.backward")?"delete":null}class jt{constructor(e,n,r=-1){this.source=e,this.state=n,this.explicitPos=r}hasResult(){return!1}update(e,n){let r=Mg(e),i=this;r?i=i.handleUserEvent(e,r,n):e.docChanged?i=i.handleChange(e):e.selection&&i.state!=0&&(i=new jt(i.source,0));for(let o of e.effects)if(o.is(Ag))i=new jt(i.source,1,o.value?xs(e.state):-1);else if(o.is(Dg))i=new jt(i.source,0);else if(o.is(fS))for(let s of o.value)s.source==i.source&&(i=s);return i}handleUserEvent(e,n,r){return n=="delete"||!r.activateOnTyping?this.map(e.changes):new jt(this.source,1)}handleChange(e){return e.changes.touchesRange(xs(e.startState))?new jt(this.source,0):this.map(e.changes)}map(e){return e.empty||this.explicitPos<0?this:new jt(this.source,this.state,e.mapPos(this.explicitPos))}}class bf extends jt{constructor(e,n,r,i,o,s){super(e,2,n);this.result=r,this.from=i,this.to=o,this.span=s}hasResult(){return!0}handleUserEvent(e,n,r){let i=e.changes.mapPos(this.from),o=e.changes.mapPos(this.to,1),s=xs(e.state);if((this.explicitPos>-1?s<i:s<=i)||s>o)return new jt(this.source,n=="input"&&r.activateOnTyping?1:0);let l=this.explicitPos<0?-1:e.changes.mapPos(this.explicitPos);return this.span&&(i==o||this.span.test(e.state.sliceDoc(i,o)))?new bf(this.source,l,this.result,i,o,this.span):new jt(this.source,1,l)}handleChange(e){return e.changes.touchesRange(this.from,this.to)?new jt(this.source,0):this.map(e.changes)}map(e){return e.empty?this:new bf(this.source,this.explicitPos<0?-1:e.mapPos(this.explicitPos),this.result,e.mapPos(this.from),e.mapPos(this.to,1),this.span)}}const Ag=ce.define(),Dg=ce.define(),fS=ce.define({map(t,e){return t.map(n=>n.map(e))}}),hS=ce.define(),$n=Rt.define({create(){return wf.start()},update(t,e){return t.update(e)},provide:t=>[wg.from(t,e=>e.tooltip),oe.contentAttributes.from(t,e=>e.attrs)]}),dS=75;function $f(t,e="option"){return n=>{let r=n.state.field($n,!1);if(!r||!r.open||Date.now()-r.open.timestamp<dS)return!1;let i=1,o;e=="page"&&(o=B_(n,r.open.tooltip))&&(i=Math.max(2,Math.floor(o.dom.offsetHeight/o.dom.querySelector("li").offsetHeight)-1));let s=r.open.selected+i*(t?1:-1),{length:l}=r.open.options;return s<0?s=e=="page"?0:l-1:s>=l&&(s=e=="page"?l-1:0),n.dispatch({effects:hS.of(s)}),!0}}const CB=t=>{let e=t.state.field($n,!1);return t.state.readOnly||!e||!e.open||Date.now()-e.open.timestamp<dS?!1:(oS(t,e.open.options[e.open.selected]),!0)},RB=t=>t.state.field($n,!1)?(t.dispatch({effects:Ag.of(!0)}),!0):!1,TB=t=>{let e=t.state.field($n,!1);return!e||!e.active.some(n=>n.state!=0)?!1:(t.dispatch({effects:Dg.of(null)}),!0)};class IB{constructor(e,n){this.active=e,this.context=n,this.time=Date.now(),this.updates=[],this.done=void 0}}const pS=50,EB=50,MB=1e3,AB=rt.fromClass(class{constructor(t){this.view=t,this.debounceUpdate=-1,this.running=[],this.debounceAccept=-1,this.composing=0;for(let e of t.state.field($n).active)e.state==1&&this.startQuery(e)}update(t){let e=t.state.field($n);if(!t.selectionSet&&!t.docChanged&&t.startState.field($n)==e)return;let n=t.transactions.some(r=>(r.selection||r.docChanged)&&!Mg(r));for(let r=0;r<this.running.length;r++){let i=this.running[r];if(n||i.updates.length+t.transactions.length>EB&&i.time-Date.now()>MB){for(let o of i.context.abortListeners)try{o()}catch(s){bn(this.view.state,s)}i.context.abortListeners=null,this.running.splice(r--,1)}else i.updates.push(...t.transactions)}if(this.debounceUpdate>-1&&clearTimeout(this.debounceUpdate),this.debounceUpdate=e.active.some(r=>r.state==1&&!this.running.some(i=>i.active.source==r.source))?setTimeout(()=>this.startUpdate(),pS):-1,this.composing!=0)for(let r of t.transactions)Mg(r)=="input"?this.composing=2:this.composing==2&&r.selection&&(this.composing=3)}startUpdate(){this.debounceUpdate=-1;let{state:t}=this.view,e=t.field($n);for(let n of e.active)n.state==1&&!this.running.some(r=>r.active.source==n.source)&&this.startQuery(n)}startQuery(t){let{state:e}=this.view,n=xs(e),r=new uB(e,n,t.explicitPos==n),i=new IB(t,r);this.running.push(i),Promise.resolve(t.source(r)).then(o=>{i.context.aborted||(i.done=o||null,this.scheduleAccept())},o=>{this.view.dispatch({effects:Dg.of(null)}),bn(this.view.state,o)})}scheduleAccept(){this.running.every(t=>t.done!==void 0)?this.accept():this.debounceAccept<0&&(this.debounceAccept=setTimeout(()=>this.accept(),pS))}accept(){var t;this.debounceAccept>-1&&clearTimeout(this.debounceAccept),this.debounceAccept=-1;let e=[],n=this.view.state.facet(ro);for(let r=0;r<this.running.length;r++){let i=this.running[r];if(i.done===void 0)continue;if(this.running.splice(r--,1),i.done){let s=new bf(i.active.source,i.active.explicitPos,i.done,i.done.from,(t=i.done.to)!==null&&t!==void 0?t:xs(i.updates.length?i.updates[0].startState:this.view.state),i.done.span&&i.done.filter!==!1?iS(i.done.span,!0):null);for(let l of i.updates)s=s.update(l,n);if(s.hasResult()){e.push(s);continue}}let o=this.view.state.field($n).active.find(s=>s.source==i.active.source);if(o&&o.state==1)if(i.done==null){let s=new jt(i.active.source,0);for(let l of i.updates)s=s.update(l,n);s.state!=1&&e.push(s)}else this.startQuery(o)}e.length&&this.view.dispatch({effects:fS.of(e)})}},{eventHandlers:{compositionstart(){this.composing=1},compositionend(){this.composing==3&&setTimeout(()=>this.view.dispatch({effects:Ag.of(!1)}),20),this.composing=0}}}),DB=oe.baseTheme({".cm-tooltip.cm-tooltip-autocomplete":{"& > ul":{fontFamily:"monospace",whiteSpace:"nowrap",overflow:"hidden auto",maxWidth_fallback:"700px",maxWidth:"min(700px, 95vw)",minWidth:"250px",maxHeight:"10em",listStyle:"none",margin:0,padding:0,"& > li":{overflowX:"hidden",textOverflow:"ellipsis",cursor:"pointer",padding:"1px 3px",lineHeight:1.2}}},"&light .cm-tooltip-autocomplete ul li[aria-selected]":{background:"#17c",color:"white"},"&dark .cm-tooltip-autocomplete ul li[aria-selected]":{background:"#347",color:"white"},".cm-completionListIncompleteTop:before, .cm-completionListIncompleteBottom:after":{content:'"\xB7\xB7\xB7"',opacity:.5,display:"block",textAlign:"center"},".cm-tooltip.cm-completionInfo":{position:"absolute",padding:"3px 9px",width:"max-content",maxWidth:"300px"},".cm-completionInfo.cm-completionInfo-left":{right:"100%"},".cm-completionInfo.cm-completionInfo-right":{left:"100%"},"&light .cm-snippetField":{backgroundColor:"#00000022"},"&dark .cm-snippetField":{backgroundColor:"#ffffff22"},".cm-snippetFieldPosition":{verticalAlign:"text-top",width:0,height:"1.15em",margin:"0 -0.7px -.7em",borderLeft:"1.4px dotted #888"},".cm-completionMatchedText":{textDecoration:"underline"},".cm-completionDetail":{marginLeft:"0.5em",fontStyle:"italic"},".cm-completionIcon":{fontSize:"90%",width:".8em",display:"inline-block",textAlign:"center",paddingRight:".6em",opacity:"0.6"},".cm-completionIcon-function, .cm-completionIcon-method":{"&:after":{content:"'\u0192'"}},".cm-completionIcon-class":{"&:after":{content:"'\u25CB'"}},".cm-completionIcon-interface":{"&:after":{content:"'\u25CC'"}},".cm-completionIcon-variable":{"&:after":{content:"'\u{1D465}'"}},".cm-completionIcon-constant":{"&:after":{content:"'\u{1D436}'"}},".cm-completionIcon-type":{"&:after":{content:"'\u{1D461}'"}},".cm-completionIcon-enum":{"&:after":{content:"'\u222A'"}},".cm-completionIcon-property":{"&:after":{content:"'\u25A1'"}},".cm-completionIcon-keyword":{"&:after":{content:"'\u{1F511}\uFE0E'"}},".cm-completionIcon-namespace":{"&:after":{content:"'\u25A2'"}},".cm-completionIcon-text":{"&:after":{content:"'abc'",fontSize:"50%",verticalAlign:"middle"}}});function LB(t={}){return[$n,ro.of(t),AB,WB,DB]}const mS=[{key:"Ctrl-Space",run:RB},{key:"Escape",run:TB},{key:"ArrowDown",run:$f(!0)},{key:"ArrowUp",run:$f(!1)},{key:"PageDown",run:$f(!0,"page")},{key:"PageUp",run:$f(!1,"page")},{key:"Enter",run:CB}],WB=da.highest(cf.computeN([ro],t=>t.facet(ro).defaultKeymap?[mS]:[]));function sF(t){let e=t.field($n,!1);return e&&e.active.some(n=>n.state==1)?"pending":e&&e.active.some(n=>n.state!=0)?"active":null}const Sf={brackets:["(","[","{","'",'"'],before:`)]}'":;>`},io=ce.define({map(t,e){let n=e.mapPos(t,-1,Dt.TrackAfter);return n==null?void 0:n}}),Lg=ce.define({map(t,e){return e.mapPos(t)}}),Wg=new class extends Gi{};Wg.startSide=1;Wg.endSide=-1;const gS=Rt.define({create(){return Te.empty},update(t,e){if(e.selection){let n=e.state.doc.lineAt(e.selection.main.head).from,r=e.startState.doc.lineAt(e.startState.selection.main.head).from;n!=e.changes.mapPos(r,-1)&&(t=Te.empty)}t=t.map(e.changes);for(let n of e.effects)n.is(io)?t=t.update({add:[Wg.range(n.value,n.value+1)]}):n.is(Lg)&&(t=t.update({filter:r=>r!=n.value}));return t}});function QB(){return[oe.inputHandler.of(NB),gS]}const Qg="()[]{}<>";function yS(t){for(let e=0;e<Qg.length;e+=2)if(Qg.charCodeAt(e)==t)return Qg.charAt(e+1);return wm(t<128?t:t+1)}function OS(t,e){return t.languageDataAt("closeBrackets",e)[0]||Sf}function NB(t,e,n,r){if(t.composing)return!1;let i=t.state.selection.main;if(r.length>2||r.length==2&&wn(Ct(r,0))==1||e!=i.from||n!=i.to)return!1;let o=qB(t.state,r);return o?(t.dispatch(o),!0):!1}const _B=({state:t,dispatch:e})=>{let r=OS(t,t.selection.main.head).brackets||Sf.brackets,i=null,o=t.changeByRange(s=>{if(s.empty){let l=zB(t.doc,s.head);for(let a of r)if(a==l&&kf(t.doc,s.head)==yS(Ct(a,0)))return{changes:{from:s.head-a.length,to:s.head+a.length},range:_.cursor(s.head-a.length),userEvent:"delete.backward"}}return{range:i=s}});return i||e(t.update(o,{scrollIntoView:!0})),!i},BB=[{key:"Backspace",run:_B}];function qB(t,e){let n=OS(t,t.selection.main.head),r=n.brackets||Sf.brackets;for(let i of r){let o=yS(Ct(i,0));if(e==i)return o==i?UB(t,i,r.indexOf(i+i+i)>-1):FB(t,i,o,n.before||Sf.before);if(e==o&&vS(t,t.selection.main.from))return VB(t,i,o)}return null}function vS(t,e){let n=!1;return t.field(gS).between(0,t.doc.length,r=>{r==e&&(n=!0)}),n}function kf(t,e){let n=t.sliceString(e,e+2);return n.slice(0,wn(Ct(n,0)))}function zB(t,e){let n=t.sliceString(e-2,e);return wn(Ct(n,0))==n.length?n:n.slice(1)}function FB(t,e,n,r){let i=null,o=t.changeByRange(s=>{if(!s.empty)return{changes:[{insert:e,from:s.from},{insert:n,from:s.to}],effects:io.of(s.to+e.length),range:_.range(s.anchor+e.length,s.head+e.length)};let l=kf(t.doc,s.head);return!l||/\s/.test(l)||r.indexOf(l)>-1?{changes:{insert:e+n,from:s.head},effects:io.of(s.head+e.length),range:_.cursor(s.head+e.length)}:{range:i=s}});return i?null:t.update(o,{scrollIntoView:!0,userEvent:"input.type"})}function VB(t,e,n){let r=null,i=t.selection.ranges.map(o=>o.empty&&kf(t.doc,o.head)==n?_.cursor(o.head+n.length):r=o);return r?null:t.update({selection:_.create(i,t.selection.mainIndex),scrollIntoView:!0,effects:t.selection.ranges.map(({from:o})=>Lg.of(o))})}function UB(t,e,n){let r=null,i=t.changeByRange(o=>{if(!o.empty)return{changes:[{insert:e,from:o.from},{insert:e,from:o.to}],effects:io.of(o.to+e.length),range:_.range(o.anchor+e.length,o.head+e.length)};let s=o.head,l=kf(t.doc,s);if(l==e){if(xS(t,s))return{changes:{insert:e+e,from:s},effects:io.of(s+e.length),range:_.cursor(s+e.length)};if(vS(t,s)){let a=n&&t.sliceDoc(s,s+e.length*3)==e+e+e;return{range:_.cursor(s+e.length*(a?3:1)),effects:Lg.of(s)}}}else{if(n&&t.sliceDoc(s-2*e.length,s)==e+e&&xS(t,s-2*e.length))return{changes:{insert:e+e+e+e,from:s},effects:io.of(s+e.length),range:_.cursor(s+e.length)};if(t.charCategorizer(s)(l)!=rn.Word){let a=t.sliceDoc(s-1,s);if(a!=e&&t.charCategorizer(s)(a)!=rn.Word)return{changes:{insert:e+e,from:s},effects:io.of(s+e.length),range:_.cursor(s+e.length)}}}return{range:r=o}});return r?null:t.update(i,{scrollIntoView:!0,userEvent:"input.type"})}function xS(t,e){let n=an(t).resolveInner(e+1);return n.parent&&n.from==e}const jB=oe.baseTheme({".cm-matchingBracket":{backgroundColor:"#328c8252"},".cm-nonmatchingBracket":{backgroundColor:"#bb555544"}}),wS=1e4,bS="()[]{}",$S=ne.define({combine(t){return mi(t,{afterCursor:!0,brackets:bS,maxScanDistance:wS})}}),XB=te.mark({class:"cm-matchingBracket"}),HB=te.mark({class:"cm-nonmatchingBracket"}),GB=Rt.define({create(){return te.none},update(t,e){if(!e.docChanged&&!e.selection)return t;let n=[],r=e.state.facet($S);for(let i of e.state.selection.ranges){if(!i.empty)continue;let o=nr(e.state,i.head,-1,r)||i.head>0&&nr(e.state,i.head-1,1,r)||r.afterCursor&&(nr(e.state,i.head,1,r)||i.head<e.state.doc.length&&nr(e.state,i.head+1,-1,r));if(!o)continue;let s=o.matched?XB:HB;n.push(s.range(o.start.from,o.start.to)),o.end&&n.push(s.range(o.end.from,o.end.to))}return te.set(n,!0)},provide:t=>oe.decorations.from(t)}),ZB=[GB,jB];function YB(t={}){return[$S.of(t),ZB]}function Ng(t,e,n){let r=t.prop(e<0?xe.openedBy:xe.closedBy);if(r)return r;if(t.name.length==1){let i=n.indexOf(t.name);if(i>-1&&i%2==(e<0?1:0))return[n[i+e]]}return null}function nr(t,e,n,r={}){let i=r.maxScanDistance||wS,o=r.brackets||bS,s=an(t),l=s.resolveInner(e,n);for(let a=l;a;a=a.parent){let c=Ng(a.type,n,o);if(c&&a.from<a.to)return KB(t,e,n,a,c,o)}return JB(t,e,n,s,l.type,i,o)}function KB(t,e,n,r,i,o){let s=r.parent,l={from:r.from,to:r.to},a=0,c=s==null?void 0:s.cursor;if(c&&(n<0?c.childBefore(r.from):c.childAfter(r.to)))do if(n<0?c.to<=r.from:c.from>=r.to){if(a==0&&i.indexOf(c.type.name)>-1&&c.from<c.to)return{start:l,end:{from:c.from,to:c.to},matched:!0};if(Ng(c.type,n,o))a++;else if(Ng(c.type,-n,o)&&(a--,a==0))return{start:l,end:c.from==c.to?void 0:{from:c.from,to:c.to},matched:!1}}while(n<0?c.prevSibling():c.nextSibling());return{start:l,matched:!1}}function JB(t,e,n,r,i,o,s){let l=n<0?t.sliceDoc(e-1,e):t.sliceDoc(e,e+1),a=s.indexOf(l);if(a<0||a%2==0!=n>0)return null;let c={from:n<0?e-1:e,to:n>0?e+1:e},u=t.doc.iterRange(e,n>0?t.doc.length:0),f=0;for(let h=0;!u.next().done&&h<=o;){let d=u.value;n<0&&(h+=d.length);let p=e+h*n;for(let m=n>0?0:d.length-1,g=n>0?d.length:-1;m!=g;m+=n){let y=s.indexOf(d[m]);if(!(y<0||r.resolve(p+m,1).type!=i))if(y%2==0==n>0)f++;else{if(f==1)return{start:c,end:{from:p+m,to:p+m+1},matched:y>>1==a>>1};f--}}n>0&&(h+=d.length)}return u.done?{start:c,matched:!1}:null}function ws(t,e){return _.create(t.ranges.map(e),t.mainIndex)}function rr(t,e){return t.update({selection:e,scrollIntoView:!0,userEvent:"select"})}function Er({state:t,dispatch:e},n){let r=ws(t.selection,n);return r.eq(t.selection)?!1:(e(rr(t,r)),!0)}function Pf(t,e){return _.cursor(e?t.to:t.from)}function SS(t,e){return Er(t,n=>n.empty?t.moveByChar(n,e):Pf(n,e))}const kS=t=>SS(t,t.textDirection!=Se.LTR),PS=t=>SS(t,t.textDirection==Se.LTR);function CS(t,e){return Er(t,n=>n.empty?t.moveByGroup(n,e):Pf(n,e))}const eq=t=>CS(t,t.textDirection!=Se.LTR),tq=t=>CS(t,t.textDirection==Se.LTR);function nq(t,e,n){if(e.type.prop(n))return!0;let r=e.to-e.from;return r&&(r>2||/[^\s,.;:]/.test(t.sliceDoc(e.from,e.to)))||e.firstChild}function Cf(t,e,n){let r=an(t).resolveInner(e.head),i=n?xe.closedBy:xe.openedBy;for(let a=e.head;;){let c=n?r.childAfter(a):r.childBefore(a);if(!c)break;nq(t,c,i)?r=c:a=n?c.to:c.from}let o=r.type.prop(i),s,l;return o&&(s=n?nr(t,r.from,1):nr(t,r.to,-1))&&s.matched?l=n?s.end.to:s.end.from:l=n?r.to:r.from,_.cursor(l,n?-1:1)}const rq=t=>Er(t,e=>Cf(t.state,e,t.textDirection!=Se.LTR)),iq=t=>Er(t,e=>Cf(t.state,e,t.textDirection==Se.LTR));function RS(t,e){return Er(t,n=>{if(!n.empty)return Pf(n,e);let r=t.moveVertically(n,e);return r.head!=n.head?r:t.moveToLineBoundary(n,e)})}const TS=t=>RS(t,!1),IS=t=>RS(t,!0);function ES(t,e){let{state:n}=t,r=ws(n.selection,s=>s.empty?t.moveVertically(s,e,t.dom.clientHeight):Pf(s,e));if(r.eq(n.selection))return!1;let i=t.coordsAtPos(n.selection.main.head),o=t.scrollDOM.getBoundingClientRect();return t.dispatch(rr(n,r),{effects:i&&i.top>o.top&&i.bottom<o.bottom?oe.scrollIntoView(r.main.head,{y:"start",yMargin:i.top-o.top}):void 0}),!0}const _g=t=>ES(t,!1),Bg=t=>ES(t,!0);function Rf(t,e,n){let r=t.lineBlockAt(e.head),i=t.moveToLineBoundary(e,n);if(i.head==e.head&&i.head!=(n?r.to:r.from)&&(i=t.moveToLineBoundary(e,n,!1)),!n&&i.head==r.from&&r.length){let o=/^\s*/.exec(t.state.sliceDoc(r.from,Math.min(r.from+100,r.to)))[0].length;o&&e.head!=r.from+o&&(i=_.cursor(r.from+o))}return i}const MS=t=>Er(t,e=>Rf(t,e,!0)),AS=t=>Er(t,e=>Rf(t,e,!1)),oq=t=>Er(t,e=>_.cursor(t.lineBlockAt(e.head).from,1)),sq=t=>Er(t,e=>_.cursor(t.lineBlockAt(e.head).to,-1));function lq(t,e,n){let r=!1,i=ws(t.selection,o=>{let s=nr(t,o.head,-1)||nr(t,o.head,1)||o.head>0&&nr(t,o.head-1,1)||o.head<t.doc.length&&nr(t,o.head+1,-1);if(!s||!s.end)return o;r=!0;let l=s.start.from==o.head?s.end.to:s.end.from;return n?_.range(o.anchor,l):_.cursor(l)});return r?(e(rr(t,i)),!0):!1}const aq=({state:t,dispatch:e})=>lq(t,e,!1);function ir(t,e){let n=ws(t.state.selection,r=>{let i=e(r);return _.range(r.anchor,i.head,i.goalColumn)});return n.eq(t.state.selection)?!1:(t.dispatch(rr(t.state,n)),!0)}function DS(t,e){return ir(t,n=>t.moveByChar(n,e))}const LS=t=>DS(t,t.textDirection!=Se.LTR),WS=t=>DS(t,t.textDirection==Se.LTR);function QS(t,e){return ir(t,n=>t.moveByGroup(n,e))}const cq=t=>QS(t,t.textDirection!=Se.LTR),uq=t=>QS(t,t.textDirection==Se.LTR),fq=t=>ir(t,e=>Cf(t.state,e,t.textDirection!=Se.LTR)),hq=t=>ir(t,e=>Cf(t.state,e,t.textDirection==Se.LTR));function NS(t,e){return ir(t,n=>t.moveVertically(n,e))}const _S=t=>NS(t,!1),BS=t=>NS(t,!0);function qS(t,e){return ir(t,n=>t.moveVertically(n,e,t.dom.clientHeight))}const zS=t=>qS(t,!1),FS=t=>qS(t,!0),VS=t=>ir(t,e=>Rf(t,e,!0)),US=t=>ir(t,e=>Rf(t,e,!1)),dq=t=>ir(t,e=>_.cursor(t.lineBlockAt(e.head).from)),pq=t=>ir(t,e=>_.cursor(t.lineBlockAt(e.head).to)),jS=({state:t,dispatch:e})=>(e(rr(t,{anchor:0})),!0),XS=({state:t,dispatch:e})=>(e(rr(t,{anchor:t.doc.length})),!0),HS=({state:t,dispatch:e})=>(e(rr(t,{anchor:t.selection.main.anchor,head:0})),!0),GS=({state:t,dispatch:e})=>(e(rr(t,{anchor:t.selection.main.anchor,head:t.doc.length})),!0),mq=({state:t,dispatch:e})=>(e(t.update({selection:{anchor:0,head:t.doc.length},userEvent:"select"})),!0),gq=({state:t,dispatch:e})=>{let n=Ef(t).map(({from:r,to:i})=>_.range(r,Math.min(i+1,t.doc.length)));return e(t.update({selection:_.create(n),userEvent:"select"})),!0},yq=({state:t,dispatch:e})=>{let n=ws(t.selection,r=>{var i;let o=an(t).resolveInner(r.head,1);for(;!(o.from<r.from&&o.to>=r.to||o.to>r.to&&o.from<=r.from||!((i=o.parent)===null||i===void 0?void 0:i.parent));)o=o.parent;return _.range(o.to,o.from)});return e(rr(t,n)),!0},Oq=({state:t,dispatch:e})=>{let n=t.selection,r=null;return n.ranges.length>1?r=_.create([n.main]):n.main.empty||(r=_.create([_.cursor(n.main.head)])),r?(e(rr(t,r)),!0):!1};function Tf({state:t,dispatch:e},n){if(t.readOnly)return!1;let r="delete.selection",i=t.changeByRange(o=>{let{from:s,to:l}=o;if(s==l){let a=n(s);a<s?r="delete.backward":a>s&&(r="delete.forward"),s=Math.min(s,a),l=Math.max(l,a)}return s==l?{range:o}:{changes:{from:s,to:l},range:_.cursor(s)}});return i.changes.empty?!1:(e(t.update(i,{scrollIntoView:!0,userEvent:r})),!0)}function If(t,e,n){if(t instanceof oe)for(let r of t.pluginField(It.atomicRanges))r.between(e,e,(i,o)=>{i<e&&o>e&&(e=n?o:i)});return e}const ZS=(t,e)=>Tf(t,n=>{let{state:r}=t,i=r.doc.lineAt(n),o,s;if(!e&&n>i.from&&n<i.from+200&&!/[^ \t]/.test(o=i.text.slice(0,n-i.from))){if(o[o.length-1]=="	")return n-1;let l=ua(o,r.tabSize),a=l%Of(r)||Of(r);for(let c=0;c<a&&o[o.length-1-c]==" ";c++)n--;s=n}else s=nn(i.text,n-i.from,e)+i.from,s==n&&i.number!=(e?r.doc.lines:1)&&(s+=e?1:-1);return If(t,s,e)}),qg=t=>ZS(t,!1),YS=t=>ZS(t,!0),KS=(t,e)=>Tf(t,n=>{let r=n,{state:i}=t,o=i.doc.lineAt(r),s=i.charCategorizer(r);for(let l=null;;){if(r==(e?o.to:o.from)){r==n&&o.number!=(e?i.doc.lines:1)&&(r+=e?1:-1);break}let a=nn(o.text,r-o.from,e)+o.from,c=o.text.slice(Math.min(r,a)-o.from,Math.max(r,a)-o.from),u=s(c);if(l!=null&&u!=l)break;(c!=" "||r!=n)&&(l=u),r=a}return If(t,r,e)}),JS=t=>KS(t,!1),vq=t=>KS(t,!0),ek=t=>Tf(t,e=>{let n=t.lineBlockAt(e).to;return If(t,e<n?n:Math.min(t.state.doc.length,e+1),!0)}),xq=t=>Tf(t,e=>{let n=t.lineBlockAt(e).from;return If(t,e>n?n:Math.max(0,e-1),!1)}),wq=({state:t,dispatch:e})=>{if(t.readOnly)return!1;let n=t.changeByRange(r=>({changes:{from:r.from,to:r.to,insert:Ie.of(["",""])},range:_.cursor(r.from)}));return e(t.update(n,{scrollIntoView:!0,userEvent:"input"})),!0},bq=({state:t,dispatch:e})=>{if(t.readOnly)return!1;let n=t.changeByRange(r=>{if(!r.empty||r.from==0||r.from==t.doc.length)return{range:r};let i=r.from,o=t.doc.lineAt(i),s=i==o.from?i-1:nn(o.text,i-o.from,!1)+o.from,l=i==o.to?i+1:nn(o.text,i-o.from,!0)+o.from;return{changes:{from:s,to:l,insert:t.doc.slice(i,l).append(t.doc.slice(s,i))},range:_.cursor(l)}});return n.changes.empty?!1:(e(t.update(n,{scrollIntoView:!0,userEvent:"move.character"})),!0)};function Ef(t){let e=[],n=-1;for(let r of t.selection.ranges){let i=t.doc.lineAt(r.from),o=t.doc.lineAt(r.to);if(!r.empty&&r.to==o.from&&(o=t.doc.lineAt(r.to-1)),n>=i.number){let s=e[e.length-1];s.to=o.to,s.ranges.push(r)}else e.push({from:i.from,to:o.to,ranges:[r]});n=o.number+1}return e}function tk(t,e,n){if(t.readOnly)return!1;let r=[],i=[];for(let o of Ef(t)){if(n?o.to==t.doc.length:o.from==0)continue;let s=t.doc.lineAt(n?o.to+1:o.from-1),l=s.length+1;if(n){r.push({from:o.to,to:s.to},{from:o.from,insert:s.text+t.lineBreak});for(let a of o.ranges)i.push(_.range(Math.min(t.doc.length,a.anchor+l),Math.min(t.doc.length,a.head+l)))}else{r.push({from:s.from,to:o.from},{from:o.to,insert:t.lineBreak+s.text});for(let a of o.ranges)i.push(_.range(a.anchor-l,a.head-l))}}return r.length?(e(t.update({changes:r,scrollIntoView:!0,selection:_.create(i,t.selection.mainIndex),userEvent:"move.line"})),!0):!1}const $q=({state:t,dispatch:e})=>tk(t,e,!1),Sq=({state:t,dispatch:e})=>tk(t,e,!0);function nk(t,e,n){if(t.readOnly)return!1;let r=[];for(let i of Ef(t))n?r.push({from:i.from,insert:t.doc.slice(i.from,i.to)+t.lineBreak}):r.push({from:i.to,insert:t.lineBreak+t.doc.slice(i.from,i.to)});return e(t.update({changes:r,scrollIntoView:!0,userEvent:"input.copyline"})),!0}const kq=({state:t,dispatch:e})=>nk(t,e,!1),Pq=({state:t,dispatch:e})=>nk(t,e,!0),Cq=t=>{if(t.state.readOnly)return!1;let{state:e}=t,n=e.changes(Ef(e).map(({from:i,to:o})=>(i>0?i--:o<e.doc.length&&o++,{from:i,to:o}))),r=ws(e.selection,i=>t.moveVertically(i,!0)).map(n);return t.dispatch({changes:n,selection:r,scrollIntoView:!0,userEvent:"delete.line"}),!0};function Rq(t,e){if(/\(\)|\[\]|\{\}/.test(t.sliceDoc(e-1,e+1)))return{from:e,to:e};let n=an(t).resolveInner(e),r=n.childBefore(e),i=n.childAfter(e),o;return r&&i&&r.to<=e&&i.from>=e&&(o=r.type.prop(xe.closedBy))&&o.indexOf(i.name)>-1&&t.doc.lineAt(r.to).from==t.doc.lineAt(i.from).from?{from:r.to,to:i.from}:null}const Tq=rk(!1),Iq=rk(!0);function rk(t){return({state:e,dispatch:n})=>{if(e.readOnly)return!1;let r=e.changeByRange(i=>{let{from:o,to:s}=i,l=e.doc.lineAt(o),a=!t&&o==s&&Rq(e,o);t&&(o=s=(s<=l.to?l:e.doc.lineAt(s)).to);let c=new vf(e,{simulateBreak:o,simulateDoubleBreak:!!a}),u=Ig(c,o);for(u==null&&(u=/^\s*/.exec(e.doc.lineAt(o).text)[0].length);s<l.to&&/\s/.test(l.text[s-l.from]);)s++;a?{from:o,to:s}=a:o>l.from&&o<l.from+100&&!/\S/.test(l.text.slice(0,o))&&(o=l.from);let f=["",Ra(e,u)];return a&&f.push(Ra(e,c.lineIndent(l.from,-1))),{changes:{from:o,to:s,insert:Ie.of(f)},range:_.cursor(o+1+f[1].length)}});return n(e.update(r,{scrollIntoView:!0,userEvent:"input"})),!0}}function zg(t,e){let n=-1;return t.changeByRange(r=>{let i=[];for(let s=r.from;s<=r.to;){let l=t.doc.lineAt(s);l.number>n&&(r.empty||r.to>l.from)&&(e(l,i,r),n=l.number),s=l.to+1}let o=t.changes(i);return{changes:i,range:_.range(o.mapPos(r.anchor,1),o.mapPos(r.head,1))}})}const Eq=({state:t,dispatch:e})=>{if(t.readOnly)return!1;let n=Object.create(null),r=new vf(t,{overrideIndentation:o=>{let s=n[o];return s==null?-1:s}}),i=zg(t,(o,s,l)=>{let a=Ig(r,o.from);if(a==null)return;/\S/.test(o.text)||(a=0);let c=/^\s*/.exec(o.text)[0],u=Ra(t,a);(c!=u||l.from<o.from+c.length)&&(n[o.from]=a,s.push({from:o.from,to:o.from+c.length,insert:u}))});return i.changes.empty||e(t.update(i,{userEvent:"indent"})),!0},ik=({state:t,dispatch:e})=>t.readOnly?!1:(e(t.update(zg(t,(n,r)=>{r.push({from:n.from,insert:t.facet(Tg)})}),{userEvent:"input.indent"})),!0),ok=({state:t,dispatch:e})=>t.readOnly?!1:(e(t.update(zg(t,(n,r)=>{let i=/^\s*/.exec(n.text)[0];if(!i)return;let o=ua(i,t.tabSize),s=0,l=Ra(t,Math.max(0,o-Of(t)));for(;s<i.length&&s<l.length&&i.charCodeAt(s)==l.charCodeAt(s);)s++;r.push({from:n.from+s,to:n.from+i.length,insert:l.slice(s)})}),{userEvent:"delete.dedent"})),!0),Mq=[{key:"Ctrl-b",run:kS,shift:LS,preventDefault:!0},{key:"Ctrl-f",run:PS,shift:WS},{key:"Ctrl-p",run:TS,shift:_S},{key:"Ctrl-n",run:IS,shift:BS},{key:"Ctrl-a",run:oq,shift:dq},{key:"Ctrl-e",run:sq,shift:pq},{key:"Ctrl-d",run:YS},{key:"Ctrl-h",run:qg},{key:"Ctrl-k",run:ek},{key:"Ctrl-Alt-h",run:JS},{key:"Ctrl-o",run:wq},{key:"Ctrl-t",run:bq},{key:"Ctrl-v",run:Bg},{key:"Alt-v",run:_g}],Aq=[{key:"ArrowLeft",run:kS,shift:LS,preventDefault:!0},{key:"Mod-ArrowLeft",mac:"Alt-ArrowLeft",run:eq,shift:cq},{mac:"Cmd-ArrowLeft",run:AS,shift:US},{key:"ArrowRight",run:PS,shift:WS,preventDefault:!0},{key:"Mod-ArrowRight",mac:"Alt-ArrowRight",run:tq,shift:uq},{mac:"Cmd-ArrowRight",run:MS,shift:VS},{key:"ArrowUp",run:TS,shift:_S,preventDefault:!0},{mac:"Cmd-ArrowUp",run:jS,shift:HS},{mac:"Ctrl-ArrowUp",run:_g,shift:zS},{key:"ArrowDown",run:IS,shift:BS,preventDefault:!0},{mac:"Cmd-ArrowDown",run:XS,shift:GS},{mac:"Ctrl-ArrowDown",run:Bg,shift:FS},{key:"PageUp",run:_g,shift:zS},{key:"PageDown",run:Bg,shift:FS},{key:"Home",run:AS,shift:US},{key:"Mod-Home",run:jS,shift:HS},{key:"End",run:MS,shift:VS},{key:"Mod-End",run:XS,shift:GS},{key:"Enter",run:Tq},{key:"Mod-a",run:mq},{key:"Backspace",run:qg,shift:qg},{key:"Delete",run:YS},{key:"Mod-Backspace",mac:"Alt-Backspace",run:JS},{key:"Mod-Delete",mac:"Alt-Delete",run:vq},{mac:"Mod-Backspace",run:xq},{mac:"Mod-Delete",run:ek}].concat(Mq.map(t=>({mac:t.key,run:t.run,shift:t.shift}))),Dq=[{key:"Alt-ArrowLeft",mac:"Ctrl-ArrowLeft",run:rq,shift:fq},{key:"Alt-ArrowRight",mac:"Ctrl-ArrowRight",run:iq,shift:hq},{key:"Alt-ArrowUp",run:$q},{key:"Shift-Alt-ArrowUp",run:kq},{key:"Alt-ArrowDown",run:Sq},{key:"Shift-Alt-ArrowDown",run:Pq},{key:"Escape",run:Oq},{key:"Mod-Enter",run:Iq},{key:"Alt-l",mac:"Ctrl-l",run:gq},{key:"Mod-i",run:yq,preventDefault:!0},{key:"Mod-[",run:ok},{key:"Mod-]",run:ik},{key:"Mod-Alt-\\",run:Eq},{key:"Shift-Mod-k",run:Cq},{key:"Shift-Mod-\\",run:aq}].concat(Aq),Lq={key:"Tab",run:ik,shift:ok},Wq=t=>{let e=Fg(t.state);return e.line?Qq(t):e.block?lk(t):!1};function sk(t,e){return({state:n,dispatch:r})=>{let i=t(e,n.selection.ranges,n);return i?(r(n.update(i)),!0):!1}}const Qq=sk(qq,0),lk=sk(Bq,0),Nq=[{key:"Mod-/",run:Wq},{key:"Alt-A",run:lk}];function Fg(t,e=t.selection.main.head){let n=t.languageDataAt("commentTokens",e);return n.length?n[0]:{}}const Ia=50;function _q(t,{open:e,close:n},r,i){let o=t.sliceDoc(r-Ia,r),s=t.sliceDoc(i,i+Ia),l=/\s*$/.exec(o)[0].length,a=/^\s*/.exec(s)[0].length,c=o.length-l;if(o.slice(c-e.length,c)==e&&s.slice(a,a+n.length)==n)return{open:{pos:r-l,margin:l&&1},close:{pos:i+a,margin:a&&1}};let u,f;i-r<=2*Ia?u=f=t.sliceDoc(r,i):(u=t.sliceDoc(r,r+Ia),f=t.sliceDoc(i-Ia,i));let h=/^\s*/.exec(u)[0].length,d=/\s*$/.exec(f)[0].length,p=f.length-d-n.length;return u.slice(h,h+e.length)==e&&f.slice(p,p+n.length)==n?{open:{pos:r+h+e.length,margin:/\s/.test(u.charAt(h+e.length))?1:0},close:{pos:i-d-n.length,margin:/\s/.test(f.charAt(p-1))?1:0}}:null}function Bq(t,e,n){let r=e.map(o=>Fg(n,o.from).block);if(!r.every(o=>o))return null;let i=e.map((o,s)=>_q(n,r[s],o.from,o.to));if(t!=2&&!i.every(o=>o)){let o=0;return n.changeByRange(s=>{let{open:l,close:a}=r[o++];if(i[o])return{range:s};let c=l.length+1;return{changes:[{from:s.from,insert:l+" "},{from:s.to,insert:" "+a}],range:_.range(s.anchor+c,s.head+c)}})}else if(t!=1&&i.some(o=>o)){let o=[];for(let s=0,l;s<i.length;s++)if(l=i[s]){let a=r[s],{open:c,close:u}=l;o.push({from:c.pos-a.open.length,to:c.pos+c.margin},{from:u.pos-u.margin,to:u.pos+a.close.length})}return{changes:o}}return null}function qq(t,e,n){let r=[],i=-1;for(let{from:o,to:s}of e){let l=r.length,a=1e9;for(let c=o;c<=s;){let u=n.doc.lineAt(c);if(u.from>i&&(o==s||s>u.from)){i=u.from;let f=Fg(n,c).line;if(!f)continue;let h=/^\s*/.exec(u.text)[0].length,d=h==u.length,p=u.text.slice(h,h+f.length)==f?h:-1;h<u.text.length&&h<a&&(a=h),r.push({line:u,comment:p,token:f,indent:h,empty:d,single:!1})}c=u.to+1}if(a<1e9)for(let c=l;c<r.length;c++)r[c].indent<r[c].line.text.length&&(r[c].indent=a);r.length==l+1&&(r[l].single=!0)}if(t!=2&&r.some(o=>o.comment<0&&(!o.empty||o.single))){let o=[];for(let{line:l,token:a,indent:c,empty:u,single:f}of r)(f||!u)&&o.push({from:l.from+c,insert:a+" "});let s=n.changes(o);return{changes:s,selection:n.selection.map(s,1)}}else if(t!=1&&r.some(o=>o.comment>=0)){let o=[];for(let{line:s,comment:l,token:a}of r)if(l>=0){let c=s.from+l,u=c+a.length;s.text[u-s.from]==" "&&u++,o.push({from:c,to:u})}return{changes:o}}return null}class Mr extends Gi{compare(e){return this==e||this.constructor==e.constructor&&this.eq(e)}eq(e){return!1}destroy(e){}}Mr.prototype.elementClass="";Mr.prototype.toDOM=void 0;Mr.prototype.mapMode=Dt.TrackBefore;Mr.prototype.startSide=Mr.prototype.endSide=-1;Mr.prototype.point=!0;const Mf=ne.define(),zq={class:"",renderEmptyElements:!1,elementStyle:"",markers:()=>Te.empty,lineMarker:()=>null,lineMarkerChange:null,initialSpacer:null,updateSpacer:null,domEventHandlers:{}},Ea=ne.define();function Fq(t){return[ak(),Ea.of(Object.assign(Object.assign({},zq),t))]}const Vq=oe.baseTheme({".cm-gutters":{display:"flex",height:"100%",boxSizing:"border-box",left:0,zIndex:200},"&light .cm-gutters":{backgroundColor:"#f5f5f5",color:"#999",borderRight:"1px solid #ddd"},"&dark .cm-gutters":{backgroundColor:"#333338",color:"#ccc"},".cm-gutter":{display:"flex !important",flexDirection:"column",flexShrink:0,boxSizing:"border-box",minHeight:"100%",overflow:"hidden"},".cm-gutterElement":{boxSizing:"border-box"},".cm-lineNumbers .cm-gutterElement":{padding:"0 3px 0 5px",minWidth:"20px",textAlign:"right",whiteSpace:"nowrap"},"&light .cm-activeLineGutter":{backgroundColor:"#e2f2ff"},"&dark .cm-activeLineGutter":{backgroundColor:"#222227"}}),Vg=ne.define({combine:t=>t.some(e=>e)});function ak(t){let e=[Uq,Vq];return t&&t.fixed===!1&&e.push(Vg.of(!0)),e}const Uq=rt.fromClass(class{constructor(t){this.view=t,this.prevViewport=t.viewport,this.dom=document.createElement("div"),this.dom.className="cm-gutters",this.dom.setAttribute("aria-hidden","true"),this.dom.style.minHeight=this.view.contentHeight+"px",this.gutters=t.state.facet(Ea).map(e=>new fk(t,e));for(let e of this.gutters)this.dom.appendChild(e.dom);this.fixed=!t.state.facet(Vg),this.fixed&&(this.dom.style.position="sticky"),this.syncGutters(!1),t.scrollDOM.insertBefore(this.dom,t.contentDOM)}update(t){if(this.updateGutters(t)){let e=this.prevViewport,n=t.view.viewport,r=Math.min(e.to,n.to)-Math.max(e.from,n.from);this.syncGutters(r<(n.to-n.from)*.8)}t.geometryChanged&&(this.dom.style.minHeight=this.view.contentHeight+"px"),this.view.state.facet(Vg)!=!this.fixed&&(this.fixed=!this.fixed,this.dom.style.position=this.fixed?"sticky":""),this.prevViewport=t.view.viewport}syncGutters(t){let e=this.dom.nextSibling;t&&this.dom.remove();let n=Te.iter(this.view.state.facet(Mf),this.view.viewport.from),r=[],i=this.gutters.map(o=>new jq(o,this.view.viewport,-this.view.documentPadding.top));for(let o of this.view.viewportLineBlocks){let s;if(Array.isArray(o.type)){for(let l of o.type)if(l.type==De.Text){s=l;break}}else s=o.type==De.Text?o:void 0;if(!!s){r.length&&(r=[]),uk(n,r,o.from);for(let l of i)l.line(this.view,s,r)}}for(let o of i)o.finish();t&&this.view.scrollDOM.insertBefore(this.dom,e)}updateGutters(t){let e=t.startState.facet(Ea),n=t.state.facet(Ea),r=t.docChanged||t.heightChanged||t.viewportChanged||!Te.eq(t.startState.facet(Mf),t.state.facet(Mf),t.view.viewport.from,t.view.viewport.to);if(e==n)for(let i of this.gutters)i.update(t)&&(r=!0);else{r=!0;let i=[];for(let o of n){let s=e.indexOf(o);s<0?i.push(new fk(this.view,o)):(this.gutters[s].update(t),i.push(this.gutters[s]))}for(let o of this.gutters)o.dom.remove(),i.indexOf(o)<0&&o.destroy();for(let o of i)this.dom.appendChild(o.dom);this.gutters=i}return r}destroy(){for(let t of this.gutters)t.destroy();this.dom.remove()}},{provide:It.scrollMargins.from(t=>t.gutters.length==0||!t.fixed?null:t.view.textDirection==Se.LTR?{left:t.dom.offsetWidth}:{right:t.dom.offsetWidth})});function ck(t){return Array.isArray(t)?t:[t]}function uk(t,e,n){for(;t.value&&t.from<=n;)t.from==n&&e.push(t.value),t.next()}class jq{constructor(e,n,r){this.gutter=e,this.height=r,this.localMarkers=[],this.i=0,this.cursor=Te.iter(e.markers,n.from)}line(e,n,r){this.localMarkers.length&&(this.localMarkers=[]),uk(this.cursor,this.localMarkers,n.from);let i=r.length?this.localMarkers.concat(r):this.localMarkers,o=this.gutter.config.lineMarker(e,n,i);o&&i.unshift(o);let s=this.gutter;if(i.length==0&&!s.config.renderEmptyElements)return;let l=n.top-this.height;if(this.i==s.elements.length){let a=new hk(e,n.height,l,i);s.elements.push(a),s.dom.appendChild(a.dom)}else s.elements[this.i].update(e,n.height,l,i);this.height=n.bottom,this.i++}finish(){let e=this.gutter;for(;e.elements.length>this.i;){let n=e.elements.pop();e.dom.removeChild(n.dom),n.destroy()}}}class fk{constructor(e,n){this.view=e,this.config=n,this.elements=[],this.spacer=null,this.dom=document.createElement("div"),this.dom.className="cm-gutter"+(this.config.class?" "+this.config.class:"");for(let r in n.domEventHandlers)this.dom.addEventListener(r,i=>{let o=e.lineBlockAtHeight(i.clientY-e.documentTop);n.domEventHandlers[r](e,o,i)&&i.preventDefault()});this.markers=ck(n.markers(e)),n.initialSpacer&&(this.spacer=new hk(e,0,0,[n.initialSpacer(e)]),this.dom.appendChild(this.spacer.dom),this.spacer.dom.style.cssText+="visibility: hidden; pointer-events: none")}update(e){let n=this.markers;if(this.markers=ck(this.config.markers(e.view)),this.spacer&&this.config.updateSpacer){let i=this.config.updateSpacer(this.spacer.markers[0],e);i!=this.spacer.markers[0]&&this.spacer.update(e.view,0,0,[i])}let r=e.view.viewport;return!Te.eq(this.markers,n,r.from,r.to)||(this.config.lineMarkerChange?this.config.lineMarkerChange(e):!1)}destroy(){for(let e of this.elements)e.destroy()}}class hk{constructor(e,n,r,i){this.height=-1,this.above=0,this.markers=[],this.dom=document.createElement("div"),this.update(e,n,r,i)}update(e,n,r,i){this.height!=n&&(this.dom.style.height=(this.height=n)+"px"),this.above!=r&&(this.dom.style.marginTop=(this.above=r)?r+"px":""),Xq(this.markers,i)||this.setMarkers(e,i)}setMarkers(e,n){let r="cm-gutterElement",i=this.dom.firstChild;for(let o=0,s=0;;){let l=s,a=o<n.length?n[o++]:null,c=!1;if(a){let u=a.elementClass;u&&(r+=" "+u);for(let f=s;f<this.markers.length;f++)if(this.markers[f].compare(a)){l=f,c=!0;break}}else l=this.markers.length;for(;s<l;){let u=this.markers[s++];if(u.toDOM){u.destroy(i);let f=i.nextSibling;i.remove(),i=f}}if(!a)break;a.toDOM&&(c?i=i.nextSibling:this.dom.insertBefore(a.toDOM(e),i)),c&&s++}this.dom.className=r,this.markers=n}destroy(){this.setMarkers(null,[])}}function Xq(t,e){if(t.length!=e.length)return!1;for(let n=0;n<t.length;n++)if(!t[n].compare(e[n]))return!1;return!0}const Hq=ne.define(),bs=ne.define({combine(t){return mi(t,{formatNumber:String,domEventHandlers:{}},{domEventHandlers(e,n){let r=Object.assign({},e);for(let i in n){let o=r[i],s=n[i];r[i]=o?(l,a,c)=>o(l,a,c)||s(l,a,c):s}return r}})}});class Ug extends Mr{constructor(e){super();this.number=e}eq(e){return this.number==e.number}toDOM(){return document.createTextNode(this.number)}}function jg(t,e){return t.state.facet(bs).formatNumber(e,t.state)}const Gq=Ea.compute([bs],t=>({class:"cm-lineNumbers",renderEmptyElements:!1,markers(e){return e.state.facet(Hq)},lineMarker(e,n,r){return r.some(i=>i.toDOM)?null:new Ug(jg(e,e.state.doc.lineAt(n.from).number))},lineMarkerChange:e=>e.startState.facet(bs)!=e.state.facet(bs),initialSpacer(e){return new Ug(jg(e,dk(e.state.doc.lines)))},updateSpacer(e,n){let r=jg(n.view,dk(n.view.state.doc.lines));return r==e.number?e:new Ug(r)},domEventHandlers:t.facet(bs).domEventHandlers}));function Zq(t={}){return[bs.of(t),ak(),Gq]}function dk(t){let e=9;for(;e<t;)e=e*10+9;return e}const Yq=new class extends Mr{constructor(){super(...arguments);this.elementClass="cm-activeLineGutter"}},Kq=Mf.compute(["selection"],t=>{let e=[],n=-1;for(let r of t.selection.ranges)if(r.empty){let i=t.doc.lineAt(r.head).from;i>n&&(n=i,e.push(Yq.range(i)))}return Te.of(e)});function Jq(){return Kq}function pk(t,e){let n=e.mapPos(t.from,1),r=e.mapPos(t.to,-1);return n>=r?void 0:{from:n,to:r}}const Af=ce.define({map:pk}),Ma=ce.define({map:pk});function mk(t){let e=[];for(let{head:n}of t.state.selection.ranges)e.some(r=>r.from<=n&&r.to>=n)||e.push(t.lineBlockAt(n));return e}const oo=Rt.define({create(){return te.none},update(t,e){t=t.map(e.changes);for(let n of e.effects)n.is(Af)&&!ez(t,n.value.from,n.value.to)?t=t.update({add:[lz.range(n.value.from,n.value.to)]}):n.is(Ma)&&(t=t.update({filter:(r,i)=>n.value.from!=r||n.value.to!=i,filterFrom:n.value.from,filterTo:n.value.to}));if(e.selection){let n=!1,{head:r}=e.selection.main;t.between(r,r,(i,o)=>{i<r&&o>r&&(n=!0)}),n&&(t=t.update({filterFrom:r,filterTo:r,filter:(i,o)=>o<=r||i>=r}))}return t},provide:t=>oe.decorations.from(t)});function Df(t,e,n){var r;let i=null;return(r=t.field(oo,!1))===null||r===void 0||r.between(e,n,(o,s)=>{(!i||i.from>o)&&(i={from:o,to:s})}),i}function ez(t,e,n){let r=!1;return t.between(e,e,(i,o)=>{i==e&&o==n&&(r=!0)}),r}function gk(t,e){return t.field(oo,!1)?e:e.concat(ce.appendConfig.of(vk()))}const tz=t=>{for(let e of mk(t)){let n=xf(t.state,e.from,e.to);if(n)return t.dispatch({effects:gk(t.state,[Af.of(n),yk(t,n)])}),!0}return!1},nz=t=>{if(!t.state.field(oo,!1))return!1;let e=[];for(let n of mk(t)){let r=Df(t.state,n.from,n.to);r&&e.push(Ma.of(r),yk(t,r,!1))}return e.length&&t.dispatch({effects:e}),e.length>0};function yk(t,e,n=!0){let r=t.state.doc.lineAt(e.from).number,i=t.state.doc.lineAt(e.to).number;return oe.announce.of(`${t.state.phrase(n?"Folded lines":"Unfolded lines")} ${r} ${t.state.phrase("to")} ${i}.`)}const rz=t=>{let{state:e}=t,n=[];for(let r=0;r<e.doc.length;){let i=t.lineBlockAt(r),o=xf(e,i.from,i.to);o&&n.push(Af.of(o)),r=(o?t.lineBlockAt(o.to):i).to+1}return n.length&&t.dispatch({effects:gk(t.state,n)}),!!n.length},iz=t=>{let e=t.state.field(oo,!1);if(!e||!e.size)return!1;let n=[];return e.between(0,t.state.doc.length,(r,i)=>{n.push(Ma.of({from:r,to:i}))}),t.dispatch({effects:n}),!0},oz=[{key:"Ctrl-Shift-[",mac:"Cmd-Alt-[",run:tz},{key:"Ctrl-Shift-]",mac:"Cmd-Alt-]",run:nz},{key:"Ctrl-Alt-[",run:rz},{key:"Ctrl-Alt-]",run:iz}],sz={placeholderDOM:null,placeholderText:"\u2026"},Ok=ne.define({combine(t){return mi(t,sz)}});function vk(t){let e=[oo,uz];return t&&e.push(Ok.of(t)),e}const lz=te.replace({widget:new class extends Cr{ignoreEvents(){return!1}toDOM(t){let{state:e}=t,n=e.facet(Ok),r=o=>{let s=t.lineBlockAt(t.posAtDOM(o.target)),l=Df(t.state,s.from,s.to);l&&t.dispatch({effects:Ma.of(l)}),o.preventDefault()};if(n.placeholderDOM)return n.placeholderDOM(t,r);let i=document.createElement("span");return i.textContent=n.placeholderText,i.setAttribute("aria-label",e.phrase("folded code")),i.title=e.phrase("unfold"),i.className="cm-foldPlaceholder",i.onclick=r,i}}}),az={openText:"\u2304",closedText:"\u203A",markerDOM:null};class Xg extends Mr{constructor(e,n){super();this.config=e,this.open=n}eq(e){return this.config==e.config&&this.open==e.open}toDOM(e){if(this.config.markerDOM)return this.config.markerDOM(this.open);let n=document.createElement("span");return n.textContent=this.open?this.config.openText:this.config.closedText,n.title=e.state.phrase(this.open?"Fold line":"Unfold line"),n}}function cz(t={}){let e=Object.assign(Object.assign({},az),t),n=new Xg(e,!0),r=new Xg(e,!1),i=rt.fromClass(class{constructor(o){this.from=o.viewport.from,this.markers=this.buildMarkers(o)}update(o){(o.docChanged||o.viewportChanged||o.startState.facet(bi)!=o.state.facet(bi)||o.startState.field(oo,!1)!=o.state.field(oo,!1))&&(this.markers=this.buildMarkers(o.view))}buildMarkers(o){let s=new yi;for(let l of o.viewportLineBlocks){let a=Df(o.state,l.from,l.to)?r:xf(o.state,l.from,l.to)?n:null;a&&s.add(l.from,l.from,a)}return s.finish()}});return[i,Fq({class:"cm-foldGutter",markers(o){var s;return((s=o.plugin(i))===null||s===void 0?void 0:s.markers)||Te.empty},initialSpacer(){return new Xg(e,!1)},domEventHandlers:{click:(o,s)=>{let l=Df(o.state,s.from,s.to);if(l)return o.dispatch({effects:Ma.of(l)}),!0;let a=xf(o.state,s.from,s.to);return a?(o.dispatch({effects:Af.of(a)}),!0):!1}}}),vk()]}const uz=oe.baseTheme({".cm-foldPlaceholder":{backgroundColor:"#eee",border:"1px solid #ddd",color:"#888",borderRadius:".2em",margin:"0 1px",padding:"0 1px",cursor:"pointer"},".cm-foldGutter span":{padding:"0 1px",cursor:"pointer"}}),Hg=pi.define(),fz=pi.define(),hz=ne.define(),xk=ne.define({combine(t){return mi(t,{minDepth:100,newGroupDelay:500},{minDepth:Math.max,newGroupDelay:Math.min})}});function dz(t){let e=0;return t.iterChangedRanges((n,r)=>e=r),e}const wk=Rt.define({create(){return or.empty},update(t,e){let n=e.state.facet(xk),r=e.annotation(Hg);if(r){let a=e.docChanged?_.single(dz(e.changes)):void 0,c=Xt.fromTransaction(e,a),u=r.side,f=u==0?t.undone:t.done;return c?f=Wf(f,f.length,n.minDepth,c):f=kk(f,e.startState.selection),new or(u==0?r.rest:f,u==0?f:r.rest)}let i=e.annotation(fz);if((i=="full"||i=="before")&&(t=t.isolate()),e.annotation(ut.addToHistory)===!1)return e.changes.empty?t:t.addMapping(e.changes.desc);let o=Xt.fromTransaction(e),s=e.annotation(ut.time),l=e.annotation(ut.userEvent);return o?t=t.addChanges(o,s,l,n.newGroupDelay,n.minDepth):e.selection&&(t=t.addSelection(e.startState.selection,s,l,n.newGroupDelay)),(i=="full"||i=="after")&&(t=t.isolate()),t},toJSON(t){return{done:t.done.map(e=>e.toJSON()),undone:t.undone.map(e=>e.toJSON())}},fromJSON(t){return new or(t.done.map(Xt.fromJSON),t.undone.map(Xt.fromJSON))}});function pz(t={}){return[wk,xk.of(t),oe.domEventHandlers({beforeinput(e,n){let r=e.inputType=="historyUndo"?bk:e.inputType=="historyRedo"?$k:null;return r?(e.preventDefault(),r(n)):!1}})]}function Lf(t,e){return function({state:n,dispatch:r}){if(!e&&n.readOnly)return!1;let i=n.field(wk,!1);if(!i)return!1;let o=i.pop(t,n,e);return o?(r(o),!0):!1}}const bk=Lf(0,!1),$k=Lf(1,!1),mz=Lf(0,!0),gz=Lf(1,!0);class Xt{constructor(e,n,r,i,o){this.changes=e,this.effects=n,this.mapped=r,this.startSelection=i,this.selectionsAfter=o}setSelAfter(e){return new Xt(this.changes,this.effects,this.mapped,this.startSelection,e)}toJSON(){var e,n,r;return{changes:(e=this.changes)===null||e===void 0?void 0:e.toJSON(),mapped:(n=this.mapped)===null||n===void 0?void 0:n.toJSON(),startSelection:(r=this.startSelection)===null||r===void 0?void 0:r.toJSON(),selectionsAfter:this.selectionsAfter.map(i=>i.toJSON())}}static fromJSON(e){return new Xt(e.changes&&ct.fromJSON(e.changes),[],e.mapped&&kr.fromJSON(e.mapped),e.startSelection&&_.fromJSON(e.startSelection),e.selectionsAfter.map(_.fromJSON))}static fromTransaction(e,n){let r=Sn;for(let i of e.startState.facet(hz)){let o=i(e);o.length&&(r=r.concat(o))}return!r.length&&e.changes.empty?null:new Xt(e.changes.invert(e.startState.doc),r,void 0,n||e.startState.selection,Sn)}static selection(e){return new Xt(void 0,Sn,void 0,void 0,e)}}function Wf(t,e,n,r){let i=e+1>n+20?e-n-1:0,o=t.slice(i,e);return o.push(r),o}function yz(t,e){let n=[],r=!1;return t.iterChangedRanges((i,o)=>n.push(i,o)),e.iterChangedRanges((i,o,s,l)=>{for(let a=0;a<n.length;){let c=n[a++],u=n[a++];l>=c&&s<=u&&(r=!0)}}),r}function Oz(t,e){return t.ranges.length==e.ranges.length&&t.ranges.filter((n,r)=>n.empty!=e.ranges[r].empty).length===0}function Sk(t,e){return t.length?e.length?t.concat(e):t:e}const Sn=[],vz=200;function kk(t,e){if(t.length){let n=t[t.length-1],r=n.selectionsAfter.slice(Math.max(0,n.selectionsAfter.length-vz));return r.length&&r[r.length-1].eq(e)?t:(r.push(e),Wf(t,t.length-1,1e9,n.setSelAfter(r)))}else return[Xt.selection([e])]}function xz(t){let e=t[t.length-1],n=t.slice();return n[t.length-1]=e.setSelAfter(e.selectionsAfter.slice(0,e.selectionsAfter.length-1)),n}function Gg(t,e){if(!t.length)return t;let n=t.length,r=Sn;for(;n;){let i=wz(t[n-1],e,r);if(i.changes&&!i.changes.empty||i.effects.length){let o=t.slice(0,n);return o[n-1]=i,o}else e=i.mapped,n--,r=i.selectionsAfter}return r.length?[Xt.selection(r)]:Sn}function wz(t,e,n){let r=Sk(t.selectionsAfter.length?t.selectionsAfter.map(l=>l.map(e)):Sn,n);if(!t.changes)return Xt.selection(r);let i=t.changes.map(e),o=e.mapDesc(t.changes,!0),s=t.mapped?t.mapped.composeDesc(o):o;return new Xt(i,ce.mapEffects(t.effects,e),s,t.startSelection.map(o),r)}const bz=/^(input\.type|delete)($|\.)/;class or{constructor(e,n,r=0,i=void 0){this.done=e,this.undone=n,this.prevTime=r,this.prevUserEvent=i}isolate(){return this.prevTime?new or(this.done,this.undone):this}addChanges(e,n,r,i,o){let s=this.done,l=s[s.length-1];return l&&l.changes&&!l.changes.empty&&e.changes&&(!r||bz.test(r))&&(!l.selectionsAfter.length&&n-this.prevTime<i&&yz(l.changes,e.changes)||r=="input.type.compose")?s=Wf(s,s.length-1,o,new Xt(e.changes.compose(l.changes),Sk(e.effects,l.effects),l.mapped,l.startSelection,Sn)):s=Wf(s,s.length,o,e),new or(s,Sn,n,r)}addSelection(e,n,r,i){let o=this.done.length?this.done[this.done.length-1].selectionsAfter:Sn;return o.length>0&&n-this.prevTime<i&&r==this.prevUserEvent&&r&&/^select($|\.)/.test(r)&&Oz(o[o.length-1],e)?this:new or(kk(this.done,e),this.undone,n,r)}addMapping(e){return new or(Gg(this.done,e),Gg(this.undone,e),this.prevTime,this.prevUserEvent)}pop(e,n,r){let i=e==0?this.done:this.undone;if(i.length==0)return null;let o=i[i.length-1];if(r&&o.selectionsAfter.length)return n.update({selection:o.selectionsAfter[o.selectionsAfter.length-1],annotations:Hg.of({side:e,rest:xz(i)}),userEvent:e==0?"select.undo":"select.redo",scrollIntoView:!0});if(o.changes){let s=i.length==1?Sn:i.slice(0,i.length-1);return o.mapped&&(s=Gg(s,o.mapped)),n.update({changes:o.changes,selection:o.startSelection,effects:o.effects,annotations:Hg.of({side:e,rest:s}),filter:!1,userEvent:e==0?"undo":"redo",scrollIntoView:!0})}else return null}}or.empty=new or(Sn,Sn);const $z=[{key:"Mod-z",run:bk,preventDefault:!0},{key:"Mod-y",mac:"Mod-Shift-z",run:$k,preventDefault:!0},{key:"Mod-u",run:mz,preventDefault:!0},{key:"Alt-u",mac:"Mod-Shift-u",run:gz,preventDefault:!0}];class Qf{constructor(e,n,r,i,o,s,l,a,c,u=0,f){this.p=e,this.stack=n,this.state=r,this.reducePos=i,this.pos=o,this.score=s,this.buffer=l,this.bufferBase=a,this.curContext=c,this.lookAhead=u,this.parent=f}toString(){return`[${this.stack.filter((e,n)=>n%3==0).concat(this.state)}]@${this.pos}${this.score?"!"+this.score:""}`}static start(e,n,r=0){let i=e.parser.context;return new Qf(e,[],n,r,r,0,[],0,i?new Pk(i,i.start):null,0,null)}get context(){return this.curContext?this.curContext.context:null}pushState(e,n){this.stack.push(this.state,n,this.bufferBase+this.buffer.length),this.state=e}reduce(e){let n=e>>19,r=e&65535,{parser:i}=this.p,o=i.dynamicPrecedence(r);if(o&&(this.score+=o),n==0){r<i.minRepeatTerm&&this.storeNode(r,this.reducePos,this.reducePos,4,!0),this.pushState(i.getGoto(this.state,r,!0),this.reducePos),this.reduceContext(r,this.reducePos);return}let s=this.stack.length-(n-1)*3-(e&262144?6:0),l=this.stack[s-2],a=this.stack[s-1],c=this.bufferBase+this.buffer.length-a;if(r<i.minRepeatTerm||e&131072){let u=i.stateFlag(this.state,1)?this.pos:this.reducePos;this.storeNode(r,l,u,c+4,!0)}if(e&262144)this.state=this.stack[s];else{let u=this.stack[s-3];this.state=i.getGoto(u,r,!0)}for(;this.stack.length>s;)this.stack.pop();this.reduceContext(r,l)}storeNode(e,n,r,i=4,o=!1){if(e==0){let s=this,l=this.buffer.length;if(l==0&&s.parent&&(l=s.bufferBase-s.parent.bufferBase,s=s.parent),l>0&&s.buffer[l-4]==0&&s.buffer[l-1]>-1){if(n==r)return;if(s.buffer[l-2]>=n){s.buffer[l-2]=r;return}}}if(!o||this.pos==r)this.buffer.push(e,n,r,i);else{let s=this.buffer.length;if(s>0&&this.buffer[s-4]!=0)for(;s>0&&this.buffer[s-2]>r;)this.buffer[s]=this.buffer[s-4],this.buffer[s+1]=this.buffer[s-3],this.buffer[s+2]=this.buffer[s-2],this.buffer[s+3]=this.buffer[s-1],s-=4,i>4&&(i-=4);this.buffer[s]=e,this.buffer[s+1]=n,this.buffer[s+2]=r,this.buffer[s+3]=i}}shift(e,n,r){let i=this.pos;if(e&131072)this.pushState(e&65535,this.pos);else if((e&262144)==0){let o=e,{parser:s}=this.p;(r>this.pos||n<=s.maxNode)&&(this.pos=r,s.stateFlag(o,1)||(this.reducePos=r)),this.pushState(o,i),this.shiftContext(n,i),n<=s.maxNode&&this.buffer.push(n,i,r,4)}else this.pos=r,this.shiftContext(n,i),n<=this.p.parser.maxNode&&this.buffer.push(n,i,r,4)}apply(e,n,r){e&65536?this.reduce(e):this.shift(e,n,r)}useNode(e,n){let r=this.p.reused.length-1;(r<0||this.p.reused[r]!=e)&&(this.p.reused.push(e),r++);let i=this.pos;this.reducePos=this.pos=i+e.length,this.pushState(n,i),this.buffer.push(r,i,this.reducePos,-1),this.curContext&&this.updateContext(this.curContext.tracker.reuse(this.curContext.context,e,this,this.p.stream.reset(this.pos-e.length)))}split(){let e=this,n=e.buffer.length;for(;n>0&&e.buffer[n-2]>e.reducePos;)n-=4;let r=e.buffer.slice(n),i=e.bufferBase+n;for(;e&&i==e.bufferBase;)e=e.parent;return new Qf(this.p,this.stack.slice(),this.state,this.reducePos,this.pos,this.score,r,i,this.curContext,this.lookAhead,e)}recoverByDelete(e,n){let r=e<=this.p.parser.maxNode;r&&this.storeNode(e,this.pos,n,4),this.storeNode(0,this.pos,n,r?8:4),this.pos=this.reducePos=n,this.score-=190}canShift(e){for(let n=new Sz(this);;){let r=this.p.parser.stateSlot(n.state,4)||this.p.parser.hasAction(n.state,e);if((r&65536)==0)return!0;if(r==0)return!1;n.reduce(r)}}recoverByInsert(e){if(this.stack.length>=300)return[];let n=this.p.parser.nextStates(this.state);if(n.length>4<<1||this.stack.length>=120){let i=[];for(let o=0,s;o<n.length;o+=2)(s=n[o+1])!=this.state&&this.p.parser.hasAction(s,e)&&i.push(n[o],s);if(this.stack.length<120)for(let o=0;i.length<4<<1&&o<n.length;o+=2){let s=n[o+1];i.some((l,a)=>a&1&&l==s)||i.push(n[o],s)}n=i}let r=[];for(let i=0;i<n.length&&r.length<4;i+=2){let o=n[i+1];if(o==this.state)continue;let s=this.split();s.storeNode(0,s.pos,s.pos,4,!0),s.pushState(o,this.pos),s.shiftContext(n[i],this.pos),s.score-=200,r.push(s)}return r}forceReduce(){let e=this.p.parser.stateSlot(this.state,5);if((e&65536)==0)return!1;let{parser:n}=this.p;if(!n.validAction(this.state,e)){let r=e>>19,i=e&65535,o=this.stack.length-r*3;if(o<0||n.getGoto(this.stack[o],i,!1)<0)return!1;this.storeNode(0,this.reducePos,this.reducePos,4,!0),this.score-=100}return this.reduce(e),!0}forceAll(){for(;!this.p.parser.stateFlag(this.state,2);)if(!this.forceReduce()){this.storeNode(0,this.pos,this.pos,4,!0);break}return this}get deadEnd(){if(this.stack.length!=3)return!1;let{parser:e}=this.p;return e.data[e.stateSlot(this.state,1)]==65535&&!e.stateSlot(this.state,4)}restart(){this.state=this.stack[0],this.stack.length=0}sameState(e){if(this.state!=e.state||this.stack.length!=e.stack.length)return!1;for(let n=0;n<this.stack.length;n+=3)if(this.stack[n]!=e.stack[n])return!1;return!0}get parser(){return this.p.parser}dialectEnabled(e){return this.p.parser.dialect.flags[e]}shiftContext(e,n){this.curContext&&this.updateContext(this.curContext.tracker.shift(this.curContext.context,e,this,this.p.stream.reset(n)))}reduceContext(e,n){this.curContext&&this.updateContext(this.curContext.tracker.reduce(this.curContext.context,e,this,this.p.stream.reset(n)))}emitContext(){let e=this.buffer.length-1;(e<0||this.buffer[e]!=-3)&&this.buffer.push(this.curContext.hash,this.reducePos,this.reducePos,-3)}emitLookAhead(){let e=this.buffer.length-1;(e<0||this.buffer[e]!=-4)&&this.buffer.push(this.lookAhead,this.reducePos,this.reducePos,-4)}updateContext(e){if(e!=this.curContext.context){let n=new Pk(this.curContext.tracker,e);n.hash!=this.curContext.hash&&this.emitContext(),this.curContext=n}}setLookAhead(e){e>this.lookAhead&&(this.emitLookAhead(),this.lookAhead=e)}close(){this.curContext&&this.curContext.tracker.strict&&this.emitContext(),this.lookAhead>0&&this.emitLookAhead()}}class Pk{constructor(e,n){this.tracker=e,this.context=n,this.hash=e.strict?e.hash(n):0}}var Ck;(function(t){t[t.Insert=200]="Insert",t[t.Delete=190]="Delete",t[t.Reduce=100]="Reduce",t[t.MaxNext=4]="MaxNext",t[t.MaxInsertStackDepth=300]="MaxInsertStackDepth",t[t.DampenInsertStackDepth=120]="DampenInsertStackDepth"})(Ck||(Ck={}));class Sz{constructor(e){this.start=e,this.state=e.state,this.stack=e.stack,this.base=this.stack.length}reduce(e){let n=e&65535,r=e>>19;r==0?(this.stack==this.start.stack&&(this.stack=this.stack.slice()),this.stack.push(this.state,0,0),this.base+=3):this.base-=(r-1)*3;let i=this.start.p.parser.getGoto(this.stack[this.base-3],n,!0);this.state=i}}class Nf{constructor(e,n,r){this.stack=e,this.pos=n,this.index=r,this.buffer=e.buffer,this.index==0&&this.maybeNext()}static create(e,n=e.bufferBase+e.buffer.length){return new Nf(e,n,n-e.bufferBase)}maybeNext(){let e=this.stack.parent;e!=null&&(this.index=this.stack.bufferBase-e.bufferBase,this.stack=e,this.buffer=e.buffer)}get id(){return this.buffer[this.index-4]}get start(){return this.buffer[this.index-3]}get end(){return this.buffer[this.index-2]}get size(){return this.buffer[this.index-1]}next(){this.index-=4,this.pos-=4,this.index==0&&this.maybeNext()}fork(){return new Nf(this.stack,this.pos,this.index)}}class _f{constructor(){this.start=-1,this.value=-1,this.end=-1,this.extended=-1,this.lookAhead=0,this.mask=0,this.context=0}}const Rk=new _f;class kz{constructor(e,n){this.input=e,this.ranges=n,this.chunk="",this.chunkOff=0,this.chunk2="",this.chunk2Pos=0,this.next=-1,this.token=Rk,this.rangeIndex=0,this.pos=this.chunkPos=n[0].from,this.range=n[0],this.end=n[n.length-1].to,this.readNext()}resolveOffset(e,n){let r=this.range,i=this.rangeIndex,o=this.pos+e;for(;o<r.from;){if(!i)return null;let s=this.ranges[--i];o-=r.from-s.to,r=s}for(;n<0?o>r.to:o>=r.to;){if(i==this.ranges.length-1)return null;let s=this.ranges[++i];o+=s.from-r.to,r=s}return o}peek(e){let n=this.chunkOff+e,r,i;if(n>=0&&n<this.chunk.length)r=this.pos+e,i=this.chunk.charCodeAt(n);else{let o=this.resolveOffset(e,1);if(o==null)return-1;if(r=o,r>=this.chunk2Pos&&r<this.chunk2Pos+this.chunk2.length)i=this.chunk2.charCodeAt(r-this.chunk2Pos);else{let s=this.rangeIndex,l=this.range;for(;l.to<=r;)l=this.ranges[++s];this.chunk2=this.input.chunk(this.chunk2Pos=r),r+this.chunk2.length>l.to&&(this.chunk2=this.chunk2.slice(0,l.to-r)),i=this.chunk2.charCodeAt(0)}}return r>=this.token.lookAhead&&(this.token.lookAhead=r+1),i}acceptToken(e,n=0){let r=n?this.resolveOffset(n,-1):this.pos;if(r==null||r<this.token.start)throw new RangeError("Token end out of bounds");this.token.value=e,this.token.end=r}getChunk(){if(this.pos>=this.chunk2Pos&&this.pos<this.chunk2Pos+this.chunk2.length){let{chunk:e,chunkPos:n}=this;this.chunk=this.chunk2,this.chunkPos=this.chunk2Pos,this.chunk2=e,this.chunk2Pos=n,this.chunkOff=this.pos-this.chunkPos}else{this.chunk2=this.chunk,this.chunk2Pos=this.chunkPos;let e=this.input.chunk(this.pos),n=this.pos+e.length;this.chunk=n>this.range.to?e.slice(0,this.range.to-this.pos):e,this.chunkPos=this.pos,this.chunkOff=0}}readNext(){return this.chunkOff>=this.chunk.length&&(this.getChunk(),this.chunkOff==this.chunk.length)?this.next=-1:this.next=this.chunk.charCodeAt(this.chunkOff)}advance(e=1){for(this.chunkOff+=e;this.pos+e>=this.range.to;){if(this.rangeIndex==this.ranges.length-1)return this.setDone();e-=this.range.to-this.pos,this.range=this.ranges[++this.rangeIndex],this.pos=this.range.from}return this.pos+=e,this.pos>=this.token.lookAhead&&(this.token.lookAhead=this.pos+1),this.readNext()}setDone(){return this.pos=this.chunkPos=this.end,this.range=this.ranges[this.rangeIndex=this.ranges.length-1],this.chunk="",this.next=-1}reset(e,n){if(n?(this.token=n,n.start=e,n.lookAhead=e+1,n.value=n.extended=-1):this.token=Rk,this.pos!=e){if(this.pos=e,e==this.end)return this.setDone(),this;for(;e<this.range.from;)this.range=this.ranges[--this.rangeIndex];for(;e>=this.range.to;)this.range=this.ranges[++this.rangeIndex];e>=this.chunkPos&&e<this.chunkPos+this.chunk.length?this.chunkOff=e-this.chunkPos:(this.chunk="",this.chunkOff=0),this.readNext()}return this}read(e,n){if(e>=this.chunkPos&&n<=this.chunkPos+this.chunk.length)return this.chunk.slice(e-this.chunkPos,n-this.chunkPos);if(e>=this.range.from&&n<=this.range.to)return this.input.read(e,n);let r="";for(let i of this.ranges){if(i.from>=n)break;i.to>e&&(r+=this.input.read(Math.max(i.from,e),Math.min(i.to,n)))}return r}}class Bf{constructor(e,n){this.data=e,this.id=n}token(e,n){Pz(this.data,e,n,this.id)}}Bf.prototype.contextual=Bf.prototype.fallback=Bf.prototype.extend=!1;class Zg{constructor(e,n={}){this.token=e,this.contextual=!!n.contextual,this.fallback=!!n.fallback,this.extend=!!n.extend}}function Pz(t,e,n,r){let i=0,o=1<<r,{parser:s}=n.p,{dialect:l}=s;e:for(;(o&t[i])!=0;){let a=t[i+1];for(let c=i+3;c<a;c+=2)if((t[c+1]&o)>0){let u=t[c];if(l.allows(u)&&(e.token.value==-1||e.token.value==u||s.overrides(u,e.token.value))){e.acceptToken(u);break}}for(let c=e.next,u=0,f=t[i+2];u<f;){let h=u+f>>1,d=a+h+(h<<1),p=t[d],m=t[d+1];if(c<p)f=h;else if(c>=m)u=h+1;else{i=t[d+2],e.advance();continue e}}break}}function qf(t,e=Uint16Array){if(typeof t!="string")return t;let n=null;for(let r=0,i=0;r<t.length;){let o=0;for(;;){let s=t.charCodeAt(r++),l=!1;if(s==126){o=65535;break}s>=92&&s--,s>=34&&s--;let a=s-32;if(a>=46&&(a-=46,l=!0),o+=a,l)break;o*=46}n?n[i++]=o:n=new e(o)}return n}const Nn=typeof process!="undefined"&&/\bparse\b/.test({}.LOG);let Yg=null;var Tk;(function(t){t[t.Margin=25]="Margin"})(Tk||(Tk={}));function Ik(t,e,n){let r=t.fullCursor();for(r.moveTo(e);;)if(!(n<0?r.childBefore(e):r.childAfter(e)))for(;;){if((n<0?r.to<e:r.from>e)&&!r.type.isError)return n<0?Math.max(0,Math.min(r.to-1,e-25)):Math.min(t.length,Math.max(r.from+1,e+25));if(n<0?r.prevSibling():r.nextSibling())break;if(!r.parent())return n<0?0:t.length}}class Cz{constructor(e,n){this.fragments=e,this.nodeSet=n,this.i=0,this.fragment=null,this.safeFrom=-1,this.safeTo=-1,this.trees=[],this.start=[],this.index=[],this.nextFragment()}nextFragment(){let e=this.fragment=this.i==this.fragments.length?null:this.fragments[this.i++];if(e){for(this.safeFrom=e.openStart?Ik(e.tree,e.from+e.offset,1)-e.offset:e.from,this.safeTo=e.openEnd?Ik(e.tree,e.to+e.offset,-1)-e.offset:e.to;this.trees.length;)this.trees.pop(),this.start.pop(),this.index.pop();this.trees.push(e.tree),this.start.push(-e.offset),this.index.push(0),this.nextStart=this.safeFrom}else this.nextStart=1e9}nodeAt(e){if(e<this.nextStart)return null;for(;this.fragment&&this.safeTo<=e;)this.nextFragment();if(!this.fragment)return null;for(;;){let n=this.trees.length-1;if(n<0)return this.nextFragment(),null;let r=this.trees[n],i=this.index[n];if(i==r.children.length){this.trees.pop(),this.start.pop(),this.index.pop();continue}let o=r.children[i],s=this.start[n]+r.positions[i];if(s>e)return this.nextStart=s,null;if(o instanceof et){if(s==e){if(s<this.safeFrom)return null;let l=s+o.length;if(l<=this.safeTo){let a=o.prop(xe.lookAhead);if(!a||l+a<this.fragment.to)return o}}this.index[n]++,s+o.length>=Math.max(this.safeFrom,e)&&(this.trees.push(o),this.start.push(s),this.index.push(0))}else this.index[n]++,this.nextStart=s+o.length}}}class Rz{constructor(e,n){this.stream=n,this.tokens=[],this.mainToken=null,this.actions=[],this.tokens=e.tokenizers.map(r=>new _f)}getActions(e){let n=0,r=null,{parser:i}=e.p,{tokenizers:o}=i,s=i.stateSlot(e.state,3),l=e.curContext?e.curContext.hash:0,a=0;for(let c=0;c<o.length;c++){if((1<<c&s)==0)continue;let u=o[c],f=this.tokens[c];if(!(r&&!u.fallback)&&((u.contextual||f.start!=e.pos||f.mask!=s||f.context!=l)&&(this.updateCachedToken(f,u,e),f.mask=s,f.context=l),f.lookAhead>f.end+25&&(a=Math.max(f.lookAhead,a)),f.value!=0)){let h=n;if(f.extended>-1&&(n=this.addActions(e,f.extended,f.end,n)),n=this.addActions(e,f.value,f.end,n),!u.extend&&(r=f,n>h))break}}for(;this.actions.length>n;)this.actions.pop();return a&&e.setLookAhead(a),!r&&e.pos==this.stream.end&&(r=new _f,r.value=e.p.parser.eofTerm,r.start=r.end=e.pos,n=this.addActions(e,r.value,r.end,n)),this.mainToken=r,this.actions}getMainToken(e){if(this.mainToken)return this.mainToken;let n=new _f,{pos:r,p:i}=e;return n.start=r,n.end=Math.min(r+1,i.stream.end),n.value=r==i.stream.end?i.parser.eofTerm:0,n}updateCachedToken(e,n,r){if(n.token(this.stream.reset(r.pos,e),r),e.value>-1){let{parser:i}=r.p;for(let o=0;o<i.specialized.length;o++)if(i.specialized[o]==e.value){let s=i.specializers[o](this.stream.read(e.start,e.end),r);if(s>=0&&r.p.parser.dialect.allows(s>>1)){(s&1)==0?e.value=s>>1:e.extended=s>>1;break}}}else e.value=0,e.end=Math.min(r.p.stream.end,r.pos+1)}putAction(e,n,r,i){for(let o=0;o<i;o+=3)if(this.actions[o]==e)return i;return this.actions[i++]=e,this.actions[i++]=n,this.actions[i++]=r,i}addActions(e,n,r,i){let{state:o}=e,{parser:s}=e.p,{data:l}=s;for(let a=0;a<2;a++)for(let c=s.stateSlot(o,a?2:1);;c+=3){if(l[c]==65535)if(l[c+1]==1)c=Ar(l,c+2);else{i==0&&l[c+1]==2&&(i=this.putAction(Ar(l,c+1),n,r,i));break}l[c]==n&&(i=this.putAction(Ar(l,c+1),n,r,i))}return i}}var Ek;(function(t){t[t.Distance=5]="Distance",t[t.MaxRemainingPerStep=3]="MaxRemainingPerStep",t[t.MinBufferLengthPrune=500]="MinBufferLengthPrune",t[t.ForceReduceLimit=10]="ForceReduceLimit",t[t.CutDepth=15e3]="CutDepth",t[t.CutTo=9e3]="CutTo"})(Ek||(Ek={}));class Tz{constructor(e,n,r,i){this.parser=e,this.input=n,this.ranges=i,this.recovering=0,this.nextStackID=9812,this.minStackPos=0,this.reused=[],this.stoppedAt=null,this.stream=new kz(n,i),this.tokens=new Rz(e,this.stream),this.topTerm=e.top[1];let{from:o}=i[0];this.stacks=[Qf.start(this,e.top[0],o)],this.fragments=r.length&&this.stream.end-o>e.bufferLength*4?new Cz(r,e.nodeSet):null}get parsedPos(){return this.minStackPos}advance(){let e=this.stacks,n=this.minStackPos,r=this.stacks=[],i,o;for(let s=0;s<e.length;s++){let l=e[s];for(;;){if(this.tokens.mainToken=null,l.pos>n)r.push(l);else{if(this.advanceStack(l,r,e))continue;{i||(i=[],o=[]),i.push(l);let a=this.tokens.getMainToken(l);o.push(a.value,a.end)}}break}}if(!r.length){let s=i&&Mz(i);if(s)return this.stackToTree(s);if(this.parser.strict)throw Nn&&i&&console.log("Stuck with token "+(this.tokens.mainToken?this.parser.getName(this.tokens.mainToken.value):"none")),new SyntaxError("No parse at "+n);this.recovering||(this.recovering=5)}if(this.recovering&&i){let s=this.stoppedAt!=null&&i[0].pos>this.stoppedAt?i[0]:this.runRecovery(i,o,r);if(s)return this.stackToTree(s.forceAll())}if(this.recovering){let s=this.recovering==1?1:this.recovering*3;if(r.length>s)for(r.sort((l,a)=>a.score-l.score);r.length>s;)r.pop();r.some(l=>l.reducePos>n)&&this.recovering--}else if(r.length>1){e:for(let s=0;s<r.length-1;s++){let l=r[s];for(let a=s+1;a<r.length;a++){let c=r[a];if(l.sameState(c)||l.buffer.length>500&&c.buffer.length>500)if((l.score-c.score||l.buffer.length-c.buffer.length)>0)r.splice(a--,1);else{r.splice(s--,1);continue e}}}}this.minStackPos=r[0].pos;for(let s=1;s<r.length;s++)r[s].pos<this.minStackPos&&(this.minStackPos=r[s].pos);return null}stopAt(e){if(this.stoppedAt!=null&&this.stoppedAt<e)throw new RangeError("Can't move stoppedAt forward");this.stoppedAt=e}advanceStack(e,n,r){let i=e.pos,{parser:o}=this,s=Nn?this.stackID(e)+" -> ":"";if(this.stoppedAt!=null&&i>this.stoppedAt)return e.forceReduce()?e:null;if(this.fragments){let c=e.curContext&&e.curContext.tracker.strict,u=c?e.curContext.hash:0;for(let f=this.fragments.nodeAt(i);f;){let h=this.parser.nodeSet.types[f.type.id]==f.type?o.getGoto(e.state,f.type.id):-1;if(h>-1&&f.length&&(!c||(f.prop(xe.contextHash)||0)==u))return e.useNode(f,h),Nn&&console.log(s+this.stackID(e)+` (via reuse of ${o.getName(f.type.id)})`),!0;if(!(f instanceof et)||f.children.length==0||f.positions[0]>0)break;let d=f.children[0];if(d instanceof et&&f.positions[0]==0)f=d;else break}}let l=o.stateSlot(e.state,4);if(l>0)return e.reduce(l),Nn&&console.log(s+this.stackID(e)+` (via always-reduce ${o.getName(l&65535)})`),!0;if(e.stack.length>=15e3)for(;e.stack.length>9e3&&e.forceReduce(););let a=this.tokens.getActions(e);for(let c=0;c<a.length;){let u=a[c++],f=a[c++],h=a[c++],d=c==a.length||!r,p=d?e:e.split();if(p.apply(u,f,h),Nn&&console.log(s+this.stackID(p)+` (via ${(u&65536)==0?"shift":`reduce of ${o.getName(u&65535)}`} for ${o.getName(f)} @ ${i}${p==e?"":", split"})`),d)return!0;p.pos>i?n.push(p):r.push(p)}return!1}advanceFully(e,n){let r=e.pos;for(;;){if(!this.advanceStack(e,null,null))return!1;if(e.pos>r)return Mk(e,n),!0}}runRecovery(e,n,r){let i=null,o=!1;for(let s=0;s<e.length;s++){let l=e[s],a=n[s<<1],c=n[(s<<1)+1],u=Nn?this.stackID(l)+" -> ":"";if(l.deadEnd&&(o||(o=!0,l.restart(),Nn&&console.log(u+this.stackID(l)+" (restarted)"),this.advanceFully(l,r))))continue;let f=l.split(),h=u;for(let d=0;f.forceReduce()&&d<10&&(Nn&&console.log(h+this.stackID(f)+" (via force-reduce)"),!this.advanceFully(f,r));d++)Nn&&(h=this.stackID(f)+" -> ");for(let d of l.recoverByInsert(a))Nn&&console.log(u+this.stackID(d)+" (via recover-insert)"),this.advanceFully(d,r);this.stream.end>l.pos?(c==l.pos&&(c++,a=0),l.recoverByDelete(a,c),Nn&&console.log(u+this.stackID(l)+` (via recover-delete ${this.parser.getName(a)})`),Mk(l,r)):(!i||i.score<l.score)&&(i=l)}return i}stackToTree(e){return e.close(),et.build({buffer:Nf.create(e),nodeSet:this.parser.nodeSet,topID:this.topTerm,maxBufferLength:this.parser.bufferLength,reused:this.reused,start:this.ranges[0].from,length:e.pos-this.ranges[0].from,minRepeatType:this.parser.minRepeatTerm})}stackID(e){let n=(Yg||(Yg=new WeakMap)).get(e);return n||Yg.set(e,n=String.fromCodePoint(this.nextStackID++)),n+e}}function Mk(t,e){for(let n=0;n<e.length;n++){let r=e[n];if(r.pos==t.pos&&r.sameState(t)){e[n].score<t.score&&(e[n]=t);return}}e.push(t)}class Iz{constructor(e,n,r){this.source=e,this.flags=n,this.disabled=r}allows(e){return!this.disabled||this.disabled[e]==0}}const Kg=t=>t;class Ez{constructor(e){this.start=e.start,this.shift=e.shift||Kg,this.reduce=e.reduce||Kg,this.reuse=e.reuse||Kg,this.hash=e.hash||(()=>0),this.strict=e.strict!==!1}}class zf extends H${constructor(e){super();if(this.wrappers=[],e.version!=13)throw new RangeError(`Parser version (${e.version}) doesn't match runtime version (${13})`);let n=e.nodeNames.split(" ");this.minRepeatTerm=n.length;for(let l=0;l<e.repeatNodeCount;l++)n.push("");let r=Object.keys(e.topRules).map(l=>e.topRules[l][1]),i=[];for(let l=0;l<n.length;l++)i.push([]);function o(l,a,c){i[l].push([a,a.deserialize(String(c))])}if(e.nodeProps)for(let l of e.nodeProps){let a=l[0];for(let c=1;c<l.length;){let u=l[c++];if(u>=0)o(u,a,l[c++]);else{let f=l[c+-u];for(let h=-u;h>0;h--)o(l[c++],a,f);c++}}}this.nodeSet=new Sg(n.map((l,a)=>ln.define({name:a>=this.minRepeatTerm?void 0:l,id:a,props:i[a],top:r.indexOf(a)>-1,error:a==0,skipped:e.skippedNodes&&e.skippedNodes.indexOf(a)>-1}))),this.strict=!1,this.bufferLength=F$;let s=qf(e.tokenData);if(this.context=e.context,this.specialized=new Uint16Array(e.specialized?e.specialized.length:0),this.specializers=[],e.specialized)for(let l=0;l<e.specialized.length;l++)this.specialized[l]=e.specialized[l].term,this.specializers[l]=e.specialized[l].get;this.states=qf(e.states,Uint32Array),this.data=qf(e.stateData),this.goto=qf(e.goto),this.maxTerm=e.maxTerm,this.tokenizers=e.tokenizers.map(l=>typeof l=="number"?new Bf(s,l):l),this.topRules=e.topRules,this.dialects=e.dialects||{},this.dynamicPrecedences=e.dynamicPrecedences||null,this.tokenPrecTable=e.tokenPrec,this.termNames=e.termNames||null,this.maxNode=this.nodeSet.types.length-1,this.dialect=this.parseDialect(),this.top=this.topRules[Object.keys(this.topRules)[0]]}createParse(e,n,r){let i=new Tz(this,e,n,r);for(let o of this.wrappers)i=o(i,e,n,r);return i}getGoto(e,n,r=!1){let i=this.goto;if(n>=i[0])return-1;for(let o=i[n+1];;){let s=i[o++],l=s&1,a=i[o++];if(l&&r)return a;for(let c=o+(s>>1);o<c;o++)if(i[o]==e)return a;if(l)return-1}}hasAction(e,n){let r=this.data;for(let i=0;i<2;i++)for(let o=this.stateSlot(e,i?2:1),s;;o+=3){if((s=r[o])==65535)if(r[o+1]==1)s=r[o=Ar(r,o+2)];else{if(r[o+1]==2)return Ar(r,o+2);break}if(s==n||s==0)return Ar(r,o+1)}return 0}stateSlot(e,n){return this.states[e*6+n]}stateFlag(e,n){return(this.stateSlot(e,0)&n)>0}validAction(e,n){if(n==this.stateSlot(e,4))return!0;for(let r=this.stateSlot(e,1);;r+=3){if(this.data[r]==65535)if(this.data[r+1]==1)r=Ar(this.data,r+2);else return!1;if(n==Ar(this.data,r+1))return!0}}nextStates(e){let n=[];for(let r=this.stateSlot(e,1);;r+=3){if(this.data[r]==65535)if(this.data[r+1]==1)r=Ar(this.data,r+2);else break;if((this.data[r+2]&65536>>16)==0){let i=this.data[r+1];n.some((o,s)=>s&1&&o==i)||n.push(this.data[r],i)}}return n}overrides(e,n){let r=Ak(this.data,this.tokenPrecTable,n);return r<0||Ak(this.data,this.tokenPrecTable,e)<r}configure(e){let n=Object.assign(Object.create(zf.prototype),this);if(e.props&&(n.nodeSet=this.nodeSet.extend(...e.props)),e.top){let r=this.topRules[e.top];if(!r)throw new RangeError(`Invalid top rule name ${e.top}`);n.top=r}return e.tokenizers&&(n.tokenizers=this.tokenizers.map(r=>{let i=e.tokenizers.find(o=>o.from==r);return i?i.to:r})),e.contextTracker&&(n.context=e.contextTracker),e.dialect&&(n.dialect=this.parseDialect(e.dialect)),e.strict!=null&&(n.strict=e.strict),e.wrap&&(n.wrappers=n.wrappers.concat(e.wrap)),e.bufferLength!=null&&(n.bufferLength=e.bufferLength),n}getName(e){return this.termNames?this.termNames[e]:String(e<=this.maxNode&&this.nodeSet.types[e].name||e)}get eofTerm(){return this.maxNode+1}get topNode(){return this.nodeSet.types[this.top[1]]}dynamicPrecedence(e){let n=this.dynamicPrecedences;return n==null?0:n[e]||0}parseDialect(e){let n=Object.keys(this.dialects),r=n.map(()=>!1);if(e)for(let o of e.split(" ")){let s=n.indexOf(o);s>=0&&(r[s]=!0)}let i=null;for(let o=0;o<n.length;o++)if(!r[o])for(let s=this.dialects[n[o]],l;(l=this.data[s++])!=65535;)(i||(i=new Uint8Array(this.maxTerm+1)))[l]=1;return new Iz(e,r,i)}static deserialize(e){return new zf(e)}}function Ar(t,e){return t[e]|t[e+1]<<16}function Ak(t,e,n){for(let r=e,i;(i=t[r])!=65535;r++)if(i==n)return r-e;return-1}function Mz(t){let e=null;for(let n of t){let r=n.p.stoppedAt;(n.pos==n.p.stream.end||r!=null&&n.pos>r)&&n.p.parser.stateFlag(n.state,2)&&(!e||e.score<n.score)&&(e=n)}return e}const Az=1,Dk=162,Lk=163,Dz=164,Lz=165,Wz=166,Qz=167,Nz=21,_z=22,Bz=47,qz=48,zz=52,Fz=53,Vz=55,Uz=56,jz=57,Xz=59,Hz=60,Gz=61,Zz=63,Yz=203,Kz=71,Jz=228,e6=121,Aa=10,Da=13,Jg=32,e0=9,t0=35,t6=40,n6=46,r6=[Nz,Bz,qz,Jz,Zz,e6,zz,Vz,Yz,Hz,Gz,Uz,Xz,Kz],i6=new Zg((t,e)=>{if(t.next<0)t.acceptToken(Qz);else if(!(t.next!=Aa&&t.next!=Da))if(e.context.depth<0)t.acceptToken(Lz,1);else{t.advance();let n=0;for(;t.next==Jg||t.next==e0;)t.advance(),n++;let r=t.next==Aa||t.next==Da||t.next==t0;t.acceptToken(r?Wz:Dz,-n)}},{contextual:!0,fallback:!0}),o6=new Zg((t,e)=>{let n=e.context.depth;if(n<0)return;let r=t.peek(-1);if((r==Aa||r==Da)&&e.context.depth>=0){let i=0,o=0;for(;;){if(t.next==Jg)i++;else if(t.next==e0)i+=8-i%8;else break;t.advance(),o++}i!=n&&t.next!=Aa&&t.next!=Da&&t.next!=t0&&(i<n?t.acceptToken(Lk,-o):t.acceptToken(Dk))}});function n0(t,e){this.parent=t,this.depth=e,this.hash=(t?t.hash+t.hash<<8:0)+e+(e<<4)}const s6=new n0(null,0),l6=new Ez({start:s6,reduce(t,e){return t.depth<0&&r6.indexOf(e)>-1?t.parent:t},shift(t,e,n,r){return e==Dk?new n0(t,n.pos-r.pos):e==Lk?t.parent:e==_z||e==Fz||e==jz?new n0(t,-1):t},hash(t){return t.hash}}),a6=new Zg(t=>{for(let e=0;e<5;e++){if(t.next!="print".charCodeAt(e))return;t.advance()}if(!/\w/.test(String.fromCharCode(t.next)))for(let e=0;;e++){let n=t.peek(e);if(!(n==Jg||n==e0)){n!=t6&&n!=n6&&n!=Aa&&n!=Da&&n!=t0&&t.acceptToken(Az);return}}}),c6={__proto__:null,await:40,or:48,and:50,in:54,not:56,is:58,if:64,else:66,lambda:70,yield:88,from:90,async:98,for:100,None:152,True:154,False:154,del:168,pass:172,break:176,continue:180,return:184,raise:192,import:196,as:198,global:202,nonlocal:204,assert:208,elif:218,while:222,try:228,except:230,finally:232,with:236,def:240,class:250},u6=zf.deserialize({version:13,states:"!?pO`Q$IXOOO%cQ$I[O'#GaOOQ$IS'#Cm'#CmOOQ$IS'#Cn'#CnO'RQ$IWO'#ClO(tQ$I[O'#G`OOQ$IS'#Ga'#GaOOQ$IS'#DR'#DROOQ$IS'#G`'#G`O)bQ$IWO'#CqO)rQ$IWO'#DbO*SQ$IWO'#DfOOQ$IS'#Ds'#DsO*gO`O'#DsO*oOpO'#DsO*wO!bO'#DtO+SO#tO'#DtO+_O&jO'#DtO+jO,UO'#DtO-lQ$I[O'#GQOOQ$IS'#GQ'#GQO'RQ$IWO'#GPO/OQ$I[O'#GPOOQ$IS'#E]'#E]O/gQ$IWO'#E^OOQ$IS'#GO'#GOO/qQ$IWO'#F}OOQ$IV'#F}'#F}O/|Q$IWO'#FPOOQ$IS'#Fr'#FrO0RQ$IWO'#FOOOQ$IV'#H]'#H]OOQ$IV'#F|'#F|OOQ$IT'#FR'#FRQ`Q$IXOOO'RQ$IWO'#CoO0aQ$IWO'#CzO0hQ$IWO'#DOO0vQ$IWO'#GeO1WQ$I[O'#EQO'RQ$IWO'#EROOQ$IS'#ET'#ETOOQ$IS'#EV'#EVOOQ$IS'#EX'#EXO1lQ$IWO'#EZO2SQ$IWO'#E_O/|Q$IWO'#EaO2gQ$I[O'#EaO/|Q$IWO'#EdO/gQ$IWO'#EgO/gQ$IWO'#EkO/gQ$IWO'#EnO2rQ$IWO'#EpO2yQ$IWO'#EuO3UQ$IWO'#EqO/gQ$IWO'#EuO/|Q$IWO'#EwO/|Q$IWO'#E|OOQ$IS'#Cc'#CcOOQ$IS'#Cd'#CdOOQ$IS'#Ce'#CeOOQ$IS'#Cf'#CfOOQ$IS'#Cg'#CgOOQ$IS'#Ch'#ChOOQ$IS'#Cj'#CjO'RQ$IWO,58|O'RQ$IWO,58|O'RQ$IWO,58|O'RQ$IWO,58|O'RQ$IWO,58|O'RQ$IWO,58|O3ZQ$IWO'#DmOOQ$IS,5:W,5:WO3nQ$IWO'#GoOOQ$IS,5:Z,5:ZO3{Q%1`O,5:ZO4QQ$I[O,59WO0aQ$IWO,59_O0aQ$IWO,59_O0aQ$IWO,59_O6pQ$IWO,59_O6uQ$IWO,59_O6|Q$IWO,59gO7TQ$IWO'#G`O8ZQ$IWO'#G_OOQ$IS'#G_'#G_OOQ$IS'#DX'#DXO8rQ$IWO,59]O'RQ$IWO,59]O9QQ$IWO,59]O9VQ$IWO,5:PO'RQ$IWO,5:POOQ$IS,59|,59|O9eQ$IWO,59|O9jQ$IWO,5:VO'RQ$IWO,5:VO'RQ$IWO,5:TOOQ$IS,5:Q,5:QO9{Q$IWO,5:QO:QQ$IWO,5:UOOOO'#FZ'#FZO:VO`O,5:_OOQ$IS,5:_,5:_OOOO'#F['#F[O:_OpO,5:_O:gQ$IWO'#DuOOOO'#F]'#F]O:wO!bO,5:`OOQ$IS,5:`,5:`OOOO'#F`'#F`O;SO#tO,5:`OOOO'#Fa'#FaO;_O&jO,5:`OOOO'#Fb'#FbO;jO,UO,5:`OOQ$IS'#Fc'#FcO;uQ$I[O,5:dO>gQ$I[O,5<kO?QQ%GlO,5<kO?qQ$I[O,5<kOOQ$IS,5:x,5:xO@YQ$IXO'#FkOAiQ$IWO,5;TOOQ$IV,5<i,5<iOAtQ$I[O'#HXOB]Q$IWO,5;kOOQ$IS-E9p-E9pOOQ$IV,5;j,5;jO3PQ$IWO'#EwOOQ$IT-E9P-E9POBeQ$I[O,59ZODlQ$I[O,59fOEVQ$IWO'#GbOEbQ$IWO'#GbO/|Q$IWO'#GbOEmQ$IWO'#DQOEuQ$IWO,59jOEzQ$IWO'#GfO'RQ$IWO'#GfO/gQ$IWO,5=POOQ$IS,5=P,5=PO/gQ$IWO'#D|OOQ$IS'#D}'#D}OFiQ$IWO'#FeOFyQ$IWO,58zOGXQ$IWO,58zO)eQ$IWO,5:jOG^Q$I[O'#GhOOQ$IS,5:m,5:mOOQ$IS,5:u,5:uOGqQ$IWO,5:yOHSQ$IWO,5:{OOQ$IS'#Fh'#FhOHbQ$I[O,5:{OHpQ$IWO,5:{OHuQ$IWO'#H[OOQ$IS,5;O,5;OOITQ$IWO'#HWOOQ$IS,5;R,5;RO3UQ$IWO,5;VO3UQ$IWO,5;YOIfQ$I[O'#H^O'RQ$IWO'#H^OIpQ$IWO,5;[O2rQ$IWO,5;[O/gQ$IWO,5;aO/|Q$IWO,5;cOIuQ$IXO'#ElOKOQ$IZO,5;]ONaQ$IWO'#H_O3UQ$IWO,5;aONlQ$IWO,5;cONqQ$IWO,5;hO!#fQ$I[O1G.hO!#mQ$I[O1G.hO!&^Q$I[O1G.hO!&hQ$I[O1G.hO!)RQ$I[O1G.hO!)fQ$I[O1G.hO!)yQ$IWO'#GnO!*XQ$I[O'#GQO/gQ$IWO'#GnO!*cQ$IWO'#GmOOQ$IS,5:X,5:XO!*kQ$IWO,5:XO!*pQ$IWO'#GpO!*{Q$IWO'#GpO!+`Q$IWO,5=ZOOQ$IS'#Dq'#DqOOQ$IS1G/u1G/uOOQ$IS1G.y1G.yO!,`Q$I[O1G.yO!,gQ$I[O1G.yO0aQ$IWO1G.yO!-SQ$IWO1G/ROOQ$IS'#DW'#DWO/gQ$IWO,59qOOQ$IS1G.w1G.wO!-ZQ$IWO1G/cO!-kQ$IWO1G/cO!-sQ$IWO1G/dO'RQ$IWO'#GgO!-xQ$IWO'#GgO!-}Q$I[O1G.wO!._Q$IWO,59fO!/eQ$IWO,5=VO!/uQ$IWO,5=VO!/}Q$IWO1G/kO!0SQ$I[O1G/kOOQ$IS1G/h1G/hO!0dQ$IWO,5=QO!1ZQ$IWO,5=QO/gQ$IWO1G/oO!1xQ$IWO1G/qO!1}Q$I[O1G/qO!2_Q$I[O1G/oOOQ$IS1G/l1G/lOOQ$IS1G/p1G/pOOOO-E9X-E9XOOQ$IS1G/y1G/yOOOO-E9Y-E9YO!2oQ$IWO'#G{O/gQ$IWO'#G{O!2}Q$IWO,5:aOOOO-E9Z-E9ZOOQ$IS1G/z1G/zOOOO-E9^-E9^OOOO-E9_-E9_OOOO-E9`-E9`OOQ$IS-E9a-E9aO!3YQ%GlO1G2VO!3yQ$I[O1G2VO'RQ$IWO,5<OOOQ$IS,5<O,5<OOOQ$IS-E9b-E9bOOQ$IS,5<V,5<VOOQ$IS-E9i-E9iOOQ$IV1G0o1G0oO/|Q$IWO'#FgO!4bQ$I[O,5=sOOQ$IS1G1V1G1VO!4yQ$IWO1G1VOOQ$IS'#DS'#DSO/gQ$IWO,5<|OOQ$IS,5<|,5<|O!5OQ$IWO'#FSO!5ZQ$IWO,59lO!5cQ$IWO1G/UO!5mQ$I[O,5=QOOQ$IS1G2k1G2kOOQ$IS,5:h,5:hO!6^Q$IWO'#GPOOQ$IS,5<P,5<POOQ$IS-E9c-E9cO!6oQ$IWO1G.fOOQ$IS1G0U1G0UO!6}Q$IWO,5=SO!7_Q$IWO,5=SO/gQ$IWO1G0eO/gQ$IWO1G0eO/|Q$IWO1G0gOOQ$IS-E9f-E9fO!7pQ$IWO1G0gO!7{Q$IWO1G0gO!8QQ$IWO,5=vO!8`Q$IWO,5=vO!8nQ$IWO,5=rO!9UQ$IWO,5=rO!9gQ$IZO1G0qO!<uQ$IZO1G0tO!@QQ$IWO,5=xO!@[Q$IWO,5=xO!@dQ$I[O,5=xO/gQ$IWO1G0vO!@nQ$IWO1G0vO3UQ$IWO1G0{ONlQ$IWO1G0}OOQ$IV,5;W,5;WO!@sQ$IYO,5;WO!@xQ$IZO1G0wO!DZQ$IWO'#FoO3UQ$IWO1G0wO3UQ$IWO1G0wO!DhQ$IWO,5=yO!DuQ$IWO,5=yO/|Q$IWO,5=yOOQ$IV1G0{1G0{O!D}Q$IWO'#EyO!E`Q%1`O1G0}OOQ$IV1G1S1G1SO3UQ$IWO1G1SOOQ$IS,5=Y,5=YOOQ$IS'#Dn'#DnO/gQ$IWO,5=YO!EhQ$IWO,5=XO!E{Q$IWO,5=XOOQ$IS1G/s1G/sO!FTQ$IWO,5=[O!FeQ$IWO,5=[O!FmQ$IWO,5=[O!GQQ$IWO,5=[O!GbQ$IWO,5=[OOQ$IS1G2u1G2uOOQ$IS7+$e7+$eO!5cQ$IWO7+$mO!ITQ$IWO1G.yO!I[Q$IWO1G.yOOQ$IS1G/]1G/]OOQ$IS,5;p,5;pO'RQ$IWO,5;pOOQ$IS7+$}7+$}O!IcQ$IWO7+$}OOQ$IS-E9S-E9SOOQ$IS7+%O7+%OO!IsQ$IWO,5=RO'RQ$IWO,5=ROOQ$IS7+$c7+$cO!IxQ$IWO7+$}O!JQQ$IWO7+%OO!JVQ$IWO1G2qOOQ$IS7+%V7+%VO!JgQ$IWO1G2qO!JoQ$IWO7+%VOOQ$IS,5;o,5;oO'RQ$IWO,5;oO!JtQ$IWO1G2lOOQ$IS-E9R-E9RO!KkQ$IWO7+%ZOOQ$IS7+%]7+%]O!KyQ$IWO1G2lO!LhQ$IWO7+%]O!LmQ$IWO1G2rO!L}Q$IWO1G2rO!MVQ$IWO7+%ZO!M[Q$IWO,5=gO!MrQ$IWO,5=gO!MrQ$IWO,5=gO!NQO!LQO'#DwO!N]OSO'#G|OOOO1G/{1G/{O!NbQ$IWO1G/{O!NjQ%GlO7+'qO# ZQ$I[O1G1jP# tQ$IWO'#FdOOQ$IS,5<R,5<ROOQ$IS-E9e-E9eOOQ$IS7+&q7+&qOOQ$IS1G2h1G2hOOQ$IS,5;n,5;nOOQ$IS-E9Q-E9QOOQ$IS7+$p7+$pO#!RQ$IWO,5<kO#!lQ$IWO,5<kO#!}Q$I[O,5;qO##bQ$IWO1G2nOOQ$IS-E9T-E9TOOQ$IS7+&P7+&PO##rQ$IWO7+&POOQ$IS7+&R7+&RO#$QQ$IWO'#HZO/|Q$IWO'#HYO#$fQ$IWO7+&ROOQ$IS,5<U,5<UO#$qQ$IWO1G3bOOQ$IS-E9h-E9hOOQ$IS,5<Q,5<QO#%PQ$IWO1G3^OOQ$IS-E9d-E9dO#%gQ$IZO7+&]O!DZQ$IWO'#FmO3UQ$IWO7+&]O3UQ$IWO7+&`O#(uQ$I[O,5<YO'RQ$IWO,5<YO#)PQ$IWO1G3dOOQ$IS-E9l-E9lO#)ZQ$IWO1G3dO3UQ$IWO7+&bO/gQ$IWO7+&bOOQ$IV7+&g7+&gO!E`Q%1`O7+&iO#)cQ$IXO1G0rOOQ$IV-E9m-E9mO3UQ$IWO7+&cO3UQ$IWO7+&cOOQ$IV,5<Z,5<ZO#+UQ$IWO,5<ZOOQ$IV7+&c7+&cO#+aQ$IZO7+&cO#.lQ$IWO,5<[O#.wQ$IWO1G3eOOQ$IS-E9n-E9nO#/UQ$IWO1G3eO#/^Q$IWO'#HaO#/lQ$IWO'#HaO/|Q$IWO'#HaOOQ$IS'#Ha'#HaO#/wQ$IWO'#H`OOQ$IS,5;e,5;eO#0PQ$IWO,5;eO/gQ$IWO'#E{OOQ$IV7+&i7+&iO3UQ$IWO7+&iOOQ$IV7+&n7+&nOOQ$IS1G2t1G2tOOQ$IS,5;s,5;sO#0UQ$IWO1G2sOOQ$IS-E9V-E9VO#0iQ$IWO,5;tO#0tQ$IWO,5;tO#1XQ$IWO1G2vOOQ$IS-E9W-E9WO#1iQ$IWO1G2vO#1qQ$IWO1G2vO#2RQ$IWO1G2vO#1iQ$IWO1G2vOOQ$IS<<HX<<HXO#2^Q$I[O1G1[OOQ$IS<<Hi<<HiP#2kQ$IWO'#FUO6|Q$IWO1G2mO#2xQ$IWO1G2mO#2}Q$IWO<<HiOOQ$IS<<Hj<<HjO#3_Q$IWO7+(]OOQ$IS<<Hq<<HqO#3oQ$I[O1G1ZP#4`Q$IWO'#FTO#4mQ$IWO7+(^O#4}Q$IWO7+(^O#5VQ$IWO<<HuO#5[Q$IWO7+(WOOQ$IS<<Hw<<HwO#6RQ$IWO,5;rO'RQ$IWO,5;rOOQ$IS-E9U-E9UOOQ$IS<<Hu<<HuOOQ$IS,5;x,5;xO/gQ$IWO,5;xO#6WQ$IWO1G3ROOQ$IS-E9[-E9[O#6nQ$IWO1G3ROOOO'#F_'#F_O#6|O!LQO,5:cOOOO,5=h,5=hOOOO7+%g7+%gO#7XQ$IWO1G2VO#7rQ$IWO1G2VP'RQ$IWO'#FVO/gQ$IWO<<IkO#8TQ$IWO,5=uO#8fQ$IWO,5=uO/|Q$IWO,5=uO#8wQ$IWO,5=tOOQ$IS<<Im<<ImP/|Q$IWO'#FjP/gQ$IWO'#FfOOQ$IV-E9k-E9kO3UQ$IWO<<IwOOQ$IV,5<X,5<XO3UQ$IWO,5<XOOQ$IV<<Iw<<IwOOQ$IV<<Iz<<IzO#8|Q$I[O1G1tP#9WQ$IWO'#FnO#9_Q$IWO7+)OO#9iQ$IZO<<I|O3UQ$IWO<<I|OOQ$IV<<JT<<JTO3UQ$IWO<<JTOOQ$IV'#Fl'#FlO#<tQ$IZO7+&^OOQ$IV<<I}<<I}O#>mQ$IZO<<I}OOQ$IV1G1u1G1uO/|Q$IWO1G1uO3UQ$IWO<<I}O/|Q$IWO1G1vP/gQ$IWO'#FpO#AxQ$IWO7+)PO#BVQ$IWO7+)POOQ$IS'#Ez'#EzO/gQ$IWO,5={O#B_Q$IWO,5={OOQ$IS,5={,5={O#BjQ$IWO,5=zO#B{Q$IWO,5=zOOQ$IS1G1P1G1POOQ$IS,5;g,5;gP#CTQ$IWO'#FXO#CeQ$IWO1G1`O#CxQ$IWO1G1`O#DYQ$IWO1G1`P#DeQ$IWO'#FYO#DrQ$IWO7+(bO#ESQ$IWO7+(bO#ESQ$IWO7+(bO#E[Q$IWO7+(bO#ElQ$IWO7+(XO6|Q$IWO7+(XOOQ$ISAN>TAN>TO#FVQ$IWO<<KxOOQ$ISAN>aAN>aO/gQ$IWO1G1^O#FgQ$I[O1G1^P#FqQ$IWO'#FWOOQ$IS1G1d1G1dP#GOQ$IWO'#F^O#G]Q$IWO7+(mOOOO-E9]-E9]O#GsQ$IWO7+'qOOQ$ISAN?VAN?VO#H^Q$IWO,5<TO#HrQ$IWO1G3aOOQ$IS-E9g-E9gO#ITQ$IWO1G3aOOQ$IS1G3`1G3`OOQ$IVAN?cAN?cOOQ$IV1G1s1G1sO3UQ$IWOAN?hO#IfQ$IZOAN?hOOQ$IVAN?oAN?oOOQ$IV-E9j-E9jOOQ$IV<<Ix<<IxO3UQ$IWOAN?iO3UQ$IWO7+'aOOQ$IVAN?iAN?iOOQ$IS7+'b7+'bO#LqQ$IWO<<LkOOQ$IS1G3g1G3gO/gQ$IWO1G3gOOQ$IS,5<],5<]O#MOQ$IWO1G3fOOQ$IS-E9o-E9oO#MaQ$IWO7+&zO#MqQ$IWO7+&zOOQ$IS7+&z7+&zO#M|Q$IWO<<K|O#N^Q$IWO<<K|O#N^Q$IWO<<K|O#NfQ$IWO'#GiOOQ$IS<<Ks<<KsO#NpQ$IWO<<KsOOQ$IS7+&x7+&xO/|Q$IWO1G1oP/|Q$IWO'#FiO$ ZQ$IWO7+({O$ lQ$IWO7+({OOQ$IVG25SG25SO3UQ$IWOG25SOOQ$IVG25TG25TOOQ$IV<<J{<<J{OOQ$IS7+)R7+)RP$ }Q$IWO'#FqOOQ$IS<<Jf<<JfO$!]Q$IWO<<JfO$!mQ$IWOANAhO$!}Q$IWOANAhO$#VQ$IWO'#GjOOQ$IS'#Gj'#GjO0hQ$IWO'#DaO$#pQ$IWO,5=TOOQ$ISANA_ANA_OOQ$IS7+'Z7+'ZO$$XQ$IWO<<LgOOQ$IVLD*nLD*nOOQ$ISAN@QAN@QO$$jQ$IWOG27SO$$zQ$IWO,59{OOQ$IS1G2o1G2oO#NfQ$IWO1G/gOOQ$IS7+%R7+%RO6|Q$IWO'#CzO6|Q$IWO,59_O6|Q$IWO,59_O6|Q$IWO,59_O$%PQ$I[O,5<kO6|Q$IWO1G.yO/gQ$IWO1G/UO/gQ$IWO7+$mP$%dQ$IWO'#FdO'RQ$IWO'#GPO$%qQ$IWO,59_O$%vQ$IWO,59_O$%}Q$IWO,59jO$&SQ$IWO1G/RO0hQ$IWO'#DOO6|Q$IWO,59g",stateData:"$&j~O$oOS$lOS$kOSQOS~OPhOTeOdsOfXOltOp!SOsuO|vO}!PO!R!VO!S!UO!VYO!ZZO!fdO!mdO!ndO!odO!vxO!xyO!zzO!|{O#O|O#S}O#U!OO#X!QO#Y!QO#[!RO#c!TO#f!WO#j!XO#l!YO#q!ZO#tlO$jqO$zQO${QO%PRO%QVO%f[O%g]O%j^O%m_O%s`O%vaO%xbO~OT!aO]!aO_!bOf!iO!V!kO!d!mO$u![O$v!]O$w!^O$x!_O$y!_O$z!`O${!`O$|!aO$}!aO%O!aO~Oh%TXi%TXj%TXk%TXl%TXm%TXp%TXw%TXx%TX!s%TX#^%TX$j%TX$m%TX%V%TX!O%TX!R%TX!S%TX%W%TX!W%TX![%TX}%TX#V%TXq%TX!j%TX~P$_OdsOfXO!VYO!ZZO!fdO!mdO!ndO!odO$zQO${QO%PRO%QVO%f[O%g]O%j^O%m_O%s`O%vaO%xbO~Ow%SXx%SX#^%SX$j%SX$m%SX%V%SX~Oh!pOi!qOj!oOk!oOl!rOm!sOp!tO!s%SX~P(`OT!zOl-dOs-rO|vO~P'ROT!}Ol-dOs-rO!W#OO~P'ROT#RO_#SOl-dOs-rO![#TO~P'RO%h#WO%i#YO~O%k#ZO%l#YO~O!Z#]O%n#^O%r#`O~O!Z#]O%t#aO%u#`O~O!Z#]O%i#`O%w#cO~O!Z#]O%l#`O%y#eO~OT$tX]$tX_$tXf$tXh$tXi$tXj$tXk$tXl$tXm$tXp$tXw$tX!V$tX!d$tX$u$tX$v$tX$w$tX$x$tX$y$tX$z$tX${$tX$|$tX$}$tX%O$tX!O$tX!R$tX!S$tX~O%f[O%g]O%j^O%m_O%s`O%vaO%xbOx$tX!s$tX#^$tX$j$tX$m$tX%V$tX%W$tX!W$tX![$tX}$tX#V$tXq$tX!j$tX~P+uOw#jOx$sX!s$sX#^$sX$j$sX$m$sX%V$sX~Ol-dOs-rO~P'RO#^#mO$j#oO$m#oO~O%QVO~O!R#tO#l!YO#q!ZO#tlO~OltO~P'ROT#yO_#zO%QVOxtP~OT$OOl-dOs-rO}$PO~P'ROx$RO!s$WO%V$SO#^!tX$j!tX$m!tX~OT$OOl-dOs-rO#^!}X$j!}X$m!}X~P'ROl-dOs-rO#^#RX$j#RX$m#RX~P'RO!d$^O!m$^O%QVO~OT$hO~P'RO!S$jO#j$kO#l$lO~Ox$mO~OT${O_${Ol-dOs-rO!O$}O~P'ROl-dOs-rOx%QO~P'RO%e%SO~O_!bOf!iO!V!kO!d!mOT`a]`ah`ai`aj`ak`al`am`ap`aw`ax`a!s`a#^`a$j`a$m`a$u`a$v`a$w`a$x`a$y`a$z`a${`a$|`a$}`a%O`a%V`a!O`a!R`a!S`a%W`a!W`a![`a}`a#V`aq`a!j`a~Ok%XO~Ol%XO~P'ROl-dO~P'ROh-fOi-gOj-eOk-eOl-nOm-oOp-sO!O%SX!R%SX!S%SX%W%SX!W%SX![%SX}%SX#V%SX!j%SX~P(`O%W%ZOw%RX!O%RX!R%RX!S%RX!W%RXx%RX~Ow%^O!O%]O!R%bO!S%aO~O!O%]O~Ow%eO!R%bO!S%aO!W%_X~O!W%iO~Ow%jOx%lO!R%bO!S%aO![%YX~O![%pO~O![%qO~O%h#WO%i%sO~O%k#ZO%l%sO~OT%vOl-dOs-rO|vO~P'RO!Z#]O%n#^O%r%yO~O!Z#]O%t#aO%u%yO~O!Z#]O%i%yO%w#cO~O!Z#]O%l%yO%y#eO~OT!la]!la_!laf!lah!lai!laj!lak!lal!lam!lap!law!lax!la!V!la!d!la!s!la#^!la$j!la$m!la$u!la$v!la$w!la$x!la$y!la$z!la${!la$|!la$}!la%O!la%V!la!O!la!R!la!S!la%W!la!W!la![!la}!la#V!laq!la!j!la~P#vOw&OOx$sa!s$sa#^$sa$j$sa$m$sa%V$sa~P$_OT&QOltOsuOx$sa!s$sa#^$sa$j$sa$m$sa%V$sa~P'ROw&OOx$sa!s$sa#^$sa$j$sa$m$sa%V$sa~OPhOTeOltOsuO|vO}!PO!vxO!xyO!zzO!|{O#O|O#S}O#U!OO#X!QO#Y!QO#[!RO#^$_X$j$_X$m$_X~P'RO#^#mO$j&VO$m&VO~O!d&WOf%{X$j%{X#V%{X#^%{X$m%{X#U%{X~Of!iO$j&YO~Ohcaicajcakcalcamcapcawcaxca!sca#^ca$jca$mca%Vca!Oca!Rca!Sca%Wca!Wca![ca}ca#Vcaqca!jca~P$_Opnawnaxna#^na$jna$mna%Vna~Oh!pOi!qOj!oOk!oOl!rOm!sO!sna~PDTO%V&[Ow%UXx%UX~O%QVOw%UXx%UX~Ow&_OxtX~Ox&aO~Ow%jO#^%YX$j%YX$m%YX!O%YXx%YX![%YX!j%YX%V%YX~OT-mOl-dOs-rO|vO~P'RO%V$SO#^Sa$jSa$mSa~O%V$SO~Ow&jO#^%[X$j%[X$m%[Xk%[X~P$_Ow&mO}&lO#^#Ra$j#Ra$m#Ra~O#V&nO#^#Ta$j#Ta$m#Ta~O!d$^O!m$^O#U&pO%QVO~O#U&pO~Ow&rO#^&OX$j&OX$m&OX~Ow&tO#^%zX$j%zX$m%zXx%zX~Ow&xOk&QX~P$_Ok&{O~OPhOTeOltOsuO|vO}!PO!vxO!xyO!zzO!|{O#O|O#S}O#U!OO#X!QO#Y!QO#[!RO$j'QO~P'ROq'UO#g'SO#h'TOP#eaT#ead#eaf#eal#eap#eas#ea|#ea}#ea!R#ea!S#ea!V#ea!Z#ea!f#ea!m#ea!n#ea!o#ea!v#ea!x#ea!z#ea!|#ea#O#ea#S#ea#U#ea#X#ea#Y#ea#[#ea#c#ea#f#ea#j#ea#l#ea#q#ea#t#ea$g#ea$j#ea$z#ea${#ea%P#ea%Q#ea%f#ea%g#ea%j#ea%m#ea%s#ea%v#ea%x#ea$i#ea$m#ea~Ow'VO#V'XOx&RX~Of'ZO~Of!iOx$mO~OT!aO]!aO_!bOf!iO!V!kO!d!mO$w!^O$x!_O$y!_O$z!`O${!`O$|!aO$}!aO%O!aOhUiiUijUikUilUimUipUiwUixUi!sUi#^Ui$jUi$mUi$uUi%VUi!OUi!RUi!SUi%WUi!WUi![Ui}Ui#VUiqUi!jUi~O$v!]O~PNyO$vUi~PNyOT!aO]!aO_!bOf!iO!V!kO!d!mO$z!`O${!`O$|!aO$}!aO%O!aOhUiiUijUikUilUimUipUiwUixUi!sUi#^Ui$jUi$mUi$uUi$vUi$wUi%VUi!OUi!RUi!SUi%WUi!WUi![Ui}Ui#VUiqUi!jUi~O$x!_O$y!_O~P!#tO$xUi$yUi~P!#tO_!bOf!iO!V!kO!d!mOhUiiUijUikUilUimUipUiwUixUi!sUi#^Ui$jUi$mUi$uUi$vUi$wUi$xUi$yUi$zUi${Ui%VUi!OUi!RUi!SUi%WUi!WUi![Ui}Ui#VUiqUi!jUi~OT!aO]!aO$|!aO$}!aO%O!aO~P!&rOTUi]Ui$|Ui$}Ui%OUi~P!&rO!R%bO!S%aOw%bX!O%bX~O%V'`O%W'`O~P+uOw'bO!O%aX~O!O'dO~Ow'eOx'gO!W%dX~Ol-dOs-rOw'eOx'hO!W%dX~P'RO!W'jO~Oj!oOk!oOl!rOm!sOhgipgiwgixgi!sgi#^gi$jgi$mgi%Vgi~Oi!qO~P!+eOigi~P!+eOh-fOi-gOj-eOk-eOl-nOm-oO~Oq'lO~P!,nOT'qOl-dOs-rO!O'rO~P'ROw'sO!O'rO~O!O'uO~O!S'wO~Ow'sO!O'xO!R%bO!S%aO~P$_Oh-fOi-gOj-eOk-eOl-nOm-oO!Ona!Rna!Sna%Wna!Wna![na}na#Vnaqna!jna~PDTOT'qOl-dOs-rO!W%_a~P'ROw'{O!W%_a~O!W'|O~Ow'{O!R%bO!S%aO!W%_a~P$_OT(QOl-dOs-rO![%Ya#^%Ya$j%Ya$m%Ya!O%Yax%Ya!j%Ya%V%Ya~P'ROw(RO![%Ya#^%Ya$j%Ya$m%Ya!O%Yax%Ya!j%Ya%V%Ya~O![(UO~Ow(RO!R%bO!S%aO![%Ya~P$_Ow(XO!R%bO!S%aO![%`a~P$_Ow([Ox%oX![%oX!j%oX~Ox(_O![(aO!j(bO~OT&QOltOsuOx$si!s$si#^$si$j$si$m$si%V$si~P'ROw(cOx$si!s$si#^$si$j$si$m$si%V$si~O!d&WOf%{a$j%{a#V%{a#^%{a$m%{a#U%{a~O$j(hO~OT#yO_#zO%QVO~Ow&_Oxta~OltOsuO~P'ROw(RO#^%Ya$j%Ya$m%Ya!O%Yax%Ya![%Ya!j%Ya%V%Ya~P$_Ow(mO#^$sX$j$sX$m$sX%V$sX~O%V$SO#^Si$jSi$mSi~O#^%[a$j%[a$m%[ak%[a~P'ROw(pO#^%[a$j%[a$m%[ak%[a~OT(tOf(vO%QVO~O#U(wO~O%QVO#^&Oa$j&Oa$m&Oa~Ow(yO#^&Oa$j&Oa$m&Oa~Ol-dOs-rO#^%za$j%za$m%zax%za~P'ROw(|O#^%za$j%za$m%zax%za~Oq)QO#a)POP#_iT#_id#_if#_il#_ip#_is#_i|#_i}#_i!R#_i!S#_i!V#_i!Z#_i!f#_i!m#_i!n#_i!o#_i!v#_i!x#_i!z#_i!|#_i#O#_i#S#_i#U#_i#X#_i#Y#_i#[#_i#c#_i#f#_i#j#_i#l#_i#q#_i#t#_i$g#_i$j#_i$z#_i${#_i%P#_i%Q#_i%f#_i%g#_i%j#_i%m#_i%s#_i%v#_i%x#_i$i#_i$m#_i~Oq)ROP#biT#bid#bif#bil#bip#bis#bi|#bi}#bi!R#bi!S#bi!V#bi!Z#bi!f#bi!m#bi!n#bi!o#bi!v#bi!x#bi!z#bi!|#bi#O#bi#S#bi#U#bi#X#bi#Y#bi#[#bi#c#bi#f#bi#j#bi#l#bi#q#bi#t#bi$g#bi$j#bi$z#bi${#bi%P#bi%Q#bi%f#bi%g#bi%j#bi%m#bi%s#bi%v#bi%x#bi$i#bi$m#bi~OT)TOk&Qa~P'ROw)UOk&Qa~Ow)UOk&Qa~P$_Ok)YO~O$h)]O~Oq)`O#g'SO#h)_OP#eiT#eid#eif#eil#eip#eis#ei|#ei}#ei!R#ei!S#ei!V#ei!Z#ei!f#ei!m#ei!n#ei!o#ei!v#ei!x#ei!z#ei!|#ei#O#ei#S#ei#U#ei#X#ei#Y#ei#[#ei#c#ei#f#ei#j#ei#l#ei#q#ei#t#ei$g#ei$j#ei$z#ei${#ei%P#ei%Q#ei%f#ei%g#ei%j#ei%m#ei%s#ei%v#ei%x#ei$i#ei$m#ei~Ol-dOs-rOx$mO~P'ROl-dOs-rOx&Ra~P'ROw)fOx&Ra~OT)jO_)kO!O)nO$|)lO%QVO~Ox$mO&U)pO~OT${O_${Ol-dOs-rO!O%aa~P'ROw)vO!O%aa~Ol-dOs-rOx)yO!W%da~P'ROw)zO!W%da~Ol-dOs-rOw)zOx)}O!W%da~P'ROl-dOs-rOw)zO!W%da~P'ROw)zOx)}O!W%da~Oj-eOk-eOl-nOm-oOhgipgiwgi!Ogi!Rgi!Sgi%Wgi!Wgixgi![gi#^gi$jgi$mgi}gi#Vgiqgi!jgi%Vgi~Oi-gO~P!GmOigi~P!GmOT'qOl-dOs-rO!O*SO~P'ROk*UO~Ow*WO!O*SO~O!O*XO~OT'qOl-dOs-rO!W%_i~P'ROw*YO!W%_i~O!W*ZO~OT(QOl-dOs-rO![%Yi#^%Yi$j%Yi$m%Yi!O%Yix%Yi!j%Yi%V%Yi~P'ROw*^O!R%bO!S%aO![%`i~Ow*aO![%Yi#^%Yi$j%Yi$m%Yi!O%Yix%Yi!j%Yi%V%Yi~O![*bO~O_*dOl-dOs-rO![%`i~P'ROw*^O![%`i~O![*fO~OT*hOl-dOs-rOx%oa![%oa!j%oa~P'ROw*iOx%oa![%oa!j%oa~O!Z#]O%q*lO![!kX~O![*nO~Ox(_O![*oO~OT&QOltOsuOx$sq!s$sq#^$sq$j$sq$m$sq%V$sq~P'ROw$Wix$Wi!s$Wi#^$Wi$j$Wi$m$Wi%V$Wi~P$_OT&QOltOsuO~P'ROT&QOl-dOs-rO#^$sa$j$sa$m$sa%V$sa~P'ROw*pO#^$sa$j$sa$m$sa%V$sa~Ow#ya#^#ya$j#ya$m#yak#ya~P$_O#^%[i$j%[i$m%[ik%[i~P'ROw*sO#^#Rq$j#Rq$m#Rq~Ow*tO#V*vO#^%}X$j%}X$m%}X!O%}X~OT*xOf(vO%QVO~O%QVO#^&Oi$j&Oi$m&Oi~Ol-dOs-rO#^%zi$j%zi$m%zix%zi~P'ROq*|O#a)POP#_qT#_qd#_qf#_ql#_qp#_qs#_q|#_q}#_q!R#_q!S#_q!V#_q!Z#_q!f#_q!m#_q!n#_q!o#_q!v#_q!x#_q!z#_q!|#_q#O#_q#S#_q#U#_q#X#_q#Y#_q#[#_q#c#_q#f#_q#j#_q#l#_q#q#_q#t#_q$g#_q$j#_q$z#_q${#_q%P#_q%Q#_q%f#_q%g#_q%j#_q%m#_q%s#_q%v#_q%x#_q$i#_q$m#_q~Ok$baw$ba~P$_OT)TOk&Qi~P'ROw+TOk&Qi~OPhOTeOltOp!SOsuO|vO}!PO!R!VO!S!UO!vxO!xyO!zzO!|{O#O|O#S}O#U!OO#X!QO#Y!QO#[!RO#c!TO#f!WO#j!XO#l!YO#q!ZO#tlO~P'ROw+_Ox$mO#V+_O~O#h+`OP#eqT#eqd#eqf#eql#eqp#eqs#eq|#eq}#eq!R#eq!S#eq!V#eq!Z#eq!f#eq!m#eq!n#eq!o#eq!v#eq!x#eq!z#eq!|#eq#O#eq#S#eq#U#eq#X#eq#Y#eq#[#eq#c#eq#f#eq#j#eq#l#eq#q#eq#t#eq$g#eq$j#eq$z#eq${#eq%P#eq%Q#eq%f#eq%g#eq%j#eq%m#eq%s#eq%v#eq%x#eq$i#eq$m#eq~O#V+aOw$dax$da~Ol-dOs-rOx&Ri~P'ROw+cOx&Ri~Ox$RO%V+eOw&TX!O&TX~O%QVOw&TX!O&TX~Ow+iO!O&SX~O!O+kO~OT${O_${Ol-dOs-rO!O%ai~P'ROx+nOw#|a!W#|a~Ol-dOs-rOx+oOw#|a!W#|a~P'ROl-dOs-rOx)yO!W%di~P'ROw+rO!W%di~Ol-dOs-rOw+rO!W%di~P'ROw+rOx+uO!W%di~Ow#xi!O#xi!W#xi~P$_OT'qOl-dOs-rO~P'ROk+wO~OT'qOl-dOs-rO!O+xO~P'ROT'qOl-dOs-rO!W%_q~P'ROw#wi![#wi#^#wi$j#wi$m#wi!O#wix#wi!j#wi%V#wi~P$_OT(QOl-dOs-rO~P'RO_*dOl-dOs-rO![%`q~P'ROw+yO![%`q~O![+zO~OT(QOl-dOs-rO![%Yq#^%Yq$j%Yq$m%Yq!O%Yqx%Yq!j%Yq%V%Yq~P'ROx+{O~OT*hOl-dOs-rOx%oi![%oi!j%oi~P'ROw,QOx%oi![%oi!j%oi~O!Z#]O%q*lO![!ka~OT&QOl-dOs-rO#^$si$j$si$m$si%V$si~P'ROw,SO#^$si$j$si$m$si%V$si~O%QVO#^%}a$j%}a$m%}a!O%}a~Ow,VO#^%}a$j%}a$m%}a!O%}a~O!O,YO~Ok$biw$bi~P$_OT)TO~P'ROT)TOk&Qq~P'ROq,]OP#dyT#dyd#dyf#dyl#dyp#dys#dy|#dy}#dy!R#dy!S#dy!V#dy!Z#dy!f#dy!m#dy!n#dy!o#dy!v#dy!x#dy!z#dy!|#dy#O#dy#S#dy#U#dy#X#dy#Y#dy#[#dy#c#dy#f#dy#j#dy#l#dy#q#dy#t#dy$g#dy$j#dy$z#dy${#dy%P#dy%Q#dy%f#dy%g#dy%j#dy%m#dy%s#dy%v#dy%x#dy$i#dy$m#dy~OPhOTeOltOp!SOsuO|vO}!PO!R!VO!S!UO!vxO!xyO!zzO!|{O#O|O#S}O#U!OO#X!QO#Y!QO#[!RO#c!TO#f!WO#j!XO#l!YO#q!ZO#tlO$i,aO$m,aO~P'RO#h,bOP#eyT#eyd#eyf#eyl#eyp#eys#ey|#ey}#ey!R#ey!S#ey!V#ey!Z#ey!f#ey!m#ey!n#ey!o#ey!v#ey!x#ey!z#ey!|#ey#O#ey#S#ey#U#ey#X#ey#Y#ey#[#ey#c#ey#f#ey#j#ey#l#ey#q#ey#t#ey$g#ey$j#ey$z#ey${#ey%P#ey%Q#ey%f#ey%g#ey%j#ey%m#ey%s#ey%v#ey%x#ey$i#ey$m#ey~Ol-dOs-rOx&Rq~P'ROw,fOx&Rq~O%V+eOw&Ta!O&Ta~OT)jO_)kO$|)lO%QVO!O&Sa~Ow,jO!O&Sa~OT${O_${Ol-dOs-rO~P'ROl-dOs-rOx,lOw#|i!W#|i~P'ROl-dOs-rOw#|i!W#|i~P'ROx,lOw#|i!W#|i~Ol-dOs-rOx)yO~P'ROl-dOs-rOx)yO!W%dq~P'ROw,oO!W%dq~Ol-dOs-rOw,oO!W%dq~P'ROp,rO!R%bO!S%aO!O%Zq!W%Zq![%Zqw%Zq~P!,nO_*dOl-dOs-rO![%`y~P'ROw#zi![#zi~P$_O_*dOl-dOs-rO~P'ROT*hOl-dOs-rO~P'ROT*hOl-dOs-rOx%oq![%oq!j%oq~P'ROT&QOl-dOs-rO#^$sq$j$sq$m$sq%V$sq~P'RO#V,vOw$]a#^$]a$j$]a$m$]a!O$]a~O%QVO#^%}i$j%}i$m%}i!O%}i~Ow,xO#^%}i$j%}i$m%}i!O%}i~Oq,{OP#d!RT#d!Rd#d!Rf#d!Rl#d!Rp#d!Rs#d!R|#d!R}#d!R!R#d!R!S#d!R!V#d!R!Z#d!R!f#d!R!m#d!R!n#d!R!o#d!R!v#d!R!x#d!R!z#d!R!|#d!R#O#d!R#S#d!R#U#d!R#X#d!R#Y#d!R#[#d!R#c#d!R#f#d!R#j#d!R#l#d!R#q#d!R#t#d!R$g#d!R$j#d!R$z#d!R${#d!R%P#d!R%Q#d!R%f#d!R%g#d!R%j#d!R%m#d!R%s#d!R%v#d!R%x#d!R$i#d!R$m#d!R~Ol-dOs-rOx&Ry~P'ROT)jO_)kO$|)lO%QVO!O&Si~Ol-dOs-rOw#|q!W#|q~P'ROx-ROw#|q!W#|q~Ol-dOs-rOx)yO!W%dy~P'ROw-SO!W%dy~Ol-dOs-WO~P'ROp,rO!R%bO!S%aO!O%Zy!W%Zy![%Zyw%Zy~P!,nO%QVO#^%}q$j%}q$m%}q!O%}q~Ow-[O#^%}q$j%}q$m%}q!O%}q~OT)jO_)kO$|)lO%QVO~Ol-dOs-rOw#|y!W#|y~P'ROl-dOs-rOx)yO!W%d!R~P'ROw-_O!W%d!R~Op%^X!O%^X!R%^X!S%^X!W%^X![%^Xw%^X~P!,nOp,rO!R%bO!S%aO!O%]a!W%]a![%]aw%]a~O%QVO#^%}y$j%}y$m%}y!O%}y~Ol-dOs-rOx)yO!W%d!Z~P'ROx-bO~Ow*pO#^$sa$j$sa$m$sa%V$sa~P$_OT&QOl-dOs-rO~P'ROk-iO~Ol-iO~P'ROx-jO~Oq-kO~P!,nO%g%j%v%x%f!Z%n%t%w%y%m%s%m%Q~",goto:"!-g&UPPPP&VP&_)p*V*m+U+n,XP,sP&_-a-a&_P&_P0rPPPPPP0r3bPP3bP5n5w:zPP:};];`PPP&_&_PP;l&_PP&_&_PP&_&_&_&_;p<d&_P<gP<j<j@PP@e&_PPP@i@o&VP&V&VP&VP&VP&VP&VP&V&V&VP&VPP&VPP&VP@uP@|ASP@|P@|@|PPP@|PB{PCUC[CbB{P@|ChPCoCuC{DXDkDqD{EREoEuE{FRF]FcFiFoFuF{G_GiGoGuG{HVH]HcHiHoHyIPIZIaPPPPPPPPPIjIrI{JVJbPPPPPPPPPPPPNw! a!%o!({PP!)T!)c!)l!*b!*X!*k!*q!*t!*w!*z!+S!+pPPPPPPPPPP!+s!+vPPPPPPPPP!+|!,Y!,f!,l!,u!,x!-O!-U!-[!-_]iOr#m$m)]+Z'odOSXYZehrstvx|}!R!S!T!U!X!c!d!e!f!g!h!i!k!o!p!q!s!t!z!}#R#S#]#j#m$O$P$R$T$W$h$j$k$m${%Q%X%[%^%a%e%j%l%v&O&Q&]&a&j&l&m&t&x&{'S'V'a'b'e'g'h'l'q's'w'{(Q(R(X([(c(e(m(p(|)P)T)U)Y)])f)p)v)y)z)}*T*U*W*Y*]*^*a*d*h*i*p*r*s*z+S+T+Z+b+c+f+m+n+o+q+r+u+w+y+{+},P,Q,S,f,h,l,o,r-R-S-_-b-d-e-f-g-i-j-k-l-m-o-sw!cP#i#v$X$g%c%h%n%o&b&z(d(o)S*R*[+R+|-hy!dP#i#v$X$g$s%c%h%n%o&b&z(d(o)S*R*[+R+|-h{!eP#i#v$X$g$s$t%c%h%n%o&b&z(d(o)S*R*[+R+|-h}!fP#i#v$X$g$s$t$u%c%h%n%o&b&z(d(o)S*R*[+R+|-h!P!gP#i#v$X$g$s$t$u$v%c%h%n%o&b&z(d(o)S*R*[+R+|-h!R!hP#i#v$X$g$s$t$u$v$w%c%h%n%o&b&z(d(o)S*R*[+R+|-h!V!hP!n#i#v$X$g$s$t$u$v$w$x%c%h%n%o&b&z(d(o)S*R*[+R+|-h'oSOSXYZehrstvx|}!R!S!T!U!X!c!d!e!f!g!h!i!k!o!p!q!s!t!z!}#R#S#]#j#m$O$P$R$T$W$h$j$k$m${%Q%X%[%^%a%e%j%l%v&O&Q&]&a&j&l&m&t&x&{'S'V'a'b'e'g'h'l'q's'w'{(Q(R(X([(c(e(m(p(|)P)T)U)Y)])f)p)v)y)z)}*T*U*W*Y*]*^*a*d*h*i*p*r*s*z+S+T+Z+b+c+f+m+n+o+q+r+u+w+y+{+},P,Q,S,f,h,l,o,r-R-S-_-b-d-e-f-g-i-j-k-l-m-o-s&ZUOXYZhrtv|}!R!S!T!X!i!k!o!p!q!s!t#]#j#m$P$R$T$W$k$m${%Q%X%[%^%e%j%l%v&O&]&a&l&m&t&{'S'V'a'b'e'g'h'l's'{(R(X([(c(e(m(|)P)Y)])f)p)v)y)z)}*T*U*W*Y*]*^*a*h*i*p*s*z+Z+b+c+f+m+n+o+q+r+u+w+y+{+},P,Q,S,f,h,l,o,r-R-S-_-b-d-e-f-g-i-j-k-l-o-s%eWOXYZhrv|}!R!S!T!X!i!k#]#j#m$P$R$T$W$k$m${%Q%[%^%e%j%l%v&O&]&a&l&m&t&{'S'V'a'b'e'g'h'l's'{(R(X([(c(e(m(|)P)Y)])f)p)v)y)z)}*T*W*Y*]*^*a*h*i*p*s*z+Z+b+c+f+m+n+o+q+r+u+y+{+},P,Q,S,f,h,l,o-R-S-_-j-k-lQ#|uQ-`-WR-p-r'fdOSXYZehrstvx|}!R!S!T!U!X!c!d!e!f!g!h!k!o!p!q!s!t!z!}#R#S#]#j#m$O$P$R$T$W$h$j$k$m${%Q%X%[%^%a%e%j%l%v&O&Q&]&a&j&l&m&t&x&{'S'V'a'e'g'h'l'q's'w'{(Q(R(X([(c(e(m(p(|)P)T)U)Y)])f)p)y)z)}*T*U*W*Y*]*^*a*d*h*i*p*r*s*z+S+T+Z+b+c+f+n+o+q+r+u+w+y+{+},P,Q,S,f,h,l,o,r-R-S-_-b-d-e-f-g-i-j-k-l-m-o-sW#pl!O!P$_W#xu&_-W-rQ$a!QQ$q!YQ$r!ZW$z!i'b)v+mS&^#y#zQ'O$lQ(f&WQ(t&nU(u&p(v(wU(x&r(y*yQ)h'XW)i'Z+i,j-PS+h)j)kY,U*t,V,w,x-[Q,X*vQ,c+_Q,e+aR-Z,vR&]#xi!wXY!S!T%^%e's'{)P*T*W*YR%[!vQ!{XQ%w#]Q&f$TR&i$WT-V,r-b!U!jP!n#i#v$X$g$s$t$u$v$w$x%c%h%n%o&b&z(d(o)S*R*[+R+|-hQ&Z#qR'^$rR'a$zR%T!m'ncOSXYZehrstvx|}!R!S!T!U!X!c!d!e!f!g!h!i!k!o!p!q!s!t!z!}#R#S#]#j#m$O$P$R$T$W$h$j$k$m${%Q%X%[%^%a%e%j%l%v&O&Q&]&a&j&l&m&t&x&{'S'V'a'b'e'g'h'l'q's'w'{(Q(R(X([(c(e(m(p(|)P)T)U)Y)])f)p)v)y)z)}*T*U*W*Y*]*^*a*d*h*i*p*r*s*z+S+T+Z+b+c+f+m+n+o+q+r+u+w+y+{+},P,Q,S,f,h,l,o,r-R-S-_-b-d-e-f-g-i-j-k-l-m-o-sT#gc#hS#^_#_S#a`#bS#ca#dS#eb#fT*l(_*mT(`%w(bQ$VwR+g)iX$Tw$U$V&hZkOr$m)]+ZXoOr)]+ZQ$n!WQ&v$eQ&w$fQ'Y$pQ']$rQ)Z&}Q)a'SQ)c'TQ)d'UQ)q'[Q)s'^Q*})PQ+P)QQ+Q)RQ+U)XS+W)[)rQ+[)_Q+])`Q+^)bQ,Z*|Q,[+OQ,^+VQ,_+XQ,d+`Q,z,]Q,|,bQ,},cR-],{WoOr)]+ZR#snQ'[$qR)['OQ+f)iR,h+gQ)r'[R+X)[ZmOnr)]+ZQrOR#urQ&`#{R(k&`S%k#Q#}S(S%k(VT(V%n&bQ%_!yQ%f!|W't%_%f'y'}Q'y%cR'}%hQ&k$XR(q&kQ(Y%oQ*_(TT*e(Y*_Q'c$|R)w'cS'f%P%QY){'f)|+s,p-TU)|'g'h'iU+s)}*O*PS,p+t+uR-T,qQ#X]R%r#XQ#[^R%t#[Q#__R%x#_Q(]%uS*j(]*kR*k(^Q*m(_R,R*mQ#b`R%z#bQ#daR%{#dQ#fbR%|#fQ#hcR%}#hQ#kfQ&P#iW&S#k&P(n*qQ(n&eR*q-hQ$UwS&g$U&hR&h$VQ&u$cR(}&uQ&X#pR(g&XQ$_!PR&o$_Q*u(uS,W*u,yR,y,XQ&s$aR(z&sQ#njR&U#nQ+Z)]R,`+ZQ)O&vR*{)OQ&y$gS)V&y)WR)W&zQ'R$nR)^'RQ'W$oS)g'W+dR+d)hQ+j)mR,k+jWnOr)]+ZR#rnSqOrT+Y)]+ZWpOr)]+ZR'P$mYjOr$m)]+ZR&T#m[wOr#m$m)]+ZR&f$T&YPOXYZhrtv|}!R!S!T!X!i!k!o!p!q!s!t#]#j#m$P$R$T$W$k$m${%Q%X%[%^%e%j%l%v&O&]&a&l&m&t&{'S'V'a'b'e'g'h'l's'{(R(X([(c(e(m(|)P)Y)])f)p)v)y)z)}*T*U*W*Y*]*^*a*h*i*p*s*z+Z+b+c+f+m+n+o+q+r+u+w+y+{+},P,Q,S,f,h,l,o,r-R-S-_-b-d-e-f-g-i-j-k-l-o-sQ!nSQ#ieQ#vsU$Xx%a'wS$g!U$jQ$s!cQ$t!dQ$u!eQ$v!fQ$w!gQ$x!hQ%c!zQ%h!}Q%n#RQ%o#SQ&b$OQ&z$hQ(d&QU(o&j(p*rW)S&x)U+S+TQ*R'qQ*[(QQ+R)TQ+|*dR-h-mQ!yXQ!|YQ$e!SQ$f!T^'p%^%e's'{*T*W*YR+O)P[fOr#m$m)]+Zh!vXY!S!T%^%e's'{)P*T*W*YQ#QZQ#lhS#}v|Q$[}W$c!R$W&{)YS$o!X$kW$y!i'b)v+mQ%P!kQ%u#]`&R#j&O(c(e(m*p,S-lQ&c$PQ&d$RQ&e$TQ'_${Q'i%QQ'o%[W(P%j(R*]*aQ(T%lQ(^%vQ(i&]S(l&a-jQ(r&lQ(s&mU({&t(|*zQ)b'SY)e'V)f+b+c,fQ)t'a^)x'e)z+q+r,o-S-_Q*O'gQ*P'hS*Q'l-kW*c(X*^+y+}W*g([*i,P,QQ+l)pQ+p)yQ+t)}Q,O*hQ,T*sQ,g+fQ,m+nQ,n+oQ,q+uQ,u+{Q-O,hQ-Q,lR-^-RhTOr#j#m$m&O&a'l(c(e)]+Z$z!uXYZhv|}!R!S!T!X!i!k#]$P$R$T$W$k${%Q%[%^%e%j%l%v&]&l&m&t&{'S'V'a'b'e'g'h's'{(R(X([(m(|)P)Y)f)p)v)y)z)}*T*W*Y*]*^*a*h*i*p*s*z+b+c+f+m+n+o+q+r+u+y+{+},P,Q,S,f,h,l,o-R-S-_-j-k-lQ#wtW%U!o!s-e-oQ%V!pQ%W!qQ%Y!tQ%d-dS'k%X-iQ'm-fQ'n-gQ+v*UQ,t+wS-U,r-bR-q-sU#{u-W-rR(j&_[gOr#m$m)]+ZX!xX#]$T$WQ#VZQ$QvR$Z|Q%`!yQ%g!|Q%m#QQ'_$yQ'z%cQ(O%hQ(W%nQ(Z%oQ*`(TQ,s+vQ-Y,tR-a-XQ$YxQ'v%aR*V'wQ-X,rR-c-bR#PYR#UZR%O!iQ$|!iV)u'b)v+m!V!lP!n#i#v$X$g$s$t$u$v$w$x%c%h%n%o&b&z(d(o)S*R*[+R+|-hR%R!kR%w#]Q(a%wR*o(bQ$d!RQ&i$WQ)X&{R+V)YQ#qlQ$]!OQ$`!PR&q$_Q(t&pR*x(wQ(t&pQ*w(vR*x(wR$b!QXpOr)]+ZQ$i!UR&|$jQ$p!XR&}$kR)o'ZQ)m'ZV,i+i,j-P",nodeNames:"\u26A0 print Comment Script AssignStatement * BinaryExpression BitOp BitOp BitOp BitOp ArithOp ArithOp @ ArithOp ** UnaryExpression ArithOp BitOp AwaitExpression await ParenthesizedExpression ( BinaryExpression or and CompareOp in not is UnaryExpression ConditionalExpression if else LambdaExpression lambda ParamList VariableName AssignOp , : NamedExpression AssignOp YieldExpression yield from ) TupleExpression ComprehensionExpression async for LambdaExpression ArrayExpression [ ] ArrayComprehensionExpression DictionaryExpression { } DictionaryComprehensionExpression SetExpression SetComprehensionExpression CallExpression ArgList AssignOp MemberExpression . PropertyName Number String FormatString FormatReplacement FormatConversion FormatSpec ContinuedString Ellipsis None Boolean TypeDef AssignOp UpdateStatement UpdateOp ExpressionStatement DeleteStatement del PassStatement pass BreakStatement break ContinueStatement continue ReturnStatement return YieldStatement PrintStatement RaiseStatement raise ImportStatement import as ScopeStatement global nonlocal AssertStatement assert StatementGroup ; IfStatement Body elif WhileStatement while ForStatement TryStatement try except finally WithStatement with FunctionDefinition def ParamList AssignOp TypeDef ClassDefinition class DecoratedStatement Decorator At",maxTerm:236,context:l6,nodeProps:[[xe.group,-14,4,80,82,83,85,87,89,91,93,94,95,97,100,103,"Statement Statement",-22,6,16,19,21,37,47,48,52,55,56,59,60,61,62,65,68,69,70,74,75,76,77,"Expression",-9,105,107,110,112,113,117,119,124,126,"Statement"]],skippedNodes:[0,2],repeatNodeCount:32,tokenData:"&AaMgR!^OX$}XY!#xY[$}[]!#x]p$}pq!#xqr!&Srs!)yst!C{tu$}uv$+}vw$.awx$/mxy$Lgyz$Mmz{$Ns{|%#c|}%$o}!O%%u!O!P%([!P!Q%3b!Q!R%6Q!R![%:S![!]%EO!]!^%Gb!^!_%Hh!_!`%KW!`!a%Ld!a!b$}!b!c& P!c!d&!_!d!e&$P!e!h&!_!h!i&.R!i!t&!_!t!u&7g!u!w&!_!w!x&,a!x!}&!_!}#O&9q#O#P!%b#P#Q&:w#Q#R&;}#R#S&!_#S#T$}#T#U&!_#U#V&$P#V#Y&!_#Y#Z&.R#Z#f&!_#f#g&7g#g#i&!_#i#j&,a#j#o&!_#o#p&=Z#p#q&>P#q#r&?]#r#s&@Z#s$g$}$g~&!_<r%`Z%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}9[&^Z%q7[%hS%n`%w!bOr'PrsCxsw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'P9['^Z%q7[%hS%kW%n`%w!bOr'Prs&Rsw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'P8z(WZ%q7[%kWOr(yrs)wsw(ywx;bx#O(y#O#P2V#P#o(y#o#p7n#p#q(y#q#r2k#r~(y8z)UZ%q7[%hS%kW%w!bOr(yrs)wsw(ywx(Px#O(y#O#P2V#P#o(y#o#p7n#p#q(y#q#r2k#r~(y8z*QZ%q7[%hS%w!bOr(yrs*ssw(ywx(Px#O(y#O#P2V#P#o(y#o#p7n#p#q(y#q#r2k#r~(y8z*|Z%q7[%hS%w!bOr(yrs+osw(ywx(Px#O(y#O#P2V#P#o(y#o#p7n#p#q(y#q#r2k#r~(y8r+xX%q7[%hS%w!bOw+owx,ex#O+o#O#P.V#P#o+o#o#p0^#p#q+o#q#r.k#r~+o8r,jX%q7[Ow+owx-Vx#O+o#O#P.V#P#o+o#o#p0^#p#q+o#q#r.k#r~+o8r-[X%q7[Ow+owx-wx#O+o#O#P.V#P#o+o#o#p0^#p#q+o#q#r.k#r~+o7[-|R%q7[O#o-w#p#q-w#r~-w8r.[T%q7[O#o+o#o#p.k#p#q+o#q#r.k#r~+o!f.rV%hS%w!bOw.kwx/Xx#O.k#O#P0W#P#o.k#o#p0^#p~.k!f/[VOw.kwx/qx#O.k#O#P0W#P#o.k#o#p0^#p~.k!f/tUOw.kx#O.k#O#P0W#P#o.k#o#p0^#p~.k!f0ZPO~.k!f0cV%hSOw0xwx1^x#O0x#O#P2P#P#o0x#o#p.k#p~0xS0}T%hSOw0xwx1^x#O0x#O#P2P#P~0xS1aTOw0xwx1px#O0x#O#P2P#P~0xS1sSOw0xx#O0x#O#P2P#P~0xS2SPO~0x8z2[T%q7[O#o(y#o#p2k#p#q(y#q#r2k#r~(y!n2tX%hS%kW%w!bOr2krs3asw2kwx4wx#O2k#O#P7h#P#o2k#o#p7n#p~2k!n3hX%hS%w!bOr2krs4Tsw2kwx4wx#O2k#O#P7h#P#o2k#o#p7n#p~2k!n4[X%hS%w!bOr2krs.ksw2kwx4wx#O2k#O#P7h#P#o2k#o#p7n#p~2k!n4|X%kWOr2krs3asw2kwx5ix#O2k#O#P7h#P#o2k#o#p7n#p~2k!n5nX%kWOr2krs3asw2kwx6Zx#O2k#O#P7h#P#o2k#o#p7n#p~2kW6`T%kWOr6Zrs6os#O6Z#O#P7b#P~6ZW6rTOr6Zrs7Rs#O6Z#O#P7b#P~6ZW7USOr6Zs#O6Z#O#P7b#P~6ZW7ePO~6Z!n7kPO~2k!n7uX%hS%kWOr8brs9Osw8bwx:Ux#O8b#O#P;[#P#o8b#o#p2k#p~8b[8iV%hS%kWOr8brs9Osw8bwx:Ux#O8b#O#P;[#P~8b[9TV%hSOr8brs9jsw8bwx:Ux#O8b#O#P;[#P~8b[9oV%hSOr8brs0xsw8bwx:Ux#O8b#O#P;[#P~8b[:ZV%kWOr8brs9Osw8bwx:px#O8b#O#P;[#P~8b[:uV%kWOr8brs9Osw8bwx6Zx#O8b#O#P;[#P~8b[;_PO~8b8z;iZ%q7[%kWOr(yrs)wsw(ywx<[x#O(y#O#P2V#P#o(y#o#p7n#p#q(y#q#r2k#r~(y7d<cX%q7[%kWOr<[rs=Os#O<[#O#P>b#P#o<[#o#p6Z#p#q<[#q#r6Z#r~<[7d=TX%q7[Or<[rs=ps#O<[#O#P>b#P#o<[#o#p6Z#p#q<[#q#r6Z#r~<[7d=uX%q7[Or<[rs-ws#O<[#O#P>b#P#o<[#o#p6Z#p#q<[#q#r6Z#r~<[7d>gT%q7[O#o<[#o#p6Z#p#q<[#q#r6Z#r~<[9[>{T%q7[O#o'P#o#p?[#p#q'P#q#r?[#r~'P#O?gX%hS%kW%n`%w!bOr?[rs@Ssw?[wx4wx#O?[#O#PCO#P#o?[#o#pCU#p~?[#O@]X%hS%n`%w!bOr?[rs@xsw?[wx4wx#O?[#O#PCO#P#o?[#o#pCU#p~?[#OARX%hS%n`%w!bOr?[rsAnsw?[wx4wx#O?[#O#PCO#P#o?[#o#pCU#p~?[!vAwV%hS%n`%w!bOwAnwx/Xx#OAn#O#PB^#P#oAn#o#pBd#p~An!vBaPO~An!vBiV%hSOw0xwx1^x#O0x#O#P2P#P#o0x#o#pAn#p~0x#OCRPO~?[#OC]X%hS%kWOr8brs9Osw8bwx:Ux#O8b#O#P;[#P#o8b#o#p?[#p~8b9[DTZ%q7[%hS%n`%w!bOr'PrsDvsw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'P9SERX%q7[%hS%n`%w!bOwDvwx,ex#ODv#O#PEn#P#oDv#o#pBd#p#qDv#q#rAn#r~Dv9SEsT%q7[O#oDv#o#pAn#p#qDv#q#rAn#r~Dv<bF_Z%q7[%kW%tp%y#tOrGQrs)wswGQwxM^x#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQ<bGaZ%q7[%hS%kW%tp%w!b%y#tOrGQrs)wswGQwxFSx#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQ<bHXT%q7[O#oGQ#o#pHh#p#qGQ#q#rHh#r~GQ&UHuX%hS%kW%tp%w!b%y#tOrHhrs3aswHhwxIbx#OHh#O#PLd#P#oHh#o#pLj#p~Hh&UIkX%kW%tp%y#tOrHhrs3aswHhwxJWx#OHh#O#PLd#P#oHh#o#pLj#p~Hh&UJaX%kW%tp%y#tOrHhrs3aswHhwxJ|x#OHh#O#PLd#P#oHh#o#pLj#p~Hh$nKVX%kW%tp%y#tOrJ|rs6oswJ|wxJ|x#OJ|#O#PKr#P#oJ|#o#pKx#p~J|$nKuPO~J|$nK}V%kWOr6Zrs6os#O6Z#O#P7b#P#o6Z#o#pJ|#p~6Z&ULgPO~Hh&ULqX%hS%kWOr8brs9Osw8bwx:Ux#O8b#O#P;[#P#o8b#o#pHh#p~8b<bMiZ%q7[%kW%tp%y#tOrGQrs)wswGQwxN[x#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQ:zNgZ%q7[%kW%tp%y#tOrN[rs=OswN[wxN[x#ON[#O#P! Y#P#oN[#o#pKx#p#qN[#q#rJ|#r~N[:z! _T%q7[O#oN[#o#pJ|#p#qN[#q#rJ|#r~N[<r! sT%q7[O#o$}#o#p!!S#p#q$}#q#r!!S#r~$}&f!!cX%hS%kW%n`%tp%w!b%y#tOr!!Srs@Ssw!!SwxIbx#O!!S#O#P!#O#P#o!!S#o#p!#U#p~!!S&f!#RPO~!!S&f!#]X%hS%kWOr8brs9Osw8bwx:Ux#O8b#O#P;[#P#o8b#o#p!!S#p~8bMg!$]a%q7[%hS%kW$o1s%n`%tp%w!b%y#tOX$}XY!#xY[$}[]!#x]p$}pq!#xqr$}rs&Rsw$}wxFSx#O$}#O#P!%b#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Mg!%gX%q7[OY$}YZ!#xZ]$}]^!#x^#o$}#o#p!!S#p#q$}#q#r!!S#r~$}<u!&eb%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`!'m!`#O$}#O#P! n#P#T$}#T#U!(s#U#f$}#f#g!(s#g#h!(s#h#o$}#o#p!#U#p#q$}#q#r!!S#r~$}<u!(QZjR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}<u!)WZ!jR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{!*Y_%up%q7[%hS%f,X%n`%w!bOY!+XYZ'PZ]!+X]^'P^r!+Xrs!BPsw!+Xwx!-gx#O!+X#O#P!>e#P#o!+X#o#p!@}#p#q!+X#q#r!>y#r~!+XDe!+h_%q7[%hS%kW%f,X%n`%w!bOY!+XYZ'PZ]!+X]^'P^r!+Xrs!,gsw!+Xwx!-gx#O!+X#O#P!>e#P#o!+X#o#p!@}#p#q!+X#q#r!>y#r~!+XDe!,tZ%q7[%hS%f,X%n`%w!bOr'PrsCxsw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'PDT!-p_%q7[%kW%f,XOY!.oYZ(yZ]!.o]^(y^r!.ors!/{sw!.owx!;Rx#O!.o#O#P!0y#P#o!.o#o#p!6m#p#q!.o#q#r!1_#r~!.oDT!.|_%q7[%hS%kW%f,X%w!bOY!.oYZ(yZ]!.o]^(y^r!.ors!/{sw!.owx!-gx#O!.o#O#P!0y#P#o!.o#o#p!6m#p#q!.o#q#r!1_#r~!.oDT!0WZ%q7[%hS%f,X%w!bOr(yrs*ssw(ywx(Px#O(y#O#P2V#P#o(y#o#p7n#p#q(y#q#r2k#r~(yDT!1OT%q7[O#o!.o#o#p!1_#p#q!.o#q#r!1_#r~!.o-w!1j]%hS%kW%f,X%w!bOY!1_YZ2kZ]!1_]^2k^r!1_rs!2csw!1_wx!3Xx#O!1_#O#P!6g#P#o!1_#o#p!6m#p~!1_-w!2lX%hS%f,X%w!bOr2krs4Tsw2kwx4wx#O2k#O#P7h#P#o2k#o#p7n#p~2k-w!3`]%kW%f,XOY!1_YZ2kZ]!1_]^2k^r!1_rs!2csw!1_wx!4Xx#O!1_#O#P!6g#P#o!1_#o#p!6m#p~!1_-w!4`]%kW%f,XOY!1_YZ2kZ]!1_]^2k^r!1_rs!2csw!1_wx!5Xx#O!1_#O#P!6g#P#o!1_#o#p!6m#p~!1_,a!5`X%kW%f,XOY!5XYZ6ZZ]!5X]^6Z^r!5Xrs!5{s#O!5X#O#P!6a#P~!5X,a!6QT%f,XOr6Zrs7Rs#O6Z#O#P7b#P~6Z,a!6dPO~!5X-w!6jPO~!1_-w!6v]%hS%kW%f,XOY!7oYZ8bZ]!7o]^8b^r!7ors!8ksw!7owx!9Xx#O!7o#O#P!:{#P#o!7o#o#p!1_#p~!7o,e!7xZ%hS%kW%f,XOY!7oYZ8bZ]!7o]^8b^r!7ors!8ksw!7owx!9Xx#O!7o#O#P!:{#P~!7o,e!8rV%hS%f,XOr8brs9jsw8bwx:Ux#O8b#O#P;[#P~8b,e!9`Z%kW%f,XOY!7oYZ8bZ]!7o]^8b^r!7ors!8ksw!7owx!:Rx#O!7o#O#P!:{#P~!7o,e!:YZ%kW%f,XOY!7oYZ8bZ]!7o]^8b^r!7ors!8ksw!7owx!5Xx#O!7o#O#P!:{#P~!7o,e!;OPO~!7oDT!;[_%q7[%kW%f,XOY!.oYZ(yZ]!.o]^(y^r!.ors!/{sw!.owx!<Zx#O!.o#O#P!0y#P#o!.o#o#p!6m#p#q!.o#q#r!1_#r~!.oBm!<d]%q7[%kW%f,XOY!<ZYZ<[Z]!<Z]^<[^r!<Zrs!=]s#O!<Z#O#P!>P#P#o!<Z#o#p!5X#p#q!<Z#q#r!5X#r~!<ZBm!=dX%q7[%f,XOr<[rs=ps#O<[#O#P>b#P#o<[#o#p6Z#p#q<[#q#r6Z#r~<[Bm!>UT%q7[O#o!<Z#o#p!5X#p#q!<Z#q#r!5X#r~!<ZDe!>jT%q7[O#o!+X#o#p!>y#p#q!+X#q#r!>y#r~!+X.X!?W]%hS%kW%f,X%n`%w!bOY!>yYZ?[Z]!>y]^?[^r!>yrs!@Psw!>ywx!3Xx#O!>y#O#P!@w#P#o!>y#o#p!@}#p~!>y.X!@[X%hS%f,X%n`%w!bOr?[rs@xsw?[wx4wx#O?[#O#PCO#P#o?[#o#pCU#p~?[.X!@zPO~!>y.X!AW]%hS%kW%f,XOY!7oYZ8bZ]!7o]^8b^r!7ors!8ksw!7owx!9Xx#O!7o#O#P!:{#P#o!7o#o#p!>y#p~!7oGZ!B^Z%q7[%hS%f,X%n`%w!bOr'Prs!CPsw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'PGZ!C`X%l#|%q7[%hS%j,X%n`%w!bOwDvwx,ex#ODv#O#PEn#P#oDv#o#pBd#p#qDv#q#rAn#r~DvMg!D`_Q1s%q7[%hS%kW%n`%tp%w!b%y#tOY!C{YZ$}Z]!C{]^$}^r!C{rs!E_sw!C{wx#Hqx#O!C{#O#P$(i#P#o!C{#o#p$*{#p#q!C{#q#r$)]#r~!C{JP!El_Q1s%q7[%hS%n`%w!bOY!FkYZ'PZ]!Fk]^'P^r!Fkrs#Eksw!Fkwx!Gyx#O!Fk#O#P#=u#P#o!Fk#o#p#Di#p#q!Fk#q#r#>i#r~!FkJP!Fz_Q1s%q7[%hS%kW%n`%w!bOY!FkYZ'PZ]!Fk]^'P^r!Fkrs!E_sw!Fkwx!Gyx#O!Fk#O#P#=u#P#o!Fk#o#p#Di#p#q!Fk#q#r#>i#r~!FkIo!HS_Q1s%q7[%kWOY!IRYZ(yZ]!IR]^(y^r!IRrs!J_sw!IRwx#8wx#O!IR#O#P#*R#P#o!IR#o#p#2}#p#q!IR#q#r#*u#r~!IRIo!I`_Q1s%q7[%hS%kW%w!bOY!IRYZ(yZ]!IR]^(y^r!IRrs!J_sw!IRwx!Gyx#O!IR#O#P#*R#P#o!IR#o#p#2}#p#q!IR#q#r#*u#r~!IRIo!Jj_Q1s%q7[%hS%w!bOY!IRYZ(yZ]!IR]^(y^r!IRrs!Kisw!IRwx!Gyx#O!IR#O#P#*R#P#o!IR#o#p#2}#p#q!IR#q#r#*u#r~!IRIo!Kt_Q1s%q7[%hS%w!bOY!IRYZ(yZ]!IR]^(y^r!IRrs!Lssw!IRwx!Gyx#O!IR#O#P#*R#P#o!IR#o#p#2}#p#q!IR#q#r#*u#r~!IRIg!MO]Q1s%q7[%hS%w!bOY!LsYZ+oZ]!Ls]^+o^w!Lswx!Mwx#O!Ls#O#P#!y#P#o!Ls#o#p#&m#p#q!Ls#q#r##m#r~!LsIg!NO]Q1s%q7[OY!LsYZ+oZ]!Ls]^+o^w!Lswx!Nwx#O!Ls#O#P#!y#P#o!Ls#o#p#&m#p#q!Ls#q#r##m#r~!LsIg# O]Q1s%q7[OY!LsYZ+oZ]!Ls]^+o^w!Lswx# wx#O!Ls#O#P#!y#P#o!Ls#o#p#&m#p#q!Ls#q#r##m#r~!LsHP#!OXQ1s%q7[OY# wYZ-wZ]# w]^-w^#o# w#o#p#!k#p#q# w#q#r#!k#r~# w1s#!pRQ1sOY#!kZ]#!k^~#!kIg##QXQ1s%q7[OY!LsYZ+oZ]!Ls]^+o^#o!Ls#o#p##m#p#q!Ls#q#r##m#r~!Ls3Z##vZQ1s%hS%w!bOY##mYZ.kZ]##m]^.k^w##mwx#$ix#O##m#O#P#&X#P#o##m#o#p#&m#p~##m3Z#$nZQ1sOY##mYZ.kZ]##m]^.k^w##mwx#%ax#O##m#O#P#&X#P#o##m#o#p#&m#p~##m3Z#%fZQ1sOY##mYZ.kZ]##m]^.k^w##mwx#!kx#O##m#O#P#&X#P#o##m#o#p#&m#p~##m3Z#&^TQ1sOY##mYZ.kZ]##m]^.k^~##m3Z#&tZQ1s%hSOY#'gYZ0xZ]#'g]^0x^w#'gwx#(Zx#O#'g#O#P#)m#P#o#'g#o#p##m#p~#'g1w#'nXQ1s%hSOY#'gYZ0xZ]#'g]^0x^w#'gwx#(Zx#O#'g#O#P#)m#P~#'g1w#(`XQ1sOY#'gYZ0xZ]#'g]^0x^w#'gwx#({x#O#'g#O#P#)m#P~#'g1w#)QXQ1sOY#'gYZ0xZ]#'g]^0x^w#'gwx#!kx#O#'g#O#P#)m#P~#'g1w#)rTQ1sOY#'gYZ0xZ]#'g]^0x^~#'gIo#*YXQ1s%q7[OY!IRYZ(yZ]!IR]^(y^#o!IR#o#p#*u#p#q!IR#q#r#*u#r~!IR3c#+Q]Q1s%hS%kW%w!bOY#*uYZ2kZ]#*u]^2k^r#*urs#+ysw#*uwx#-}x#O#*u#O#P#2i#P#o#*u#o#p#2}#p~#*u3c#,S]Q1s%hS%w!bOY#*uYZ2kZ]#*u]^2k^r#*urs#,{sw#*uwx#-}x#O#*u#O#P#2i#P#o#*u#o#p#2}#p~#*u3c#-U]Q1s%hS%w!bOY#*uYZ2kZ]#*u]^2k^r#*urs##msw#*uwx#-}x#O#*u#O#P#2i#P#o#*u#o#p#2}#p~#*u3c#.U]Q1s%kWOY#*uYZ2kZ]#*u]^2k^r#*urs#+ysw#*uwx#.}x#O#*u#O#P#2i#P#o#*u#o#p#2}#p~#*u3c#/U]Q1s%kWOY#*uYZ2kZ]#*u]^2k^r#*urs#+ysw#*uwx#/}x#O#*u#O#P#2i#P#o#*u#o#p#2}#p~#*u1{#0UXQ1s%kWOY#/}YZ6ZZ]#/}]^6Z^r#/}rs#0qs#O#/}#O#P#2T#P~#/}1{#0vXQ1sOY#/}YZ6ZZ]#/}]^6Z^r#/}rs#1cs#O#/}#O#P#2T#P~#/}1{#1hXQ1sOY#/}YZ6ZZ]#/}]^6Z^r#/}rs#!ks#O#/}#O#P#2T#P~#/}1{#2YTQ1sOY#/}YZ6ZZ]#/}]^6Z^~#/}3c#2nTQ1sOY#*uYZ2kZ]#*u]^2k^~#*u3c#3W]Q1s%hS%kWOY#4PYZ8bZ]#4P]^8b^r#4Prs#4{sw#4Pwx#6ox#O#4P#O#P#8c#P#o#4P#o#p#*u#p~#4P2P#4YZQ1s%hS%kWOY#4PYZ8bZ]#4P]^8b^r#4Prs#4{sw#4Pwx#6ox#O#4P#O#P#8c#P~#4P2P#5SZQ1s%hSOY#4PYZ8bZ]#4P]^8b^r#4Prs#5usw#4Pwx#6ox#O#4P#O#P#8c#P~#4P2P#5|ZQ1s%hSOY#4PYZ8bZ]#4P]^8b^r#4Prs#'gsw#4Pwx#6ox#O#4P#O#P#8c#P~#4P2P#6vZQ1s%kWOY#4PYZ8bZ]#4P]^8b^r#4Prs#4{sw#4Pwx#7ix#O#4P#O#P#8c#P~#4P2P#7pZQ1s%kWOY#4PYZ8bZ]#4P]^8b^r#4Prs#4{sw#4Pwx#/}x#O#4P#O#P#8c#P~#4P2P#8hTQ1sOY#4PYZ8bZ]#4P]^8b^~#4PIo#9Q_Q1s%q7[%kWOY!IRYZ(yZ]!IR]^(y^r!IRrs!J_sw!IRwx#:Px#O!IR#O#P#*R#P#o!IR#o#p#2}#p#q!IR#q#r#*u#r~!IRHX#:Y]Q1s%q7[%kWOY#:PYZ<[Z]#:P]^<[^r#:Prs#;Rs#O#:P#O#P#=R#P#o#:P#o#p#/}#p#q#:P#q#r#/}#r~#:PHX#;Y]Q1s%q7[OY#:PYZ<[Z]#:P]^<[^r#:Prs#<Rs#O#:P#O#P#=R#P#o#:P#o#p#/}#p#q#:P#q#r#/}#r~#:PHX#<Y]Q1s%q7[OY#:PYZ<[Z]#:P]^<[^r#:Prs# ws#O#:P#O#P#=R#P#o#:P#o#p#/}#p#q#:P#q#r#/}#r~#:PHX#=YXQ1s%q7[OY#:PYZ<[Z]#:P]^<[^#o#:P#o#p#/}#p#q#:P#q#r#/}#r~#:PJP#=|XQ1s%q7[OY!FkYZ'PZ]!Fk]^'P^#o!Fk#o#p#>i#p#q!Fk#q#r#>i#r~!Fk3s#>v]Q1s%hS%kW%n`%w!bOY#>iYZ?[Z]#>i]^?[^r#>irs#?osw#>iwx#-}x#O#>i#O#P#DT#P#o#>i#o#p#Di#p~#>i3s#?z]Q1s%hS%n`%w!bOY#>iYZ?[Z]#>i]^?[^r#>irs#@ssw#>iwx#-}x#O#>i#O#P#DT#P#o#>i#o#p#Di#p~#>i3s#AO]Q1s%hS%n`%w!bOY#>iYZ?[Z]#>i]^?[^r#>irs#Awsw#>iwx#-}x#O#>i#O#P#DT#P#o#>i#o#p#Di#p~#>i3k#BSZQ1s%hS%n`%w!bOY#AwYZAnZ]#Aw]^An^w#Awwx#$ix#O#Aw#O#P#Bu#P#o#Aw#o#p#CZ#p~#Aw3k#BzTQ1sOY#AwYZAnZ]#Aw]^An^~#Aw3k#CbZQ1s%hSOY#'gYZ0xZ]#'g]^0x^w#'gwx#(Zx#O#'g#O#P#)m#P#o#'g#o#p#Aw#p~#'g3s#DYTQ1sOY#>iYZ?[Z]#>i]^?[^~#>i3s#Dr]Q1s%hS%kWOY#4PYZ8bZ]#4P]^8b^r#4Prs#4{sw#4Pwx#6ox#O#4P#O#P#8c#P#o#4P#o#p#>i#p~#4PJP#Ex_Q1s%q7[%hS%n`%w!bOY!FkYZ'PZ]!Fk]^'P^r!Fkrs#Fwsw!Fkwx!Gyx#O!Fk#O#P#=u#P#o!Fk#o#p#Di#p#q!Fk#q#r#>i#r~!FkIw#GU]Q1s%q7[%hS%n`%w!bOY#FwYZDvZ]#Fw]^Dv^w#Fwwx!Mwx#O#Fw#O#P#G}#P#o#Fw#o#p#CZ#p#q#Fw#q#r#Aw#r~#FwIw#HUXQ1s%q7[OY#FwYZDvZ]#Fw]^Dv^#o#Fw#o#p#Aw#p#q#Fw#q#r#Aw#r~#FwMV#IO_Q1s%q7[%kW%tp%y#tOY#I}YZGQZ]#I}]^GQ^r#I}rs!J_sw#I}wx$%]x#O#I}#O#P#K_#P#o#I}#o#p$$Z#p#q#I}#q#r#LR#r~#I}MV#J`_Q1s%q7[%hS%kW%tp%w!b%y#tOY#I}YZGQZ]#I}]^GQ^r#I}rs!J_sw#I}wx#Hqx#O#I}#O#P#K_#P#o#I}#o#p$$Z#p#q#I}#q#r#LR#r~#I}MV#KfXQ1s%q7[OY#I}YZGQZ]#I}]^GQ^#o#I}#o#p#LR#p#q#I}#q#r#LR#r~#I}6y#Lb]Q1s%hS%kW%tp%w!b%y#tOY#LRYZHhZ]#LR]^Hh^r#LRrs#+ysw#LRwx#MZx#O#LR#O#P$#u#P#o#LR#o#p$$Z#p~#LR6y#Mf]Q1s%kW%tp%y#tOY#LRYZHhZ]#LR]^Hh^r#LRrs#+ysw#LRwx#N_x#O#LR#O#P$#u#P#o#LR#o#p$$Z#p~#LR6y#Nj]Q1s%kW%tp%y#tOY#LRYZHhZ]#LR]^Hh^r#LRrs#+ysw#LRwx$ cx#O#LR#O#P$#u#P#o#LR#o#p$$Z#p~#LR5c$ n]Q1s%kW%tp%y#tOY$ cYZJ|Z]$ c]^J|^r$ crs#0qsw$ cwx$ cx#O$ c#O#P$!g#P#o$ c#o#p$!{#p~$ c5c$!lTQ1sOY$ cYZJ|Z]$ c]^J|^~$ c5c$#SZQ1s%kWOY#/}YZ6ZZ]#/}]^6Z^r#/}rs#0qs#O#/}#O#P#2T#P#o#/}#o#p$ c#p~#/}6y$#zTQ1sOY#LRYZHhZ]#LR]^Hh^~#LR6y$$d]Q1s%hS%kWOY#4PYZ8bZ]#4P]^8b^r#4Prs#4{sw#4Pwx#6ox#O#4P#O#P#8c#P#o#4P#o#p#LR#p~#4PMV$%j_Q1s%q7[%kW%tp%y#tOY#I}YZGQZ]#I}]^GQ^r#I}rs!J_sw#I}wx$&ix#O#I}#O#P#K_#P#o#I}#o#p$$Z#p#q#I}#q#r#LR#r~#I}Ko$&v_Q1s%q7[%kW%tp%y#tOY$&iYZN[Z]$&i]^N[^r$&irs#;Rsw$&iwx$&ix#O$&i#O#P$'u#P#o$&i#o#p$!{#p#q$&i#q#r$ c#r~$&iKo$'|XQ1s%q7[OY$&iYZN[Z]$&i]^N[^#o$&i#o#p$ c#p#q$&i#q#r$ c#r~$&iMg$(pXQ1s%q7[OY!C{YZ$}Z]!C{]^$}^#o!C{#o#p$)]#p#q!C{#q#r$)]#r~!C{7Z$)n]Q1s%hS%kW%n`%tp%w!b%y#tOY$)]YZ!!SZ]$)]]^!!S^r$)]rs#?osw$)]wx#MZx#O$)]#O#P$*g#P#o$)]#o#p$*{#p~$)]7Z$*lTQ1sOY$)]YZ!!SZ]$)]]^!!S^~$)]7Z$+U]Q1s%hS%kWOY#4PYZ8bZ]#4P]^8b^r#4Prs#4{sw#4Pwx#6ox#O#4P#O#P#8c#P#o#4P#o#p$)]#p~#4PGz$,b]$}Q%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gz$-nZ!s,W%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gz$.t]$wQ%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{$/|_%r`%q7[%kW%f,X%tp%y#tOY$0{YZGQZ]$0{]^GQ^r$0{rs$2]sw$0{wx$Jex#O$0{#O#P$Fw#P#o$0{#o#p$Ic#p#q$0{#q#r$G]#r~$0{Gk$1^_%q7[%hS%kW%f,X%tp%w!b%y#tOY$0{YZGQZ]$0{]^GQ^r$0{rs$2]sw$0{wx$Ewx#O$0{#O#P$Fw#P#o$0{#o#p$Ic#p#q$0{#q#r$G]#r~$0{DT$2h_%q7[%hS%f,X%w!bOY$3gYZ(yZ]$3g]^(y^r$3grs$Basw$3gwx$4sx#O$3g#O#P$5o#P#o$3g#o#p$={#p#q$3g#q#r$6T#r~$3gDT$3t_%q7[%hS%kW%f,X%w!bOY$3gYZ(yZ]$3g]^(y^r$3grs$2]sw$3gwx$4sx#O$3g#O#P$5o#P#o$3g#o#p$={#p#q$3g#q#r$6T#r~$3gDT$4|Z%q7[%kW%f,XOr(yrs)wsw(ywx;bx#O(y#O#P2V#P#o(y#o#p7n#p#q(y#q#r2k#r~(yDT$5tT%q7[O#o$3g#o#p$6T#p#q$3g#q#r$6T#r~$3g-w$6`]%hS%kW%f,X%w!bOY$6TYZ2kZ]$6T]^2k^r$6Trs$7Xsw$6Twx$=Rx#O$6T#O#P$=u#P#o$6T#o#p$={#p~$6T-w$7b]%hS%f,X%w!bOY$6TYZ2kZ]$6T]^2k^r$6Trs$8Zsw$6Twx$=Rx#O$6T#O#P$=u#P#o$6T#o#p$={#p~$6T-w$8d]%hS%f,X%w!bOY$6TYZ2kZ]$6T]^2k^r$6Trs$9]sw$6Twx$=Rx#O$6T#O#P$=u#P#o$6T#o#p$={#p~$6T-o$9fZ%hS%f,X%w!bOY$9]YZ.kZ]$9]]^.k^w$9]wx$:Xx#O$9]#O#P$:s#P#o$9]#o#p$:y#p~$9]-o$:^V%f,XOw.kwx/qx#O.k#O#P0W#P#o.k#o#p0^#p~.k-o$:vPO~$9]-o$;QZ%hS%f,XOY$;sYZ0xZ]$;s]^0x^w$;swx$<gx#O$;s#O#P$<{#P#o$;s#o#p$9]#p~$;s,]$;zX%hS%f,XOY$;sYZ0xZ]$;s]^0x^w$;swx$<gx#O$;s#O#P$<{#P~$;s,]$<lT%f,XOw0xwx1px#O0x#O#P2P#P~0x,]$=OPO~$;s-w$=YX%kW%f,XOr2krs3asw2kwx5ix#O2k#O#P7h#P#o2k#o#p7n#p~2k-w$=xPO~$6T-w$>U]%hS%kW%f,XOY$>}YZ8bZ]$>}]^8b^r$>}rs$?ysw$>}wx$Amx#O$>}#O#P$BZ#P#o$>}#o#p$6T#p~$>},e$?WZ%hS%kW%f,XOY$>}YZ8bZ]$>}]^8b^r$>}rs$?ysw$>}wx$Amx#O$>}#O#P$BZ#P~$>},e$@QZ%hS%f,XOY$>}YZ8bZ]$>}]^8b^r$>}rs$@ssw$>}wx$Amx#O$>}#O#P$BZ#P~$>},e$@zZ%hS%f,XOY$>}YZ8bZ]$>}]^8b^r$>}rs$;ssw$>}wx$Amx#O$>}#O#P$BZ#P~$>},e$AtV%kW%f,XOr8brs9Osw8bwx:px#O8b#O#P;[#P~8b,e$B^PO~$>}DT$Bl_%q7[%hS%f,X%w!bOY$3gYZ(yZ]$3g]^(y^r$3grs$Cksw$3gwx$4sx#O$3g#O#P$5o#P#o$3g#o#p$={#p#q$3g#q#r$6T#r~$3gC{$Cv]%q7[%hS%f,X%w!bOY$CkYZ+oZ]$Ck]^+o^w$Ckwx$Dox#O$Ck#O#P$Ec#P#o$Ck#o#p$:y#p#q$Ck#q#r$9]#r~$CkC{$DvX%q7[%f,XOw+owx-Vx#O+o#O#P.V#P#o+o#o#p0^#p#q+o#q#r.k#r~+oC{$EhT%q7[O#o$Ck#o#p$9]#p#q$Ck#q#r$9]#r~$CkGk$FUZ%q7[%kW%f,X%tp%y#tOrGQrs)wswGQwxM^x#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQGk$F|T%q7[O#o$0{#o#p$G]#p#q$0{#q#r$G]#r~$0{1_$Gl]%hS%kW%f,X%tp%w!b%y#tOY$G]YZHhZ]$G]]^Hh^r$G]rs$7Xsw$G]wx$Hex#O$G]#O#P$I]#P#o$G]#o#p$Ic#p~$G]1_$HpX%kW%f,X%tp%y#tOrHhrs3aswHhwxJWx#OHh#O#PLd#P#oHh#o#pLj#p~Hh1_$I`PO~$G]1_$Il]%hS%kW%f,XOY$>}YZ8bZ]$>}]^8b^r$>}rs$?ysw$>}wx$Amx#O$>}#O#P$BZ#P#o$>}#o#p$G]#p~$>}Gk$JrZ%q7[%kW%f,X%tp%y#tOrGQrs)wswGQwx$Kex#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQGk$KtZ%i!f%q7[%kW%g,X%tp%y#tOrN[rs=OswN[wxN[x#ON[#O#P! Y#P#oN[#o#pKx#p#qN[#q#rJ|#r~N[G{$LzZf,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}<u$NQZ!OR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{% W_T,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSxz$}z{%!V{!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%!j]_R%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%#v]$z,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}<u%%SZwR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Mg%&Y^${,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`!a%'U!a#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}B^%'iZ&U&j%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%(o_!dQ%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!O$}!O!P%)n!P!Q$}!Q![%,O![#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%*P]%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!O$}!O!P%*x!P#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%+]Z!m,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%,cg!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q![%,O![!g$}!g!h%-z!h!l$}!l!m%2[!m#O$}#O#P! n#P#R$}#R#S%,O#S#X$}#X#Y%-z#Y#^$}#^#_%2[#_#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%.]a%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx{$}{|%/b|}$}}!O%/b!O!Q$}!Q![%0l![#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%/s]%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q![%0l![#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%1Pc!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q![%0l![!l$}!l!m%2[!m#O$}#O#P! n#P#R$}#R#S%0l#S#^$}#^#_%2[#_#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%2oZ!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%3u_$|R%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!P$}!P!Q%4t!Q!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gz%5X]%OQ%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%6eu!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!O$}!O!P%8x!P!Q$}!Q![%:S![!d$}!d!e%<U!e!g$}!g!h%-z!h!l$}!l!m%2[!m!q$}!q!r%?O!r!z$}!z!{%Ar!{#O$}#O#P! n#P#R$}#R#S%:S#S#U$}#U#V%<U#V#X$}#X#Y%-z#Y#^$}#^#_%2[#_#c$}#c#d%?O#d#l$}#l#m%Ar#m#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%9Z]%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q![%,O![#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%:gi!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!O$}!O!P%8x!P!Q$}!Q![%:S![!g$}!g!h%-z!h!l$}!l!m%2[!m#O$}#O#P! n#P#R$}#R#S%:S#S#X$}#X#Y%-z#Y#^$}#^#_%2[#_#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%<g`%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q!R%=i!R!S%=i!S#O$}#O#P! n#P#R$}#R#S%=i#S#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%=|`!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q!R%=i!R!S%=i!S#O$}#O#P! n#P#R$}#R#S%=i#S#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%?a_%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q!Y%@`!Y#O$}#O#P! n#P#R$}#R#S%@`#S#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%@s_!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q!Y%@`!Y#O$}#O#P! n#P#R$}#R#S%@`#S#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%BTc%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q![%C`![!c$}!c!i%C`!i#O$}#O#P! n#P#R$}#R#S%C`#S#T$}#T#Z%C`#Z#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy%Csc!f,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!Q$}!Q![%C`![!c$}!c!i%C`!i#O$}#O#P! n#P#R$}#R#S%C`#S#T$}#T#Z%C`#Z#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Mg%Ec]x1s%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`%F[!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}<u%FoZ%WR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%GuZ#^,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%H{_jR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!^$}!^!_%Iz!_!`!'m!`!a!'m!a#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gz%J_]$xQ%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%Kk]%V,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`!'m!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{%Lw^jR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`!'m!`!a%Ms!a#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gz%NW]$yQ%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}G{& f]]Q#tP%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Mg&!tc%q7[%hS%kW%e&j%n`%tp%w!b%y#t%Q,XOr$}rs&Rsw$}wxFSx!Q$}!Q![&!_![!c$}!c!}&!_!}#O$}#O#P! n#P#R$}#R#S&!_#S#T$}#T#o&!_#o#p!#U#p#q$}#q#r!!S#r$g$}$g~&!_Mg&$fg%q7[%hS%kW%e&j%n`%tp%w!b%y#t%Q,XOr$}rs&%}sw$}wx&)Tx!Q$}!Q![&!_![!c$}!c!t&!_!t!u&,a!u!}&!_!}#O$}#O#P! n#P#R$}#R#S&!_#S#T$}#T#f&!_#f#g&,a#g#o&!_#o#p!#U#p#q$}#q#r!!S#r$g$}$g~&!_De&&[_%q7[%hS%f,X%n`%w!bOY!+XYZ'PZ]!+X]^'P^r!+Xrs&'Zsw!+Xwx!-gx#O!+X#O#P!>e#P#o!+X#o#p!@}#p#q!+X#q#r!>y#r~!+XDe&'hZ%q7[%hS%f,X%n`%w!bOr'Prs&(Zsw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'PD]&(hX%q7[%hS%j,X%n`%w!bOwDvwx,ex#ODv#O#PEn#P#oDv#o#pBd#p#qDv#q#rAn#r~DvGk&)b_%q7[%kW%f,X%tp%y#tOY$0{YZGQZ]$0{]^GQ^r$0{rs$2]sw$0{wx&*ax#O$0{#O#P$Fw#P#o$0{#o#p$Ic#p#q$0{#q#r$G]#r~$0{Gk&*nZ%q7[%kW%f,X%tp%y#tOrGQrs)wswGQwx&+ax#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQFT&+nZ%q7[%kW%g,X%tp%y#tOrN[rs=OswN[wxN[x#ON[#O#P! Y#P#oN[#o#pKx#p#qN[#q#rJ|#r~N[Mg&,vc%q7[%hS%kW%e&j%n`%tp%w!b%y#t%Q,XOr$}rs&%}sw$}wx&)Tx!Q$}!Q![&!_![!c$}!c!}&!_!}#O$}#O#P! n#P#R$}#R#S&!_#S#T$}#T#o&!_#o#p!#U#p#q$}#q#r!!S#r$g$}$g~&!_Mg&.hg%q7[%hS%kW%e&j%n`%tp%w!b%y#t%Q,XOr$}rs&0Psw$}wx&2wx!Q$}!Q![&!_![!c$}!c!t&!_!t!u&5u!u!}&!_!}#O$}#O#P! n#P#R$}#R#S&!_#S#T$}#T#f&!_#f#g&5u#g#o&!_#o#p!#U#p#q$}#q#r!!S#r$g$}$g~&!_De&0^Z%q7[%hS%n`%w!b%s,XOr'Prs&1Psw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'PDe&1[Z%q7[%hS%n`%w!bOr'Prs&1}sw'Pwx(Px#O'P#O#P>v#P#o'P#o#pCU#p#q'P#q#r?[#r~'PD]&2[X%q7[%hS%x,X%n`%w!bOwDvwx,ex#ODv#O#PEn#P#oDv#o#pBd#p#qDv#q#rAn#r~DvGk&3UZ%q7[%kW%tp%y#t%m,XOrGQrs)wswGQwx&3wx#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQGk&4SZ%q7[%kW%tp%y#tOrGQrs)wswGQwx&4ux#OGQ#O#PHS#P#oGQ#o#pLj#p#qGQ#q#rHh#r~GQFT&5SZ%q7[%kW%v,X%tp%y#tOrN[rs=OswN[wxN[x#ON[#O#P! Y#P#oN[#o#pKx#p#qN[#q#rJ|#r~N[Mg&6[c%q7[%hS%kW%e&j%n`%tp%w!b%y#t%Q,XOr$}rs&0Psw$}wx&2wx!Q$}!Q![&!_![!c$}!c!}&!_!}#O$}#O#P! n#P#R$}#R#S&!_#S#T$}#T#o&!_#o#p!#U#p#q$}#q#r!!S#r$g$}$g~&!_Mg&7|k%q7[%hS%kW%e&j%n`%tp%w!b%y#t%Q,XOr$}rs&%}sw$}wx&)Tx!Q$}!Q![&!_![!c$}!c!h&!_!h!i&5u!i!t&!_!t!u&,a!u!}&!_!}#O$}#O#P! n#P#R$}#R#S&!_#S#T$}#T#U&!_#U#V&,a#V#Y&!_#Y#Z&5u#Z#o&!_#o#p!#U#p#q$}#q#r!!S#r$g$}$g~&!_G{&:UZ!V,X%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}<u&;[Z!WR%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gz&<b]$vQ%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}Gy&=dX%hS%kW!ZGmOr8brs9Osw8bwx:Ux#O8b#O#P;[#P#o8b#o#p!!S#p~8bGz&>d]$uQ%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx!_$}!_!`$-Z!`#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}<u&?nX![7_%hS%kW%n`%tp%w!b%y#tOr!!Srs@Ssw!!SwxIbx#O!!S#O#P!#O#P#o!!S#o#p!#U#p~!!SGy&@nZ%P,V%q7[%hS%kW%n`%tp%w!b%y#tOr$}rs&Rsw$}wxFSx#O$}#O#P! n#P#o$}#o#p!#U#p#q$}#q#r!!S#r~$}",tokenizers:[a6,o6,0,1,2,3,4,5,6,7,8,9,10,i6],topRules:{Script:[0,3]},specialized:[{term:186,get:t=>c6[t]||-1}],tokenPrec:6589});let f6=0;class sr{constructor(e,n,r){this.set=e,this.base=n,this.modified=r,this.id=f6++}static define(e){if(e==null?void 0:e.base)throw new Error("Can not derive from a modified tag");let n=new sr([],null,[]);if(n.set.push(n),e)for(let r of e.set)n.set.push(r);return n}static defineModifier(){let e=new Ff;return n=>n.modified.indexOf(e)>-1?n:Ff.get(n.base||n,n.modified.concat(e).sort((r,i)=>r.id-i.id))}}let h6=0;class Ff{constructor(){this.instances=[],this.id=h6++}static get(e,n){if(!n.length)return e;let r=n[0].instances.find(l=>l.base==e&&d6(n,l.modified));if(r)return r;let i=[],o=new sr(i,e,n);for(let l of n)l.instances.push(o);let s=Wk(n);for(let l of e.set)for(let a of s)i.push(Ff.get(l,a));return o}}function d6(t,e){return t.length==e.length&&t.every((n,r)=>n==e[r])}function Wk(t){let e=[t];for(let n=0;n<t.length;n++)for(let r of Wk(t.slice(0,n).concat(t.slice(n+1))))e.push(r);return e}function p6(t){let e=Object.create(null);for(let n in t){let r=t[n];Array.isArray(r)||(r=[r]);for(let i of n.split(" "))if(i){let o=[],s=2,l=i;for(let f=0;;){if(l=="..."&&f>0&&f+3==i.length){s=1;break}let h=/^"(?:[^"\\]|\\.)*?"|[^\/!]+/.exec(l);if(!h)throw new RangeError("Invalid path: "+i);if(o.push(h[0]=="*"?null:h[0][0]=='"'?JSON.parse(h[0]):h[0]),f+=h[0].length,f==i.length)break;let d=i[f++];if(f==i.length&&d=="!"){s=0;break}if(d!="/")throw new RangeError("Invalid path: "+i);l=i.slice(f)}let a=o.length-1,c=o[a];if(!c)throw new RangeError("Invalid path: "+i);let u=new m6(r,s,a>0?o.slice(0,a):null);e[c]=u.sort(e[c])}}return Qk.add(e)}const Qk=new xe,r0=ne.define({combine(t){return t.length?La.combinedMatch(t):null}}),Nk=ne.define({combine(t){return t.length?t[0].match:null}});function i0(t){return t.facet(r0)||t.facet(Nk)}class m6{constructor(e,n,r,i){this.tags=e,this.mode=n,this.context=r,this.next=i}sort(e){return!e||e.depth<this.depth?(this.next=e,this):(e.next=this.sort(e.next),e)}get depth(){return this.context?this.context.length:0}}class La{constructor(e,n){this.map=Object.create(null);let r;function i(s){let l=gi.newName();return(r||(r=Object.create(null)))["."+l]=s,l}this.all=typeof n.all=="string"?n.all:n.all?i(n.all):null;for(let s of e){let l=(s.class||i(Object.assign({},s,{tag:null})))+(this.all?" "+this.all:""),a=s.tag;if(!Array.isArray(a))this.map[a.id]=l;else for(let c of a)this.map[c.id]=l}this.module=r?new gi(r):null,this.scope=n.scope||null,this.match=this.match.bind(this);let o=[y6];this.module&&o.push(oe.styleModule.of(this.module)),this.extension=o.concat(r0.of(this)),this.fallback=o.concat(Nk.of(this))}match(e,n){if(this.scope&&n!=this.scope)return null;for(let r of e.set){let i=this.map[r.id];if(i!==void 0)return r!=e&&(this.map[e.id]=i),i}return this.map[e.id]=this.all}static combinedMatch(e){if(e.length==1)return e[0].match;let n=e.some(r=>r.scope)?void 0:Object.create(null);return(r,i)=>{let o=n&&n[r.id];if(o!==void 0)return o;let s=null;for(let l of e){let a=l.match(r,i);a&&(s=s?s+" "+a:a)}return n&&(n[r.id]=s),s}}static define(e,n){return new La(e,n||{})}static get(e,n,r){let i=i0(e);return i&&i(n,r||ln.none)}}function lF(t,e,n,r=0,i=t.length){Bk(t,r,i,e,n)}class g6{constructor(e){this.markCache=Object.create(null),this.tree=an(e.state),this.decorations=this.buildDeco(e,i0(e.state))}update(e){let n=an(e.state),r=i0(e.state),i=r!=e.startState.facet(r0);n.length<e.view.viewport.to&&!i&&n.type==this.tree.type?this.decorations=this.decorations.map(e.changes):(n!=this.tree||e.viewportChanged||i)&&(this.tree=n,this.decorations=this.buildDeco(e.view,r))}buildDeco(e,n){if(!n||!this.tree.length)return te.none;let r=new yi;for(let{from:i,to:o}of e.visibleRanges)Bk(this.tree,i,o,n,(s,l,a)=>{r.add(s,l,this.markCache[a]||(this.markCache[a]=te.mark({class:a})))});return r.finish()}}const y6=da.high(rt.fromClass(g6,{decorations:t=>t.decorations})),_k=[""];class O6{constructor(e,n,r){this.at=e,this.style=n,this.span=r,this.class=""}startSpan(e,n){n!=this.class&&(this.flush(e),e>this.at&&(this.at=e),this.class=n)}flush(e){e>this.at&&this.class&&this.span(this.at,e,this.class)}highlightRange(e,n,r,i,o,s){let{type:l,from:a,to:c}=e;if(a>=r||c<=n)return;_k[o]=l.name,l.isTop&&(s=l);let u=i,f=l.prop(Qk),h=!1;for(;f;){if(!f.context||v6(f.context,_k,o)){for(let p of f.tags){let m=this.style(p,s);m&&(u&&(u+=" "),u+=m,f.mode==1?i+=(i?" ":"")+m:f.mode==0&&(h=!0))}break}f=f.next}if(this.startSpan(e.from,u),h)return;let d=e.tree&&e.tree.prop(xe.mounted);if(d&&d.overlay){let p=e.node.enter(d.overlay[0].from+a,1),m=e.firstChild();for(let g=0,y=a;;g++){let O=g<d.overlay.length?d.overlay[g]:null,v=O?O.from+a:c,x=Math.max(n,y),b=Math.min(r,v);if(x<b&&m)for(;e.from<b&&(this.highlightRange(e,x,b,i,o+1,s),this.startSpan(Math.min(r,e.to),u),!(e.to>=v||!e.nextSibling())););if(!O||v>r)break;y=O.to+a,y>n&&(this.highlightRange(p.cursor,Math.max(n,O.from+a),Math.min(r,y),i,o,d.tree.type),this.startSpan(y,u))}m&&e.parent()}else if(e.firstChild()){do if(!(e.to<=n)){if(e.from>=r)break;this.highlightRange(e,n,r,i,o+1,s),this.startSpan(Math.min(r,e.to),u)}while(e.nextSibling());e.parent()}}}function Bk(t,e,n,r,i){let o=new O6(e,r,i);o.highlightRange(t.cursor(),e,n,"",0,t.type),o.flush(n)}function v6(t,e,n){if(t.length>n-1)return!1;for(let r=n-1,i=t.length-1;i>=0;i--,r--){let o=t[i];if(o&&o!=e[r])return!1}return!0}const U=sr.define,Vf=U(),$i=U(),qk=U($i),zk=U($i),Si=U(),Uf=U(Si),o0=U(Si),lr=U(),so=U(lr),ar=U(),cr=U(),s0=U(),Wa=U(s0),jf=U(),A={comment:Vf,lineComment:U(Vf),blockComment:U(Vf),docComment:U(Vf),name:$i,variableName:U($i),typeName:qk,tagName:U(qk),propertyName:zk,attributeName:U(zk),className:U($i),labelName:U($i),namespace:U($i),macroName:U($i),literal:Si,string:Uf,docString:U(Uf),character:U(Uf),attributeValue:U(Uf),number:o0,integer:U(o0),float:U(o0),bool:U(Si),regexp:U(Si),escape:U(Si),color:U(Si),url:U(Si),keyword:ar,self:U(ar),null:U(ar),atom:U(ar),unit:U(ar),modifier:U(ar),operatorKeyword:U(ar),controlKeyword:U(ar),definitionKeyword:U(ar),moduleKeyword:U(ar),operator:cr,derefOperator:U(cr),arithmeticOperator:U(cr),logicOperator:U(cr),bitwiseOperator:U(cr),compareOperator:U(cr),updateOperator:U(cr),definitionOperator:U(cr),typeOperator:U(cr),controlOperator:U(cr),punctuation:s0,separator:U(s0),bracket:Wa,angleBracket:U(Wa),squareBracket:U(Wa),paren:U(Wa),brace:U(Wa),content:lr,heading:so,heading1:U(so),heading2:U(so),heading3:U(so),heading4:U(so),heading5:U(so),heading6:U(so),contentSeparator:U(lr),list:U(lr),quote:U(lr),emphasis:U(lr),strong:U(lr),link:U(lr),monospace:U(lr),strikethrough:U(lr),inserted:U(),deleted:U(),changed:U(),invalid:U(),meta:jf,documentMeta:U(jf),annotation:U(jf),processingInstruction:U(jf),definition:sr.defineModifier(),constant:sr.defineModifier(),function:sr.defineModifier(),standard:sr.defineModifier(),local:sr.defineModifier(),special:sr.defineModifier()},x6=La.define([{tag:A.link,textDecoration:"underline"},{tag:A.heading,textDecoration:"underline",fontWeight:"bold"},{tag:A.emphasis,fontStyle:"italic"},{tag:A.strong,fontWeight:"bold"},{tag:A.strikethrough,textDecoration:"line-through"},{tag:A.keyword,color:"#708"},{tag:[A.atom,A.bool,A.url,A.contentSeparator,A.labelName],color:"#219"},{tag:[A.literal,A.inserted],color:"#164"},{tag:[A.string,A.deleted],color:"#a11"},{tag:[A.regexp,A.escape,A.special(A.string)],color:"#e40"},{tag:A.definition(A.variableName),color:"#00f"},{tag:A.local(A.variableName),color:"#30a"},{tag:[A.typeName,A.namespace],color:"#085"},{tag:A.className,color:"#167"},{tag:[A.special(A.variableName),A.macroName],color:"#256"},{tag:A.definition(A.propertyName),color:"#00c"},{tag:A.comment,color:"#940"},{tag:A.meta,color:"#7a757a"},{tag:A.invalid,color:"#f00"}]);function Fk(t,e){let n=t.lineIndent(e.from),r=t.lineAt(t.pos,-1),i=r.from+r.text.length;return!/\S/.test(r.text)&&t.node.to<i+100&&!/\S/.test(t.state.sliceDoc(i,t.node.to))&&t.lineIndent(t.pos,-1)<=n||/^\s*(else:|elif |except |finally:)/.test(t.textAfter)&&t.lineIndent(t.pos,-1)>n?null:n+t.unit}const w6=yf.define({parser:u6.configure({props:[K$.add({Body:t=>{var e;return(e=Fk(t,t.node))!==null&&e!==void 0?e:t.continue()},IfStatement:t=>/^\s*(else:|elif )/.test(t.textAfter)?t.baseIndent:t.continue(),TryStatement:t=>/^\s*(except |finally:)/.test(t.textAfter)?t.baseIndent:t.continue(),"TupleExpression ComprehensionExpression ParamList ArgList ParenthesizedExpression":Eg({closing:")"}),"DictionaryExpression DictionaryComprehensionExpression SetExpression SetComprehensionExpression":Eg({closing:"}"}),"ArrayExpression ArrayComprehensionExpression":Eg({closing:"]"}),Script:t=>{if(t.pos+/\s*/.exec(t.textAfter)[0].length>=t.node.to){let e=null;for(let n=t.node,r=n.to;n=n.lastChild,!(!n||n.to!=r);)n.type.name=="Body"&&(e=n);if(e){let n=Fk(t,e);if(n!=null)return n}}return t.continue()}}),tS.add({"Body ArrayExpression DictionaryExpression":aB}),p6({"async '*' '**' FormatConversion":A.modifier,"for while if elif else try except finally return raise break continue with pass assert await yield":A.controlKeyword,"in not and or is del":A.operatorKeyword,"import from def class global nonlocal lambda":A.definitionKeyword,"with as print":A.keyword,self:A.self,Boolean:A.bool,None:A.null,VariableName:A.variableName,"CallExpression/VariableName":A.function(A.variableName),"FunctionDefinition/VariableName":A.function(A.definition(A.variableName)),"ClassDefinition/VariableName":A.definition(A.className),PropertyName:A.propertyName,"CallExpression/MemberExpression/PropertyName":A.function(A.propertyName),Comment:A.lineComment,Number:A.number,String:A.string,FormatString:A.special(A.string),UpdateOp:A.updateOperator,ArithOp:A.arithmeticOperator,BitOp:A.bitwiseOperator,CompareOp:A.compareOperator,AssignOp:A.definitionOperator,Ellipsis:A.punctuation,At:A.meta,"( )":A.paren,"[ ]":A.squareBracket,"{ }":A.brace,".":A.derefOperator,", ;":A.separator})]}),languageData:{closeBrackets:{brackets:["(","[","{","'",'"',"'''",'"""']},commentTokens:{line:"#"},indentOnInput:/^\s*([\}\]\)]|else:|elif |except |finally:)$/}});function aF(){return new Z_(w6)}const Vk=ne.define({combine(t){let e,n;for(let r of t)e=e||r.topContainer,n=n||r.bottomContainer;return{topContainer:e,bottomContainer:n}}});function Qa(t,e){let n=t.plugin(Uk),r=n?n.specs.indexOf(e):-1;return r>-1?n.panels[r]:null}const Uk=rt.fromClass(class{constructor(t){this.input=t.state.facet(Na),this.specs=this.input.filter(n=>n),this.panels=this.specs.map(n=>n(t));let e=t.state.facet(Vk);this.top=new Xf(t,!0,e.topContainer),this.bottom=new Xf(t,!1,e.bottomContainer),this.top.sync(this.panels.filter(n=>n.top)),this.bottom.sync(this.panels.filter(n=>!n.top));for(let n of this.panels)n.dom.classList.add("cm-panel"),n.mount&&n.mount()}update(t){let e=t.state.facet(Vk);this.top.container!=e.topContainer&&(this.top.sync([]),this.top=new Xf(t.view,!0,e.topContainer)),this.bottom.container!=e.bottomContainer&&(this.bottom.sync([]),this.bottom=new Xf(t.view,!1,e.bottomContainer)),this.top.syncClasses(),this.bottom.syncClasses();let n=t.state.facet(Na);if(n!=this.input){let r=n.filter(a=>a),i=[],o=[],s=[],l=[];for(let a of r){let c=this.specs.indexOf(a),u;c<0?(u=a(t.view),l.push(u)):(u=this.panels[c],u.update&&u.update(t)),i.push(u),(u.top?o:s).push(u)}this.specs=r,this.panels=i,this.top.sync(o),this.bottom.sync(s);for(let a of l)a.dom.classList.add("cm-panel"),a.mount&&a.mount()}else for(let r of this.panels)r.update&&r.update(t)}destroy(){this.top.sync([]),this.bottom.sync([])}},{provide:It.scrollMargins.from(t=>({top:t.top.scrollMargin(),bottom:t.bottom.scrollMargin()}))});class Xf{constructor(e,n,r){this.view=e,this.top=n,this.container=r,this.dom=void 0,this.classes="",this.panels=[],this.syncClasses()}sync(e){for(let n of this.panels)n.destroy&&e.indexOf(n)<0&&n.destroy();this.panels=e,this.syncDOM()}syncDOM(){if(this.panels.length==0){this.dom&&(this.dom.remove(),this.dom=void 0);return}if(!this.dom){this.dom=document.createElement("div"),this.dom.className=this.top?"cm-panels cm-panels-top":"cm-panels cm-panels-bottom",this.dom.style[this.top?"top":"bottom"]="0";let n=this.container||this.view.dom;n.insertBefore(this.dom,this.top?n.firstChild:null)}let e=this.dom.firstChild;for(let n of this.panels)if(n.dom.parentNode==this.dom){for(;e!=n.dom;)e=jk(e);e=e.nextSibling}else this.dom.insertBefore(n.dom,e);for(;e;)e=jk(e)}scrollMargin(){return!this.dom||this.container?0:Math.max(0,this.top?this.dom.getBoundingClientRect().bottom-Math.max(0,this.view.scrollDOM.getBoundingClientRect().top):Math.min(innerHeight,this.view.scrollDOM.getBoundingClientRect().bottom)-this.dom.getBoundingClientRect().top)}syncClasses(){if(!(!this.container||this.classes==this.view.themeClasses)){for(let e of this.classes.split(" "))e&&this.container.classList.remove(e);for(let e of(this.classes=this.view.themeClasses).split(" "))e&&this.container.classList.add(e)}}}function jk(t){let e=t.nextSibling;return t.remove(),e}const b6=oe.baseTheme({".cm-panels":{boxSizing:"border-box",position:"sticky",left:0,right:0},"&light .cm-panels":{backgroundColor:"#f5f5f5",color:"black"},"&light .cm-panels-top":{borderBottom:"1px solid #ddd"},"&light .cm-panels-bottom":{borderTop:"1px solid #ddd"},"&dark .cm-panels":{backgroundColor:"#333338",color:"white"}}),Na=ne.define({enables:[Uk,b6]});function Fe(){var t=arguments[0];typeof t=="string"&&(t=document.createElement(t));var e=1,n=arguments[1];if(n&&typeof n=="object"&&n.nodeType==null&&!Array.isArray(n)){for(var r in n)if(Object.prototype.hasOwnProperty.call(n,r)){var i=n[r];typeof i=="string"?t.setAttribute(r,i):i!=null&&(t[r]=i)}e++}for(;e<arguments.length;e++)Xk(t,arguments[e]);return t}function Xk(t,e){if(typeof e=="string")t.appendChild(document.createTextNode(e));else if(e!=null)if(e.nodeType!=null)t.appendChild(e);else if(Array.isArray(e))for(var n=0;n<e.length;n++)Xk(t,e[n]);else throw new RangeError("Unsupported child node: "+e)}class $6{constructor(e,n,r){this.from=e,this.to=n,this.diagnostic=r}}class lo{constructor(e,n,r){this.diagnostics=e,this.panel=n,this.selected=r}static init(e,n,r){let i=te.set(e.map(o=>o.from==o.to||o.from==o.to-1&&r.doc.lineAt(o.from).to==o.from?te.widget({widget:new M6(o),diagnostic:o}).range(o.from):te.mark({attributes:{class:"cm-lintRange cm-lintRange-"+o.severity},diagnostic:o}).range(o.from,o.to)),!0);return new lo(i,n,$s(i))}}function $s(t,e=null,n=0){let r=null;return t.between(n,1e9,(i,o,{spec:s})=>{if(!(e&&s.diagnostic!=e))return r=new $6(i,o,s.diagnostic),!1}),r}function S6(t,e){return t.field(cn,!1)?e:e.concat(ce.appendConfig.of([cn,oe.decorations.compute([cn],n=>{let{selected:r,panel:i}=n.field(cn);return!r||!i||r.from==r.to?te.none:te.set([P6.range(r.from,r.to)])}),__(C6),D6]))}const k6=ce.define(),l0=ce.define(),Hk=ce.define(),cn=Rt.define({create(){return new lo(te.none,null,null)},update(t,e){if(e.docChanged){let n=t.diagnostics.map(e.changes),r=null;if(t.selected){let i=e.changes.mapPos(t.selected.from,1);r=$s(n,t.selected.diagnostic,i)||$s(n,null,i)}t=new lo(n,t.panel,r)}for(let n of e.effects)n.is(k6)?t=lo.init(n.value,t.panel,e.state):n.is(l0)?t=new lo(t.diagnostics,n.value?Hf.open:null,t.selected):n.is(Hk)&&(t=new lo(t.diagnostics,t.panel,n.value));return t},provide:t=>[Na.from(t,e=>e.panel),oe.decorations.from(t,e=>e.diagnostics)]}),P6=te.mark({class:"cm-lintRange cm-lintRange-active"});function C6(t,e,n){let{diagnostics:r}=t.state.field(cn),i=[],o=2e8,s=0;return r.between(e-(n<0?1:0),e+(n>0?1:0),(l,a,{spec:c})=>{e>=l&&e<=a&&(l==a||(e>l||n>0)&&(e<a||n<0))&&(i.push(c.diagnostic),o=Math.min(l,o),s=Math.max(a,s))}),i.length?{pos:o,end:s,above:t.state.doc.lineAt(o).to<s,create(){return{dom:R6(t,i)}}}:null}function R6(t,e){return Fe("ul",{class:"cm-tooltip-lint"},e.map(n=>Yk(t,n,!1)))}const T6=t=>{let e=t.state.field(cn,!1);(!e||!e.panel)&&t.dispatch({effects:S6(t.state,[l0.of(!0)])});let n=Qa(t,Hf.open);return n&&n.dom.querySelector(".cm-panel-lint ul").focus(),!0},Gk=t=>{let e=t.state.field(cn,!1);return!e||!e.panel?!1:(t.dispatch({effects:l0.of(!1)}),!0)},I6=t=>{let e=t.state.field(cn,!1);if(!e)return!1;let n=t.state.selection.main,r=e.diagnostics.iter(n.to+1);return!r.value&&(r=e.diagnostics.iter(0),!r.value||r.from==n.from&&r.to==n.to)?!1:(t.dispatch({selection:{anchor:r.from,head:r.to},scrollIntoView:!0}),!0)},E6=[{key:"Mod-Shift-m",run:T6},{key:"F8",run:I6}];function Zk(t){let e=[];if(t)e:for(let{name:n}of t){for(let r=0;r<n.length;r++){let i=n[r];if(/[a-zA-Z]/.test(i)&&!e.some(o=>o.toLowerCase()==i.toLowerCase())){e.push(i);continue e}}e.push("")}return e}function Yk(t,e,n){var r;let i=n?Zk(e.actions):[];return Fe("li",{class:"cm-diagnostic cm-diagnostic-"+e.severity},Fe("span",{class:"cm-diagnosticText"},e.message),(r=e.actions)===null||r===void 0?void 0:r.map((o,s)=>{let l=f=>{f.preventDefault();let h=$s(t.state.field(cn).diagnostics,e);h&&o.apply(t,h.from,h.to)},{name:a}=o,c=i[s]?a.indexOf(i[s]):-1,u=c<0?a:[a.slice(0,c),Fe("u",a.slice(c,c+1)),a.slice(c+1)];return Fe("button",{type:"button",class:"cm-diagnosticAction",onclick:l,onmousedown:l,"aria-label":` Action: ${a}${c<0?"":` (access key "${i[s]})"`}.`},u)}),e.source&&Fe("div",{class:"cm-diagnosticSource"},e.source))}class M6 extends Cr{constructor(e){super();this.diagnostic=e}eq(e){return e.diagnostic==this.diagnostic}toDOM(){return Fe("span",{class:"cm-lintPoint cm-lintPoint-"+this.diagnostic.severity})}}class Kk{constructor(e,n){this.diagnostic=n,this.id="item_"+Math.floor(Math.random()*4294967295).toString(16),this.dom=Yk(e,n,!0),this.dom.id=this.id,this.dom.setAttribute("role","option")}}class Hf{constructor(e){this.view=e,this.items=[];let n=i=>{if(i.keyCode==27)Gk(this.view),this.view.focus();else if(i.keyCode==38||i.keyCode==33)this.moveSelection((this.selectedIndex-1+this.items.length)%this.items.length);else if(i.keyCode==40||i.keyCode==34)this.moveSelection((this.selectedIndex+1)%this.items.length);else if(i.keyCode==36)this.moveSelection(0);else if(i.keyCode==35)this.moveSelection(this.items.length-1);else if(i.keyCode==13)this.view.focus();else if(i.keyCode>=65&&i.keyCode<=90&&this.selectedIndex>=0){let{diagnostic:o}=this.items[this.selectedIndex],s=Zk(o.actions);for(let l=0;l<s.length;l++)if(s[l].toUpperCase().charCodeAt(0)==i.keyCode){let a=$s(this.view.state.field(cn).diagnostics,o);a&&o.actions[l].apply(e,a.from,a.to)}}else return;i.preventDefault()},r=i=>{for(let o=0;o<this.items.length;o++)this.items[o].dom.contains(i.target)&&this.moveSelection(o)};this.list=Fe("ul",{tabIndex:0,role:"listbox","aria-label":this.view.state.phrase("Diagnostics"),onkeydown:n,onclick:r}),this.dom=Fe("div",{class:"cm-panel-lint"},this.list,Fe("button",{type:"button",name:"close","aria-label":this.view.state.phrase("close"),onclick:()=>Gk(this.view)},"\xD7")),this.update()}get selectedIndex(){let e=this.view.state.field(cn).selected;if(!e)return-1;for(let n=0;n<this.items.length;n++)if(this.items[n].diagnostic==e.diagnostic)return n;return-1}update(){let{diagnostics:e,selected:n}=this.view.state.field(cn),r=0,i=!1,o=null;for(e.between(0,this.view.state.doc.length,(s,l,{spec:a})=>{let c=-1,u;for(let f=r;f<this.items.length;f++)if(this.items[f].diagnostic==a.diagnostic){c=f;break}c<0?(u=new Kk(this.view,a.diagnostic),this.items.splice(r,0,u),i=!0):(u=this.items[c],c>r&&(this.items.splice(r,c-r),i=!0)),n&&u.diagnostic==n.diagnostic?u.dom.hasAttribute("aria-selected")||(u.dom.setAttribute("aria-selected","true"),o=u):u.dom.hasAttribute("aria-selected")&&u.dom.removeAttribute("aria-selected"),r++});r<this.items.length&&!(this.items.length==1&&this.items[0].diagnostic.from<0);)i=!0,this.items.pop();this.items.length==0&&(this.items.push(new Kk(this.view,{from:-1,to:-1,severity:"info",message:this.view.state.phrase("No diagnostics")})),i=!0),o?(this.list.setAttribute("aria-activedescendant",o.id),this.view.requestMeasure({key:this,read:()=>({sel:o.dom.getBoundingClientRect(),panel:this.list.getBoundingClientRect()}),write:({sel:s,panel:l})=>{s.top<l.top?this.list.scrollTop-=l.top-s.top:s.bottom>l.bottom&&(this.list.scrollTop+=s.bottom-l.bottom)}})):this.selectedIndex<0&&this.list.removeAttribute("aria-activedescendant"),i&&this.sync()}sync(){let e=this.list.firstChild;function n(){let r=e;e=r.nextSibling,r.remove()}for(let r of this.items)if(r.dom.parentNode==this.list){for(;e!=r.dom;)n();e=r.dom.nextSibling}else this.list.insertBefore(r.dom,e);for(;e;)n()}moveSelection(e){if(this.selectedIndex<0)return;let n=this.view.state.field(cn),r=$s(n.diagnostics,this.items[e].diagnostic);!r||this.view.dispatch({selection:{anchor:r.from,head:r.to},scrollIntoView:!0,effects:Hk.of(r)})}static open(e){return new Hf(e)}}function A6(t,e='viewBox="0 0 40 40"'){return`url('data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" ${e}>${encodeURIComponent(t)}</svg>')`}function a0(t){return A6(`<path d="m0 2.5 l2 -1.5 l1 0 l2 1.5 l1 0" stroke="${t}" fill="none" stroke-width=".7"/>`,'width="6" height="3"')}const D6=oe.baseTheme({".cm-diagnostic":{padding:"3px 6px 3px 8px",marginLeft:"-1px",display:"block",whiteSpace:"pre-wrap"},".cm-diagnostic-error":{borderLeft:"5px solid #d11"},".cm-diagnostic-warning":{borderLeft:"5px solid orange"},".cm-diagnostic-info":{borderLeft:"5px solid #999"},".cm-diagnosticAction":{font:"inherit",border:"none",padding:"2px 4px",backgroundColor:"#444",color:"white",borderRadius:"3px",marginLeft:"8px"},".cm-diagnosticSource":{fontSize:"70%",opacity:.7},".cm-lintRange":{backgroundPosition:"left bottom",backgroundRepeat:"repeat-x",paddingBottom:"0.7px"},".cm-lintRange-error":{backgroundImage:a0("#d11")},".cm-lintRange-warning":{backgroundImage:a0("orange")},".cm-lintRange-info":{backgroundImage:a0("#999")},".cm-lintRange-active":{backgroundColor:"#ffdd9980"},".cm-tooltip-lint":{padding:0,margin:0},".cm-lintPoint":{position:"relative","&:after":{content:'""',position:"absolute",bottom:0,left:"-2px",borderLeft:"3px solid transparent",borderRight:"3px solid transparent",borderBottom:"4px solid #d11"}},".cm-lintPoint-warning":{"&:after":{borderBottomColor:"orange"}},".cm-lintPoint-info":{"&:after":{borderBottomColor:"#999"}},".cm-panel.cm-panel-lint":{position:"relative","& ul":{maxHeight:"100px",overflowY:"auto","& [aria-selected]":{backgroundColor:"#ddd","& u":{textDecoration:"underline"}},"&:focus [aria-selected]":{background_fallback:"#bdf",backgroundColor:"Highlight",color_fallback:"white",color:"HighlightText"},"& u":{textDecoration:"none"},padding:0,margin:0},"& [name=close]":{position:"absolute",top:"0",right:"2px",background:"inherit",border:"none",font:"inherit",padding:0,margin:0}}}),Jk=typeof String.prototype.normalize=="function"?t=>t.normalize("NFKD"):t=>t;class Ss{constructor(e,n,r=0,i=e.length,o){this.value={from:0,to:0},this.done=!1,this.matches=[],this.buffer="",this.bufferPos=0,this.iter=e.iterRange(r,i),this.bufferStart=r,this.normalize=o?s=>o(Jk(s)):Jk,this.query=this.normalize(n)}peek(){if(this.bufferPos==this.buffer.length){if(this.bufferStart+=this.buffer.length,this.iter.next(),this.iter.done)return-1;this.bufferPos=0,this.buffer=this.iter.value}return Ct(this.buffer,this.bufferPos)}next(){for(;this.matches.length;)this.matches.pop();return this.nextOverlapping()}nextOverlapping(){for(;;){let e=this.peek();if(e<0)return this.done=!0,this;let n=wm(e),r=this.bufferStart+this.bufferPos;this.bufferPos+=wn(e);let i=this.normalize(n);for(let o=0,s=r;;o++){let l=i.charCodeAt(o),a=this.match(l,s);if(a)return this.value=a,this;if(o==i.length-1)break;s==r&&o<n.length&&n.charCodeAt(o)==l&&s++}}}match(e,n){let r=null;for(let i=0;i<this.matches.length;i+=2){let o=this.matches[i],s=!1;this.query.charCodeAt(o)==e&&(o==this.query.length-1?r={from:this.matches[i+1],to:n+1}:(this.matches[i]++,s=!0)),s||(this.matches.splice(i,2),i-=2)}return this.query.charCodeAt(0)==e&&(this.query.length==1?r={from:n,to:n+1}:this.matches.push(1,n)),r}}typeof Symbol!="undefined"&&(Ss.prototype[Symbol.iterator]=function(){return this});const eP={from:-1,to:-1,match:/.*/.exec("")},c0="gm"+(/x/.unicode==null?"":"u");class tP{constructor(e,n,r,i=0,o=e.length){if(this.to=o,this.curLine="",this.done=!1,this.value=eP,/\\[sWDnr]|\n|\r|\[\^/.test(n))return new nP(e,n,r,i,o);this.re=new RegExp(n,c0+((r==null?void 0:r.ignoreCase)?"i":"")),this.iter=e.iter();let s=e.lineAt(i);this.curLineStart=s.from,this.matchPos=i,this.getLine(this.curLineStart)}getLine(e){this.iter.next(e),this.iter.lineBreak?this.curLine="":(this.curLine=this.iter.value,this.curLineStart+this.curLine.length>this.to&&(this.curLine=this.curLine.slice(0,this.to-this.curLineStart)),this.iter.next())}nextLine(){this.curLineStart=this.curLineStart+this.curLine.length+1,this.curLineStart>this.to?this.curLine="":this.getLine(0)}next(){for(let e=this.matchPos-this.curLineStart;;){this.re.lastIndex=e;let n=this.matchPos<=this.to&&this.re.exec(this.curLine);if(n){let r=this.curLineStart+n.index,i=r+n[0].length;if(this.matchPos=i+(r==i?1:0),r==this.curLine.length&&this.nextLine(),r<i||r>this.value.to)return this.value={from:r,to:i,match:n},this;e=this.matchPos-this.curLineStart}else if(this.curLineStart+this.curLine.length<this.to)this.nextLine(),e=0;else return this.done=!0,this}}}const u0=new WeakMap;class ks{constructor(e,n){this.from=e,this.text=n}get to(){return this.from+this.text.length}static get(e,n,r){let i=u0.get(e);if(!i||i.from>=r||i.to<=n){let l=new ks(n,e.sliceString(n,r));return u0.set(e,l),l}if(i.from==n&&i.to==r)return i;let{text:o,from:s}=i;return s>n&&(o=e.sliceString(n,s)+o,s=n),i.to<r&&(o+=e.sliceString(i.to,r)),u0.set(e,new ks(s,o)),new ks(n,o.slice(n-s,r-s))}}class nP{constructor(e,n,r,i,o){this.text=e,this.to=o,this.done=!1,this.value=eP,this.matchPos=i,this.re=new RegExp(n,c0+((r==null?void 0:r.ignoreCase)?"i":"")),this.flat=ks.get(e,i,this.chunkEnd(i+5e3))}chunkEnd(e){return e>=this.to?this.to:this.text.lineAt(e).to}next(){for(;;){let e=this.re.lastIndex=this.matchPos-this.flat.from,n=this.re.exec(this.flat.text);if(n&&!n[0]&&n.index==e&&(this.re.lastIndex=e+1,n=this.re.exec(this.flat.text)),n&&this.flat.to<this.to&&n.index+n[0].length>this.flat.text.length-10&&(n=null),n){let r=this.flat.from+n.index,i=r+n[0].length;return this.value={from:r,to:i,match:n},this.matchPos=i+(r==i?1:0),this}else{if(this.flat.to==this.to)return this.done=!0,this;this.flat=ks.get(this.text,this.flat.from,this.chunkEnd(this.flat.from+this.flat.text.length*2))}}}}typeof Symbol!="undefined"&&(tP.prototype[Symbol.iterator]=nP.prototype[Symbol.iterator]=function(){return this});function L6(t){try{return new RegExp(t,c0),!0}catch{return!1}}function f0(t){let e=Fe("input",{class:"cm-textfield",name:"line"}),n=Fe("form",{class:"cm-gotoLine",onkeydown:i=>{i.keyCode==27?(i.preventDefault(),t.dispatch({effects:Gf.of(!1)}),t.focus()):i.keyCode==13&&(i.preventDefault(),r())},onsubmit:i=>{i.preventDefault(),r()}},Fe("label",t.state.phrase("Go to line"),": ",e)," ",Fe("button",{class:"cm-button",type:"submit"},t.state.phrase("go")));function r(){let i=/^([+-])?(\d+)?(:\d+)?(%)?$/.exec(e.value);if(!i)return;let{state:o}=t,s=o.doc.lineAt(o.selection.main.head),[,l,a,c,u]=i,f=c?+c.slice(1):0,h=a?+a:s.number;if(a&&u){let p=h/100;l&&(p=p*(l=="-"?-1:1)+s.number/o.doc.lines),h=Math.round(o.doc.lines*p)}else a&&l&&(h=h*(l=="-"?-1:1)+s.number);let d=o.doc.line(Math.max(1,Math.min(o.doc.lines,h)));t.dispatch({effects:Gf.of(!1),selection:_.cursor(d.from+Math.max(0,Math.min(f,d.length))),scrollIntoView:!0}),t.focus()}return{dom:n,pos:-10}}const Gf=ce.define(),rP=Rt.define({create(){return!0},update(t,e){for(let n of e.effects)n.is(Gf)&&(t=n.value);return t},provide:t=>Na.from(t,e=>e?f0:null)}),W6=t=>{let e=Qa(t,f0);if(!e){let n=[Gf.of(!0)];t.state.field(rP,!1)==null&&n.push(ce.appendConfig.of([rP,Q6])),t.dispatch({effects:n}),e=Qa(t,f0)}return e&&e.dom.querySelector("input").focus(),!0},Q6=oe.baseTheme({".cm-panel.cm-gotoLine":{padding:"2px 6px 4px","& label":{fontSize:"80%"}}}),N6={highlightWordAroundCursor:!1,minSelectionLength:1,maxMatches:100},iP=ne.define({combine(t){return mi(t,N6,{highlightWordAroundCursor:(e,n)=>e||n,minSelectionLength:Math.min,maxMatches:Math.min})}});function _6(t){let e=[F6,z6];return t&&e.push(iP.of(t)),e}const B6=te.mark({class:"cm-selectionMatch"}),q6=te.mark({class:"cm-selectionMatch cm-selectionMatch-main"}),z6=rt.fromClass(class{constructor(t){this.decorations=this.getDeco(t)}update(t){(t.selectionSet||t.docChanged||t.viewportChanged)&&(this.decorations=this.getDeco(t.view))}getDeco(t){let e=t.state.facet(iP),{state:n}=t,r=n.selection;if(r.ranges.length>1)return te.none;let i=r.main,o,s=null;if(i.empty){if(!e.highlightWordAroundCursor)return te.none;let a=n.wordAt(i.head);if(!a)return te.none;s=n.charCategorizer(i.head),o=n.sliceDoc(a.from,a.to)}else{let a=i.to-i.from;if(a<e.minSelectionLength||a>200||(o=n.sliceDoc(i.from,i.to).trim(),!o))return te.none}let l=[];for(let a of t.visibleRanges){let c=new Ss(n.doc,o,a.from,a.to);for(;!c.next().done;){let{from:u,to:f}=c.value;if((!s||(u==0||s(n.sliceDoc(u-1,u))!=rn.Word)&&(f==n.doc.length||s(n.sliceDoc(f,f+1))!=rn.Word))&&(s&&u<=i.from&&f>=i.to?l.push(q6.range(u,f)):(u>=i.to||f<=i.from)&&l.push(B6.range(u,f)),l.length>e.maxMatches))return te.none}}return te.set(l)}},{decorations:t=>t.decorations}),F6=oe.baseTheme({".cm-selectionMatch":{backgroundColor:"#99ff7780"},".cm-searchMatch .cm-selectionMatch":{backgroundColor:"transparent"}}),V6=({state:t,dispatch:e})=>{let{selection:n}=t,r=_.create(n.ranges.map(i=>t.wordAt(i.head)||_.cursor(i.head)),n.mainIndex);return r.eq(n)?!1:(e(t.update({selection:r})),!0)};function U6(t,e){let{main:n,ranges:r}=t.selection,i=t.wordAt(n.head),o=i&&i.from==n.from&&i.to==n.to;for(let s=!1,l=new Ss(t.doc,e,r[r.length-1].to);;)if(l.next(),l.done){if(s)return null;l=new Ss(t.doc,e,0,Math.max(0,r[r.length-1].from-1)),s=!0}else{if(s&&r.some(a=>a.from==l.value.from))continue;if(o){let a=t.wordAt(l.value.from);if(!a||a.from!=l.value.from||a.to!=l.value.to)continue}return l.value}}const j6=({state:t,dispatch:e})=>{let{ranges:n}=t.selection;if(n.some(o=>o.from===o.to))return V6({state:t,dispatch:e});let r=t.sliceDoc(n[0].from,n[0].to);if(t.selection.ranges.some(o=>t.sliceDoc(o.from,o.to)!=r))return!1;let i=U6(t,r);return i?(e(t.update({selection:t.selection.addRange(_.range(i.from,i.to),!1),scrollIntoView:!0})),!0):!1},h0=ne.define({combine(t){var e;return{top:t.reduce((n,r)=>n!=null?n:r.top,void 0)||!1,caseSensitive:t.reduce((n,r)=>n!=null?n:r.caseSensitive||r.matchCase,void 0)||!1,createPanel:((e=t.find(n=>n.createPanel))===null||e===void 0?void 0:e.createPanel)||(n=>new n8(n))}}});class oP{constructor(e){this.search=e.search,this.caseSensitive=!!e.caseSensitive,this.regexp=!!e.regexp,this.replace=e.replace||"",this.valid=!!this.search&&(!this.regexp||L6(this.search))}eq(e){return this.search==e.search&&this.replace==e.replace&&this.caseSensitive==e.caseSensitive&&this.regexp==e.regexp}create(){return this.regexp?new H6(this):new X6(this)}}class sP{constructor(e){this.spec=e}}class X6 extends sP{constructor(e){super(e);this.unquoted=e.search.replace(/\\([nrt\\])/g,(n,r)=>r=="n"?`
`:r=="r"?"\r":r=="t"?"	":"\\")}cursor(e,n=0,r=e.length){return new Ss(e,this.unquoted,n,r,this.spec.caseSensitive?void 0:i=>i.toLowerCase())}nextMatch(e,n,r){let i=this.cursor(e,r).nextOverlapping();return i.done&&(i=this.cursor(e,0,n).nextOverlapping()),i.done?null:i.value}prevMatchInRange(e,n,r){for(let i=r;;){let o=Math.max(n,i-1e4-this.unquoted.length),s=this.cursor(e,o,i),l=null;for(;!s.nextOverlapping().done;)l=s.value;if(l)return l;if(o==n)return null;i-=1e4}}prevMatch(e,n,r){return this.prevMatchInRange(e,0,n)||this.prevMatchInRange(e,r,e.length)}getReplacement(e){return this.spec.replace}matchAll(e,n){let r=this.cursor(e),i=[];for(;!r.next().done;){if(i.length>=n)return null;i.push(r.value)}return i}highlight(e,n,r,i){let o=this.cursor(e,Math.max(0,n-this.unquoted.length),Math.min(r+this.unquoted.length,e.length));for(;!o.next().done;)i(o.value.from,o.value.to)}}class H6 extends sP{cursor(e,n=0,r=e.length){return new tP(e,this.spec.search,this.spec.caseSensitive?void 0:{ignoreCase:!0},n,r)}nextMatch(e,n,r){let i=this.cursor(e,r).next();return i.done&&(i=this.cursor(e,0,n).next()),i.done?null:i.value}prevMatchInRange(e,n,r){for(let i=1;;i++){let o=Math.max(n,r-i*1e4),s=this.cursor(e,o,r),l=null;for(;!s.next().done;)l=s.value;if(l&&(o==n||l.from>o+10))return l;if(o==n)return null}}prevMatch(e,n,r){return this.prevMatchInRange(e,0,n)||this.prevMatchInRange(e,r,e.length)}getReplacement(e){return this.spec.replace.replace(/\$([$&\d+])/g,(n,r)=>r=="$"?"$":r=="&"?e.match[0]:r!="0"&&+r<e.match.length?e.match[r]:n)}matchAll(e,n){let r=this.cursor(e),i=[];for(;!r.next().done;){if(i.length>=n)return null;i.push(r.value)}return i}highlight(e,n,r,i){let o=this.cursor(e,Math.max(0,n-250),Math.min(r+250,e.length));for(;!o.next().done;)i(o.value.from,o.value.to)}}const _a=ce.define(),d0=ce.define(),ki=Rt.define({create(t){return new p0(m0(t).create(),Kf)},update(t,e){for(let n of e.effects)n.is(_a)?t=new p0(n.value.create(),t.panel):n.is(d0)&&(t=new p0(t.query,n.value?Kf:null));return t},provide:t=>Na.from(t,e=>e.panel)});class p0{constructor(e,n){this.query=e,this.panel=n}}const G6=te.mark({class:"cm-searchMatch"}),Z6=te.mark({class:"cm-searchMatch cm-searchMatch-selected"}),Y6=rt.fromClass(class{constructor(t){this.view=t,this.decorations=this.highlight(t.state.field(ki))}update(t){let e=t.state.field(ki);(e!=t.startState.field(ki)||t.docChanged||t.selectionSet)&&(this.decorations=this.highlight(e))}highlight({query:t,panel:e}){if(!e||!t.spec.valid)return te.none;let{view:n}=this,r=new yi;for(let i=0,o=n.visibleRanges,s=o.length;i<s;i++){let{from:l,to:a}=o[i];for(;i<s-1&&a>o[i+1].from-2*250;)a=o[++i].to;t.highlight(n.state.doc,l,a,(c,u)=>{let f=n.state.selection.ranges.some(h=>h.from==c&&h.to==u);r.add(c,u,f?Z6:G6)})}return r.finish()}},{decorations:t=>t.decorations});function Ba(t){return e=>{let n=e.state.field(ki,!1);return n&&n.query.spec.valid?t(e,n):aP(e)}}const Zf=Ba((t,{query:e})=>{let{from:n,to:r}=t.state.selection.main,i=e.nextMatch(t.state.doc,n,r);return!i||i.from==n&&i.to==r?!1:(t.dispatch({selection:{anchor:i.from,head:i.to},scrollIntoView:!0,effects:g0(t,i),userEvent:"select.search"}),!0)}),Yf=Ba((t,{query:e})=>{let{state:n}=t,{from:r,to:i}=n.selection.main,o=e.prevMatch(n.doc,r,i);return o?(t.dispatch({selection:{anchor:o.from,head:o.to},scrollIntoView:!0,effects:g0(t,o),userEvent:"select.search"}),!0):!1}),K6=Ba((t,{query:e})=>{let n=e.matchAll(t.state.doc,1e3);return!n||!n.length?!1:(t.dispatch({selection:_.create(n.map(r=>_.range(r.from,r.to))),userEvent:"select.search.matches"}),!0)}),J6=({state:t,dispatch:e})=>{let n=t.selection;if(n.ranges.length>1||n.main.empty)return!1;let{from:r,to:i}=n.main,o=[],s=0;for(let l=new Ss(t.doc,t.sliceDoc(r,i));!l.next().done;){if(o.length>1e3)return!1;l.value.from==r&&(s=o.length),o.push(_.range(l.value.from,l.value.to))}return e(t.update({selection:_.create(o,s),userEvent:"select.search.matches"})),!0},lP=Ba((t,{query:e})=>{let{state:n}=t,{from:r,to:i}=n.selection.main;if(n.readOnly)return!1;let o=e.nextMatch(n.doc,r,r);if(!o)return!1;let s=[],l,a;if(o.from==r&&o.to==i&&(a=n.toText(e.getReplacement(o)),s.push({from:o.from,to:o.to,insert:a}),o=e.nextMatch(n.doc,o.from,o.to)),o){let c=s.length==0||s[0].from>=o.to?0:o.to-o.from-a.length;l={anchor:o.from-c,head:o.to-c}}return t.dispatch({changes:s,selection:l,scrollIntoView:!!l,effects:o?g0(t,o):void 0,userEvent:"input.replace"}),!0}),e8=Ba((t,{query:e})=>{if(t.state.readOnly)return!1;let n=e.matchAll(t.state.doc,1e9).map(r=>{let{from:i,to:o}=r;return{from:i,to:o,insert:e.getReplacement(r)}});return n.length?(t.dispatch({changes:n,userEvent:"input.replace.all"}),!0):!1});function Kf(t){return t.state.facet(h0).createPanel(t)}function m0(t,e){var n;let r=t.selection.main,i=r.empty||r.to>r.from+100?"":t.sliceDoc(r.from,r.to),o=(n=e==null?void 0:e.caseSensitive)!==null&&n!==void 0?n:t.facet(h0).caseSensitive;return e&&!i?e:new oP({search:i.replace(/\n/g,"\\n"),caseSensitive:o})}const aP=t=>{let e=t.state.field(ki,!1);if(e&&e.panel){let n=Qa(t,Kf);if(!n)return!1;let r=n.dom.querySelector("[name=search]");if(r!=t.root.activeElement){let i=m0(t.state,e.query.spec);i.valid&&t.dispatch({effects:_a.of(i)}),r.focus(),r.select()}}else t.dispatch({effects:[d0.of(!0),e?_a.of(m0(t.state,e.query.spec)):ce.appendConfig.of(i8)]});return!0},cP=t=>{let e=t.state.field(ki,!1);if(!e||!e.panel)return!1;let n=Qa(t,Kf);return n&&n.dom.contains(t.root.activeElement)&&t.focus(),t.dispatch({effects:d0.of(!1)}),!0},t8=[{key:"Mod-f",run:aP,scope:"editor search-panel"},{key:"F3",run:Zf,shift:Yf,scope:"editor search-panel",preventDefault:!0},{key:"Mod-g",run:Zf,shift:Yf,scope:"editor search-panel",preventDefault:!0},{key:"Escape",run:cP,scope:"editor search-panel"},{key:"Mod-Shift-l",run:J6},{key:"Alt-g",run:W6},{key:"Mod-d",run:j6,preventDefault:!0}];class n8{constructor(e){this.view=e;let n=this.query=e.state.field(ki).query.spec;this.commit=this.commit.bind(this),this.searchField=Fe("input",{value:n.search,placeholder:kn(e,"Find"),"aria-label":kn(e,"Find"),class:"cm-textfield",name:"search",onchange:this.commit,onkeyup:this.commit}),this.replaceField=Fe("input",{value:n.replace,placeholder:kn(e,"Replace"),"aria-label":kn(e,"Replace"),class:"cm-textfield",name:"replace",onchange:this.commit,onkeyup:this.commit}),this.caseField=Fe("input",{type:"checkbox",name:"case",checked:n.caseSensitive,onchange:this.commit}),this.reField=Fe("input",{type:"checkbox",name:"re",checked:n.regexp,onchange:this.commit});function r(i,o,s){return Fe("button",{class:"cm-button",name:i,onclick:o,type:"button"},s)}this.dom=Fe("div",{onkeydown:i=>this.keydown(i),class:"cm-search"},[this.searchField,r("next",()=>Zf(e),[kn(e,"next")]),r("prev",()=>Yf(e),[kn(e,"previous")]),r("select",()=>K6(e),[kn(e,"all")]),Fe("label",null,[this.caseField,kn(e,"match case")]),Fe("label",null,[this.reField,kn(e,"regexp")]),...e.state.readOnly?[]:[Fe("br"),this.replaceField,r("replace",()=>lP(e),[kn(e,"replace")]),r("replaceAll",()=>e8(e),[kn(e,"replace all")]),Fe("button",{name:"close",onclick:()=>cP(e),"aria-label":kn(e,"close"),type:"button"},["\xD7"])]])}commit(){let e=new oP({search:this.searchField.value,caseSensitive:this.caseField.checked,regexp:this.reField.checked,replace:this.replaceField.value});e.eq(this.query)||(this.query=e,this.view.dispatch({effects:_a.of(e)}))}keydown(e){o_(this.view,e,"search-panel")?e.preventDefault():e.keyCode==13&&e.target==this.searchField?(e.preventDefault(),(e.shiftKey?Yf:Zf)(this.view)):e.keyCode==13&&e.target==this.replaceField&&(e.preventDefault(),lP(this.view))}update(e){for(let n of e.transactions)for(let r of n.effects)r.is(_a)&&!r.value.eq(this.query)&&this.setQuery(r.value)}setQuery(e){this.query=e,this.searchField.value=e.search,this.replaceField.value=e.replace,this.caseField.checked=e.caseSensitive,this.reField.checked=e.regexp}mount(){this.searchField.select()}get pos(){return 80}get top(){return this.view.state.facet(h0).top}}function kn(t,e){return t.state.phrase(e)}const Jf=30,eh=/[\s\.,:;?!]/;function g0(t,{from:e,to:n}){let r=t.state.doc.lineAt(e).from,i=t.state.doc.lineAt(n).to,o=Math.max(r,e-Jf),s=Math.min(i,n+Jf),l=t.state.sliceDoc(o,s);if(o!=r){for(let a=0;a<Jf;a++)if(!eh.test(l[a+1])&&eh.test(l[a])){l=l.slice(a);break}}if(s!=i){for(let a=l.length-1;a>l.length-Jf;a--)if(!eh.test(l[a-1])&&eh.test(l[a])){l=l.slice(0,a);break}}return oe.announce.of(`${t.state.phrase("current match")}. ${l} ${t.state.phrase("on line")} ${t.state.doc.lineAt(e).number}`)}const r8=oe.baseTheme({".cm-panel.cm-search":{padding:"2px 6px 4px",position:"relative","& [name=close]":{position:"absolute",top:"0",right:"4px",backgroundColor:"inherit",border:"none",font:"inherit",padding:0,margin:0},"& input, & button, & label":{margin:".2em .6em .2em 0"},"& input[type=checkbox]":{marginRight:".2em"},"& label":{fontSize:"80%",whiteSpace:"pre"}},"&light .cm-searchMatch":{backgroundColor:"#ffff0054"},"&dark .cm-searchMatch":{backgroundColor:"#00ffff8a"},"&light .cm-searchMatch-selected":{backgroundColor:"#ff6a0054"},"&dark .cm-searchMatch-selected":{backgroundColor:"#ff00ff8a"}}),i8=[ki,da.lowest(Y6),r8],o8="#e5c07b",uP="#e06c75",s8="#56b6c2",l8="#ffffff",th="#abb2bf",y0="#7d8799",a8="#61afef",c8="#98c379",fP="#d19a66",u8="#c678dd",f8="#21252b",O0="#2c313a",hP="#282c34",v0="#353a42",h8="#3E4451",dP="#528bff",d8=oe.theme({"&":{color:th,backgroundColor:hP},".cm-content":{caretColor:dP},"&.cm-focused .cm-cursor":{borderLeftColor:dP},"&.cm-focused .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection":{backgroundColor:h8},".cm-panels":{backgroundColor:f8,color:th},".cm-panels.cm-panels-top":{borderBottom:"2px solid black"},".cm-panels.cm-panels-bottom":{borderTop:"2px solid black"},".cm-searchMatch":{backgroundColor:"#72a1ff59",outline:"1px solid #457dff"},".cm-searchMatch.cm-searchMatch-selected":{backgroundColor:"#6199ff2f"},".cm-activeLine":{backgroundColor:O0},".cm-selectionMatch":{backgroundColor:"#aafe661a"},"&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket":{backgroundColor:"#bad0f847",outline:"1px solid #515a6b"},".cm-gutters":{backgroundColor:hP,color:y0,border:"none"},".cm-activeLineGutter":{backgroundColor:O0},".cm-foldPlaceholder":{backgroundColor:"transparent",border:"none",color:"#ddd"},".cm-tooltip":{border:"none",backgroundColor:v0},".cm-tooltip .cm-tooltip-arrow:before":{borderTopColor:"transparent",borderBottomColor:"transparent"},".cm-tooltip .cm-tooltip-arrow:after":{borderTopColor:v0,borderBottomColor:v0},".cm-tooltip-autocomplete":{"& > ul > li[aria-selected]":{backgroundColor:O0,color:th}}},{dark:!0}),p8=La.define([{tag:A.keyword,color:u8},{tag:[A.name,A.deleted,A.character,A.propertyName,A.macroName],color:uP},{tag:[A.function(A.variableName),A.labelName],color:a8},{tag:[A.color,A.constant(A.name),A.standard(A.name)],color:fP},{tag:[A.definition(A.name),A.separator],color:th},{tag:[A.typeName,A.className,A.number,A.changed,A.annotation,A.modifier,A.self,A.namespace],color:o8},{tag:[A.operator,A.operatorKeyword,A.url,A.escape,A.regexp,A.link,A.special(A.string)],color:s8},{tag:[A.meta,A.comment],color:y0},{tag:A.strong,fontWeight:"bold"},{tag:A.emphasis,fontStyle:"italic"},{tag:A.strikethrough,textDecoration:"line-through"},{tag:A.link,color:y0,textDecoration:"underline"},{tag:A.heading,fontWeight:"bold",color:uP},{tag:[A.atom,A.bool,A.special(A.variableName)],color:fP},{tag:[A.processingInstruction,A.string,A.inserted],color:c8},{tag:A.invalid,color:l8}]),m8=[d8,p8],x0=2e3;function g8(t,e,n){let r=Math.min(e.line,n.line),i=Math.max(e.line,n.line),o=[];if(e.off>x0||n.off>x0||e.col<0||n.col<0){let s=Math.min(e.off,n.off),l=Math.max(e.off,n.off);for(let a=r;a<=i;a++){let c=t.doc.line(a);c.length<=l&&o.push(_.range(c.from+s,c.to+l))}}else{let s=Math.min(e.col,n.col),l=Math.max(e.col,n.col);for(let a=r;a<=i;a++){let c=t.doc.line(a),u=bm(c.text,s,t.tabSize,!0);if(u>-1){let f=bm(c.text,l,t.tabSize);o.push(_.range(c.from+u,c.from+f))}}}return o}function y8(t,e){let n=t.coordsAtPos(t.viewport.from);return n?Math.round(Math.abs((n.left-e)/t.defaultCharacterWidth)):-1}function pP(t,e){let n=t.posAtCoords({x:e.clientX,y:e.clientY},!1),r=t.state.doc.lineAt(n),i=n-r.from,o=i>x0?-1:i==r.length?y8(t,e.clientX):ua(r.text,t.state.tabSize,n-r.from);return{line:r.number,col:o,off:i}}function O8(t,e){let n=pP(t,e),r=t.state.selection;return n?{update(i){if(i.docChanged){let o=i.changes.mapPos(i.startState.doc.line(n.line).from),s=i.state.doc.lineAt(o);n={line:s.number,col:n.col,off:Math.min(n.off,s.length)},r=r.map(i.changes)}},get(i,o,s){let l=pP(t,i);if(!l)return r;let a=g8(t.state,n,l);return a.length?s?_.create(a.concat(r.ranges)):_.create(a):r}}:null}function v8(t){let e=(t==null?void 0:t.eventFilter)||(n=>n.altKey&&n.button==0);return oe.mouseSelectionStyle.of((n,r)=>e(r)?O8(n,r):null)}const x8=[Zq(),Jq(),x_(),pz(),cz(),a_(),p_(),Pe.allowMultipleSelections.of(!0),sB(),x6.fallback,YB(),QB(),LB(),v8(),P_(),_6(),cf.of([...BB,...Dq,...t8,...$z,...oz,...Nq,...mS,...E6])];var w8=oe.theme({"&":{backgroundColor:"#fff"}},{dark:!1});function b8(t){var{value:e,selection:n,onChange:r,onUpdate:i,extensions:o=[],autoFocus:s,theme:l="light",height:a="",minHeight:c="",maxHeight:u="",placeholder:f="",width:h="",minWidth:d="",maxWidth:p="",editable:m=!0,indentWithTab:g=!0,basicSetup:y=!0,root:O}=t,[v,x]=w.exports.useState(t.container),[b,S]=w.exports.useState(),[k,P]=w.exports.useState(),C=oe.theme({"&":{height:a,minHeight:c,maxHeight:u,width:h,minWidth:d,maxWidth:p}}),R=oe.updateListener.of(L=>{if(L.docChanged&&typeof r=="function"){var M=L.state.doc,q=M.toString();r(q,L)}}),T=[R,C];switch(g&&T.unshift(cf.of([Lq])),y&&T.unshift(x8),f&&T.unshift(I_(f)),l){case"light":T.push(w8);break;case"dark":T.push(m8);break;default:T.push(l);break}return m===!1&&T.push(oe.editable.of(!1)),i&&typeof i=="function"&&T.push(oe.updateListener.of(i)),T=T.concat(o),w.exports.useEffect(()=>{if(v&&!k){var L=Pe.create({doc:e,selection:n,extensions:T});if(P(L),!b){var M=new oe({state:L,parent:v,root:O});S(M)}}},[v,k]),w.exports.useEffect(()=>()=>{b&&b.destroy()},[b]),w.exports.useEffect(()=>{if(b){var L=b.state.doc.toString();e!==L&&b.dispatch({changes:{from:0,to:L.length,insert:e||""}})}},[e,b]),w.exports.useEffect(()=>{b&&b.dispatch({effects:ce.reconfigure.of(T)})},[l,o,f,a,c,u,h,d,p,m,g,y]),w.exports.useEffect(()=>{s&&b&&b.focus()},[s,b]),{state:k,setState:P,view:b,setView:S,container:v,setContainer:x}}var $8=["className","value","selection","extensions","onChange","onUpdate","autoFocus","theme","height","minHeight","maxHeight","width","minWidth","maxWidth","basicSetup","placeholder","indentWithTab","editable","root"],S8=In.forwardRef((t,e)=>{var{className:n,value:r="",selection:i,extensions:o=[],onChange:s,onUpdate:l,autoFocus:a,theme:c="light",height:u,minHeight:f,maxHeight:h,width:d,minWidth:p,maxWidth:m,basicSetup:g,placeholder:y,indentWithTab:O,editable:v,root:x}=t,b=H(t,$8),S=w.exports.useRef(null),{state:k,view:P,container:C,setContainer:R}=b8({container:S.current,root:x,value:r,autoFocus:a,theme:c,height:u,minHeight:f,maxHeight:h,width:d,minWidth:p,maxWidth:m,basicSetup:g,placeholder:y,indentWithTab:O,editable:v,selection:i,onChange:s,onUpdate:l,extensions:o});if(w.exports.useImperativeHandle(e,()=>({editor:C,state:k,view:P}),[C,k,P]),w.exports.useEffect(()=>(R(S.current),()=>{P&&P.destroy()}),[]),typeof r!="string")throw new Error("value must be typeof string but got "+typeof r);var T=typeof c=="string"?"cm-theme-"+c:"cm-theme";return I("div",$({ref:S,className:""+T+(n?" "+n:"")},b))});S8.displayName="CodeMirror";var cF=Ne(I("path",{d:"m20 12-1.41-1.41L13 16.17V4h-2v12.17l-5.58-5.59L4 12l8 8 8-8z"}),"ArrowDownward"),uF=Ne(I("path",{d:"m12 4-1.41 1.41L16.17 11H4v2h12.17l-5.58 5.59L12 20l8-8z"}),"ArrowForward"),fF=Ne(I("path",{d:"m4 12 1.41 1.41L11 7.83V20h2V7.83l5.58 5.59L20 12l-8-8-8 8z"}),"ArrowUpward"),hF=Ne(I("path",{d:"M15.41 7.41 14 6l-6 6 6 6 1.41-1.41L10.83 12z"}),"ChevronLeft"),dF=Ne(I("path",{d:"M10 6 8.59 7.41 13.17 12l-4.58 4.59L10 18l6-6z"}),"ChevronRight"),pF=Ne(I("path",{d:"M19 6.41 17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z"}),"Close"),mF=Ne(I("path",{d:"M5 17h14v2H5zm7-12L5.33 15h13.34z"}),"Eject"),gF=Ne(I("path",{d:"m12 8-6 6 1.41 1.41L12 10.83l4.59 4.58L18 14z"}),"ExpandLess"),yF=Ne(I("path",{d:"M16.59 8.59 12 13.17 7.41 8.59 6 10l6 6 6-6z"}),"ExpandMore"),OF=Ne(I("path",{d:"m4 18 8.5-6L4 6v12zm9-12v12l8.5-6L13 6z"}),"FastForward"),vF=Ne(I("path",{d:"M6 10c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm12 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm-6 0c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"}),"MoreHoriz"),xF=Ne(I("path",{d:"M12 8c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2zm0 2c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2zm0 6c-1.1 0-2 .9-2 2s.9 2 2 2 2-.9 2-2-.9-2-2-2z"}),"MoreVert"),wF=Ne(I("path",{d:"M19 19H5V5h7V3H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2v-7h-2v7zM14 3v2h3.59l-9.83 9.83 1.41 1.41L19 6.41V10h2V3h-7z"}),"OpenInNew"),bF=Ne(I("path",{d:"M6 19h4V5H6v14zm8-14v14h4V5h-4z"}),"Pause"),$F=Ne(I("path",{d:"M8 5v14l11-7z"}),"PlayArrow"),SF=Ne(I("path",{d:"M18.4 10.6C16.55 8.99 14.15 8 11.5 8c-4.65 0-8.58 3.03-9.96 7.22L3.9 16c1.05-3.19 4.05-5.5 7.6-5.5 1.95 0 3.73.72 5.12 1.88L13 16h9V7l-3.6 3.6z"}),"Redo"),kF=Ne(I("path",{d:"M12 4.5C7 4.5 2.73 7.61 1 12c1.73 4.39 6 7.5 11 7.5s9.27-3.11 11-7.5c-1.73-4.39-6-7.5-11-7.5zM12 17c-2.76 0-5-2.24-5-5s2.24-5 5-5 5 2.24 5 5-2.24 5-5 5zm0-8c-1.66 0-3 1.34-3 3s1.34 3 3 3 3-1.34 3-3-1.34-3-3-3z"}),"RemoveRedEye"),PF=Ne(I("path",{d:"m6 18 8.5-6L6 6v12zM16 6v12h2V6h-2z"}),"SkipNext"),CF=Ne(I("path",{d:"M12 17.27 18.18 21l-1.64-7.03L22 9.24l-7.19-.61L12 2 9.19 8.63 2 9.24l5.46 4.73L5.82 21z"}),"Star"),RF=Ne(I("path",{d:"M7.41 18.59 8.83 20 12 16.83 15.17 20l1.41-1.41L12 14l-4.59 4.59zm9.18-13.18L15.17 4 12 7.17 8.83 4 7.41 5.41 12 10l4.59-4.59z"}),"UnfoldLess"),TF=Ne(I("path",{d:"M12 5.83 15.17 9l1.41-1.41L12 3 7.41 7.59 8.83 9 12 5.83zm0 12.34L8.83 15l-1.41 1.41L12 21l4.59-4.59L15.17 15 12 18.17z"}),"UnfoldMore"),IF=Ne(I("path",{d:"M1 21h22L12 2 1 21zm12-3h-2v-2h2v2zm0-4h-2v-4h2v4z"}),"Warning"),mP={};function k8(t){if(t&&t.__esModule)return t;var e={};if(t!=null){for(var n in t)if(Object.prototype.hasOwnProperty.call(t,n)){var r=Object.defineProperty&&Object.getOwnPropertyDescriptor?Object.getOwnPropertyDescriptor(t,n):{};r.get||r.set?Object.defineProperty(e,n,r):e[n]=t[n]}}return e.default=t,e}var gP=k8,Pi={};function P8(t){return t&&t.__esModule?t:{default:t}}var C8=P8;function w0(){return yP=w0=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(t[r]=n[r])}return t},w0.apply(this,arguments)}var yP=w0;function R8(t,e){if(t==null)return{};var n={},r=Object.keys(t),i,o;for(o=0;o<r.length;o++)i=r[o],!(e.indexOf(i)>=0)&&(n[i]=t[i]);return n}var T8=R8,I8=T8;function E8(t,e){if(t==null)return{};var n=I8(t,e),r,i;if(Object.getOwnPropertySymbols){var o=Object.getOwnPropertySymbols(t);for(i=0;i<o.length;i++)r=o[i],!(e.indexOf(r)>=0)&&(!Object.prototype.propertyIsEnumerable.call(t,r)||(n[r]=t[r]))}return n}var M8=E8;function A8(t,e){if(!(t instanceof e))throw new TypeError("Cannot call a class as a function")}var D8=A8;function OP(t,e){for(var n=0;n<e.length;n++){var r=e[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(t,r.key,r)}}function L8(t,e,n){return e&&OP(t.prototype,e),n&&OP(t,n),t}var W8=L8;function Ps(t){return typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?Ps=function(n){return typeof n}:Ps=function(n){return n&&typeof Symbol=="function"&&n.constructor===Symbol&&n!==Symbol.prototype?"symbol":typeof n},Ps(t)}function nh(t){return typeof Symbol=="function"&&Ps(Symbol.iterator)==="symbol"?b0=nh=function(n){return Ps(n)}:b0=nh=function(n){return n&&typeof Symbol=="function"&&n.constructor===Symbol&&n!==Symbol.prototype?"symbol":Ps(n)},nh(t)}var b0=nh;function Q8(t){if(t===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return t}var N8=Q8,_8=b0,B8=N8;function q8(t,e){return e&&(_8(e)==="object"||typeof e=="function")?e:B8(t)}var z8=q8;function $0(t){return vP=$0=Object.setPrototypeOf?Object.getPrototypeOf:function(n){return n.__proto__||Object.getPrototypeOf(n)},$0(t)}var vP=$0;function S0(t,e){return xP=S0=Object.setPrototypeOf||function(r,i){return r.__proto__=i,r},S0(t,e)}var xP=S0,F8=xP;function V8(t,e){if(typeof e!="function"&&e!==null)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&F8(t,e)}var U8=V8,j8=function(){},wP=j8,bP={};function X8(t){return t&&t.__esModule?t:{default:t}}var rh=X8,ih={},$P=rh;Object.defineProperty(ih,"__esModule",{value:!0});ih.default=void 0;var H8=$P(w.exports);$P(wP);var G8=function(e){e.index;var n=e.children;H8.default.Children.count(n)},Z8=G8;ih.default=Z8;var k0={},qa={};Object.defineProperty(qa,"__esModule",{value:!0});qa.default=void 0;var Y8={RESISTANCE_COEF:.6,UNCERTAINTY_THRESHOLD:3};qa.default=Y8;var SP=rh;Object.defineProperty(k0,"__esModule",{value:!0});k0.default=J8;var K8=SP(w.exports),kP=SP(qa);function J8(t){var e=t.children,n=t.startIndex,r=t.startX,i=t.pageX,o=t.viewLength,s=t.resistance,l=K8.default.Children.count(e)-1,a=n+(r-i)/o,c;return s?a<0?a=Math.exp(a*kP.default.RESISTANCE_COEF)-1:a>l&&(a=l+1-Math.exp((l-a)*kP.default.RESISTANCE_COEF)):a<0?(a=0,c=(a-n)*o+i):a>l&&(a=l,c=(a-n)*o+i),{index:a,startX:c}}var oh={},e7=rh;Object.defineProperty(oh,"__esModule",{value:!0});oh.default=void 0;var PP=e7(w.exports),t7=function(e,n){var r=!1,i=function(u){return u?u.key:"empty"};if(e.children.length&&n.children.length){var o=PP.default.Children.map(e.children,i),s=o[e.index];if(s!=null){var l=PP.default.Children.map(n.children,i),a=l[n.index];s===a&&(r=!0)}}return r},n7=t7;oh.default=n7;var sh={};Object.defineProperty(sh,"__esModule",{value:!0});sh.default=void 0;function r7(t,e){var n=t%e;return n<0?n+e:n}var i7=r7;sh.default=i7;(function(t){var e=rh;Object.defineProperty(t,"__esModule",{value:!0}),Object.defineProperty(t,"checkIndexBounds",{enumerable:!0,get:function(){return n.default}}),Object.defineProperty(t,"computeIndex",{enumerable:!0,get:function(){return r.default}}),Object.defineProperty(t,"constant",{enumerable:!0,get:function(){return i.default}}),Object.defineProperty(t,"getDisplaySameSlide",{enumerable:!0,get:function(){return o.default}}),Object.defineProperty(t,"mod",{enumerable:!0,get:function(){return s.default}});var n=e(ih),r=e(k0),i=e(qa),o=e(oh),s=e(sh)})(bP);var o7=gP,Dr=C8;Object.defineProperty(Pi,"__esModule",{value:!0});Pi.getDomTreeShapes=TP;Pi.findNativeHandler=IP;Pi.default=Pi.SwipeableViewsContext=void 0;var lh=Dr(yP),s7=Dr(M8),l7=Dr(D8),a7=Dr(W8),c7=Dr(z8),u7=Dr(vP),f7=Dr(U8),Lr=o7(w.exports);Dr(Z0.exports);Dr(wP);var ah=bP;function CP(t,e,n,r){return t.addEventListener(e,n,r),{remove:function(){t.removeEventListener(e,n,r)}}}var RP={container:{direction:"ltr",display:"flex",willChange:"transform"},slide:{width:"100%",WebkitFlexShrink:0,flexShrink:0,overflow:"auto"}},ur={root:{x:{overflowX:"hidden"},"x-reverse":{overflowX:"hidden"},y:{overflowY:"hidden"},"y-reverse":{overflowY:"hidden"}},flexDirection:{x:"row","x-reverse":"row-reverse",y:"column","y-reverse":"column-reverse"},transform:{x:function(e){return"translate(".concat(-e,"%, 0)")},"x-reverse":function(e){return"translate(".concat(e,"%, 0)")},y:function(e){return"translate(0, ".concat(-e,"%)")},"y-reverse":function(e){return"translate(0, ".concat(e,"%)")}},length:{x:"width","x-reverse":"width",y:"height","y-reverse":"height"},rotationMatrix:{x:{x:[1,0],y:[0,1]},"x-reverse":{x:[-1,0],y:[0,1]},y:{x:[0,1],y:[1,0]},"y-reverse":{x:[0,-1],y:[1,0]}},scrollPosition:{x:"scrollLeft","x-reverse":"scrollLeft",y:"scrollTop","y-reverse":"scrollTop"},scrollLength:{x:"scrollWidth","x-reverse":"scrollWidth",y:"scrollHeight","y-reverse":"scrollHeight"},clientLength:{x:"clientWidth","x-reverse":"clientWidth",y:"clientHeight","y-reverse":"clientHeight"}};function P0(t,e){var n=e.duration,r=e.easeFunction,i=e.delay;return"".concat(t," ").concat(n," ").concat(r," ").concat(i)}function C0(t,e){var n=ur.rotationMatrix[e];return{pageX:n.x[0]*t.pageX+n.x[1]*t.pageY,pageY:n.y[0]*t.pageX+n.y[1]*t.pageY}}function ch(t){return t.touches=[{pageX:t.pageX,pageY:t.pageY}],t}function TP(t,e){for(var n=[];t&&t!==e&&t!==document.body&&!t.hasAttribute("data-swipeable");){var r=window.getComputedStyle(t);r.getPropertyValue("position")==="absolute"||r.getPropertyValue("overflow-x")==="hidden"?n=[]:(t.clientWidth>0&&t.scrollWidth>t.clientWidth||t.clientHeight>0&&t.scrollHeight>t.clientHeight)&&n.push({element:t,scrollWidth:t.scrollWidth,scrollHeight:t.scrollHeight,clientWidth:t.clientWidth,clientHeight:t.clientHeight,scrollLeft:t.scrollLeft,scrollTop:t.scrollTop}),t=t.parentNode}return n}var ao=null;function IP(t){var e=t.domTreeShapes,n=t.pageX,r=t.startX,i=t.axis;return e.some(function(o){var s=n>=r;(i==="x"||i==="y")&&(s=!s);var l=Math.round(o[ur.scrollPosition[i]]),a=l>0,c=l+o[ur.clientLength[i]]<o[ur.scrollLength[i]];return s&&c||!s&&a?(ao=o.element,!0):!1})}var EP=Lr.createContext();Pi.SwipeableViewsContext=EP;var uh=function(t){(0,f7.default)(e,t);function e(n){var r;return(0,l7.default)(this,e),r=(0,c7.default)(this,(0,u7.default)(e).call(this,n)),r.rootNode=null,r.containerNode=null,r.ignoreNextScrollEvents=!1,r.viewLength=0,r.startX=0,r.lastX=0,r.vx=0,r.startY=0,r.isSwiping=void 0,r.started=!1,r.startIndex=0,r.transitionListener=null,r.touchMoveListener=null,r.activeSlide=null,r.indexCurrent=null,r.firstRenderTimeout=null,r.setRootNode=function(i){r.rootNode=i},r.setContainerNode=function(i){r.containerNode=i},r.setActiveSlide=function(i){r.activeSlide=i,r.updateHeight()},r.handleSwipeStart=function(i){var o=r.props.axis,s=C0(i.touches[0],o);r.viewLength=r.rootNode.getBoundingClientRect()[ur.length[o]],r.startX=s.pageX,r.lastX=s.pageX,r.vx=0,r.startY=s.pageY,r.isSwiping=void 0,r.started=!0;var l=window.getComputedStyle(r.containerNode),a=l.getPropertyValue("-webkit-transform")||l.getPropertyValue("transform");if(a&&a!=="none"){var c=a.split("(")[1].split(")")[0].split(","),u=window.getComputedStyle(r.rootNode),f=C0({pageX:parseInt(c[4],10),pageY:parseInt(c[5],10)},o);r.startIndex=-f.pageX/(r.viewLength-parseInt(u.paddingLeft,10)-parseInt(u.paddingRight,10))||0}},r.handleSwipeMove=function(i){if(!r.started){r.handleTouchStart(i);return}if(!(ao!==null&&ao!==r.rootNode)){var o=r.props,s=o.axis,l=o.children,a=o.ignoreNativeScroll,c=o.onSwitching,u=o.resistance,f=C0(i.touches[0],s);if(r.isSwiping===void 0){var h=Math.abs(f.pageX-r.startX),d=Math.abs(f.pageY-r.startY),p=h>d&&h>ah.constant.UNCERTAINTY_THRESHOLD;if(!u&&(s==="y"||s==="y-reverse")&&(r.indexCurrent===0&&r.startX<f.pageX||r.indexCurrent===Lr.Children.count(r.props.children)-1&&r.startX>f.pageX)){r.isSwiping=!1;return}if(h>d&&i.preventDefault(),p===!0||d>ah.constant.UNCERTAINTY_THRESHOLD){r.isSwiping=p,r.startX=f.pageX;return}}if(r.isSwiping===!0){i.preventDefault(),r.vx=r.vx*.5+(f.pageX-r.lastX)*.5,r.lastX=f.pageX;var m=(0,ah.computeIndex)({children:l,resistance:u,pageX:f.pageX,startIndex:r.startIndex,startX:r.startX,viewLength:r.viewLength}),g=m.index,y=m.startX;if(ao===null&&!a){var O=TP(i.target,r.rootNode),v=IP({domTreeShapes:O,startX:r.startX,pageX:f.pageX,axis:s});if(v)return}y?r.startX=y:ao===null&&(ao=r.rootNode),r.setIndexCurrent(g);var x=function(){c&&c(g,"move")};(r.state.displaySameSlide||!r.state.isDragging)&&r.setState({displaySameSlide:!1,isDragging:!0},x),x()}}},r.handleSwipeEnd=function(){if(ao=null,!!r.started&&(r.started=!1,r.isSwiping===!0)){var i=r.state.indexLatest,o=r.indexCurrent,s=i-o,l;Math.abs(r.vx)>r.props.threshold?r.vx>0?l=Math.floor(o):l=Math.ceil(o):Math.abs(s)>r.props.hysteresis?l=s>0?Math.floor(o):Math.ceil(o):l=i;var a=Lr.Children.count(r.props.children)-1;l<0?l=0:l>a&&(l=a),r.setIndexCurrent(l),r.setState({indexLatest:l,isDragging:!1},function(){r.props.onSwitching&&r.props.onSwitching(l,"end"),r.props.onChangeIndex&&l!==i&&r.props.onChangeIndex(l,i,{reason:"swipe"}),o===i&&r.handleTransitionEnd()})}},r.handleTouchStart=function(i){r.props.onTouchStart&&r.props.onTouchStart(i),r.handleSwipeStart(i)},r.handleTouchEnd=function(i){r.props.onTouchEnd&&r.props.onTouchEnd(i),r.handleSwipeEnd(i)},r.handleMouseDown=function(i){r.props.onMouseDown&&r.props.onMouseDown(i),i.persist(),r.handleSwipeStart(ch(i))},r.handleMouseUp=function(i){r.props.onMouseUp&&r.props.onMouseUp(i),r.handleSwipeEnd(ch(i))},r.handleMouseLeave=function(i){r.props.onMouseLeave&&r.props.onMouseLeave(i),r.started&&r.handleSwipeEnd(ch(i))},r.handleMouseMove=function(i){r.props.onMouseMove&&r.props.onMouseMove(i),r.started&&r.handleSwipeMove(ch(i))},r.handleScroll=function(i){if(r.props.onScroll&&r.props.onScroll(i),i.target===r.rootNode){if(r.ignoreNextScrollEvents){r.ignoreNextScrollEvents=!1;return}var o=r.state.indexLatest,s=Math.ceil(i.target.scrollLeft/i.target.clientWidth)+o;r.ignoreNextScrollEvents=!0,i.target.scrollLeft=0,r.props.onChangeIndex&&s!==o&&r.props.onChangeIndex(s,o,{reason:"focus"})}},r.updateHeight=function(){if(r.activeSlide!==null){var i=r.activeSlide.children[0];i!==void 0&&i.offsetHeight!==void 0&&r.state.heightLatest!==i.offsetHeight&&r.setState({heightLatest:i.offsetHeight})}},r.state={indexLatest:n.index,isDragging:!1,renderOnlyActive:!n.disableLazyLoading,heightLatest:0,displaySameSlide:!0},r.setIndexCurrent(n.index),r}return(0,a7.default)(e,[{key:"componentDidMount",value:function(){var r=this;this.transitionListener=CP(this.containerNode,"transitionend",function(i){i.target===r.containerNode&&r.handleTransitionEnd()}),this.touchMoveListener=CP(this.rootNode,"touchmove",function(i){r.props.disabled||r.handleSwipeMove(i)},{passive:!1}),this.props.disableLazyLoading||(this.firstRenderTimeout=setTimeout(function(){r.setState({renderOnlyActive:!1})},0)),this.props.action&&this.props.action({updateHeight:this.updateHeight})}},{key:"UNSAFE_componentWillReceiveProps",value:function(r){var i=r.index;typeof i=="number"&&i!==this.props.index&&(this.setIndexCurrent(i),this.setState({displaySameSlide:(0,ah.getDisplaySameSlide)(this.props,r),indexLatest:i}))}},{key:"componentWillUnmount",value:function(){this.transitionListener.remove(),this.touchMoveListener.remove(),clearTimeout(this.firstRenderTimeout)}},{key:"getSwipeableViewsContext",value:function(){var r=this;return{slideUpdateHeight:function(){r.updateHeight()}}}},{key:"setIndexCurrent",value:function(r){if(!this.props.animateTransitions&&this.indexCurrent!==r&&this.handleTransitionEnd(),this.indexCurrent=r,this.containerNode){var i=this.props.axis,o=ur.transform[i](r*100);this.containerNode.style.WebkitTransform=o,this.containerNode.style.transform=o}}},{key:"handleTransitionEnd",value:function(){!this.props.onTransitionEnd||this.state.displaySameSlide||this.state.isDragging||this.props.onTransitionEnd()}},{key:"render",value:function(){var r=this,i=this.props;i.action;var o=i.animateHeight,s=i.animateTransitions,l=i.axis,a=i.children,c=i.containerStyle,u=i.disabled;i.disableLazyLoading;var f=i.enableMouseEvents;i.hysteresis,i.ignoreNativeScroll,i.index,i.onChangeIndex,i.onSwitching,i.onTransitionEnd,i.resistance;var h=i.slideStyle,d=i.slideClassName,p=i.springConfig,m=i.style;i.threshold;var g=(0,s7.default)(i,["action","animateHeight","animateTransitions","axis","children","containerStyle","disabled","disableLazyLoading","enableMouseEvents","hysteresis","ignoreNativeScroll","index","onChangeIndex","onSwitching","onTransitionEnd","resistance","slideStyle","slideClassName","springConfig","style","threshold"]),y=this.state,O=y.displaySameSlide,v=y.heightLatest,x=y.indexLatest,b=y.isDragging,S=y.renderOnlyActive,k=u?{}:{onTouchStart:this.handleTouchStart,onTouchEnd:this.handleTouchEnd},P=!u&&f?{onMouseDown:this.handleMouseDown,onMouseUp:this.handleMouseUp,onMouseLeave:this.handleMouseLeave,onMouseMove:this.handleMouseMove}:{},C=(0,lh.default)({},RP.slide,h),R,T;if(b||!s||O)R="all 0s ease 0s",T="all 0s ease 0s";else if(R=P0("transform",p),T=P0("-webkit-transform",p),v!==0){var L=", ".concat(P0("height",p));R+=L,T+=L}var M={height:null,WebkitFlexDirection:ur.flexDirection[l],flexDirection:ur.flexDirection[l],WebkitTransition:T,transition:R};if(!S){var q=ur.transform[l](this.indexCurrent*100);M.WebkitTransform=q,M.transform=q}return o&&(M.height=v),Lr.createElement(EP.Provider,{value:this.getSwipeableViewsContext()},Lr.createElement("div",(0,lh.default)({ref:this.setRootNode,style:(0,lh.default)({},ur.root[l],m)},g,k,P,{onScroll:this.handleScroll}),Lr.createElement("div",{ref:this.setContainerNode,style:(0,lh.default)({},M,RP.container,c),className:"react-swipeable-view-container"},Lr.Children.map(a,function(F,Q){if(S&&Q!==x)return null;var E,D=!0;return Q===x&&(D=!1,o&&(E=r.setActiveSlide,C.overflowY="hidden")),Lr.createElement("div",{ref:E,style:C,className:d,"aria-hidden":D,"data-swipeable":"true"},F)}))))}}]),e}(Lr.Component);uh.displayName="ReactSwipableView";uh.propTypes={};uh.defaultProps={animateHeight:!1,animateTransitions:!0,axis:"x",disabled:!1,disableLazyLoading:!1,enableMouseEvents:!1,hysteresis:.6,ignoreNativeScroll:!1,index:0,threshold:5,springConfig:{duration:"0.35s",easeFunction:"cubic-bezier(0.15, 0.3, 0.25, 1)",delay:"0s"},resistance:!1};var h7=uh;Pi.default=h7;(function(t){var e=gP;Object.defineProperty(t,"__esModule",{value:!0}),Object.defineProperty(t,"default",{enumerable:!0,get:function(){return n.default}}),Object.defineProperty(t,"SwipeableViewsContext",{enumerable:!0,get:function(){return n.SwipeableViewsContext}});var n=e(Pi)})(mP);var EF=_P(mP);export{K7 as $,P7 as A,A7 as B,N7 as C,te as D,oe as E,ne as F,lF as G,x6 as H,M7 as I,D7 as J,C7 as K,T7 as L,yF as M,z7 as N,Sr as O,is as P,B7 as Q,yi as R,R7 as S,Y7 as T,GW as U,rt as V,IF as W,j7 as X,H7 as Y,d7 as Z,iF as _,_6 as a,S7 as a$,EF as a0,J7 as a1,eF as a2,rF as a3,tF as a4,V7 as a5,wF as a6,I7 as a7,TF as a8,RF as a9,bF as aA,$F as aB,uF as aC,SF as aD,cF as aE,fF as aF,PF as aG,OF as aH,mF as aI,G7 as aJ,Z7 as aK,X7 as aL,E7 as aM,nF as aN,dF as aO,hF as aP,br as aQ,q5 as aR,_7 as aS,Fo as aT,qx as aU,q2 as aV,Uo as aW,w7 as aX,b7 as aY,$7 as aZ,x7 as a_,vF as aa,xF as ab,gF as ac,L7 as ad,Q7 as ae,kF as af,pF as ag,CD as ah,W7 as ai,pz as aj,p_ as ak,sB as al,YB as am,QB as an,LB as ao,v8 as ap,sF as aq,CB as ar,RB as as,da as at,mm as au,F7 as av,U7 as aw,cQ as ax,CF as ay,q7 as az,Dq as b,k7 as b0,y7 as b1,k2 as b2,C2 as b3,O7 as b4,rc as b5,p7 as b6,BB as c,a_ as d,$z as e,cz as f,oz as g,x_ as h,Nq as i,I as j,cf as k,Zq as l,mS as m,E6 as n,Pe as o,v7 as p,aF as q,qo as r,t8 as s,m7 as t,g7 as u,w as v,S8 as w,m8 as x,In as y,Ue as z};
